
#include "pmu-events/pmu-events.h"
#include "util/header.h"
#include "util/pmu.h"
#include <string.h>
#include <stddef.h>

struct compact_pmu_event {
  int offset;
};

static const char *const big_c_string =
/* offset=0 */ "\000CPI\000\0001 / IPC\000\000\000\000\000\000\000\00000"
/* offset=23 */ "\000DCache_L2_All\000\000DCache_L2_All_Hits + DCache_L2_All_Miss\000\000\000\000\000\000\000\00000"
/* offset=88 */ "\000DCache_L2_All_Hits\000\000l2_rqsts.demand_data_rd_hit + l2_rqsts.pf_hit + l2_rqsts.rfo_hit\000\000\000\000\000\000\000\00000"
/* offset=183 */ "\000DCache_L2_All_Miss\000\000max(l2_rqsts.all_demand_data_rd - l2_rqsts.demand_data_rd_hit, 0) + l2_rqsts.pf_miss + l2_rqsts.rfo_miss\000\000\000\000\000\000\000\00000"
/* offset=318 */ "\000DCache_L2_Hits\000\000d_ratio(DCache_L2_All_Hits, DCache_L2_All)\000\000\000\000\000\000\000\00000"
/* offset=387 */ "\000DCache_L2_Misses\000\000d_ratio(DCache_L2_All_Miss, DCache_L2_All)\000\000\000\000\000\000\000\00000"
/* offset=458 */ "\000Frontend_Bound_SMT\000\000idq_uops_not_delivered.core / (4 * (cpu_clk_unhalted.thread / 2 * (1 + cpu_clk_unhalted.one_thread_active / cpu_clk_unhalted.ref_xclk)))\000\000\000\000\000\000\000\00000"
/* offset=625 */ "\000IPC\000group1\000inst_retired.any / cpu_clk_unhalted.thread\000\000\000\000\000\000\000\00000"
/* offset=689 */ "\000L1D_Cache_Fill_BW\000\00064 * l1d.replacement / 1e9 / duration_time\000\000\000\000\000\000\000\00000"
/* offset=761 */ "\000M1\000\000ipc + M2\000\000\000\000\000\000\000\00000"
/* offset=784 */ "\000M2\000\000ipc + M1\000\000\000\000\000\000\000\00000"
/* offset=807 */ "\000M3\000\0001 / M3\000\000\000\000\000\000\000\00000"
/* offset=828 */ "\000advanced_simd_spec_rate\000InstructionMix\000ASE_SPEC / INST_SPEC\000\000The rate of advanced SIMD instructions speculatively executed to overall instructions speclatively executed\000\000100%\000\000\000\00000"
/* offset=1009 */ "\000backend_bound\000Default;TopDownL1\0001 - (frontend_bound + bad_speculation + retiring)\000\000Backend Bound L1 topdown metric\000Backend Bound L1 topdown metric\000\000\000\000TopDownL1\00000"
/* offset=1172 */ "\000backend_bound\000Default;TopdownL1\000stall_slot_backend / (#slots * cpu_cycles)\000\000Backend Bound L1 topdown metric\000\000100%\000\000\000TopdownL1\00000"
/* offset=1301 */ "\000backend_stalled_cycles\000Cycle_Accounting\000STALL_BACKEND / CPU_CYCLES * 100\000\000This metric is the percentage of cycles that were stalled due to resource constraints in the backend unit of the processor\000\0001percent of cycles\000\000\000\00000"
/* offset=1524 */ "\000bad_speculation\000Default;TopDownL1\000(INST_SPEC - INST_RETIRED) / (4 * CPU_CYCLES)\000\000Bad Speculation L1 topdown metric\000Bad Speculation L1 topdown metric\000\000\000\000TopDownL1\00000"
/* offset=1689 */ "\000bad_speculation\000Default;TopdownL1\000(1 - op_retired / op_spec) * (1 - (stall_slot if #slots - 5 else stall_slot - cpu_cycles) / (#slots * cpu_cycles))\000\000Bad speculation L1 topdown metric\000\000100%\000\000\000TopdownL1\00000"
/* offset=1894 */ "\000bp_misp_flush\000TopDownL3\000BR_MIS_PRED * 5 / CPU_CYCLES\000\000BP misp flush L3 topdown metric\000BP misp flush L3 topdown metric\000\000\000\000\00000"
/* offset=2019 */ "\000branch_immed_spec_rate\000InstructionMix\000BR_IMMED_SPEC / INST_SPEC\000\000The rate of branch immediate instructions speculatively executed to overall instructions speclatively executed\000\000100%\000\000\000\00000"
/* offset=2207 */ "\000branch_indirect_spec_rate\000InstructionMix\000BR_INDIRECT_SPEC / INST_SPEC\000\000The rate of indirect branch instructions speculatively executed to overall instructions speclatively executed\000\000100%\000\000\000\00000"
/* offset=2400 */ "\000branch_misprediction_ratio\000Miss_Ratio;Branch_Effectiveness\000BR_MIS_PRED_RETIRED / BR_RETIRED\000\000This metric measures the ratio of branches mispredicted to the total number of branches architecturally executed. This gives an indication of the effectiveness of the branch prediction unit\000\0001per branch\000\000\000\00000"
/* offset=2702 */ "\000branch_mispredicts\000TopDownL2\000bad_speculation * BR_MIS_PRED / (BR_MIS_PRED + armv8_pmuv3_0@event\\=0x2013@)\000\000Branch mispredicts L2 topdown metric\000Branch mispredicts L2 topdown metric\000\000\000\000\00000"
/* offset=2890 */ "\000branch_miss_pred_rate\000Branch\000BR_MIS_PRED_RETIRED / BR_RETIRED\000\000The rate of branches mis-predited to the overall branches\000\000100%\000\000\000\00000"
/* offset=3023 */ "\000branch_mpki\000Branch\000BR_MIS_PRED_RETIRED / INST_RETIRED * 1e3\000\000The rate of branches mis-predicted per kilo instructions\000\0001MPKI\000\000\000\00000"
/* offset=3154 */ "\000branch_mpki\000MPKI;Branch_Effectiveness\000BR_MIS_PRED_RETIRED / INST_RETIRED * 1e3\000\000This metric measures the number of branch mispredictions per thousand instructions executed\000\0001MPKI\000\000\000\00000"
/* offset=3339 */ "\000branch_percentage\000Operation_Mix\000(BR_IMMED_SPEC + BR_INDIRECT_SPEC) / INST_SPEC * 100\000\000This metric measures branch operations as a percentage of operations speculatively executed\000\0001percent of operations\000\000\000\00000"
/* offset=3547 */ "\000branch_pki\000Branch\000BR_RETIRED / INST_RETIRED * 1e3\000\000The rate of branches retired per kilo instructions\000\0001PKI\000\000\000\00000"
/* offset=3661 */ "\000branch_return_spec_rate\000InstructionMix\000BR_RETURN_SPEC / INST_SPEC\000\000The rate of procedure return instructions speculatively executed to overall instructions speclatively executed\000\000100%\000\000\000\00000"
/* offset=3851 */ "\000cache_miss_cycles\000group1\000dcache_miss_cpi + icache_miss_cycles\000\000\000\000\000\000\000\00000"
/* offset=3923 */ "\000core_bound\000TopDownL2\000(EXE_STALL_CYCLE - (MEM_STALL_ANYLOAD + armv8_pmuv3_0@event\\=0x7005@)) / CPU_CYCLES\000\000Core bound L2 topdown metric\000Core bound L2 topdown metric\000\000\000\000\00000"
/* offset=4094 */ "\000cpu_utilization\000PEutilization\000OP_RETIRED / OP_SPEC * (1 - (STALL_SLOT if #slots - 5 else STALL_SLOT - CPU_CYCLES) / (#slots * CPU_CYCLES))\000\000The truly effective ratio of micro-operations executed by the CPU, which means that misprediction and stall are not included\000\000100%\000\000\000\00000"
/* offset=4371 */ "\000crypto_percentage\000Operation_Mix\000CRYPTO_SPEC / INST_SPEC * 100\000\000This metric measures crypto operations as a percentage of operations speculatively executed\000\0001percent of operations\000\000\000\00000"
/* offset=4556 */ "\000crypto_spec_rate\000InstructionMix\000CRYPTO_SPEC / INST_SPEC\000\000The rate of crypto instructions speculatively executed to overall instructions speclatively executed\000\000100%\000\000\000\00000"
/* offset=4726 */ "\000data_process_spec_rate\000InstructionMix\000DP_SPEC / INST_SPEC\000\000The rate of integer data-processing instructions speculatively executed to overall instructions speclatively executed\000\000100%\000\000\000\00000"
/* offset=4915 */ "\000dcache_miss_cpi\000\000l1d\\-loads\\-misses / inst_retired.any\000\000\000\000\000\000\000\00000"
/* offset=4980 */ "\000divider\000TopDownL3\000armv8_pmuv3_0@event\\=0x7002@ / CPU_CYCLES\000\000Divider L3 topdown metric\000Divider L3 topdown metric\000\000\000\000\00000"
/* offset=5100 */ "\000dtlb_mpki\000MPKI;DTLB_Effectiveness\000DTLB_WALK / INST_RETIRED * 1e3\000\000This metric measures the number of data TLB Walks per thousand instructions executed\000\0001MPKI\000\000\000\00000"
/* offset=5264 */ "\000dtlb_mpki\000TLB\000DTLB_WALK / INST_RETIRED * 1e3\000\000The rate of TLB Walks per kilo instructions for data accesses\000\0001MPKI\000\000\000\00000"
/* offset=5385 */ "\000dtlb_walk_rate\000TLB\000DTLB_WALK / L1D_TLB\000\000The rate of DTLB Walks to the overall L1D TLB lookups\000\000100%\000\000\000\00000"
/* offset=5491 */ "\000dtlb_walk_ratio\000Miss_Ratio;DTLB_Effectiveness\000DTLB_WALK / L1D_TLB\000\000This metric measures the ratio of instruction TLB Walks to the total number of data TLB accesses. This gives an indication of the effectiveness of the data TLB accesses\000\0001per TLB access\000\000\000\00000"
/* offset=5750 */ "\000exe_ports_util\000TopDownL3\000core_bound - divider - fsu_stall\000\000EXE ports util L3 topdown metric\000EXE ports util L3 topdown metric\000\000\000\000\00000"
/* offset=5882 */ "\000fetch_bandwidth_bound\000TopDownL2\000frontend_bound - fetch_latency_bound\000\000Fetch bandwidth bound L2 topdown metric\000Fetch bandwidth bound L2 topdown metric\000\000\000\000\00000"
/* offset=6039 */ "\000fetch_latency_bound\000TopDownL2\000armv8_pmuv3_0@event\\=0x201d@ / CPU_CYCLES\000\000Fetch latency bound L2 topdown metric\000Fetch latency bound L2 topdown metric\000\000\000\000\00000"
/* offset=6195 */ "\000float_point_spec_rate\000InstructionMix\000VFP_SPEC / INST_SPEC\000\000The rate of floating point instructions speculatively executed to overall instructions speclatively executed\000\000100%\000\000\000\00000"
/* offset=6375 */ "\000frontend_bound\000Default;TopDownL1\000FETCH_BUBBLE / (4 * CPU_CYCLES)\000\000Frontend bound L1 topdown metric\000Frontend bound L1 topdown metric\000\000\000\000TopDownL1\00000"
/* offset=6523 */ "\000frontend_bound\000Default;TopdownL1\000(stall_slot_frontend if #slots - 5 else stall_slot_frontend - cpu_cycles) / (#slots * cpu_cycles)\000\000Frontend bound L1 topdown metric\000\000100%\000\000\000TopdownL1\00000"
/* offset=6709 */ "\000frontend_stalled_cycles\000Cycle_Accounting\000STALL_FRONTEND / CPU_CYCLES * 100\000\000This metric is the percentage of cycles that were stalled due to resource constraints in the frontend unit of the processor\000\0001percent of cycles\000\000\000\00000"
/* offset=6935 */ "\000fsu_stall\000TopDownL3\000armv8_pmuv3_0@event\\=0x7003@ / CPU_CYCLES\000\000FSU stall L3 topdown metric\000FSU stall L3 topdown metric\000\000\000\000\00000"
/* offset=7061 */ "\000icache_miss_cycles\000\000l1i\\-loads\\-misses / inst_retired.any\000\000\000\000\000\000\000\00000"
/* offset=7129 */ "\000idle_by_icache_miss\000TopDownL3\000((L2I_CACHE - L2I_CACHE_REFILL) * 15 + L2I_CACHE_REFILL * 100) / CPU_CYCLES\000\000Idle by icache miss L3 topdown metric\000Idle by icache miss L3 topdown metric\000\000\000\000\00000"
/* offset=7319 */ "\000idle_by_itlb_miss\000TopDownL3\000((L2I_TLB - L2I_TLB_REFILL) * 15 + L2I_TLB_REFILL * 100) / CPU_CYCLES\000\000Idle by itlb miss L3 topdown metric\000Idle by itlb miss L3 topdown metric\000\000\000\000\00000"
/* offset=7497 */ "\000indirect_branch\000TopDownL3\000armv8_pmuv3_0@event\\=0x1010@ / BR_MIS_PRED\000\000Indirect branch L3 topdown metric\000Indirect branch L3 topdown metric\000\000\000\000\00000"
/* offset=7642 */ "\000integer_dp_percentage\000Operation_Mix\000DP_SPEC / INST_SPEC * 100\000\000This metric measures scalar integer operations as a percentage of operations speculatively executed\000\0001percent of operations\000\000\000\00000"
/* offset=7835 */ "\000ipc\000General\000INST_RETIRED / CPU_CYCLES\000\000This metric measures the number of instructions retired per cycle\000\0001per cycle\000\000\000\00000"
/* offset=7958 */ "\000ipc\000PEutilization\000instructions / CPU_CYCLES\000\000The average number of instructions executed for each cycle\000\000\000\000\000\00000"
/* offset=8070 */ "\000ipc_rate\000PEutilization\000ipc / 5\000\000IPC percentage of peak. The peak of IPC is 5\000\000100%\000\000\000\00000"
/* offset=8159 */ "\000itlb_mpki\000MPKI;ITLB_Effectiveness\000ITLB_WALK / INST_RETIRED * 1e3\000\000This metric measures the number of instruction TLB Walks per thousand instructions executed\000\0001MPKI\000\000\000\00000"
/* offset=8330 */ "\000itlb_mpki\000TLB\000ITLB_WALK / INST_RETIRED * 1e3\000\000The rate of TLB Walks per kilo instructions for instruction accesses\000\0001MPKI\000\000\000\00000"
/* offset=8458 */ "\000itlb_walk_rate\000TLB\000ITLB_WALK / L1I_TLB\000\000The rate of ITLB Walks to the overall L1I TLB lookups\000\000100%\000\000\000\00000"
/* offset=8564 */ "\000itlb_walk_ratio\000Miss_Ratio;ITLB_Effectiveness\000ITLB_WALK / L1I_TLB\000\000This metric measures the ratio of instruction TLB Walks to the total number of instruction TLB accesses. This gives an indication of the effectiveness of the instruction TLB accesses\000\0001per TLB access\000\000\000\00000"
/* offset=8837 */ "\000l1_bound\000TopDownL3\000(MEM_STALL_ANYLOAD - MEM_STALL_L1MISS) / CPU_CYCLES\000\000L1 bound L3 topdown metric\000L1 bound L3 topdown metric\000\000\000\000\00000"
/* offset=8970 */ "\000l1d_cache_miss_rate\000Cache\000L1D_CACHE_REFILL / L1D_CACHE\000\000The rate of L1 D-Cache misses to the overall L1 D-Cache\000\000100%\000\000\000\00000"
/* offset=9094 */ "\000l1d_cache_miss_ratio\000Miss_Ratio;L1D_Cache_Effectiveness\000L1D_CACHE_REFILL / L1D_CACHE\000\000This metric measures the ratio of level 1 data cache accesses missed to the total number of level 1 data cache accesses. This gives an indication of the effectiveness of the level 1 data cache\000\0001per cache access\000\000\000\00000"
/* offset=9398 */ "\000l1d_cache_mpki\000Cache\000L1D_CACHE_REFILL / INST_RETIRED * 1e3\000\000The rate of L1 D-Cache misses per kilo instructions\000\0001MPKI\000\000\000\00000"
/* offset=9523 */ "\000l1d_cache_mpki\000MPKI;L1D_Cache_Effectiveness\000L1D_CACHE_REFILL / INST_RETIRED * 1e3\000\000This metric measures the number of level 1 data cache accesses missed per thousand instructions executed\000\0001MPKI\000\000\000\00000"
/* offset=9724 */ "\000l1d_tlb_miss_rate\000TLB\000L1D_TLB_REFILL / L1D_TLB\000\000The rate of L1D TLB refill to the overall L1D TLB lookups\000\000100%\000\000\000\00000"
/* offset=9842 */ "\000l1d_tlb_miss_ratio\000Miss_Ratio;DTLB_Effectiveness\000L1D_TLB_REFILL / L1D_TLB\000\000This metric measures the ratio of level 1 data TLB accesses missed to the total number of level 1 data TLB accesses. This gives an indication of the effectiveness of the level 1 data TLB\000\0001per TLB access\000\000\000\00000"
/* offset=10127 */ "\000l1d_tlb_mpki\000MPKI;DTLB_Effectiveness\000L1D_TLB_REFILL / INST_RETIRED * 1e3\000\000This metric measures the number of level 1 instruction TLB accesses missed per thousand instructions executed\000\0001MPKI\000\000\000\00000"
/* offset=10324 */ "\000l1i_cache_miss_rate\000Cache\000L1I_CACHE_REFILL / L1I_CACHE\000\000The rate of L1 I-Cache misses to the overall L1 I-Cache\000\000100%\000\000\000\00000"
/* offset=10448 */ "\000l1i_cache_miss_ratio\000Miss_Ratio;L1I_Cache_Effectiveness\000L1I_CACHE_REFILL / L1I_CACHE\000\000This metric measures the ratio of level 1 instruction cache accesses missed to the total number of level 1 instruction cache accesses. This gives an indication of the effectiveness of the level 1 instruction cache\000\0001per cache access\000\000\000\00000"
/* offset=10773 */ "\000l1i_cache_mpki\000Cache\000L1I_CACHE_REFILL / INST_RETIRED * 1e3\000\000The rate of L1 I-Cache misses per kilo instructions\000\0001MPKI\000\000\000\00000"
/* offset=10898 */ "\000l1i_cache_mpki\000MPKI;L1I_Cache_Effectiveness\000L1I_CACHE_REFILL / INST_RETIRED * 1e3\000\000This metric measures the number of level 1 instruction cache accesses missed per thousand instructions executed\000\0001MPKI\000\000\000\00000"
/* offset=11106 */ "\000l1i_tlb_miss_rate\000TLB\000L1I_TLB_REFILL / L1I_TLB\000\000The rate of L1I TLB refill to the overall L1I TLB lookups\000\000100%\000\000\000\00000"
/* offset=11224 */ "\000l1i_tlb_miss_ratio\000Miss_Ratio;ITLB_Effectiveness\000L1I_TLB_REFILL / L1I_TLB\000\000This metric measures the ratio of level 1 instruction TLB accesses missed to the total number of level 1 instruction TLB accesses. This gives an indication of the effectiveness of the level 1 instruction TLB\000\0001per TLB access\000\000\000\00000"
/* offset=11530 */ "\000l1i_tlb_mpki\000MPKI;ITLB_Effectiveness\000L1I_TLB_REFILL / INST_RETIRED * 1e3\000\000This metric measures the number of level 1 instruction TLB accesses missed per thousand instructions executed\000\0001MPKI\000\000\000\00000"
/* offset=11727 */ "\000l2_bound\000TopDownL3\000(MEM_STALL_L1MISS - MEM_STALL_L2MISS) / CPU_CYCLES\000\000L2 bound L3 topdown metric\000L2 bound L3 topdown metric\000\000\000\000\00000"
/* offset=11859 */ "\000l2_cache_miss_ratio\000Miss_Ratio;L2_Cache_Effectiveness\000L2D_CACHE_REFILL / L2D_CACHE\000\000This metric measures the ratio of level 2 cache accesses missed to the total number of level 2 cache accesses. This gives an indication of the effectiveness of the level 2 cache, which is a unified cache that stores both data and instruction. Note that cache accesses in this cache are either data memory access or instruction fetch as this is a unified cache\000\0001per cache access\000\000\000\00000"
/* offset=12328 */ "\000l2_cache_mpki\000MPKI;L2_Cache_Effectiveness\000L2D_CACHE_REFILL / INST_RETIRED * 1e3\000\000This metric measures the number of level 2 unified cache accesses missed per thousand instructions executed. Note that cache accesses in this cache are either data memory access or instruction fetch as this is a unified cache\000\0001MPKI\000\000\000\00000"
/* offset=12648 */ "\000l2_tlb_miss_rate\000TLB\000L2D_TLB_REFILL / L2D_TLB\000\000The rate of L2D TLB refill to the overall L2D TLB lookups\000\000100%\000\000\000\00000"
/* offset=12765 */ "\000l2_tlb_miss_ratio\000Miss_Ratio;ITLB_Effectiveness;DTLB_Effectiveness\000L2D_TLB_REFILL / L2D_TLB\000\000This metric measures the ratio of level 2 unified TLB accesses missed to the total number of level 2 unified TLB accesses. This gives an indication of the effectiveness of the level 2 TLB\000\0001per TLB access\000\000\000\00000"
/* offset=13069 */ "\000l2_tlb_mpki\000MPKI;ITLB_Effectiveness;DTLB_Effectiveness\000L2D_TLB_REFILL / INST_RETIRED * 1e3\000\000This metric measures the number of level 2 unified TLB accesses missed per thousand instructions executed\000\0001MPKI\000\000\000\00000"
/* offset=13280 */ "\000l2d_cache_miss_rate\000Cache\000L2D_CACHE_REFILL / L2D_CACHE\000\000The rate of L2 D-Cache misses to the overall L2 D-Cache\000\000100%\000\000\000\00000"
/* offset=13404 */ "\000l2d_cache_mpki\000Cache\000L2D_CACHE_REFILL / INST_RETIRED * 1e3\000\000The rate of L2 D-Cache misses per kilo instructions\000\0001MPKI\000\000\000\00000"
/* offset=13529 */ "\000l3d_cache_miss_rate\000Cache\000L3D_CACHE_REFILL / L3D_CACHE\000\000The rate of L3 D-Cache misses to the overall L3 D-Cache\000\000100%\000\000\000\00000"
/* offset=13653 */ "\000l3d_cache_mpki\000Cache\000L3D_CACHE_REFILL / INST_RETIRED * 1e3\000\000The rate of L3 D-Cache misses per kilo instructions\000\0001MPKI\000\000\000\00000"
/* offset=13778 */ "\000ll_cache_read_hit_rate\000Cache\000(LL_CACHE_RD - LL_CACHE_MISS_RD) / LL_CACHE_RD\000\000The rate of LL Cache read hit to the overall LL Cache read\000\000100%\000\000\000\00000"
/* offset=13926 */ "\000ll_cache_read_hit_ratio\000LL_Cache_Effectiveness\000(LL_CACHE_RD - LL_CACHE_MISS_RD) / LL_CACHE_RD\000\000This metric measures the ratio of last level cache read accesses hit in the cache to the total number of last level cache accesses. This gives an indication of the effectiveness of the last level cache for read traffic. Note that cache accesses in this cache are either data memory access or instruction fetch as this is a system level cache\000\0001per cache access\000\000\000\00000"
/* offset=14388 */ "\000ll_cache_read_miss_rate\000Cache\000LL_CACHE_MISS_RD / LL_CACHE_RD\000\000The rate of LL Cache read misses to the overall LL Cache read\000\000100%\000\000\000\00000"
/* offset=14524 */ "\000ll_cache_read_miss_ratio\000Miss_Ratio;LL_Cache_Effectiveness\000LL_CACHE_MISS_RD / LL_CACHE_RD\000\000This metric measures the ratio of last level cache read accesses missed to the total number of last level cache accesses. This gives an indication of the effectiveness of the last level cache for read traffic. Note that cache accesses in this cache are either data memory access or instruction fetch as this is a system level cache\000\0001per cache access\000\000\000\00000"
/* offset=14972 */ "\000ll_cache_read_mpki\000Cache\000LL_CACHE_MISS_RD / INST_RETIRED * 1e3\000\000The rate of LL Cache read misses per kilo instructions\000\0001MPKI\000\000\000\00000"
/* offset=15104 */ "\000ll_cache_read_mpki\000MPKI;LL_Cache_Effectiveness\000LL_CACHE_MISS_RD / INST_RETIRED * 1e3\000\000This metric measures the number of last level cache read accesses missed per thousand instructions executed\000\0001MPKI\000\000\000\00000"
/* offset=15311 */ "\000load_percentage\000Operation_Mix\000LD_SPEC / INST_SPEC * 100\000\000This metric measures load operations as a percentage of operations speculatively executed\000\0001percent of operations\000\000\000\00000"
/* offset=15488 */ "\000load_spec_rate\000InstructionMix\000LD_SPEC / INST_SPEC\000\000The rate of load instructions speculatively executed to overall instructions speclatively executed\000\000100%\000\000\000\00000"
/* offset=15650 */ "\000machine_clears\000TopDownL2\000bad_speculation - branch_mispredicts\000\000Machine clears L2 topdown metric\000Machine clears L2 topdown metric\000\000\000\000\00000"
/* offset=15786 */ "\000mem_bound\000TopDownL3\000MEM_STALL_L2MISS / CPU_CYCLES\000\000Mem bound L3 topdown metric\000Mem bound L3 topdown metric\000\000\000\000\00000"
/* offset=15900 */ "\000memory_bound\000TopDownL2\000(MEM_STALL_ANYLOAD + armv8_pmuv3_0@event\\=0x7005@) / CPU_CYCLES\000\000Memory bound L2 topdown metric\000Memory bound L2 topdown metric\000\000\000\000\00000"
/* offset=16057 */ "\000nuke_flush\000TopDownL3\000armv8_pmuv3_0@event\\=0x2012@ / armv8_pmuv3_0@event\\=0x2013@\000\000Nuke flush L3 topdown metric\000Nuke flush L3 topdown metric\000\000\000\000\00000"
/* offset=16204 */ "\000ooo_flush\000TopDownL3\000armv8_pmuv3_0@event\\=0x2013@ * 5 / CPU_CYCLES\000\000OOO flush L3 topdown metric\000OOO flush L3 topdown metric\000\000\000\000\00000"
/* offset=16334 */ "\000other_branch\000TopDownL3\000(BR_MIS_PRED - armv8_pmuv3_0@event\\=0x1010@ - armv8_pmuv3_0@event\\=0x1013@ - armv8_pmuv3_0@event\\=0x1016@ - armv8_pmuv3_0@event\\=0x100d@) / BR_MIS_PRED\000\000Other branch L3 topdown metric\000Other branch L3 topdown metric\000\000\000\000\00000"
/* offset=16579 */ "\000other_flush\000TopDownL3\0001 - nuke_flush\000\000Other flush L3 topdown metric\000Other flush L3 topdown metric\000\000\000\000\00000"
/* offset=16684 */ "\000pc_buffer_stall\000TopDownL3\000armv8_pmuv3_0@event\\=0x2005@ / CPU_CYCLES\000\000PC buffer stall L3 topdown metric\000PC buffer stall L3 topdown metric\000\000\000\000\00000"
/* offset=16828 */ "\000pop_branch\000TopDownL3\000armv8_pmuv3_0@event\\=0x100d@ / BR_MIS_PRED\000\000Pop branch L3 topdown metric\000Pop branch L3 topdown metric\000\000\000\000\00000"
/* offset=16958 */ "\000ptag_stall\000TopDownL3\000(armv8_pmuv3_0@event\\=0x2006@ + armv8_pmuv3_0@event\\=0x2007@ + armv8_pmuv3_0@event\\=0x2008@) / CPU_CYCLES\000\000Ptag stall L3 topdown metric\000Ptag stall L3 topdown metric\000\000\000\000\00000"
/* offset=17151 */ "\000push_branch\000TopDownL3\000(armv8_pmuv3_0@event\\=0x1013@ + armv8_pmuv3_0@event\\=0x1016@) / BR_MIS_PRED\000\000Push branch L3 topdown metric\000Push branch L3 topdown metric\000\000\000\000\00000"
/* offset=17317 */ "\000retired_ipc\000PEutilization\000INST_RETIRED / CPU_CYCLES\000\000Architecturally executed Instructions Per Cycle (IPC)\000\000\000\000\000\00000"
/* offset=17432 */ "\000retired_rate\000PEutilization\000OP_RETIRED / OP_SPEC\000\000Of all the micro-operations issued, what percentage are retired(committed)\000\000100%\000\000\000\00000"
/* offset=17568 */ "\000retiring\000Default;TopDownL1\000INST_RETIRED / (CPU_CYCLES * 4)\000\000Retiring L1 topdown metric\000Retiring L1 topdown metric\000\000\000\000TopDownL1\00000"
/* offset=17698 */ "\000retiring\000Default;TopdownL1\000op_retired / op_spec * (1 - (stall_slot if #slots - 5 else stall_slot - cpu_cycles) / (#slots * cpu_cycles))\000\000Retiring L1 topdown metric\000\000100%\000\000\000TopdownL1\00000"
/* offset=17883 */ "\000rob_stall\000TopDownL3\000armv8_pmuv3_0@event\\=0x2004@ / CPU_CYCLES\000\000Rob stall L3 topdown metric\000Rob stall L3 topdown metric\000\000\000\000\00000"
/* offset=18009 */ "\000saveopq_stall\000TopDownL3\000armv8_pmuv3_0@event\\=0x201e@ / CPU_CYCLES\000\000SaveOpQ stall L3 topdown metric\000SaveOpQ stall L3 topdown metric\000\000\000\000\00000"
/* offset=18147 */ "\000scalar_fp_percentage\000Operation_Mix\000VFP_SPEC / INST_SPEC * 100\000\000This metric measures scalar floating point operations as a percentage of operations speculatively executed\000\0001percent of operations\000\000\000\00000"
/* offset=18347 */ "\000simd_percentage\000Operation_Mix\000ASE_SPEC / INST_SPEC * 100\000\000This metric measures advanced SIMD operations as a percentage of total operations speculatively executed\000\0001percent of operations\000\000\000\00000"
/* offset=18540 */ "\000sp_flush\000TopDownL3\000armv8_pmuv3_0@event\\=0x1001@ * 5 / CPU_CYCLES\000\000Static predictor flush L3 topdown metric\000Static predictor flush L3 topdown metric\000\000\000\000\00000"
/* offset=18695 */ "\000spec_ipc\000PEutilization\000INST_SPEC / CPU_CYCLES\000\000Speculatively executed Instructions Per Cycle (IPC)\000\000\000\000\000\00000"
/* offset=18802 */ "\000store_bound\000TopDownL3\000armv8_pmuv3_0@event\\=0x7005@ / CPU_CYCLES\000\000Store bound L3 topdown metric\000Store bound L3 topdown metric\000\000\000\000\00000"
/* offset=18934 */ "\000store_percentage\000Operation_Mix\000ST_SPEC / INST_SPEC * 100\000\000This metric measures store operations as a percentage of operations speculatively executed\000\0001percent of operations\000\000\000\00000"
/* offset=19113 */ "\000store_spec_rate\000InstructionMix\000ST_SPEC / INST_SPEC\000\000The rate of store instructions speculatively executed to overall instructions speclatively executed\000\000100%\000\000\000\00000"
/* offset=19277 */ "\000sync_stall\000TopDownL3\000armv8_pmuv3_0@event\\=0x2010@ / CPU_CYCLES\000\000Sync stall L3 topdown metric\000Sync stall L3 topdown metric\000\000\000\000\00000"
/* offset=19406 */ "\000wasted_rate\000PEutilization\0001 - OP_RETIRED / OP_SPEC\000\000Of all the micro-operations issued, what percentage are not retired(committed)\000\000100%\000\000\000\00000"
/* offset=19549 */ "_0inst_commit\000\000other\000This event counts every cycle that no instruction was committed, but counts at the time when commits MOVPRFX only\000event=0x190\000\00000\000This event counts every cycle that no instruction was committed, but counts at the time when commits MOVPRFX only\000"
/* offset=19814 */ "_1inst_commit\000\000other\000This event counts every cycle that one instruction is committed\000event=0x191\000\00000\000This event counts every cycle that one instruction is committed\000"
/* offset=19979 */ "_2inst_commit\000\000other\000This event counts every cycle that two instructions are committed\000event=0x192\000\00000\000This event counts every cycle that two instructions are committed\000"
/* offset=20148 */ "_3inst_commit\000\000other\000This event counts every cycle that three instructions are committed\000event=0x193\000\00000\000This event counts every cycle that three instructions are committed\000"
/* offset=20321 */ "_4inst_commit\000\000other\000This event counts every cycle that four instructions are committed\000event=0x194\000\00000\000This event counts every cycle that four instructions are committed\000"
/* offset=20492 */ "act_cmd\000hisi_sccl,ddrc\000uncore ddrc\000DDRC active commands. Unit: hisi_sccl,ddrc \000config=0x5\000\00000\000DDRC active commands\000"
/* offset=20607 */ "agu_dep_stall\000\000pipeline\000Cycles there is an interlock for a load/store instruction waiting for data to calculate the address in the AGU\000event=0xe5\000\00000\000\000"
/* offset=20758 */ "ase_inst_spec\000\000instruction\000ASE operations speculatively executed\000event=0x8005\000\00000\000ASE operations speculatively executed\000"
/* offset=20878 */ "ase_spec\000\000instruction\000Operation speculatively executed, Advanced SIMD instruction\000event=0x74\000\00000\000Operation speculatively executed, Advanced SIMD instruction\000"
/* offset=21035 */ "ase_spec\000\000spec_operation\000Operation speculatively executed, Advanced SIMD instruction\000event=0x74\000\00000\000Counts speculatively executed Advanced SIMD operations excluding load, store and move micro-operations that move data to or from SIMD (vector) registers\000"
/* offset=21288 */ "ase_sve_int16_spec\000\000instruction\000Advanced SIMD and SVE 16-bit integer operations speculatively executed\000event=0x80e7\000\00000\000Advanced SIMD and SVE 16-bit integer operations speculatively executed\000"
/* offset=21479 */ "ase_sve_int32_spec\000\000instruction\000Advanced SIMD and SVE 32-bit integer operations speculatively executed\000event=0x80eb\000\00000\000Advanced SIMD and SVE 32-bit integer operations speculatively executed\000"
/* offset=21670 */ "ase_sve_int64_spec\000\000instruction\000Advanced SIMD and SVE 64-bit integer operations speculatively executed\000event=0x80ef\000\00000\000Advanced SIMD and SVE 64-bit integer operations speculatively executed\000"
/* offset=21861 */ "ase_sve_int8_spec\000\000instruction\000Advanced SIMD and SVE 8-bit integer operations speculatively executed\000event=0x80e3\000\00000\000Advanced SIMD and SVE 8-bit integer operations speculatively executed\000"
/* offset=22049 */ "ase_sve_int_spec\000\000sve\000Advanced SIMD and SVE integer Operations speculatively executed\000event=0x8043\000\00000\000Advanced SIMD and SVE integer Operations speculatively executed\000"
/* offset=22216 */ "ase_sve_ld_multi_spec\000\000sve\000Advanced SIMD and SVE contiguous load multiple vector Operations speculatively executed\000event=0x80a5\000\00000\000Advanced SIMD and SVE contiguous load multiple vector Operations speculatively executed\000"
/* offset=22436 */ "ase_sve_ld_spec\000\000sve\000Advanced SIMD and SVE load Operations speculatively executed\000event=0x8085\000\00000\000Advanced SIMD and SVE load Operations speculatively executed\000"
/* offset=22596 */ "ase_sve_st_multi_spec\000\000sve\000Advanced SIMD and SVE contiguous store multiple vector Operations speculatively executed\000event=0x80a6\000\00000\000Advanced SIMD and SVE contiguous store multiple vector Operations speculatively executed\000"
/* offset=22818 */ "ase_sve_st_spec\000\000sve\000Advanced SIMD and SVE store Operations speculatively executed\000event=0x8086\000\00000\000Advanced SIMD and SVE store Operations speculatively executed\000"
/* offset=22980 */ "back_invalid\000hisi_sccl,l3c\000uncore l3c\000Count of the number of L3C back invalid operations. Unit: hisi_sccl,l3c \000config=0x29\000\00000\000Count of the number of L3C back invalid operations\000"
/* offset=23158 */ "base_ld_reg_spec\000\000sve\000General-purpose register load Operations speculatively executed\000event=0x8089\000\00000\000General-purpose register load Operations speculatively executed\000"
/* offset=23325 */ "base_st_reg_spec\000\000sve\000General-purpose register store Operations speculatively executed\000event=0x808a\000\00000\000General-purpose register store Operations speculatively executed\000"
/* offset=23494 */ "bc_ld_spec\000\000instruction\000This event counts architecturally executed SIMD broadcast floating-point load operations\000event=0x11a\000\00000\000This event counts architecturally executed SIMD broadcast floating-point load operations\000"
/* offset=23712 */ "bi_num\000hisi_sccl,hha\000uncore hha\000Unit: hisi_sccl,hha \000config=0x23\000\00000\000\000"
/* offset=23782 */ "biu_ext_mem_req\000\000memory\000External memory request\000event=0xc1\000\00000\000External memory request\000"
/* offset=23869 */ "biu_ext_mem_req_nc\000\000memory\000External memory request to non-cacheable memory\000event=0xc2\000\00000\000External memory request to non-cacheable memory\000"
/* offset=24007 */ "bp_l1_btb_correct\000\000branch\000L1 BTB Correction\000event=0x8a\000\00000\000\000"
/* offset=24067 */ "bp_l2_btb_correct\000\000branch\000L2 BTB Correction\000event=0x8b\000\00000\000\000"
/* offset=24127 */ "bpu_branch_no_hit\000\000core imp def\000Predictable branch speculatively executed, unpredicted\000event=0x115\000\00000\000Predictable branch speculatively executed, unpredicted\000"
/* offset=24285 */ "bpu_btb_update\000\000core imp def\000Predictable branch speculatively executed, unpredicted, that mispredict\000event=0x11c\000\00000\000Predictable branch speculatively executed, unpredicted, that mispredict\000"
/* offset=24474 */ "bpu_conditional_branch_hit_btb\000\000core imp def\000Predictable conditional branch speculatively executed that hit any level of BTB\000event=0x111\000\00000\000Predictable conditional branch speculatively executed that hit any level of BTB\000"
/* offset=24695 */ "bpu_conditional_branch_hit_btb_and_mispredict\000\000core imp def\000Predictable conditional branch speculatively executed that hit any level of BTB that (direction) mispredict\000event=0x117\000\00000\000Predictable conditional branch speculatively executed that hit any level of BTB that (direction) mispredict\000"
/* offset=24987 */ "bpu_flush_mem_fault\000\000core imp def\000Flushes due to memory hazards\000event=0x121\000\00000\000Flushes due to memory hazards\000"
/* offset=25097 */ "bpu_hit_btb\000\000core imp def\000Predictable branch speculatively executed that hit any level of BTB\000event=0x110\000\00000\000Predictable branch speculatively executed that hit any level of BTB\000"
/* offset=25275 */ "bpu_hit_btb_and_mispredict\000\000core imp def\000Predictable branch speculatively executed that hit any level of BTB that mispredict\000event=0x116\000\00000\000Predictable branch speculatively executed that hit any level of BTB that mispredict\000"
/* offset=25500 */ "bpu_hit_indirect_predictor\000\000core imp def\000Predictable taken branch speculatively executed that hit any level of BTB that access the indirect predictor\000event=0x112\000\00000\000Predictable taken branch speculatively executed that hit any level of BTB that access the indirect predictor\000"
/* offset=25775 */ "bpu_hit_rsb\000\000core imp def\000Predictable taken branch speculatively executed that hit any level of BTB that access the return predictor\000event=0x113\000\00000\000Predictable taken branch speculatively executed that hit any level of BTB that access the return predictor\000"
/* offset=26031 */ "bpu_hit_rsb_and_mispredict\000\000core imp def\000Predictable taken branch speculatively executed that hit any level of BTB that access the return predictor that mispredict\000event=0x119\000\00000\000Predictable taken branch speculatively executed that hit any level of BTB that access the return predictor that mispredict\000"
/* offset=26334 */ "bpu_indirect_branch_hit_btb_and_mispredict\000\000core imp def\000Predictable taken branch speculatively executed that hit any level of BTB that access the indirect predictor that mispredict\000event=0x118\000\00000\000Predictable taken branch speculatively executed that hit any level of BTB that access the indirect predictor that mispredict\000"
/* offset=26657 */ "bpu_miss_rsb_and_mispredict\000\000core imp def\000Predictable taken branch speculatively executed that hit any level of BTB that access the overflow/underflow return predictor that mispredict\000event=0x11a\000\00000\000Predictable taken branch speculatively executed that hit any level of BTB that access the overflow/underflow return predictor that mispredict\000"
/* offset=26999 */ "bpu_no_prediction_mispredict\000\000core imp def\000Predictable branch speculatively executed, unpredicted, that mispredict\000event=0x11b\000\00000\000Predictable branch speculatively executed, unpredicted, that mispredict\000"
/* offset=27202 */ "bpu_rsb_full_stall\000\000core imp def\000Count predict pipe stalls due to speculative return address predictor full\000event=0x11d\000\00000\000Count predict pipe stalls due to speculative return address predictor full\000"
/* offset=27401 */ "bpu_unconditional_branch_miss_btb\000\000core imp def\000Predictable unconditional branch speculatively executed that did not hit any level of BTB\000event=0x114\000\00000\000Predictable unconditional branch speculatively executed that did not hit any level of BTB\000"
/* offset=27645 */ "br_comp_wait\000\000other\000This event counts every cycle that no instruction was committed and the oldest and uncommitted instruction is a branch instruction\000event=0x18b\000\00000\000This event counts every cycle that no instruction was committed and the oldest and uncommitted instruction is a branch instruction\000"
/* offset=27943 */ "br_cond\000\000branch\000Conditional branch executed\000event=0xc9\000\00000\000\000"
/* offset=28003 */ "br_cond_mis_pred\000\000branch\000Conditional branch mis-predicted.This event counts when any branch which can be predicted by the conditional predictor is retired, and has mis-predicted the condition. This event still counts when branch prediction is disabled due to the MMU being off. Conditional indirect branches which correctly predicted the condition but mis-predicted on the address do not count this event\000event=0xcc\000\00000\000Conditional branch mis-predicted.This event counts when any branch which can be predicted by the conditional predictor is retired, and has mis-predicted the condition. This event still counts when branch prediction is disabled due to the MMU being off. Conditional indirect branches which correctly predicted the condition but mis-predicted on the address do not count this event\000"
/* offset=28803 */ "br_cond_mis_pred\000\000branch\000Conditional branch mispredicted. This event counts when any branch that the conditional predictor can predict is retired and has mispredicted the condition. This event still counts when branch prediction is disabled due to the MMU being off. Conditional indirect branches that correctly predict the condition but mispredict the address do not count\000event=0xcc\000\00000\000Conditional branch mispredicted. This event counts when any branch that the conditional predictor can predict is retired and has mispredicted the condition. This event still counts when branch prediction is disabled due to the MMU being off. Conditional indirect branches that correctly predict the condition but mispredict the address do not count\000"
/* offset=29541 */ "br_cond_mispred\000\000branch\000Conditional branch mispredicted\000event=0xcc\000\00000\000\000"
/* offset=29613 */ "br_cond_pred\000\000branch\000Predicted conditional branch executed. This event counts when any branch that the conditional predictor can predict is retired. This event still counts when branch prediction is disabled due to the Memory Management Unit (MMU) being off\000event=0xc9\000\00000\000Predicted conditional branch executed. This event counts when any branch that the conditional predictor can predict is retired. This event still counts when branch prediction is disabled due to the Memory Management Unit (MMU) being off\000"
/* offset=30123 */ "br_cond_pred\000\000branch\000Predicted conditional branch executed.This event counts when any branch which can be predicted by the conditional predictor is retired. This event still counts when branch prediction is disabled due to the MMU being off\000event=0xc9\000\00000\000Predicted conditional branch executed.This event counts when any branch which can be predicted by the conditional predictor is retired. This event still counts when branch prediction is disabled due to the MMU being off\000"
/* offset=30599 */ "br_immed_retired\000\000instruction\000Instruction architecturally executed, immediate branch\000event=0xd\000\00000\000Instruction architecturally executed, immediate branch\000"
/* offset=30753 */ "br_immed_spec\000\000branch\000Branch speculatively executed, immediate branch\000event=0x78\000\00000\000Branch speculatively executed, immediate branch\000"
/* offset=30886 */ "br_immed_spec\000\000instruction\000Branch speculatively executed, immediate branch\000event=0x78\000\00000\000Branch speculatively executed, immediate branch\000"
/* offset=31024 */ "br_immed_spec\000\000spec_operation\000Branch speculatively executed, immediate branch\000event=0x78\000\00000\000Counts immediate branch operations which are speculatively executed\000"
/* offset=31185 */ "br_indirect_addr_mis_pred\000\000branch\000Indirect branch mis-predicted due to address mis-compare.This event counts when any indirect branch which can be predicted by the BTAC is retired, was taken and correctly predicted the condition, and has mispredicted the address. This event still counts when branch prediction is disabled due to the MMU being off\000event=0xcb\000\00000\000Indirect branch mis-predicted due to address mis-compare.This event counts when any indirect branch which can be predicted by the BTAC is retired, was taken and correctly predicted the condition, and has mispredicted the address. This event still counts when branch prediction is disabled due to the MMU being off\000"
/* offset=31862 */ "br_indirect_addr_mis_pred\000\000branch\000Indirect branch mispredicted due to address miscompare. This event counts when any indirect branch that the BTAC can predict is retired, was taken, correctly predicted the condition, and has mispredicted the address. This event still counts when branch prediction is disabled due to the MMU being off\000event=0xcb\000\00000\000Indirect branch mispredicted due to address miscompare. This event counts when any indirect branch that the BTAC can predict is retired, was taken, correctly predicted the condition, and has mispredicted the address. This event still counts when branch prediction is disabled due to the MMU being off\000"
/* offset=32513 */ "br_indirect_addr_pred\000\000branch\000Indirect branch with predicted address executed. This event counts when any indirect branch that the BTAC can predict is retired, was taken, and correctly predicted the condition. This event still counts when branch prediction is disabled due to the MMU being off\000event=0xcd\000\00000\000Indirect branch with predicted address executed. This event counts when any indirect branch that the BTAC can predict is retired, was taken, and correctly predicted the condition. This event still counts when branch prediction is disabled due to the MMU being off\000"
/* offset=33086 */ "br_indirect_addr_pred\000\000branch\000Indirect branch with predicted address executed.This event counts when any indirect branch which can be predicted by the BTAC is retired, was taken and correctly predicted the condition. This event still counts when branch prediction is disabled due to the MMU being off\000event=0xcd\000\00000\000Indirect branch with predicted address executed.This event counts when any indirect branch which can be predicted by the BTAC is retired, was taken and correctly predicted the condition. This event still counts when branch prediction is disabled due to the MMU being off\000"
/* offset=33673 */ "br_indirect_mis_pred\000\000branch\000Indirect branch mis-predicted.This event counts when any indirect branch which can be predicted by the BTAC is retired, and has mispredicted for either the condition or the address. This event still counts when branch prediction is disabled due to the MMU being off\000event=0xca\000\00000\000Indirect branch mis-predicted.This event counts when any indirect branch which can be predicted by the BTAC is retired, and has mispredicted for either the condition or the address. This event still counts when branch prediction is disabled due to the MMU being off\000"
/* offset=34249 */ "br_indirect_mis_pred\000\000branch\000Indirect branch mispredicted. This event counts when any indirect branch that the Branch Target Address Cache (BTAC) can predict is retired and has mispredicted either the condition or the address. This event still counts when branch prediction is disabled due to the MMU being off\000event=0xca\000\00000\000Indirect branch mispredicted. This event counts when any indirect branch that the Branch Target Address Cache (BTAC) can predict is retired and has mispredicted either the condition or the address. This event still counts when branch prediction is disabled due to the MMU being off\000"
/* offset=34857 */ "br_indirect_mispred\000\000branch\000Indirect branch mispredicted\000event=0xca\000\00000\000\000"
/* offset=34930 */ "br_indirect_mispred_addr\000\000branch\000Indirect branch mispredicted because of address miscompare\000event=0xcb\000\00000\000\000"
/* offset=35038 */ "br_indirect_spec\000\000branch\000Branch speculatively executed, indirect branch\000event=0x7a\000\00000\000Branch speculatively executed, indirect branch\000"
/* offset=35172 */ "br_indirect_spec\000\000instruction\000Branch speculatively executed, indirect branch\000event=0x7a\000\00000\000Branch speculatively executed, indirect branch\000"
/* offset=35311 */ "br_indirect_spec\000\000spec_operation\000Branch speculatively executed, indirect branch\000event=0x7a\000\00000\000Counts indirect branch operations including procedure returns, which are speculatively executed. This includes operations that force a software change of the PC, other than exception-generating operations.  Eg: BR Xn, RET\000"
/* offset=35628 */ "br_mis_pred\000\000branch\000Branch mispredicted\000event=0x10\000\00000\000Mispredicted or not predicted branch speculatively executed\000"
/* offset=35743 */ "br_mis_pred\000\000branch\000Mispredicted or not predicted branch speculatively executed\000event=0x10\000\00000\000Mispredicted or not predicted branch speculatively executed\000"
/* offset=35898 */ "br_mis_pred\000\000branch\000Mispredicted or not predicted branch speculatively executed\000event=0x10\000\00000\000This event counts any predictable branch instruction which is mispredicted either due to dynamic misprediction or because the MMU is off and the branches are statically predicted not taken\000"
/* offset=36182 */ "br_mis_pred\000\000spec_operation\000Mispredicted or not predicted branch speculatively executed\000event=0x10\000\00000\000Counts branches which are speculatively executed and mispredicted\000"
/* offset=36351 */ "br_mis_pred_retired\000\000instruction\000Instruction architecturally executed, mispredicted branch\000event=0x22\000\00000\000Instruction architecturally executed, mispredicted branch\000"
/* offset=36515 */ "br_mis_pred_retired\000\000instruction\000Instruction architecturally executed, mispredicted branch\000event=0x22\000\00000\000This event counts any branch counted by BR_RETIRED which is not correctly predicted and causes a pipeline flush\000"
/* offset=36733 */ "br_mis_pred_retired\000\000retired\000Instruction architecturally executed, mispredicted branch\000event=0x22\000\00000\000Counts branches counted by BR_RETIRED which were mispredicted and caused a pipeline flush\000"
/* offset=36925 */ "br_pred\000\000branch\000Predictable branch speculatively executed\000event=0x12\000\00000\000Predictable branch speculatively executed\000"
/* offset=37040 */ "br_pred\000\000branch\000Predictable branch speculatively executed\000event=0x12\000\00000\000This event counts all predictable branches\000"
/* offset=37156 */ "br_pred\000\000branch\000Predictable branch\000event=0x12\000\00000\000Predictable branch speculatively executed\000"
/* offset=37248 */ "br_pred\000\000spec_operation\000Predictable branch speculatively executed\000event=0x12\000\00000\000Counts branches speculatively executed and were predicted right\000"
/* offset=37393 */ "br_retired\000\000instruction\000Instruction architecturally executed, branch\000event=0x21\000\00000\000Instruction architecturally executed, branch\000"
/* offset=37522 */ "br_retired\000\000instruction\000Instruction architecturally executed, branch\000event=0x21\000\00000\000This event counts all branches, taken or not. This excludes exception entries, debug entries and CCFAIL branches\000"
/* offset=37719 */ "br_retired\000\000retired\000Instruction architecturally executed, branch\000event=0x21\000\00000\000Counts architecturally executed branches, whether the branch is taken or not. Instructions that explicitly write to the PC are also counted\000"
/* offset=37939 */ "br_return_addr_mis_pred\000\000branch\000Procedure return mis-predicted due to address mis-compare.This event counts when any procedure return which can be predicted by the CRS is retired, was taken and correctly predicted the condition, and has mispredicted the address. This event still counts when branch prediction is disabled due to the MMU being off\000event=0xcf\000\00000\000Procedure return mis-predicted due to address mis-compare.This event counts when any procedure return which can be predicted by the CRS is retired, was taken and correctly predicted the condition, and has mispredicted the address. This event still counts when branch prediction is disabled due to the MMU being off\000"
/* offset=38616 */ "br_return_addr_mis_pred\000\000branch\000Procedure return mispredicted due to address miscompare. This event counts when any procedure return that the call-return stack can predict is retired, was taken, correctly predicted the condition, and has mispredicted the address. This event still counts when branch prediction is disabled due to the MMU being off\000event=0xcf\000\00000\000Procedure return mispredicted due to address miscompare. This event counts when any procedure return that the call-return stack can predict is retired, was taken, correctly predicted the condition, and has mispredicted the address. This event still counts when branch prediction is disabled due to the MMU being off\000"
/* offset=39295 */ "br_return_addr_pred\000\000branch\000Procedure return with predicted address executed. This event counts when any procedure return that the call-return stack can predict is retired, was taken, and correctly predicted the condition. This event still counts when branch prediction is disabled due to the MMU being off\000event=0xce\000\00000\000Procedure return with predicted address executed. This event counts when any procedure return that the call-return stack can predict is retired, was taken, and correctly predicted the condition. This event still counts when branch prediction is disabled due to the MMU being off\000"
/* offset=39896 */ "br_return_addr_pred\000\000branch\000Procedure return with predicted address executed.This event counts when any procedure return which can be predicted by the CRS is retired, was taken and correctly predicted the condition. This event still counts when branch prediction is disabled due to the MMU being off\000event=0xce\000\00000\000Procedure return with predicted address executed.This event counts when any procedure return which can be predicted by the CRS is retired, was taken and correctly predicted the condition. This event still counts when branch prediction is disabled due to the MMU being off\000"
/* offset=40483 */ "br_return_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, procedure return\000event=0xe\000\00000\000Instruction architecturally executed, condition code check pass, procedure return\000"
/* offset=40692 */ "br_return_spec\000\000branch\000Branch speculatively executed, procedure return\000event=0x79\000\00000\000Branch speculatively executed, procedure return\000"
/* offset=40826 */ "br_return_spec\000\000instruction\000Branch speculatively executed, procedure return\000event=0x79\000\00000\000Branch speculatively executed, procedure return\000"
/* offset=40965 */ "br_return_spec\000\000spec_operation\000Branch speculatively executed, procedure return\000event=0x79\000\00000\000Counts procedure return operations (RET) which are speculatively executed\000"
/* offset=41133 */ "breakpoint_match\000\000core imp def\000GPC detected a Breakpoint instruction match\000event=0xd501\000\00000\000GPC detected a Breakpoint instruction match\000"
/* offset=41269 */ "btb_mis_pred\000\000cache\000BTB misprediction\000event=0x102\000\00000\000Branch target buffer misprediction\000"
/* offset=41358 */ "bus_access\000\000bus\000Attributable Bus access\000event=0x19\000\00000\000Attributable Bus access\000"
/* offset=41437 */ "bus_access\000\000bus\000Attributable Bus access\000event=0x19\000\00000\000Counts memory transactions issued by the CPU to the external bus, including snoop requests and snoop responses. Each beat of data is counted individually\000"
/* offset=41646 */ "bus_access\000\000bus\000Attributable Bus access\000event=0x19\000\00000\000This event counts for every beat of data transferred over the data channels between the core and the SCU. If both read and write data beats are transferred on a given cycle, this event is counted twice on that cycle. This event counts the sum of BUS_ACCESS_RD and BUS_ACCESS_WR\000"
/* offset=41979 */ "bus_access_normal\000\000bus\000Bus access, Normal\000event=0x64\000\00000\000Bus access, Normal\000"
/* offset=42055 */ "bus_access_not_shared\000\000bus\000Bus access, not Normal, Cacheable, Shareable\000event=0x63\000\00000\000Bus access, not Normal, Cacheable, Shareable\000"
/* offset=42187 */ "bus_access_periph\000\000bus\000Bus access, peripheral\000event=0x65\000\00000\000Bus access, peripheral\000"
/* offset=42271 */ "bus_access_rd\000\000bus\000Bus access read\000event=0x60\000\00000\000Bus access read\000"
/* offset=42337 */ "bus_access_rd\000\000bus\000Bus access read\000event=0x60\000\00000\000Counts memory read transactions seen on the external bus. Each beat of data is counted individually\000"
/* offset=42487 */ "bus_access_rd\000\000core imp def\000Bus access read\000event=0x60\000\00000\000Bus access read\000"
/* offset=42562 */ "bus_access_shared\000\000bus\000Bus access, Normal, Cacheable, Shareable\000event=0x62\000\00000\000Bus access, Normal, Cacheable, Shareable\000"
/* offset=42682 */ "bus_access_wr\000\000bus\000Bus access write\000event=0x61\000\00000\000Bus access write\000"
/* offset=42750 */ "bus_access_wr\000\000bus\000Bus access write\000event=0x61\000\00000\000Counts memory write transactions seen on the external bus. Each beat of data is counted individually\000"
/* offset=42902 */ "bus_access_wr\000\000core imp def\000Bus access write\000event=0x61\000\00000\000Bus access write\000"
/* offset=42979 */ "bus_cycles\000\000bus\000Bus cycle\000event=0x1d\000\00000\000Bus cycle\000"
/* offset=43030 */ "bus_cycles\000\000bus\000Bus cycle\000event=0x1d\000\00000\000Counts bus cycles in the CPU. Bus cycles represent a clock cycle in which a transaction could be sent or received on the interface from the CPU to the external bus. Since that interface is driven at the same clock speed as the CPU, this event is a duplicate of CPU_CYCLES\000"
/* offset=43343 */ "bus_cycles\000\000bus\000Bus cycle\000event=0x1d\000\00000\000This event duplicates CPU_CYCLES\000"
/* offset=43417 */ "bus_read_total_mem\000\000bus\000This event counts read transactions from measured CMG local memory to measured CMG\000event=0x316\000\00000\000This event counts read transactions from measured CMG local memory to measured CMG\000"
/* offset=43623 */ "bus_read_total_pci\000\000bus\000This event counts read transactions from PCI controller to measured CMG\000event=0x315\000\00000\000This event counts read transactions from PCI controller to measured CMG\000"
/* offset=43807 */ "bus_read_total_tofu\000\000bus\000This event counts read transactions from tofu controller to measured CMG\000event=0x314\000\00000\000This event counts read transactions from tofu controller to measured CMG\000"
/* offset=43994 */ "bus_write_total_cmg0\000\000bus\000This event counts write transactions from measured CMG to CMG0, if measured CMG is not CMG0\000event=0x318\000\00000\000This event counts write transactions from measured CMG to CMG0, if measured CMG is not CMG0\000"
/* offset=44220 */ "bus_write_total_cmg1\000\000bus\000This event counts write transactions from measured CMG to CMG1, if measured CMG is not CMG1\000event=0x319\000\00000\000This event counts write transactions from measured CMG to CMG1, if measured CMG is not CMG1\000"
/* offset=44446 */ "bus_write_total_cmg2\000\000bus\000This event counts write transactions from measured CMG to CMG2, if measured CMG is not CMG2\000event=0x31a\000\00000\000This event counts write transactions from measured CMG to CMG2, if measured CMG is not CMG2\000"
/* offset=44672 */ "bus_write_total_cmg3\000\000bus\000This event counts write transactions from measured CMG to CMG3, if measured CMG is not CMG3\000event=0x31b\000\00000\000This event counts write transactions from measured CMG to CMG3, if measured CMG is not CMG3\000"
/* offset=44898 */ "bus_write_total_mem\000\000bus\000This event counts write transactions from measured CMG to measured CMG local memory\000event=0x31e\000\00000\000This event counts write transactions from measured CMG to measured CMG local memory\000"
/* offset=45107 */ "bus_write_total_pci\000\000bus\000This event counts write transactions from measured CMG to PCI controller\000event=0x31d\000\00000\000This event counts write transactions from measured CMG to PCI controller\000"
/* offset=45294 */ "bus_write_total_tofu\000\000bus\000This event counts write transactions from measured CMG to tofu controller\000event=0x31c\000\00000\000This event counts write transactions from measured CMG to tofu controller\000"
/* offset=45484 */ "bx_stall\000\000pipeline\000BX stalled\000event=0x10c\000\00000\000BX Op non-issue\000"
/* offset=45546 */ "cid_write_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000event=0xb\000\00000\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000"
/* offset=45761 */ "cid_write_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000event=0xb\000\00000\000This event only counts writes to CONTEXTIDR in AArch32 state, and via the CONTEXTIDR_EL1 mnemonic in AArch64 state\000"
/* offset=46006 */ "cid_write_retired\000\000instruction\000Write to CONTEXTIDR\000event=0xb\000\00000\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000"
/* offset=46156 */ "cid_write_retired\000\000retired\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000event=0xb\000\00000\000Counts architecturally executed writes to the CONTEXTIDR register, which usually contain the kernel PID and can be output with hardware trace\000"
/* offset=46424 */ "cnt_cycles\000\000bus\000Constant frequency cycles\000event=0x4004\000\00000\000Constant frequency cycles.  The counter increments at a constant frequency equal to the rate of increment of the system counter, CNTPCT_EL0\000"
/* offset=46623 */ "cp15_utlb_refill\000\000cache\000Level 1 CP15 TLB refill\000event=0xe8\000\00000\000Level 1 CP15 TLB refill\000"
/* offset=46710 */ "cpa_cycles\000hisi_sicl,cpa\000uncore cpa\000count of CPA cycles. Unit: hisi_sicl,cpa \000config=0\0000x00000030\00000\000\000"
/* offset=46812 */ "cpa_p0_rd_dat\000hisi_sicl,cpa\000uncore cpa\000Number of read ops transmitted by the P0 port. Unit: hisi_sicl,cpa \000config=0xe2\0000x00000030\00000\000\000"
/* offset=46946 */ "cpa_p0_rd_dat_32b\000hisi_sicl,cpa\000uncore cpa\000Number of read ops transmitted by the P0 port which size is 32 bytes. Unit: hisi_sicl,cpa \000config=0x84\0000x00000030\00000\000\000"
/* offset=47107 */ "cpa_p0_rd_dat_64b\000hisi_sicl,cpa\000uncore cpa\000Number of read ops transmitted by the P0 port which size is 64 bytes. Unit: hisi_sicl,cpa \000config=0x83\0000x00000030\00000\000\000"
/* offset=47268 */ "cpa_p0_wr_dat\000hisi_sicl,cpa\000uncore cpa\000Number of write ops transmitted by the P0 port. Unit: hisi_sicl,cpa \000config=0xe1\0000x00000030\00000\000\000"
/* offset=47403 */ "cpa_p1_rd_dat\000hisi_sicl,cpa\000uncore cpa\000Number of read ops transmitted by the P1 port. Unit: hisi_sicl,cpa \000config=0x62\0000x00000030\00000\000\000"
/* offset=47537 */ "cpa_p1_rd_dat_32b\000hisi_sicl,cpa\000uncore cpa\000Number of read ops transmitted by the P1 port which size is 32 bytes. Unit: hisi_sicl,cpa \000config=0x4\0000x00000030\00000\000\000"
/* offset=47697 */ "cpa_p1_rd_dat_64b\000hisi_sicl,cpa\000uncore cpa\000Number of read ops transmitted by the P1 port which size is 64 bytes. Unit: hisi_sicl,cpa \000config=0x3\0000x00000030\00000\000\000"
/* offset=47857 */ "cpa_p1_wr_dat\000hisi_sicl,cpa\000uncore cpa\000Number of write ops transmitted by the P1 port. Unit: hisi_sicl,cpa \000config=0x61\0000x00000030\00000\000\000"
/* offset=47992 */ "cpu_cycles\000\000bus\000Cycle\000event=0x11\000\00000\000Cycle\000"
/* offset=48035 */ "cpu_cycles\000\000bus\000The number of core clock cycles\000event=0x11\000\00000\000The number of core clock cycles\000"
/* offset=48130 */ "cpu_cycles\000\000clock\000Cycle\000event=0x11\000\00000\000The number of core clock cycles\000"
/* offset=48201 */ "cpu_cycles\000\000cycle\000Cycle\000event=0x11\000\00000\000Cycle\000"
/* offset=48246 */ "cpu_cycles\000\000general\000Cycle\000event=0x11\000\00000\000Counts CPU clock cycles (not timer cycles). The clock measured by this event is defined as the physical clock driving the CPU logic\000"
/* offset=48419 */ "crypto_spec\000\000instruction\000Operation speculatively executed, Cryptographic instruction\000event=0x77\000\00000\000Operation speculatively executed, Cryptographic instruction\000"
/* offset=48579 */ "crypto_spec\000\000spec_operation\000Operation speculatively executed, Cryptographic instruction\000event=0x77\000\00000\000Counts speculatively executed cryptographic operations except for PMULL and VMULL operations\000"
/* offset=48775 */ "cti_trigout4\000\000trace\000Cross-trigger Interface output trigger 4\000event=0x4018\000\00000\000Cross-trigger Interface output trigger 4\000"
/* offset=48894 */ "cti_trigout5\000\000trace\000Cross-trigger Interface output trigger 5 \000event=0x4019\000\00000\000Cross-trigger Interface output trigger 5 \000"
/* offset=49015 */ "cti_trigout6\000\000trace\000Cross-trigger Interface output trigger 6\000event=0x401a\000\00000\000Cross-trigger Interface output trigger 6\000"
/* offset=49134 */ "cti_trigout7\000\000trace\000Cross-trigger Interface output trigger 7\000event=0x401b\000\00000\000Cross-trigger Interface output trigger 7\000"
/* offset=49253 */ "d_lsu_slot_full\000\000pipeline\000Duration for which all slots in the Load-Store Unit (LSU) are busy\000event=0xd3\000\00000\000Duration for which all slots in the Load-Store Unit (LSU) are busy\000"
/* offset=49428 */ "d_lsu_slot_full\000\000pipeline\000Duration for which all slots in the Load-Store Unit are busy\000event=0xd3\000\00000\000Duration for which all slots in the Load-Store Unit are busy\000"
/* offset=49591 */ "dcache_self_hit_vipt\000\000cache\000Data cache hit in itself due to VIPT aliasing\000event=0xec\000\00000\000Data cache hit in itself due to VIPT aliasing\000"
/* offset=49726 */ "dczva_spec\000\000instruction\000This event counts architecturally executed zero blocking operations due to the 'DC ZVA' instruction\000event=0x9f\000\00000\000This event counts architecturally executed zero blocking operations due to the 'DC ZVA' instruction\000"
/* offset=49965 */ "de_iq_full\000\000pipeline\000Duration for which all slots in the Data Engine issue queue are busy\000event=0xda\000\00000\000Duration for which all slots in the Data Engine issue queue are busy\000"
/* offset=50139 */ "de_iq_full\000\000pipeline\000Duration for which all slots in the data engine issue queue are busy. This event is set every time that the data engine rename has at least one valid instruction, excluding No Operations (NOPs), that cannot move to the issue stage because accpt_instr is LOW\000event=0xda\000\00000\000Duration for which all slots in the data engine issue queue are busy. This event is set every time that the data engine rename has at least one valid instruction, excluding No Operations (NOPs), that cannot move to the issue stage because accpt_instr is LOW\000"
/* offset=50691 */ "decode_dep_stall\000\000pipeline\000Cycles the DPU IQ is empty and there is a pre-decode error being processed\000event=0xe3\000\00000\000\000"
/* offset=50809 */ "decode_stall\000\000pipeline\000Decode starved\000event=0x108\000\00000\000Decode starved for instruction cycle\000"
/* offset=50900 */ "dispatch_blocked.any\000\000other\000Memory cluster signals to block micro-op dispatch for any reason\000event=0x9,period=200000,umask=0x20\000\00000\000\000"
/* offset=51033 */ "dispatch_stall\000\000pipeline\000Dispatch stalled\000event=0x109\000\00000\000Op dispatch stalled cycle\000"
/* offset=51117 */ "dmb_spec\000\000instruction\000Barrier speculatively executed, DMB\000event=0x7e\000\00000\000Barrier speculatively executed, DMB\000"
/* offset=51226 */ "dmb_spec\000\000spec_operation\000Barrier speculatively executed, DMB\000event=0x7e\000\00000\000Counts DMB operations that are speculatively issued to the Load/Store unit in the CPU. This event does not count implied barriers from load acquire/store release operations\000"
/* offset=51475 */ "dp_iq_full\000\000pipeline\000Duration for which all slots in the data processing issue queue are busy. This event counts the cycles where all slots in the DP0 and DP1 IQs are full with micro-operations waiting for issuing, and the despatch stage is not empty\000event=0xd9\000\00000\000Duration for which all slots in the data processing issue queue are busy. This event counts the cycles where all slots in the DP0 and DP1 IQs are full with micro-operations waiting for issuing, and the despatch stage is not empty\000"
/* offset=51971 */ "dp_iq_full\000\000pipeline\000Duration for which all slots in the data processing issue queue are busy\000event=0xd9\000\00000\000Duration for which all slots in the data processing issue queue are busy\000"
/* offset=52153 */ "dp_spec\000\000instruction\000Operation speculatively executed, integer data processing\000event=0x73\000\00000\000Operation speculatively executed, integer data processing\000"
/* offset=52305 */ "dp_spec\000\000spec_operation\000Operation speculatively executed, integer data processing\000event=0x73\000\00000\000Counts speculatively executed logical or arithmetic instructions such as MOV/MVN operations\000"
/* offset=52494 */ "dpu_br_cond_mis\000\000dpu\000Instruction retired, conditional branch, mispredicted\000event=0xea\000\00000\000Instruction retired, conditional branch, mispredicted\000"
/* offset=52638 */ "dpu_br_cond_retired\000\000instruction\000Instruction retired, conditional branch\000event=0xe8\000\00000\000Instruction retired, conditional branch\000"
/* offset=52766 */ "dpu_br_ind_mis\000\000dpu\000Instruction retired, indirect branch, mispredicted\000event=0xe9\000\00000\000Instruction retired, indirect branch, mispredicted\000"
/* offset=52903 */ "dpu_mem_err_dcu\000\000dpu\000Memory error (any type) from DCU\000event=0xec\000\00000\000Memory error (any type) from DCU\000"
/* offset=53005 */ "dpu_mem_err_ifu\000\000dpu\000Memory error (any type) from IFU\000event=0xeb\000\00000\000Memory error (any type) from IFU\000"
/* offset=53107 */ "dpu_mem_err_tlb\000\000dpu\000Memory error (any type) from TLB\000event=0xed\000\00000\000Memory error (any type) from TLB\000"
/* offset=53209 */ "dsb_spec\000\000instruction\000Barrier speculatively executed, DSB\000event=0x7d\000\00000\000Barrier speculatively executed, DSB\000"
/* offset=53318 */ "dsb_spec\000\000spec_operation\000Barrier speculatively executed, DSB\000event=0x7d\000\00000\000Counts DSB operations that are speculatively issued to Load/Store unit in the CPU\000"
/* offset=53476 */ "dtb_miss\000\000cache\000DTB miss\000event=0x104\000\00000\000DTB miss\000"
/* offset=53526 */ "dtlb_walk\000\000cache\000Access to data TLB causes a translation table walk\000event=0x34\000\00000\000Access to data TLB causes a translation table walk\000"
/* offset=53660 */ "dtlb_walk\000\000cache\000Access to data TLB causes a translation table walk\000event=0x34\000\00000\000This event counts on any data access which causes L2D_TLB_REFILL to count\000"
/* offset=53817 */ "dtlb_walk\000\000tlb\000Access to data TLB causes a translation table walk\000event=0x34\000\00000\000Counts data memory translation table walks caused by a miss in the L2 TLB driven by a memory access. Note that partial translations that also cause a table walk are counted. This event does not count table walks caused by TLB maintenance operations\000"
/* offset=54147 */ "ea_core\000\000other\000This event counts energy consumption per cycle of core\000event=0x1e0\000\00000\000This event counts energy consumption per cycle of core\000"
/* offset=54288 */ "ea_l2\000\000cache\000This event counts energy consumption per cycle of L2 cache\000event=0x3e0\000\00000\000This event counts energy consumption per cycle of L2 cache\000"
/* offset=54435 */ "ea_memory\000\000memory\000This event counts energy consumption per cycle of CMG local memory\000event=0x3e8\000\00000\000This event counts energy consumption per cycle of CMG local memory\000"
/* offset=54603 */ "eaga_val\000\000pipeline\000This event counts valid cycles of EAGA pipeline\000event=0x1a0\000\00000\000This event counts valid cycles of EAGA pipeline\000"
/* offset=54734 */ "eagb_val\000\000pipeline\000This event counts valid cycles of EAGB pipeline\000event=0x1a1\000\00000\000This event counts valid cycles of EAGB pipeline\000"
/* offset=54865 */ "edir-hit\000hisi_sccl,hha\000uncore hha\000Unit: hisi_sccl,hha \000config=0x43\000\00000\000\000"
/* offset=54937 */ "edir-home-migrate\000hisi_sccl,hha\000uncore hha\000Unit: hisi_sccl,hha \000config=0x4d\000\00000\000\000"
/* offset=55018 */ "edir-lookup\000hisi_sccl,hha\000uncore hha\000Unit: hisi_sccl,hha \000config=0x41\000\00000\000\000"
/* offset=55093 */ "effective_inst_spec\000\000instruction\000This event counts architecturally executed instructions, excluding the MOVPRFX instruction\000event=0x121\000\00000\000This event counts architecturally executed instructions, excluding the MOVPRFX instruction\000"
/* offset=55324 */ "eist_trans\000\000other\000Number of Enhanced Intel SpeedStep(R) Technology (EIST) transitions\000event=0x3a,period=200000,umask=0x0\000\00000\000\000"
/* offset=55450 */ "etm_ext_out0\000\000etm\000ETM trace unit output 0\000event=0xde\000\00000\000ETM trace unit output 0\000"
/* offset=55531 */ "etm_ext_out1\000\000etm\000ETM trace unit output 1\000event=0xdf\000\00000\000ETM trace unit output 1\000"
/* offset=55612 */ "eu_comp_wait\000\000other\000This event counts every cycle that no instruction was committed and the oldest and uncommitted instruction is an integer or floating-point/SIMD instruction\000event=0x189\000\00000\000This event counts every cycle that no instruction was committed and the oldest and uncommitted instruction is an integer or floating-point/SIMD instruction\000"
/* offset=55960 */ "event-hyphen\000uncore_cbox\000uncore\000UNC_CBO_HYPHEN. Unit: uncore_cbox \000event=0xe0,umask=0x00\000\00000\000UNC_CBO_HYPHEN\000"
/* offset=56068 */ "event-two-hyph\000uncore_cbox\000uncore\000UNC_CBO_TWO_HYPH. Unit: uncore_cbox \000event=0xc0,umask=0x00\000\00000\000UNC_CBO_TWO_HYPH\000"
/* offset=56182 */ "exa_val\000\000pipeline\000This event counts valid cycles of EXA pipeline\000event=0x1a2\000\00000\000This event counts valid cycles of EXA pipeline\000"
/* offset=56310 */ "exb_val\000\000pipeline\000This event counts valid cycles of EXB pipeline\000event=0x1a3\000\00000\000This event counts valid cycles of EXB pipeline\000"
/* offset=56438 */ "exc_dabort\000\000core imp def\000Exception taken, Data Abort and SError\000event=0x84\000\00000\000Exception taken, Data Abort and SError\000"
/* offset=56556 */ "exc_dabort\000\000exception\000Exception taken, Data Abort and SError\000event=0x84\000\00000\000Counts exceptions that are taken locally and are caused by data aborts or SErrors. Conditions that could cause those exceptions are attempting to read or write memory where the MMU generates a fault, attempting to read or write memory with a misaligned address, interrupts from the nSEI inputs and internally generated SErrors\000"
/* offset=56959 */ "exc_dabort\000\000exception\000Exception taken, Data Abort and SError\000event=0x84\000\00000\000Exception taken, Data Abort and SError\000"
/* offset=57074 */ "exc_fiq\000\000core imp def\000Exception taken, FIQ\000event=0x87\000\00000\000Exception taken, FIQ\000"
/* offset=57153 */ "exc_fiq\000\000exception\000Exception taken, FIQ\000event=0x87\000\00000\000Counts FIQ exceptions including the virtual FIQs that are taken locally\000"
/* offset=57280 */ "exc_fiq\000\000exception\000Exception taken, FIQ\000event=0x87\000\00000\000Exception taken, FIQ\000"
/* offset=57356 */ "exc_fiq\000\000other\000Exception taken, FIQ\000event=0x87\000\00000\000Exception taken, FIQ\000"
/* offset=57428 */ "exc_hvc\000\000core imp def\000Exception taken, Hypervisor Call\000event=0x8a\000\00000\000Exception taken, Hypervisor Call\000"
/* offset=57531 */ "exc_hvc\000\000exception\000Exception taken, Hypervisor Call\000event=0x8a\000\00000\000Counts HVC exceptions taken to EL2\000"
/* offset=57633 */ "exc_hvc\000\000exception\000Exception taken, Hypervisor Call\000event=0x8a\000\00000\000Exception taken, Hypervisor Call\000"
/* offset=57733 */ "exc_irq\000\000core imp def\000Exception taken, IRQ\000event=0x86\000\00000\000Exception taken, IRQ\000"
/* offset=57812 */ "exc_irq\000\000exception\000Exception taken, IRQ\000event=0x86\000\00000\000Counts IRQ exceptions including the virtual IRQs that are taken locally\000"
/* offset=57939 */ "exc_irq\000\000exception\000Exception taken, IRQ\000event=0x86\000\00000\000Exception taken, IRQ\000"
/* offset=58015 */ "exc_irq\000\000other\000Exception taken, IRQ\000event=0x86\000\00000\000Exception taken, IRQ\000"
/* offset=58087 */ "exc_pabort\000\000core imp def\000Exception taken, Instruction Abort\000event=0x83\000\00000\000Exception taken, Instruction Abort\000"
/* offset=58197 */ "exc_pabort\000\000exception\000Exception taken, Instruction Abort\000event=0x83\000\00000\000Counts synchronous exceptions that are taken locally and caused by Instruction Aborts\000"
/* offset=58355 */ "exc_pabort\000\000exception\000Exception taken, Instruction Abort\000event=0x83\000\00000\000Exception taken, Instruction Abort\000"
/* offset=58462 */ "exc_return\000\000exception\000Instruction architecturally executed, condition check pass, exception return\000event=0xa\000\00000\000Counts any architecturally executed exception return instructions. Eg: AArch64: ERET\000"
/* offset=58660 */ "exc_return\000\000exception\000Instruction architecturally executed, condition check pass, exception return\000event=0xa\000\00000\000Instruction architecturally executed, condition check pass, exception return\000"
/* offset=58850 */ "exc_return\000\000instruction\000Instruction architecturally executed, condition check pass, exception return\000event=0xa\000\00000\000Instruction architecturally executed, condition check pass, exception return\000"
/* offset=59042 */ "exc_smc\000\000core imp def\000Exception taken, Secure Monitor Call\000event=0x88\000\00000\000Exception taken, Secure Monitor Call\000"
/* offset=59153 */ "exc_smc\000\000exception\000Exception taken, Secure Monitor Call\000event=0x88\000\00000\000Counts SMC exceptions take to EL3\000"
/* offset=59258 */ "exc_smc\000\000exception\000Exception taken, Secure Monitor Call\000event=0x88\000\00000\000Exception taken, Secure Monitor Call\000"
/* offset=59366 */ "exc_svc\000\000core imp def\000Exception taken, Supervisor Call\000event=0x82\000\00000\000Exception taken, Supervisor Call\000"
/* offset=59469 */ "exc_svc\000\000exception\000Exception taken, Supervisor Call\000event=0x82\000\00000\000Counts SVC exceptions taken locally\000"
/* offset=59572 */ "exc_svc\000\000exception\000Exception taken, Supervisor Call\000event=0x82\000\00000\000Exception taken, Supervisor Call\000"
/* offset=59672 */ "exc_taken\000\000exception\000Exception taken\000event=0x9\000\00000\000Counts any taken architecturally visible exceptions such as IRQ, FIQ, SError, and other synchronous exceptions. Exceptions are counted whether or not they are taken locally\000"
/* offset=59896 */ "exc_taken\000\000exception\000Exception taken\000event=0x9\000\00000\000Exception taken\000"
/* offset=59963 */ "exc_trap_dabort\000\000core imp def\000Exception taken, Data Abort or SError not taken locally\000event=0x8c\000\00000\000Exception taken, Data Abort or SError not taken locally\000"
/* offset=60120 */ "exc_trap_dabort\000\000exception\000Exception taken, Data Abort or SError not taken locally\000event=0x8c\000\00000\000Counts exceptions which are traps not taken locally and are caused by Data Aborts or SError interrupts. Conditions that could cause those exceptions are:\n\n1. Attempting to read or write memory where the MMU generates a fault,\n2. Attempting to read or write memory with a misaligned address,\n3. Interrupts from the SEI input.\n4. internally generated SErrors\000"
/* offset=60575 */ "exc_trap_dabort\000\000exception\000Exception taken, Data Abort or SError not taken locally\000event=0x8c\000\00000\000Exception taken, Data Abort or SError not taken locally\000"
/* offset=60729 */ "exc_trap_fiq\000\000core imp def\000Exception taken, FIQ not taken locally\000event=0x8f\000\00000\000Exception taken, FIQ not taken locally\000"
/* offset=60849 */ "exc_trap_fiq\000\000exception\000Exception taken, FIQ not taken locally\000event=0x8f\000\00000\000Counts FIQs which are not taken locally but taken from EL0, EL1,\n or EL2 to EL3 (which would be the normal behavior for FIQs when not executing\n in EL3)\000"
/* offset=61080 */ "exc_trap_fiq\000\000exception\000Exception taken, FIQ not taken locally\000event=0x8f\000\00000\000Exception taken, FIQ not taken locally\000"
/* offset=61197 */ "exc_trap_hyp\000\000exception\000Number of Traps to hypervisor\000event=0xdc\000\00000\000Number of Traps to hypervisor\000"
/* offset=61296 */ "exc_trap_hyp\000\000exception\000Number of traps to hypervisor. This event counts the number of exception traps taken to EL2, excluding HVC instructions. This event is set every time that an exception is executed because of a decoded trap to the hypervisor. CCFAIL exceptions and traps caused by HVC instructions are excluded. This event is not counted when it is accessible from Non-secure EL0 or EL1\000event=0xdc\000\00000\000Number of traps to hypervisor. This event counts the number of exception traps taken to EL2, excluding HVC instructions. This event is set every time that an exception is executed because of a decoded trap to the hypervisor. CCFAIL exceptions and traps caused by HVC instructions are excluded. This event is not counted when it is accessible from Non-secure EL0 or EL1\000"
/* offset=62073 */ "exc_trap_irq\000\000core imp def\000Exception taken, IRQ not taken locally\000event=0x8e\000\00000\000Exception taken, IRQ not taken locally\000"
/* offset=62193 */ "exc_trap_irq\000\000exception\000Exception taken, IRQ not taken locally\000event=0x8e\000\00000\000Counts IRQ exceptions including the virtual IRQs that are not taken locally\000"
/* offset=62347 */ "exc_trap_irq\000\000exception\000Exception taken, IRQ not taken locally\000event=0x8e\000\00000\000Exception taken, IRQ not taken locally\000"
/* offset=62464 */ "exc_trap_other\000\000core imp def\000Exception taken, Other traps not taken locally\000event=0x8d\000\00000\000Exception taken, Other traps not taken locally\000"
/* offset=62602 */ "exc_trap_other\000\000exception\000Exception taken, Other traps not taken locally\000event=0x8d\000\00000\000Counts the number of synchronous trap exceptions which are not taken locally and are not SVC, SMC, HVC, data aborts, Instruction Aborts, or interrupts\000"
/* offset=62841 */ "exc_trap_other\000\000exception\000Exception taken, Other traps not taken locally\000event=0x8d\000\00000\000Exception taken, Other traps not taken locally\000"
/* offset=62976 */ "exc_trap_pabort\000\000core imp def\000Exception taken, Instruction Abort not taken locally\000event=0x8b\000\00000\000Exception taken, Instruction Abort not taken locally\000"
/* offset=63127 */ "exc_trap_pabort\000\000exception\000Exception taken, Instruction Abort not taken locally\000event=0x8b\000\00000\000Counts exceptions which are traps not taken locally and are caused by Instruction Aborts. For example, attempting to execute an instruction with a misaligned PC\000"
/* offset=63383 */ "exc_trap_pabort\000\000exception\000Exception taken, Instruction Abort not taken locally\000event=0x8b\000\00000\000Exception taken, Instruction Abort not taken locally\000"
/* offset=63531 */ "exc_undef\000\000core imp def\000Exception taken, Other synchronous\000event=0x81\000\00000\000Exception taken, Other synchronous\000"
/* offset=63640 */ "exc_undef\000\000exception\000Exception taken, Other synchronous\000event=0x81\000\00000\000Counts the number of synchronous exceptions which are taken locally that are due to attempting to execute an instruction that is UNDEFINED. Attempting to execute instruction bit patterns that have not been allocated. Attempting to execute instructions when they are disabled. Attempting to execute instructions at an inappropriate Exception level. Attempting to execute an instruction when the value of PSTATE.IL is 1\000"
/* offset=64129 */ "exc_undef\000\000exception\000Exception taken, Other synchronous\000event=0x81\000\00000\000Exception taken, Other synchronous\000"
/* offset=64235 */ "exe_stall_cycle\000\000core imp def\000Cycles of that the number of issue ups are less than 4\000event=0x7001\000\00000\000Cycles of that the number of issuing micro operations are less than 4\000"
/* offset=64407 */ "ext_mem_req\000\000memory\000External memory request\000event=0xc0\000\00000\000\000"
/* offset=64467 */ "ext_mem_req_nc\000\000memory\000Non-cacheable external memory request\000event=0xc1\000\00000\000\000"
/* offset=64544 */ "ext_snoop\000\000cache\000SCU Snooped data from another CPU for this CPU\000event=0xc8\000\00000\000\000"
/* offset=64624 */ "fetch_bubble\000\000core imp def\000Instructions can receive, but not send\000event=0x2014\000\00000\000Instructions can receive, but not send\000"
/* offset=64746 */ "fl_comp_wait\000\000other\000This event counts every cycle that no instruction was committed and the oldest and uncommitted instruction is a floating-point/SIMD instruction\000event=0x18a\000\00000\000This event counts every cycle that no instruction was committed and the oldest and uncommitted instruction is a floating-point/SIMD instruction\000"
/* offset=65070 */ "fla_val\000\000pipeline\000This event counts valid cycles of FLA pipeline\000event=0x1a4\000\00000\000This event counts valid cycles of FLA pipeline\000"
/* offset=65198 */ "fla_val_prd_cnt\000\000pipeline\000This event counts the number of 1's in the predicate bits of request in FLA pipeline, where it is corrected so that it becomes 16 when all bits are 1\000event=0x1b4\000\00000\000This event counts the number of 1's in the predicate bits of request in FLA pipeline, where it is corrected so that it becomes 16 when all bits are 1\000"
/* offset=65540 */ "flb_val\000\000pipeline\000This event counts valid cycles of FLB pipeline\000event=0x1a5\000\00000\000This event counts valid cycles of FLB pipeline\000"
/* offset=65668 */ "flb_val_prd_cnt\000\000pipeline\000This event counts the number of 1's in the predicate bits of request in FLB pipeline, where it is corrected so that it becomes 16 when all bits are 1\000event=0x1b5\000\00000\000This event counts the number of 1's in the predicate bits of request in FLB pipeline, where it is corrected so that it becomes 16 when all bits are 1\000"
/* offset=66010 */ "flux_rcmd\000hisi_sccl,ddrc\000uncore ddrc\000DDRC read commands. Unit: hisi_sccl,ddrc \000config=0x3\000\00000\000DDRC read commands\000"
/* offset=66123 */ "flux_rd\000hisi_sccl,ddrc\000uncore ddrc\000DDRC total read operations. Unit: hisi_sccl,ddrc \000config=0x1\000\00000\000DDRC total read operations\000"
/* offset=66250 */ "flux_wcmd\000hisi_sccl,ddrc\000uncore ddrc\000DDRC write commands. Unit: hisi_sccl,ddrc \000config=0x2\000\00000\000DDRC write commands\000"
/* offset=66365 */ "flux_wr\000hisi_sccl,ddrc\000uncore ddrc\000DDRC total write operations. Unit: hisi_sccl,ddrc \000config=0\000\00000\000DDRC total write operations\000"
/* offset=66492 */ "fp_cvt_spec\000\000sve\000floating-point convert Operations speculatively executed\000event=0x8038\000\00000\000floating-point convert Operations speculatively executed\000"
/* offset=66640 */ "fp_dp_fixed_ops_spec\000\000sve\000Non-scalable double-precision floating-point element Operations speculatively executed\000event=0x80c7\000\00000\000Non-scalable double-precision floating-point element Operations speculatively executed\000"
/* offset=66857 */ "fp_dp_scale_ops_spec\000\000sve\000Scalable double-precision floating-point element Operations speculatively executed\000event=0x80c6\000\00000\000Scalable double-precision floating-point element Operations speculatively executed\000"
/* offset=67066 */ "fp_dp_spec\000\000instruction\000Floating-point double-precision operations speculatively executed\000event=0x801c\000\00000\000Floating-point double-precision operations speculatively executed\000"
/* offset=67239 */ "fp_fixed_ops_spec\000\000instruction\000Non-scalable floating-point element Operations speculatively executed\000event=0x80c1\000\00000\000Non-scalable floating-point element Operations speculatively executed\000"
/* offset=67427 */ "fp_fixed_ops_spec\000\000sve\000Non-scalable floating-point element Operations speculatively executed\000event=0x80c1\000\00000\000Non-scalable floating-point element Operations speculatively executed\000"
/* offset=67607 */ "fp_fma_spec\000\000sve\000Floating-point FMA Operations speculatively executed\000event=0x8028\000\00000\000Floating-point FMA Operations speculatively executed\000"
/* offset=67747 */ "fp_hp_fixed_ops_spec\000\000sve\000Non-scalable half-precision floating-point element Operations speculatively executed\000event=0x80c3\000\00000\000Non-scalable half-precision floating-point element Operations speculatively executed\000"
/* offset=67960 */ "fp_hp_scale_ops_spec\000\000sve\000Scalable half-precision floating-point element Operations speculatively executed\000event=0x80c2\000\00000\000Scalable half-precision floating-point element Operations speculatively executed\000"
/* offset=68165 */ "fp_hp_spec\000\000instruction\000Floating-point half-precision operations speculatively executed\000event=0x8014\000\00000\000Floating-point half-precision operations speculatively executed\000"
/* offset=68334 */ "fp_ld_spec\000\000instruction\000This event counts architecturally executed NOSIMD load operations that using SIMD&FP registers\000event=0x112\000\00000\000This event counts architecturally executed NOSIMD load operations that using SIMD&FP registers\000"
/* offset=68564 */ "fp_mv_spec\000\000instruction\000This event counts architecturally executed floating-point move operations\000event=0x105\000\00000\000This event counts architecturally executed floating-point move operations\000"
/* offset=68752 */ "fp_recpe_spec\000\000sve\000Floating-point reciprocal estimate Operations speculatively executed\000event=0x8034\000\00000\000Floating-point reciprocal estimate Operations speculatively executed\000"
/* offset=68926 */ "fp_scale_ops_spec\000\000instruction\000Scalable floating-point element Operations speculatively executed\000event=0x80c0\000\00000\000Scalable floating-point element Operations speculatively executed\000"
/* offset=69106 */ "fp_scale_ops_spec\000\000sve\000Scalable floating-point element Operations speculatively executed\000event=0x80c0\000\00000\000Scalable floating-point element Operations speculatively executed\000"
/* offset=69278 */ "fp_sp_fixed_ops_spec\000\000sve\000Non-scalable single-precision floating-point element Operations speculatively executed\000event=0x80c5\000\00000\000Non-scalable single-precision floating-point element Operations speculatively executed\000"
/* offset=69495 */ "fp_sp_scale_ops_spec\000\000sve\000Scalable single-precision floating-point element Operations speculatively executed\000event=0x80c4\000\00000\000Scalable single-precision floating-point element Operations speculatively executed\000"
/* offset=69704 */ "fp_sp_spec\000\000instruction\000Floating-point single-precision operations speculatively executed\000event=0x8018\000\00000\000Floating-point single-precision operations speculatively executed\000"
/* offset=69877 */ "fp_spec\000\000sve\000Floating-point Operations speculatively executed\000event=0x8010\000\00000\000Floating-point Operations speculatively executed\000"
/* offset=70005 */ "fp_st_spec\000\000instruction\000This event counts architecturally executed NOSIMD store operations that using SIMD&FP registers\000event=0x113\000\00000\000This event counts architecturally executed NOSIMD store operations that using SIMD&FP registers\000"
/* offset=70237 */ "fsu_cancel\000\000core imp def\000Uops canceled (load cancels)\000event=0xd404\000\00000\000Uops canceled (load cancels)\000"
/* offset=70337 */ "fsu_clock_off_cycles\000\000clock\000FSU clocking gated off cycle\000event=0x101\000\00000\000FSU clocking gated off cycle\000"
/* offset=70439 */ "fsu_div_sqrt_stall\000\000core imp def\000Count scheduler stalls due to divide/sqrt\000event=0xd405\000\00000\000Count scheduler stalls due to divide/sqrt\000"
/* offset=70573 */ "fsu_fsx_issued\000\000core imp def\000Uops issued by the scheduler on pipe X\000event=0xd401\000\00000\000Uops issued by the scheduler on pipe X\000"
/* offset=70697 */ "fsu_fsy_issued\000\000core imp def\000Uops issued by the scheduler on pipe Y\000event=0xd402\000\00000\000Uops issued by the scheduler on pipe Y\000"
/* offset=70821 */ "fsu_fsz_issued\000\000core imp def\000Uops issued by the scheduler on pipe Z\000event=0xd403\000\00000\000Uops issued by the scheduler on pipe Z\000"
/* offset=70945 */ "fsu_issued\000\000core imp def\000Uops issued by the FSU scheduler\000event=0xd400\000\00000\000Uops issued by the FSU scheduler\000"
/* offset=71053 */ "fx_stall\000\000pipeline\000FX stalled\000event=0x10f\000\00000\000FX Op non-issue\000"
/* offset=71115 */ "gpc_flush\000\000core imp def\000Flushes\000event=0x120\000\00000\000Flushes\000"
/* offset=71171 */ "gpc_swob_drain\000\000core imp def\000Number of SWOB drains\000event=0xd500\000\00000\000Number of SWOB drains\000"
/* offset=71261 */ "hisi_sicl,cpa\000cpa_p0_avg_bw\000CPA\000(cpa_p0_wr_dat * 64 + cpa_p0_rd_dat_64b * 64 + cpa_p0_rd_dat_32b * 32) / cpa_cycles\000\000Average bandwidth of CPA Port 0. Unit: hisi_sicl,cpa \000\000\0000x00000030\000\000\00000"
/* offset=71449 */ "hisi_sicl,cpa\000cpa_p1_avg_bw\000CPA\000(cpa_p1_wr_dat * 64 + cpa_p1_rd_dat_64b * 64 + cpa_p1_rd_dat_32b * 32) / cpa_cycles\000\000Average bandwidth of CPA Port 1. Unit: hisi_sicl,cpa \000\000\0000x00000030\000\000\00000"
/* offset=71637 */ "hit_on_prf\000\000core imp def\000Hit on prefetched data\000event=0x6014\000\00000\000Hit on prefetched data\000"
/* offset=71725 */ "i_btac_ram_rd\000\000cache\000Number of ways read in the instruction BTAC RAM\000event=0xc4\000\00000\000Number of ways read in the instruction BTAC RAM\000"
/* offset=71857 */ "i_data_ram_rd\000\000cache\000Number of ways read in the instruction cache - Data RAM\000event=0xc3\000\00000\000Number of ways read in the instruction cache - Data RAM\000"
/* offset=72005 */ "i_tag_ram_rd\000\000cache\000Number of ways read in the instruction cache - Tag RAM\000event=0xc2\000\00000\000Number of ways read in the instruction cache - Tag RAM\000"
/* offset=72150 */ "ic_dep_stall\000\000pipeline\000Cycles the DPU IQ is empty and there is an instruction cache miss being processed\000event=0xe1\000\00000\000\000"
/* offset=72271 */ "icf_early_mis_pred\000\000core imp def\000Early mispredict\000event=0xd100\000\00000\000Early mispredict\000"
/* offset=72355 */ "icf_feq_full\000\000core imp def\000FEQ full cycles\000event=0xd101\000\00000\000FEQ full cycles\000"
/* offset=72431 */ "icf_inst_fifo_full\000\000core imp def\000Instruction FIFO Full\000event=0xd102\000\00000\000Instruction FIFO Full\000"
/* offset=72525 */ "icf_inst_spec_decode\000\000core imp def\000Macro-ops speculatively decoded\000event=0x11f\000\00000\000Macro-ops speculatively decoded\000"
/* offset=72640 */ "icf_pc_fifo_full\000\000core imp def\000PC FIFO Full\000event=0xd105\000\00000\000PC FIFO Full\000"
/* offset=72714 */ "icf_stall\000\000core imp def\000ICF sent 0 instructions to IDR this cycle\000event=0xd104\000\00000\000ICF sent 0 instructions to IDR this cycle\000"
/* offset=72839 */ "idr_stall_bob_id\000\000core imp def\000Stall due to BOB ID\000event=0xd200\000\00000\000Stall due to BOB ID\000"
/* offset=72927 */ "idr_stall_flush\000\000core imp def\000Dispatch stall due to flush (6 cycles)\000event=0xd206\000\00000\000Dispatch stall due to flush (6 cycles)\000"
/* offset=73052 */ "idr_stall_fsu_sched\000\000core imp def\000Dispatch stall due to FSU scheduler entries\000event=0xd204\000\00000\000Dispatch stall due to FSU scheduler entries\000"
/* offset=73191 */ "idr_stall_ixu_sched\000\000core imp def\000Dispatch stall due to IXU scheduler entries\000event=0xd203\000\00000\000Dispatch stall due to IXU scheduler entries\000"
/* offset=73330 */ "idr_stall_lob_id\000\000core imp def\000Dispatch stall due to LOB entries\000event=0xd201\000\00000\000Dispatch stall due to LOB entries\000"
/* offset=73446 */ "idr_stall_rob_id\000\000core imp def\000Dispatch stall due to ROB entries\000event=0xd205\000\00000\000Dispatch stall due to ROB entries\000"
/* offset=73562 */ "idr_stall_sob_id\000\000core imp def\000Dispatch stall due to SOB entries\000event=0xd202\000\00000\000Dispatch stall due to SOB entries\000"
/* offset=73678 */ "idr_stall_swob_full\000\000core imp def\000Number of SWOB drains triggered by system register write when SWOB full\000event=0xd20a\000\00000\000Number of SWOB drains triggered by system register write when SWOB full\000"
/* offset=73873 */ "idr_stall_swob_raw\000\000core imp def\000Number of SWOB drains triggered by system register or special-purpose register read-after-write or specific special-purpose register writes that cause SWOB drain\000event=0xd209\000\00000\000Number of SWOB drains triggered by system register or special-purpose register read-after-write or specific special-purpose register writes that cause SWOB drain\000"
/* offset=74247 */ "idr_stall_swob_timeout\000\000core imp def\000Number of SWOB drains triggered by timeout\000event=0xd208\000\00000\000Number of SWOB drains triggered by timeout\000"
/* offset=74387 */ "idr_stall_wfi\000\000core imp def\000Dispatch stall due to WFI\000event=0xd207\000\00000\000Dispatch stall due to WFI\000"
/* offset=74484 */ "iel_spec\000\000instruction\000This event counts architecturally executed inter-element manipulation operations\000event=0x109\000\00000\000This event counts architecturally executed inter-element manipulation operations\000"
/* offset=74684 */ "if_is_stall\000\000core imp def\000Instruction fetch stall cycles\000event=0x1044\000\00000\000Instruction fetch stall cycles\000"
/* offset=74789 */ "ifu_all_thrds_rdy\000\000ifu\000This thread and the other thread both ready for scheduling in if0\000event=0xdc\000\00000\000This thread and the other thread both ready for scheduling in if0\000"
/* offset=74959 */ "ifu_flushed_excl_tlb_miss\000\000ifu\000Thread flushed due to reasons other than TLB miss\000event=0xdb\000\00000\000Thread flushed due to reasons other than TLB miss\000"
/* offset=75105 */ "ifu_flushed_tlb_miss\000\000ifu\000Thread flushed due to TLB miss\000event=0xda\000\00000\000Thread flushed due to TLB miss\000"
/* offset=75208 */ "ifu_goto_idle\000\000ifu\000The thread moved from an active state to an inactive state (long-term sleep state, causing deallocation of some resources)\000event=0xe0\000\00000\000The thread moved from an active state to an inactive state (long-term sleep state, causing deallocation of some resources)\000"
/* offset=75488 */ "ifu_ic_lf_sp\000\000ifu\000I-Cache Speculative line fill\000event=0xe4\000\00000\000I-Cache Speculative line fill\000"
/* offset=75581 */ "ifu_ic_lookup_under_miss\000\000ifu\000I-Cache lookup under miss from other thread\000event=0xe1\000\00000\000I-Cache lookup under miss from other thread\000"
/* offset=75714 */ "ifu_ic_miss_under_miss\000\000ifu\000I-Cache miss under miss from other thread\000event=0xe2\000\00000\000I-Cache miss under miss from other thread\000"
/* offset=75841 */ "ifu_ic_miss_wait\000\000ifu\000I-Cache miss on an access from the prefetch block\000event=0xd0\000\00000\000I-Cache miss on an access from the prefetch block\000"
/* offset=75978 */ "ifu_instr_pushed\000\000ifu\000This thread pushed an instruction into the IQ\000event=0xe3\000\00000\000This thread pushed an instruction into the IQ\000"
/* offset=76107 */ "ifu_iutlb_miss_wait\000\000ifu\000Counts the cycles spent on a request for Level 2 TLB lookup after a Level 1l ITLB miss\000event=0xd1\000\00000\000Counts the cycles spent on a request for Level 2 TLB lookup after a Level 1l ITLB miss\000"
/* offset=76321 */ "ifu_micro_caddr_mispred\000\000ifu\000Micro-predictor address mispredict, with respect to if3/if4 predictor\000event=0xd3\000\00000\000Micro-predictor address mispredict, with respect to if3/if4 predictor\000"
/* offset=76505 */ "ifu_micro_cond_mispred\000\000ifu\000Micro-predictor conditional/direction mispredict, with respect to. if3/if4 predictor\000event=0xd2\000\00000\000Micro-predictor conditional/direction mispredict, with respect to. if3/if4 predictor\000"
/* offset=76718 */ "ifu_micro_correction\000\000ifu\000Micro-predictor correction\000event=0xd7\000\00000\000Micro-predictor correction\000"
/* offset=76813 */ "ifu_micro_hit\000\000ifu\000Micro-predictor hit with immediate redirect\000event=0xd4\000\00000\000Micro-predictor hit with immediate redirect\000"
/* offset=76935 */ "ifu_micro_neg_hit\000\000ifu\000Micro-predictor negative cache hit\000event=0xd6\000\00000\000Micro-predictor negative cache hit\000"
/* offset=77043 */ "ifu_micro_no_instr1\000\000ifu\000A 2nd instruction could have been pushed but was not because it was nonsequential\000event=0xd8\000\00000\000A 2nd instruction could have been pushed but was not because it was nonsequential\000"
/* offset=77247 */ "ifu_micro_no_pred\000\000ifu\000Micro-predictor miss\000event=0xd9\000\00000\000Micro-predictor miss\000"
/* offset=77327 */ "ifu_not_rdy_for_arb\000\000ifu\000This thread was not arbitrated because it was not ready for scheduling. For example, due to a cache miss or TLB miss\000event=0xdf\000\00000\000This thread was not arbitrated because it was not ready for scheduling. For example, due to a cache miss or TLB miss\000"
/* offset=77601 */ "ifu_win_arb_other_act\000\000ifu\000This thread was arbitrated when the other thread was also active, but not necessarily ready. For example, waiting for I-Cache or TLB\000event=0xde\000\00000\000This thread was arbitrated when the other thread was also active, but not necessarily ready. For example, waiting for I-Cache or TLB\000"
/* offset=77909 */ "ifu_win_arb_other_rdy\000\000ifu\000This thread was arbitrated when the other thread was also ready for scheduling\000event=0xdd\000\00000\000This thread was arbitrated when the other thread was also ready for scheduling\000"
/* offset=78109 */ "imx8_ddr\000imx8mm_ddr_read.all\000\000imx8mm_ddr.read_cycles * 4 * 4\000\000bytes all masters read from ddr based on read-cycles event. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MM\000\000\00000"
/* offset=78274 */ "imx8_ddr\000imx8mm_ddr_write.all\000\000imx8mm_ddr.write_cycles * 4 * 4\000\000bytes all masters write to ddr based on write-cycles event. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MM\000\000\00000"
/* offset=78441 */ "imx8_ddr\000imx8mn_ddr_read.all\000\000imx8mn_ddr.read_cycles * 4 * 2\000\000bytes all masters read from ddr based on read-cycles event. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MN\000\000\00000"
/* offset=78606 */ "imx8_ddr\000imx8mn_ddr_write.all\000\000imx8mn_ddr.write_cycles * 4 * 2\000\000bytes all masters write to ddr based on write-cycles event. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MN\000\000\00000"
/* offset=78773 */ "imx8_ddr\000imx8mp_ddr_read.2d\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x0071@\000\000bytes of gpu 2d read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=78933 */ "imx8_ddr\000imx8mp_ddr_read.3d\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x0070@\000\000bytes of gpu 3d read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=79093 */ "imx8_ddr\000imx8mp_ddr_read.a53\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x0000@\000\000bytes of a53 core read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=79256 */ "imx8_ddr\000imx8mp_ddr_read.all\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0xffff\\,axi_id\\=0x0000@\000\000bytes of all masters read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=79422 */ "imx8_ddr\000imx8mp_ddr_read.audio_dsp\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x0041@\000\000bytes of audio dsp read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=79592 */ "imx8_ddr\000imx8mp_ddr_read.audio_sdma2_burst\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x0063@\000\000bytes of audio sdma2_burst read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=79778 */ "imx8_ddr\000imx8mp_ddr_read.audio_sdma2_per\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x0062@\000\000bytes of audio sdma2_per read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=79960 */ "imx8_ddr\000imx8mp_ddr_read.audio_sdma3_burst\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x0065@\000\000bytes of audio sdma3_burst read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=80146 */ "imx8_ddr\000imx8mp_ddr_read.audio_sdma3_per\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x0064@\000\000bytes of audio sdma3_per read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=80328 */ "imx8_ddr\000imx8mp_ddr_read.audio_sdma_pif\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x0066@\000\000bytes of audio sdma_pif read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=80508 */ "imx8_ddr\000imx8mp_ddr_read.dewarp\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x006f@\000\000bytes of display dewarp read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=80680 */ "imx8_ddr\000imx8mp_ddr_read.hdmi_hdcp\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x0076@\000\000bytes of hdmi_tx tx_hdcp read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=80856 */ "imx8_ddr\000imx8mp_ddr_read.hdmi_hrv_mwr\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x0074@\000\000bytes of hdmi_tx hrv_mwr read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=81035 */ "imx8_ddr\000imx8mp_ddr_read.hdmi_lcdif\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x0075@\000\000bytes of hdmi_tx lcdif read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=81210 */ "imx8_ddr\000imx8mp_ddr_read.isi1\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x006a@\000\000bytes of display isi1 read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=81378 */ "imx8_ddr\000imx8mp_ddr_read.isi2\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x006b@\000\000bytes of display isi2 read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=81546 */ "imx8_ddr\000imx8mp_ddr_read.isi3\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x006c@\000\000bytes of display isi3 read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=81714 */ "imx8_ddr\000imx8mp_ddr_read.isp1\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x006d@\000\000bytes of display isp1 read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=81882 */ "imx8_ddr\000imx8mp_ddr_read.isp2\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x006e@\000\000bytes of display isp2 read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=82050 */ "imx8_ddr\000imx8mp_ddr_read.lcdif1\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x0068@\000\000bytes of display lcdif1 read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=82222 */ "imx8_ddr\000imx8mp_ddr_read.lcdif2\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x0069@\000\000bytes of display lcdif2 read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=82394 */ "imx8_ddr\000imx8mp_ddr_read.npu\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x0073@\000\000bytes of npu read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=82552 */ "imx8_ddr\000imx8mp_ddr_read.pci\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x007a@\000\000bytes of hsio pci read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=82715 */ "imx8_ddr\000imx8mp_ddr_read.supermix\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x000f\\,axi_id\\=0x0020@\000\000bytes of supermix(m7) core read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=82892 */ "imx8_ddr\000imx8mp_ddr_read.usb1\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x0078@\000\000bytes of hsio usb1 read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=83057 */ "imx8_ddr\000imx8mp_ddr_read.usb2\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x0079@\000\000bytes of hsio usb2 read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=83222 */ "imx8_ddr\000imx8mp_ddr_read.vpu1\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x007c@\000\000bytes of vpu1 read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=83382 */ "imx8_ddr\000imx8mp_ddr_read.vpu2\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x007d@\000\000bytes of vpu2 read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=83542 */ "imx8_ddr\000imx8mp_ddr_read.vpu3\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x007e@\000\000bytes of vpu3 read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=83702 */ "imx8_ddr\000imx8mp_ddr_write.2d\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x0071@\000\000bytes of gpu 2d write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=83863 */ "imx8_ddr\000imx8mp_ddr_write.3d\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x0070@\000\000bytes of gpu 3d write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=84024 */ "imx8_ddr\000imx8mp_ddr_write.a53\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x0000@\000\000bytes of a53 core write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=84188 */ "imx8_ddr\000imx8mp_ddr_write.all\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0xffff\\,axi_id\\=0x0000@\000\000bytes of all masters write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=84355 */ "imx8_ddr\000imx8mp_ddr_write.audio_dsp\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x0041@\000\000bytes of audio dsp write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=84526 */ "imx8_ddr\000imx8mp_ddr_write.audio_sdma2_burst\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x0063@\000\000bytes of audio sdma2_burst write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=84713 */ "imx8_ddr\000imx8mp_ddr_write.audio_sdma2_per\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x0062@\000\000bytes of audio sdma2_per write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=84896 */ "imx8_ddr\000imx8mp_ddr_write.audio_sdma3_burst\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x0065@\000\000bytes of audio sdma3_burst write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=85083 */ "imx8_ddr\000imx8mp_ddr_write.audio_sdma3_per\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x0064@\000\000bytes of audio sdma3_per write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=85266 */ "imx8_ddr\000imx8mp_ddr_write.audio_sdma_pif\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x0066@\000\000bytes of audio sdma_pif write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=85447 */ "imx8_ddr\000imx8mp_ddr_write.dewarp\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x006f@\000\000bytes of display dewarp write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=85620 */ "imx8_ddr\000imx8mp_ddr_write.hdmi_hdcp\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x0076@\000\000bytes of hdmi_tx tx_hdcp write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=85797 */ "imx8_ddr\000imx8mp_ddr_write.hdmi_hrv_mwr\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x0074@\000\000bytes of hdmi_tx hrv_mwr write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=85977 */ "imx8_ddr\000imx8mp_ddr_write.hdmi_lcdif\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x0075@\000\000bytes of hdmi_tx lcdif write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=86153 */ "imx8_ddr\000imx8mp_ddr_write.isi1\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x006a@\000\000bytes of display isi1 write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=86322 */ "imx8_ddr\000imx8mp_ddr_write.isi2\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x006b@\000\000bytes of display isi2 write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=86491 */ "imx8_ddr\000imx8mp_ddr_write.isi3\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x006c@\000\000bytes of display isi3 write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=86660 */ "imx8_ddr\000imx8mp_ddr_write.isp1\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x006d@\000\000bytes of display isp1 write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=86829 */ "imx8_ddr\000imx8mp_ddr_write.isp2\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x006e@\000\000bytes of display isp2 write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=86998 */ "imx8_ddr\000imx8mp_ddr_write.lcdif1\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x0068@\000\000bytes of display lcdif1 write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=87171 */ "imx8_ddr\000imx8mp_ddr_write.lcdif2\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x0069@\000\000bytes of display lcdif2 write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=87344 */ "imx8_ddr\000imx8mp_ddr_write.npu\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x0073@\000\000bytes of npu write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=87503 */ "imx8_ddr\000imx8mp_ddr_write.pci\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x007a@\000\000bytes of hsio pci write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=87667 */ "imx8_ddr\000imx8mp_ddr_write.supermix\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x000f\\,axi_id\\=0x0020@\000\000bytes of supermix(m7) write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=87840 */ "imx8_ddr\000imx8mp_ddr_write.usb1\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x0078@\000\000bytes of hsio usb1 write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=88006 */ "imx8_ddr\000imx8mp_ddr_write.usb2\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x0079@\000\000bytes of hsio usb2 write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=88172 */ "imx8_ddr\000imx8mp_ddr_write.vpu1\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x007c@\000\000bytes of vpu1 write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=88333 */ "imx8_ddr\000imx8mp_ddr_write.vpu2\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x007d@\000\000bytes of vpu2 write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=88494 */ "imx8_ddr\000imx8mp_ddr_write.vpu3\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x007e@\000\000bytes of vpu3 write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000"
/* offset=88655 */ "imx8_ddr\000imx8mq_ddr_read.all\000\000imx8mq_ddr.read_cycles * 4 * 4\000\000bytes all masters read from ddr based on read-cycles event. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MQ\000\000\00000"
/* offset=88820 */ "imx8_ddr\000imx8mq_ddr_write.all\000\000imx8mq_ddr.write_cycles * 4 * 4\000\000bytes all masters write to ddr based on write-cycles event. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MQ\000\000\00000"
/* offset=88987 */ "imx8mm_ddr.cycles\000imx8_ddr\000ddrc\000ddr cycles event. Unit: imx8_ddr \000event=0\000i.MX8MM\00000\000\000"
/* offset=89073 */ "imx8mm_ddr.read\000imx8_ddr\000ddrc\000ddr read event. Unit: imx8_ddr \000event=0x35\000i.MX8MM\00000\000\000"
/* offset=89158 */ "imx8mm_ddr.read_cycles\000imx8_ddr\000ddrc\000ddr read-cycles event. Unit: imx8_ddr \000event=0x2a\000i.MX8MM\00000\000\000"
/* offset=89257 */ "imx8mm_ddr.write\000imx8_ddr\000ddrc\000ddr write event. Unit: imx8_ddr \000event=0x38\000i.MX8MM\00000\000\000"
/* offset=89344 */ "imx8mm_ddr.write_cycles\000imx8_ddr\000ddrc\000ddr write-cycles event. Unit: imx8_ddr \000event=0x2b\000i.MX8MM\00000\000\000"
/* offset=89445 */ "imx8mn_ddr.cycles\000imx8_ddr\000ddrc\000ddr cycles event. Unit: imx8_ddr \000event=0\000i.MX8MN\00000\000\000"
/* offset=89531 */ "imx8mn_ddr.read\000imx8_ddr\000ddrc\000ddr read event. Unit: imx8_ddr \000event=0x35\000i.MX8MN\00000\000\000"
/* offset=89616 */ "imx8mn_ddr.read_cycles\000imx8_ddr\000ddrc\000ddr read-cycles event. Unit: imx8_ddr \000event=0x2a\000i.MX8MN\00000\000\000"
/* offset=89715 */ "imx8mn_ddr.write\000imx8_ddr\000ddrc\000ddr write event. Unit: imx8_ddr \000event=0x38\000i.MX8MN\00000\000\000"
/* offset=89802 */ "imx8mn_ddr.write_cycles\000imx8_ddr\000ddrc\000ddr write-cycles event. Unit: imx8_ddr \000event=0x2b\000i.MX8MN\00000\000\000"
/* offset=89903 */ "imx8mp_ddr.cycles\000imx8_ddr\000ddrc\000ddr cycles event. Unit: imx8_ddr \000event=0\000i.MX8MP\00000\000\000"
/* offset=89989 */ "imx8mp_ddr.read\000imx8_ddr\000ddrc\000ddr read event. Unit: imx8_ddr \000event=0x35\000i.MX8MP\00000\000\000"
/* offset=90074 */ "imx8mp_ddr.read_cycles\000imx8_ddr\000ddrc\000ddr read-cycles event. Unit: imx8_ddr \000event=0x2a\000i.MX8MP\00000\000\000"
/* offset=90173 */ "imx8mp_ddr.write\000imx8_ddr\000ddrc\000ddr write event. Unit: imx8_ddr \000event=0x38\000i.MX8MP\00000\000\000"
/* offset=90260 */ "imx8mp_ddr.write_cycles\000imx8_ddr\000ddrc\000ddr write-cycles event. Unit: imx8_ddr \000event=0x2b\000i.MX8MP\00000\000\000"
/* offset=90361 */ "imx8mq_ddr.cycles\000imx8_ddr\000ddrc\000ddr cycles event. Unit: imx8_ddr \000event=0\000i.MX8MQ\00000\000\000"
/* offset=90447 */ "imx8mq_ddr.read\000imx8_ddr\000ddrc\000ddr read event. Unit: imx8_ddr \000event=0x35\000i.MX8MQ\00000\000\000"
/* offset=90532 */ "imx8mq_ddr.read_cycles\000imx8_ddr\000ddrc\000ddr read-cycles event. Unit: imx8_ddr \000event=0x2a\000i.MX8MQ\00000\000\000"
/* offset=90631 */ "imx8mq_ddr.write\000imx8_ddr\000ddrc\000ddr write event. Unit: imx8_ddr \000event=0x38\000i.MX8MQ\00000\000\000"
/* offset=90718 */ "imx8mq_ddr.write_cycles\000imx8_ddr\000ddrc\000ddr write-cycles event. Unit: imx8_ddr \000event=0x2b\000i.MX8MQ\00000\000\000"
/* offset=90819 */ "inst_retired\000\000instruction\000Instruction architecturally executed\000event=0x8\000\00000\000Instruction architecturally executed\000"
/* offset=90933 */ "inst_retired\000\000instruction\000Instruction architecturally executed\000event=0x8\000\00000\000This event counts all retired instructions, including those that fail their condition check\000"
/* offset=91102 */ "inst_retired\000\000retired\000Instruction architecturally executed\000event=0x8\000\00000\000Counts instructions that have been architecturally executed\000"
/* offset=91235 */ "inst_spec\000\000instruction\000Operation speculatively executed\000event=0x1b\000\00000\000Operation speculatively executed\000"
/* offset=91339 */ "inst_spec\000\000spec_operation\000Operation speculatively executed\000event=0x1b\000\00000\000Counts operations that have been speculatively executed\000"
/* offset=91469 */ "iq_is_empty\000\000core imp def\000Instruction queue is empty\000event=0x1043\000\00000\000Instruction queue is empty\000"
/* offset=91566 */ "ireg_spec\000\000instruction\000This event counts architecturally executed inter-register manipulation operations\000event=0x10a\000\00000\000This event counts architecturally executed inter-register manipulation operations\000"
/* offset=91769 */ "isb_spec\000\000instruction\000Barrier speculatively executed, ISB\000event=0x7c\000\00000\000Barrier speculatively executed, ISB\000"
/* offset=91878 */ "isb_spec\000\000spec_operation\000Barrier speculatively executed, ISB\000event=0x7c\000\00000\000Counts ISB operations that are executed\000"
/* offset=91994 */ "itb_miss\000\000cache\000ITB miss\000event=0x103\000\00000\000ITB miss\000"
/* offset=92044 */ "itlb_walk\000\000cache\000Access to instruction TLB that causes a translation table walk\000event=0x35\000\00000\000Access to instruction TLB that causes a translation table walk\000"
/* offset=92202 */ "itlb_walk\000\000cache\000Access to instruction TLB that causes a translation table walk\000event=0x35\000\00000\000This event counts on any instruction access which causes L2D_TLB_REFILL to count\000"
/* offset=92378 */ "itlb_walk\000\000tlb\000Access to instruction TLB that causes a translation table walk\000event=0x35\000\00000\000Counts instruction memory translation table walks caused by a miss in the L2 TLB driven by a memory access. Partial translations that also cause a table walk are counted. This event does not count table walks caused by TLB maintenance operations\000"
/* offset=92717 */ "iutlb_dep_stall\000\000pipeline\000Cycles the DPU IQ is empty and there is an instruction micro-TLB miss being processed\000event=0xe2\000\00000\000\000"
/* offset=92845 */ "ixa_stall\000\000pipeline\000IXA stalled\000event=0x10a\000\00000\000IXA Op non-issue\000"
/* offset=92910 */ "ixb_stall\000\000pipeline\000IXB stalled\000event=0x10b\000\00000\000IXB Op non-issue\000"
/* offset=92975 */ "ixu_issue_cancel\000\000core imp def\000Any uop issued was canceled for any reason\000event=0xd301\000\00000\000Any uop issued was canceled for any reason\000"
/* offset=93109 */ "ixu_ixa_issued\000\000core imp def\000Uops issued by the scheduler on IXA\000event=0xd304\000\00000\000Uops issued by the scheduler on IXA\000"
/* offset=93227 */ "ixu_ixa_par0_issued\000\000core imp def\000Uops issued by the scheduler on IXA Par 0\000event=0xd305\000\00000\000Uops issued by the scheduler on IXA Par 0\000"
/* offset=93362 */ "ixu_ixa_par1_issued\000\000core imp def\000Uops issued by the scheduler on IXA Par 1\000event=0xd306\000\00000\000Uops issued by the scheduler on IXA Par 1\000"
/* offset=93497 */ "ixu_ixb_issued\000\000core imp def\000Uops issued by the scheduler on IXB\000event=0xd307\000\00000\000Uops issued by the scheduler on IXB\000"
/* offset=93615 */ "ixu_ixb_par0_issued\000\000core imp def\000Uops issued by the scheduler on IXB Par 0\000event=0xd308\000\00000\000Uops issued by the scheduler on IXB Par 0\000"
/* offset=93750 */ "ixu_ixb_par1_issued\000\000core imp def\000Uops issued by the scheduler on IXB Par 1\000event=0xd309\000\00000\000Uops issued by the scheduler on IXB Par 1\000"
/* offset=93885 */ "ixu_ixc_issued\000\000core imp def\000Uops issued by the scheduler on IXC\000event=0xd30a\000\00000\000Uops issued by the scheduler on IXC\000"
/* offset=94003 */ "ixu_ixc_par0_issued\000\000core imp def\000Uops issued by the scheduler on IXC Par 0\000event=0xd30b\000\00000\000Uops issued by the scheduler on IXC Par 0\000"
/* offset=94138 */ "ixu_ixc_par1_issued\000\000core imp def\000Uops issued by the scheduler on IXC Par 1\000event=0xd30c\000\00000\000Uops issued by the scheduler on IXC Par 1\000"
/* offset=94273 */ "ixu_ixd_issued\000\000core imp def\000Uops issued by the scheduler on IXD\000event=0xd30d\000\00000\000Uops issued by the scheduler on IXD\000"
/* offset=94391 */ "ixu_ixd_par0_issued\000\000core imp def\000Uops issued by the scheduler on IXD Par 0\000event=0xd30e\000\00000\000Uops issued by the scheduler on IXD Par 0\000"
/* offset=94526 */ "ixu_ixd_par1_issued\000\000core imp def\000Uops issued by the scheduler on IXD Par 1\000event=0xd30f\000\00000\000Uops issued by the scheduler on IXD Par 1\000"
/* offset=94661 */ "ixu_load_cancel\000\000core imp def\000A load wakeup to the scheduler has been cancelled\000event=0xd302\000\00000\000A load wakeup to the scheduler has been cancelled\000"
/* offset=94808 */ "ixu_num_uops_issued\000\000core imp def\000Instructions issued by the scheduler\000event=0xd300\000\00000\000Instructions issued by the scheduler\000"
/* offset=94933 */ "ixu_slow_cancel\000\000core imp def\000The scheduler had to cancel one slow Uop due to resource conflict\000event=0xd303\000\00000\000The scheduler had to cancel one slow Uop due to resource conflict\000"
/* offset=95112 */ "l1_miss_wait\000\000cache\000This event counts outstanding L1D cache miss requests per cycle\000event=0x208\000\00000\000This event counts outstanding L1D cache miss requests per cycle\000"
/* offset=95276 */ "l1_pfetch_dist_inc\000\000core imp def\000L1 prefetcher, distance was increased\000event=0xd60a\000\00000\000L1 prefetcher, distance was increased\000"
/* offset=95402 */ "l1_pfetch_dist_rst\000\000core imp def\000L1 prefetcher, distance was reset\000event=0xd609\000\00000\000L1 prefetcher, distance was reset\000"
/* offset=95520 */ "l1_pfetch_entry_trained\000\000core imp def\000L1 prefetcher, table entry is trained\000event=0xd60b\000\00000\000L1 prefetcher, table entry is trained\000"
/* offset=95651 */ "l1_pfetch_l2_req\000\000core imp def\000L1 prefetcher, load prefetch to L2 generated\000event=0xd608\000\00000\000L1 prefetcher, load prefetch to L2 generated\000"
/* offset=95789 */ "l1_pfetch_ld_fill\000\000core imp def\000L1 prefetcher, load prefetch fills into the L1 cache\000event=0xd607\000\00000\000L1 prefetcher, load prefetch fills into the L1 cache\000"
/* offset=95944 */ "l1_pfetch_ld_gen\000\000core imp def\000L1 prefetcher, load prefetch requests generated\000event=0xd606\000\00000\000L1 prefetcher, load prefetch requests generated\000"
/* offset=96088 */ "l1_pipe0_comp\000\000pipeline\000This event counts completed requests in L1D cache pipeline#0\000event=0x260\000\00000\000This event counts completed requests in L1D cache pipeline#0\000"
/* offset=96250 */ "l1_pipe0_comp_prd_cnt\000\000pipeline\000This event counts the number of 1's in the predicate bits of request in L1D cache pipeline#0, where it is corrected so that it becomes 16 when all bits are 1\000event=0x2b8\000\00000\000This event counts the number of 1's in the predicate bits of request in L1D cache pipeline#0, where it is corrected so that it becomes 16 when all bits are 1\000"
/* offset=96614 */ "l1_pipe0_val\000\000pipeline\000This event counts valid cycles of L1D cache pipeline#0\000event=0x240\000\00000\000This event counts valid cycles of L1D cache pipeline#0\000"
/* offset=96763 */ "l1_pipe0_val_iu_not_sec0\000\000pipeline\000This event counts requests in L1D cache pipeline#0 that its sector cache ID is not 0\000event=0x2a0\000\00000\000This event counts requests in L1D cache pipeline#0 that its sector cache ID is not 0\000"
/* offset=96984 */ "l1_pipe0_val_iu_tag_adrs_pfe\000\000pipeline\000This event counts requests in L1D cache pipeline#0 that its pfe bit of tagged address is 1\000event=0x251\000\00000\000This event counts requests in L1D cache pipeline#0 that its pfe bit of tagged address is 1\000"
/* offset=97221 */ "l1_pipe0_val_iu_tag_adrs_sce\000\000pipeline\000This event counts requests in L1D cache pipeline#0 that its sce bit of tagged address is 1\000event=0x250\000\00000\000This event counts requests in L1D cache pipeline#0 that its sce bit of tagged address is 1\000"
/* offset=97458 */ "l1_pipe1_comp\000\000pipeline\000This event counts completed requests in L1D cache pipeline#1\000event=0x261\000\00000\000This event counts completed requests in L1D cache pipeline#1\000"
/* offset=97620 */ "l1_pipe1_comp_prd_cnt\000\000pipeline\000This event counts the number of 1's in the predicate bits of request in L1D cache pipeline#1, where it is corrected so that it becomes 16 when all bits are 1\000event=0x2b9\000\00000\000This event counts the number of 1's in the predicate bits of request in L1D cache pipeline#1, where it is corrected so that it becomes 16 when all bits are 1\000"
/* offset=97984 */ "l1_pipe1_val\000\000pipeline\000This event counts valid cycles of L1D cache pipeline#1\000event=0x241\000\00000\000This event counts valid cycles of L1D cache pipeline#1\000"
/* offset=98133 */ "l1_pipe1_val_iu_not_sec0\000\000pipeline\000This event counts requests in L1D cache pipeline#1 that its sector cache ID is not 0\000event=0x2a1\000\00000\000This event counts requests in L1D cache pipeline#1 that its sector cache ID is not 0\000"
/* offset=98354 */ "l1_pipe1_val_iu_tag_adrs_pfe\000\000pipeline\000This event counts requests in L1D cache pipeline#1 that its pfe bit of tagged address is 1\000event=0x253\000\00000\000This event counts requests in L1D cache pipeline#1 that its pfe bit of tagged address is 1\000"
/* offset=98591 */ "l1_pipe1_val_iu_tag_adrs_sce\000\000pipeline\000This event counts requests in L1D cache pipeline#1 that its sce bit of tagged address is 1\000event=0x252\000\00000\000This event counts requests in L1D cache pipeline#1 that its sce bit of tagged address is 1\000"
/* offset=98828 */ "l1_pipe_abort_stld_intlk\000\000pipeline\000This event counts aborted requests in L1D pipelines that due to store-load interlock\000event=0x274\000\00000\000This event counts aborted requests in L1D pipelines that due to store-load interlock\000"
/* offset=99049 */ "l1_pipe_comp_gather_0flow\000\000pipeline\000This event counts the number of times where 2 elements of the gather instructions became 0 flow because both predicate values are 0\000event=0x2b2\000\00000\000This event counts the number of times where 2 elements of the gather instructions became 0 flow because both predicate values are 0\000"
/* offset=99365 */ "l1_pipe_comp_gather_1flow\000\000pipeline\000This event counts the number of times where 2 elements of the gather instructions became 1 flow because 2 elements could be combined\000event=0x2b1\000\00000\000This event counts the number of times where 2 elements of the gather instructions became 1 flow because 2 elements could be combined\000"
/* offset=99683 */ "l1_pipe_comp_gather_2flow\000\000pipeline\000This event counts the number of times where 2 elements of the gather instructions became 2 flows because 2 elements could not be combined\000event=0x2b0\000\00000\000This event counts the number of times where 2 elements of the gather instructions became 2 flows because 2 elements could not be combined\000"
/* offset=100011 */ "l1_pipe_comp_scatter_1flow\000\000pipeline\000This event counts the number of flows of the scatter instructions\000event=0x2b3\000\00000\000This event counts the number of flows of the scatter instructions\000"
/* offset=100196 */ "l1_stage2_tlb_refill\000\000cache\000L1 stage 2 TLB refill\000event=0x111\000\00000\000Level 1 stage 2 TLB refill\000"
/* offset=100289 */ "l1d_cache\000\000cache\000Level 1 data cache access\000event=0x4\000\00000\000Level 1 data cache access\000"
/* offset=100372 */ "l1d_cache\000\000cache\000Level 1 data cache access\000event=0x4\000\00000\000This event counts any load or store operation or page table walk access which looks up in the L1 data cache. In particular, any access which could count the L1D_CACHE_REFILL event causes this event to count\000"
/* offset=100636 */ "l1d_cache\000\000l1d_cache\000Level 1 data cache access\000event=0x4\000\00000\000Counts level 1 data cache accesses from any load/store operations. Atomic operations that resolve in the CPUs caches (near atomic operations) counts as both a write access and read access. Each access to a cache line is counted including the multiple accesses caused by single instructions such as LDM or STM. Each access to other level 1 data or unified memory structures, for example refill buffers, write buffers, and write-back buffers, are also counted\000"
/* offset=101155 */ "l1d_cache_allocate\000\000cache\000Level 1 data cache allocation without refill\000event=0x1f\000\00000\000Level 1 data cache allocation without refill\000"
/* offset=101286 */ "l1d_cache_err\000\000other\000L1 Data Cache (data, tag or dirty) memory error, correctable or non-correctable\000event=0xd1\000\00000\000\000"
/* offset=101403 */ "l1d_cache_inval\000\000cache\000L1D cache invalidate\000event=0x48\000\00000\000Attributable Level 1 data cache invalidate\000"
/* offset=101505 */ "l1d_cache_inval\000\000core imp def\000L1D cache invalidate\000event=0x48\000\00000\000Attributable Level 1 data cache invalidate\000"
/* offset=101614 */ "l1d_cache_inval\000\000l1d_cache\000L1D cache invalidate\000event=0x48\000\00000\000Counts each explicit invalidation of a cache line in the level 1 data cache caused by:\n\n- Cache Maintenance Operations (CMO) that operate by a virtual address.\n- Broadcast cache coherency operations from another CPU in the system.\n\nThis event does not count for the following conditions:\n\n1. A cache refill invalidates a cache line.\n2. A CMO which is executed on that CPU and invalidates a cache line specified by set/way.\n\nNote that CMOs that operate by set/way cannot be broadcast from one CPU to another\000"
/* offset=102184 */ "l1d_cache_late_miss\000\000cache\000L1D cache late miss\000event=0x105\000\00000\000Level 1 data cache late miss\000"
/* offset=102276 */ "l1d_cache_lmiss_rd\000\000cache\000Level 1 data cache long-latency read miss\000event=0x39\000\00000\000Level 1 data cache long-latency read miss.  The counter counts each memory read access counted by L1D_CACHE that incurs additional latency because it returns data from outside the Level 1 data or unified cache of this processing element\000"
/* offset=102596 */ "l1d_cache_prefetch\000\000cache\000L1D cache prefetch\000event=0x106\000\00000\000Level 1 data cache prefetch request\000"
/* offset=102693 */ "l1d_cache_rd\000\000cache\000L1D cache access, read\000event=0x40\000\00000\000Attributable Level 1 data cache access, read\000"
/* offset=102796 */ "l1d_cache_rd\000\000core imp def\000L1D cache access, read\000event=0x40\000\00000\000Attributable Level 1 data cache access, read\000"
/* offset=102906 */ "l1d_cache_rd\000\000l1d_cache\000L1D cache access, read\000event=0x40\000\00000\000Counts level 1 data cache accesses from any load operation. Atomic load operations that resolve in the CPUs caches counts as both a write access and read access\000"
/* offset=103129 */ "l1d_cache_refill\000\000cache\000Level 1 data cache refill\000event=0x3\000\00000\000Level 1 data cache refill\000"
/* offset=103219 */ "l1d_cache_refill\000\000cache\000Level 1 data cache refill\000event=0x3\000\00000\000This event counts any load or store operation or page table walk access which causes data to be read from outside the L1, including accesses which do not allocate into L1\000"
/* offset=103454 */ "l1d_cache_refill\000\000l1d_cache\000Level 1 data cache refill\000event=0x3\000\00000\000Counts level 1 data cache refills caused by speculatively executed load or store operations that missed in the level 1 data cache. This event only counts one event per cache line. This event does not count cache line allocations from preload instructions or from hardware cache prefetching\000"
/* offset=103812 */ "l1d_cache_refill_dm\000\000cache\000This event counts L1D_CACHE_REFILL caused by demand access\000event=0x200\000\00000\000This event counts L1D_CACHE_REFILL caused by demand access\000"
/* offset=103973 */ "l1d_cache_refill_hwprf\000\000cache\000This event counts L1D_CACHE_REFILL caused by hardware prefetch\000event=0x202\000\00000\000This event counts L1D_CACHE_REFILL caused by hardware prefetch\000"
/* offset=104145 */ "l1d_cache_refill_inner\000\000cache\000L1D cache refill, inner\000event=0x44\000\00000\000Attributable Level 1 data cache refill, inner\000"
/* offset=104260 */ "l1d_cache_refill_inner\000\000core imp def\000L1D cache refill, inner\000event=0x44\000\00000\000Attributable Level 1 data cache refill, inner\000"
/* offset=104382 */ "l1d_cache_refill_inner\000\000l1d_cache\000L1D cache refill, inner\000event=0x44\000\00000\000Counts level 1 data cache refills where the cache line data came from caches inside the immediate cluster of the core\000"
/* offset=104573 */ "l1d_cache_refill_outer\000\000cache\000L1D cache refill, outer\000event=0x45\000\00000\000Attributable Level 1 data cache refill, outer\000"
/* offset=104688 */ "l1d_cache_refill_outer\000\000core imp def\000L1D cache refill, outer\000event=0x45\000\00000\000Attributable Level 1 data cache refill, outer\000"
/* offset=104810 */ "l1d_cache_refill_outer\000\000l1d_cache\000L1D cache refill, outer\000event=0x45\000\00000\000Counts level 1 data cache refills for which the cache line data came from outside the immediate cluster of the core, like an SLC in the system interconnect or DRAM\000"
/* offset=105047 */ "l1d_cache_refill_prefetch\000\000cache\000L1 data cache refill due to prefetch. This event counts any linefills from the prefetcher that cause an allocation into the L1 data cache\000event=0xc2\000\00000\000L1 data cache refill due to prefetch. This event counts any linefills from the prefetcher that cause an allocation into the L1 data cache\000"
/* offset=105371 */ "l1d_cache_refill_prefetch\000\000cache\000Level 1 data cache refill due to prefetch. This event counts any linefills from the prefetcher which cause an allocation into the L1 D-cache\000event=0xc2\000\00000\000Level 1 data cache refill due to prefetch. This event counts any linefills from the prefetcher which cause an allocation into the L1 D-cache\000"
/* offset=105701 */ "l1d_cache_refill_prf\000\000cache\000This event counts L1D_CACHE_REFILL caused by software or hardware prefetch\000event=0x49\000\00000\000This event counts L1D_CACHE_REFILL caused by software or hardware prefetch\000"
/* offset=105894 */ "l1d_cache_refill_rd\000\000cache\000L1D cache refill, read\000event=0x42\000\00000\000Attributable Level 1 data cache refill, read\000"
/* offset=106004 */ "l1d_cache_refill_rd\000\000core imp def\000L1D cache refill, read\000event=0x42\000\00000\000Attributable Level 1 data cache refill, read\000"
/* offset=106121 */ "l1d_cache_refill_rd\000\000l1d_cache\000L1D cache refill, read\000event=0x42\000\00000\000Counts level 1 data cache refills caused by speculatively executed load instructions where the memory read operation misses in the level 1 data cache. This event only counts one event per cache line\000"
/* offset=106389 */ "l1d_cache_refill_wr\000\000cache\000L1D cache refill, write\000event=0x43\000\00000\000Attributable Level 1 data cache refill, write\000"
/* offset=106501 */ "l1d_cache_refill_wr\000\000core imp def\000L1D cache refill, write\000event=0x43\000\00000\000Attributable Level 1 data cache refill, write\000"
/* offset=106620 */ "l1d_cache_refill_wr\000\000l1d_cache\000L1D cache refill, write\000event=0x43\000\00000\000Counts level 1 data cache refills caused by speculatively executed store instructions where the memory write operation misses in the level 1 data cache. This event only counts one event per cache line\000"
/* offset=106891 */ "l1d_cache_wb\000\000cache\000Attributable Level 1 data cache write-back\000event=0x15\000\00000\000Attributable Level 1 data cache write-back\000"
/* offset=107012 */ "l1d_cache_wb\000\000cache\000Attributable Level 1 data cache write-back\000event=0x15\000\00000\000This event counts any write-back of data from the L1 data cache to L2 or L3. This counts both victim line evictions and snoops, including cache maintenance operations\000"
/* offset=107257 */ "l1d_cache_wb\000\000l1d_cache\000Attributable Level 1 data cache write-back\000event=0x15\000\00000\000Counts write-backs of dirty data from the L1 data cache to the L2 cache. This occurs when either a dirty cache line is evicted from L1 data cache and allocated in the L2 cache or dirty data is written to the L2 and possibly to the next level of cache. This event counts both victim cache line evictions and cache write-backs from snoops or cache maintenance operations. The following cache operations are not counted:\n\n1. Invalidations which do not result in data being transferred out of the L1 (such as evictions of clean data),\n2. Full line writes which write to L2 without writing L1, such as write streaming mode\000"
/* offset=107957 */ "l1d_cache_wb_clean\000\000cache\000L1D cache Write-Back, cleaning and coherency\000event=0x47\000\00000\000Level 1 data cache Write-Back, cleaning and coherency\000"
/* offset=108097 */ "l1d_cache_wb_clean\000\000core imp def\000L1D cache Write-Back, cleaning and coherency\000event=0x47\000\00000\000Level 1 data cache Write-Back, cleaning and coherency\000"
/* offset=108244 */ "l1d_cache_wb_clean\000\000l1d_cache\000L1D cache Write-Back, cleaning and coherency\000event=0x47\000\00000\000Counts write-backs from the level 1 data cache that are a result of a coherency operation made by another CPU. Event count includes cache maintenance operations\000"
/* offset=108495 */ "l1d_cache_wb_victim\000\000cache\000L1D cache Write-Back, victim\000event=0x46\000\00000\000Attributable Level 1 data cache Write-Back, victim\000"
/* offset=108617 */ "l1d_cache_wb_victim\000\000core imp def\000L1D cache Write-Back, victim\000event=0x46\000\00000\000Attributable Level 1 data cache Write-Back, victim\000"
/* offset=108746 */ "l1d_cache_wb_victim\000\000l1d_cache\000L1D cache Write-Back, victim\000event=0x46\000\00000\000Counts dirty cache line evictions from the level 1 data cache caused by a new cache line allocation. This event does not count evictions caused by cache maintenance operations\000"
/* offset=108997 */ "l1d_cache_wr\000\000cache\000L1D cache access, write\000event=0x41\000\00000\000Attributable Level 1 data cache access, write\000"
/* offset=109102 */ "l1d_cache_wr\000\000core imp def\000L1D cache access, write\000event=0x41\000\00000\000Attributable Level 1 data cache access, write\000"
/* offset=109214 */ "l1d_cache_wr\000\000l1d_cache\000L1D cache access, write\000event=0x41\000\00000\000Counts level 1 data cache accesses generated by store operations. This event also counts accesses caused by a DC ZVA (data cache zero, specified by virtual address) instruction. Near atomic operations that resolve in the CPUs caches count as a write access and read access\000"
/* offset=109550 */ "l1d_pref_line_fill\000\000cache\000Level 1 data cache refill started due to prefetch. Counts any linefills from the prefetcher which cause an allocation into the L1 D-cache\000event=0xc3\000\00000\000Level 1 data cache refill started due to prefetch. Counts any linefills from the prefetcher which cause an allocation into the L1 D-cache\000"
/* offset=109867 */ "l1d_tlb\000\000cache\000Attributable Level 1 data or unified TLB access\000event=0x25\000\00000\000Attributable Level 1 data or unified TLB access\000"
/* offset=109993 */ "l1d_tlb\000\000cache\000L1D TLB access\000event=0x25\000\00000\000This event counts any load or store operation which accesses the data L1 TLB\000"
/* offset=110115 */ "l1d_tlb\000\000cache\000Level 1 data TLB access\000event=0x25\000\00000\000This event counts any load or store operation which accesses the data L1 TLB. If both a load and a store are executed on a cycle, this event counts twice. This event counts regardless of whether the MMU is enabled\000"
/* offset=110383 */ "l1d_tlb\000\000tlb\000Attributable Level 1 data or unified TLB access\000event=0x25\000\00000\000Counts level 1 data TLB accesses caused by any memory load or store operation. Note that load or store instructions can be broken up into multiple memory operations. This event does not count TLB maintenance operations\000"
/* offset=110678 */ "l1d_tlb_miss\000\000core imp def\000L1D TLB miss\000event=0xd600\000\00000\000L1D TLB miss\000"
/* offset=110748 */ "l1d_tlb_rd\000\000cache\000L1D tlb access, read\000event=0x4e\000\00000\000Attributable Level 1 data or unified TLB access, read\000"
/* offset=110856 */ "l1d_tlb_rd\000\000core imp def\000L1D tlb access, read\000event=0x4e\000\00000\000Attributable Level 1 data or unified TLB access, read\000"
/* offset=110971 */ "l1d_tlb_rd\000\000tlb\000L1D tlb access, read\000event=0x4e\000\00000\000Counts level 1 data TLB accesses caused by memory read operations. This event counts whether the access hits or misses in the TLB. This event does not count TLB maintenance operations\000"
/* offset=111207 */ "l1d_tlb_refill\000\000cache\000Attributable Level 1 data TLB refill\000event=0x5\000\00000\000Attributable Level 1 data TLB refill\000"
/* offset=111317 */ "l1d_tlb_refill\000\000cache\000Attributable Level 1 data TLB refill\000event=0x5\000\00000\000This event counts any refill of the data L1 TLB from the L2 TLB. This includes refills that result in a translation fault\000"
/* offset=111512 */ "l1d_tlb_refill\000\000tlb\000Attributable Level 1 data TLB refill\000event=0x5\000\00000\000Counts level 1 data TLB accesses that resulted in TLB refills. If there are multiple misses in the TLB that are resolved by the refill, then this event only counts once. This event counts for refills caused by preload instructions or hardware prefetch accesses. This event counts regardless of whether the miss hits in L2 or results in a translation table walk. This event will not count if the translation table walk results in a fault (such as a translation or access fault), since there is no new translation created for the TLB. This event will not count on an access from an AT(address translation) instruction\000"
/* offset=112199 */ "l1d_tlb_refill_rd\000\000cache\000L1D tlb refill, read\000event=0x4c\000\00000\000Attributable Level 1 data TLB refill, read\000"
/* offset=112303 */ "l1d_tlb_refill_rd\000\000core imp def\000L1D tlb refill, read\000event=0x4c\000\00000\000Attributable Level 1 data TLB refill, read\000"
/* offset=112414 */ "l1d_tlb_refill_rd\000\000tlb\000L1D tlb refill, read\000event=0x4c\000\00000\000Counts level 1 data TLB refills caused by memory read operations. If there are multiple misses in the TLB that are resolved by the refill, then this event only counts once. This event counts for refills caused by preload instructions or hardware prefetch accesses. This event counts regardless of whether the miss hits in L2 or results in a translation table walk. This event will not count if the translation table walk results in a fault (such as a translation or access fault), since there is no new translation created for the TLB. This event will not count on an access from an Address Translation (AT) instruction\000"
/* offset=113093 */ "l1d_tlb_refill_wr\000\000cache\000L1D tlb refill, write\000event=0x4d\000\00000\000Attributable Level 1 data TLB refill, write\000"
/* offset=113199 */ "l1d_tlb_refill_wr\000\000core imp def\000L1D tlb refill, write\000event=0x4d\000\00000\000Attributable Level 1 data TLB refill, write\000"
/* offset=113312 */ "l1d_tlb_refill_wr\000\000tlb\000L1D tlb refill, write\000event=0x4d\000\00000\000Counts level 1 data TLB refills caused by data side memory write operations. If there are multiple misses in the TLB that are resolved by the refill, then this event only counts once. This event counts for refills caused by preload instructions or hardware prefetch accesses. This event counts regardless of whether the miss hits in L2 or results in a translation table walk. This event will not count if the table walk results in a fault (such as a translation or access fault), since there is no new translation created for the TLB. This event will not count with an access from an Address Translation (AT) instruction\000"
/* offset=113993 */ "l1d_tlb_wr\000\000cache\000L1D tlb access, write\000event=0x4f\000\00000\000Attributable Level 1 data or unified TLB access, write\000"
/* offset=114103 */ "l1d_tlb_wr\000\000core imp def\000L1D tlb access, write\000event=0x4f\000\00000\000Attributable Level 1 data or unified TLB access, write\000"
/* offset=114220 */ "l1d_tlb_wr\000\000tlb\000L1D tlb access, write\000event=0x4f\000\00000\000Counts any L1 data side TLB accesses caused by memory write operations. This event counts whether the access hits or misses in the TLB. This event does not count TLB maintenance operations\000"
/* offset=114462 */ "l1d_ws_mode\000\000cache\000L1 data cache write streaming mode. This event counts for each cycle where the core is in write streaming mode and is not allocating writes into the L1 data cache\000event=0xc5\000\00000\000L1 data cache write streaming mode. This event counts for each cycle where the core is in write streaming mode and is not allocating writes into the L1 data cache\000"
/* offset=114822 */ "l1d_ws_mode\000\000cache\000L1D is in write stream mode\000event=0xc7\000\00000\000L1D is in write stream mode\000"
/* offset=114912 */ "l1d_ws_mode\000\000cache\000Level 1 data cache write streaming mode.This event counts for each cycle where the core is in write-streaming mode and not allocating writes into the L1 D-cache\000event=0xc5\000\00000\000Level 1 data cache write streaming mode.This event counts for each cycle where the core is in write-streaming mode and not allocating writes into the L1 D-cache\000"
/* offset=115268 */ "l1d_ws_mode_enter\000\000cache\000L1D entering write stream mode\000event=0xc6\000\00000\000L1D entering write stream mode\000"
/* offset=115370 */ "l1d_ws_mode_entry\000\000cache\000L1 data cache entering write streaming mode. This event counts for each entry into write streaming mode\000event=0xc4\000\00000\000L1 data cache entering write streaming mode. This event counts for each entry into write streaming mode\000"
/* offset=115618 */ "l1d_ws_mode_entry\000\000cache\000Level 1 data cache entering write streaming mode.This event counts for each entry into write-streaming mode\000event=0xc4\000\00000\000Level 1 data cache entering write streaming mode.This event counts for each entry into write-streaming mode\000"
/* offset=115874 */ "l1hwpf_inj_alloc_pf\000\000other\000This event counts allocation type prefetch injection requests to L1D cache generated by hardware prefetcher\000event=0x231\000\00000\000This event counts allocation type prefetch injection requests to L1D cache generated by hardware prefetcher\000"
/* offset=116133 */ "l1hwpf_inj_noalloc_pf\000\000other\000This event counts non-allocation type prefetch injection requests to L1D cache generated by hardware prefetcher\000event=0x232\000\00000\000This event counts non-allocation type prefetch injection requests to L1D cache generated by hardware prefetcher\000"
/* offset=116402 */ "l1hwpf_stream_pf\000\000other\000This event counts streaming prefetch requests to L1D cache generated by hardware prefetcher\000event=0x230\000\00000\000This event counts streaming prefetch requests to L1D cache generated by hardware prefetcher\000"
/* offset=116626 */ "l1i_cache\000\000cache\000Attributable Level 1 instruction cache access\000event=0x14\000\00000\000Attributable Level 1 instruction cache access\000"
/* offset=116750 */ "l1i_cache\000\000cache\000Attributable Level 1 instruction cache access\000event=0x14\000\00000\000Level 1 instruction cache access or Level 0 Macro-op cache access. This event counts any instruction fetch which accesses the L1 instruction cache or L0 Macro-op cache\000"
/* offset=116996 */ "l1i_cache\000\000l1i_cache\000Attributable Level 1 instruction cache access\000event=0x14\000\00000\000Counts instruction fetches which access the level 1 instruction cache. Instruction cache accesses caused by cache maintenance operations are not counted\000"
/* offset=117231 */ "l1i_cache_err\000\000other\000L1 Instruction Cache (data or tag) memory error\000event=0xd0\000\00000\000\000"
/* offset=117316 */ "l1i_cache_lmiss\000\000cache\000Level 1 instruction cache long-latency read miss\000event=0x4006\000\00000\000Level 1 instruction cache long-latency read miss.  If the L1I_CACHE_RD event is implemented, the counter counts each access counted by L1I_CACHE_RD that incurs additional latency because it returns instructions from outside of the Level 1 instruction cache of this PE.  If the L1I_CACHE_RD event is not implemented, the counter counts each access counted by L1I_CACHE that incurs additional latency because it returns instructions from outside the Level 1 instruction cache of this PE.  The event indicates to software that the access missed in the Level 1 instruction cache and might have a significant performance impact due to the additional latency, compared to the latency of an access that hits in the Level 1 instruction cache\000"
/* offset=118139 */ "l1i_cache_prf\000\000core imp def\000L1I cache prefetch access count\000event=0x102e\000\00000\000Level 1 instruction cache prefetch access count\000"
/* offset=118264 */ "l1i_cache_prf_refill\000\000core imp def\000L1I cache miss due to prefetch access count\000event=0x102f\000\00000\000Level 1 instruction cache miss due to prefetch access count\000"
/* offset=118420 */ "l1i_cache_refill\000\000cache\000Level 1 instruction cache refill\000event=0x1\000\00000\000Level 1 instruction cache refill\000"
/* offset=118524 */ "l1i_cache_refill\000\000cache\000Level 1 instruction cache refill\000event=0x1\000\00000\000This event counts any instruction fetch which misses in the cache\000"
/* offset=118661 */ "l1i_cache_refill\000\000l1i_cache\000Level 1 instruction cache refill\000event=0x1\000\00000\000Counts cache line refills in the level 1 instruction cache caused by a missed instruction fetch. Instruction fetches may include accessing multiple instructions, but the single cache line allocation is counted once\000"
/* offset=118951 */ "l1i_miss_wait\000\000cache\000This event counts outstanding L1I cache miss requests per cycle\000event=0x209\000\00000\000This event counts outstanding L1I cache miss requests per cycle\000"
/* offset=119116 */ "l1i_pipe_comp\000\000pipeline\000This event counts completed requests in L1I cache pipeline\000event=0x268\000\00000\000This event counts completed requests in L1I cache pipeline\000"
/* offset=119274 */ "l1i_pipe_val\000\000pipeline\000This event counts valid cycles of L1I cache pipeline\000event=0x269\000\00000\000This event counts valid cycles of L1I cache pipeline\000"
/* offset=119419 */ "l1i_tlb\000\000cache\000Attributable Level 1 instruction TLB access\000event=0x26\000\00000\000Attributable Level 1 instruction TLB access\000"
/* offset=119537 */ "l1i_tlb\000\000cache\000Attributable Level 1 instruction TLB access\000event=0x26\000\00000\000This event counts any instruction fetch which accesses the instruction L1 TLB\000"
/* offset=119689 */ "l1i_tlb\000\000cache\000Level 1 instruction TLB access\000event=0x26\000\00000\000This event counts any instruction fetch which accesses the instruction L1 TLB.This event counts regardless of whether the MMU is enabled\000"
/* offset=119887 */ "l1i_tlb\000\000tlb\000Attributable Level 1 instruction TLB access\000event=0x26\000\00000\000Counts level 1 instruction TLB accesses, whether the access hits or misses in the TLB. This event counts both demand accesses and prefetch or preload generated accesses\000"
/* offset=120128 */ "l1i_tlb_miss\000\000core imp def\000L1I TLB miss\000event=0xd103\000\00000\000L1I TLB miss\000"
/* offset=120198 */ "l1i_tlb_refill\000\000cache\000Attributable Level 1 instruction TLB refill\000event=0x2\000\00000\000Attributable Level 1 instruction TLB refill\000"
/* offset=120322 */ "l1i_tlb_refill\000\000cache\000Attributable Level 1 instruction TLB refill\000event=0x2\000\00000\000This event counts any refill of the instruction L1 TLB from the L2 TLB. This includes refills that result in a translation fault\000"
/* offset=120531 */ "l1i_tlb_refill\000\000tlb\000Attributable Level 1 instruction TLB refill\000event=0x2\000\00000\000Counts level 1 instruction TLB refills from any Instruction fetch. If there are multiple misses in the TLB that are resolved by the refill, then this event only counts once. This event will not count if the translation table walk results in a fault (such as a translation or access fault), since there is no new translation created for the TLB\000"
/* offset=120953 */ "l1pld_tlb\000\000cache\000Level 1 preload TLB access. This event only counts software and hardware prefetches at Level 1. This event counts all accesses to the preload data micro TLB, that is L1 prefetcher and preload instructions. This event does not take into account whether the MMU is enabled or not\000event=0xeb\000\00000\000Level 1 preload TLB access. This event only counts software and hardware prefetches at Level 1. This event counts all accesses to the preload data micro TLB, that is L1 prefetcher and preload instructions. This event does not take into account whether the MMU is enabled or not\000"
/* offset=121541 */ "l1pld_tlb_refill\000\000cache\000Level 1 PLD TLB refill\000event=0xe7\000\00000\000Level 1 PLD TLB refill\000"
/* offset=121626 */ "l2_l1d_cache_wb_unatt\000\000cache\000Unattributable Level 1 data cache write-back. This event occurs when a requestor outside the PE makes a coherency request that results in writeback\000event=0xf0\000\00000\000Unattributable Level 1 data cache write-back. This event occurs when a requestor outside the PE makes a coherency request that results in writeback\000"
/* offset=121966 */ "l2_l2d_cache_rd_unatt\000\000cache\000Unattributable Level 2 data cache access, read. This event occurs when a requestor outside the PE makes a coherency request that results in level 2 data cache read access\000event=0xf2\000\00000\000Unattributable Level 2 data cache access, read. This event occurs when a requestor outside the PE makes a coherency request that results in level 2 data cache read access\000"
/* offset=122352 */ "l2_l2d_cache_unatt\000\000cache\000Unattributable Level 2 data cache access. This event occurs when a requestor outside the PE makes a coherency request that results in level 2 data cache access\000event=0xf1\000\00000\000Unattributable Level 2 data cache access. This event occurs when a requestor outside the PE makes a coherency request that results in level 2 data cache access\000"
/* offset=122713 */ "l2_l3d_cache_alloc_unatt\000\000cache\000Unattributable Level 3 data or unified cache allocation without refill. This event occurs when a requestor outside the PE makes a coherency request that results in level 3 cache allocate without refill\000event=0xf5\000\00000\000Unattributable Level 3 data or unified cache allocation without refill. This event occurs when a requestor outside the PE makes a coherency request that results in level 3 cache allocate without refill\000"
/* offset=123164 */ "l2_l3d_cache_rd_unatt\000\000cache\000Unattributable Level 3 data cache access, read. This event occurs when a requestor outside the PE makes a coherency request that results in level 3 data cache read access\000event=0xf4\000\00000\000Unattributable Level 3 data cache access, read. This event occurs when a requestor outside the PE makes a coherency request that results in level 3 data cache read access\000"
/* offset=123550 */ "l2_l3d_cache_refill_unatt\000\000cache\000Unattributable Level 3 data or unified cache refill. This event occurs when a requestor outside the PE makes a coherency request that results in level 3 cache refill\000event=0xf6\000\00000\000Unattributable Level 3 data or unified cache refill. This event occurs when a requestor outside the PE makes a coherency request that results in level 3 cache refill\000"
/* offset=123930 */ "l2_l3d_cache_unatt\000\000cache\000Unattributable Level 3 data cache access. This event occurs when a requestor outside the PE makes a coherency request that results in level 3 data cache read access\000event=0xf3\000\00000\000Unattributable Level 3 data cache access. This event occurs when a requestor outside the PE makes a coherency request that results in level 3 data cache read access\000"
/* offset=124301 */ "l2_miss_count\000\000cache\000This event counts the number of times of L2 cache miss\000event=0x309\000\00000\000This event counts the number of times of L2 cache miss\000"
/* offset=124448 */ "l2_miss_wait\000\000cache\000This event counts outstanding L2 cache miss requests per cycle\000event=0x308\000\00000\000This event counts outstanding L2 cache miss requests per cycle\000"
/* offset=124610 */ "l2_pipe_comp_all\000\000pipeline\000This event counts completed requests in L2 cache pipeline\000event=0x350\000\00000\000This event counts completed requests in L2 cache pipeline\000"
/* offset=124769 */ "l2_pipe_comp_pf_l2mib_mch\000\000pipeline\000This event counts operations where software or hardware prefetch hits an L2 cache refill buffer allocated by demand access\000event=0x370\000\00000\000This event counts operations where software or hardware prefetch hits an L2 cache refill buffer allocated by demand access\000"
/* offset=125067 */ "l2_pipe_val\000\000pipeline\000This event counts valid cycles of L2 cache pipeline\000event=0x330\000\00000\000This event counts valid cycles of L2 cache pipeline\000"
/* offset=125209 */ "l2_prefetch_refill\000\000core imp def\000Level 2 prefetch requests, refilled to L2 cache\000event=0x10a\000\00000\000Level 2 prefetch requests, refilled to L2 cache\000"
/* offset=125354 */ "l2_prefetch_req\000\000core imp def\000L2 prefetcher, load prefetch requests generated\000event=0xd703\000\00000\000L2 prefetcher, load prefetch requests generated\000"
/* offset=125497 */ "l2_prefetch_upgrade\000\000core imp def\000Level 2 prefetch requests, late\000event=0x10b\000\00000\000Level 2 prefetch requests, late\000"
/* offset=125611 */ "l2c_dat_crd_stall\000\000core imp def\000L2 TXDAT LCRD blocked\000event=0x169\000\00000\000L2 TXDAT LCRD blocked\000"
/* offset=125703 */ "l2c_data_refill\000\000core imp def\000L2 refill from D-side miss\000event=0xd702\000\00000\000L2 refill from D-side miss\000"
/* offset=125804 */ "l2c_inst_refill\000\000core imp def\000L2 refill from I-side miss\000event=0xd701\000\00000\000L2 refill from I-side miss\000"
/* offset=125905 */ "l2c_pipe_replay\000\000core imp def\000L2 pipeline replay\000event=0xd700\000\00000\000L2 pipeline replay\000"
/* offset=125990 */ "l2c_req_crd_stall\000\000core imp def\000L2 TXREQ LCRD blocked\000event=0x16b\000\00000\000L2 TXREQ LCRD blocked\000"
/* offset=126082 */ "l2c_rsp_crd_stall\000\000core imp def\000L2 TXRSP LCRD blocked\000event=0x16a\000\00000\000L2 TXRSP LCRD blocked\000"
/* offset=126174 */ "l2c_snoop\000\000core imp def\000Bus request sn\000event=0x156\000\00000\000Bus request sn\000"
/* offset=126244 */ "l2d_cache\000\000cache\000Level 2 data cache access\000event=0x16\000\00000\000Level 2 data cache access\000"
/* offset=126328 */ "l2d_cache\000\000cache\000Level 2 data cache access\000event=0x16\000\00000\000This event counts any transaction from L1 which looks up in the L2 cache, and any write-back from the L1 to the L2. Snoops from outside the core and cache maintenance operations are not counted\000"
/* offset=126580 */ "l2d_cache\000\000l2_cache\000Level 2 data cache access\000event=0x16\000\00000\000Counts level 2 cache accesses. level 2 cache is a unified cache for data and instruction accesses. Accesses are for misses in the first level caches or translation resolutions due to accesses. This event also counts write back of dirty data from level 1 data cache to the L2 cache\000"
/* offset=126922 */ "l2d_cache_allocate\000\000cache\000Attributable Level 2 data cache allocation without refill\000event=0x20\000\00000\000Attributable Level 2 data cache allocation without refill\000"
/* offset=127079 */ "l2d_cache_allocate\000\000cache\000Attributable Level 2 data cache allocation without refill\000event=0x20\000\00000\000This event counts any full cache line write into the L2 cache which does not cause a linefill, including write-backs from L1 to L2 and full-line writes which do not allocate into L1\000"
/* offset=127360 */ "l2d_cache_allocate\000\000l2_cache\000Attributable Level 2 data cache allocation without refill\000event=0x20\000\00000\000TBD\000"
/* offset=127466 */ "l2d_cache_inval\000\000cache\000L2D cache invalidate\000event=0x58\000\00000\000Attributable Level 2 data cache invalidate\000"
/* offset=127568 */ "l2d_cache_inval\000\000core imp def\000L2D cache invalidate\000event=0x58\000\00000\000Attributable Level 2 data cache invalidate\000"
/* offset=127677 */ "l2d_cache_inval\000\000l2_cache\000L2D cache invalidate\000event=0x58\000\00000\000Counts each explicit invalidation of a cache line in the level 2 cache by cache maintenance operations that operate by a virtual address, or by external coherency operations. This event does not count if either:\n\n1. A cache refill invalidates a cache line or,\n2. A Cache Maintenance Operation (CMO), which invalidates a cache line specified by set/way, is executed on that CPU.\n\nCMOs that operate by set/way cannot be broadcast from one CPU to another\000"
/* offset=128191 */ "l2d_cache_lmiss_rd\000\000cache\000Level 2 data cache long-latency read miss\000event=0x4009\000\00000\000Level 2 data cache long-latency read miss.  The counter counts each memory read access counted by L2D_CACHE that incurs additional latency because it returns data from outside the Level 2 data or unified cache of this processing element.  The event indicates to software that the access missed in the Level 2 data or unified cache and might have a significant performance impact compared to the latency of an access that hits in the Level 2 data or unified cache\000"
/* offset=128739 */ "l2d_cache_mibmch_prf\000\000cache\000This event counts operations where software or hardware prefetch hits an L2 cache refill buffer allocated by demand access\000event=0x326\000\00000\000This event counts operations where software or hardware prefetch hits an L2 cache refill buffer allocated by demand access\000"
/* offset=129029 */ "l2d_cache_prefetch\000\000cache\000L2D cache prefetch\000event=0x107\000\00000\000Level 2 data cache prefetch request\000"
/* offset=129126 */ "l2d_cache_rd\000\000cache\000L2D cache access, read\000event=0x50\000\00000\000Attributable Level 2 data cache access, read\000"
/* offset=129229 */ "l2d_cache_rd\000\000core imp def\000L2D cache access, read\000event=0x50\000\00000\000Attributable Level 2 data cache access, read\000"
/* offset=129339 */ "l2d_cache_rd\000\000l2_cache\000L2D cache access, read\000event=0x50\000\00000\000Counts level 2 cache accesses due to memory read operations. level 2 cache is a unified cache for data and instruction accesses, accesses are for misses in the level 1 caches or translation resolutions due to accesses\000"
/* offset=129618 */ "l2d_cache_refill\000\000cache\000Level 2 data refill\000event=0x17\000\00000\000L2 data cache refill. This event counts any cacheable transaction from L1 which causes data to be read from outside the core. L2 refills caused by stashes into L2 should not be counted\000"
/* offset=129862 */ "l2d_cache_refill\000\000cache\000Level 2 data refill\000event=0x17\000\00000\000Level 2 data refill\000"
/* offset=129941 */ "l2d_cache_refill\000\000l2_cache\000Level 2 data refill\000event=0x17\000\00000\000Counts cache line refills into the level 2 cache. level 2 cache is a unified cache for data and instruction accesses. Accesses are for misses in the level 1 caches or translation resolutions due to accesses\000"
/* offset=130210 */ "l2d_cache_refill_dm\000\000cache\000This event counts L2D_CACHE_REFILL caused by demand access\000event=0x300\000\00000\000This event counts L2D_CACHE_REFILL caused by demand access\000"
/* offset=130371 */ "l2d_cache_refill_hwprf\000\000cache\000This event counts L2D_CACHE_REFILL caused by hardware prefetch\000event=0x302\000\00000\000This event counts L2D_CACHE_REFILL caused by hardware prefetch\000"
/* offset=130543 */ "l2d_cache_refill_prefetch\000\000cache\000L2 cache refill due to prefetch. If the complex is configured with a per-complex L2 cache, this event does not count. If the complex is configured without a per-complex L2 cache, this event counts the cluster cache event, as defined by L3D_CACHE_REFILL_PREFETCH. If neither a per-complex cache or a cluster cache is configured, this event is not implemented\000event=0xc1\000\00000\000L2 cache refill due to prefetch. If the complex is configured with a per-complex L2 cache, this event does not count. If the complex is configured without a per-complex L2 cache, this event counts the cluster cache event, as defined by L3D_CACHE_REFILL_PREFETCH. If neither a per-complex cache or a cluster cache is configured, this event is not implemented\000"
/* offset=131307 */ "l2d_cache_refill_prefetch\000\000cache\000Level 2 cache refill due to prefetch. +//0 If the core is configured with a per-core L2 cache: This event does not count. +//0 If the core is configured without a per-core L2 cache: This event counts the cluster cache event, as defined by L3D_CACHE_REFILL_PREFETCH. +//0 If there is neither a per-core cache nor a cluster cache configured, this event is not implemented\000event=0xc1\000\00000\000Level 2 cache refill due to prefetch. +//0 If the core is configured with a per-core L2 cache: This event does not count. +//0 If the core is configured without a per-core L2 cache: This event counts the cluster cache event, as defined by L3D_CACHE_REFILL_PREFETCH. +//0 If there is neither a per-core cache nor a cluster cache configured, this event is not implemented\000"
/* offset=132095 */ "l2d_cache_refill_prf\000\000cache\000This event counts L2D_CACHE_REFILL caused by software or hardware prefetch\000event=0x59\000\00000\000This event counts L2D_CACHE_REFILL caused by software or hardware prefetch\000"
/* offset=132288 */ "l2d_cache_refill_rd\000\000cache\000L2D cache refill, read\000event=0x52\000\00000\000Attributable Level 2 data cache refill, read\000"
/* offset=132398 */ "l2d_cache_refill_rd\000\000core imp def\000L2D cache refill, read\000event=0x52\000\00000\000Attributable Level 2 data cache refill, read\000"
/* offset=132515 */ "l2d_cache_refill_rd\000\000l2_cache\000L2D cache refill, read\000event=0x52\000\00000\000Counts refills for memory accesses due to memory read operation counted by L2D_CACHE_RD. level 2 cache is a unified cache for data and instruction accesses, accesses are for misses in the level 1 caches or translation resolutions due to accesses\000"
/* offset=132829 */ "l2d_cache_refill_wr\000\000cache\000L2D cache refill, write\000event=0x53\000\00000\000Attributable Level 2 data cache refill, write\000"
/* offset=132941 */ "l2d_cache_refill_wr\000\000core imp def\000L2D cache refill, write\000event=0x53\000\00000\000Attributable Level 2 data cache refill, write\000"
/* offset=133060 */ "l2d_cache_refill_wr\000\000l2_cache\000L2D cache refill, write\000event=0x53\000\00000\000Counts refills for memory accesses due to memory write operation counted by L2D_CACHE_WR. level 2 cache is a unified cache for data and instruction accesses, accesses are for misses in the level 1 caches or translation resolutions due to accesses\000"
/* offset=133376 */ "l2d_cache_stash_dropped\000\000cache\000L2 cache stash dropped. This event counts on each stash request that is received from the interconnect or the Accelerator Coherency Port (ACP), that targets L2 cache and is dropped due to lack of buffer space to hold the request\000event=0xd6\000\00000\000L2 cache stash dropped. This event counts on each stash request that is received from the interconnect or the Accelerator Coherency Port (ACP), that targets L2 cache and is dropped due to lack of buffer space to hold the request\000"
/* offset=133880 */ "l2d_cache_stash_dropped\000\000cache\000Level 2 cache stash dropped. This event counts on each stash request received from the interconnect or ACP, that is targeting L2 and gets dropped due to lack of buffer space to hold the request. L2 and L3 cache events (L2D_CACHE*, L3D_CACHE*) The behavior of these events depends on the configuration of the core. If the private L2 cache is present, the L2D_CACHE* events count the activity in the private L2 cache, and the L3D_CACHE* events count the activity in the DSU L3 cache (if present). If the private L2 cache is not present but the DSU L3 cache is present, the L2D_CACHE* events count activity in the DSU L3 cache and the L3D_CACHE* events do not count. The L2D_CACHE_WB, L2D_CACHE_WR and L2D_CACHE_REFILL_WR events do not count in this configuration. If neither the private L2 cache nor the DSU L3 cache are present, neither the L2D_CACHE* or L3D_CACHE* events will count\000event=0xf7\000\00000\000Level 2 cache stash dropped. This event counts on each stash request received from the interconnect or ACP, that is targeting L2 and gets dropped due to lack of buffer space to hold the request. L2 and L3 cache events (L2D_CACHE*, L3D_CACHE*) The behavior of these events depends on the configuration of the core. If the private L2 cache is present, the L2D_CACHE* events count the activity in the private L2 cache, and the L3D_CACHE* events count the activity in the DSU L3 cache (if present). If the private L2 cache is not present but the DSU L3 cache is present, the L2D_CACHE* events count activity in the DSU L3 cache and the L3D_CACHE* events do not count. The L2D_CACHE_WB, L2D_CACHE_WR and L2D_CACHE_REFILL_WR events do not count in this configuration. If neither the private L2 cache nor the DSU L3 cache are present, neither the L2D_CACHE* or L3D_CACHE* events will count\000"
/* offset=135692 */ "l2d_cache_stash_dropped\000\000cache\000Level 2 cache stash dropped.This event counts on each stash request received from the interconnect or ACP, that is targeting L2 and gets dropped due to lack of buffer space to hold the request\000event=0xd6\000\00000\000Level 2 cache stash dropped.This event counts on each stash request received from the interconnect or ACP, that is targeting L2 and gets dropped due to lack of buffer space to hold the request\000"
/* offset=136124 */ "l2d_cache_swap_local\000\000cache\000This event counts operations where demand access hits an L2 cache refill buffer allocated by software or hardware prefetch\000event=0x396\000\00000\000This event counts operations where demand access hits an L2 cache refill buffer allocated by software or hardware prefetch\000"
/* offset=136414 */ "l2d_cache_wb\000\000cache\000Attributable Level 2 data cache write-back\000event=0x18\000\00000\000Attributable Level 2 data cache write-back\000"
/* offset=136535 */ "l2d_cache_wb\000\000cache\000Attributable Level 2 data cache write-back\000event=0x18\000\00000\000This event counts any write-back of data from the L2 cache to outside the core. This includes snoops to the L2 which return data, regardless of whether they cause an invalidation. Invalidations from the L2 which do not write data outside of the core and snoops which return data from the L1 are not counted\000"
/* offset=136920 */ "l2d_cache_wb\000\000l2_cache\000Attributable Level 2 data cache write-back\000event=0x18\000\00000\000Counts write-backs of data from the L2 cache to outside the CPU. This includes snoops to the L2 (from other CPUs) which return data even if the snoops cause an invalidation. L2 cache line invalidations which do not write data outside the CPU and snoops which return data from an L1 cache are not counted. Data would not be written outside the cache when invalidating a clean cache line\000"
/* offset=137387 */ "l2d_cache_wb_clean\000\000cache\000L2D cache Write-Back, cleaning and coherency\000event=0x57\000\00000\000Level 2 data cache Write-Back, cleaning and coherency\000"
/* offset=137527 */ "l2d_cache_wb_clean\000\000core imp def\000L2D cache Write-Back, cleaning and coherency\000event=0x57\000\00000\000Level 2 data cache Write-Back, cleaning and coherency\000"
/* offset=137674 */ "l2d_cache_wb_clean\000\000l2_cache\000L2D cache Write-Back, cleaning and coherency\000event=0x57\000\00000\000Counts write-backs from the level 2 cache that are a result of either:\n\n1. Cache maintenance operations,\n\n2. Snoop responses or,\n\n3. Direct cache transfers to another CPU due to a forwarding snoop request\000"
/* offset=137968 */ "l2d_cache_wb_victim\000\000cache\000L2D cache Write-Back, victim\000event=0x56\000\00000\000Attributable Level 2 data cache Write-Back, victim\000"
/* offset=138090 */ "l2d_cache_wb_victim\000\000core imp def\000L2D cache Write-Back, victim\000event=0x56\000\00000\000Attributable Level 2 data cache Write-Back, victim\000"
/* offset=138219 */ "l2d_cache_wb_victim\000\000l2_cache\000L2D cache Write-Back, victim\000event=0x56\000\00000\000Counts evictions from the level 2 cache because of a line being allocated into the L2 cache\000"
/* offset=138385 */ "l2d_cache_wr\000\000cache\000L2D cache access, write\000event=0x51\000\00000\000Attributable Level 2 data cache access, write\000"
/* offset=138490 */ "l2d_cache_wr\000\000core imp def\000L2D cache access, write\000event=0x51\000\00000\000Attributable Level 2 data cache access, write\000"
/* offset=138602 */ "l2d_cache_wr\000\000l2_cache\000L2D cache access, write\000event=0x51\000\00000\000Counts level 2 cache accesses due to memory write operations. level 2 cache is a unified cache for data and instruction accesses, accesses are for misses in the level 1 caches or translation resolutions due to accesses\000"
/* offset=138883 */ "l2d_l2walk_tlb\000\000cache\000Level 2 TLB level-2 walk cache access.This event counts accesses to the level-2 walk cache where the last-level walk cache has missed. The event only counts when the translation regime of the pagewalk uses level 2 descriptors. This event does not count if the MMU is disabled\000event=0xd2\000\00000\000Level 2 TLB level-2 walk cache access.This event counts accesses to the level-2 walk cache where the last-level walk cache has missed. The event only counts when the translation regime of the pagewalk uses level 2 descriptors. This event does not count if the MMU is disabled\000"
/* offset=139472 */ "l2d_l2walk_tlb_refill\000\000cache\000Level 2 TLB level-2 walk cache refill.This event does not count if the MMU is disabled\000event=0xd3\000\00000\000Level 2 TLB level-2 walk cache refill.This event does not count if the MMU is disabled\000"
/* offset=139690 */ "l2d_llwalk_tlb\000\000cache\000Level 2 TLB last-level walk cache access.This event does not count if the MMU is disabled\000event=0xd0\000\00000\000Level 2 TLB last-level walk cache access.This event does not count if the MMU is disabled\000"
/* offset=139907 */ "l2d_llwalk_tlb_refill\000\000cache\000Level 2 TLB last-level walk cache refill.This event does not count if the MMU is disabled\000event=0xd1\000\00000\000Level 2 TLB last-level walk cache refill.This event does not count if the MMU is disabled\000"
/* offset=140131 */ "l2d_pref_line_fill\000\000cache\000Level 2 cache refill due to prefetch. +//0 If the core is configured with a per-core L2 cache: This event does not count. +//0 If the core is configured without a per-core L2 cache: This event counts the cluster cache event, as defined by L3_PREF_LINE_FILL. +//0 If there is neither a per-core cache nor a cluster cache configured, this event is not implemented\000event=0xc4\000\00000\000Level 2 cache refill due to prefetch. +//0 If the core is configured with a per-core L2 cache: This event does not count. +//0 If the core is configured without a per-core L2 cache: This event counts the cluster cache event, as defined by L3_PREF_LINE_FILL. +//0 If there is neither a per-core cache nor a cluster cache configured, this event is not implemented\000"
/* offset=140896 */ "l2d_s2_tlb\000\000cache\000L2 TLB IPA cache access. This event counts on each access to the IPA cache. If a single translation table walk needs to make multiple accesses to the IPA cache, each access is counted. If stage 2 translation is disabled, this event does not count\000event=0xd4\000\00000\000L2 TLB IPA cache access. This event counts on each access to the IPA cache. If a single translation table walk needs to make multiple accesses to the IPA cache, each access is counted. If stage 2 translation is disabled, this event does not count\000"
/* offset=141423 */ "l2d_s2_tlb\000\000cache\000Level 2 TLB IPA cache access. This event counts on each access to the IPA cache. +//0 If a single pagewalk needs to make multiple accesses to the IPA cache, each access is counted. +//0 If stage 2 translation is disabled, this event does not count\000event=0xd4\000\00000\000Level 2 TLB IPA cache access. This event counts on each access to the IPA cache. +//0 If a single pagewalk needs to make multiple accesses to the IPA cache, each access is counted. +//0 If stage 2 translation is disabled, this event does not count\000"
/* offset=141952 */ "l2d_s2_tlb_refill\000\000cache\000L2 TLB IPA cache refill. This event counts on each refill of the IPA cache. If a single translation table walk needs to make multiple accesses to the IPA cache, each access that causes a refill is counted. If stage 2 translation is disabled, this event does not count\000event=0xd5\000\00000\000L2 TLB IPA cache refill. This event counts on each refill of the IPA cache. If a single translation table walk needs to make multiple accesses to the IPA cache, each access that causes a refill is counted. If stage 2 translation is disabled, this event does not count\000"
/* offset=142528 */ "l2d_s2_tlb_refill\000\000cache\000Level 2 TLB IPA cache refill. This event counts on each refill of the IPA cache. +//0 If a single pagewalk needs to make multiple accesses to the IPA cache, each access which causes a refill is counted. +//0 If stage 2 translation is disabled, this event does not count\000event=0xd5\000\00000\000Level 2 TLB IPA cache refill. This event counts on each refill of the IPA cache. +//0 If a single pagewalk needs to make multiple accesses to the IPA cache, each access which causes a refill is counted. +//0 If stage 2 translation is disabled, this event does not count\000"
/* offset=143108 */ "l2d_swap_dm\000\000cache\000This event counts operations where demand access hits an L2 cache refill buffer allocated by software or hardware prefetch\000event=0x325\000\00000\000This event counts operations where demand access hits an L2 cache refill buffer allocated by software or hardware prefetch\000"
/* offset=143389 */ "l2d_tlb\000\000cache\000Attributable Level 2 data or unified TLB access\000event=0x2f\000\00000\000Attributable Level 2 data or unified TLB access\000"
/* offset=143515 */ "l2d_tlb\000\000cache\000Attributable Level 2 data or unified TLB access\000event=0x2f\000\00000\000This event counts on any access to the L2 TLB (caused by a refill of any of the L1 TLBs). This event does not count if the MMU is disabled\000"
/* offset=143732 */ "l2d_tlb\000\000tlb\000Attributable Level 2 data or unified TLB access\000event=0x2f\000\00000\000Counts level 2 TLB accesses except those caused by TLB maintenance operations\000"
/* offset=143886 */ "l2d_tlb_access\000\000cache\000L2D TLB access\000event=0x34\000\00000\000Level 2 access to data TLB that caused a page table walk. This event counts on any data access which causes L2D_TLB_REFILL to count\000"
/* offset=144070 */ "l2d_tlb_rd\000\000cache\000L2D cache access, read\000event=0x5e\000\00000\000Attributable Level 2 data or unified TLB access, read\000"
/* offset=144180 */ "l2d_tlb_rd\000\000core imp def\000L2D cache access, read\000event=0x5e\000\00000\000Attributable Level 2 data or unified TLB access, read\000"
/* offset=144297 */ "l2d_tlb_rd\000\000tlb\000L2D cache access, read\000event=0x5e\000\00000\000Counts level 2 TLB accesses caused by memory read operations from both data and instruction fetch except for those caused by TLB maintenance operations\000"
/* offset=144503 */ "l2d_tlb_refill\000\000cache\000Attributable L2 data or unified TLB refill\000event=0x2d\000\00000\000This event counts on anyrefill of the L2 TLB, caused by either an instruction or data access.This event does not count if the MMU is disabled\000"
/* offset=144725 */ "l2d_tlb_refill\000\000cache\000Attributable Level 2 data TLB refill\000event=0x2d\000\00000\000Attributable Level 2 data TLB refill\000"
/* offset=144836 */ "l2d_tlb_refill\000\000tlb\000Attributable Level 2 data TLB refill\000event=0x2d\000\00000\000Counts level 2 TLB refills caused by memory operations from both data and instruction fetch, except for those caused by TLB maintenance operations and hardware prefetches\000"
/* offset=145079 */ "l2d_tlb_refill_rd\000\000cache\000L2D cache refill, read\000event=0x5c\000\00000\000Attributable Level 2 data or unified TLB refill, read\000"
/* offset=145196 */ "l2d_tlb_refill_rd\000\000core imp def\000L2D cache refill, read\000event=0x5c\000\00000\000Attributable Level 2 data or unified TLB refill, read\000"
/* offset=145320 */ "l2d_tlb_refill_rd\000\000tlb\000L2D cache refill, read\000event=0x5c\000\00000\000Counts level 2 TLB refills caused by memory read operations from both data and instruction fetch except for those caused by TLB maintenance operations or hardware prefetches\000"
/* offset=145555 */ "l2d_tlb_refill_wr\000\000cache\000L2D cache refill, write\000event=0x5d\000\00000\000Attributable Level 2 data or unified TLB refill, write\000"
/* offset=145674 */ "l2d_tlb_refill_wr\000\000core imp def\000L2D cache refill, write\000event=0x5d\000\00000\000Attributable Level 2 data or unified TLB refill, write\000"
/* offset=145800 */ "l2d_tlb_refill_wr\000\000tlb\000L2D cache refill, write\000event=0x5d\000\00000\000Counts level 2 TLB refills caused by memory write operations from both data and instruction fetch except for those caused by TLB maintenance operations\000"
/* offset=146014 */ "l2d_tlb_wr\000\000cache\000L2D cache access, write\000event=0x5f\000\00000\000Attributable Level 2 data or unified TLB access, write\000"
/* offset=146126 */ "l2d_tlb_wr\000\000core imp def\000L2D cache access, write\000event=0x5f\000\00000\000Attributable Level 2 data or unified TLB access, write\000"
/* offset=146245 */ "l2d_tlb_wr\000\000tlb\000L2D cache access, write\000event=0x5f\000\00000\000Counts level 2 TLB accesses caused by memory write operations from both data and instruction fetch except for those caused by TLB maintenance operations\000"
/* offset=146453 */ "l2d_walk_tlb\000\000cache\000L2 TLB walk cache access. This event does not count if the MMU is disabled\000event=0xd0\000\00000\000L2 TLB walk cache access. This event does not count if the MMU is disabled\000"
/* offset=146638 */ "l2d_walk_tlb_refill\000\000cache\000L2 TLB walk cache refill. This event does not count if the MMU is disabled\000event=0xd1\000\00000\000L2 TLB walk cache refill. This event does not count if the MMU is disabled\000"
/* offset=146830 */ "l2d_ws_mode\000\000cache\000L2 cache write streaming mode. This event counts for each cycle where the core is in write streaming mode and is not allocating writes into the L2 cache\000event=0xc3\000\00000\000L2 cache write streaming mode. This event counts for each cycle where the core is in write streaming mode and is not allocating writes into the L2 cache\000"
/* offset=147170 */ "l2d_ws_mode\000\000cache\000Level 2 cache write streaming mode. This event counts for each cycle where the core is in write-streaming mode and not allocating writes into the L2 cache\000event=0xc3\000\00000\000Level 2 cache write streaming mode. This event counts for each cycle where the core is in write-streaming mode and not allocating writes into the L2 cache\000"
/* offset=147514 */ "l2d_ws_mode\000\000cache\000Level 2 cache write streaming mode. This event counts for each cycle where the core is in write-streaming mode and not allocating writes into the L2 cache\000event=0xc8\000\00000\000Level 2 cache write streaming mode. This event counts for each cycle where the core is in write-streaming mode and not allocating writes into the L2 cache\000"
/* offset=147858 */ "l2hwpf_inj_alloc_pf\000\000other\000This event counts allocation type prefetch injection requests to L2 cache generated by hardware prefetcher\000event=0x234\000\00000\000This event counts allocation type prefetch injection requests to L2 cache generated by hardware prefetcher\000"
/* offset=148115 */ "l2hwpf_inj_noalloc_pf\000\000other\000This event counts non-allocation type prefetch injection requests to L2 cache generated by hardware prefetcher\000event=0x235\000\00000\000This event counts non-allocation type prefetch injection requests to L2 cache generated by hardware prefetcher\000"
/* offset=148382 */ "l2hwpf_other\000\000other\000This event counts prefetch requests to L2 cache generated by the other causes\000event=0x236\000\00000\000This event counts prefetch requests to L2 cache generated by the other causes\000"
/* offset=148574 */ "l2hwpf_stream_pf\000\000other\000This event counts streaming prefetch requests to L2 cache generated by hardware prefecher\000event=0x233\000\00000\000This event counts streaming prefetch requests to L2 cache generated by hardware prefecher\000"
/* offset=148794 */ "l2i_tlb\000\000cache\000Attributable Level 2 instruction TLB access\000event=0x30\000\00000\000Attributable Level 2 instruction TLB access\000"
/* offset=148912 */ "l2i_tlb_access\000\000cache\000L2I TLB access\000event=0x35\000\00000\000Level 2 access to instruciton TLB that caused a page table walk. This event counts on any instruciton access which causes L2I_TLB_REFILL to count\000"
/* offset=149110 */ "l2i_tlb_refill\000\000cache\000Attributable Level 2 instruction TLB refill\000event=0x2e\000\00000\000Attributable Level 2 instruction TLB refill\000"
/* offset=149235 */ "l2pld_tlb\000\000cache\000Level 2 preload and MMU prefetcher TLB access. This event only counts software and hardware prefetches at Level 2\000event=0xe8\000\00000\000Level 2 preload and MMU prefetcher TLB access. This event only counts software and hardware prefetches at Level 2\000"
/* offset=149495 */ "l3_cache_rd\000\000cache\000L3 cache access, read\000event=0x40\000\00000\000Attributable Level 3 cache access, read\000"
/* offset=149591 */ "l3_pref_line_fill\000\000cache\000Level 3 cache refill due to prefetch. This event counts any linefills from the hardware prefetcher which cause an allocation into the L3 cache. Note It might not be possible to distinguish between both hardware and software prefetches and also which prefetches cause an allocation. If so, only hardware prefetches should be counted, regardless of whether they allocate. If either the core is configured without a per-core L2 or the cluster is configured without an L3 cache, this event is not implemented\000event=0xc5\000\00000\000Level 3 cache refill due to prefetch. This event counts any linefills from the hardware prefetcher which cause an allocation into the L3 cache. Note It might not be possible to distinguish between both hardware and software prefetches and also which prefetches cause an allocation. If so, only hardware prefetches should be counted, regardless of whether they allocate. If either the core is configured without a per-core L2 or the cluster is configured without an L3 cache, this event is not implemented\000"
/* offset=150641 */ "l3d_cache\000\000cache\000Attributable Level 3 data cache access\000event=0x2b\000\00000\000Attributable Level 3 data cache access\000"
/* offset=150751 */ "l3d_cache\000\000cache\000Attributable Level 3 unified cache access\000event=0x2b\000\00000\000This event counts for any cacheable read transaction returning datafrom the SCU, or for any cacheable write to the SCU\000"
/* offset=150944 */ "l3d_cache\000\000l3_cache\000Attributable Level 3 data cache access\000event=0x2b\000\00000\000Counts level 3 cache accesses. level 3 cache is a unified cache for data and instruction accesses. Accesses are for misses in the lower level caches or translation resolutions due to accesses\000"
/* offset=151210 */ "l3d_cache_allocate\000\000cache\000Allocation without refill\000event=0x29\000\00000\000This event counts any full cache line write into the L3 cache which does not cause a linefill, including write-backs from L2 to L3 and full-line writes which do not allocate into L2\000"
/* offset=151459 */ "l3d_cache_allocate\000\000cache\000Attributable Level 3 data cache allocation without refill\000event=0x29\000\00000\000Attributable Level 3 data cache allocation without refill\000"
/* offset=151616 */ "l3d_cache_allocate\000\000l3_cache\000Attributable Level 3 data cache allocation without refill\000event=0x29\000\00000\000Counts level 3 cache line allocates that do not fetch data from outside the level 3 data or unified cache. For example, allocates due to streaming stores\000"
/* offset=151872 */ "l3d_cache_lmiss_rd\000\000cache\000Level 3 data cache long-latency read miss\000event=0x400b\000\00000\000Level 3 data cache long-latency read miss.  The counter counts each memory read access counted by L3D_CACHE that incurs additional latency because it returns data from outside the Level 3 data or unified cache of this processing element.  The event indicates to software that the access missed in the Level 3 data or unified cache and might have a significant performance impact compared to the latency of an access that hits in the Level 3 data or unified cache\000"
/* offset=152420 */ "l3d_cache_rd\000\000cache\000Attributable Level 3 data or unified cache access, read\000event=0xa0\000\00000\000Attributable Level 3 data or unified cache access, read\000"
/* offset=152567 */ "l3d_cache_rd\000\000l3_cache\000Attributable Level 3 data or unified cache access, read\000event=0xa0\000\00000\000TBD\000"
/* offset=152665 */ "l3d_cache_refill\000\000cache\000Attributable Level 3 data cache refill\000event=0x2a\000\00000\000Attributable Level 3 data cache refill\000"
/* offset=152782 */ "l3d_cache_refill\000\000cache\000Attributable Level 3 unified cache refill\000event=0x2a\000\00000\000This event counts for any cacheable read transaction returning datafrom the SCU for which the data source was outside the cluster. Transactions such as ReadUnique are counted here as 'read' transactions, even though they can be generated by store instructions\000"
/* offset=153123 */ "l3d_cache_refill\000\000l3_cache\000Attributable Level 3 data cache refill\000event=0x2a\000\00000\000Counts level 3 accesses that receive data from outside the L3 cache\000"
/* offset=153272 */ "l3d_cache_refill_prefetch\000\000cache\000Level 3 cache refill due to prefetch. This event counts any linefills from the hardware prefetcher which cause an allocation into the L3 cache. Note It might not be possible to both distinguish hardware vs software prefetches and also which prefetches cause an allocation. If so, only hardware prefetches should be counted, regardless of whether they allocate. If either the core is configured without a per-core L2 or the cluster is configured without an L3 cache, this event is not implemented\000event=0xc0\000\00000\000Level 3 cache refill due to prefetch. This event counts any linefills from the hardware prefetcher which cause an allocation into the L3 cache. Note It might not be possible to both distinguish hardware vs software prefetches and also which prefetches cause an allocation. If so, only hardware prefetches should be counted, regardless of whether they allocate. If either the core is configured without a per-core L2 or the cluster is configured without an L3 cache, this event is not implemented\000"
/* offset=154312 */ "l3d_cache_refill_rd\000\000cache\000Attributable Level 3 data or unified cache refill, read\000event=0xa2\000\00000\000Attributable Level 3 data or unified cache refill, read\000"
/* offset=154466 */ "l3d_ws_mode\000\000cache\000L3 cache write streaming mode. This event counts for each cycle where the core is in write streaming mode and is not allocating writes into the L3 cache\000event=0xc7\000\00000\000L3 cache write streaming mode. This event counts for each cycle where the core is in write streaming mode and is not allocating writes into the L3 cache\000"
/* offset=154806 */ "l3d_ws_mode\000\000cache\000Level 3 cache write streaming mode. This event counts for each cycle where the core is in write-streaming mode and not allocating writes into the L3 cache\000event=0xc9\000\00000\000Level 3 cache write streaming mode. This event counts for each cycle where the core is in write-streaming mode and not allocating writes into the L3 cache\000"
/* offset=155150 */ "l3d_ws_mode\000\000cache\000Level 3 cache write streaming mode.This event counts for each cycle where the core is in write-streaming mode and not allocating writes into the L3 cache\000event=0xc7\000\00000\000Level 3 cache write streaming mode.This event counts for each cycle where the core is in write-streaming mode and not allocating writes into the L3 cache\000"
/* offset=155492 */ "ld_align_lat\000\000memory\000Load with additional latency from alignment\000event=0x4021\000\00000\000Load with additional latency from alignment\000"
/* offset=155618 */ "ld_comp_wait\000\000other\000This event counts every cycle that no instruction was committed because the oldest and uncommitted load/store/prefetch operation waits for L1D cache, L2 cache and memory access\000event=0x184\000\00000\000This event counts every cycle that no instruction was committed because the oldest and uncommitted load/store/prefetch operation waits for L1D cache, L2 cache and memory access\000"
/* offset=156008 */ "ld_comp_wait_ex\000\000other\000This event counts every cycle that no instruction was committed because the oldest and uncommitted integer load operation waits for L1D cache, L2 cache and memory access\000event=0x185\000\00000\000This event counts every cycle that no instruction was committed because the oldest and uncommitted integer load operation waits for L1D cache, L2 cache and memory access\000"
/* offset=156387 */ "ld_comp_wait_l1_miss\000\000other\000This event counts every cycle that no instruction was committed because the oldest and uncommitted load/store/prefetch operation waits for L2 cache access\000event=0x182\000\00000\000This event counts every cycle that no instruction was committed because the oldest and uncommitted load/store/prefetch operation waits for L2 cache access\000"
/* offset=156741 */ "ld_comp_wait_l1_miss_ex\000\000other\000This event counts every cycle that no instruction was committed because the oldest and uncommitted integer load operation waits for L2 cache access\000event=0x183\000\00000\000This event counts every cycle that no instruction was committed because the oldest and uncommitted integer load operation waits for L2 cache access\000"
/* offset=157084 */ "ld_comp_wait_l2_miss\000\000other\000This event counts every cycle that no operation was committed because the oldest and uncommitted load/store/prefetch operation waits for memory access\000event=0x180\000\00000\000This event counts every cycle that no operation was committed because the oldest and uncommitted load/store/prefetch operation waits for memory access\000"
/* offset=157430 */ "ld_comp_wait_l2_miss_ex\000\000other\000This event counts every cycle that no instruction was committed because the oldest and uncommitted integer load operation waits for memory access\000event=0x181\000\00000\000This event counts every cycle that no instruction was committed because the oldest and uncommitted integer load operation waits for memory access\000"
/* offset=157769 */ "ld_comp_wait_pfp_busy\000\000other\000This event counts every cycle that no instruction was committed due to the lack of an available prefetch port\000event=0x186\000\00000\000This event counts every cycle that no instruction was committed due to the lack of an available prefetch port\000"
/* offset=158034 */ "ld_comp_wait_pfp_busy_ex\000\000other\000This event counts the LD_COMP_WAIT_PFP_BUSY caused by an integer load operation\000event=0x187\000\00000\000This event counts the LD_COMP_WAIT_PFP_BUSY caused by an integer load operation\000"
/* offset=158242 */ "ld_comp_wait_pfp_busy_swpf\000\000other\000This event counts the LD_COMP_WAIT_PFP_BUSY caused by a software prefetch instruction\000event=0x188\000\00000\000This event counts the LD_COMP_WAIT_PFP_BUSY caused by a software prefetch instruction\000"
/* offset=158464 */ "ld_dep_stall\000\000pipeline\000Cycles there is a stall in the Wr stage because of a load miss\000event=0xe7\000\00000\000\000"
/* offset=158566 */ "ld_from_st_fwd\000\000core imp def\000Load satisified from store forwarded data\000event=0xd605\000\00000\000Load satisified from store forwarded data\000"
/* offset=158696 */ "ld_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, load\000event=0x6\000\00000\000Instruction architecturally executed, condition code check pass, load\000"
/* offset=158874 */ "ld_retired\000\000memory\000Instruction architecturally executed, condition code check pass, load\000event=0x6\000\00000\000Instruction architecturally executed, condition code check pass, load\000"
/* offset=159047 */ "ld_spec\000\000instruction\000Operation speculatively executed, load\000event=0x70\000\00000\000Operation speculatively executed, load\000"
/* offset=159161 */ "ld_spec\000\000spec_operation\000Operation speculatively executed, load\000event=0x70\000\00000\000Counts speculatively executed load operations including Single Instruction Multiple Data (SIMD) load operations\000"
/* offset=159351 */ "ldrex_spec\000\000instruction\000Exclusive operation speculatively executed, LDREX or LDX\000event=0x6c\000\00000\000Exclusive operation speculatively executed, LDREX or LDX\000"
/* offset=159504 */ "ldrex_spec\000\000intrinsic\000Exclusive operation speculatively executed, LDREX or LDX\000event=0x6c\000\00000\000Exclusive operation speculatively executed, LDREX or LDX\000"
/* offset=159655 */ "ldrex_spec\000\000spec_operation\000Exclusive operation speculatively executed, LDREX or LDX\000event=0x6c\000\00000\000Counts Load-Exclusive operations that have been speculatively executed. Eg: LDREX, LDX\000"
/* offset=159841 */ "ldst_align_lat\000\000memory\000Access with additional latency from alignment\000event=0x4020\000\00000\000Access with additional latency from alignment\000"
/* offset=159973 */ "ldst_spec\000\000instruction\000Operation speculatively executed, load or store\000event=0x72\000\00000\000Operation speculatively executed, load or store\000"
/* offset=160107 */ "lf_stall\000\000pipeline\000A linefill caused an instruction side stall\000event=0xc0\000\00000\000A linefill caused an instruction side stall\000"
/* offset=160229 */ "ll_cache_miss_rd\000\000cache\000Last level cache miss, read\000event=0x37\000\00000\000Last level cache miss, read\000"
/* offset=160324 */ "ll_cache_miss_rd\000\000ll_cache\000Last level cache miss, read\000event=0x37\000\00000\000Counts read transactions that were returned from outside the core cluster but missed in the system level cache. This event counts when the system register CPUECTLR.EXTLLC bit is set. This event counts read transactions returned from outside the core if those transactions are missed in the System level Cache. The data source of the transaction is indicated by a field in the CHI transaction returning to the CPU. This event does not count reads caused by cache maintenance operations\000"
/* offset=160879 */ "ll_cache_rd\000\000cache\000Attributable Last level cache memory read\000event=0x36\000\00000\000Attributable Last level cache memory read\000"
/* offset=160997 */ "ll_cache_rd\000\000ll_cache\000Attributable Last level cache memory read\000event=0x36\000\00000\000Counts read transactions that were returned from outside the core cluster. This event counts when the system register CPUECTLR.EXTLLC bit is set. This event counts read transactions returned from outside the core if those transactions are either hit in the system level cache or missed in the SLC and are returned from any other external sources\000"
/* offset=161422 */ "ll_ws_mode\000\000cache\000Last level cache write streaming mode. This event counts for each cycle where the core is in write streaming mode and is not allocating writes into the system cache\000event=0xc8\000\00000\000Last level cache write streaming mode. This event counts for each cycle where the core is in write streaming mode and is not allocating writes into the system cache\000"
/* offset=161785 */ "ls_iq_full\000\000pipeline\000Duration for which all slots in the load-store issue queue are busy. This event counts the cycles where all slots in the LS IQs are full with micro-operations waiting for issuing, and the dispatch stage is not empty\000event=0xd8\000\00000\000Duration for which all slots in the load-store issue queue are busy. This event counts the cycles where all slots in the LS IQs are full with micro-operations waiting for issuing, and the dispatch stage is not empty\000"
/* offset=162253 */ "ls_iq_full\000\000pipeline\000Duration for which all slots in the load-store issue queue are busy\000event=0xd8\000\00000\000Duration for which all slots in the load-store issue queue are busy\000"
/* offset=162425 */ "lsu_st_retire_stall\000\000core imp def\000Store retirement pipe stall\000event=0xd60c\000\00000\000Store retirement pipe stall\000"
/* offset=162532 */ "lx_stall\000\000pipeline\000LX stalled\000event=0x10d\000\00000\000LX Op non-issue\000"
/* offset=162594 */ "mediated_num\000hisi_sccl,hha\000uncore hha\000Unit: hisi_sccl,hha \000config=0x32\000\00000\000\000"
/* offset=162670 */ "mem_access\000\000memory\000Data memory access\000event=0x13\000\00000\000Counts memory accesses issued by the CPU load store unit, where those accesses are issued due to load or store operations. This event counts memory accesses no matter whether the data is received from any level of cache hierarchy or external memory. If memory accesses are broken up into smaller transactions than what were specified in the load or store instructions, then the event counts those smaller memory transactions\000"
/* offset=163148 */ "mem_access\000\000memory\000Data memory access\000event=0x13\000\00000\000Data memory access\000"
/* offset=163220 */ "mem_access\000\000memory\000Data memory access\000event=0x13\000\00000\000This event counts memory accesses due to load or store instructions. This event counts the sum of MEM_ACCESS_RD and MEM_ACCESS_WR\000"
/* offset=163403 */ "mem_access_checked\000\000memory\000Checked data memory access\000event=0x4024\000\00000\000Checked data memory access\000"
/* offset=163501 */ "mem_access_checked_rd\000\000memory\000Checked data memory access, read\000event=0x4025\000\00000\000Checked data memory access, read\000"
/* offset=163614 */ "mem_access_checked_wr\000\000memory\000Checked data memory access, write\000event=0x4026\000\00000\000Checked data memory access, write\000"
/* offset=163729 */ "mem_access_rd\000\000core imp def\000Data memory access, read\000event=0x66\000\00000\000Data memory access, read\000"
/* offset=163822 */ "mem_access_rd\000\000memory\000Data memory access, read\000event=0x66\000\00000\000Counts memory accesses issued by the CPU due to load operations. The event counts any memory load access, no matter whether the data is received from any level of cache hierarchy or external memory. The event also counts atomic load operations. If memory accesses are broken up by the load/store unit into smaller transactions that are issued by the bus interface, then the event counts those smaller transactions\000"
/* offset=164298 */ "mem_access_rd\000\000memory\000Data memory access, read\000event=0x66\000\00000\000Data memory access, read\000"
/* offset=164385 */ "mem_access_wr\000\000core imp def\000Data memory access, write\000event=0x67\000\00000\000Data memory access, write\000"
/* offset=164480 */ "mem_access_wr\000\000memory\000Data memory access, write\000event=0x67\000\00000\000Counts memory accesses issued by the CPU due to store operations. The event counts any memory store access, no matter whether the data is located in any level of cache or external memory. The event also counts atomic load and store operations. If memory accesses are broken up by the load/store unit into smaller transactions that are issued by the bus interface, then the event counts those smaller transactions\000"
/* offset=164956 */ "mem_access_wr\000\000memory\000Data memory access, write\000event=0x67\000\00000\000Data memory access, write\000"
/* offset=165045 */ "mem_stall_anyload\000\000core imp def\000No any micro operation is issued and meanwhile any load operation is not resolved\000event=0x7004\000\00000\000No any micro operation is issued and meanwhile any load operation is not resolved\000"
/* offset=165258 */ "mem_stall_l1miss\000\000core imp def\000No any micro operation is issued and meanwhile there is any load operation missing L1 cache and pending data refill\000event=0x7006\000\00000\000No any micro operation is issued and meanwhile there is any load operation missing L1 cache and pending data refill\000"
/* offset=165538 */ "mem_stall_l2miss\000\000core imp def\000No any micro operation is issued and meanwhile there is any load operation missing both L1 and L2 cache and pending data refill from L3 cache\000event=0x7007\000\00000\000No any micro operation is issued and meanwhile there is any load operation missing both L1 and L2 cache and pending data refill from L3 cache\000"
/* offset=165870 */ "memory_error\000\000exception\000Local memory error\000event=0x1a\000\00000\000Local memory error\000"
/* offset=165947 */ "memory_error\000\000exception\000Local memory error\000event=0x1a\000\00000\000This event counts any correctable or uncorrectable memory error (ECC or parity) in the protected core RAMs\000"
/* offset=166112 */ "memory_error\000\000memory\000Local memory error\000event=0x1a\000\00000\000Counts any detected correctable or uncorrectable physical memory errors (ECC or parity) in protected CPUs RAMs. On the core, this event counts errors in the caches (including data and tag rams). Any detected memory error (from either a speculative and abandoned access, or an architecturally executed access) is counted. Note that errors are only detected when the actual protected memory is accessed by an operation\000"
/* offset=166584 */ "memory_error\000\000memory\000Local memory error\000event=0x1a\000\00000\000Local memory error\000"
/* offset=166658 */ "memory_error\000\000memory\000Local memory error\000event=0x1a\000\00000\000This event counts any correctable or uncorrectable memory error (ECC or parity) in the protected core RAMs\000"
/* offset=166820 */ "mmu_d_otb_alloc\000\000core imp def\000L2D OTB allocate\000event=0xd800\000\00000\000L2D OTB allocate\000"
/* offset=166901 */ "mmu_d_s1_walk_cache_lookup\000\000core imp def\000D-side S1 Page walk cache lookup\000event=0xd807\000\00000\000D-side S1 Page walk cache lookup\000"
/* offset=167025 */ "mmu_d_s1_walk_cache_refill\000\000core imp def\000D-side S1 Page walk cache refill\000event=0xd808\000\00000\000D-side S1 Page walk cache refill\000"
/* offset=167149 */ "mmu_d_s1_walk_fault\000\000core imp def\000D-side Stage1 tablewalk fault\000event=0xd80b\000\00000\000D-side Stage1 tablewalk fault\000"
/* offset=167260 */ "mmu_d_s2_walk_cache_lookup\000\000core imp def\000D-side S2 Page walk cache lookup\000event=0xd809\000\00000\000D-side S2 Page walk cache lookup\000"
/* offset=167384 */ "mmu_d_s2_walk_cache_refill\000\000core imp def\000D-side S2 Page walk cache refill\000event=0xd80a\000\00000\000D-side S2 Page walk cache refill\000"
/* offset=167508 */ "mmu_d_s2_walk_fault\000\000core imp def\000D-side Stage2 tablewalk fault\000event=0xd80c\000\00000\000D-side Stage2 tablewalk fault\000"
/* offset=167619 */ "mmu_d_trans_cache_hit_s1l0_walk\000\000core imp def\000DTLB Translation cache hit on S1L0 walk cache entry\000event=0xd803\000\00000\000DTLB Translation cache hit on S1L0 walk cache entry\000"
/* offset=167786 */ "mmu_d_trans_cache_hit_s1l1_walk\000\000core imp def\000DTLB Translation cache hit on S1L1 walk cache entry\000event=0xd802\000\00000\000DTLB Translation cache hit on S1L1 walk cache entry\000"
/* offset=167953 */ "mmu_d_trans_cache_hit_s1l2_walk\000\000core imp def\000DTLB Translation cache hit on S1L2 walk cache entry\000event=0xd801\000\00000\000DTLB Translation cache hit on S1L2 walk cache entry\000"
/* offset=168120 */ "mmu_d_trans_cache_hit_s2l0_walk\000\000core imp def\000DTLB Translation cache hit on S2L0 walk cache entry\000event=0xd806\000\00000\000DTLB Translation cache hit on S2L0 walk cache entry\000"
/* offset=168287 */ "mmu_d_trans_cache_hit_s2l1_walk\000\000core imp def\000DTLB Translation cache hit on S2L1 walk cache entry\000event=0xd805\000\00000\000DTLB Translation cache hit on S2L1 walk cache entry\000"
/* offset=168454 */ "mmu_d_trans_cache_hit_s2l2_walk\000\000core imp def\000DTLB Translation cache hit on S2L2 walk cache entry\000event=0xd804\000\00000\000DTLB Translation cache hit on S2L2 walk cache entry\000"
/* offset=168621 */ "mmu_d_walk_steps\000\000core imp def\000D-side Tablewalk steps or descriptor fetches\000event=0xd80d\000\00000\000D-side Tablewalk steps or descriptor fetches\000"
/* offset=168759 */ "mmu_i_otb_alloc\000\000core imp def\000L2I OTB allocate\000event=0xd900\000\00000\000L2I OTB allocate\000"
/* offset=168840 */ "mmu_i_s1_walk_cache_lookup\000\000core imp def\000I-side S1 Page walk cache lookup\000event=0xd907\000\00000\000I-side S1 Page walk cache lookup\000"
/* offset=168964 */ "mmu_i_s1_walk_cache_refill\000\000core imp def\000I-side S1 Page walk cache refill\000event=0xd908\000\00000\000I-side S1 Page walk cache refill\000"
/* offset=169088 */ "mmu_i_s1_walk_fault\000\000core imp def\000I-side Stage1 tablewalk fault\000event=0xd90b\000\00000\000I-side Stage1 tablewalk fault\000"
/* offset=169199 */ "mmu_i_s2_walk_cache_lookup\000\000core imp def\000I-side S2 Page walk cache lookup\000event=0xd909\000\00000\000I-side S2 Page walk cache lookup\000"
/* offset=169323 */ "mmu_i_s2_walk_cache_refill\000\000core imp def\000I-side S2 Page walk cache refill\000event=0xd90a\000\00000\000I-side S2 Page walk cache refill\000"
/* offset=169447 */ "mmu_i_s2_walk_fault\000\000core imp def\000I-side Stage2 tablewalk fault\000event=0xd90c\000\00000\000I-side Stage2 tablewalk fault\000"
/* offset=169558 */ "mmu_i_trans_cache_hit_s1l0_walk\000\000core imp def\000ITLB Translation cache hit on S1L0 walk cache entry\000event=0xd903\000\00000\000ITLB Translation cache hit on S1L0 walk cache entry\000"
/* offset=169725 */ "mmu_i_trans_cache_hit_s1l1_walk\000\000core imp def\000ITLB Translation cache hit on S1L1 walk cache entry\000event=0xd902\000\00000\000ITLB Translation cache hit on S1L1 walk cache entry\000"
/* offset=169892 */ "mmu_i_trans_cache_hit_s1l2_walk\000\000core imp def\000ITLB Translation cache hit on S1L2 walk cache entry\000event=0xd901\000\00000\000ITLB Translation cache hit on S1L2 walk cache entry\000"
/* offset=170059 */ "mmu_i_trans_cache_hit_s2l0_walk\000\000core imp def\000ITLB Translation cache hit on S2L0 walk cache entry\000event=0xd906\000\00000\000ITLB Translation cache hit on S2L0 walk cache entry\000"
/* offset=170226 */ "mmu_i_trans_cache_hit_s2l1_walk\000\000core imp def\000ITLB Translation cache hit on S2L1 walk cache entry\000event=0xd905\000\00000\000ITLB Translation cache hit on S2L1 walk cache entry\000"
/* offset=170393 */ "mmu_i_trans_cache_hit_s2l2_walk\000\000core imp def\000ITLB Translation cache hit on S2L2 walk cache entry\000event=0xd904\000\00000\000ITLB Translation cache hit on S2L2 walk cache entry\000"
/* offset=170560 */ "mmu_i_walk_steps\000\000core imp def\000I-side Tablewalk steps or descriptor fetches\000event=0xd90d\000\00000\000I-side Tablewalk steps or descriptor fetches\000"
/* offset=170698 */ "mmu_ptw\000\000mmu\000Duration of a translation table walk handled by the MMU\000event=0xe0\000\00000\000Duration of a translation table walk handled by the MMU\000"
/* offset=170838 */ "mmu_ptw_cp15\000\000mmu\000Duration of a translation table walk requested by a CP15 operation (maintenance by MVA and VA to PA operations)\000event=0xe6\000\00000\000Duration of a translation table walk requested by a CP15 operation (maintenance by MVA and VA to PA operations)\000"
/* offset=171095 */ "mmu_ptw_cp15\000\000mmu\000Duration of a translation table walk requested by an address translation operation\000event=0xe6\000\00000\000Duration of a translation table walk requested by an address translation operation\000"
/* offset=171294 */ "mmu_ptw_iside\000\000mmu\000Duration of a translation table walk requested by the Instruction Side\000event=0xe4\000\00000\000Duration of a translation table walk requested by the Instruction Side\000"
/* offset=171470 */ "mmu_ptw_iside\000\000mmu\000Duration of a translation table walk requested by the instruction side\000event=0xe4\000\00000\000Duration of a translation table walk requested by the instruction side\000"
/* offset=171646 */ "mmu_ptw_lsu\000\000mmu\000Duration of a translation table walk requested by the LSU\000event=0xe3\000\00000\000Duration of a translation table walk requested by the LSU\000"
/* offset=171794 */ "mmu_ptw_pld\000\000mmu\000Duration of a translation table walk requested by a Preload instruction or Prefetch request\000event=0xe5\000\00000\000Duration of a translation table walk requested by a Preload instruction or Prefetch request\000"
/* offset=172010 */ "mmu_ptw_st1\000\000mmu\000Duration of a Stage 1 translation table walk handled by the MMU. This event is not counted when it is accessible from Non-secure EL0 or EL1\000event=0xe1\000\00000\000Duration of a Stage 1 translation table walk handled by the MMU. This event is not counted when it is accessible from Non-secure EL0 or EL1\000"
/* offset=172322 */ "mmu_ptw_st1\000\000mmu\000Duration of a Stage 1 translation table walk handled by the MMU\000event=0xe1\000\00000\000Duration of a Stage 1 translation table walk handled by the MMU\000"
/* offset=172482 */ "mmu_ptw_st2\000\000mmu\000Duration of a Stage 2 translation table walk handled by the MMU. This event is not counted when it is accessible from Non-secure EL0 or EL1\000event=0xe2\000\00000\000Duration of a Stage 2 translation table walk handled by the MMU. This event is not counted when it is accessible from Non-secure EL0 or EL1\000"
/* offset=172794 */ "mmu_ptw_st2\000\000mmu\000Duration of a Stage 2 translation table walk handled by the MMU\000event=0xe2\000\00000\000Duration of a Stage 2 translation table walk handled by the MMU\000"
/* offset=172954 */ "msc_etm_extout0\000\000core imp def\000ETM extout bit 0\000event=0x141\000\00000\000ETM extout bit 0\000"
/* offset=173034 */ "msc_etm_extout1\000\000core imp def\000ETM extout bit 1\000event=0x142\000\00000\000ETM extout bit 1\000"
/* offset=173114 */ "msc_etm_extout2\000\000core imp def\000ETM extout bit 2\000event=0x143\000\00000\000ETM extout bit 2\000"
/* offset=173194 */ "msc_etm_extout3\000\000core imp def\000ETM extout bit 3\000event=0x144\000\00000\000ETM extout bit 3\000"
/* offset=173274 */ "nop_spec\000\000instruction\000Speculatively executed, NOP\000event=0x100\000\00000\000Operation speculatively executed, NOP\000"
/* offset=173378 */ "ofb_full\000\000core imp def\000OFB full cycles\000event=0xd601\000\00000\000OFB full cycles\000"
/* offset=173450 */ "op_retired\000\000instruction\000Micro-operation architecturally executed\000event=0x3a\000\00000\000Micro-operation architecturally executed.  The counter counts each operation counted by OP_SPEC that would be executed in a simple sequential execution of the program\000"
/* offset=173697 */ "op_spec\000\000instruction\000Micro-operation speculatively executed\000event=0x3b\000\00000\000Micro-operation speculatively executed.  The counter counts the number of operations executed by the processing element, including those that are executed speculatively and would not be executed in a simple sequential execution of the program\000"
/* offset=174015 */ "other_interlock_stall\000\000pipeline\000Cycles there is an interlock other than  Advanced SIMD/Floating-point instructions or load/store instruction\000event=0xe4\000\00000\000\000"
/* offset=174172 */ "other_iq_dep_stall\000\000pipeline\000Cycles that the DPU IQ is empty and that is not because of a recent micro-TLB miss, instruction cache miss or pre-decode error\000event=0xe0\000\00000\000\000"
/* offset=174344 */ "page_walk_l0_stage1_hit\000\000cache\000Page walk, L0 stage-1 hit\000event=0x112\000\00000\000Page walk cache level-0 stage-1 hit\000"
/* offset=174453 */ "page_walk_l1_stage1_hit\000\000cache\000Page walk, L1 stage-1 hit\000event=0x113\000\00000\000Page walk cache level-1 stage-1 hit\000"
/* offset=174562 */ "page_walk_l1_stage2_hit\000\000cache\000Page walk, L1 stage-2 hit\000event=0x115\000\00000\000Page walk cache level-1 stage-2 hit\000"
/* offset=174671 */ "page_walk_l2_stage1_hit\000\000cache\000Page walk, L2 stage-1 hit\000event=0x114\000\00000\000Page walk cache level-2 stage-1 hit\000"
/* offset=174780 */ "page_walk_l2_stage2_hit\000\000cache\000Page walk, L2 stage-2 hit\000event=0x116\000\00000\000Page walk cache level-2 stage-2 hit\000"
/* offset=174889 */ "pc_write_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, software change of the PC\000event=0xc\000\00000\000Instruction architecturally executed, condition code check pass, software change of the PC\000"
/* offset=175115 */ "pc_write_spec\000\000instruction\000Operation speculatively executed, software change of the PC\000event=0x76\000\00000\000Operation speculatively executed, software change of the PC\000"
/* offset=175277 */ "pc_write_spec\000\000spec_operation\000Operation speculatively executed, software change of the PC\000event=0x76\000\00000\000Counts speculatively executed operations which cause software changes of the PC. Those operations include all taken branch operations\000"
/* offset=175516 */ "pld_utlb_refill\000\000cache\000Level 1 PLD TLB refill\000event=0xe7\000\00000\000Level 1 PLD TLB refill\000"
/* offset=175600 */ "pldtlb_walk\000\000cache\000Prefetch access to unified TLB that caused a page table walk. This event counts software and hardware prefetches\000event=0xec\000\00000\000Prefetch access to unified TLB that caused a page table walk. This event counts software and hardware prefetches\000"
/* offset=175860 */ "pmu_hovfs\000\000pmu\000PMU overflow, counters reserved for use by EL2\000event=0x400f\000\00000\000PMU overflow, counters reserved for use by EL2\000"
/* offset=175986 */ "pmu_ovfs\000\000pmu\000PMU overflow, counters accessible to EL1 and EL0\000event=0x400d\000\00000\000PMU overflow, counters accessible to EL1 and EL0\000"
/* offset=176115 */ "post_index_spec\000\000instruction\000This event counts architecturally executed operations that uses 'post-index' as its addressing mode\000event=0x124\000\00000\000This event counts architecturally executed operations that uses 'post-index' as its addressing mode\000"
/* offset=176360 */ "prd_spec\000\000instruction\000This event counts architecturally executed operations that using predicate register\000event=0x108\000\00000\000This event counts architecturally executed operations that using predicate register\000"
/* offset=176566 */ "pre_cmd\000hisi_sccl,ddrc\000uncore ddrc\000DDRC precharge commands. Unit: hisi_sccl,ddrc \000config=0x4\000\00000\000DDRC precharge commands\000"
/* offset=176687 */ "pre_decode_err\000\000other\000Pre-decode error\000event=0xc6\000\00000\000\000"
/* offset=176742 */ "pre_index_spec\000\000instruction\000This event counts architecturally executed operations that uses 'pre-index' as its addressing mode\000event=0x123\000\00000\000This event counts architecturally executed operations that uses 'pre-index' as its addressing mode\000"
/* offset=176984 */ "predecode_error\000\000exception\000Predecode error\000event=0xc6\000\00000\000Predecode error\000"
/* offset=177058 */ "prefetch_drop\000hisi_sccl,l3c\000uncore l3c\000Count of the number of prefetch drops from this L3C. Unit: hisi_sccl,l3c \000config=0x42\000\00000\000Count of the number of prefetch drops from this L3C\000"
/* offset=177239 */ "prefetch_linefill\000\000cache\000Linefill because of prefetch\000event=0xc2\000\00000\000\000"
/* offset=177309 */ "prefetch_linefill_drop\000\000cache\000Instruction Cache Throttle occurred\000event=0xc3\000\00000\000\000"
/* offset=177391 */ "prf_req\000\000core imp def\000Prefetch request from LSU\000event=0x6013\000\00000\000Prefetch request from LSU\000"
/* offset=177482 */ "prf_spec\000\000sve\000Prefetch Operations speculatively executed\000event=0x8087\000\00000\000Prefetch Operations speculatively executed\000"
/* offset=177599 */ "prx_val\000\000pipeline\000This event counts valid cycles of PRX pipeline\000event=0x1a6\000\00000\000This event counts valid cycles of PRX pipeline\000"
/* offset=177727 */ "ptw_stall\000\000pipeline\000A translation table walk caused an instruction side stall\000event=0xc1\000\00000\000A translation table walk caused an instruction side stall\000"
/* offset=177878 */ "rc_ld_spec\000\000instruction\000Release consistency operation speculatively executed, Load-Acquire\000event=0x90\000\00000\000Release consistency operation speculatively executed, Load-Acquire\000"
/* offset=178051 */ "rc_ld_spec\000\000spec_operation\000Release consistency operation speculatively executed, Load-Acquire\000event=0x90\000\00000\000Counts any load acquire operations that are speculatively executed. Eg: LDAR, LDARH, LDARB\000"
/* offset=178251 */ "rc_st_spec\000\000instruction\000Release consistency operation speculatively executed, Store-Release\000event=0x91\000\00000\000Release consistency operation speculatively executed, Store-Release\000"
/* offset=178426 */ "rc_st_spec\000\000spec_operation\000Release consistency operation speculatively executed, Store-Release\000event=0x91\000\00000\000Counts any store release operations that are speculatively executed. Eg: STLR, STLRH, STLRB'\000"
/* offset=178629 */ "rd_cpipe\000hisi_sccl,l3c\000uncore l3c\000Total read accesses. Unit: hisi_sccl,l3c \000config=0\000\00000\000Total read accesses\000"
/* offset=178738 */ "rd_ddr_128b\000hisi_sccl,hha\000uncore hha\000The number of read operations sent by HHA to DDRC which size is 128 bytes. Unit: hisi_sccl,hha \000config=0x1e\000\00000\000The number of read operations sent by HHA to DDRC which size is 128 bytes\000"
/* offset=178961 */ "rd_ddr_64b\000hisi_sccl,hha\000uncore hha\000The number of read operations sent by HHA to DDRC which size is 64 bytes. Unit: hisi_sccl,hha \000config=0x1c\000\00000\000The number of read operations sent by HHA to DDRC which size is 64bytes\000"
/* offset=179180 */ "rd_hit_cpipe\000hisi_sccl,l3c\000uncore l3c\000Total read hits. Unit: hisi_sccl,l3c \000config=0x2\000\00000\000Total read hits\000"
/* offset=179287 */ "rd_hit_spipe\000hisi_sccl,l3c\000uncore l3c\000Count of the number of read lines that hits in spipe of this L3C. Unit: hisi_sccl,l3c \000config=0x22\000\00000\000Count of the number of read lines that hits in spipe of this L3C\000"
/* offset=179493 */ "rd_spipe\000hisi_sccl,l3c\000uncore l3c\000Count of the number of read lines that come from this cluster of CPU core in spipe. Unit: hisi_sccl,l3c \000config=0x20\000\00000\000Count of the number of read lines that come from this cluster of CPU core in spipe\000"
/* offset=179731 */ "read_alloc\000\000cache\000Read allocate mode\000event=0xc5\000\00000\000\000"
/* offset=179784 */ "read_alloc_enter\000\000cache\000Entering read allocate mode\000event=0xc4\000\00000\000\000"
/* offset=179852 */ "remote_access\000\000memory\000Access to another socket in a multi-socket system\000event=0x31\000\00000\000Access to another socket in a multi-socket system\000"
/* offset=179989 */ "remote_access\000\000memory\000Access to another socket in a multi-socket system\000event=0x31\000\00000\000Counts accesses to another chip, which is implemented as a different CMN mesh in the system. If the CHI bus response back to the core indicates that the data source is from another chip (mesh), then the counter is updated. If no data is returned, even if the system snoops another chip/mesh, then the counter is not updated\000"
/* offset=180400 */ "remote_access_rd\000\000memory\000Attributable memory read access to another socket in a multi-socket system\000event=0x38\000\00000\000Attributable memory read access to another socket in a multi-socket system\000"
/* offset=180590 */ "retry_cpu\000hisi_sccl,l3c\000uncore l3c\000Count of the number of retry that L3C suppresses the CPU operations. Unit: hisi_sccl,l3c \000config=0x40\000\00000\000Count of the number of retry that L3C suppresses the CPU operations\000"
/* offset=180799 */ "retry_ring\000hisi_sccl,l3c\000uncore l3c\000Count of the number of retry that L3C suppresses the ring operations. Unit: hisi_sccl,l3c \000config=0x41\000\00000\000Count of the number of retry that L3C suppresses the ring operations\000"
/* offset=181011 */ "rnk_chg\000hisi_sccl,ddrc\000uncore ddrc\000DDRC rank commands. Unit: hisi_sccl,ddrc \000config=0x6\000\00000\000DDRC rank commands\000"
/* offset=181122 */ "rob_empty\000\000other\000This event counts every cycle that no instruction was committed because the CSE is empty\000event=0x18c\000\00000\000This event counts every cycle that no instruction was committed because the CSE is empty\000"
/* offset=181333 */ "rob_empty_stq_busy\000\000other\000This event counts every cycle that no instruction was committed because the CSE is empty and the store port (SP) is full\000event=0x18d\000\00000\000This event counts every cycle that no instruction was committed because the CSE is empty and the store port (SP) is full\000"
/* offset=181617 */ "rw_chg\000hisi_sccl,ddrc\000uncore ddrc\000DDRC read and write changes. Unit: hisi_sccl,ddrc \000config=0x7\000\00000\000DDRC read and write changes\000"
/* offset=181745 */ "rx_ccix\000hisi_sccl,hha\000uncore hha\000Count of the number of operations that HHA has received from CCIX. Unit: hisi_sccl,hha \000config=0x3\000\00000\000Count of the number of operations that HHA has received from CCIX\000"
/* offset=181947 */ "rx_ops_num\000hisi_sccl,hha\000uncore hha\000The number of all operations received by the HHA. Unit: hisi_sccl,hha \000config=0\000\00000\000The number of all operations received by the HHA\000"
/* offset=182116 */ "rx_outer\000hisi_sccl,hha\000uncore hha\000The number of all operations received by the HHA from another socket. Unit: hisi_sccl,hha \000config=0x1\000\00000\000The number of all operations received by the HHA from another socket\000"
/* offset=182325 */ "rx_sccl\000hisi_sccl,hha\000uncore hha\000The number of all operations received by the HHA from another SCCL in this socket. Unit: hisi_sccl,hha \000config=0x2\000\00000\000The number of all operations received by the HHA from another SCCL in this socket\000"
/* offset=182559 */ "rx_snprsp_outer\000hisi_sccl,hha\000uncore hha\000Unit: hisi_sccl,hha \000config=0x3c\000\00000\000\000"
/* offset=182638 */ "rx_snprspdata\000hisi_sccl,hha\000uncore hha\000Unit: hisi_sccl,hha \000config=0x38\000\00000\000\000"
/* offset=182715 */ "rx_wbi\000hisi_sccl,hha\000uncore hha\000Unit: hisi_sccl,hha \000config=0x4\000\00000\000\000"
/* offset=182784 */ "rx_wbip\000hisi_sccl,hha\000uncore hha\000Unit: hisi_sccl,hha \000config=0x5\000\00000\000\000"
/* offset=182854 */ "rx_wtistash\000hisi_sccl,hha\000uncore hha\000Unit: hisi_sccl,hha \000config=0x11\000\00000\000\000"
/* offset=182929 */ "sample_collision\000\000spe\000Sample collided with previous sample\000event=0x4003\000\00000\000Counts statistical profiling samples that have collided with a previous sample and so therefore not taken\000"
/* offset=183111 */ "sample_collision\000\000spe\000Sample collided with previous sample\000event=0x4003\000\00000\000Sample collided with previous sample\000"
/* offset=183224 */ "sample_feed\000\000spe\000Sample Taken\000event=0x4001\000\00000\000Counts statistical profiling samples taken for sampling\000"
/* offset=183327 */ "sample_feed\000\000spe\000Sample Taken\000event=0x4001\000\00000\000Sample Taken\000"
/* offset=183387 */ "sample_filtrate\000\000spe\000Sample Taken and not removed by filtering\000event=0x4002\000\00000\000Counts statistical profiling samples taken which are not removed by filtering\000"
/* offset=183545 */ "sample_filtrate\000\000spe\000Sample Taken and not removed by filtering\000event=0x4002\000\00000\000Sample Taken and not removed by filtering\000"
/* offset=183667 */ "sample_pop\000\000spe\000Sample Population\000event=0x4000\000\00000\000Counts statistical profiling sample population, the count of all operations that could be sampled but may or may not be chosen for sampling\000"
/* offset=183858 */ "sample_pop\000\000spe\000Sample Population\000event=0x4000\000\00000\000Sample Population\000"
/* offset=183927 */ "sdir-hit\000hisi_sccl,hha\000uncore hha\000Unit: hisi_sccl,hha \000config=0x42\000\00000\000\000"
/* offset=183999 */ "sdir-home-migrate\000hisi_sccl,hha\000uncore hha\000Unit: hisi_sccl,hha \000config=0x4c\000\00000\000\000"
/* offset=184080 */ "sdir-lookup\000hisi_sccl,hha\000uncore hha\000Unit: hisi_sccl,hha \000config=0x40\000\00000\000\000"
/* offset=184155 */ "segment_reg_loads.any\000\000other\000Number of segment register loads\000event=0x6,period=200000,umask=0x80\000\00000\000\000"
/* offset=184257 */ "simd_dep_stall\000\000pipeline\000Cycles there is an interlock for an Advanced SIMD/Floating-point operation\000event=0xe6\000\00000\000\000"
/* offset=184373 */ "simd_inst_retired\000\000sve\000SIMD Instruction architecturally executed\000event=0x8000\000\00000\000SIMD Instruction architecturally executed\000"
/* offset=184497 */ "single_movprfx_commit\000\000other\000This event counts every cycle that only the MOVPRFX instruction is committed\000event=0x199\000\00000\000This event counts every cycle that only the MOVPRFX instruction is committed\000"
/* offset=184696 */ "spill_num\000hisi_sccl,hha\000uncore hha\000Count of the number of spill operations that the HHA has sent. Unit: hisi_sccl,hha \000config=0x20\000\00000\000Count of the number of spill operations that the HHA has sent\000"
/* offset=184893 */ "spill_success\000hisi_sccl,hha\000uncore hha\000Count of the number of successful spill operations that the HHA has sent. Unit: hisi_sccl,hha \000config=0x21\000\00000\000Count of the number of successful spill operations that the HHA has sent\000"
/* offset=185116 */ "st_align_lat\000\000memory\000Store with additional latency from alignment\000event=0x4022\000\00000\000Store with additional latency from alignment\000"
/* offset=185244 */ "st_dep_stall\000\000pipeline\000Cycles there is a stall in the Wr stage because of a store\000event=0xe8\000\00000\000\000"
/* offset=185342 */ "st_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, store\000event=0x7\000\00000\000Instruction architecturally executed, condition code check pass, store\000"
/* offset=185522 */ "st_spec\000\000instruction\000Operation speculatively executed, store\000event=0x71\000\00000\000Operation speculatively executed, store\000"
/* offset=185638 */ "st_spec\000\000spec_operation\000Operation speculatively executed, store\000event=0x71\000\00000\000Counts speculatively executed store operations including Single Instruction Multiple Data (SIMD) store operations\000"
/* offset=185831 */ "stall\000\000pipeline\000No operation sent for execution\000event=0x3c\000\00000\000No operation sent for execution.  The counter counts every attributable cycle on which no attributable instruction or operation was sent for execution on this processing element\000"
/* offset=186072 */ "stall_backend\000\000pipeline\000No operation issued due to the backend\000event=0x24\000\00000\000No operation issued due to the backend\000"
/* offset=186189 */ "stall_backend\000\000pipeline\000No operation issued due to the backend\000event=0x24\000\00000\000The counter counts on any cycle fetched instructions are not dispatched due to resource constraints\000"
/* offset=186367 */ "stall_backend\000\000stall\000No operation issued due to the backend\000event=0x24\000\00000\000Counts cycles whenever the rename unit is unable to send any micro-operations to the backend of the pipeline because of backend resource constraints. Backend resource constraints can include issue stage fullness, execution stage fullness, or other internal pipeline resource fullness. All the backend slots were empty during the cycle when this event counts\000"
/* offset=186800 */ "stall_backend_cache\000\000core imp def\000Dispatch stall due to L1 data cache miss\000event=0xd20d\000\00000\000Dispatch stall due to L1 data cache miss\000"
/* offset=186933 */ "stall_backend_ilock\000\000pipeline\000No operation issued due to the backend interlock. This event counts every cycle where the issue of an operation is stalled and there is an interlock. Stall cycles due to a stall in the Wr stage are excluded\000event=0xe4\000\00000\000No operation issued due to the backend interlock. This event counts every cycle where the issue of an operation is stalled and there is an interlock. Stall cycles due to a stall in the Wr stage are excluded\000"
/* offset=187392 */ "stall_backend_ilock\000\000pipeline\000No operation issued due to the backend interlock.This event counts every cycle that issue is stalled and there is an interlock. Stall cycles due to a stall in Wr (typically awaiting load data) are excluded\000event=0xe4\000\00000\000No operation issued due to the backend interlock.This event counts every cycle that issue is stalled and there is an interlock. Stall cycles due to a stall in Wr (typically awaiting load data) are excluded\000"
/* offset=187849 */ "stall_backend_ilock_addr\000\000pipeline\000No operation issued due to the backend, address interlock. This event counts every cycle where the issue of an operation is stalled and there is an interlock on an address operand. This type of interlock is caused by a load/store instruction waiting for data to calculate the address. Stall cycles due to a stall in the Wr stage are excluded\000event=0xe5\000\00000\000No operation issued due to the backend, address interlock. This event counts every cycle where the issue of an operation is stalled and there is an interlock on an address operand. This type of interlock is caused by a load/store instruction waiting for data to calculate the address. Stall cycles due to a stall in the Wr stage are excluded\000"
/* offset=188583 */ "stall_backend_ilock_agu\000\000pipeline\000No operation issued due to the backend, interlock, AGU.This event counts every cycle that issue is stalled and there is an interlock that is due to a load/store instruction waiting for data to calculate the address in the AGU. Stall cycles due to a stall in Wr (typically awaiting load data) are excluded\000event=0xe5\000\00000\000No operation issued due to the backend, interlock, AGU.This event counts every cycle that issue is stalled and there is an interlock that is due to a load/store instruction waiting for data to calculate the address in the AGU. Stall cycles due to a stall in Wr (typically awaiting load data) are excluded\000"
/* offset=189242 */ "stall_backend_ilock_fpu\000\000pipeline\000No operation issued due to the backend, interlock, FPU.This event counts every cycle that issue is stalled and there is an interlock that is due to an FPU/NEON instruction. Stall cycles due to a stall in the Wr stage (typically awaiting load data) are excluded\000event=0xe6\000\00000\000No operation issued due to the backend, interlock, FPU.This event counts every cycle that issue is stalled and there is an interlock that is due to an FPU/NEON instruction. Stall cycles due to a stall in the Wr stage (typically awaiting load data) are excluded\000"
/* offset=189813 */ "stall_backend_ilock_vpu\000\000pipeline\000No operation issued due to the backend, interlock, or the Vector Processing Unit (VPU). This event counts every cycle where there is a stall or an interlock that is caused by a VPU instruction. Stall cycles due to a stall in the Wr stage are excluded\000event=0xe6\000\00000\000No operation issued due to the backend, interlock, or the Vector Processing Unit (VPU). This event counts every cycle where there is a stall or an interlock that is caused by a VPU instruction. Stall cycles due to a stall in the Wr stage are excluded\000"
/* offset=190364 */ "stall_backend_ld\000\000pipeline\000No operation issued due to the backend, load. This event counts every cycle where there is a stall in the Wr stage due to a load\000event=0xe7\000\00000\000No operation issued due to the backend, load. This event counts every cycle where there is a stall in the Wr stage due to a load\000"
/* offset=190664 */ "stall_backend_ld\000\000pipeline\000No operation issued due to the backend, load.This event counts every cycle there is a stall in the Wr stage due to a load\000event=0xe7\000\00000\000No operation issued due to the backend, load.This event counts every cycle there is a stall in the Wr stage due to a load\000"
/* offset=190950 */ "stall_backend_ld_cache\000\000pipeline\000No operation issued due to the backend, load, cache miss. This event counts every cycle where there is a stall in the Wr stage due to a load that is waiting on data. The event counts for stalls that are caused by missing the cache or where the data is Non-cacheable\000event=0xe9\000\00000\000No operation issued due to the backend, load, cache miss. This event counts every cycle where there is a stall in the Wr stage due to a load that is waiting on data. The event counts for stalls that are caused by missing the cache or where the data is Non-cacheable\000"
/* offset=191530 */ "stall_backend_ld_cache\000\000pipeline\000No operation issued due to the backend, load, cache miss.This event counts every cycle there is a stall in the Wr stage due to a load which is waiting on data (due to missing the cache or being non-cacheable)\000event=0xe9\000\00000\000No operation issued due to the backend, load, cache miss.This event counts every cycle there is a stall in the Wr stage due to a load which is waiting on data (due to missing the cache or being non-cacheable)\000"
/* offset=191996 */ "stall_backend_ld_tlb\000\000pipeline\000No operation issued due to the backend, load, TLB miss. This event counts every cycle where there is a stall in the Wr stage due to a load that misses in the L1 TLB\000event=0xea\000\00000\000No operation issued due to the backend, load, TLB miss. This event counts every cycle where there is a stall in the Wr stage due to a load that misses in the L1 TLB\000"
/* offset=192372 */ "stall_backend_ld_tlb\000\000pipeline\000No operation issued due to the backend, load, TLB miss.This event counts every cycle there is a stall in the Wr stage due to a load which has missed in the L1 TLB\000event=0xea\000\00000\000No operation issued due to the backend, load, TLB miss.This event counts every cycle there is a stall in the Wr stage due to a load which has missed in the L1 TLB\000"
/* offset=192744 */ "stall_backend_mem\000\000pipeline\000Memory stall cycles\000event=0x4005\000\00000\000Memory stall cycles.  The counter counts each cycle counted by STALL_BACKEND where there is a cache miss in the last level of cache within the processing element clock domain\000"
/* offset=192984 */ "stall_backend_resource\000\000core imp def\000Dispatch stall due to lack of any core resource\000event=0xd20f\000\00000\000Dispatch stall due to lack of any core resource\000"
/* offset=193134 */ "stall_backend_st\000\000pipeline\000No operation issued due to the backend, store. This event counts every cycle where there is a stall in the Wr stage due to a store\000event=0xe8\000\00000\000No operation issued due to the backend, store. This event counts every cycle where there is a stall in the Wr stage due to a store\000"
/* offset=193438 */ "stall_backend_st\000\000pipeline\000No operation issued due to the backend, store.This event counts every cycle there is a stall in the Wr stage due to a store\000event=0xe8\000\00000\000No operation issued due to the backend, store.This event counts every cycle there is a stall in the Wr stage due to a store\000"
/* offset=193728 */ "stall_backend_st_stb\000\000pipeline\000No operation issued due to the backend, store, STB full.This event counts every cycle there is a stall in the Wr stage due to a store which is waiting due to the STB being full\000event=0xeb\000\00000\000No operation issued due to the backend, store, STB full.This event counts every cycle there is a stall in the Wr stage due to a store which is waiting due to the STB being full\000"
/* offset=194128 */ "stall_backend_st_stb\000\000pipeline\000No operation issued due to the backend, store, Store Buffer (STB) full. This event counts every cycle where there is a stall in the Wr stage because of a store operation that is waiting due to the STB being full\000event=0xeb\000\00000\000No operation issued due to the backend, store, Store Buffer (STB) full. This event counts every cycle where there is a stall in the Wr stage because of a store operation that is waiting due to the STB being full\000"
/* offset=194598 */ "stall_backend_st_tlb\000\000pipeline\000No operation issued due to the backend, store, TLB miss. This event counts every cycle where there is a stall in the Wr stage because of a store operation that has missed in the L1 TLB\000event=0xec\000\00000\000No operation issued due to the backend, store, TLB miss. This event counts every cycle where there is a stall in the Wr stage because of a store operation that has missed in the L1 TLB\000"
/* offset=195014 */ "stall_backend_st_tlb\000\000pipeline\000No operation issued due to the backend, store, TLB miss.This event counts every cycle there is a stall in the Wr stage due to a store which has missed in the L1 TLB\000event=0xec\000\00000\000No operation issued due to the backend, store, TLB miss.This event counts every cycle there is a stall in the Wr stage due to a store which has missed in the L1 TLB\000"
/* offset=195390 */ "stall_backend_tlb\000\000core imp def\000Dispatch stall due to L1 data TLB miss\000event=0xd20e\000\00000\000Dispatch stall due to L1 data TLB miss\000"
/* offset=195517 */ "stall_backend_vpu_hazard\000\000pipeline\000No operation issued due to the backend, VPU hazard. This event counts every cycle where the core stalls due to contention for the VPU with the other core\000event=0xed\000\00000\000No operation issued due to the backend, VPU hazard. This event counts every cycle where the core stalls due to contention for the VPU with the other core\000"
/* offset=195875 */ "stall_frontend\000\000pipeline\000No operation issued because of the frontend\000event=0x23\000\00000\000No operation issued because of the frontend\000"
/* offset=196003 */ "stall_frontend\000\000pipeline\000No operation issued because of the frontend\000event=0x23\000\00000\000The counter counts on any cycle when there are no fetched instructions available to dispatch\000"
/* offset=196180 */ "stall_frontend\000\000stall\000No operation issued because of the frontend\000event=0x23\000\00000\000Counts cycles when frontend could not send any micro-operations to the rename stage because of frontend resource stalls caused by fetch memory latency or branch prediction flow stalls. All the frontend slots were empty during the cycle when this event counts\000"
/* offset=196520 */ "stall_frontend_cache\000\000core imp def\000Dispatch stall due to L1 instruction cache miss\000event=0xd20b\000\00000\000Dispatch stall due to L1 instruction cache miss\000"
/* offset=196668 */ "stall_frontend_cache\000\000pipeline\000No operation issued due to the frontend, cache miss. This event counts every cycle that the Data Processing Unit (DPU) instruction queue is empty and there is an instruction cache miss being processed\000event=0xe1\000\00000\000No operation issued due to the frontend, cache miss. This event counts every cycle that the Data Processing Unit (DPU) instruction queue is empty and there is an instruction cache miss being processed\000"
/* offset=197116 */ "stall_frontend_cache\000\000pipeline\000No operation issued due to the frontend, cache miss.This event counts every cycle the DPU IQ is empty and there is an instruction cache miss being processed\000event=0xe1\000\00000\000No operation issued due to the frontend, cache miss.This event counts every cycle the DPU IQ is empty and there is an instruction cache miss being processed\000"
/* offset=197476 */ "stall_frontend_pderr\000\000pipeline\000No operation issued due to the frontend, pre-decode error.This event counts every cycle the DPU IQ is empty and there is a pre-decode error being processed\000event=0xe3\000\00000\000No operation issued due to the frontend, pre-decode error.This event counts every cycle the DPU IQ is empty and there is a pre-decode error being processed\000"
/* offset=197834 */ "stall_frontend_pderr\000\000pipeline\000No operation issued due to the frontend, pre-decode error\000event=0xe3\000\00000\000No operation issued due to the frontend, pre-decode error\000"
/* offset=197996 */ "stall_frontend_tlb\000\000core imp def\000Dispatch stall due to L1 instruction TLB miss\000event=0xd20c\000\00000\000Dispatch stall due to L1 instruction TLB miss\000"
/* offset=198138 */ "stall_frontend_tlb\000\000pipeline\000No operation issued due to the frontend, TLB miss. This event counts every cycle that the DPU instruction queue is empty and there is an instruction L1 TLB miss being processed\000event=0xe2\000\00000\000No operation issued due to the frontend, TLB miss. This event counts every cycle that the DPU instruction queue is empty and there is an instruction L1 TLB miss being processed\000"
/* offset=198536 */ "stall_frontend_tlb\000\000pipeline\000No operation issued due to the frontend, TLB miss.This event counts every cycle the DPU IQ is empty and there is an instruction L1 TLB miss being processed\000event=0xe2\000\00000\000No operation issued due to the frontend, TLB miss.This event counts every cycle the DPU IQ is empty and there is an instruction L1 TLB miss being processed\000"
/* offset=198892 */ "stall_sb_full\000\000pipeline\000Data Write operation that stalls the pipeline because the store buffer is full\000event=0xc7\000\00000\000\000"
/* offset=199011 */ "stall_slot\000\000pipeline\000No operation sent for execution on a slot\000event=0x3f\000\00000\000No operation sent for execution on a slot.  The counter counts on each attributable cycle the number of instruction or operation slots that were not occupied by an instruction or operation attributable to the processing element\000"
/* offset=199317 */ "stall_slot_backend\000\000pipeline\000No operation sent for execution on a slot due to the backend\000event=0x3d\000\00000\000No operation sent for execution on a slot due to the backend.  Counts each slot counted by STALL_SLOT where no attributable instruction or operation was sent for execution because the backend is unable to accept it\000"
/* offset=199637 */ "stall_slot_backend_ilock\000\000pipeline\000Issue slot not issued due to interlock. For each cycle, this event counts each dispatch slot that does not issue due to an interlock\000event=0xee\000\00000\000Issue slot not issued due to interlock. For each cycle, this event counts each dispatch slot that does not issue due to an interlock\000"
/* offset=199953 */ "stall_slot_frontend\000\000pipeline\000No operation sent for execution on a slot due to the frontend\000event=0x3e\000\00000\000No operation sent for execution on a slot due to the frontend.  Counts each slot counted by STALL_SLOT where no attributable instruction or operation was sent for execution because there was no attributable instruction or operation available to issue from the processing element from the frontend for the slot\000"
/* offset=200370 */ "stb_stall\000\000cache\000Merge in the store buffer\000event=0xc0\000\00000\000Merge in the store buffer\000"
/* offset=200454 */ "strex_fail_spec\000\000instruction\000Exclusive operation speculatively executed, STREX or STX fail\000event=0x6e\000\00000\000Exclusive operation speculatively executed, STREX or STX fail\000"
/* offset=200622 */ "strex_fail_spec\000\000intrinsic\000Exclusive operation speculatively executed, STREX or STX fail\000event=0x6e\000\00000\000Exclusive operation speculatively executed, STREX or STX fail\000"
/* offset=200788 */ "strex_fail_spec\000\000spec_operation\000Exclusive operation speculatively executed, STREX or STX fail\000event=0x6e\000\00000\000Counts store-exclusive operations that have been speculatively executed and have not successfully completed the store operation\000"
/* offset=201025 */ "strex_pass_spec\000\000instruction\000Exclusive operation speculatively executed, STREX or STX pass\000event=0x6d\000\00000\000Exclusive operation speculatively executed, STREX or STX pass\000"
/* offset=201193 */ "strex_pass_spec\000\000intrinsic\000Exclusive operation speculatively executed, STREX or STX pass\000event=0x6d\000\00000\000Exclusive operation speculatively executed, STREX or STX pass\000"
/* offset=201359 */ "strex_pass_spec\000\000spec_operation\000Exclusive operation speculatively executed, STREX or STX pass\000event=0x6d\000\00000\000Counts store-exclusive operations that have been speculatively executed and have successfully completed the store operation\000"
/* offset=201592 */ "strex_spec\000\000instruction\000Exclusive operation speculatively executed, STREX or STX\000event=0x6f\000\00000\000Exclusive operation speculatively executed, STREX or STX\000"
/* offset=201745 */ "strex_spec\000\000intrinsic\000Exclusive operation speculatively executed, STREX or STX\000event=0x6f\000\00000\000Exclusive operation speculatively executed, STREX or STX\000"
/* offset=201896 */ "strex_spec\000\000spec_operation\000Exclusive operation speculatively executed, STREX or STX\000event=0x6f\000\00000\000Counts store-exclusive operations that have been speculatively executed\000"
/* offset=202067 */ "sve_inst_retired\000\000instruction\000Instruction architecturally executed, SVE\000event=0x8002\000\00000\000Instruction architecturally executed, SVE\000"
/* offset=202198 */ "sve_inst_retired\000\000sve\000Instruction architecturally executed, SVE\000event=0x8002\000\00000\000Instruction architecturally executed, SVE\000"
/* offset=202321 */ "sve_inst_spec\000\000instruction\000SVE operations speculatively executed\000event=0x8006\000\00000\000SVE operations speculatively executed\000"
/* offset=202441 */ "sve_ld_gather_spec\000\000sve\000SVE gather-load Operations speculatively executed\000event=0x80ad\000\00000\000SVE gather-load Operations speculatively executed\000"
/* offset=202582 */ "sve_ldff_fault_spec\000\000instruction\000SVE first-fault load operations speculatively executed which set FFR bit to 0\000event=0x80bd\000\00000\000SVE first-fault load operations speculatively executed which set FFR bit to 0\000"
/* offset=202788 */ "sve_ldff_spec\000\000instruction\000SVE First-fault load Operations speculatively executed\000event=0x80bc\000\00000\000SVE First-fault load Operations speculatively executed\000"
/* offset=202942 */ "sve_ldff_spec\000\000sve\000SVE First-fault load Operations speculatively executed\000event=0x80bc\000\00000\000SVE First-fault load Operations speculatively executed\000"
/* offset=203088 */ "sve_ldr_preg_spec\000\000sve\000SVE load predicate register Operations speculatively executed\000event=0x8095\000\00000\000SVE load predicate register Operations speculatively executed\000"
/* offset=203252 */ "sve_ldr_reg_spec\000\000sve\000SVE unpredicated load register Operations speculatively executed\000event=0x8091\000\00000\000SVE unpredicated load register Operations speculatively executed\000"
/* offset=203421 */ "sve_math_spec\000\000sve\000SVE Math accelerator Operations speculatively executed\000event=0x800e\000\00000\000SVE Math accelerator Operations speculatively executed\000"
/* offset=203567 */ "sve_movprfx_spec\000\000sve\000SVE MOVPRFX Operations speculatively executed\000event=0x807c\000\00000\000SVE MOVPRFX Operations speculatively executed\000"
/* offset=203698 */ "sve_movprfx_u_spec\000\000sve\000SVE MOVPRFX unfused Operations speculatively executed\000event=0x807f\000\00000\000SVE MOVPRFX unfused Operations speculatively executed\000"
/* offset=203847 */ "sve_pred_empty_spec\000\000instruction\000SVE predicated operations with no active predicates speculatively executed\000event=0x8075\000\00000\000SVE predicated operations with no active predicates speculatively executed\000"
/* offset=204047 */ "sve_pred_full_spec\000\000instruction\000SVE predicated operations speculatively executed with all active predicates\000event=0x8076\000\00000\000SVE predicated operations speculatively executed with all active predicates\000"
/* offset=204248 */ "sve_pred_not_full_spec\000\000instruction\000SVE predicated operations with empty or partially active predicates\000event=0x8079\000\00000\000SVE predicated operations with empty or partially active predicates\000"
/* offset=204437 */ "sve_pred_partial_spec\000\000instruction\000SVE predicated operations speculatively executed with partially active predicates\000event=0x8077\000\00000\000SVE predicated operations speculatively executed with partially active predicates\000"
/* offset=204653 */ "sve_pred_spec\000\000instruction\000SVE predicated Operations speculatively executed\000event=0x8074\000\00000\000SVE predicated Operations speculatively executed\000"
/* offset=204795 */ "sve_pred_spec\000\000sve\000SVE predicated Operations speculatively executed\000event=0x8074\000\00000\000SVE predicated Operations speculatively executed\000"
/* offset=204929 */ "sve_prf_contig_spec\000\000sve\000SVE contiguous prefetch element Operations speculatively executed\000event=0x809f\000\00000\000SVE contiguous prefetch element Operations speculatively executed\000"
/* offset=205103 */ "sve_prf_gather_spec\000\000sve\000SVE gather-prefetch Operations speculatively executed\000event=0x80af\000\00000\000SVE gather-prefetch Operations speculatively executed\000"
/* offset=205253 */ "sve_st_scatter_spec\000\000sve\000SVE scatter-store Operations speculatively executed\000event=0x80ae\000\00000\000SVE scatter-store Operations speculatively executed\000"
/* offset=205399 */ "sve_str_preg_spec\000\000sve\000SVE store predicate register Operations speculatively executed\000event=0x8096\000\00000\000SVE store predicate register Operations speculatively executed\000"
/* offset=205565 */ "sve_str_reg_spec\000\000sve\000SVE unpredicated store register Operations speculatively executed\000event=0x8092\000\00000\000SVE unpredicated store register Operations speculatively executed\000"
/* offset=205736 */ "sw_incr\000\000instruction\000Instruction architecturally executed, Condition code check pass, software increment\000event=0\000\00000\000Instruction architecturally executed, Condition code check pass, software increment\000"
/* offset=205937 */ "sw_incr\000\000instruction\000Software increment\000event=0\000\00000\000Instruction architecturally executed, software increment\000"
/* offset=206046 */ "sw_incr\000\000retired\000Instruction architecturally executed, Condition code check pass, software increment\000event=0\000\00000\000Counts software writes to the PMSWINC_EL0 (software PMU increment) register. The PMSWINC_EL0 register is a manually updated counter for use by application software.\n\nThis event could be used to measure any user program event, such as accesses to a particular data structure (by writing to the PMSWINC_EL0 register each time the data structure is accessed).\n\nTo use the PMSWINC_EL0 register and event, developers must insert instructions that write to the PMSWINC_EL0 register into the source code.\n\nSince the SW_INCR event records writes to the PMSWINC_EL0 register, there is no need to do a read/increment/write sequence to the PMSWINC_EL0 register\000"
/* offset=206809 */ "sx_stall\000\000pipeline\000SX stalled\000event=0x10e\000\00000\000SX Op non-issue\000"
/* offset=206871 */ "sys_ccn_pmu.read_cycles\000uncore_sys_ccn_pmu\000uncore\000ccn read-cycles event. Unit: uncore_sys_ccn_pmu \000config=0x2c\0000x01\00000\000\000"
/* offset=206991 */ "sys_ddr_pmu.write_cycles\000uncore_sys_ddr_pmu\000uncore\000ddr write-cycles event. Unit: uncore_sys_ddr_pmu \000event=0x2b\000v8\00000\000\000"
/* offset=207110 */ "tlb_access\000\000cache\000Level 2 TLB access\000event=0xea\000\00000\000Level 2 TLB access\000"
/* offset=207181 */ "tlb_err\000\000other\000TLB memory error\000event=0xd2\000\00000\000\000"
/* offset=207229 */ "tlb_l2tlb_l2walk_access\000\000cache\000Level 2 TLB level-2 walk cache access. This event counts accesses to the level-2 walk cache where the last-level walk cache has missed. The event only counts when the translation regime of the pagewalk uses level 2 descriptors. This event does not count if the MMU is disabled\000event=0xcc\000\00000\000Level 2 TLB level-2 walk cache access. This event counts accesses to the level-2 walk cache where the last-level walk cache has missed. The event only counts when the translation regime of the pagewalk uses level 2 descriptors. This event does not count if the MMU is disabled\000"
/* offset=207829 */ "tlb_l2tlb_l2walk_refill\000\000cache\000Level 2 TLB level-2 walk cache refill. This event does not count if the MMU is disabled\000event=0xcd\000\00000\000Level 2 TLB level-2 walk cache refill. This event does not count if the MMU is disabled\000"
/* offset=208051 */ "tlb_l2tlb_llwalk_access\000\000cache\000Level 2 TLB last-level walk cache access. This event does not count if the MMU is disabled\000event=0xca\000\00000\000Level 2 TLB last-level walk cache access. This event does not count if the MMU is disabled\000"
/* offset=208279 */ "tlb_l2tlb_llwalk_refill\000\000cache\000Level 2 TLB last-level walk cache refill. This event does not count if the MMU is disabled\000event=0xcb\000\00000\000Level 2 TLB last-level walk cache refill. This event does not count if the MMU is disabled\000"
/* offset=208507 */ "tlb_l2tlb_s2_access\000\000cache\000Level 2 TLB IPA cache access. This event counts on each access to the IPA cache. +//0 If a single pagewalk needs to make multiple accesses to the IPA cache, each access is counted. +//0 If stage 2 translation is disabled, this event does not count\000event=0xce\000\00000\000Level 2 TLB IPA cache access. This event counts on each access to the IPA cache. +//0 If a single pagewalk needs to make multiple accesses to the IPA cache, each access is counted. +//0 If stage 2 translation is disabled, this event does not count\000"
/* offset=209045 */ "tlb_l2tlb_s2_refill\000\000cache\000Level 2 TLB IPA cache refill. This event counts on each refill of the IPA cache. +//0 If a single pagewalk needs to make multiple accesses to the IPA cache, each access which causes a refill is counted. +//0 If stage 2 translation is disabled, this event does not count\000event=0xcf\000\00000\000Level 2 TLB IPA cache refill. This event counts on each refill of the IPA cache. +//0 If a single pagewalk needs to make multiple accesses to the IPA cache, each access which causes a refill is counted. +//0 If stage 2 translation is disabled, this event does not count\000"
/* offset=209627 */ "tlb_miss\000\000cache\000Level 2 TLB miss\000event=0xeb\000\00000\000Level 2 TLB miss\000"
/* offset=209692 */ "trb_trig\000\000trace\000Trace buffer Trigger Event\000event=0x400e\000\00000\000Trace buffer Trigger Event\000"
/* offset=209779 */ "trb_wrap\000\000trace\000Trace buffer current write pointer wrapped\000event=0x400c\000\00000\000Trace buffer current write pointer wrapped\000"
/* offset=209898 */ "trcextout0\000\000trace\000PE Trace Unit external output 0\000event=0x4010\000\00000\000PE Trace Unit external output 0\000"
/* offset=209997 */ "trcextout1\000\000trace\000PE Trace Unit external output 1\000event=0x4011\000\00000\000PE Trace Unit external output 1\000"
/* offset=210096 */ "trcextout2\000\000trace\000PE Trace Unit external output 2\000event=0x4012\000\00000\000PE Trace Unit external output 2\000"
/* offset=210195 */ "trcextout3\000\000trace\000PE Trace Unit external output 3\000event=0x4013\000\00000\000PE Trace Unit external output 3\000"
/* offset=210294 */ "ttbr_write_retired\000\000instruction\000Instruction architecturally executed, Condition code check pass, write to TTBR\000event=0x1c\000\00000\000Instruction architecturally executed, Condition code check pass, write to TTBR\000"
/* offset=210499 */ "ttbr_write_retired\000\000instruction\000Instruction architecturally executed, Condition code check pass, write to TTBR\000event=0x1c\000\00000\000This event only counts writes to TTBR0/TTBR1 in AArch32 state and TTBR0_EL1/TTBR1_EL1 in AArch64 state\000"
/* offset=210728 */ "ttbr_write_retired\000\000retired\000Instruction architecturally executed, Condition code check pass, write to TTBR\000event=0x1c\000\00000\000Counts architectural writes to TTBR0/1_EL1. If virtualization host extensions are enabled (by setting the HCR_EL2.E2H bit to 1), then accesses to TTBR0/1_EL1 that are redirected to TTBR0/1_EL2, or accesses to TTBR0/1_EL12, are counted. TTBRn registers are typically updated when the kernel is swapping user-space threads or applications\000"
/* offset=211187 */ "tx_snp_ccix\000hisi_sccl,hha\000uncore hha\000Unit: hisi_sccl,hha \000config=0x35\000\00000\000\000"
/* offset=211262 */ "tx_snp_num\000hisi_sccl,hha\000uncore hha\000Unit: hisi_sccl,hha \000config=0x33\000\00000\000\000"
/* offset=211336 */ "tx_snp_outer\000hisi_sccl,hha\000uncore hha\000Unit: hisi_sccl,hha \000config=0x34\000\00000\000\000"
/* offset=211412 */ "unaligned_ld_spec\000\000core imp def\000Unaligned access, read\000event=0x68\000\00000\000Unaligned access, read\000"
/* offset=211505 */ "unaligned_ld_spec\000\000memory\000Unaligned access, read\000event=0x68\000\00000\000Unaligned access, read\000"
/* offset=211592 */ "unaligned_ld_spec\000\000spec_operation\000Unaligned access, read\000event=0x68\000\00000\000Counts unaligned memory read operations issued by the CPU. This event counts unaligned accesses (as defined by the actual instruction), even if they are subsequently issued as multiple aligned accesses. The event does not count preload operations (PLD, PLI)\000"
/* offset=211922 */ "unaligned_ldst_retired\000\000memory\000Instruction architecturally executed, condition code check pass, unaligned\000event=0xf\000\00000\000Instruction architecturally executed, condition code check pass, unaligned\000"
/* offset=212117 */ "unaligned_ldst_spec\000\000core imp def\000Unaligned access\000event=0x6a\000\00000\000Unaligned access\000"
/* offset=212200 */ "unaligned_ldst_spec\000\000memory\000Unaligned access\000event=0x6a\000\00000\000Unaligned access\000"
/* offset=212277 */ "unaligned_ldst_spec\000\000spec_operation\000Unaligned access\000event=0x6a\000\00000\000Counts unaligned memory operations issued by the CPU. This event counts unaligned accesses (as defined by the actual instruction), even if they are subsequently issued as multiple aligned accesses\000"
/* offset=212542 */ "unaligned_st_spec\000\000core imp def\000Unaligned access, write\000event=0x69\000\00000\000Unaligned access, write\000"
/* offset=212637 */ "unaligned_st_spec\000\000memory\000Unaligned access, write\000event=0x69\000\00000\000Unaligned access, write\000"
/* offset=212726 */ "unaligned_st_spec\000\000spec_operation\000Unaligned access, write\000event=0x69\000\00000\000Counts unaligned memory write operations issued by the CPU. This event counts unaligned accesses (as defined by the actual instruction), even if they are subsequently issued as multiple aligned accesses\000"
/* offset=213002 */ "unc_cbo_xsnp_response.miss_eviction\000uncore_cbox\000uncore\000A cross-core snoop resulted from L3 Eviction which misses in some processor core. Unit: uncore_cbox \000event=0x22,umask=0x81\000\00000\000A cross-core snoop resulted from L3 Eviction which misses in some processor core\000"
/* offset=213265 */ "uncore_hisi_ddrc.flux_wcmd\000hisi_sccl,ddrc\000uncore\000DDRC write commands. Unit: hisi_sccl,ddrc \000event=0x2\000\00000\000DDRC write commands\000"
/* offset=213391 */ "uncore_hisi_l3c.rd_hit_cpipe\000hisi_sccl,l3c\000uncore\000Total read hits. Unit: hisi_sccl,l3c \000event=0x7\000\00000\000Total read hits\000"
/* offset=213509 */ "uncore_imc.cache_hits\000uncore_imc\000uncore\000Total cache hits. Unit: uncore_imc \000event=0x34\000\00000\000Total cache hits\000"
/* offset=213617 */ "uncore_imc_free_running.cache_miss\000uncore_imc_free_running\000uncore\000Total cache misses. Unit: uncore_imc_free_running \000event=0x12\000\00000\000Total cache misses\000"
/* offset=213768 */ "uop_only_commit\000\000other\000This event counts every cycle that only any micro-operations are committed\000event=0x198\000\00000\000This event counts every cycle that only any micro-operations are committed\000"
/* offset=213957 */ "uop_spec\000\000sve\000Microarchitectural operation, Operations speculatively executed\000event=0x8008\000\00000\000Microarchitectural operation, Operations speculatively executed\000"
/* offset=214116 */ "uop_split\000\000other\000This event counts the occurrence count of the micro-operation split\000event=0x139\000\00000\000This event counts the occurrence count of the micro-operation split\000"
/* offset=214285 */ "utlb_flush\000\000cache\000Level 1 TLB flush\000event=0xe9\000\00000\000Level 1 TLB flush\000"
/* offset=214354 */ "vfp_spec\000\000instruction\000Operation speculatively executed, floating-point instruction\000event=0x75\000\00000\000Operation speculatively executed, floating-point instruction\000"
/* offset=214513 */ "vfp_spec\000\000spec_operation\000Operation speculatively executed, floating-point instruction\000event=0x75\000\00000\000Counts speculatively executed floating point operations. This event does not count operations that move data to or from floating point (vector) registers\000"
/* offset=214768 */ "victim_num\000hisi_sccl,l3c\000uncore l3c\000l3c precharge commands. Unit: hisi_sccl,l3c \000config=0x4\000\00000\000l3c precharge commands\000"
/* offset=214887 */ "wait_cycles\000\000clock\000Wait state cycle\000event=0x110\000\00000\000Wait state cycle\000"
/* offset=214956 */ "watchpoint_match\000\000core imp def\000LSU detected a Watchpoint data match\000event=0xd60d\000\00000\000LSU detected a Watchpoint data match\000"
/* offset=215078 */ "wfe_wfi_cycle\000\000other\000This event counts every cycle that the instruction unit is halted by the WFE/WFI instruction\000event=0x18e\000\00000\000This event counts every cycle that the instruction unit is halted by the WFE/WFI instruction\000"
/* offset=215301 */ "wr_cpipe\000hisi_sccl,l3c\000uncore l3c\000Total write accesses. Unit: hisi_sccl,l3c \000config=0x1\000\00000\000Total write accesses\000"
/* offset=215414 */ "wr_ddr_128b\000hisi_sccl,hha\000uncore hha\000The number of write operations sent by HHA to DDRC which size is 128 bytes. Unit: hisi_sccl,hha \000config=0x1f\000\00000\000The number of write operations sent by HHA to DDRC which size is 128 bytes\000"
/* offset=215639 */ "wr_ddr_64b\000hisi_sccl,hha\000uncore hha\000The number of write operations sent by HHA to DDRC which size is 64 bytes. Unit: hisi_sccl,hha \000config=0x1d\000\00000\000The number of write operations sent by HHA to DDRC which size is 64 bytes\000"
/* offset=215861 */ "wr_hit_cpipe\000hisi_sccl,l3c\000uncore l3c\000Total write hits. Unit: hisi_sccl,l3c \000config=0x3\000\00000\000Total write hits\000"
/* offset=215970 */ "wr_hit_spipe\000hisi_sccl,l3c\000uncore l3c\000Count of the number of write lines that hits in spipe of this L3C. Unit: hisi_sccl,l3c \000config=0x23\000\00000\000Count of the number of write lines that hits in spipe of this L3C\000"
/* offset=216178 */ "wr_spipe\000hisi_sccl,l3c\000uncore l3c\000Count of the number of write lines that come from this cluster of CPU core in spipe. Unit: hisi_sccl,l3c \000config=0x21\000\00000\000Count of the number of write lines that come from this cluster of CPU core in spipe\000"
;

static const struct compact_pmu_event pmu_events__ampere_ampereone[] = {
{ 30753 }, /* br_immed_spec\000\000branch\000Branch speculatively executed, immediate branch\000event=0x78\000\00000\000Branch speculatively executed, immediate branch\000 */
{ 35038 }, /* br_indirect_spec\000\000branch\000Branch speculatively executed, indirect branch\000event=0x7a\000\00000\000Branch speculatively executed, indirect branch\000 */
{ 35743 }, /* br_mis_pred\000\000branch\000Mispredicted or not predicted branch speculatively executed\000event=0x10\000\00000\000Mispredicted or not predicted branch speculatively executed\000 */
{ 36925 }, /* br_pred\000\000branch\000Predictable branch speculatively executed\000event=0x12\000\00000\000Predictable branch speculatively executed\000 */
{ 40692 }, /* br_return_spec\000\000branch\000Branch speculatively executed, procedure return\000event=0x79\000\00000\000Branch speculatively executed, procedure return\000 */
{ 41358 }, /* bus_access\000\000bus\000Attributable Bus access\000event=0x19\000\00000\000Attributable Bus access\000 */
{ 41979 }, /* bus_access_normal\000\000bus\000Bus access, Normal\000event=0x64\000\00000\000Bus access, Normal\000 */
{ 42055 }, /* bus_access_not_shared\000\000bus\000Bus access, not Normal, Cacheable, Shareable\000event=0x63\000\00000\000Bus access, not Normal, Cacheable, Shareable\000 */
{ 42187 }, /* bus_access_periph\000\000bus\000Bus access, peripheral\000event=0x65\000\00000\000Bus access, peripheral\000 */
{ 42271 }, /* bus_access_rd\000\000bus\000Bus access read\000event=0x60\000\00000\000Bus access read\000 */
{ 42562 }, /* bus_access_shared\000\000bus\000Bus access, Normal, Cacheable, Shareable\000event=0x62\000\00000\000Bus access, Normal, Cacheable, Shareable\000 */
{ 42682 }, /* bus_access_wr\000\000bus\000Bus access write\000event=0x61\000\00000\000Bus access write\000 */
{ 42979 }, /* bus_cycles\000\000bus\000Bus cycle\000event=0x1d\000\00000\000Bus cycle\000 */
{ 46424 }, /* cnt_cycles\000\000bus\000Constant frequency cycles\000event=0x4004\000\00000\000Constant frequency cycles.  The counter increments at a constant frequency equal to the rate of increment of the system counter, CNTPCT_EL0\000 */
{ 47992 }, /* cpu_cycles\000\000bus\000Cycle\000event=0x11\000\00000\000Cycle\000 */
{ 53526 }, /* dtlb_walk\000\000cache\000Access to data TLB causes a translation table walk\000event=0x34\000\00000\000Access to data TLB causes a translation table walk\000 */
{ 92044 }, /* itlb_walk\000\000cache\000Access to instruction TLB that causes a translation table walk\000event=0x35\000\00000\000Access to instruction TLB that causes a translation table walk\000 */
{ 100289 }, /* l1d_cache\000\000cache\000Level 1 data cache access\000event=0x4\000\00000\000Level 1 data cache access\000 */
{ 101403 }, /* l1d_cache_inval\000\000cache\000L1D cache invalidate\000event=0x48\000\00000\000Attributable Level 1 data cache invalidate\000 */
{ 102276 }, /* l1d_cache_lmiss_rd\000\000cache\000Level 1 data cache long-latency read miss\000event=0x39\000\00000\000Level 1 data cache long-latency read miss.  The counter counts each memory read access counted by L1D_CACHE that incurs additional latency because it returns data from outside the Level 1 data or unified cache of this processing element\000 */
{ 102693 }, /* l1d_cache_rd\000\000cache\000L1D cache access, read\000event=0x40\000\00000\000Attributable Level 1 data cache access, read\000 */
{ 103129 }, /* l1d_cache_refill\000\000cache\000Level 1 data cache refill\000event=0x3\000\00000\000Level 1 data cache refill\000 */
{ 105894 }, /* l1d_cache_refill_rd\000\000cache\000L1D cache refill, read\000event=0x42\000\00000\000Attributable Level 1 data cache refill, read\000 */
{ 108997 }, /* l1d_cache_wr\000\000cache\000L1D cache access, write\000event=0x41\000\00000\000Attributable Level 1 data cache access, write\000 */
{ 109867 }, /* l1d_tlb\000\000cache\000Attributable Level 1 data or unified TLB access\000event=0x25\000\00000\000Attributable Level 1 data or unified TLB access\000 */
{ 111207 }, /* l1d_tlb_refill\000\000cache\000Attributable Level 1 data TLB refill\000event=0x5\000\00000\000Attributable Level 1 data TLB refill\000 */
{ 112199 }, /* l1d_tlb_refill_rd\000\000cache\000L1D tlb refill, read\000event=0x4c\000\00000\000Attributable Level 1 data TLB refill, read\000 */
{ 113093 }, /* l1d_tlb_refill_wr\000\000cache\000L1D tlb refill, write\000event=0x4d\000\00000\000Attributable Level 1 data TLB refill, write\000 */
{ 116626 }, /* l1i_cache\000\000cache\000Attributable Level 1 instruction cache access\000event=0x14\000\00000\000Attributable Level 1 instruction cache access\000 */
{ 117316 }, /* l1i_cache_lmiss\000\000cache\000Level 1 instruction cache long-latency read miss\000event=0x4006\000\00000\000Level 1 instruction cache long-latency read miss.  If the L1I_CACHE_RD event is implemented, the counter counts each access counted by L1I_CACHE_RD that incurs additional latency because it returns instructions from outside of the Level 1 instruction cache of this PE.  If the L1I_CACHE_RD event is not implemented, the counter counts each access counted by L1I_CACHE that incurs additional latency because it returns instructions from outside the Level 1 instruction cache of this PE.  The event indicates to software that the access missed in the Level 1 instruction cache and might have a significant performance impact due to the additional latency, compared to the latency of an access that hits in the Level 1 instruction cache\000 */
{ 118420 }, /* l1i_cache_refill\000\000cache\000Level 1 instruction cache refill\000event=0x1\000\00000\000Level 1 instruction cache refill\000 */
{ 119419 }, /* l1i_tlb\000\000cache\000Attributable Level 1 instruction TLB access\000event=0x26\000\00000\000Attributable Level 1 instruction TLB access\000 */
{ 120198 }, /* l1i_tlb_refill\000\000cache\000Attributable Level 1 instruction TLB refill\000event=0x2\000\00000\000Attributable Level 1 instruction TLB refill\000 */
{ 126244 }, /* l2d_cache\000\000cache\000Level 2 data cache access\000event=0x16\000\00000\000Level 2 data cache access\000 */
{ 127466 }, /* l2d_cache_inval\000\000cache\000L2D cache invalidate\000event=0x58\000\00000\000Attributable Level 2 data cache invalidate\000 */
{ 128191 }, /* l2d_cache_lmiss_rd\000\000cache\000Level 2 data cache long-latency read miss\000event=0x4009\000\00000\000Level 2 data cache long-latency read miss.  The counter counts each memory read access counted by L2D_CACHE that incurs additional latency because it returns data from outside the Level 2 data or unified cache of this processing element.  The event indicates to software that the access missed in the Level 2 data or unified cache and might have a significant performance impact compared to the latency of an access that hits in the Level 2 data or unified cache\000 */
{ 129126 }, /* l2d_cache_rd\000\000cache\000L2D cache access, read\000event=0x50\000\00000\000Attributable Level 2 data cache access, read\000 */
{ 129862 }, /* l2d_cache_refill\000\000cache\000Level 2 data refill\000event=0x17\000\00000\000Level 2 data refill\000 */
{ 132288 }, /* l2d_cache_refill_rd\000\000cache\000L2D cache refill, read\000event=0x52\000\00000\000Attributable Level 2 data cache refill, read\000 */
{ 132829 }, /* l2d_cache_refill_wr\000\000cache\000L2D cache refill, write\000event=0x53\000\00000\000Attributable Level 2 data cache refill, write\000 */
{ 136414 }, /* l2d_cache_wb\000\000cache\000Attributable Level 2 data cache write-back\000event=0x18\000\00000\000Attributable Level 2 data cache write-back\000 */
{ 137387 }, /* l2d_cache_wb_clean\000\000cache\000L2D cache Write-Back, cleaning and coherency\000event=0x57\000\00000\000Level 2 data cache Write-Back, cleaning and coherency\000 */
{ 137968 }, /* l2d_cache_wb_victim\000\000cache\000L2D cache Write-Back, victim\000event=0x56\000\00000\000Attributable Level 2 data cache Write-Back, victim\000 */
{ 138385 }, /* l2d_cache_wr\000\000cache\000L2D cache access, write\000event=0x51\000\00000\000Attributable Level 2 data cache access, write\000 */
{ 143389 }, /* l2d_tlb\000\000cache\000Attributable Level 2 data or unified TLB access\000event=0x2f\000\00000\000Attributable Level 2 data or unified TLB access\000 */
{ 144725 }, /* l2d_tlb_refill\000\000cache\000Attributable Level 2 data TLB refill\000event=0x2d\000\00000\000Attributable Level 2 data TLB refill\000 */
{ 148794 }, /* l2i_tlb\000\000cache\000Attributable Level 2 instruction TLB access\000event=0x30\000\00000\000Attributable Level 2 instruction TLB access\000 */
{ 149110 }, /* l2i_tlb_refill\000\000cache\000Attributable Level 2 instruction TLB refill\000event=0x2e\000\00000\000Attributable Level 2 instruction TLB refill\000 */
{ 24127 }, /* bpu_branch_no_hit\000\000core imp def\000Predictable branch speculatively executed, unpredicted\000event=0x115\000\00000\000Predictable branch speculatively executed, unpredicted\000 */
{ 24285 }, /* bpu_btb_update\000\000core imp def\000Predictable branch speculatively executed, unpredicted, that mispredict\000event=0x11c\000\00000\000Predictable branch speculatively executed, unpredicted, that mispredict\000 */
{ 24474 }, /* bpu_conditional_branch_hit_btb\000\000core imp def\000Predictable conditional branch speculatively executed that hit any level of BTB\000event=0x111\000\00000\000Predictable conditional branch speculatively executed that hit any level of BTB\000 */
{ 24695 }, /* bpu_conditional_branch_hit_btb_and_mispredict\000\000core imp def\000Predictable conditional branch speculatively executed that hit any level of BTB that (direction) mispredict\000event=0x117\000\00000\000Predictable conditional branch speculatively executed that hit any level of BTB that (direction) mispredict\000 */
{ 24987 }, /* bpu_flush_mem_fault\000\000core imp def\000Flushes due to memory hazards\000event=0x121\000\00000\000Flushes due to memory hazards\000 */
{ 25097 }, /* bpu_hit_btb\000\000core imp def\000Predictable branch speculatively executed that hit any level of BTB\000event=0x110\000\00000\000Predictable branch speculatively executed that hit any level of BTB\000 */
{ 25275 }, /* bpu_hit_btb_and_mispredict\000\000core imp def\000Predictable branch speculatively executed that hit any level of BTB that mispredict\000event=0x116\000\00000\000Predictable branch speculatively executed that hit any level of BTB that mispredict\000 */
{ 25500 }, /* bpu_hit_indirect_predictor\000\000core imp def\000Predictable taken branch speculatively executed that hit any level of BTB that access the indirect predictor\000event=0x112\000\00000\000Predictable taken branch speculatively executed that hit any level of BTB that access the indirect predictor\000 */
{ 25775 }, /* bpu_hit_rsb\000\000core imp def\000Predictable taken branch speculatively executed that hit any level of BTB that access the return predictor\000event=0x113\000\00000\000Predictable taken branch speculatively executed that hit any level of BTB that access the return predictor\000 */
{ 26031 }, /* bpu_hit_rsb_and_mispredict\000\000core imp def\000Predictable taken branch speculatively executed that hit any level of BTB that access the return predictor that mispredict\000event=0x119\000\00000\000Predictable taken branch speculatively executed that hit any level of BTB that access the return predictor that mispredict\000 */
{ 26334 }, /* bpu_indirect_branch_hit_btb_and_mispredict\000\000core imp def\000Predictable taken branch speculatively executed that hit any level of BTB that access the indirect predictor that mispredict\000event=0x118\000\00000\000Predictable taken branch speculatively executed that hit any level of BTB that access the indirect predictor that mispredict\000 */
{ 26657 }, /* bpu_miss_rsb_and_mispredict\000\000core imp def\000Predictable taken branch speculatively executed that hit any level of BTB that access the overflow/underflow return predictor that mispredict\000event=0x11a\000\00000\000Predictable taken branch speculatively executed that hit any level of BTB that access the overflow/underflow return predictor that mispredict\000 */
{ 26999 }, /* bpu_no_prediction_mispredict\000\000core imp def\000Predictable branch speculatively executed, unpredicted, that mispredict\000event=0x11b\000\00000\000Predictable branch speculatively executed, unpredicted, that mispredict\000 */
{ 27202 }, /* bpu_rsb_full_stall\000\000core imp def\000Count predict pipe stalls due to speculative return address predictor full\000event=0x11d\000\00000\000Count predict pipe stalls due to speculative return address predictor full\000 */
{ 27401 }, /* bpu_unconditional_branch_miss_btb\000\000core imp def\000Predictable unconditional branch speculatively executed that did not hit any level of BTB\000event=0x114\000\00000\000Predictable unconditional branch speculatively executed that did not hit any level of BTB\000 */
{ 41133 }, /* breakpoint_match\000\000core imp def\000GPC detected a Breakpoint instruction match\000event=0xd501\000\00000\000GPC detected a Breakpoint instruction match\000 */
{ 70237 }, /* fsu_cancel\000\000core imp def\000Uops canceled (load cancels)\000event=0xd404\000\00000\000Uops canceled (load cancels)\000 */
{ 70439 }, /* fsu_div_sqrt_stall\000\000core imp def\000Count scheduler stalls due to divide/sqrt\000event=0xd405\000\00000\000Count scheduler stalls due to divide/sqrt\000 */
{ 70573 }, /* fsu_fsx_issued\000\000core imp def\000Uops issued by the scheduler on pipe X\000event=0xd401\000\00000\000Uops issued by the scheduler on pipe X\000 */
{ 70697 }, /* fsu_fsy_issued\000\000core imp def\000Uops issued by the scheduler on pipe Y\000event=0xd402\000\00000\000Uops issued by the scheduler on pipe Y\000 */
{ 70821 }, /* fsu_fsz_issued\000\000core imp def\000Uops issued by the scheduler on pipe Z\000event=0xd403\000\00000\000Uops issued by the scheduler on pipe Z\000 */
{ 70945 }, /* fsu_issued\000\000core imp def\000Uops issued by the FSU scheduler\000event=0xd400\000\00000\000Uops issued by the FSU scheduler\000 */
{ 71115 }, /* gpc_flush\000\000core imp def\000Flushes\000event=0x120\000\00000\000Flushes\000 */
{ 71171 }, /* gpc_swob_drain\000\000core imp def\000Number of SWOB drains\000event=0xd500\000\00000\000Number of SWOB drains\000 */
{ 72271 }, /* icf_early_mis_pred\000\000core imp def\000Early mispredict\000event=0xd100\000\00000\000Early mispredict\000 */
{ 72355 }, /* icf_feq_full\000\000core imp def\000FEQ full cycles\000event=0xd101\000\00000\000FEQ full cycles\000 */
{ 72431 }, /* icf_inst_fifo_full\000\000core imp def\000Instruction FIFO Full\000event=0xd102\000\00000\000Instruction FIFO Full\000 */
{ 72525 }, /* icf_inst_spec_decode\000\000core imp def\000Macro-ops speculatively decoded\000event=0x11f\000\00000\000Macro-ops speculatively decoded\000 */
{ 72640 }, /* icf_pc_fifo_full\000\000core imp def\000PC FIFO Full\000event=0xd105\000\00000\000PC FIFO Full\000 */
{ 72714 }, /* icf_stall\000\000core imp def\000ICF sent 0 instructions to IDR this cycle\000event=0xd104\000\00000\000ICF sent 0 instructions to IDR this cycle\000 */
{ 72839 }, /* idr_stall_bob_id\000\000core imp def\000Stall due to BOB ID\000event=0xd200\000\00000\000Stall due to BOB ID\000 */
{ 72927 }, /* idr_stall_flush\000\000core imp def\000Dispatch stall due to flush (6 cycles)\000event=0xd206\000\00000\000Dispatch stall due to flush (6 cycles)\000 */
{ 73052 }, /* idr_stall_fsu_sched\000\000core imp def\000Dispatch stall due to FSU scheduler entries\000event=0xd204\000\00000\000Dispatch stall due to FSU scheduler entries\000 */
{ 73191 }, /* idr_stall_ixu_sched\000\000core imp def\000Dispatch stall due to IXU scheduler entries\000event=0xd203\000\00000\000Dispatch stall due to IXU scheduler entries\000 */
{ 73330 }, /* idr_stall_lob_id\000\000core imp def\000Dispatch stall due to LOB entries\000event=0xd201\000\00000\000Dispatch stall due to LOB entries\000 */
{ 73446 }, /* idr_stall_rob_id\000\000core imp def\000Dispatch stall due to ROB entries\000event=0xd205\000\00000\000Dispatch stall due to ROB entries\000 */
{ 73562 }, /* idr_stall_sob_id\000\000core imp def\000Dispatch stall due to SOB entries\000event=0xd202\000\00000\000Dispatch stall due to SOB entries\000 */
{ 73678 }, /* idr_stall_swob_full\000\000core imp def\000Number of SWOB drains triggered by system register write when SWOB full\000event=0xd20a\000\00000\000Number of SWOB drains triggered by system register write when SWOB full\000 */
{ 73873 }, /* idr_stall_swob_raw\000\000core imp def\000Number of SWOB drains triggered by system register or special-purpose register read-after-write or specific special-purpose register writes that cause SWOB drain\000event=0xd209\000\00000\000Number of SWOB drains triggered by system register or special-purpose register read-after-write or specific special-purpose register writes that cause SWOB drain\000 */
{ 74247 }, /* idr_stall_swob_timeout\000\000core imp def\000Number of SWOB drains triggered by timeout\000event=0xd208\000\00000\000Number of SWOB drains triggered by timeout\000 */
{ 74387 }, /* idr_stall_wfi\000\000core imp def\000Dispatch stall due to WFI\000event=0xd207\000\00000\000Dispatch stall due to WFI\000 */
{ 92975 }, /* ixu_issue_cancel\000\000core imp def\000Any uop issued was canceled for any reason\000event=0xd301\000\00000\000Any uop issued was canceled for any reason\000 */
{ 93109 }, /* ixu_ixa_issued\000\000core imp def\000Uops issued by the scheduler on IXA\000event=0xd304\000\00000\000Uops issued by the scheduler on IXA\000 */
{ 93227 }, /* ixu_ixa_par0_issued\000\000core imp def\000Uops issued by the scheduler on IXA Par 0\000event=0xd305\000\00000\000Uops issued by the scheduler on IXA Par 0\000 */
{ 93362 }, /* ixu_ixa_par1_issued\000\000core imp def\000Uops issued by the scheduler on IXA Par 1\000event=0xd306\000\00000\000Uops issued by the scheduler on IXA Par 1\000 */
{ 93497 }, /* ixu_ixb_issued\000\000core imp def\000Uops issued by the scheduler on IXB\000event=0xd307\000\00000\000Uops issued by the scheduler on IXB\000 */
{ 93615 }, /* ixu_ixb_par0_issued\000\000core imp def\000Uops issued by the scheduler on IXB Par 0\000event=0xd308\000\00000\000Uops issued by the scheduler on IXB Par 0\000 */
{ 93750 }, /* ixu_ixb_par1_issued\000\000core imp def\000Uops issued by the scheduler on IXB Par 1\000event=0xd309\000\00000\000Uops issued by the scheduler on IXB Par 1\000 */
{ 93885 }, /* ixu_ixc_issued\000\000core imp def\000Uops issued by the scheduler on IXC\000event=0xd30a\000\00000\000Uops issued by the scheduler on IXC\000 */
{ 94003 }, /* ixu_ixc_par0_issued\000\000core imp def\000Uops issued by the scheduler on IXC Par 0\000event=0xd30b\000\00000\000Uops issued by the scheduler on IXC Par 0\000 */
{ 94138 }, /* ixu_ixc_par1_issued\000\000core imp def\000Uops issued by the scheduler on IXC Par 1\000event=0xd30c\000\00000\000Uops issued by the scheduler on IXC Par 1\000 */
{ 94273 }, /* ixu_ixd_issued\000\000core imp def\000Uops issued by the scheduler on IXD\000event=0xd30d\000\00000\000Uops issued by the scheduler on IXD\000 */
{ 94391 }, /* ixu_ixd_par0_issued\000\000core imp def\000Uops issued by the scheduler on IXD Par 0\000event=0xd30e\000\00000\000Uops issued by the scheduler on IXD Par 0\000 */
{ 94526 }, /* ixu_ixd_par1_issued\000\000core imp def\000Uops issued by the scheduler on IXD Par 1\000event=0xd30f\000\00000\000Uops issued by the scheduler on IXD Par 1\000 */
{ 94661 }, /* ixu_load_cancel\000\000core imp def\000A load wakeup to the scheduler has been cancelled\000event=0xd302\000\00000\000A load wakeup to the scheduler has been cancelled\000 */
{ 94808 }, /* ixu_num_uops_issued\000\000core imp def\000Instructions issued by the scheduler\000event=0xd300\000\00000\000Instructions issued by the scheduler\000 */
{ 94933 }, /* ixu_slow_cancel\000\000core imp def\000The scheduler had to cancel one slow Uop due to resource conflict\000event=0xd303\000\00000\000The scheduler had to cancel one slow Uop due to resource conflict\000 */
{ 95276 }, /* l1_pfetch_dist_inc\000\000core imp def\000L1 prefetcher, distance was increased\000event=0xd60a\000\00000\000L1 prefetcher, distance was increased\000 */
{ 95402 }, /* l1_pfetch_dist_rst\000\000core imp def\000L1 prefetcher, distance was reset\000event=0xd609\000\00000\000L1 prefetcher, distance was reset\000 */
{ 95520 }, /* l1_pfetch_entry_trained\000\000core imp def\000L1 prefetcher, table entry is trained\000event=0xd60b\000\00000\000L1 prefetcher, table entry is trained\000 */
{ 95651 }, /* l1_pfetch_l2_req\000\000core imp def\000L1 prefetcher, load prefetch to L2 generated\000event=0xd608\000\00000\000L1 prefetcher, load prefetch to L2 generated\000 */
{ 95789 }, /* l1_pfetch_ld_fill\000\000core imp def\000L1 prefetcher, load prefetch fills into the L1 cache\000event=0xd607\000\00000\000L1 prefetcher, load prefetch fills into the L1 cache\000 */
{ 95944 }, /* l1_pfetch_ld_gen\000\000core imp def\000L1 prefetcher, load prefetch requests generated\000event=0xd606\000\00000\000L1 prefetcher, load prefetch requests generated\000 */
{ 110678 }, /* l1d_tlb_miss\000\000core imp def\000L1D TLB miss\000event=0xd600\000\00000\000L1D TLB miss\000 */
{ 120128 }, /* l1i_tlb_miss\000\000core imp def\000L1I TLB miss\000event=0xd103\000\00000\000L1I TLB miss\000 */
{ 125209 }, /* l2_prefetch_refill\000\000core imp def\000Level 2 prefetch requests, refilled to L2 cache\000event=0x10a\000\00000\000Level 2 prefetch requests, refilled to L2 cache\000 */
{ 125354 }, /* l2_prefetch_req\000\000core imp def\000L2 prefetcher, load prefetch requests generated\000event=0xd703\000\00000\000L2 prefetcher, load prefetch requests generated\000 */
{ 125497 }, /* l2_prefetch_upgrade\000\000core imp def\000Level 2 prefetch requests, late\000event=0x10b\000\00000\000Level 2 prefetch requests, late\000 */
{ 125611 }, /* l2c_dat_crd_stall\000\000core imp def\000L2 TXDAT LCRD blocked\000event=0x169\000\00000\000L2 TXDAT LCRD blocked\000 */
{ 125703 }, /* l2c_data_refill\000\000core imp def\000L2 refill from D-side miss\000event=0xd702\000\00000\000L2 refill from D-side miss\000 */
{ 125804 }, /* l2c_inst_refill\000\000core imp def\000L2 refill from I-side miss\000event=0xd701\000\00000\000L2 refill from I-side miss\000 */
{ 125905 }, /* l2c_pipe_replay\000\000core imp def\000L2 pipeline replay\000event=0xd700\000\00000\000L2 pipeline replay\000 */
{ 125990 }, /* l2c_req_crd_stall\000\000core imp def\000L2 TXREQ LCRD blocked\000event=0x16b\000\00000\000L2 TXREQ LCRD blocked\000 */
{ 126082 }, /* l2c_rsp_crd_stall\000\000core imp def\000L2 TXRSP LCRD blocked\000event=0x16a\000\00000\000L2 TXRSP LCRD blocked\000 */
{ 126174 }, /* l2c_snoop\000\000core imp def\000Bus request sn\000event=0x156\000\00000\000Bus request sn\000 */
{ 158566 }, /* ld_from_st_fwd\000\000core imp def\000Load satisified from store forwarded data\000event=0xd605\000\00000\000Load satisified from store forwarded data\000 */
{ 162425 }, /* lsu_st_retire_stall\000\000core imp def\000Store retirement pipe stall\000event=0xd60c\000\00000\000Store retirement pipe stall\000 */
{ 166820 }, /* mmu_d_otb_alloc\000\000core imp def\000L2D OTB allocate\000event=0xd800\000\00000\000L2D OTB allocate\000 */
{ 166901 }, /* mmu_d_s1_walk_cache_lookup\000\000core imp def\000D-side S1 Page walk cache lookup\000event=0xd807\000\00000\000D-side S1 Page walk cache lookup\000 */
{ 167025 }, /* mmu_d_s1_walk_cache_refill\000\000core imp def\000D-side S1 Page walk cache refill\000event=0xd808\000\00000\000D-side S1 Page walk cache refill\000 */
{ 167149 }, /* mmu_d_s1_walk_fault\000\000core imp def\000D-side Stage1 tablewalk fault\000event=0xd80b\000\00000\000D-side Stage1 tablewalk fault\000 */
{ 167260 }, /* mmu_d_s2_walk_cache_lookup\000\000core imp def\000D-side S2 Page walk cache lookup\000event=0xd809\000\00000\000D-side S2 Page walk cache lookup\000 */
{ 167384 }, /* mmu_d_s2_walk_cache_refill\000\000core imp def\000D-side S2 Page walk cache refill\000event=0xd80a\000\00000\000D-side S2 Page walk cache refill\000 */
{ 167508 }, /* mmu_d_s2_walk_fault\000\000core imp def\000D-side Stage2 tablewalk fault\000event=0xd80c\000\00000\000D-side Stage2 tablewalk fault\000 */
{ 167619 }, /* mmu_d_trans_cache_hit_s1l0_walk\000\000core imp def\000DTLB Translation cache hit on S1L0 walk cache entry\000event=0xd803\000\00000\000DTLB Translation cache hit on S1L0 walk cache entry\000 */
{ 167786 }, /* mmu_d_trans_cache_hit_s1l1_walk\000\000core imp def\000DTLB Translation cache hit on S1L1 walk cache entry\000event=0xd802\000\00000\000DTLB Translation cache hit on S1L1 walk cache entry\000 */
{ 167953 }, /* mmu_d_trans_cache_hit_s1l2_walk\000\000core imp def\000DTLB Translation cache hit on S1L2 walk cache entry\000event=0xd801\000\00000\000DTLB Translation cache hit on S1L2 walk cache entry\000 */
{ 168120 }, /* mmu_d_trans_cache_hit_s2l0_walk\000\000core imp def\000DTLB Translation cache hit on S2L0 walk cache entry\000event=0xd806\000\00000\000DTLB Translation cache hit on S2L0 walk cache entry\000 */
{ 168287 }, /* mmu_d_trans_cache_hit_s2l1_walk\000\000core imp def\000DTLB Translation cache hit on S2L1 walk cache entry\000event=0xd805\000\00000\000DTLB Translation cache hit on S2L1 walk cache entry\000 */
{ 168454 }, /* mmu_d_trans_cache_hit_s2l2_walk\000\000core imp def\000DTLB Translation cache hit on S2L2 walk cache entry\000event=0xd804\000\00000\000DTLB Translation cache hit on S2L2 walk cache entry\000 */
{ 168621 }, /* mmu_d_walk_steps\000\000core imp def\000D-side Tablewalk steps or descriptor fetches\000event=0xd80d\000\00000\000D-side Tablewalk steps or descriptor fetches\000 */
{ 168759 }, /* mmu_i_otb_alloc\000\000core imp def\000L2I OTB allocate\000event=0xd900\000\00000\000L2I OTB allocate\000 */
{ 168840 }, /* mmu_i_s1_walk_cache_lookup\000\000core imp def\000I-side S1 Page walk cache lookup\000event=0xd907\000\00000\000I-side S1 Page walk cache lookup\000 */
{ 168964 }, /* mmu_i_s1_walk_cache_refill\000\000core imp def\000I-side S1 Page walk cache refill\000event=0xd908\000\00000\000I-side S1 Page walk cache refill\000 */
{ 169088 }, /* mmu_i_s1_walk_fault\000\000core imp def\000I-side Stage1 tablewalk fault\000event=0xd90b\000\00000\000I-side Stage1 tablewalk fault\000 */
{ 169199 }, /* mmu_i_s2_walk_cache_lookup\000\000core imp def\000I-side S2 Page walk cache lookup\000event=0xd909\000\00000\000I-side S2 Page walk cache lookup\000 */
{ 169323 }, /* mmu_i_s2_walk_cache_refill\000\000core imp def\000I-side S2 Page walk cache refill\000event=0xd90a\000\00000\000I-side S2 Page walk cache refill\000 */
{ 169447 }, /* mmu_i_s2_walk_fault\000\000core imp def\000I-side Stage2 tablewalk fault\000event=0xd90c\000\00000\000I-side Stage2 tablewalk fault\000 */
{ 169558 }, /* mmu_i_trans_cache_hit_s1l0_walk\000\000core imp def\000ITLB Translation cache hit on S1L0 walk cache entry\000event=0xd903\000\00000\000ITLB Translation cache hit on S1L0 walk cache entry\000 */
{ 169725 }, /* mmu_i_trans_cache_hit_s1l1_walk\000\000core imp def\000ITLB Translation cache hit on S1L1 walk cache entry\000event=0xd902\000\00000\000ITLB Translation cache hit on S1L1 walk cache entry\000 */
{ 169892 }, /* mmu_i_trans_cache_hit_s1l2_walk\000\000core imp def\000ITLB Translation cache hit on S1L2 walk cache entry\000event=0xd901\000\00000\000ITLB Translation cache hit on S1L2 walk cache entry\000 */
{ 170059 }, /* mmu_i_trans_cache_hit_s2l0_walk\000\000core imp def\000ITLB Translation cache hit on S2L0 walk cache entry\000event=0xd906\000\00000\000ITLB Translation cache hit on S2L0 walk cache entry\000 */
{ 170226 }, /* mmu_i_trans_cache_hit_s2l1_walk\000\000core imp def\000ITLB Translation cache hit on S2L1 walk cache entry\000event=0xd905\000\00000\000ITLB Translation cache hit on S2L1 walk cache entry\000 */
{ 170393 }, /* mmu_i_trans_cache_hit_s2l2_walk\000\000core imp def\000ITLB Translation cache hit on S2L2 walk cache entry\000event=0xd904\000\00000\000ITLB Translation cache hit on S2L2 walk cache entry\000 */
{ 170560 }, /* mmu_i_walk_steps\000\000core imp def\000I-side Tablewalk steps or descriptor fetches\000event=0xd90d\000\00000\000I-side Tablewalk steps or descriptor fetches\000 */
{ 172954 }, /* msc_etm_extout0\000\000core imp def\000ETM extout bit 0\000event=0x141\000\00000\000ETM extout bit 0\000 */
{ 173034 }, /* msc_etm_extout1\000\000core imp def\000ETM extout bit 1\000event=0x142\000\00000\000ETM extout bit 1\000 */
{ 173114 }, /* msc_etm_extout2\000\000core imp def\000ETM extout bit 2\000event=0x143\000\00000\000ETM extout bit 2\000 */
{ 173194 }, /* msc_etm_extout3\000\000core imp def\000ETM extout bit 3\000event=0x144\000\00000\000ETM extout bit 3\000 */
{ 173378 }, /* ofb_full\000\000core imp def\000OFB full cycles\000event=0xd601\000\00000\000OFB full cycles\000 */
{ 186800 }, /* stall_backend_cache\000\000core imp def\000Dispatch stall due to L1 data cache miss\000event=0xd20d\000\00000\000Dispatch stall due to L1 data cache miss\000 */
{ 192984 }, /* stall_backend_resource\000\000core imp def\000Dispatch stall due to lack of any core resource\000event=0xd20f\000\00000\000Dispatch stall due to lack of any core resource\000 */
{ 195390 }, /* stall_backend_tlb\000\000core imp def\000Dispatch stall due to L1 data TLB miss\000event=0xd20e\000\00000\000Dispatch stall due to L1 data TLB miss\000 */
{ 196520 }, /* stall_frontend_cache\000\000core imp def\000Dispatch stall due to L1 instruction cache miss\000event=0xd20b\000\00000\000Dispatch stall due to L1 instruction cache miss\000 */
{ 197996 }, /* stall_frontend_tlb\000\000core imp def\000Dispatch stall due to L1 instruction TLB miss\000event=0xd20c\000\00000\000Dispatch stall due to L1 instruction TLB miss\000 */
{ 214956 }, /* watchpoint_match\000\000core imp def\000LSU detected a Watchpoint data match\000event=0xd60d\000\00000\000LSU detected a Watchpoint data match\000 */
{ 56959 }, /* exc_dabort\000\000exception\000Exception taken, Data Abort and SError\000event=0x84\000\00000\000Exception taken, Data Abort and SError\000 */
{ 57280 }, /* exc_fiq\000\000exception\000Exception taken, FIQ\000event=0x87\000\00000\000Exception taken, FIQ\000 */
{ 57633 }, /* exc_hvc\000\000exception\000Exception taken, Hypervisor Call\000event=0x8a\000\00000\000Exception taken, Hypervisor Call\000 */
{ 57939 }, /* exc_irq\000\000exception\000Exception taken, IRQ\000event=0x86\000\00000\000Exception taken, IRQ\000 */
{ 58355 }, /* exc_pabort\000\000exception\000Exception taken, Instruction Abort\000event=0x83\000\00000\000Exception taken, Instruction Abort\000 */
{ 58660 }, /* exc_return\000\000exception\000Instruction architecturally executed, condition check pass, exception return\000event=0xa\000\00000\000Instruction architecturally executed, condition check pass, exception return\000 */
{ 59572 }, /* exc_svc\000\000exception\000Exception taken, Supervisor Call\000event=0x82\000\00000\000Exception taken, Supervisor Call\000 */
{ 59896 }, /* exc_taken\000\000exception\000Exception taken\000event=0x9\000\00000\000Exception taken\000 */
{ 60575 }, /* exc_trap_dabort\000\000exception\000Exception taken, Data Abort or SError not taken locally\000event=0x8c\000\00000\000Exception taken, Data Abort or SError not taken locally\000 */
{ 61080 }, /* exc_trap_fiq\000\000exception\000Exception taken, FIQ not taken locally\000event=0x8f\000\00000\000Exception taken, FIQ not taken locally\000 */
{ 62347 }, /* exc_trap_irq\000\000exception\000Exception taken, IRQ not taken locally\000event=0x8e\000\00000\000Exception taken, IRQ not taken locally\000 */
{ 62841 }, /* exc_trap_other\000\000exception\000Exception taken, Other traps not taken locally\000event=0x8d\000\00000\000Exception taken, Other traps not taken locally\000 */
{ 63383 }, /* exc_trap_pabort\000\000exception\000Exception taken, Instruction Abort not taken locally\000event=0x8b\000\00000\000Exception taken, Instruction Abort not taken locally\000 */
{ 64129 }, /* exc_undef\000\000exception\000Exception taken, Other synchronous\000event=0x81\000\00000\000Exception taken, Other synchronous\000 */
{ 20878 }, /* ase_spec\000\000instruction\000Operation speculatively executed, Advanced SIMD instruction\000event=0x74\000\00000\000Operation speculatively executed, Advanced SIMD instruction\000 */
{ 30599 }, /* br_immed_retired\000\000instruction\000Instruction architecturally executed, immediate branch\000event=0xd\000\00000\000Instruction architecturally executed, immediate branch\000 */
{ 36351 }, /* br_mis_pred_retired\000\000instruction\000Instruction architecturally executed, mispredicted branch\000event=0x22\000\00000\000Instruction architecturally executed, mispredicted branch\000 */
{ 37393 }, /* br_retired\000\000instruction\000Instruction architecturally executed, branch\000event=0x21\000\00000\000Instruction architecturally executed, branch\000 */
{ 40483 }, /* br_return_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, procedure return\000event=0xe\000\00000\000Instruction architecturally executed, condition code check pass, procedure return\000 */
{ 45546 }, /* cid_write_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000event=0xb\000\00000\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000 */
{ 48419 }, /* crypto_spec\000\000instruction\000Operation speculatively executed, Cryptographic instruction\000event=0x77\000\00000\000Operation speculatively executed, Cryptographic instruction\000 */
{ 51117 }, /* dmb_spec\000\000instruction\000Barrier speculatively executed, DMB\000event=0x7e\000\00000\000Barrier speculatively executed, DMB\000 */
{ 52153 }, /* dp_spec\000\000instruction\000Operation speculatively executed, integer data processing\000event=0x73\000\00000\000Operation speculatively executed, integer data processing\000 */
{ 53209 }, /* dsb_spec\000\000instruction\000Barrier speculatively executed, DSB\000event=0x7d\000\00000\000Barrier speculatively executed, DSB\000 */
{ 90819 }, /* inst_retired\000\000instruction\000Instruction architecturally executed\000event=0x8\000\00000\000Instruction architecturally executed\000 */
{ 91235 }, /* inst_spec\000\000instruction\000Operation speculatively executed\000event=0x1b\000\00000\000Operation speculatively executed\000 */
{ 91769 }, /* isb_spec\000\000instruction\000Barrier speculatively executed, ISB\000event=0x7c\000\00000\000Barrier speculatively executed, ISB\000 */
{ 159047 }, /* ld_spec\000\000instruction\000Operation speculatively executed, load\000event=0x70\000\00000\000Operation speculatively executed, load\000 */
{ 159973 }, /* ldst_spec\000\000instruction\000Operation speculatively executed, load or store\000event=0x72\000\00000\000Operation speculatively executed, load or store\000 */
{ 173274 }, /* nop_spec\000\000instruction\000Speculatively executed, NOP\000event=0x100\000\00000\000Operation speculatively executed, NOP\000 */
{ 173450 }, /* op_retired\000\000instruction\000Micro-operation architecturally executed\000event=0x3a\000\00000\000Micro-operation architecturally executed.  The counter counts each operation counted by OP_SPEC that would be executed in a simple sequential execution of the program\000 */
{ 173697 }, /* op_spec\000\000instruction\000Micro-operation speculatively executed\000event=0x3b\000\00000\000Micro-operation speculatively executed.  The counter counts the number of operations executed by the processing element, including those that are executed speculatively and would not be executed in a simple sequential execution of the program\000 */
{ 173697 }, /* op_spec\000\000instruction\000Micro-operation speculatively executed\000event=0x3b\000\00000\000Micro-operation speculatively executed.  The counter counts the number of operations executed by the processing element, including those that are executed speculatively and would not be executed in a simple sequential execution of the program\000 */
{ 174889 }, /* pc_write_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, software change of the PC\000event=0xc\000\00000\000Instruction architecturally executed, condition code check pass, software change of the PC\000 */
{ 175115 }, /* pc_write_spec\000\000instruction\000Operation speculatively executed, software change of the PC\000event=0x76\000\00000\000Operation speculatively executed, software change of the PC\000 */
{ 177878 }, /* rc_ld_spec\000\000instruction\000Release consistency operation speculatively executed, Load-Acquire\000event=0x90\000\00000\000Release consistency operation speculatively executed, Load-Acquire\000 */
{ 178251 }, /* rc_st_spec\000\000instruction\000Release consistency operation speculatively executed, Store-Release\000event=0x91\000\00000\000Release consistency operation speculatively executed, Store-Release\000 */
{ 185342 }, /* st_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, store\000event=0x7\000\00000\000Instruction architecturally executed, condition code check pass, store\000 */
{ 185522 }, /* st_spec\000\000instruction\000Operation speculatively executed, store\000event=0x71\000\00000\000Operation speculatively executed, store\000 */
{ 205736 }, /* sw_incr\000\000instruction\000Instruction architecturally executed, Condition code check pass, software increment\000event=0\000\00000\000Instruction architecturally executed, Condition code check pass, software increment\000 */
{ 210294 }, /* ttbr_write_retired\000\000instruction\000Instruction architecturally executed, Condition code check pass, write to TTBR\000event=0x1c\000\00000\000Instruction architecturally executed, Condition code check pass, write to TTBR\000 */
{ 214354 }, /* vfp_spec\000\000instruction\000Operation speculatively executed, floating-point instruction\000event=0x75\000\00000\000Operation speculatively executed, floating-point instruction\000 */
{ 159504 }, /* ldrex_spec\000\000intrinsic\000Exclusive operation speculatively executed, LDREX or LDX\000event=0x6c\000\00000\000Exclusive operation speculatively executed, LDREX or LDX\000 */
{ 200622 }, /* strex_fail_spec\000\000intrinsic\000Exclusive operation speculatively executed, STREX or STX fail\000event=0x6e\000\00000\000Exclusive operation speculatively executed, STREX or STX fail\000 */
{ 201193 }, /* strex_pass_spec\000\000intrinsic\000Exclusive operation speculatively executed, STREX or STX pass\000event=0x6d\000\00000\000Exclusive operation speculatively executed, STREX or STX pass\000 */
{ 201745 }, /* strex_spec\000\000intrinsic\000Exclusive operation speculatively executed, STREX or STX\000event=0x6f\000\00000\000Exclusive operation speculatively executed, STREX or STX\000 */
{ 155492 }, /* ld_align_lat\000\000memory\000Load with additional latency from alignment\000event=0x4021\000\00000\000Load with additional latency from alignment\000 */
{ 158874 }, /* ld_retired\000\000memory\000Instruction architecturally executed, condition code check pass, load\000event=0x6\000\00000\000Instruction architecturally executed, condition code check pass, load\000 */
{ 159841 }, /* ldst_align_lat\000\000memory\000Access with additional latency from alignment\000event=0x4020\000\00000\000Access with additional latency from alignment\000 */
{ 163148 }, /* mem_access\000\000memory\000Data memory access\000event=0x13\000\00000\000Data memory access\000 */
{ 163403 }, /* mem_access_checked\000\000memory\000Checked data memory access\000event=0x4024\000\00000\000Checked data memory access\000 */
{ 163501 }, /* mem_access_checked_rd\000\000memory\000Checked data memory access, read\000event=0x4025\000\00000\000Checked data memory access, read\000 */
{ 163614 }, /* mem_access_checked_wr\000\000memory\000Checked data memory access, write\000event=0x4026\000\00000\000Checked data memory access, write\000 */
{ 164298 }, /* mem_access_rd\000\000memory\000Data memory access, read\000event=0x66\000\00000\000Data memory access, read\000 */
{ 164956 }, /* mem_access_wr\000\000memory\000Data memory access, write\000event=0x67\000\00000\000Data memory access, write\000 */
{ 166584 }, /* memory_error\000\000memory\000Local memory error\000event=0x1a\000\00000\000Local memory error\000 */
{ 185116 }, /* st_align_lat\000\000memory\000Store with additional latency from alignment\000event=0x4022\000\00000\000Store with additional latency from alignment\000 */
{ 211505 }, /* unaligned_ld_spec\000\000memory\000Unaligned access, read\000event=0x68\000\00000\000Unaligned access, read\000 */
{ 212200 }, /* unaligned_ldst_spec\000\000memory\000Unaligned access\000event=0x6a\000\00000\000Unaligned access\000 */
{ 212637 }, /* unaligned_st_spec\000\000memory\000Unaligned access, write\000event=0x69\000\00000\000Unaligned access, write\000 */
{ 185831 }, /* stall\000\000pipeline\000No operation sent for execution\000event=0x3c\000\00000\000No operation sent for execution.  The counter counts every attributable cycle on which no attributable instruction or operation was sent for execution on this processing element\000 */
{ 186072 }, /* stall_backend\000\000pipeline\000No operation issued due to the backend\000event=0x24\000\00000\000No operation issued due to the backend\000 */
{ 192744 }, /* stall_backend_mem\000\000pipeline\000Memory stall cycles\000event=0x4005\000\00000\000Memory stall cycles.  The counter counts each cycle counted by STALL_BACKEND where there is a cache miss in the last level of cache within the processing element clock domain\000 */
{ 195875 }, /* stall_frontend\000\000pipeline\000No operation issued because of the frontend\000event=0x23\000\00000\000No operation issued because of the frontend\000 */
{ 199011 }, /* stall_slot\000\000pipeline\000No operation sent for execution on a slot\000event=0x3f\000\00000\000No operation sent for execution on a slot.  The counter counts on each attributable cycle the number of instruction or operation slots that were not occupied by an instruction or operation attributable to the processing element\000 */
{ 199317 }, /* stall_slot_backend\000\000pipeline\000No operation sent for execution on a slot due to the backend\000event=0x3d\000\00000\000No operation sent for execution on a slot due to the backend.  Counts each slot counted by STALL_SLOT where no attributable instruction or operation was sent for execution because the backend is unable to accept it\000 */
{ 199953 }, /* stall_slot_frontend\000\000pipeline\000No operation sent for execution on a slot due to the frontend\000event=0x3e\000\00000\000No operation sent for execution on a slot due to the frontend.  Counts each slot counted by STALL_SLOT where no attributable instruction or operation was sent for execution because there was no attributable instruction or operation available to issue from the processing element from the frontend for the slot\000 */
{ 183111 }, /* sample_collision\000\000spe\000Sample collided with previous sample\000event=0x4003\000\00000\000Sample collided with previous sample\000 */
{ 183327 }, /* sample_feed\000\000spe\000Sample Taken\000event=0x4001\000\00000\000Sample Taken\000 */
{ 183545 }, /* sample_filtrate\000\000spe\000Sample Taken and not removed by filtering\000event=0x4002\000\00000\000Sample Taken and not removed by filtering\000 */
{ 183858 }, /* sample_pop\000\000spe\000Sample Population\000event=0x4000\000\00000\000Sample Population\000 */
};

static const struct compact_pmu_event pmu_events__ampere_emag[] = {
{ 30753 }, /* br_immed_spec\000\000branch\000Branch speculatively executed, immediate branch\000event=0x78\000\00000\000Branch speculatively executed, immediate branch\000 */
{ 35038 }, /* br_indirect_spec\000\000branch\000Branch speculatively executed, indirect branch\000event=0x7a\000\00000\000Branch speculatively executed, indirect branch\000 */
{ 35628 }, /* br_mis_pred\000\000branch\000Branch mispredicted\000event=0x10\000\00000\000Mispredicted or not predicted branch speculatively executed\000 */
{ 37156 }, /* br_pred\000\000branch\000Predictable branch\000event=0x12\000\00000\000Predictable branch speculatively executed\000 */
{ 40692 }, /* br_return_spec\000\000branch\000Branch speculatively executed, procedure return\000event=0x79\000\00000\000Branch speculatively executed, procedure return\000 */
{ 41358 }, /* bus_access\000\000bus\000Attributable Bus access\000event=0x19\000\00000\000Attributable Bus access\000 */
{ 41979 }, /* bus_access_normal\000\000bus\000Bus access, Normal\000event=0x64\000\00000\000Bus access, Normal\000 */
{ 42055 }, /* bus_access_not_shared\000\000bus\000Bus access, not Normal, Cacheable, Shareable\000event=0x63\000\00000\000Bus access, not Normal, Cacheable, Shareable\000 */
{ 42187 }, /* bus_access_periph\000\000bus\000Bus access, peripheral\000event=0x65\000\00000\000Bus access, peripheral\000 */
{ 42271 }, /* bus_access_rd\000\000bus\000Bus access read\000event=0x60\000\00000\000Bus access read\000 */
{ 42562 }, /* bus_access_shared\000\000bus\000Bus access, Normal, Cacheable, Shareable\000event=0x62\000\00000\000Bus access, Normal, Cacheable, Shareable\000 */
{ 42682 }, /* bus_access_wr\000\000bus\000Bus access write\000event=0x61\000\00000\000Bus access write\000 */
{ 41269 }, /* btb_mis_pred\000\000cache\000BTB misprediction\000event=0x102\000\00000\000Branch target buffer misprediction\000 */
{ 53476 }, /* dtb_miss\000\000cache\000DTB miss\000event=0x104\000\00000\000DTB miss\000 */
{ 91994 }, /* itb_miss\000\000cache\000ITB miss\000event=0x103\000\00000\000ITB miss\000 */
{ 100196 }, /* l1_stage2_tlb_refill\000\000cache\000L1 stage 2 TLB refill\000event=0x111\000\00000\000Level 1 stage 2 TLB refill\000 */
{ 100289 }, /* l1d_cache\000\000cache\000Level 1 data cache access\000event=0x4\000\00000\000Level 1 data cache access\000 */
{ 101403 }, /* l1d_cache_inval\000\000cache\000L1D cache invalidate\000event=0x48\000\00000\000Attributable Level 1 data cache invalidate\000 */
{ 102184 }, /* l1d_cache_late_miss\000\000cache\000L1D cache late miss\000event=0x105\000\00000\000Level 1 data cache late miss\000 */
{ 102596 }, /* l1d_cache_prefetch\000\000cache\000L1D cache prefetch\000event=0x106\000\00000\000Level 1 data cache prefetch request\000 */
{ 102693 }, /* l1d_cache_rd\000\000cache\000L1D cache access, read\000event=0x40\000\00000\000Attributable Level 1 data cache access, read\000 */
{ 103129 }, /* l1d_cache_refill\000\000cache\000Level 1 data cache refill\000event=0x3\000\00000\000Level 1 data cache refill\000 */
{ 105894 }, /* l1d_cache_refill_rd\000\000cache\000L1D cache refill, read\000event=0x42\000\00000\000Attributable Level 1 data cache refill, read\000 */
{ 108997 }, /* l1d_cache_wr\000\000cache\000L1D cache access, write\000event=0x41\000\00000\000Attributable Level 1 data cache access, write\000 */
{ 109993 }, /* l1d_tlb\000\000cache\000L1D TLB access\000event=0x25\000\00000\000This event counts any load or store operation which accesses the data L1 TLB\000 */
{ 111207 }, /* l1d_tlb_refill\000\000cache\000Attributable Level 1 data TLB refill\000event=0x5\000\00000\000Attributable Level 1 data TLB refill\000 */
{ 112199 }, /* l1d_tlb_refill_rd\000\000cache\000L1D tlb refill, read\000event=0x4c\000\00000\000Attributable Level 1 data TLB refill, read\000 */
{ 113093 }, /* l1d_tlb_refill_wr\000\000cache\000L1D tlb refill, write\000event=0x4d\000\00000\000Attributable Level 1 data TLB refill, write\000 */
{ 116626 }, /* l1i_cache\000\000cache\000Attributable Level 1 instruction cache access\000event=0x14\000\00000\000Attributable Level 1 instruction cache access\000 */
{ 118420 }, /* l1i_cache_refill\000\000cache\000Level 1 instruction cache refill\000event=0x1\000\00000\000Level 1 instruction cache refill\000 */
{ 119537 }, /* l1i_tlb\000\000cache\000Attributable Level 1 instruction TLB access\000event=0x26\000\00000\000This event counts any instruction fetch which accesses the instruction L1 TLB\000 */
{ 120198 }, /* l1i_tlb_refill\000\000cache\000Attributable Level 1 instruction TLB refill\000event=0x2\000\00000\000Attributable Level 1 instruction TLB refill\000 */
{ 126244 }, /* l2d_cache\000\000cache\000Level 2 data cache access\000event=0x16\000\00000\000Level 2 data cache access\000 */
{ 127466 }, /* l2d_cache_inval\000\000cache\000L2D cache invalidate\000event=0x58\000\00000\000Attributable Level 2 data cache invalidate\000 */
{ 129029 }, /* l2d_cache_prefetch\000\000cache\000L2D cache prefetch\000event=0x107\000\00000\000Level 2 data cache prefetch request\000 */
{ 129126 }, /* l2d_cache_rd\000\000cache\000L2D cache access, read\000event=0x50\000\00000\000Attributable Level 2 data cache access, read\000 */
{ 129862 }, /* l2d_cache_refill\000\000cache\000Level 2 data refill\000event=0x17\000\00000\000Level 2 data refill\000 */
{ 132288 }, /* l2d_cache_refill_rd\000\000cache\000L2D cache refill, read\000event=0x52\000\00000\000Attributable Level 2 data cache refill, read\000 */
{ 132829 }, /* l2d_cache_refill_wr\000\000cache\000L2D cache refill, write\000event=0x53\000\00000\000Attributable Level 2 data cache refill, write\000 */
{ 136414 }, /* l2d_cache_wb\000\000cache\000Attributable Level 2 data cache write-back\000event=0x18\000\00000\000Attributable Level 2 data cache write-back\000 */
{ 137387 }, /* l2d_cache_wb_clean\000\000cache\000L2D cache Write-Back, cleaning and coherency\000event=0x57\000\00000\000Level 2 data cache Write-Back, cleaning and coherency\000 */
{ 137968 }, /* l2d_cache_wb_victim\000\000cache\000L2D cache Write-Back, victim\000event=0x56\000\00000\000Attributable Level 2 data cache Write-Back, victim\000 */
{ 138385 }, /* l2d_cache_wr\000\000cache\000L2D cache access, write\000event=0x51\000\00000\000Attributable Level 2 data cache access, write\000 */
{ 143886 }, /* l2d_tlb_access\000\000cache\000L2D TLB access\000event=0x34\000\00000\000Level 2 access to data TLB that caused a page table walk. This event counts on any data access which causes L2D_TLB_REFILL to count\000 */
{ 148912 }, /* l2i_tlb_access\000\000cache\000L2I TLB access\000event=0x35\000\00000\000Level 2 access to instruciton TLB that caused a page table walk. This event counts on any instruciton access which causes L2I_TLB_REFILL to count\000 */
{ 174344 }, /* page_walk_l0_stage1_hit\000\000cache\000Page walk, L0 stage-1 hit\000event=0x112\000\00000\000Page walk cache level-0 stage-1 hit\000 */
{ 174453 }, /* page_walk_l1_stage1_hit\000\000cache\000Page walk, L1 stage-1 hit\000event=0x113\000\00000\000Page walk cache level-1 stage-1 hit\000 */
{ 174562 }, /* page_walk_l1_stage2_hit\000\000cache\000Page walk, L1 stage-2 hit\000event=0x115\000\00000\000Page walk cache level-1 stage-2 hit\000 */
{ 174671 }, /* page_walk_l2_stage1_hit\000\000cache\000Page walk, L2 stage-1 hit\000event=0x114\000\00000\000Page walk cache level-2 stage-1 hit\000 */
{ 174780 }, /* page_walk_l2_stage2_hit\000\000cache\000Page walk, L2 stage-2 hit\000event=0x116\000\00000\000Page walk cache level-2 stage-2 hit\000 */
{ 48130 }, /* cpu_cycles\000\000clock\000Cycle\000event=0x11\000\00000\000The number of core clock cycles\000 */
{ 70337 }, /* fsu_clock_off_cycles\000\000clock\000FSU clocking gated off cycle\000event=0x101\000\00000\000FSU clocking gated off cycle\000 */
{ 214887 }, /* wait_cycles\000\000clock\000Wait state cycle\000event=0x110\000\00000\000Wait state cycle\000 */
{ 56959 }, /* exc_dabort\000\000exception\000Exception taken, Data Abort and SError\000event=0x84\000\00000\000Exception taken, Data Abort and SError\000 */
{ 57280 }, /* exc_fiq\000\000exception\000Exception taken, FIQ\000event=0x87\000\00000\000Exception taken, FIQ\000 */
{ 57633 }, /* exc_hvc\000\000exception\000Exception taken, Hypervisor Call\000event=0x8a\000\00000\000Exception taken, Hypervisor Call\000 */
{ 57939 }, /* exc_irq\000\000exception\000Exception taken, IRQ\000event=0x86\000\00000\000Exception taken, IRQ\000 */
{ 58355 }, /* exc_pabort\000\000exception\000Exception taken, Instruction Abort\000event=0x83\000\00000\000Exception taken, Instruction Abort\000 */
{ 58660 }, /* exc_return\000\000exception\000Instruction architecturally executed, condition check pass, exception return\000event=0xa\000\00000\000Instruction architecturally executed, condition check pass, exception return\000 */
{ 59572 }, /* exc_svc\000\000exception\000Exception taken, Supervisor Call\000event=0x82\000\00000\000Exception taken, Supervisor Call\000 */
{ 59896 }, /* exc_taken\000\000exception\000Exception taken\000event=0x9\000\00000\000Exception taken\000 */
{ 60575 }, /* exc_trap_dabort\000\000exception\000Exception taken, Data Abort or SError not taken locally\000event=0x8c\000\00000\000Exception taken, Data Abort or SError not taken locally\000 */
{ 61080 }, /* exc_trap_fiq\000\000exception\000Exception taken, FIQ not taken locally\000event=0x8f\000\00000\000Exception taken, FIQ not taken locally\000 */
{ 62347 }, /* exc_trap_irq\000\000exception\000Exception taken, IRQ not taken locally\000event=0x8e\000\00000\000Exception taken, IRQ not taken locally\000 */
{ 62841 }, /* exc_trap_other\000\000exception\000Exception taken, Other traps not taken locally\000event=0x8d\000\00000\000Exception taken, Other traps not taken locally\000 */
{ 63383 }, /* exc_trap_pabort\000\000exception\000Exception taken, Instruction Abort not taken locally\000event=0x8b\000\00000\000Exception taken, Instruction Abort not taken locally\000 */
{ 64129 }, /* exc_undef\000\000exception\000Exception taken, Other synchronous\000event=0x81\000\00000\000Exception taken, Other synchronous\000 */
{ 20878 }, /* ase_spec\000\000instruction\000Operation speculatively executed, Advanced SIMD instruction\000event=0x74\000\00000\000Operation speculatively executed, Advanced SIMD instruction\000 */
{ 36515 }, /* br_mis_pred_retired\000\000instruction\000Instruction architecturally executed, mispredicted branch\000event=0x22\000\00000\000This event counts any branch counted by BR_RETIRED which is not correctly predicted and causes a pipeline flush\000 */
{ 37522 }, /* br_retired\000\000instruction\000Instruction architecturally executed, branch\000event=0x21\000\00000\000This event counts all branches, taken or not. This excludes exception entries, debug entries and CCFAIL branches\000 */
{ 46006 }, /* cid_write_retired\000\000instruction\000Write to CONTEXTIDR\000event=0xb\000\00000\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000 */
{ 48419 }, /* crypto_spec\000\000instruction\000Operation speculatively executed, Cryptographic instruction\000event=0x77\000\00000\000Operation speculatively executed, Cryptographic instruction\000 */
{ 51117 }, /* dmb_spec\000\000instruction\000Barrier speculatively executed, DMB\000event=0x7e\000\00000\000Barrier speculatively executed, DMB\000 */
{ 52153 }, /* dp_spec\000\000instruction\000Operation speculatively executed, integer data processing\000event=0x73\000\00000\000Operation speculatively executed, integer data processing\000 */
{ 53209 }, /* dsb_spec\000\000instruction\000Barrier speculatively executed, DSB\000event=0x7d\000\00000\000Barrier speculatively executed, DSB\000 */
{ 90819 }, /* inst_retired\000\000instruction\000Instruction architecturally executed\000event=0x8\000\00000\000Instruction architecturally executed\000 */
{ 91235 }, /* inst_spec\000\000instruction\000Operation speculatively executed\000event=0x1b\000\00000\000Operation speculatively executed\000 */
{ 91769 }, /* isb_spec\000\000instruction\000Barrier speculatively executed, ISB\000event=0x7c\000\00000\000Barrier speculatively executed, ISB\000 */
{ 159047 }, /* ld_spec\000\000instruction\000Operation speculatively executed, load\000event=0x70\000\00000\000Operation speculatively executed, load\000 */
{ 159973 }, /* ldst_spec\000\000instruction\000Operation speculatively executed, load or store\000event=0x72\000\00000\000Operation speculatively executed, load or store\000 */
{ 173274 }, /* nop_spec\000\000instruction\000Speculatively executed, NOP\000event=0x100\000\00000\000Operation speculatively executed, NOP\000 */
{ 175115 }, /* pc_write_spec\000\000instruction\000Operation speculatively executed, software change of the PC\000event=0x76\000\00000\000Operation speculatively executed, software change of the PC\000 */
{ 177878 }, /* rc_ld_spec\000\000instruction\000Release consistency operation speculatively executed, Load-Acquire\000event=0x90\000\00000\000Release consistency operation speculatively executed, Load-Acquire\000 */
{ 178251 }, /* rc_st_spec\000\000instruction\000Release consistency operation speculatively executed, Store-Release\000event=0x91\000\00000\000Release consistency operation speculatively executed, Store-Release\000 */
{ 185522 }, /* st_spec\000\000instruction\000Operation speculatively executed, store\000event=0x71\000\00000\000Operation speculatively executed, store\000 */
{ 205937 }, /* sw_incr\000\000instruction\000Software increment\000event=0\000\00000\000Instruction architecturally executed, software increment\000 */
{ 210294 }, /* ttbr_write_retired\000\000instruction\000Instruction architecturally executed, Condition code check pass, write to TTBR\000event=0x1c\000\00000\000Instruction architecturally executed, Condition code check pass, write to TTBR\000 */
{ 214354 }, /* vfp_spec\000\000instruction\000Operation speculatively executed, floating-point instruction\000event=0x75\000\00000\000Operation speculatively executed, floating-point instruction\000 */
{ 159504 }, /* ldrex_spec\000\000intrinsic\000Exclusive operation speculatively executed, LDREX or LDX\000event=0x6c\000\00000\000Exclusive operation speculatively executed, LDREX or LDX\000 */
{ 200622 }, /* strex_fail_spec\000\000intrinsic\000Exclusive operation speculatively executed, STREX or STX fail\000event=0x6e\000\00000\000Exclusive operation speculatively executed, STREX or STX fail\000 */
{ 201193 }, /* strex_pass_spec\000\000intrinsic\000Exclusive operation speculatively executed, STREX or STX pass\000event=0x6d\000\00000\000Exclusive operation speculatively executed, STREX or STX pass\000 */
{ 201745 }, /* strex_spec\000\000intrinsic\000Exclusive operation speculatively executed, STREX or STX\000event=0x6f\000\00000\000Exclusive operation speculatively executed, STREX or STX\000 */
{ 163148 }, /* mem_access\000\000memory\000Data memory access\000event=0x13\000\00000\000Data memory access\000 */
{ 164298 }, /* mem_access_rd\000\000memory\000Data memory access, read\000event=0x66\000\00000\000Data memory access, read\000 */
{ 164956 }, /* mem_access_wr\000\000memory\000Data memory access, write\000event=0x67\000\00000\000Data memory access, write\000 */
{ 166658 }, /* memory_error\000\000memory\000Local memory error\000event=0x1a\000\00000\000This event counts any correctable or uncorrectable memory error (ECC or parity) in the protected core RAMs\000 */
{ 211505 }, /* unaligned_ld_spec\000\000memory\000Unaligned access, read\000event=0x68\000\00000\000Unaligned access, read\000 */
{ 212200 }, /* unaligned_ldst_spec\000\000memory\000Unaligned access\000event=0x6a\000\00000\000Unaligned access\000 */
{ 212637 }, /* unaligned_st_spec\000\000memory\000Unaligned access, write\000event=0x69\000\00000\000Unaligned access, write\000 */
{ 45484 }, /* bx_stall\000\000pipeline\000BX stalled\000event=0x10c\000\00000\000BX Op non-issue\000 */
{ 50809 }, /* decode_stall\000\000pipeline\000Decode starved\000event=0x108\000\00000\000Decode starved for instruction cycle\000 */
{ 51033 }, /* dispatch_stall\000\000pipeline\000Dispatch stalled\000event=0x109\000\00000\000Op dispatch stalled cycle\000 */
{ 71053 }, /* fx_stall\000\000pipeline\000FX stalled\000event=0x10f\000\00000\000FX Op non-issue\000 */
{ 92845 }, /* ixa_stall\000\000pipeline\000IXA stalled\000event=0x10a\000\00000\000IXA Op non-issue\000 */
{ 92910 }, /* ixb_stall\000\000pipeline\000IXB stalled\000event=0x10b\000\00000\000IXB Op non-issue\000 */
{ 162532 }, /* lx_stall\000\000pipeline\000LX stalled\000event=0x10d\000\00000\000LX Op non-issue\000 */
{ 206809 }, /* sx_stall\000\000pipeline\000SX stalled\000event=0x10e\000\00000\000SX Op non-issue\000 */
};

static const struct compact_pmu_event pmu_events__arm_cortex_a34[] = {
{ 35038 }, /* br_indirect_spec\000\000branch\000Branch speculatively executed, indirect branch\000event=0x7a\000\00000\000Branch speculatively executed, indirect branch\000 */
{ 35743 }, /* br_mis_pred\000\000branch\000Mispredicted or not predicted branch speculatively executed\000event=0x10\000\00000\000Mispredicted or not predicted branch speculatively executed\000 */
{ 36925 }, /* br_pred\000\000branch\000Predictable branch speculatively executed\000event=0x12\000\00000\000Predictable branch speculatively executed\000 */
{ 41358 }, /* bus_access\000\000bus\000Attributable Bus access\000event=0x19\000\00000\000Attributable Bus access\000 */
{ 42271 }, /* bus_access_rd\000\000bus\000Bus access read\000event=0x60\000\00000\000Bus access read\000 */
{ 42682 }, /* bus_access_wr\000\000bus\000Bus access write\000event=0x61\000\00000\000Bus access write\000 */
{ 42979 }, /* bus_cycles\000\000bus\000Bus cycle\000event=0x1d\000\00000\000Bus cycle\000 */
{ 47992 }, /* cpu_cycles\000\000bus\000Cycle\000event=0x11\000\00000\000Cycle\000 */
{ 100289 }, /* l1d_cache\000\000cache\000Level 1 data cache access\000event=0x4\000\00000\000Level 1 data cache access\000 */
{ 103129 }, /* l1d_cache_refill\000\000cache\000Level 1 data cache refill\000event=0x3\000\00000\000Level 1 data cache refill\000 */
{ 106891 }, /* l1d_cache_wb\000\000cache\000Attributable Level 1 data cache write-back\000event=0x15\000\00000\000Attributable Level 1 data cache write-back\000 */
{ 111207 }, /* l1d_tlb_refill\000\000cache\000Attributable Level 1 data TLB refill\000event=0x5\000\00000\000Attributable Level 1 data TLB refill\000 */
{ 116626 }, /* l1i_cache\000\000cache\000Attributable Level 1 instruction cache access\000event=0x14\000\00000\000Attributable Level 1 instruction cache access\000 */
{ 118420 }, /* l1i_cache_refill\000\000cache\000Level 1 instruction cache refill\000event=0x1\000\00000\000Level 1 instruction cache refill\000 */
{ 120198 }, /* l1i_tlb_refill\000\000cache\000Attributable Level 1 instruction TLB refill\000event=0x2\000\00000\000Attributable Level 1 instruction TLB refill\000 */
{ 126244 }, /* l2d_cache\000\000cache\000Level 2 data cache access\000event=0x16\000\00000\000Level 2 data cache access\000 */
{ 129862 }, /* l2d_cache_refill\000\000cache\000Level 2 data refill\000event=0x17\000\00000\000Level 2 data refill\000 */
{ 136414 }, /* l2d_cache_wb\000\000cache\000Attributable Level 2 data cache write-back\000event=0x18\000\00000\000Attributable Level 2 data cache write-back\000 */
{ 57280 }, /* exc_fiq\000\000exception\000Exception taken, FIQ\000event=0x87\000\00000\000Exception taken, FIQ\000 */
{ 57939 }, /* exc_irq\000\000exception\000Exception taken, IRQ\000event=0x86\000\00000\000Exception taken, IRQ\000 */
{ 59896 }, /* exc_taken\000\000exception\000Exception taken\000event=0x9\000\00000\000Exception taken\000 */
{ 165870 }, /* memory_error\000\000exception\000Local memory error\000event=0x1a\000\00000\000Local memory error\000 */
{ 30599 }, /* br_immed_retired\000\000instruction\000Instruction architecturally executed, immediate branch\000event=0xd\000\00000\000Instruction architecturally executed, immediate branch\000 */
{ 40483 }, /* br_return_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, procedure return\000event=0xe\000\00000\000Instruction architecturally executed, condition code check pass, procedure return\000 */
{ 45546 }, /* cid_write_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000event=0xb\000\00000\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000 */
{ 58850 }, /* exc_return\000\000instruction\000Instruction architecturally executed, condition check pass, exception return\000event=0xa\000\00000\000Instruction architecturally executed, condition check pass, exception return\000 */
{ 90819 }, /* inst_retired\000\000instruction\000Instruction architecturally executed\000event=0x8\000\00000\000Instruction architecturally executed\000 */
{ 158696 }, /* ld_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, load\000event=0x6\000\00000\000Instruction architecturally executed, condition code check pass, load\000 */
{ 174889 }, /* pc_write_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, software change of the PC\000event=0xc\000\00000\000Instruction architecturally executed, condition code check pass, software change of the PC\000 */
{ 185342 }, /* st_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, store\000event=0x7\000\00000\000Instruction architecturally executed, condition code check pass, store\000 */
{ 205736 }, /* sw_incr\000\000instruction\000Instruction architecturally executed, Condition code check pass, software increment\000event=0\000\00000\000Instruction architecturally executed, Condition code check pass, software increment\000 */
{ 163148 }, /* mem_access\000\000memory\000Data memory access\000event=0x13\000\00000\000Data memory access\000 */
{ 211922 }, /* unaligned_ldst_retired\000\000memory\000Instruction architecturally executed, condition code check pass, unaligned\000event=0xf\000\00000\000Instruction architecturally executed, condition code check pass, unaligned\000 */
};

static const struct compact_pmu_event pmu_events__arm_cortex_a35[] = {
{ 35038 }, /* br_indirect_spec\000\000branch\000Branch speculatively executed, indirect branch\000event=0x7a\000\00000\000Branch speculatively executed, indirect branch\000 */
{ 35743 }, /* br_mis_pred\000\000branch\000Mispredicted or not predicted branch speculatively executed\000event=0x10\000\00000\000Mispredicted or not predicted branch speculatively executed\000 */
{ 36925 }, /* br_pred\000\000branch\000Predictable branch speculatively executed\000event=0x12\000\00000\000Predictable branch speculatively executed\000 */
{ 41358 }, /* bus_access\000\000bus\000Attributable Bus access\000event=0x19\000\00000\000Attributable Bus access\000 */
{ 42271 }, /* bus_access_rd\000\000bus\000Bus access read\000event=0x60\000\00000\000Bus access read\000 */
{ 42682 }, /* bus_access_wr\000\000bus\000Bus access write\000event=0x61\000\00000\000Bus access write\000 */
{ 42979 }, /* bus_cycles\000\000bus\000Bus cycle\000event=0x1d\000\00000\000Bus cycle\000 */
{ 47992 }, /* cpu_cycles\000\000bus\000Cycle\000event=0x11\000\00000\000Cycle\000 */
{ 100289 }, /* l1d_cache\000\000cache\000Level 1 data cache access\000event=0x4\000\00000\000Level 1 data cache access\000 */
{ 103129 }, /* l1d_cache_refill\000\000cache\000Level 1 data cache refill\000event=0x3\000\00000\000Level 1 data cache refill\000 */
{ 106891 }, /* l1d_cache_wb\000\000cache\000Attributable Level 1 data cache write-back\000event=0x15\000\00000\000Attributable Level 1 data cache write-back\000 */
{ 111207 }, /* l1d_tlb_refill\000\000cache\000Attributable Level 1 data TLB refill\000event=0x5\000\00000\000Attributable Level 1 data TLB refill\000 */
{ 116626 }, /* l1i_cache\000\000cache\000Attributable Level 1 instruction cache access\000event=0x14\000\00000\000Attributable Level 1 instruction cache access\000 */
{ 118420 }, /* l1i_cache_refill\000\000cache\000Level 1 instruction cache refill\000event=0x1\000\00000\000Level 1 instruction cache refill\000 */
{ 120198 }, /* l1i_tlb_refill\000\000cache\000Attributable Level 1 instruction TLB refill\000event=0x2\000\00000\000Attributable Level 1 instruction TLB refill\000 */
{ 126244 }, /* l2d_cache\000\000cache\000Level 2 data cache access\000event=0x16\000\00000\000Level 2 data cache access\000 */
{ 129862 }, /* l2d_cache_refill\000\000cache\000Level 2 data refill\000event=0x17\000\00000\000Level 2 data refill\000 */
{ 136414 }, /* l2d_cache_wb\000\000cache\000Attributable Level 2 data cache write-back\000event=0x18\000\00000\000Attributable Level 2 data cache write-back\000 */
{ 57280 }, /* exc_fiq\000\000exception\000Exception taken, FIQ\000event=0x87\000\00000\000Exception taken, FIQ\000 */
{ 57939 }, /* exc_irq\000\000exception\000Exception taken, IRQ\000event=0x86\000\00000\000Exception taken, IRQ\000 */
{ 59896 }, /* exc_taken\000\000exception\000Exception taken\000event=0x9\000\00000\000Exception taken\000 */
{ 165870 }, /* memory_error\000\000exception\000Local memory error\000event=0x1a\000\00000\000Local memory error\000 */
{ 20878 }, /* ase_spec\000\000instruction\000Operation speculatively executed, Advanced SIMD instruction\000event=0x74\000\00000\000Operation speculatively executed, Advanced SIMD instruction\000 */
{ 30599 }, /* br_immed_retired\000\000instruction\000Instruction architecturally executed, immediate branch\000event=0xd\000\00000\000Instruction architecturally executed, immediate branch\000 */
{ 40483 }, /* br_return_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, procedure return\000event=0xe\000\00000\000Instruction architecturally executed, condition code check pass, procedure return\000 */
{ 45546 }, /* cid_write_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000event=0xb\000\00000\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000 */
{ 48419 }, /* crypto_spec\000\000instruction\000Operation speculatively executed, Cryptographic instruction\000event=0x77\000\00000\000Operation speculatively executed, Cryptographic instruction\000 */
{ 52153 }, /* dp_spec\000\000instruction\000Operation speculatively executed, integer data processing\000event=0x73\000\00000\000Operation speculatively executed, integer data processing\000 */
{ 58850 }, /* exc_return\000\000instruction\000Instruction architecturally executed, condition check pass, exception return\000event=0xa\000\00000\000Instruction architecturally executed, condition check pass, exception return\000 */
{ 90819 }, /* inst_retired\000\000instruction\000Instruction architecturally executed\000event=0x8\000\00000\000Instruction architecturally executed\000 */
{ 91235 }, /* inst_spec\000\000instruction\000Operation speculatively executed\000event=0x1b\000\00000\000Operation speculatively executed\000 */
{ 158696 }, /* ld_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, load\000event=0x6\000\00000\000Instruction architecturally executed, condition code check pass, load\000 */
{ 174889 }, /* pc_write_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, software change of the PC\000event=0xc\000\00000\000Instruction architecturally executed, condition code check pass, software change of the PC\000 */
{ 185342 }, /* st_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, store\000event=0x7\000\00000\000Instruction architecturally executed, condition code check pass, store\000 */
{ 205736 }, /* sw_incr\000\000instruction\000Instruction architecturally executed, Condition code check pass, software increment\000event=0\000\00000\000Instruction architecturally executed, Condition code check pass, software increment\000 */
{ 214354 }, /* vfp_spec\000\000instruction\000Operation speculatively executed, floating-point instruction\000event=0x75\000\00000\000Operation speculatively executed, floating-point instruction\000 */
{ 163148 }, /* mem_access\000\000memory\000Data memory access\000event=0x13\000\00000\000Data memory access\000 */
{ 211922 }, /* unaligned_ldst_retired\000\000memory\000Instruction architecturally executed, condition code check pass, unaligned\000event=0xf\000\00000\000Instruction architecturally executed, condition code check pass, unaligned\000 */
};

static const struct compact_pmu_event pmu_events__arm_cortex_a510[] = {
{ 28803 }, /* br_cond_mis_pred\000\000branch\000Conditional branch mispredicted. This event counts when any branch that the conditional predictor can predict is retired and has mispredicted the condition. This event still counts when branch prediction is disabled due to the MMU being off. Conditional indirect branches that correctly predict the condition but mispredict the address do not count\000event=0xcc\000\00000\000Conditional branch mispredicted. This event counts when any branch that the conditional predictor can predict is retired and has mispredicted the condition. This event still counts when branch prediction is disabled due to the MMU being off. Conditional indirect branches that correctly predict the condition but mispredict the address do not count\000 */
{ 29613 }, /* br_cond_pred\000\000branch\000Predicted conditional branch executed. This event counts when any branch that the conditional predictor can predict is retired. This event still counts when branch prediction is disabled due to the Memory Management Unit (MMU) being off\000event=0xc9\000\00000\000Predicted conditional branch executed. This event counts when any branch that the conditional predictor can predict is retired. This event still counts when branch prediction is disabled due to the Memory Management Unit (MMU) being off\000 */
{ 30753 }, /* br_immed_spec\000\000branch\000Branch speculatively executed, immediate branch\000event=0x78\000\00000\000Branch speculatively executed, immediate branch\000 */
{ 31862 }, /* br_indirect_addr_mis_pred\000\000branch\000Indirect branch mispredicted due to address miscompare. This event counts when any indirect branch that the BTAC can predict is retired, was taken, correctly predicted the condition, and has mispredicted the address. This event still counts when branch prediction is disabled due to the MMU being off\000event=0xcb\000\00000\000Indirect branch mispredicted due to address miscompare. This event counts when any indirect branch that the BTAC can predict is retired, was taken, correctly predicted the condition, and has mispredicted the address. This event still counts when branch prediction is disabled due to the MMU being off\000 */
{ 32513 }, /* br_indirect_addr_pred\000\000branch\000Indirect branch with predicted address executed. This event counts when any indirect branch that the BTAC can predict is retired, was taken, and correctly predicted the condition. This event still counts when branch prediction is disabled due to the MMU being off\000event=0xcd\000\00000\000Indirect branch with predicted address executed. This event counts when any indirect branch that the BTAC can predict is retired, was taken, and correctly predicted the condition. This event still counts when branch prediction is disabled due to the MMU being off\000 */
{ 34249 }, /* br_indirect_mis_pred\000\000branch\000Indirect branch mispredicted. This event counts when any indirect branch that the Branch Target Address Cache (BTAC) can predict is retired and has mispredicted either the condition or the address. This event still counts when branch prediction is disabled due to the MMU being off\000event=0xca\000\00000\000Indirect branch mispredicted. This event counts when any indirect branch that the Branch Target Address Cache (BTAC) can predict is retired and has mispredicted either the condition or the address. This event still counts when branch prediction is disabled due to the MMU being off\000 */
{ 35038 }, /* br_indirect_spec\000\000branch\000Branch speculatively executed, indirect branch\000event=0x7a\000\00000\000Branch speculatively executed, indirect branch\000 */
{ 35743 }, /* br_mis_pred\000\000branch\000Mispredicted or not predicted branch speculatively executed\000event=0x10\000\00000\000Mispredicted or not predicted branch speculatively executed\000 */
{ 36925 }, /* br_pred\000\000branch\000Predictable branch speculatively executed\000event=0x12\000\00000\000Predictable branch speculatively executed\000 */
{ 38616 }, /* br_return_addr_mis_pred\000\000branch\000Procedure return mispredicted due to address miscompare. This event counts when any procedure return that the call-return stack can predict is retired, was taken, correctly predicted the condition, and has mispredicted the address. This event still counts when branch prediction is disabled due to the MMU being off\000event=0xcf\000\00000\000Procedure return mispredicted due to address miscompare. This event counts when any procedure return that the call-return stack can predict is retired, was taken, correctly predicted the condition, and has mispredicted the address. This event still counts when branch prediction is disabled due to the MMU being off\000 */
{ 39295 }, /* br_return_addr_pred\000\000branch\000Procedure return with predicted address executed. This event counts when any procedure return that the call-return stack can predict is retired, was taken, and correctly predicted the condition. This event still counts when branch prediction is disabled due to the MMU being off\000event=0xce\000\00000\000Procedure return with predicted address executed. This event counts when any procedure return that the call-return stack can predict is retired, was taken, and correctly predicted the condition. This event still counts when branch prediction is disabled due to the MMU being off\000 */
{ 40692 }, /* br_return_spec\000\000branch\000Branch speculatively executed, procedure return\000event=0x79\000\00000\000Branch speculatively executed, procedure return\000 */
{ 41358 }, /* bus_access\000\000bus\000Attributable Bus access\000event=0x19\000\00000\000Attributable Bus access\000 */
{ 42271 }, /* bus_access_rd\000\000bus\000Bus access read\000event=0x60\000\00000\000Bus access read\000 */
{ 42682 }, /* bus_access_wr\000\000bus\000Bus access write\000event=0x61\000\00000\000Bus access write\000 */
{ 42979 }, /* bus_cycles\000\000bus\000Bus cycle\000event=0x1d\000\00000\000Bus cycle\000 */
{ 47992 }, /* cpu_cycles\000\000bus\000Cycle\000event=0x11\000\00000\000Cycle\000 */
{ 53526 }, /* dtlb_walk\000\000cache\000Access to data TLB causes a translation table walk\000event=0x34\000\00000\000Access to data TLB causes a translation table walk\000 */
{ 92044 }, /* itlb_walk\000\000cache\000Access to instruction TLB that causes a translation table walk\000event=0x35\000\00000\000Access to instruction TLB that causes a translation table walk\000 */
{ 100289 }, /* l1d_cache\000\000cache\000Level 1 data cache access\000event=0x4\000\00000\000Level 1 data cache access\000 */
{ 102276 }, /* l1d_cache_lmiss_rd\000\000cache\000Level 1 data cache long-latency read miss\000event=0x39\000\00000\000Level 1 data cache long-latency read miss.  The counter counts each memory read access counted by L1D_CACHE that incurs additional latency because it returns data from outside the Level 1 data or unified cache of this processing element\000 */
{ 102693 }, /* l1d_cache_rd\000\000cache\000L1D cache access, read\000event=0x40\000\00000\000Attributable Level 1 data cache access, read\000 */
{ 103129 }, /* l1d_cache_refill\000\000cache\000Level 1 data cache refill\000event=0x3\000\00000\000Level 1 data cache refill\000 */
{ 104145 }, /* l1d_cache_refill_inner\000\000cache\000L1D cache refill, inner\000event=0x44\000\00000\000Attributable Level 1 data cache refill, inner\000 */
{ 104573 }, /* l1d_cache_refill_outer\000\000cache\000L1D cache refill, outer\000event=0x45\000\00000\000Attributable Level 1 data cache refill, outer\000 */
{ 105047 }, /* l1d_cache_refill_prefetch\000\000cache\000L1 data cache refill due to prefetch. This event counts any linefills from the prefetcher that cause an allocation into the L1 data cache\000event=0xc2\000\00000\000L1 data cache refill due to prefetch. This event counts any linefills from the prefetcher that cause an allocation into the L1 data cache\000 */
{ 105894 }, /* l1d_cache_refill_rd\000\000cache\000L1D cache refill, read\000event=0x42\000\00000\000Attributable Level 1 data cache refill, read\000 */
{ 106389 }, /* l1d_cache_refill_wr\000\000cache\000L1D cache refill, write\000event=0x43\000\00000\000Attributable Level 1 data cache refill, write\000 */
{ 106891 }, /* l1d_cache_wb\000\000cache\000Attributable Level 1 data cache write-back\000event=0x15\000\00000\000Attributable Level 1 data cache write-back\000 */
{ 108997 }, /* l1d_cache_wr\000\000cache\000L1D cache access, write\000event=0x41\000\00000\000Attributable Level 1 data cache access, write\000 */
{ 109867 }, /* l1d_tlb\000\000cache\000Attributable Level 1 data or unified TLB access\000event=0x25\000\00000\000Attributable Level 1 data or unified TLB access\000 */
{ 111207 }, /* l1d_tlb_refill\000\000cache\000Attributable Level 1 data TLB refill\000event=0x5\000\00000\000Attributable Level 1 data TLB refill\000 */
{ 114462 }, /* l1d_ws_mode\000\000cache\000L1 data cache write streaming mode. This event counts for each cycle where the core is in write streaming mode and is not allocating writes into the L1 data cache\000event=0xc5\000\00000\000L1 data cache write streaming mode. This event counts for each cycle where the core is in write streaming mode and is not allocating writes into the L1 data cache\000 */
{ 115370 }, /* l1d_ws_mode_entry\000\000cache\000L1 data cache entering write streaming mode. This event counts for each entry into write streaming mode\000event=0xc4\000\00000\000L1 data cache entering write streaming mode. This event counts for each entry into write streaming mode\000 */
{ 116626 }, /* l1i_cache\000\000cache\000Attributable Level 1 instruction cache access\000event=0x14\000\00000\000Attributable Level 1 instruction cache access\000 */
{ 117316 }, /* l1i_cache_lmiss\000\000cache\000Level 1 instruction cache long-latency read miss\000event=0x4006\000\00000\000Level 1 instruction cache long-latency read miss.  If the L1I_CACHE_RD event is implemented, the counter counts each access counted by L1I_CACHE_RD that incurs additional latency because it returns instructions from outside of the Level 1 instruction cache of this PE.  If the L1I_CACHE_RD event is not implemented, the counter counts each access counted by L1I_CACHE that incurs additional latency because it returns instructions from outside the Level 1 instruction cache of this PE.  The event indicates to software that the access missed in the Level 1 instruction cache and might have a significant performance impact due to the additional latency, compared to the latency of an access that hits in the Level 1 instruction cache\000 */
{ 118420 }, /* l1i_cache_refill\000\000cache\000Level 1 instruction cache refill\000event=0x1\000\00000\000Level 1 instruction cache refill\000 */
{ 119419 }, /* l1i_tlb\000\000cache\000Attributable Level 1 instruction TLB access\000event=0x26\000\00000\000Attributable Level 1 instruction TLB access\000 */
{ 120198 }, /* l1i_tlb_refill\000\000cache\000Attributable Level 1 instruction TLB refill\000event=0x2\000\00000\000Attributable Level 1 instruction TLB refill\000 */
{ 126244 }, /* l2d_cache\000\000cache\000Level 2 data cache access\000event=0x16\000\00000\000Level 2 data cache access\000 */
{ 126922 }, /* l2d_cache_allocate\000\000cache\000Attributable Level 2 data cache allocation without refill\000event=0x20\000\00000\000Attributable Level 2 data cache allocation without refill\000 */
{ 128191 }, /* l2d_cache_lmiss_rd\000\000cache\000Level 2 data cache long-latency read miss\000event=0x4009\000\00000\000Level 2 data cache long-latency read miss.  The counter counts each memory read access counted by L2D_CACHE that incurs additional latency because it returns data from outside the Level 2 data or unified cache of this processing element.  The event indicates to software that the access missed in the Level 2 data or unified cache and might have a significant performance impact compared to the latency of an access that hits in the Level 2 data or unified cache\000 */
{ 129126 }, /* l2d_cache_rd\000\000cache\000L2D cache access, read\000event=0x50\000\00000\000Attributable Level 2 data cache access, read\000 */
{ 129862 }, /* l2d_cache_refill\000\000cache\000Level 2 data refill\000event=0x17\000\00000\000Level 2 data refill\000 */
{ 130543 }, /* l2d_cache_refill_prefetch\000\000cache\000L2 cache refill due to prefetch. If the complex is configured with a per-complex L2 cache, this event does not count. If the complex is configured without a per-complex L2 cache, this event counts the cluster cache event, as defined by L3D_CACHE_REFILL_PREFETCH. If neither a per-complex cache or a cluster cache is configured, this event is not implemented\000event=0xc1\000\00000\000L2 cache refill due to prefetch. If the complex is configured with a per-complex L2 cache, this event does not count. If the complex is configured without a per-complex L2 cache, this event counts the cluster cache event, as defined by L3D_CACHE_REFILL_PREFETCH. If neither a per-complex cache or a cluster cache is configured, this event is not implemented\000 */
{ 132288 }, /* l2d_cache_refill_rd\000\000cache\000L2D cache refill, read\000event=0x52\000\00000\000Attributable Level 2 data cache refill, read\000 */
{ 132829 }, /* l2d_cache_refill_wr\000\000cache\000L2D cache refill, write\000event=0x53\000\00000\000Attributable Level 2 data cache refill, write\000 */
{ 133376 }, /* l2d_cache_stash_dropped\000\000cache\000L2 cache stash dropped. This event counts on each stash request that is received from the interconnect or the Accelerator Coherency Port (ACP), that targets L2 cache and is dropped due to lack of buffer space to hold the request\000event=0xd6\000\00000\000L2 cache stash dropped. This event counts on each stash request that is received from the interconnect or the Accelerator Coherency Port (ACP), that targets L2 cache and is dropped due to lack of buffer space to hold the request\000 */
{ 136414 }, /* l2d_cache_wb\000\000cache\000Attributable Level 2 data cache write-back\000event=0x18\000\00000\000Attributable Level 2 data cache write-back\000 */
{ 138385 }, /* l2d_cache_wr\000\000cache\000L2D cache access, write\000event=0x51\000\00000\000Attributable Level 2 data cache access, write\000 */
{ 140896 }, /* l2d_s2_tlb\000\000cache\000L2 TLB IPA cache access. This event counts on each access to the IPA cache. If a single translation table walk needs to make multiple accesses to the IPA cache, each access is counted. If stage 2 translation is disabled, this event does not count\000event=0xd4\000\00000\000L2 TLB IPA cache access. This event counts on each access to the IPA cache. If a single translation table walk needs to make multiple accesses to the IPA cache, each access is counted. If stage 2 translation is disabled, this event does not count\000 */
{ 141952 }, /* l2d_s2_tlb_refill\000\000cache\000L2 TLB IPA cache refill. This event counts on each refill of the IPA cache. If a single translation table walk needs to make multiple accesses to the IPA cache, each access that causes a refill is counted. If stage 2 translation is disabled, this event does not count\000event=0xd5\000\00000\000L2 TLB IPA cache refill. This event counts on each refill of the IPA cache. If a single translation table walk needs to make multiple accesses to the IPA cache, each access that causes a refill is counted. If stage 2 translation is disabled, this event does not count\000 */
{ 143389 }, /* l2d_tlb\000\000cache\000Attributable Level 2 data or unified TLB access\000event=0x2f\000\00000\000Attributable Level 2 data or unified TLB access\000 */
{ 144725 }, /* l2d_tlb_refill\000\000cache\000Attributable Level 2 data TLB refill\000event=0x2d\000\00000\000Attributable Level 2 data TLB refill\000 */
{ 146453 }, /* l2d_walk_tlb\000\000cache\000L2 TLB walk cache access. This event does not count if the MMU is disabled\000event=0xd0\000\00000\000L2 TLB walk cache access. This event does not count if the MMU is disabled\000 */
{ 146638 }, /* l2d_walk_tlb_refill\000\000cache\000L2 TLB walk cache refill. This event does not count if the MMU is disabled\000event=0xd1\000\00000\000L2 TLB walk cache refill. This event does not count if the MMU is disabled\000 */
{ 146830 }, /* l2d_ws_mode\000\000cache\000L2 cache write streaming mode. This event counts for each cycle where the core is in write streaming mode and is not allocating writes into the L2 cache\000event=0xc3\000\00000\000L2 cache write streaming mode. This event counts for each cycle where the core is in write streaming mode and is not allocating writes into the L2 cache\000 */
{ 150641 }, /* l3d_cache\000\000cache\000Attributable Level 3 data cache access\000event=0x2b\000\00000\000Attributable Level 3 data cache access\000 */
{ 151872 }, /* l3d_cache_lmiss_rd\000\000cache\000Level 3 data cache long-latency read miss\000event=0x400b\000\00000\000Level 3 data cache long-latency read miss.  The counter counts each memory read access counted by L3D_CACHE that incurs additional latency because it returns data from outside the Level 3 data or unified cache of this processing element.  The event indicates to software that the access missed in the Level 3 data or unified cache and might have a significant performance impact compared to the latency of an access that hits in the Level 3 data or unified cache\000 */
{ 152420 }, /* l3d_cache_rd\000\000cache\000Attributable Level 3 data or unified cache access, read\000event=0xa0\000\00000\000Attributable Level 3 data or unified cache access, read\000 */
{ 154312 }, /* l3d_cache_refill_rd\000\000cache\000Attributable Level 3 data or unified cache refill, read\000event=0xa2\000\00000\000Attributable Level 3 data or unified cache refill, read\000 */
{ 154466 }, /* l3d_ws_mode\000\000cache\000L3 cache write streaming mode. This event counts for each cycle where the core is in write streaming mode and is not allocating writes into the L3 cache\000event=0xc7\000\00000\000L3 cache write streaming mode. This event counts for each cycle where the core is in write streaming mode and is not allocating writes into the L3 cache\000 */
{ 160229 }, /* ll_cache_miss_rd\000\000cache\000Last level cache miss, read\000event=0x37\000\00000\000Last level cache miss, read\000 */
{ 160879 }, /* ll_cache_rd\000\000cache\000Attributable Last level cache memory read\000event=0x36\000\00000\000Attributable Last level cache memory read\000 */
{ 161422 }, /* ll_ws_mode\000\000cache\000Last level cache write streaming mode. This event counts for each cycle where the core is in write streaming mode and is not allocating writes into the system cache\000event=0xc8\000\00000\000Last level cache write streaming mode. This event counts for each cycle where the core is in write streaming mode and is not allocating writes into the system cache\000 */
{ 57280 }, /* exc_fiq\000\000exception\000Exception taken, FIQ\000event=0x87\000\00000\000Exception taken, FIQ\000 */
{ 57939 }, /* exc_irq\000\000exception\000Exception taken, IRQ\000event=0x86\000\00000\000Exception taken, IRQ\000 */
{ 59896 }, /* exc_taken\000\000exception\000Exception taken\000event=0x9\000\00000\000Exception taken\000 */
{ 165870 }, /* memory_error\000\000exception\000Local memory error\000event=0x1a\000\00000\000Local memory error\000 */
{ 20878 }, /* ase_spec\000\000instruction\000Operation speculatively executed, Advanced SIMD instruction\000event=0x74\000\00000\000Operation speculatively executed, Advanced SIMD instruction\000 */
{ 21288 }, /* ase_sve_int16_spec\000\000instruction\000Advanced SIMD and SVE 16-bit integer operations speculatively executed\000event=0x80e7\000\00000\000Advanced SIMD and SVE 16-bit integer operations speculatively executed\000 */
{ 21479 }, /* ase_sve_int32_spec\000\000instruction\000Advanced SIMD and SVE 32-bit integer operations speculatively executed\000event=0x80eb\000\00000\000Advanced SIMD and SVE 32-bit integer operations speculatively executed\000 */
{ 21670 }, /* ase_sve_int64_spec\000\000instruction\000Advanced SIMD and SVE 64-bit integer operations speculatively executed\000event=0x80ef\000\00000\000Advanced SIMD and SVE 64-bit integer operations speculatively executed\000 */
{ 21861 }, /* ase_sve_int8_spec\000\000instruction\000Advanced SIMD and SVE 8-bit integer operations speculatively executed\000event=0x80e3\000\00000\000Advanced SIMD and SVE 8-bit integer operations speculatively executed\000 */
{ 30599 }, /* br_immed_retired\000\000instruction\000Instruction architecturally executed, immediate branch\000event=0xd\000\00000\000Instruction architecturally executed, immediate branch\000 */
{ 36351 }, /* br_mis_pred_retired\000\000instruction\000Instruction architecturally executed, mispredicted branch\000event=0x22\000\00000\000Instruction architecturally executed, mispredicted branch\000 */
{ 37393 }, /* br_retired\000\000instruction\000Instruction architecturally executed, branch\000event=0x21\000\00000\000Instruction architecturally executed, branch\000 */
{ 40483 }, /* br_return_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, procedure return\000event=0xe\000\00000\000Instruction architecturally executed, condition code check pass, procedure return\000 */
{ 45546 }, /* cid_write_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000event=0xb\000\00000\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000 */
{ 48419 }, /* crypto_spec\000\000instruction\000Operation speculatively executed, Cryptographic instruction\000event=0x77\000\00000\000Operation speculatively executed, Cryptographic instruction\000 */
{ 52153 }, /* dp_spec\000\000instruction\000Operation speculatively executed, integer data processing\000event=0x73\000\00000\000Operation speculatively executed, integer data processing\000 */
{ 58850 }, /* exc_return\000\000instruction\000Instruction architecturally executed, condition check pass, exception return\000event=0xa\000\00000\000Instruction architecturally executed, condition check pass, exception return\000 */
{ 67066 }, /* fp_dp_spec\000\000instruction\000Floating-point double-precision operations speculatively executed\000event=0x801c\000\00000\000Floating-point double-precision operations speculatively executed\000 */
{ 68165 }, /* fp_hp_spec\000\000instruction\000Floating-point half-precision operations speculatively executed\000event=0x8014\000\00000\000Floating-point half-precision operations speculatively executed\000 */
{ 69704 }, /* fp_sp_spec\000\000instruction\000Floating-point single-precision operations speculatively executed\000event=0x8018\000\00000\000Floating-point single-precision operations speculatively executed\000 */
{ 90819 }, /* inst_retired\000\000instruction\000Instruction architecturally executed\000event=0x8\000\00000\000Instruction architecturally executed\000 */
{ 91235 }, /* inst_spec\000\000instruction\000Operation speculatively executed\000event=0x1b\000\00000\000Operation speculatively executed\000 */
{ 158696 }, /* ld_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, load\000event=0x6\000\00000\000Instruction architecturally executed, condition code check pass, load\000 */
{ 159047 }, /* ld_spec\000\000instruction\000Operation speculatively executed, load\000event=0x70\000\00000\000Operation speculatively executed, load\000 */
{ 159973 }, /* ldst_spec\000\000instruction\000Operation speculatively executed, load or store\000event=0x72\000\00000\000Operation speculatively executed, load or store\000 */
{ 173450 }, /* op_retired\000\000instruction\000Micro-operation architecturally executed\000event=0x3a\000\00000\000Micro-operation architecturally executed.  The counter counts each operation counted by OP_SPEC that would be executed in a simple sequential execution of the program\000 */
{ 173697 }, /* op_spec\000\000instruction\000Micro-operation speculatively executed\000event=0x3b\000\00000\000Micro-operation speculatively executed.  The counter counts the number of operations executed by the processing element, including those that are executed speculatively and would not be executed in a simple sequential execution of the program\000 */
{ 174889 }, /* pc_write_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, software change of the PC\000event=0xc\000\00000\000Instruction architecturally executed, condition code check pass, software change of the PC\000 */
{ 175115 }, /* pc_write_spec\000\000instruction\000Operation speculatively executed, software change of the PC\000event=0x76\000\00000\000Operation speculatively executed, software change of the PC\000 */
{ 185342 }, /* st_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, store\000event=0x7\000\00000\000Instruction architecturally executed, condition code check pass, store\000 */
{ 185522 }, /* st_spec\000\000instruction\000Operation speculatively executed, store\000event=0x71\000\00000\000Operation speculatively executed, store\000 */
{ 202067 }, /* sve_inst_retired\000\000instruction\000Instruction architecturally executed, SVE\000event=0x8002\000\00000\000Instruction architecturally executed, SVE\000 */
{ 202321 }, /* sve_inst_spec\000\000instruction\000SVE operations speculatively executed\000event=0x8006\000\00000\000SVE operations speculatively executed\000 */
{ 210294 }, /* ttbr_write_retired\000\000instruction\000Instruction architecturally executed, Condition code check pass, write to TTBR\000event=0x1c\000\00000\000Instruction architecturally executed, Condition code check pass, write to TTBR\000 */
{ 214354 }, /* vfp_spec\000\000instruction\000Operation speculatively executed, floating-point instruction\000event=0x75\000\00000\000Operation speculatively executed, floating-point instruction\000 */
{ 155492 }, /* ld_align_lat\000\000memory\000Load with additional latency from alignment\000event=0x4021\000\00000\000Load with additional latency from alignment\000 */
{ 159841 }, /* ldst_align_lat\000\000memory\000Access with additional latency from alignment\000event=0x4020\000\00000\000Access with additional latency from alignment\000 */
{ 163148 }, /* mem_access\000\000memory\000Data memory access\000event=0x13\000\00000\000Data memory access\000 */
{ 163403 }, /* mem_access_checked\000\000memory\000Checked data memory access\000event=0x4024\000\00000\000Checked data memory access\000 */
{ 163501 }, /* mem_access_checked_rd\000\000memory\000Checked data memory access, read\000event=0x4025\000\00000\000Checked data memory access, read\000 */
{ 163614 }, /* mem_access_checked_wr\000\000memory\000Checked data memory access, write\000event=0x4026\000\00000\000Checked data memory access, write\000 */
{ 164298 }, /* mem_access_rd\000\000memory\000Data memory access, read\000event=0x66\000\00000\000Data memory access, read\000 */
{ 164956 }, /* mem_access_wr\000\000memory\000Data memory access, write\000event=0x67\000\00000\000Data memory access, write\000 */
{ 180400 }, /* remote_access_rd\000\000memory\000Attributable memory read access to another socket in a multi-socket system\000event=0x38\000\00000\000Attributable memory read access to another socket in a multi-socket system\000 */
{ 185116 }, /* st_align_lat\000\000memory\000Store with additional latency from alignment\000event=0x4022\000\00000\000Store with additional latency from alignment\000 */
{ 185831 }, /* stall\000\000pipeline\000No operation sent for execution\000event=0x3c\000\00000\000No operation sent for execution.  The counter counts every attributable cycle on which no attributable instruction or operation was sent for execution on this processing element\000 */
{ 186072 }, /* stall_backend\000\000pipeline\000No operation issued due to the backend\000event=0x24\000\00000\000No operation issued due to the backend\000 */
{ 186933 }, /* stall_backend_ilock\000\000pipeline\000No operation issued due to the backend interlock. This event counts every cycle where the issue of an operation is stalled and there is an interlock. Stall cycles due to a stall in the Wr stage are excluded\000event=0xe4\000\00000\000No operation issued due to the backend interlock. This event counts every cycle where the issue of an operation is stalled and there is an interlock. Stall cycles due to a stall in the Wr stage are excluded\000 */
{ 187849 }, /* stall_backend_ilock_addr\000\000pipeline\000No operation issued due to the backend, address interlock. This event counts every cycle where the issue of an operation is stalled and there is an interlock on an address operand. This type of interlock is caused by a load/store instruction waiting for data to calculate the address. Stall cycles due to a stall in the Wr stage are excluded\000event=0xe5\000\00000\000No operation issued due to the backend, address interlock. This event counts every cycle where the issue of an operation is stalled and there is an interlock on an address operand. This type of interlock is caused by a load/store instruction waiting for data to calculate the address. Stall cycles due to a stall in the Wr stage are excluded\000 */
{ 189813 }, /* stall_backend_ilock_vpu\000\000pipeline\000No operation issued due to the backend, interlock, or the Vector Processing Unit (VPU). This event counts every cycle where there is a stall or an interlock that is caused by a VPU instruction. Stall cycles due to a stall in the Wr stage are excluded\000event=0xe6\000\00000\000No operation issued due to the backend, interlock, or the Vector Processing Unit (VPU). This event counts every cycle where there is a stall or an interlock that is caused by a VPU instruction. Stall cycles due to a stall in the Wr stage are excluded\000 */
{ 190364 }, /* stall_backend_ld\000\000pipeline\000No operation issued due to the backend, load. This event counts every cycle where there is a stall in the Wr stage due to a load\000event=0xe7\000\00000\000No operation issued due to the backend, load. This event counts every cycle where there is a stall in the Wr stage due to a load\000 */
{ 190950 }, /* stall_backend_ld_cache\000\000pipeline\000No operation issued due to the backend, load, cache miss. This event counts every cycle where there is a stall in the Wr stage due to a load that is waiting on data. The event counts for stalls that are caused by missing the cache or where the data is Non-cacheable\000event=0xe9\000\00000\000No operation issued due to the backend, load, cache miss. This event counts every cycle where there is a stall in the Wr stage due to a load that is waiting on data. The event counts for stalls that are caused by missing the cache or where the data is Non-cacheable\000 */
{ 191996 }, /* stall_backend_ld_tlb\000\000pipeline\000No operation issued due to the backend, load, TLB miss. This event counts every cycle where there is a stall in the Wr stage due to a load that misses in the L1 TLB\000event=0xea\000\00000\000No operation issued due to the backend, load, TLB miss. This event counts every cycle where there is a stall in the Wr stage due to a load that misses in the L1 TLB\000 */
{ 192744 }, /* stall_backend_mem\000\000pipeline\000Memory stall cycles\000event=0x4005\000\00000\000Memory stall cycles.  The counter counts each cycle counted by STALL_BACKEND where there is a cache miss in the last level of cache within the processing element clock domain\000 */
{ 193134 }, /* stall_backend_st\000\000pipeline\000No operation issued due to the backend, store. This event counts every cycle where there is a stall in the Wr stage due to a store\000event=0xe8\000\00000\000No operation issued due to the backend, store. This event counts every cycle where there is a stall in the Wr stage due to a store\000 */
{ 194128 }, /* stall_backend_st_stb\000\000pipeline\000No operation issued due to the backend, store, Store Buffer (STB) full. This event counts every cycle where there is a stall in the Wr stage because of a store operation that is waiting due to the STB being full\000event=0xeb\000\00000\000No operation issued due to the backend, store, Store Buffer (STB) full. This event counts every cycle where there is a stall in the Wr stage because of a store operation that is waiting due to the STB being full\000 */
{ 194598 }, /* stall_backend_st_tlb\000\000pipeline\000No operation issued due to the backend, store, TLB miss. This event counts every cycle where there is a stall in the Wr stage because of a store operation that has missed in the L1 TLB\000event=0xec\000\00000\000No operation issued due to the backend, store, TLB miss. This event counts every cycle where there is a stall in the Wr stage because of a store operation that has missed in the L1 TLB\000 */
{ 195517 }, /* stall_backend_vpu_hazard\000\000pipeline\000No operation issued due to the backend, VPU hazard. This event counts every cycle where the core stalls due to contention for the VPU with the other core\000event=0xed\000\00000\000No operation issued due to the backend, VPU hazard. This event counts every cycle where the core stalls due to contention for the VPU with the other core\000 */
{ 195875 }, /* stall_frontend\000\000pipeline\000No operation issued because of the frontend\000event=0x23\000\00000\000No operation issued because of the frontend\000 */
{ 196668 }, /* stall_frontend_cache\000\000pipeline\000No operation issued due to the frontend, cache miss. This event counts every cycle that the Data Processing Unit (DPU) instruction queue is empty and there is an instruction cache miss being processed\000event=0xe1\000\00000\000No operation issued due to the frontend, cache miss. This event counts every cycle that the Data Processing Unit (DPU) instruction queue is empty and there is an instruction cache miss being processed\000 */
{ 197834 }, /* stall_frontend_pderr\000\000pipeline\000No operation issued due to the frontend, pre-decode error\000event=0xe3\000\00000\000No operation issued due to the frontend, pre-decode error\000 */
{ 198138 }, /* stall_frontend_tlb\000\000pipeline\000No operation issued due to the frontend, TLB miss. This event counts every cycle that the DPU instruction queue is empty and there is an instruction L1 TLB miss being processed\000event=0xe2\000\00000\000No operation issued due to the frontend, TLB miss. This event counts every cycle that the DPU instruction queue is empty and there is an instruction L1 TLB miss being processed\000 */
{ 199011 }, /* stall_slot\000\000pipeline\000No operation sent for execution on a slot\000event=0x3f\000\00000\000No operation sent for execution on a slot.  The counter counts on each attributable cycle the number of instruction or operation slots that were not occupied by an instruction or operation attributable to the processing element\000 */
{ 199317 }, /* stall_slot_backend\000\000pipeline\000No operation sent for execution on a slot due to the backend\000event=0x3d\000\00000\000No operation sent for execution on a slot due to the backend.  Counts each slot counted by STALL_SLOT where no attributable instruction or operation was sent for execution because the backend is unable to accept it\000 */
{ 199637 }, /* stall_slot_backend_ilock\000\000pipeline\000Issue slot not issued due to interlock. For each cycle, this event counts each dispatch slot that does not issue due to an interlock\000event=0xee\000\00000\000Issue slot not issued due to interlock. For each cycle, this event counts each dispatch slot that does not issue due to an interlock\000 */
{ 199953 }, /* stall_slot_frontend\000\000pipeline\000No operation sent for execution on a slot due to the frontend\000event=0x3e\000\00000\000No operation sent for execution on a slot due to the frontend.  Counts each slot counted by STALL_SLOT where no attributable instruction or operation was sent for execution because there was no attributable instruction or operation available to issue from the processing element from the frontend for the slot\000 */
{ 175860 }, /* pmu_hovfs\000\000pmu\000PMU overflow, counters reserved for use by EL2\000event=0x400f\000\00000\000PMU overflow, counters reserved for use by EL2\000 */
{ 175986 }, /* pmu_ovfs\000\000pmu\000PMU overflow, counters accessible to EL1 and EL0\000event=0x400d\000\00000\000PMU overflow, counters accessible to EL1 and EL0\000 */
{ 48775 }, /* cti_trigout4\000\000trace\000Cross-trigger Interface output trigger 4\000event=0x4018\000\00000\000Cross-trigger Interface output trigger 4\000 */
{ 48894 }, /* cti_trigout5\000\000trace\000Cross-trigger Interface output trigger 5 \000event=0x4019\000\00000\000Cross-trigger Interface output trigger 5 \000 */
{ 49015 }, /* cti_trigout6\000\000trace\000Cross-trigger Interface output trigger 6\000event=0x401a\000\00000\000Cross-trigger Interface output trigger 6\000 */
{ 49134 }, /* cti_trigout7\000\000trace\000Cross-trigger Interface output trigger 7\000event=0x401b\000\00000\000Cross-trigger Interface output trigger 7\000 */
{ 209692 }, /* trb_trig\000\000trace\000Trace buffer Trigger Event\000event=0x400e\000\00000\000Trace buffer Trigger Event\000 */
{ 209779 }, /* trb_wrap\000\000trace\000Trace buffer current write pointer wrapped\000event=0x400c\000\00000\000Trace buffer current write pointer wrapped\000 */
{ 209898 }, /* trcextout0\000\000trace\000PE Trace Unit external output 0\000event=0x4010\000\00000\000PE Trace Unit external output 0\000 */
{ 209997 }, /* trcextout1\000\000trace\000PE Trace Unit external output 1\000event=0x4011\000\00000\000PE Trace Unit external output 1\000 */
{ 210096 }, /* trcextout2\000\000trace\000PE Trace Unit external output 2\000event=0x4012\000\00000\000PE Trace Unit external output 2\000 */
{ 210195 }, /* trcextout3\000\000trace\000PE Trace Unit external output 3\000event=0x4013\000\00000\000PE Trace Unit external output 3\000 */
};

static const struct compact_pmu_event pmu_events__arm_cortex_a53[] = {
{ 27943 }, /* br_cond\000\000branch\000Conditional branch executed\000event=0xc9\000\00000\000\000 */
{ 29541 }, /* br_cond_mispred\000\000branch\000Conditional branch mispredicted\000event=0xcc\000\00000\000\000 */
{ 34857 }, /* br_indirect_mispred\000\000branch\000Indirect branch mispredicted\000event=0xca\000\00000\000\000 */
{ 34930 }, /* br_indirect_mispred_addr\000\000branch\000Indirect branch mispredicted because of address miscompare\000event=0xcb\000\00000\000\000 */
{ 35038 }, /* br_indirect_spec\000\000branch\000Branch speculatively executed, indirect branch\000event=0x7a\000\00000\000Branch speculatively executed, indirect branch\000 */
{ 42271 }, /* bus_access_rd\000\000bus\000Bus access read\000event=0x60\000\00000\000Bus access read\000 */
{ 42682 }, /* bus_access_wr\000\000bus\000Bus access write\000event=0x61\000\00000\000Bus access write\000 */
{ 64544 }, /* ext_snoop\000\000cache\000SCU Snooped data from another CPU for this CPU\000event=0xc8\000\00000\000\000 */
{ 177239 }, /* prefetch_linefill\000\000cache\000Linefill because of prefetch\000event=0xc2\000\00000\000\000 */
{ 177309 }, /* prefetch_linefill_drop\000\000cache\000Instruction Cache Throttle occurred\000event=0xc3\000\00000\000\000 */
{ 179731 }, /* read_alloc\000\000cache\000Read allocate mode\000event=0xc5\000\00000\000\000 */
{ 179784 }, /* read_alloc_enter\000\000cache\000Entering read allocate mode\000event=0xc4\000\00000\000\000 */
{ 64407 }, /* ext_mem_req\000\000memory\000External memory request\000event=0xc0\000\00000\000\000 */
{ 64467 }, /* ext_mem_req_nc\000\000memory\000Non-cacheable external memory request\000event=0xc1\000\00000\000\000 */
{ 57356 }, /* exc_fiq\000\000other\000Exception taken, FIQ\000event=0x87\000\00000\000Exception taken, FIQ\000 */
{ 58015 }, /* exc_irq\000\000other\000Exception taken, IRQ\000event=0x86\000\00000\000Exception taken, IRQ\000 */
{ 101286 }, /* l1d_cache_err\000\000other\000L1 Data Cache (data, tag or dirty) memory error, correctable or non-correctable\000event=0xd1\000\00000\000\000 */
{ 117231 }, /* l1i_cache_err\000\000other\000L1 Instruction Cache (data or tag) memory error\000event=0xd0\000\00000\000\000 */
{ 176687 }, /* pre_decode_err\000\000other\000Pre-decode error\000event=0xc6\000\00000\000\000 */
{ 207181 }, /* tlb_err\000\000other\000TLB memory error\000event=0xd2\000\00000\000\000 */
{ 20607 }, /* agu_dep_stall\000\000pipeline\000Cycles there is an interlock for a load/store instruction waiting for data to calculate the address in the AGU\000event=0xe5\000\00000\000\000 */
{ 50691 }, /* decode_dep_stall\000\000pipeline\000Cycles the DPU IQ is empty and there is a pre-decode error being processed\000event=0xe3\000\00000\000\000 */
{ 72150 }, /* ic_dep_stall\000\000pipeline\000Cycles the DPU IQ is empty and there is an instruction cache miss being processed\000event=0xe1\000\00000\000\000 */
{ 92717 }, /* iutlb_dep_stall\000\000pipeline\000Cycles the DPU IQ is empty and there is an instruction micro-TLB miss being processed\000event=0xe2\000\00000\000\000 */
{ 158464 }, /* ld_dep_stall\000\000pipeline\000Cycles there is a stall in the Wr stage because of a load miss\000event=0xe7\000\00000\000\000 */
{ 174015 }, /* other_interlock_stall\000\000pipeline\000Cycles there is an interlock other than  Advanced SIMD/Floating-point instructions or load/store instruction\000event=0xe4\000\00000\000\000 */
{ 174172 }, /* other_iq_dep_stall\000\000pipeline\000Cycles that the DPU IQ is empty and that is not because of a recent micro-TLB miss, instruction cache miss or pre-decode error\000event=0xe0\000\00000\000\000 */
{ 184257 }, /* simd_dep_stall\000\000pipeline\000Cycles there is an interlock for an Advanced SIMD/Floating-point operation\000event=0xe6\000\00000\000\000 */
{ 185244 }, /* st_dep_stall\000\000pipeline\000Cycles there is a stall in the Wr stage because of a store\000event=0xe8\000\00000\000\000 */
{ 198892 }, /* stall_sb_full\000\000pipeline\000Data Write operation that stalls the pipeline because the store buffer is full\000event=0xc7\000\00000\000\000 */
};

static const struct compact_pmu_event pmu_events__arm_cortex_a55[] = {
{ 28003 }, /* br_cond_mis_pred\000\000branch\000Conditional branch mis-predicted.This event counts when any branch which can be predicted by the conditional predictor is retired, and has mis-predicted the condition. This event still counts when branch prediction is disabled due to the MMU being off. Conditional indirect branches which correctly predicted the condition but mis-predicted on the address do not count this event\000event=0xcc\000\00000\000Conditional branch mis-predicted.This event counts when any branch which can be predicted by the conditional predictor is retired, and has mis-predicted the condition. This event still counts when branch prediction is disabled due to the MMU being off. Conditional indirect branches which correctly predicted the condition but mis-predicted on the address do not count this event\000 */
{ 30123 }, /* br_cond_pred\000\000branch\000Predicted conditional branch executed.This event counts when any branch which can be predicted by the conditional predictor is retired. This event still counts when branch prediction is disabled due to the MMU being off\000event=0xc9\000\00000\000Predicted conditional branch executed.This event counts when any branch which can be predicted by the conditional predictor is retired. This event still counts when branch prediction is disabled due to the MMU being off\000 */
{ 30753 }, /* br_immed_spec\000\000branch\000Branch speculatively executed, immediate branch\000event=0x78\000\00000\000Branch speculatively executed, immediate branch\000 */
{ 31185 }, /* br_indirect_addr_mis_pred\000\000branch\000Indirect branch mis-predicted due to address mis-compare.This event counts when any indirect branch which can be predicted by the BTAC is retired, was taken and correctly predicted the condition, and has mispredicted the address. This event still counts when branch prediction is disabled due to the MMU being off\000event=0xcb\000\00000\000Indirect branch mis-predicted due to address mis-compare.This event counts when any indirect branch which can be predicted by the BTAC is retired, was taken and correctly predicted the condition, and has mispredicted the address. This event still counts when branch prediction is disabled due to the MMU being off\000 */
{ 33086 }, /* br_indirect_addr_pred\000\000branch\000Indirect branch with predicted address executed.This event counts when any indirect branch which can be predicted by the BTAC is retired, was taken and correctly predicted the condition. This event still counts when branch prediction is disabled due to the MMU being off\000event=0xcd\000\00000\000Indirect branch with predicted address executed.This event counts when any indirect branch which can be predicted by the BTAC is retired, was taken and correctly predicted the condition. This event still counts when branch prediction is disabled due to the MMU being off\000 */
{ 33673 }, /* br_indirect_mis_pred\000\000branch\000Indirect branch mis-predicted.This event counts when any indirect branch which can be predicted by the BTAC is retired, and has mispredicted for either the condition or the address. This event still counts when branch prediction is disabled due to the MMU being off\000event=0xca\000\00000\000Indirect branch mis-predicted.This event counts when any indirect branch which can be predicted by the BTAC is retired, and has mispredicted for either the condition or the address. This event still counts when branch prediction is disabled due to the MMU being off\000 */
{ 35038 }, /* br_indirect_spec\000\000branch\000Branch speculatively executed, indirect branch\000event=0x7a\000\00000\000Branch speculatively executed, indirect branch\000 */
{ 35743 }, /* br_mis_pred\000\000branch\000Mispredicted or not predicted branch speculatively executed\000event=0x10\000\00000\000Mispredicted or not predicted branch speculatively executed\000 */
{ 36925 }, /* br_pred\000\000branch\000Predictable branch speculatively executed\000event=0x12\000\00000\000Predictable branch speculatively executed\000 */
{ 37939 }, /* br_return_addr_mis_pred\000\000branch\000Procedure return mis-predicted due to address mis-compare.This event counts when any procedure return which can be predicted by the CRS is retired, was taken and correctly predicted the condition, and has mispredicted the address. This event still counts when branch prediction is disabled due to the MMU being off\000event=0xcf\000\00000\000Procedure return mis-predicted due to address mis-compare.This event counts when any procedure return which can be predicted by the CRS is retired, was taken and correctly predicted the condition, and has mispredicted the address. This event still counts when branch prediction is disabled due to the MMU being off\000 */
{ 39896 }, /* br_return_addr_pred\000\000branch\000Procedure return with predicted address executed.This event counts when any procedure return which can be predicted by the CRS is retired, was taken and correctly predicted the condition. This event still counts when branch prediction is disabled due to the MMU being off\000event=0xce\000\00000\000Procedure return with predicted address executed.This event counts when any procedure return which can be predicted by the CRS is retired, was taken and correctly predicted the condition. This event still counts when branch prediction is disabled due to the MMU being off\000 */
{ 40692 }, /* br_return_spec\000\000branch\000Branch speculatively executed, procedure return\000event=0x79\000\00000\000Branch speculatively executed, procedure return\000 */
{ 41358 }, /* bus_access\000\000bus\000Attributable Bus access\000event=0x19\000\00000\000Attributable Bus access\000 */
{ 42271 }, /* bus_access_rd\000\000bus\000Bus access read\000event=0x60\000\00000\000Bus access read\000 */
{ 42682 }, /* bus_access_wr\000\000bus\000Bus access write\000event=0x61\000\00000\000Bus access write\000 */
{ 42979 }, /* bus_cycles\000\000bus\000Bus cycle\000event=0x1d\000\00000\000Bus cycle\000 */
{ 47992 }, /* cpu_cycles\000\000bus\000Cycle\000event=0x11\000\00000\000Cycle\000 */
{ 53526 }, /* dtlb_walk\000\000cache\000Access to data TLB causes a translation table walk\000event=0x34\000\00000\000Access to data TLB causes a translation table walk\000 */
{ 92044 }, /* itlb_walk\000\000cache\000Access to instruction TLB that causes a translation table walk\000event=0x35\000\00000\000Access to instruction TLB that causes a translation table walk\000 */
{ 100289 }, /* l1d_cache\000\000cache\000Level 1 data cache access\000event=0x4\000\00000\000Level 1 data cache access\000 */
{ 102693 }, /* l1d_cache_rd\000\000cache\000L1D cache access, read\000event=0x40\000\00000\000Attributable Level 1 data cache access, read\000 */
{ 103129 }, /* l1d_cache_refill\000\000cache\000Level 1 data cache refill\000event=0x3\000\00000\000Level 1 data cache refill\000 */
{ 104145 }, /* l1d_cache_refill_inner\000\000cache\000L1D cache refill, inner\000event=0x44\000\00000\000Attributable Level 1 data cache refill, inner\000 */
{ 104573 }, /* l1d_cache_refill_outer\000\000cache\000L1D cache refill, outer\000event=0x45\000\00000\000Attributable Level 1 data cache refill, outer\000 */
{ 105371 }, /* l1d_cache_refill_prefetch\000\000cache\000Level 1 data cache refill due to prefetch. This event counts any linefills from the prefetcher which cause an allocation into the L1 D-cache\000event=0xc2\000\00000\000Level 1 data cache refill due to prefetch. This event counts any linefills from the prefetcher which cause an allocation into the L1 D-cache\000 */
{ 105894 }, /* l1d_cache_refill_rd\000\000cache\000L1D cache refill, read\000event=0x42\000\00000\000Attributable Level 1 data cache refill, read\000 */
{ 106389 }, /* l1d_cache_refill_wr\000\000cache\000L1D cache refill, write\000event=0x43\000\00000\000Attributable Level 1 data cache refill, write\000 */
{ 106891 }, /* l1d_cache_wb\000\000cache\000Attributable Level 1 data cache write-back\000event=0x15\000\00000\000Attributable Level 1 data cache write-back\000 */
{ 108997 }, /* l1d_cache_wr\000\000cache\000L1D cache access, write\000event=0x41\000\00000\000Attributable Level 1 data cache access, write\000 */
{ 109867 }, /* l1d_tlb\000\000cache\000Attributable Level 1 data or unified TLB access\000event=0x25\000\00000\000Attributable Level 1 data or unified TLB access\000 */
{ 111207 }, /* l1d_tlb_refill\000\000cache\000Attributable Level 1 data TLB refill\000event=0x5\000\00000\000Attributable Level 1 data TLB refill\000 */
{ 114912 }, /* l1d_ws_mode\000\000cache\000Level 1 data cache write streaming mode.This event counts for each cycle where the core is in write-streaming mode and not allocating writes into the L1 D-cache\000event=0xc5\000\00000\000Level 1 data cache write streaming mode.This event counts for each cycle where the core is in write-streaming mode and not allocating writes into the L1 D-cache\000 */
{ 115618 }, /* l1d_ws_mode_entry\000\000cache\000Level 1 data cache entering write streaming mode.This event counts for each entry into write-streaming mode\000event=0xc4\000\00000\000Level 1 data cache entering write streaming mode.This event counts for each entry into write-streaming mode\000 */
{ 116626 }, /* l1i_cache\000\000cache\000Attributable Level 1 instruction cache access\000event=0x14\000\00000\000Attributable Level 1 instruction cache access\000 */
{ 118420 }, /* l1i_cache_refill\000\000cache\000Level 1 instruction cache refill\000event=0x1\000\00000\000Level 1 instruction cache refill\000 */
{ 119419 }, /* l1i_tlb\000\000cache\000Attributable Level 1 instruction TLB access\000event=0x26\000\00000\000Attributable Level 1 instruction TLB access\000 */
{ 120198 }, /* l1i_tlb_refill\000\000cache\000Attributable Level 1 instruction TLB refill\000event=0x2\000\00000\000Attributable Level 1 instruction TLB refill\000 */
{ 126244 }, /* l2d_cache\000\000cache\000Level 2 data cache access\000event=0x16\000\00000\000Level 2 data cache access\000 */
{ 126922 }, /* l2d_cache_allocate\000\000cache\000Attributable Level 2 data cache allocation without refill\000event=0x20\000\00000\000Attributable Level 2 data cache allocation without refill\000 */
{ 129126 }, /* l2d_cache_rd\000\000cache\000L2D cache access, read\000event=0x50\000\00000\000Attributable Level 2 data cache access, read\000 */
{ 129862 }, /* l2d_cache_refill\000\000cache\000Level 2 data refill\000event=0x17\000\00000\000Level 2 data refill\000 */
{ 131307 }, /* l2d_cache_refill_prefetch\000\000cache\000Level 2 cache refill due to prefetch. +//0 If the core is configured with a per-core L2 cache: This event does not count. +//0 If the core is configured without a per-core L2 cache: This event counts the cluster cache event, as defined by L3D_CACHE_REFILL_PREFETCH. +//0 If there is neither a per-core cache nor a cluster cache configured, this event is not implemented\000event=0xc1\000\00000\000Level 2 cache refill due to prefetch. +//0 If the core is configured with a per-core L2 cache: This event does not count. +//0 If the core is configured without a per-core L2 cache: This event counts the cluster cache event, as defined by L3D_CACHE_REFILL_PREFETCH. +//0 If there is neither a per-core cache nor a cluster cache configured, this event is not implemented\000 */
{ 132288 }, /* l2d_cache_refill_rd\000\000cache\000L2D cache refill, read\000event=0x52\000\00000\000Attributable Level 2 data cache refill, read\000 */
{ 132829 }, /* l2d_cache_refill_wr\000\000cache\000L2D cache refill, write\000event=0x53\000\00000\000Attributable Level 2 data cache refill, write\000 */
{ 135692 }, /* l2d_cache_stash_dropped\000\000cache\000Level 2 cache stash dropped.This event counts on each stash request received from the interconnect or ACP, that is targeting L2 and gets dropped due to lack of buffer space to hold the request\000event=0xd6\000\00000\000Level 2 cache stash dropped.This event counts on each stash request received from the interconnect or ACP, that is targeting L2 and gets dropped due to lack of buffer space to hold the request\000 */
{ 136414 }, /* l2d_cache_wb\000\000cache\000Attributable Level 2 data cache write-back\000event=0x18\000\00000\000Attributable Level 2 data cache write-back\000 */
{ 138385 }, /* l2d_cache_wr\000\000cache\000L2D cache access, write\000event=0x51\000\00000\000Attributable Level 2 data cache access, write\000 */
{ 138883 }, /* l2d_l2walk_tlb\000\000cache\000Level 2 TLB level-2 walk cache access.This event counts accesses to the level-2 walk cache where the last-level walk cache has missed. The event only counts when the translation regime of the pagewalk uses level 2 descriptors. This event does not count if the MMU is disabled\000event=0xd2\000\00000\000Level 2 TLB level-2 walk cache access.This event counts accesses to the level-2 walk cache where the last-level walk cache has missed. The event only counts when the translation regime of the pagewalk uses level 2 descriptors. This event does not count if the MMU is disabled\000 */
{ 139472 }, /* l2d_l2walk_tlb_refill\000\000cache\000Level 2 TLB level-2 walk cache refill.This event does not count if the MMU is disabled\000event=0xd3\000\00000\000Level 2 TLB level-2 walk cache refill.This event does not count if the MMU is disabled\000 */
{ 139690 }, /* l2d_llwalk_tlb\000\000cache\000Level 2 TLB last-level walk cache access.This event does not count if the MMU is disabled\000event=0xd0\000\00000\000Level 2 TLB last-level walk cache access.This event does not count if the MMU is disabled\000 */
{ 139907 }, /* l2d_llwalk_tlb_refill\000\000cache\000Level 2 TLB last-level walk cache refill.This event does not count if the MMU is disabled\000event=0xd1\000\00000\000Level 2 TLB last-level walk cache refill.This event does not count if the MMU is disabled\000 */
{ 141423 }, /* l2d_s2_tlb\000\000cache\000Level 2 TLB IPA cache access. This event counts on each access to the IPA cache. +//0 If a single pagewalk needs to make multiple accesses to the IPA cache, each access is counted. +//0 If stage 2 translation is disabled, this event does not count\000event=0xd4\000\00000\000Level 2 TLB IPA cache access. This event counts on each access to the IPA cache. +//0 If a single pagewalk needs to make multiple accesses to the IPA cache, each access is counted. +//0 If stage 2 translation is disabled, this event does not count\000 */
{ 142528 }, /* l2d_s2_tlb_refill\000\000cache\000Level 2 TLB IPA cache refill. This event counts on each refill of the IPA cache. +//0 If a single pagewalk needs to make multiple accesses to the IPA cache, each access which causes a refill is counted. +//0 If stage 2 translation is disabled, this event does not count\000event=0xd5\000\00000\000Level 2 TLB IPA cache refill. This event counts on each refill of the IPA cache. +//0 If a single pagewalk needs to make multiple accesses to the IPA cache, each access which causes a refill is counted. +//0 If stage 2 translation is disabled, this event does not count\000 */
{ 143389 }, /* l2d_tlb\000\000cache\000Attributable Level 2 data or unified TLB access\000event=0x2f\000\00000\000Attributable Level 2 data or unified TLB access\000 */
{ 144725 }, /* l2d_tlb_refill\000\000cache\000Attributable Level 2 data TLB refill\000event=0x2d\000\00000\000Attributable Level 2 data TLB refill\000 */
{ 147170 }, /* l2d_ws_mode\000\000cache\000Level 2 cache write streaming mode. This event counts for each cycle where the core is in write-streaming mode and not allocating writes into the L2 cache\000event=0xc3\000\00000\000Level 2 cache write streaming mode. This event counts for each cycle where the core is in write-streaming mode and not allocating writes into the L2 cache\000 */
{ 150641 }, /* l3d_cache\000\000cache\000Attributable Level 3 data cache access\000event=0x2b\000\00000\000Attributable Level 3 data cache access\000 */
{ 151459 }, /* l3d_cache_allocate\000\000cache\000Attributable Level 3 data cache allocation without refill\000event=0x29\000\00000\000Attributable Level 3 data cache allocation without refill\000 */
{ 152420 }, /* l3d_cache_rd\000\000cache\000Attributable Level 3 data or unified cache access, read\000event=0xa0\000\00000\000Attributable Level 3 data or unified cache access, read\000 */
{ 152665 }, /* l3d_cache_refill\000\000cache\000Attributable Level 3 data cache refill\000event=0x2a\000\00000\000Attributable Level 3 data cache refill\000 */
{ 153272 }, /* l3d_cache_refill_prefetch\000\000cache\000Level 3 cache refill due to prefetch. This event counts any linefills from the hardware prefetcher which cause an allocation into the L3 cache. Note It might not be possible to both distinguish hardware vs software prefetches and also which prefetches cause an allocation. If so, only hardware prefetches should be counted, regardless of whether they allocate. If either the core is configured without a per-core L2 or the cluster is configured without an L3 cache, this event is not implemented\000event=0xc0\000\00000\000Level 3 cache refill due to prefetch. This event counts any linefills from the hardware prefetcher which cause an allocation into the L3 cache. Note It might not be possible to both distinguish hardware vs software prefetches and also which prefetches cause an allocation. If so, only hardware prefetches should be counted, regardless of whether they allocate. If either the core is configured without a per-core L2 or the cluster is configured without an L3 cache, this event is not implemented\000 */
{ 154312 }, /* l3d_cache_refill_rd\000\000cache\000Attributable Level 3 data or unified cache refill, read\000event=0xa2\000\00000\000Attributable Level 3 data or unified cache refill, read\000 */
{ 155150 }, /* l3d_ws_mode\000\000cache\000Level 3 cache write streaming mode.This event counts for each cycle where the core is in write-streaming mode and not allocating writes into the L3 cache\000event=0xc7\000\00000\000Level 3 cache write streaming mode.This event counts for each cycle where the core is in write-streaming mode and not allocating writes into the L3 cache\000 */
{ 160229 }, /* ll_cache_miss_rd\000\000cache\000Last level cache miss, read\000event=0x37\000\00000\000Last level cache miss, read\000 */
{ 160879 }, /* ll_cache_rd\000\000cache\000Attributable Last level cache memory read\000event=0x36\000\00000\000Attributable Last level cache memory read\000 */
{ 57280 }, /* exc_fiq\000\000exception\000Exception taken, FIQ\000event=0x87\000\00000\000Exception taken, FIQ\000 */
{ 57939 }, /* exc_irq\000\000exception\000Exception taken, IRQ\000event=0x86\000\00000\000Exception taken, IRQ\000 */
{ 59896 }, /* exc_taken\000\000exception\000Exception taken\000event=0x9\000\00000\000Exception taken\000 */
{ 165870 }, /* memory_error\000\000exception\000Local memory error\000event=0x1a\000\00000\000Local memory error\000 */
{ 176984 }, /* predecode_error\000\000exception\000Predecode error\000event=0xc6\000\00000\000Predecode error\000 */
{ 20878 }, /* ase_spec\000\000instruction\000Operation speculatively executed, Advanced SIMD instruction\000event=0x74\000\00000\000Operation speculatively executed, Advanced SIMD instruction\000 */
{ 30599 }, /* br_immed_retired\000\000instruction\000Instruction architecturally executed, immediate branch\000event=0xd\000\00000\000Instruction architecturally executed, immediate branch\000 */
{ 36351 }, /* br_mis_pred_retired\000\000instruction\000Instruction architecturally executed, mispredicted branch\000event=0x22\000\00000\000Instruction architecturally executed, mispredicted branch\000 */
{ 37393 }, /* br_retired\000\000instruction\000Instruction architecturally executed, branch\000event=0x21\000\00000\000Instruction architecturally executed, branch\000 */
{ 40483 }, /* br_return_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, procedure return\000event=0xe\000\00000\000Instruction architecturally executed, condition code check pass, procedure return\000 */
{ 45546 }, /* cid_write_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000event=0xb\000\00000\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000 */
{ 48419 }, /* crypto_spec\000\000instruction\000Operation speculatively executed, Cryptographic instruction\000event=0x77\000\00000\000Operation speculatively executed, Cryptographic instruction\000 */
{ 52153 }, /* dp_spec\000\000instruction\000Operation speculatively executed, integer data processing\000event=0x73\000\00000\000Operation speculatively executed, integer data processing\000 */
{ 58850 }, /* exc_return\000\000instruction\000Instruction architecturally executed, condition check pass, exception return\000event=0xa\000\00000\000Instruction architecturally executed, condition check pass, exception return\000 */
{ 90819 }, /* inst_retired\000\000instruction\000Instruction architecturally executed\000event=0x8\000\00000\000Instruction architecturally executed\000 */
{ 91235 }, /* inst_spec\000\000instruction\000Operation speculatively executed\000event=0x1b\000\00000\000Operation speculatively executed\000 */
{ 158696 }, /* ld_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, load\000event=0x6\000\00000\000Instruction architecturally executed, condition code check pass, load\000 */
{ 159047 }, /* ld_spec\000\000instruction\000Operation speculatively executed, load\000event=0x70\000\00000\000Operation speculatively executed, load\000 */
{ 159973 }, /* ldst_spec\000\000instruction\000Operation speculatively executed, load or store\000event=0x72\000\00000\000Operation speculatively executed, load or store\000 */
{ 174889 }, /* pc_write_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, software change of the PC\000event=0xc\000\00000\000Instruction architecturally executed, condition code check pass, software change of the PC\000 */
{ 175115 }, /* pc_write_spec\000\000instruction\000Operation speculatively executed, software change of the PC\000event=0x76\000\00000\000Operation speculatively executed, software change of the PC\000 */
{ 185342 }, /* st_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, store\000event=0x7\000\00000\000Instruction architecturally executed, condition code check pass, store\000 */
{ 185522 }, /* st_spec\000\000instruction\000Operation speculatively executed, store\000event=0x71\000\00000\000Operation speculatively executed, store\000 */
{ 205736 }, /* sw_incr\000\000instruction\000Instruction architecturally executed, Condition code check pass, software increment\000event=0\000\00000\000Instruction architecturally executed, Condition code check pass, software increment\000 */
{ 210294 }, /* ttbr_write_retired\000\000instruction\000Instruction architecturally executed, Condition code check pass, write to TTBR\000event=0x1c\000\00000\000Instruction architecturally executed, Condition code check pass, write to TTBR\000 */
{ 214354 }, /* vfp_spec\000\000instruction\000Operation speculatively executed, floating-point instruction\000event=0x75\000\00000\000Operation speculatively executed, floating-point instruction\000 */
{ 163148 }, /* mem_access\000\000memory\000Data memory access\000event=0x13\000\00000\000Data memory access\000 */
{ 164298 }, /* mem_access_rd\000\000memory\000Data memory access, read\000event=0x66\000\00000\000Data memory access, read\000 */
{ 164956 }, /* mem_access_wr\000\000memory\000Data memory access, write\000event=0x67\000\00000\000Data memory access, write\000 */
{ 180400 }, /* remote_access_rd\000\000memory\000Attributable memory read access to another socket in a multi-socket system\000event=0x38\000\00000\000Attributable memory read access to another socket in a multi-socket system\000 */
{ 211922 }, /* unaligned_ldst_retired\000\000memory\000Instruction architecturally executed, condition code check pass, unaligned\000event=0xf\000\00000\000Instruction architecturally executed, condition code check pass, unaligned\000 */
{ 186072 }, /* stall_backend\000\000pipeline\000No operation issued due to the backend\000event=0x24\000\00000\000No operation issued due to the backend\000 */
{ 187392 }, /* stall_backend_ilock\000\000pipeline\000No operation issued due to the backend interlock.This event counts every cycle that issue is stalled and there is an interlock. Stall cycles due to a stall in Wr (typically awaiting load data) are excluded\000event=0xe4\000\00000\000No operation issued due to the backend interlock.This event counts every cycle that issue is stalled and there is an interlock. Stall cycles due to a stall in Wr (typically awaiting load data) are excluded\000 */
{ 188583 }, /* stall_backend_ilock_agu\000\000pipeline\000No operation issued due to the backend, interlock, AGU.This event counts every cycle that issue is stalled and there is an interlock that is due to a load/store instruction waiting for data to calculate the address in the AGU. Stall cycles due to a stall in Wr (typically awaiting load data) are excluded\000event=0xe5\000\00000\000No operation issued due to the backend, interlock, AGU.This event counts every cycle that issue is stalled and there is an interlock that is due to a load/store instruction waiting for data to calculate the address in the AGU. Stall cycles due to a stall in Wr (typically awaiting load data) are excluded\000 */
{ 189242 }, /* stall_backend_ilock_fpu\000\000pipeline\000No operation issued due to the backend, interlock, FPU.This event counts every cycle that issue is stalled and there is an interlock that is due to an FPU/NEON instruction. Stall cycles due to a stall in the Wr stage (typically awaiting load data) are excluded\000event=0xe6\000\00000\000No operation issued due to the backend, interlock, FPU.This event counts every cycle that issue is stalled and there is an interlock that is due to an FPU/NEON instruction. Stall cycles due to a stall in the Wr stage (typically awaiting load data) are excluded\000 */
{ 190664 }, /* stall_backend_ld\000\000pipeline\000No operation issued due to the backend, load.This event counts every cycle there is a stall in the Wr stage due to a load\000event=0xe7\000\00000\000No operation issued due to the backend, load.This event counts every cycle there is a stall in the Wr stage due to a load\000 */
{ 191530 }, /* stall_backend_ld_cache\000\000pipeline\000No operation issued due to the backend, load, cache miss.This event counts every cycle there is a stall in the Wr stage due to a load which is waiting on data (due to missing the cache or being non-cacheable)\000event=0xe9\000\00000\000No operation issued due to the backend, load, cache miss.This event counts every cycle there is a stall in the Wr stage due to a load which is waiting on data (due to missing the cache or being non-cacheable)\000 */
{ 192372 }, /* stall_backend_ld_tlb\000\000pipeline\000No operation issued due to the backend, load, TLB miss.This event counts every cycle there is a stall in the Wr stage due to a load which has missed in the L1 TLB\000event=0xea\000\00000\000No operation issued due to the backend, load, TLB miss.This event counts every cycle there is a stall in the Wr stage due to a load which has missed in the L1 TLB\000 */
{ 193438 }, /* stall_backend_st\000\000pipeline\000No operation issued due to the backend, store.This event counts every cycle there is a stall in the Wr stage due to a store\000event=0xe8\000\00000\000No operation issued due to the backend, store.This event counts every cycle there is a stall in the Wr stage due to a store\000 */
{ 193728 }, /* stall_backend_st_stb\000\000pipeline\000No operation issued due to the backend, store, STB full.This event counts every cycle there is a stall in the Wr stage due to a store which is waiting due to the STB being full\000event=0xeb\000\00000\000No operation issued due to the backend, store, STB full.This event counts every cycle there is a stall in the Wr stage due to a store which is waiting due to the STB being full\000 */
{ 195014 }, /* stall_backend_st_tlb\000\000pipeline\000No operation issued due to the backend, store, TLB miss.This event counts every cycle there is a stall in the Wr stage due to a store which has missed in the L1 TLB\000event=0xec\000\00000\000No operation issued due to the backend, store, TLB miss.This event counts every cycle there is a stall in the Wr stage due to a store which has missed in the L1 TLB\000 */
{ 195875 }, /* stall_frontend\000\000pipeline\000No operation issued because of the frontend\000event=0x23\000\00000\000No operation issued because of the frontend\000 */
{ 197116 }, /* stall_frontend_cache\000\000pipeline\000No operation issued due to the frontend, cache miss.This event counts every cycle the DPU IQ is empty and there is an instruction cache miss being processed\000event=0xe1\000\00000\000No operation issued due to the frontend, cache miss.This event counts every cycle the DPU IQ is empty and there is an instruction cache miss being processed\000 */
{ 197476 }, /* stall_frontend_pderr\000\000pipeline\000No operation issued due to the frontend, pre-decode error.This event counts every cycle the DPU IQ is empty and there is a pre-decode error being processed\000event=0xe3\000\00000\000No operation issued due to the frontend, pre-decode error.This event counts every cycle the DPU IQ is empty and there is a pre-decode error being processed\000 */
{ 198536 }, /* stall_frontend_tlb\000\000pipeline\000No operation issued due to the frontend, TLB miss.This event counts every cycle the DPU IQ is empty and there is an instruction L1 TLB miss being processed\000event=0xe2\000\00000\000No operation issued due to the frontend, TLB miss.This event counts every cycle the DPU IQ is empty and there is an instruction L1 TLB miss being processed\000 */
};

static const struct compact_pmu_event pmu_events__arm_cortex_a57_a72[] = {
{ 30753 }, /* br_immed_spec\000\000branch\000Branch speculatively executed, immediate branch\000event=0x78\000\00000\000Branch speculatively executed, immediate branch\000 */
{ 35038 }, /* br_indirect_spec\000\000branch\000Branch speculatively executed, indirect branch\000event=0x7a\000\00000\000Branch speculatively executed, indirect branch\000 */
{ 35743 }, /* br_mis_pred\000\000branch\000Mispredicted or not predicted branch speculatively executed\000event=0x10\000\00000\000Mispredicted or not predicted branch speculatively executed\000 */
{ 36925 }, /* br_pred\000\000branch\000Predictable branch speculatively executed\000event=0x12\000\00000\000Predictable branch speculatively executed\000 */
{ 40692 }, /* br_return_spec\000\000branch\000Branch speculatively executed, procedure return\000event=0x79\000\00000\000Branch speculatively executed, procedure return\000 */
{ 41358 }, /* bus_access\000\000bus\000Attributable Bus access\000event=0x19\000\00000\000Attributable Bus access\000 */
{ 41979 }, /* bus_access_normal\000\000bus\000Bus access, Normal\000event=0x64\000\00000\000Bus access, Normal\000 */
{ 42055 }, /* bus_access_not_shared\000\000bus\000Bus access, not Normal, Cacheable, Shareable\000event=0x63\000\00000\000Bus access, not Normal, Cacheable, Shareable\000 */
{ 42187 }, /* bus_access_periph\000\000bus\000Bus access, peripheral\000event=0x65\000\00000\000Bus access, peripheral\000 */
{ 42271 }, /* bus_access_rd\000\000bus\000Bus access read\000event=0x60\000\00000\000Bus access read\000 */
{ 42562 }, /* bus_access_shared\000\000bus\000Bus access, Normal, Cacheable, Shareable\000event=0x62\000\00000\000Bus access, Normal, Cacheable, Shareable\000 */
{ 42682 }, /* bus_access_wr\000\000bus\000Bus access write\000event=0x61\000\00000\000Bus access write\000 */
{ 42979 }, /* bus_cycles\000\000bus\000Bus cycle\000event=0x1d\000\00000\000Bus cycle\000 */
{ 47992 }, /* cpu_cycles\000\000bus\000Cycle\000event=0x11\000\00000\000Cycle\000 */
{ 100289 }, /* l1d_cache\000\000cache\000Level 1 data cache access\000event=0x4\000\00000\000Level 1 data cache access\000 */
{ 101403 }, /* l1d_cache_inval\000\000cache\000L1D cache invalidate\000event=0x48\000\00000\000Attributable Level 1 data cache invalidate\000 */
{ 102693 }, /* l1d_cache_rd\000\000cache\000L1D cache access, read\000event=0x40\000\00000\000Attributable Level 1 data cache access, read\000 */
{ 103129 }, /* l1d_cache_refill\000\000cache\000Level 1 data cache refill\000event=0x3\000\00000\000Level 1 data cache refill\000 */
{ 105894 }, /* l1d_cache_refill_rd\000\000cache\000L1D cache refill, read\000event=0x42\000\00000\000Attributable Level 1 data cache refill, read\000 */
{ 106389 }, /* l1d_cache_refill_wr\000\000cache\000L1D cache refill, write\000event=0x43\000\00000\000Attributable Level 1 data cache refill, write\000 */
{ 106891 }, /* l1d_cache_wb\000\000cache\000Attributable Level 1 data cache write-back\000event=0x15\000\00000\000Attributable Level 1 data cache write-back\000 */
{ 107957 }, /* l1d_cache_wb_clean\000\000cache\000L1D cache Write-Back, cleaning and coherency\000event=0x47\000\00000\000Level 1 data cache Write-Back, cleaning and coherency\000 */
{ 108495 }, /* l1d_cache_wb_victim\000\000cache\000L1D cache Write-Back, victim\000event=0x46\000\00000\000Attributable Level 1 data cache Write-Back, victim\000 */
{ 108997 }, /* l1d_cache_wr\000\000cache\000L1D cache access, write\000event=0x41\000\00000\000Attributable Level 1 data cache access, write\000 */
{ 111207 }, /* l1d_tlb_refill\000\000cache\000Attributable Level 1 data TLB refill\000event=0x5\000\00000\000Attributable Level 1 data TLB refill\000 */
{ 112199 }, /* l1d_tlb_refill_rd\000\000cache\000L1D tlb refill, read\000event=0x4c\000\00000\000Attributable Level 1 data TLB refill, read\000 */
{ 113093 }, /* l1d_tlb_refill_wr\000\000cache\000L1D tlb refill, write\000event=0x4d\000\00000\000Attributable Level 1 data TLB refill, write\000 */
{ 116626 }, /* l1i_cache\000\000cache\000Attributable Level 1 instruction cache access\000event=0x14\000\00000\000Attributable Level 1 instruction cache access\000 */
{ 118420 }, /* l1i_cache_refill\000\000cache\000Level 1 instruction cache refill\000event=0x1\000\00000\000Level 1 instruction cache refill\000 */
{ 120198 }, /* l1i_tlb_refill\000\000cache\000Attributable Level 1 instruction TLB refill\000event=0x2\000\00000\000Attributable Level 1 instruction TLB refill\000 */
{ 126244 }, /* l2d_cache\000\000cache\000Level 2 data cache access\000event=0x16\000\00000\000Level 2 data cache access\000 */
{ 127466 }, /* l2d_cache_inval\000\000cache\000L2D cache invalidate\000event=0x58\000\00000\000Attributable Level 2 data cache invalidate\000 */
{ 129126 }, /* l2d_cache_rd\000\000cache\000L2D cache access, read\000event=0x50\000\00000\000Attributable Level 2 data cache access, read\000 */
{ 129862 }, /* l2d_cache_refill\000\000cache\000Level 2 data refill\000event=0x17\000\00000\000Level 2 data refill\000 */
{ 132288 }, /* l2d_cache_refill_rd\000\000cache\000L2D cache refill, read\000event=0x52\000\00000\000Attributable Level 2 data cache refill, read\000 */
{ 132829 }, /* l2d_cache_refill_wr\000\000cache\000L2D cache refill, write\000event=0x53\000\00000\000Attributable Level 2 data cache refill, write\000 */
{ 136414 }, /* l2d_cache_wb\000\000cache\000Attributable Level 2 data cache write-back\000event=0x18\000\00000\000Attributable Level 2 data cache write-back\000 */
{ 137387 }, /* l2d_cache_wb_clean\000\000cache\000L2D cache Write-Back, cleaning and coherency\000event=0x57\000\00000\000Level 2 data cache Write-Back, cleaning and coherency\000 */
{ 137968 }, /* l2d_cache_wb_victim\000\000cache\000L2D cache Write-Back, victim\000event=0x56\000\00000\000Attributable Level 2 data cache Write-Back, victim\000 */
{ 138385 }, /* l2d_cache_wr\000\000cache\000L2D cache access, write\000event=0x51\000\00000\000Attributable Level 2 data cache access, write\000 */
{ 56959 }, /* exc_dabort\000\000exception\000Exception taken, Data Abort and SError\000event=0x84\000\00000\000Exception taken, Data Abort and SError\000 */
{ 57280 }, /* exc_fiq\000\000exception\000Exception taken, FIQ\000event=0x87\000\00000\000Exception taken, FIQ\000 */
{ 57633 }, /* exc_hvc\000\000exception\000Exception taken, Hypervisor Call\000event=0x8a\000\00000\000Exception taken, Hypervisor Call\000 */
{ 57939 }, /* exc_irq\000\000exception\000Exception taken, IRQ\000event=0x86\000\00000\000Exception taken, IRQ\000 */
{ 58355 }, /* exc_pabort\000\000exception\000Exception taken, Instruction Abort\000event=0x83\000\00000\000Exception taken, Instruction Abort\000 */
{ 59258 }, /* exc_smc\000\000exception\000Exception taken, Secure Monitor Call\000event=0x88\000\00000\000Exception taken, Secure Monitor Call\000 */
{ 59572 }, /* exc_svc\000\000exception\000Exception taken, Supervisor Call\000event=0x82\000\00000\000Exception taken, Supervisor Call\000 */
{ 59896 }, /* exc_taken\000\000exception\000Exception taken\000event=0x9\000\00000\000Exception taken\000 */
{ 60575 }, /* exc_trap_dabort\000\000exception\000Exception taken, Data Abort or SError not taken locally\000event=0x8c\000\00000\000Exception taken, Data Abort or SError not taken locally\000 */
{ 61080 }, /* exc_trap_fiq\000\000exception\000Exception taken, FIQ not taken locally\000event=0x8f\000\00000\000Exception taken, FIQ not taken locally\000 */
{ 62347 }, /* exc_trap_irq\000\000exception\000Exception taken, IRQ not taken locally\000event=0x8e\000\00000\000Exception taken, IRQ not taken locally\000 */
{ 62841 }, /* exc_trap_other\000\000exception\000Exception taken, Other traps not taken locally\000event=0x8d\000\00000\000Exception taken, Other traps not taken locally\000 */
{ 63383 }, /* exc_trap_pabort\000\000exception\000Exception taken, Instruction Abort not taken locally\000event=0x8b\000\00000\000Exception taken, Instruction Abort not taken locally\000 */
{ 64129 }, /* exc_undef\000\000exception\000Exception taken, Other synchronous\000event=0x81\000\00000\000Exception taken, Other synchronous\000 */
{ 165870 }, /* memory_error\000\000exception\000Local memory error\000event=0x1a\000\00000\000Local memory error\000 */
{ 20878 }, /* ase_spec\000\000instruction\000Operation speculatively executed, Advanced SIMD instruction\000event=0x74\000\00000\000Operation speculatively executed, Advanced SIMD instruction\000 */
{ 45546 }, /* cid_write_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000event=0xb\000\00000\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000 */
{ 48419 }, /* crypto_spec\000\000instruction\000Operation speculatively executed, Cryptographic instruction\000event=0x77\000\00000\000Operation speculatively executed, Cryptographic instruction\000 */
{ 51117 }, /* dmb_spec\000\000instruction\000Barrier speculatively executed, DMB\000event=0x7e\000\00000\000Barrier speculatively executed, DMB\000 */
{ 52153 }, /* dp_spec\000\000instruction\000Operation speculatively executed, integer data processing\000event=0x73\000\00000\000Operation speculatively executed, integer data processing\000 */
{ 53209 }, /* dsb_spec\000\000instruction\000Barrier speculatively executed, DSB\000event=0x7d\000\00000\000Barrier speculatively executed, DSB\000 */
{ 58850 }, /* exc_return\000\000instruction\000Instruction architecturally executed, condition check pass, exception return\000event=0xa\000\00000\000Instruction architecturally executed, condition check pass, exception return\000 */
{ 90819 }, /* inst_retired\000\000instruction\000Instruction architecturally executed\000event=0x8\000\00000\000Instruction architecturally executed\000 */
{ 91235 }, /* inst_spec\000\000instruction\000Operation speculatively executed\000event=0x1b\000\00000\000Operation speculatively executed\000 */
{ 91769 }, /* isb_spec\000\000instruction\000Barrier speculatively executed, ISB\000event=0x7c\000\00000\000Barrier speculatively executed, ISB\000 */
{ 159047 }, /* ld_spec\000\000instruction\000Operation speculatively executed, load\000event=0x70\000\00000\000Operation speculatively executed, load\000 */
{ 159351 }, /* ldrex_spec\000\000instruction\000Exclusive operation speculatively executed, LDREX or LDX\000event=0x6c\000\00000\000Exclusive operation speculatively executed, LDREX or LDX\000 */
{ 159973 }, /* ldst_spec\000\000instruction\000Operation speculatively executed, load or store\000event=0x72\000\00000\000Operation speculatively executed, load or store\000 */
{ 175115 }, /* pc_write_spec\000\000instruction\000Operation speculatively executed, software change of the PC\000event=0x76\000\00000\000Operation speculatively executed, software change of the PC\000 */
{ 177878 }, /* rc_ld_spec\000\000instruction\000Release consistency operation speculatively executed, Load-Acquire\000event=0x90\000\00000\000Release consistency operation speculatively executed, Load-Acquire\000 */
{ 178251 }, /* rc_st_spec\000\000instruction\000Release consistency operation speculatively executed, Store-Release\000event=0x91\000\00000\000Release consistency operation speculatively executed, Store-Release\000 */
{ 185522 }, /* st_spec\000\000instruction\000Operation speculatively executed, store\000event=0x71\000\00000\000Operation speculatively executed, store\000 */
{ 200454 }, /* strex_fail_spec\000\000instruction\000Exclusive operation speculatively executed, STREX or STX fail\000event=0x6e\000\00000\000Exclusive operation speculatively executed, STREX or STX fail\000 */
{ 201025 }, /* strex_pass_spec\000\000instruction\000Exclusive operation speculatively executed, STREX or STX pass\000event=0x6d\000\00000\000Exclusive operation speculatively executed, STREX or STX pass\000 */
{ 205736 }, /* sw_incr\000\000instruction\000Instruction architecturally executed, Condition code check pass, software increment\000event=0\000\00000\000Instruction architecturally executed, Condition code check pass, software increment\000 */
{ 210294 }, /* ttbr_write_retired\000\000instruction\000Instruction architecturally executed, Condition code check pass, write to TTBR\000event=0x1c\000\00000\000Instruction architecturally executed, Condition code check pass, write to TTBR\000 */
{ 214354 }, /* vfp_spec\000\000instruction\000Operation speculatively executed, floating-point instruction\000event=0x75\000\00000\000Operation speculatively executed, floating-point instruction\000 */
{ 163148 }, /* mem_access\000\000memory\000Data memory access\000event=0x13\000\00000\000Data memory access\000 */
{ 164298 }, /* mem_access_rd\000\000memory\000Data memory access, read\000event=0x66\000\00000\000Data memory access, read\000 */
{ 164956 }, /* mem_access_wr\000\000memory\000Data memory access, write\000event=0x67\000\00000\000Data memory access, write\000 */
{ 211505 }, /* unaligned_ld_spec\000\000memory\000Unaligned access, read\000event=0x68\000\00000\000Unaligned access, read\000 */
{ 212200 }, /* unaligned_ldst_spec\000\000memory\000Unaligned access\000event=0x6a\000\00000\000Unaligned access\000 */
{ 212637 }, /* unaligned_st_spec\000\000memory\000Unaligned access, write\000event=0x69\000\00000\000Unaligned access, write\000 */
};

static const struct compact_pmu_event pmu_events__arm_cortex_a65_e1[] = {
{ 30753 }, /* br_immed_spec\000\000branch\000Branch speculatively executed, immediate branch\000event=0x78\000\00000\000Branch speculatively executed, immediate branch\000 */
{ 35038 }, /* br_indirect_spec\000\000branch\000Branch speculatively executed, indirect branch\000event=0x7a\000\00000\000Branch speculatively executed, indirect branch\000 */
{ 35743 }, /* br_mis_pred\000\000branch\000Mispredicted or not predicted branch speculatively executed\000event=0x10\000\00000\000Mispredicted or not predicted branch speculatively executed\000 */
{ 36925 }, /* br_pred\000\000branch\000Predictable branch speculatively executed\000event=0x12\000\00000\000Predictable branch speculatively executed\000 */
{ 40692 }, /* br_return_spec\000\000branch\000Branch speculatively executed, procedure return\000event=0x79\000\00000\000Branch speculatively executed, procedure return\000 */
{ 41358 }, /* bus_access\000\000bus\000Attributable Bus access\000event=0x19\000\00000\000Attributable Bus access\000 */
{ 42271 }, /* bus_access_rd\000\000bus\000Bus access read\000event=0x60\000\00000\000Bus access read\000 */
{ 42682 }, /* bus_access_wr\000\000bus\000Bus access write\000event=0x61\000\00000\000Bus access write\000 */
{ 42979 }, /* bus_cycles\000\000bus\000Bus cycle\000event=0x1d\000\00000\000Bus cycle\000 */
{ 47992 }, /* cpu_cycles\000\000bus\000Cycle\000event=0x11\000\00000\000Cycle\000 */
{ 53526 }, /* dtlb_walk\000\000cache\000Access to data TLB causes a translation table walk\000event=0x34\000\00000\000Access to data TLB causes a translation table walk\000 */
{ 92044 }, /* itlb_walk\000\000cache\000Access to instruction TLB that causes a translation table walk\000event=0x35\000\00000\000Access to instruction TLB that causes a translation table walk\000 */
{ 100289 }, /* l1d_cache\000\000cache\000Level 1 data cache access\000event=0x4\000\00000\000Level 1 data cache access\000 */
{ 102693 }, /* l1d_cache_rd\000\000cache\000L1D cache access, read\000event=0x40\000\00000\000Attributable Level 1 data cache access, read\000 */
{ 103129 }, /* l1d_cache_refill\000\000cache\000Level 1 data cache refill\000event=0x3\000\00000\000Level 1 data cache refill\000 */
{ 104145 }, /* l1d_cache_refill_inner\000\000cache\000L1D cache refill, inner\000event=0x44\000\00000\000Attributable Level 1 data cache refill, inner\000 */
{ 104573 }, /* l1d_cache_refill_outer\000\000cache\000L1D cache refill, outer\000event=0x45\000\00000\000Attributable Level 1 data cache refill, outer\000 */
{ 105894 }, /* l1d_cache_refill_rd\000\000cache\000L1D cache refill, read\000event=0x42\000\00000\000Attributable Level 1 data cache refill, read\000 */
{ 106389 }, /* l1d_cache_refill_wr\000\000cache\000L1D cache refill, write\000event=0x43\000\00000\000Attributable Level 1 data cache refill, write\000 */
{ 106891 }, /* l1d_cache_wb\000\000cache\000Attributable Level 1 data cache write-back\000event=0x15\000\00000\000Attributable Level 1 data cache write-back\000 */
{ 108997 }, /* l1d_cache_wr\000\000cache\000L1D cache access, write\000event=0x41\000\00000\000Attributable Level 1 data cache access, write\000 */
{ 109550 }, /* l1d_pref_line_fill\000\000cache\000Level 1 data cache refill started due to prefetch. Counts any linefills from the prefetcher which cause an allocation into the L1 D-cache\000event=0xc3\000\00000\000Level 1 data cache refill started due to prefetch. Counts any linefills from the prefetcher which cause an allocation into the L1 D-cache\000 */
{ 109867 }, /* l1d_tlb\000\000cache\000Attributable Level 1 data or unified TLB access\000event=0x25\000\00000\000Attributable Level 1 data or unified TLB access\000 */
{ 111207 }, /* l1d_tlb_refill\000\000cache\000Attributable Level 1 data TLB refill\000event=0x5\000\00000\000Attributable Level 1 data TLB refill\000 */
{ 114822 }, /* l1d_ws_mode\000\000cache\000L1D is in write stream mode\000event=0xc7\000\00000\000L1D is in write stream mode\000 */
{ 115268 }, /* l1d_ws_mode_enter\000\000cache\000L1D entering write stream mode\000event=0xc6\000\00000\000L1D entering write stream mode\000 */
{ 116626 }, /* l1i_cache\000\000cache\000Attributable Level 1 instruction cache access\000event=0x14\000\00000\000Attributable Level 1 instruction cache access\000 */
{ 118420 }, /* l1i_cache_refill\000\000cache\000Level 1 instruction cache refill\000event=0x1\000\00000\000Level 1 instruction cache refill\000 */
{ 119419 }, /* l1i_tlb\000\000cache\000Attributable Level 1 instruction TLB access\000event=0x26\000\00000\000Attributable Level 1 instruction TLB access\000 */
{ 120198 }, /* l1i_tlb_refill\000\000cache\000Attributable Level 1 instruction TLB refill\000event=0x2\000\00000\000Attributable Level 1 instruction TLB refill\000 */
{ 121626 }, /* l2_l1d_cache_wb_unatt\000\000cache\000Unattributable Level 1 data cache write-back. This event occurs when a requestor outside the PE makes a coherency request that results in writeback\000event=0xf0\000\00000\000Unattributable Level 1 data cache write-back. This event occurs when a requestor outside the PE makes a coherency request that results in writeback\000 */
{ 121966 }, /* l2_l2d_cache_rd_unatt\000\000cache\000Unattributable Level 2 data cache access, read. This event occurs when a requestor outside the PE makes a coherency request that results in level 2 data cache read access\000event=0xf2\000\00000\000Unattributable Level 2 data cache access, read. This event occurs when a requestor outside the PE makes a coherency request that results in level 2 data cache read access\000 */
{ 122352 }, /* l2_l2d_cache_unatt\000\000cache\000Unattributable Level 2 data cache access. This event occurs when a requestor outside the PE makes a coherency request that results in level 2 data cache access\000event=0xf1\000\00000\000Unattributable Level 2 data cache access. This event occurs when a requestor outside the PE makes a coherency request that results in level 2 data cache access\000 */
{ 122713 }, /* l2_l3d_cache_alloc_unatt\000\000cache\000Unattributable Level 3 data or unified cache allocation without refill. This event occurs when a requestor outside the PE makes a coherency request that results in level 3 cache allocate without refill\000event=0xf5\000\00000\000Unattributable Level 3 data or unified cache allocation without refill. This event occurs when a requestor outside the PE makes a coherency request that results in level 3 cache allocate without refill\000 */
{ 123164 }, /* l2_l3d_cache_rd_unatt\000\000cache\000Unattributable Level 3 data cache access, read. This event occurs when a requestor outside the PE makes a coherency request that results in level 3 data cache read access\000event=0xf4\000\00000\000Unattributable Level 3 data cache access, read. This event occurs when a requestor outside the PE makes a coherency request that results in level 3 data cache read access\000 */
{ 123550 }, /* l2_l3d_cache_refill_unatt\000\000cache\000Unattributable Level 3 data or unified cache refill. This event occurs when a requestor outside the PE makes a coherency request that results in level 3 cache refill\000event=0xf6\000\00000\000Unattributable Level 3 data or unified cache refill. This event occurs when a requestor outside the PE makes a coherency request that results in level 3 cache refill\000 */
{ 123930 }, /* l2_l3d_cache_unatt\000\000cache\000Unattributable Level 3 data cache access. This event occurs when a requestor outside the PE makes a coherency request that results in level 3 data cache read access\000event=0xf3\000\00000\000Unattributable Level 3 data cache access. This event occurs when a requestor outside the PE makes a coherency request that results in level 3 data cache read access\000 */
{ 126244 }, /* l2d_cache\000\000cache\000Level 2 data cache access\000event=0x16\000\00000\000Level 2 data cache access\000 */
{ 126922 }, /* l2d_cache_allocate\000\000cache\000Attributable Level 2 data cache allocation without refill\000event=0x20\000\00000\000Attributable Level 2 data cache allocation without refill\000 */
{ 129126 }, /* l2d_cache_rd\000\000cache\000L2D cache access, read\000event=0x50\000\00000\000Attributable Level 2 data cache access, read\000 */
{ 129862 }, /* l2d_cache_refill\000\000cache\000Level 2 data refill\000event=0x17\000\00000\000Level 2 data refill\000 */
{ 132288 }, /* l2d_cache_refill_rd\000\000cache\000L2D cache refill, read\000event=0x52\000\00000\000Attributable Level 2 data cache refill, read\000 */
{ 132829 }, /* l2d_cache_refill_wr\000\000cache\000L2D cache refill, write\000event=0x53\000\00000\000Attributable Level 2 data cache refill, write\000 */
{ 133880 }, /* l2d_cache_stash_dropped\000\000cache\000Level 2 cache stash dropped. This event counts on each stash request received from the interconnect or ACP, that is targeting L2 and gets dropped due to lack of buffer space to hold the request. L2 and L3 cache events (L2D_CACHE*, L3D_CACHE*) The behavior of these events depends on the configuration of the core. If the private L2 cache is present, the L2D_CACHE* events count the activity in the private L2 cache, and the L3D_CACHE* events count the activity in the DSU L3 cache (if present). If the private L2 cache is not present but the DSU L3 cache is present, the L2D_CACHE* events count activity in the DSU L3 cache and the L3D_CACHE* events do not count. The L2D_CACHE_WB, L2D_CACHE_WR and L2D_CACHE_REFILL_WR events do not count in this configuration. If neither the private L2 cache nor the DSU L3 cache are present, neither the L2D_CACHE* or L3D_CACHE* events will count\000event=0xf7\000\00000\000Level 2 cache stash dropped. This event counts on each stash request received from the interconnect or ACP, that is targeting L2 and gets dropped due to lack of buffer space to hold the request. L2 and L3 cache events (L2D_CACHE*, L3D_CACHE*) The behavior of these events depends on the configuration of the core. If the private L2 cache is present, the L2D_CACHE* events count the activity in the private L2 cache, and the L3D_CACHE* events count the activity in the DSU L3 cache (if present). If the private L2 cache is not present but the DSU L3 cache is present, the L2D_CACHE* events count activity in the DSU L3 cache and the L3D_CACHE* events do not count. The L2D_CACHE_WB, L2D_CACHE_WR and L2D_CACHE_REFILL_WR events do not count in this configuration. If neither the private L2 cache nor the DSU L3 cache are present, neither the L2D_CACHE* or L3D_CACHE* events will count\000 */
{ 136414 }, /* l2d_cache_wb\000\000cache\000Attributable Level 2 data cache write-back\000event=0x18\000\00000\000Attributable Level 2 data cache write-back\000 */
{ 138385 }, /* l2d_cache_wr\000\000cache\000L2D cache access, write\000event=0x51\000\00000\000Attributable Level 2 data cache access, write\000 */
{ 140131 }, /* l2d_pref_line_fill\000\000cache\000Level 2 cache refill due to prefetch. +//0 If the core is configured with a per-core L2 cache: This event does not count. +//0 If the core is configured without a per-core L2 cache: This event counts the cluster cache event, as defined by L3_PREF_LINE_FILL. +//0 If there is neither a per-core cache nor a cluster cache configured, this event is not implemented\000event=0xc4\000\00000\000Level 2 cache refill due to prefetch. +//0 If the core is configured with a per-core L2 cache: This event does not count. +//0 If the core is configured without a per-core L2 cache: This event counts the cluster cache event, as defined by L3_PREF_LINE_FILL. +//0 If there is neither a per-core cache nor a cluster cache configured, this event is not implemented\000 */
{ 143389 }, /* l2d_tlb\000\000cache\000Attributable Level 2 data or unified TLB access\000event=0x2f\000\00000\000Attributable Level 2 data or unified TLB access\000 */
{ 144725 }, /* l2d_tlb_refill\000\000cache\000Attributable Level 2 data TLB refill\000event=0x2d\000\00000\000Attributable Level 2 data TLB refill\000 */
{ 147514 }, /* l2d_ws_mode\000\000cache\000Level 2 cache write streaming mode. This event counts for each cycle where the core is in write-streaming mode and not allocating writes into the L2 cache\000event=0xc8\000\00000\000Level 2 cache write streaming mode. This event counts for each cycle where the core is in write-streaming mode and not allocating writes into the L2 cache\000 */
{ 149591 }, /* l3_pref_line_fill\000\000cache\000Level 3 cache refill due to prefetch. This event counts any linefills from the hardware prefetcher which cause an allocation into the L3 cache. Note It might not be possible to distinguish between both hardware and software prefetches and also which prefetches cause an allocation. If so, only hardware prefetches should be counted, regardless of whether they allocate. If either the core is configured without a per-core L2 or the cluster is configured without an L3 cache, this event is not implemented\000event=0xc5\000\00000\000Level 3 cache refill due to prefetch. This event counts any linefills from the hardware prefetcher which cause an allocation into the L3 cache. Note It might not be possible to distinguish between both hardware and software prefetches and also which prefetches cause an allocation. If so, only hardware prefetches should be counted, regardless of whether they allocate. If either the core is configured without a per-core L2 or the cluster is configured without an L3 cache, this event is not implemented\000 */
{ 150641 }, /* l3d_cache\000\000cache\000Attributable Level 3 data cache access\000event=0x2b\000\00000\000Attributable Level 3 data cache access\000 */
{ 151459 }, /* l3d_cache_allocate\000\000cache\000Attributable Level 3 data cache allocation without refill\000event=0x29\000\00000\000Attributable Level 3 data cache allocation without refill\000 */
{ 152420 }, /* l3d_cache_rd\000\000cache\000Attributable Level 3 data or unified cache access, read\000event=0xa0\000\00000\000Attributable Level 3 data or unified cache access, read\000 */
{ 152665 }, /* l3d_cache_refill\000\000cache\000Attributable Level 3 data cache refill\000event=0x2a\000\00000\000Attributable Level 3 data cache refill\000 */
{ 154312 }, /* l3d_cache_refill_rd\000\000cache\000Attributable Level 3 data or unified cache refill, read\000event=0xa2\000\00000\000Attributable Level 3 data or unified cache refill, read\000 */
{ 154806 }, /* l3d_ws_mode\000\000cache\000Level 3 cache write streaming mode. This event counts for each cycle where the core is in write-streaming mode and not allocating writes into the L3 cache\000event=0xc9\000\00000\000Level 3 cache write streaming mode. This event counts for each cycle where the core is in write-streaming mode and not allocating writes into the L3 cache\000 */
{ 160229 }, /* ll_cache_miss_rd\000\000cache\000Last level cache miss, read\000event=0x37\000\00000\000Last level cache miss, read\000 */
{ 160879 }, /* ll_cache_rd\000\000cache\000Attributable Last level cache memory read\000event=0x36\000\00000\000Attributable Last level cache memory read\000 */
{ 200370 }, /* stb_stall\000\000cache\000Merge in the store buffer\000event=0xc0\000\00000\000Merge in the store buffer\000 */
{ 207229 }, /* tlb_l2tlb_l2walk_access\000\000cache\000Level 2 TLB level-2 walk cache access. This event counts accesses to the level-2 walk cache where the last-level walk cache has missed. The event only counts when the translation regime of the pagewalk uses level 2 descriptors. This event does not count if the MMU is disabled\000event=0xcc\000\00000\000Level 2 TLB level-2 walk cache access. This event counts accesses to the level-2 walk cache where the last-level walk cache has missed. The event only counts when the translation regime of the pagewalk uses level 2 descriptors. This event does not count if the MMU is disabled\000 */
{ 207829 }, /* tlb_l2tlb_l2walk_refill\000\000cache\000Level 2 TLB level-2 walk cache refill. This event does not count if the MMU is disabled\000event=0xcd\000\00000\000Level 2 TLB level-2 walk cache refill. This event does not count if the MMU is disabled\000 */
{ 208051 }, /* tlb_l2tlb_llwalk_access\000\000cache\000Level 2 TLB last-level walk cache access. This event does not count if the MMU is disabled\000event=0xca\000\00000\000Level 2 TLB last-level walk cache access. This event does not count if the MMU is disabled\000 */
{ 208279 }, /* tlb_l2tlb_llwalk_refill\000\000cache\000Level 2 TLB last-level walk cache refill. This event does not count if the MMU is disabled\000event=0xcb\000\00000\000Level 2 TLB last-level walk cache refill. This event does not count if the MMU is disabled\000 */
{ 208507 }, /* tlb_l2tlb_s2_access\000\000cache\000Level 2 TLB IPA cache access. This event counts on each access to the IPA cache. +//0 If a single pagewalk needs to make multiple accesses to the IPA cache, each access is counted. +//0 If stage 2 translation is disabled, this event does not count\000event=0xce\000\00000\000Level 2 TLB IPA cache access. This event counts on each access to the IPA cache. +//0 If a single pagewalk needs to make multiple accesses to the IPA cache, each access is counted. +//0 If stage 2 translation is disabled, this event does not count\000 */
{ 209045 }, /* tlb_l2tlb_s2_refill\000\000cache\000Level 2 TLB IPA cache refill. This event counts on each refill of the IPA cache. +//0 If a single pagewalk needs to make multiple accesses to the IPA cache, each access which causes a refill is counted. +//0 If stage 2 translation is disabled, this event does not count\000event=0xcf\000\00000\000Level 2 TLB IPA cache refill. This event counts on each refill of the IPA cache. +//0 If a single pagewalk needs to make multiple accesses to the IPA cache, each access which causes a refill is counted. +//0 If stage 2 translation is disabled, this event does not count\000 */
{ 52494 }, /* dpu_br_cond_mis\000\000dpu\000Instruction retired, conditional branch, mispredicted\000event=0xea\000\00000\000Instruction retired, conditional branch, mispredicted\000 */
{ 52766 }, /* dpu_br_ind_mis\000\000dpu\000Instruction retired, indirect branch, mispredicted\000event=0xe9\000\00000\000Instruction retired, indirect branch, mispredicted\000 */
{ 52903 }, /* dpu_mem_err_dcu\000\000dpu\000Memory error (any type) from DCU\000event=0xec\000\00000\000Memory error (any type) from DCU\000 */
{ 53005 }, /* dpu_mem_err_ifu\000\000dpu\000Memory error (any type) from IFU\000event=0xeb\000\00000\000Memory error (any type) from IFU\000 */
{ 53107 }, /* dpu_mem_err_tlb\000\000dpu\000Memory error (any type) from TLB\000event=0xed\000\00000\000Memory error (any type) from TLB\000 */
{ 57280 }, /* exc_fiq\000\000exception\000Exception taken, FIQ\000event=0x87\000\00000\000Exception taken, FIQ\000 */
{ 57939 }, /* exc_irq\000\000exception\000Exception taken, IRQ\000event=0x86\000\00000\000Exception taken, IRQ\000 */
{ 59896 }, /* exc_taken\000\000exception\000Exception taken\000event=0x9\000\00000\000Exception taken\000 */
{ 165870 }, /* memory_error\000\000exception\000Local memory error\000event=0x1a\000\00000\000Local memory error\000 */
{ 74789 }, /* ifu_all_thrds_rdy\000\000ifu\000This thread and the other thread both ready for scheduling in if0\000event=0xdc\000\00000\000This thread and the other thread both ready for scheduling in if0\000 */
{ 74959 }, /* ifu_flushed_excl_tlb_miss\000\000ifu\000Thread flushed due to reasons other than TLB miss\000event=0xdb\000\00000\000Thread flushed due to reasons other than TLB miss\000 */
{ 75105 }, /* ifu_flushed_tlb_miss\000\000ifu\000Thread flushed due to TLB miss\000event=0xda\000\00000\000Thread flushed due to TLB miss\000 */
{ 75208 }, /* ifu_goto_idle\000\000ifu\000The thread moved from an active state to an inactive state (long-term sleep state, causing deallocation of some resources)\000event=0xe0\000\00000\000The thread moved from an active state to an inactive state (long-term sleep state, causing deallocation of some resources)\000 */
{ 75488 }, /* ifu_ic_lf_sp\000\000ifu\000I-Cache Speculative line fill\000event=0xe4\000\00000\000I-Cache Speculative line fill\000 */
{ 75581 }, /* ifu_ic_lookup_under_miss\000\000ifu\000I-Cache lookup under miss from other thread\000event=0xe1\000\00000\000I-Cache lookup under miss from other thread\000 */
{ 75714 }, /* ifu_ic_miss_under_miss\000\000ifu\000I-Cache miss under miss from other thread\000event=0xe2\000\00000\000I-Cache miss under miss from other thread\000 */
{ 75841 }, /* ifu_ic_miss_wait\000\000ifu\000I-Cache miss on an access from the prefetch block\000event=0xd0\000\00000\000I-Cache miss on an access from the prefetch block\000 */
{ 75978 }, /* ifu_instr_pushed\000\000ifu\000This thread pushed an instruction into the IQ\000event=0xe3\000\00000\000This thread pushed an instruction into the IQ\000 */
{ 76107 }, /* ifu_iutlb_miss_wait\000\000ifu\000Counts the cycles spent on a request for Level 2 TLB lookup after a Level 1l ITLB miss\000event=0xd1\000\00000\000Counts the cycles spent on a request for Level 2 TLB lookup after a Level 1l ITLB miss\000 */
{ 76321 }, /* ifu_micro_caddr_mispred\000\000ifu\000Micro-predictor address mispredict, with respect to if3/if4 predictor\000event=0xd3\000\00000\000Micro-predictor address mispredict, with respect to if3/if4 predictor\000 */
{ 76505 }, /* ifu_micro_cond_mispred\000\000ifu\000Micro-predictor conditional/direction mispredict, with respect to. if3/if4 predictor\000event=0xd2\000\00000\000Micro-predictor conditional/direction mispredict, with respect to. if3/if4 predictor\000 */
{ 76718 }, /* ifu_micro_correction\000\000ifu\000Micro-predictor correction\000event=0xd7\000\00000\000Micro-predictor correction\000 */
{ 76813 }, /* ifu_micro_hit\000\000ifu\000Micro-predictor hit with immediate redirect\000event=0xd4\000\00000\000Micro-predictor hit with immediate redirect\000 */
{ 76935 }, /* ifu_micro_neg_hit\000\000ifu\000Micro-predictor negative cache hit\000event=0xd6\000\00000\000Micro-predictor negative cache hit\000 */
{ 77043 }, /* ifu_micro_no_instr1\000\000ifu\000A 2nd instruction could have been pushed but was not because it was nonsequential\000event=0xd8\000\00000\000A 2nd instruction could have been pushed but was not because it was nonsequential\000 */
{ 77247 }, /* ifu_micro_no_pred\000\000ifu\000Micro-predictor miss\000event=0xd9\000\00000\000Micro-predictor miss\000 */
{ 77327 }, /* ifu_not_rdy_for_arb\000\000ifu\000This thread was not arbitrated because it was not ready for scheduling. For example, due to a cache miss or TLB miss\000event=0xdf\000\00000\000This thread was not arbitrated because it was not ready for scheduling. For example, due to a cache miss or TLB miss\000 */
{ 77601 }, /* ifu_win_arb_other_act\000\000ifu\000This thread was arbitrated when the other thread was also active, but not necessarily ready. For example, waiting for I-Cache or TLB\000event=0xde\000\00000\000This thread was arbitrated when the other thread was also active, but not necessarily ready. For example, waiting for I-Cache or TLB\000 */
{ 77909 }, /* ifu_win_arb_other_rdy\000\000ifu\000This thread was arbitrated when the other thread was also ready for scheduling\000event=0xdd\000\00000\000This thread was arbitrated when the other thread was also ready for scheduling\000 */
{ 20878 }, /* ase_spec\000\000instruction\000Operation speculatively executed, Advanced SIMD instruction\000event=0x74\000\00000\000Operation speculatively executed, Advanced SIMD instruction\000 */
{ 30599 }, /* br_immed_retired\000\000instruction\000Instruction architecturally executed, immediate branch\000event=0xd\000\00000\000Instruction architecturally executed, immediate branch\000 */
{ 36351 }, /* br_mis_pred_retired\000\000instruction\000Instruction architecturally executed, mispredicted branch\000event=0x22\000\00000\000Instruction architecturally executed, mispredicted branch\000 */
{ 37393 }, /* br_retired\000\000instruction\000Instruction architecturally executed, branch\000event=0x21\000\00000\000Instruction architecturally executed, branch\000 */
{ 40483 }, /* br_return_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, procedure return\000event=0xe\000\00000\000Instruction architecturally executed, condition code check pass, procedure return\000 */
{ 45546 }, /* cid_write_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000event=0xb\000\00000\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000 */
{ 48419 }, /* crypto_spec\000\000instruction\000Operation speculatively executed, Cryptographic instruction\000event=0x77\000\00000\000Operation speculatively executed, Cryptographic instruction\000 */
{ 52153 }, /* dp_spec\000\000instruction\000Operation speculatively executed, integer data processing\000event=0x73\000\00000\000Operation speculatively executed, integer data processing\000 */
{ 52638 }, /* dpu_br_cond_retired\000\000instruction\000Instruction retired, conditional branch\000event=0xe8\000\00000\000Instruction retired, conditional branch\000 */
{ 58850 }, /* exc_return\000\000instruction\000Instruction architecturally executed, condition check pass, exception return\000event=0xa\000\00000\000Instruction architecturally executed, condition check pass, exception return\000 */
{ 90819 }, /* inst_retired\000\000instruction\000Instruction architecturally executed\000event=0x8\000\00000\000Instruction architecturally executed\000 */
{ 91235 }, /* inst_spec\000\000instruction\000Operation speculatively executed\000event=0x1b\000\00000\000Operation speculatively executed\000 */
{ 91769 }, /* isb_spec\000\000instruction\000Barrier speculatively executed, ISB\000event=0x7c\000\00000\000Barrier speculatively executed, ISB\000 */
{ 158696 }, /* ld_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, load\000event=0x6\000\00000\000Instruction architecturally executed, condition code check pass, load\000 */
{ 159047 }, /* ld_spec\000\000instruction\000Operation speculatively executed, load\000event=0x70\000\00000\000Operation speculatively executed, load\000 */
{ 159973 }, /* ldst_spec\000\000instruction\000Operation speculatively executed, load or store\000event=0x72\000\00000\000Operation speculatively executed, load or store\000 */
{ 174889 }, /* pc_write_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, software change of the PC\000event=0xc\000\00000\000Instruction architecturally executed, condition code check pass, software change of the PC\000 */
{ 185342 }, /* st_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, store\000event=0x7\000\00000\000Instruction architecturally executed, condition code check pass, store\000 */
{ 185522 }, /* st_spec\000\000instruction\000Operation speculatively executed, store\000event=0x71\000\00000\000Operation speculatively executed, store\000 */
{ 205736 }, /* sw_incr\000\000instruction\000Instruction architecturally executed, Condition code check pass, software increment\000event=0\000\00000\000Instruction architecturally executed, Condition code check pass, software increment\000 */
{ 210294 }, /* ttbr_write_retired\000\000instruction\000Instruction architecturally executed, Condition code check pass, write to TTBR\000event=0x1c\000\00000\000Instruction architecturally executed, Condition code check pass, write to TTBR\000 */
{ 214354 }, /* vfp_spec\000\000instruction\000Operation speculatively executed, floating-point instruction\000event=0x75\000\00000\000Operation speculatively executed, floating-point instruction\000 */
{ 23782 }, /* biu_ext_mem_req\000\000memory\000External memory request\000event=0xc1\000\00000\000External memory request\000 */
{ 23869 }, /* biu_ext_mem_req_nc\000\000memory\000External memory request to non-cacheable memory\000event=0xc2\000\00000\000External memory request to non-cacheable memory\000 */
{ 163148 }, /* mem_access\000\000memory\000Data memory access\000event=0x13\000\00000\000Data memory access\000 */
{ 164298 }, /* mem_access_rd\000\000memory\000Data memory access, read\000event=0x66\000\00000\000Data memory access, read\000 */
{ 164956 }, /* mem_access_wr\000\000memory\000Data memory access, write\000event=0x67\000\00000\000Data memory access, write\000 */
{ 180400 }, /* remote_access_rd\000\000memory\000Attributable memory read access to another socket in a multi-socket system\000event=0x38\000\00000\000Attributable memory read access to another socket in a multi-socket system\000 */
{ 211505 }, /* unaligned_ld_spec\000\000memory\000Unaligned access, read\000event=0x68\000\00000\000Unaligned access, read\000 */
{ 212200 }, /* unaligned_ldst_spec\000\000memory\000Unaligned access\000event=0x6a\000\00000\000Unaligned access\000 */
{ 212637 }, /* unaligned_st_spec\000\000memory\000Unaligned access, write\000event=0x69\000\00000\000Unaligned access, write\000 */
{ 186072 }, /* stall_backend\000\000pipeline\000No operation issued due to the backend\000event=0x24\000\00000\000No operation issued due to the backend\000 */
{ 195875 }, /* stall_frontend\000\000pipeline\000No operation issued because of the frontend\000event=0x23\000\00000\000No operation issued because of the frontend\000 */
};

static const struct compact_pmu_event pmu_events__arm_cortex_a710[] = {
{ 30753 }, /* br_immed_spec\000\000branch\000Branch speculatively executed, immediate branch\000event=0x78\000\00000\000Branch speculatively executed, immediate branch\000 */
{ 35038 }, /* br_indirect_spec\000\000branch\000Branch speculatively executed, indirect branch\000event=0x7a\000\00000\000Branch speculatively executed, indirect branch\000 */
{ 35743 }, /* br_mis_pred\000\000branch\000Mispredicted or not predicted branch speculatively executed\000event=0x10\000\00000\000Mispredicted or not predicted branch speculatively executed\000 */
{ 36925 }, /* br_pred\000\000branch\000Predictable branch speculatively executed\000event=0x12\000\00000\000Predictable branch speculatively executed\000 */
{ 40692 }, /* br_return_spec\000\000branch\000Branch speculatively executed, procedure return\000event=0x79\000\00000\000Branch speculatively executed, procedure return\000 */
{ 41358 }, /* bus_access\000\000bus\000Attributable Bus access\000event=0x19\000\00000\000Attributable Bus access\000 */
{ 42271 }, /* bus_access_rd\000\000bus\000Bus access read\000event=0x60\000\00000\000Bus access read\000 */
{ 42682 }, /* bus_access_wr\000\000bus\000Bus access write\000event=0x61\000\00000\000Bus access write\000 */
{ 42979 }, /* bus_cycles\000\000bus\000Bus cycle\000event=0x1d\000\00000\000Bus cycle\000 */
{ 46424 }, /* cnt_cycles\000\000bus\000Constant frequency cycles\000event=0x4004\000\00000\000Constant frequency cycles.  The counter increments at a constant frequency equal to the rate of increment of the system counter, CNTPCT_EL0\000 */
{ 47992 }, /* cpu_cycles\000\000bus\000Cycle\000event=0x11\000\00000\000Cycle\000 */
{ 53526 }, /* dtlb_walk\000\000cache\000Access to data TLB causes a translation table walk\000event=0x34\000\00000\000Access to data TLB causes a translation table walk\000 */
{ 92044 }, /* itlb_walk\000\000cache\000Access to instruction TLB that causes a translation table walk\000event=0x35\000\00000\000Access to instruction TLB that causes a translation table walk\000 */
{ 100289 }, /* l1d_cache\000\000cache\000Level 1 data cache access\000event=0x4\000\00000\000Level 1 data cache access\000 */
{ 101403 }, /* l1d_cache_inval\000\000cache\000L1D cache invalidate\000event=0x48\000\00000\000Attributable Level 1 data cache invalidate\000 */
{ 102276 }, /* l1d_cache_lmiss_rd\000\000cache\000Level 1 data cache long-latency read miss\000event=0x39\000\00000\000Level 1 data cache long-latency read miss.  The counter counts each memory read access counted by L1D_CACHE that incurs additional latency because it returns data from outside the Level 1 data or unified cache of this processing element\000 */
{ 102693 }, /* l1d_cache_rd\000\000cache\000L1D cache access, read\000event=0x40\000\00000\000Attributable Level 1 data cache access, read\000 */
{ 103129 }, /* l1d_cache_refill\000\000cache\000Level 1 data cache refill\000event=0x3\000\00000\000Level 1 data cache refill\000 */
{ 104145 }, /* l1d_cache_refill_inner\000\000cache\000L1D cache refill, inner\000event=0x44\000\00000\000Attributable Level 1 data cache refill, inner\000 */
{ 104573 }, /* l1d_cache_refill_outer\000\000cache\000L1D cache refill, outer\000event=0x45\000\00000\000Attributable Level 1 data cache refill, outer\000 */
{ 105894 }, /* l1d_cache_refill_rd\000\000cache\000L1D cache refill, read\000event=0x42\000\00000\000Attributable Level 1 data cache refill, read\000 */
{ 106389 }, /* l1d_cache_refill_wr\000\000cache\000L1D cache refill, write\000event=0x43\000\00000\000Attributable Level 1 data cache refill, write\000 */
{ 106891 }, /* l1d_cache_wb\000\000cache\000Attributable Level 1 data cache write-back\000event=0x15\000\00000\000Attributable Level 1 data cache write-back\000 */
{ 107957 }, /* l1d_cache_wb_clean\000\000cache\000L1D cache Write-Back, cleaning and coherency\000event=0x47\000\00000\000Level 1 data cache Write-Back, cleaning and coherency\000 */
{ 108495 }, /* l1d_cache_wb_victim\000\000cache\000L1D cache Write-Back, victim\000event=0x46\000\00000\000Attributable Level 1 data cache Write-Back, victim\000 */
{ 108997 }, /* l1d_cache_wr\000\000cache\000L1D cache access, write\000event=0x41\000\00000\000Attributable Level 1 data cache access, write\000 */
{ 109867 }, /* l1d_tlb\000\000cache\000Attributable Level 1 data or unified TLB access\000event=0x25\000\00000\000Attributable Level 1 data or unified TLB access\000 */
{ 110748 }, /* l1d_tlb_rd\000\000cache\000L1D tlb access, read\000event=0x4e\000\00000\000Attributable Level 1 data or unified TLB access, read\000 */
{ 111207 }, /* l1d_tlb_refill\000\000cache\000Attributable Level 1 data TLB refill\000event=0x5\000\00000\000Attributable Level 1 data TLB refill\000 */
{ 112199 }, /* l1d_tlb_refill_rd\000\000cache\000L1D tlb refill, read\000event=0x4c\000\00000\000Attributable Level 1 data TLB refill, read\000 */
{ 113093 }, /* l1d_tlb_refill_wr\000\000cache\000L1D tlb refill, write\000event=0x4d\000\00000\000Attributable Level 1 data TLB refill, write\000 */
{ 113993 }, /* l1d_tlb_wr\000\000cache\000L1D tlb access, write\000event=0x4f\000\00000\000Attributable Level 1 data or unified TLB access, write\000 */
{ 116626 }, /* l1i_cache\000\000cache\000Attributable Level 1 instruction cache access\000event=0x14\000\00000\000Attributable Level 1 instruction cache access\000 */
{ 117316 }, /* l1i_cache_lmiss\000\000cache\000Level 1 instruction cache long-latency read miss\000event=0x4006\000\00000\000Level 1 instruction cache long-latency read miss.  If the L1I_CACHE_RD event is implemented, the counter counts each access counted by L1I_CACHE_RD that incurs additional latency because it returns instructions from outside of the Level 1 instruction cache of this PE.  If the L1I_CACHE_RD event is not implemented, the counter counts each access counted by L1I_CACHE that incurs additional latency because it returns instructions from outside the Level 1 instruction cache of this PE.  The event indicates to software that the access missed in the Level 1 instruction cache and might have a significant performance impact due to the additional latency, compared to the latency of an access that hits in the Level 1 instruction cache\000 */
{ 118420 }, /* l1i_cache_refill\000\000cache\000Level 1 instruction cache refill\000event=0x1\000\00000\000Level 1 instruction cache refill\000 */
{ 119419 }, /* l1i_tlb\000\000cache\000Attributable Level 1 instruction TLB access\000event=0x26\000\00000\000Attributable Level 1 instruction TLB access\000 */
{ 120198 }, /* l1i_tlb_refill\000\000cache\000Attributable Level 1 instruction TLB refill\000event=0x2\000\00000\000Attributable Level 1 instruction TLB refill\000 */
{ 126244 }, /* l2d_cache\000\000cache\000Level 2 data cache access\000event=0x16\000\00000\000Level 2 data cache access\000 */
{ 126922 }, /* l2d_cache_allocate\000\000cache\000Attributable Level 2 data cache allocation without refill\000event=0x20\000\00000\000Attributable Level 2 data cache allocation without refill\000 */
{ 127466 }, /* l2d_cache_inval\000\000cache\000L2D cache invalidate\000event=0x58\000\00000\000Attributable Level 2 data cache invalidate\000 */
{ 128191 }, /* l2d_cache_lmiss_rd\000\000cache\000Level 2 data cache long-latency read miss\000event=0x4009\000\00000\000Level 2 data cache long-latency read miss.  The counter counts each memory read access counted by L2D_CACHE that incurs additional latency because it returns data from outside the Level 2 data or unified cache of this processing element.  The event indicates to software that the access missed in the Level 2 data or unified cache and might have a significant performance impact compared to the latency of an access that hits in the Level 2 data or unified cache\000 */
{ 129126 }, /* l2d_cache_rd\000\000cache\000L2D cache access, read\000event=0x50\000\00000\000Attributable Level 2 data cache access, read\000 */
{ 129862 }, /* l2d_cache_refill\000\000cache\000Level 2 data refill\000event=0x17\000\00000\000Level 2 data refill\000 */
{ 132288 }, /* l2d_cache_refill_rd\000\000cache\000L2D cache refill, read\000event=0x52\000\00000\000Attributable Level 2 data cache refill, read\000 */
{ 132829 }, /* l2d_cache_refill_wr\000\000cache\000L2D cache refill, write\000event=0x53\000\00000\000Attributable Level 2 data cache refill, write\000 */
{ 136414 }, /* l2d_cache_wb\000\000cache\000Attributable Level 2 data cache write-back\000event=0x18\000\00000\000Attributable Level 2 data cache write-back\000 */
{ 137387 }, /* l2d_cache_wb_clean\000\000cache\000L2D cache Write-Back, cleaning and coherency\000event=0x57\000\00000\000Level 2 data cache Write-Back, cleaning and coherency\000 */
{ 137968 }, /* l2d_cache_wb_victim\000\000cache\000L2D cache Write-Back, victim\000event=0x56\000\00000\000Attributable Level 2 data cache Write-Back, victim\000 */
{ 138385 }, /* l2d_cache_wr\000\000cache\000L2D cache access, write\000event=0x51\000\00000\000Attributable Level 2 data cache access, write\000 */
{ 143389 }, /* l2d_tlb\000\000cache\000Attributable Level 2 data or unified TLB access\000event=0x2f\000\00000\000Attributable Level 2 data or unified TLB access\000 */
{ 144070 }, /* l2d_tlb_rd\000\000cache\000L2D cache access, read\000event=0x5e\000\00000\000Attributable Level 2 data or unified TLB access, read\000 */
{ 144725 }, /* l2d_tlb_refill\000\000cache\000Attributable Level 2 data TLB refill\000event=0x2d\000\00000\000Attributable Level 2 data TLB refill\000 */
{ 145079 }, /* l2d_tlb_refill_rd\000\000cache\000L2D cache refill, read\000event=0x5c\000\00000\000Attributable Level 2 data or unified TLB refill, read\000 */
{ 145555 }, /* l2d_tlb_refill_wr\000\000cache\000L2D cache refill, write\000event=0x5d\000\00000\000Attributable Level 2 data or unified TLB refill, write\000 */
{ 146014 }, /* l2d_tlb_wr\000\000cache\000L2D cache access, write\000event=0x5f\000\00000\000Attributable Level 2 data or unified TLB access, write\000 */
{ 150641 }, /* l3d_cache\000\000cache\000Attributable Level 3 data cache access\000event=0x2b\000\00000\000Attributable Level 3 data cache access\000 */
{ 151459 }, /* l3d_cache_allocate\000\000cache\000Attributable Level 3 data cache allocation without refill\000event=0x29\000\00000\000Attributable Level 3 data cache allocation without refill\000 */
{ 151872 }, /* l3d_cache_lmiss_rd\000\000cache\000Level 3 data cache long-latency read miss\000event=0x400b\000\00000\000Level 3 data cache long-latency read miss.  The counter counts each memory read access counted by L3D_CACHE that incurs additional latency because it returns data from outside the Level 3 data or unified cache of this processing element.  The event indicates to software that the access missed in the Level 3 data or unified cache and might have a significant performance impact compared to the latency of an access that hits in the Level 3 data or unified cache\000 */
{ 152420 }, /* l3d_cache_rd\000\000cache\000Attributable Level 3 data or unified cache access, read\000event=0xa0\000\00000\000Attributable Level 3 data or unified cache access, read\000 */
{ 152665 }, /* l3d_cache_refill\000\000cache\000Attributable Level 3 data cache refill\000event=0x2a\000\00000\000Attributable Level 3 data cache refill\000 */
{ 160229 }, /* ll_cache_miss_rd\000\000cache\000Last level cache miss, read\000event=0x37\000\00000\000Last level cache miss, read\000 */
{ 160879 }, /* ll_cache_rd\000\000cache\000Attributable Last level cache memory read\000event=0x36\000\00000\000Attributable Last level cache memory read\000 */
{ 56959 }, /* exc_dabort\000\000exception\000Exception taken, Data Abort and SError\000event=0x84\000\00000\000Exception taken, Data Abort and SError\000 */
{ 57280 }, /* exc_fiq\000\000exception\000Exception taken, FIQ\000event=0x87\000\00000\000Exception taken, FIQ\000 */
{ 57633 }, /* exc_hvc\000\000exception\000Exception taken, Hypervisor Call\000event=0x8a\000\00000\000Exception taken, Hypervisor Call\000 */
{ 57939 }, /* exc_irq\000\000exception\000Exception taken, IRQ\000event=0x86\000\00000\000Exception taken, IRQ\000 */
{ 58355 }, /* exc_pabort\000\000exception\000Exception taken, Instruction Abort\000event=0x83\000\00000\000Exception taken, Instruction Abort\000 */
{ 59258 }, /* exc_smc\000\000exception\000Exception taken, Secure Monitor Call\000event=0x88\000\00000\000Exception taken, Secure Monitor Call\000 */
{ 59572 }, /* exc_svc\000\000exception\000Exception taken, Supervisor Call\000event=0x82\000\00000\000Exception taken, Supervisor Call\000 */
{ 59896 }, /* exc_taken\000\000exception\000Exception taken\000event=0x9\000\00000\000Exception taken\000 */
{ 60575 }, /* exc_trap_dabort\000\000exception\000Exception taken, Data Abort or SError not taken locally\000event=0x8c\000\00000\000Exception taken, Data Abort or SError not taken locally\000 */
{ 61080 }, /* exc_trap_fiq\000\000exception\000Exception taken, FIQ not taken locally\000event=0x8f\000\00000\000Exception taken, FIQ not taken locally\000 */
{ 62347 }, /* exc_trap_irq\000\000exception\000Exception taken, IRQ not taken locally\000event=0x8e\000\00000\000Exception taken, IRQ not taken locally\000 */
{ 62841 }, /* exc_trap_other\000\000exception\000Exception taken, Other traps not taken locally\000event=0x8d\000\00000\000Exception taken, Other traps not taken locally\000 */
{ 63383 }, /* exc_trap_pabort\000\000exception\000Exception taken, Instruction Abort not taken locally\000event=0x8b\000\00000\000Exception taken, Instruction Abort not taken locally\000 */
{ 64129 }, /* exc_undef\000\000exception\000Exception taken, Other synchronous\000event=0x81\000\00000\000Exception taken, Other synchronous\000 */
{ 165870 }, /* memory_error\000\000exception\000Local memory error\000event=0x1a\000\00000\000Local memory error\000 */
{ 20758 }, /* ase_inst_spec\000\000instruction\000ASE operations speculatively executed\000event=0x8005\000\00000\000ASE operations speculatively executed\000 */
{ 20878 }, /* ase_spec\000\000instruction\000Operation speculatively executed, Advanced SIMD instruction\000event=0x74\000\00000\000Operation speculatively executed, Advanced SIMD instruction\000 */
{ 21288 }, /* ase_sve_int16_spec\000\000instruction\000Advanced SIMD and SVE 16-bit integer operations speculatively executed\000event=0x80e7\000\00000\000Advanced SIMD and SVE 16-bit integer operations speculatively executed\000 */
{ 21479 }, /* ase_sve_int32_spec\000\000instruction\000Advanced SIMD and SVE 32-bit integer operations speculatively executed\000event=0x80eb\000\00000\000Advanced SIMD and SVE 32-bit integer operations speculatively executed\000 */
{ 21670 }, /* ase_sve_int64_spec\000\000instruction\000Advanced SIMD and SVE 64-bit integer operations speculatively executed\000event=0x80ef\000\00000\000Advanced SIMD and SVE 64-bit integer operations speculatively executed\000 */
{ 21861 }, /* ase_sve_int8_spec\000\000instruction\000Advanced SIMD and SVE 8-bit integer operations speculatively executed\000event=0x80e3\000\00000\000Advanced SIMD and SVE 8-bit integer operations speculatively executed\000 */
{ 36351 }, /* br_mis_pred_retired\000\000instruction\000Instruction architecturally executed, mispredicted branch\000event=0x22\000\00000\000Instruction architecturally executed, mispredicted branch\000 */
{ 37393 }, /* br_retired\000\000instruction\000Instruction architecturally executed, branch\000event=0x21\000\00000\000Instruction architecturally executed, branch\000 */
{ 45546 }, /* cid_write_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000event=0xb\000\00000\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000 */
{ 48419 }, /* crypto_spec\000\000instruction\000Operation speculatively executed, Cryptographic instruction\000event=0x77\000\00000\000Operation speculatively executed, Cryptographic instruction\000 */
{ 51117 }, /* dmb_spec\000\000instruction\000Barrier speculatively executed, DMB\000event=0x7e\000\00000\000Barrier speculatively executed, DMB\000 */
{ 52153 }, /* dp_spec\000\000instruction\000Operation speculatively executed, integer data processing\000event=0x73\000\00000\000Operation speculatively executed, integer data processing\000 */
{ 53209 }, /* dsb_spec\000\000instruction\000Barrier speculatively executed, DSB\000event=0x7d\000\00000\000Barrier speculatively executed, DSB\000 */
{ 58850 }, /* exc_return\000\000instruction\000Instruction architecturally executed, condition check pass, exception return\000event=0xa\000\00000\000Instruction architecturally executed, condition check pass, exception return\000 */
{ 67066 }, /* fp_dp_spec\000\000instruction\000Floating-point double-precision operations speculatively executed\000event=0x801c\000\00000\000Floating-point double-precision operations speculatively executed\000 */
{ 67239 }, /* fp_fixed_ops_spec\000\000instruction\000Non-scalable floating-point element Operations speculatively executed\000event=0x80c1\000\00000\000Non-scalable floating-point element Operations speculatively executed\000 */
{ 68165 }, /* fp_hp_spec\000\000instruction\000Floating-point half-precision operations speculatively executed\000event=0x8014\000\00000\000Floating-point half-precision operations speculatively executed\000 */
{ 68926 }, /* fp_scale_ops_spec\000\000instruction\000Scalable floating-point element Operations speculatively executed\000event=0x80c0\000\00000\000Scalable floating-point element Operations speculatively executed\000 */
{ 69704 }, /* fp_sp_spec\000\000instruction\000Floating-point single-precision operations speculatively executed\000event=0x8018\000\00000\000Floating-point single-precision operations speculatively executed\000 */
{ 90819 }, /* inst_retired\000\000instruction\000Instruction architecturally executed\000event=0x8\000\00000\000Instruction architecturally executed\000 */
{ 91235 }, /* inst_spec\000\000instruction\000Operation speculatively executed\000event=0x1b\000\00000\000Operation speculatively executed\000 */
{ 91769 }, /* isb_spec\000\000instruction\000Barrier speculatively executed, ISB\000event=0x7c\000\00000\000Barrier speculatively executed, ISB\000 */
{ 159047 }, /* ld_spec\000\000instruction\000Operation speculatively executed, load\000event=0x70\000\00000\000Operation speculatively executed, load\000 */
{ 159351 }, /* ldrex_spec\000\000instruction\000Exclusive operation speculatively executed, LDREX or LDX\000event=0x6c\000\00000\000Exclusive operation speculatively executed, LDREX or LDX\000 */
{ 173450 }, /* op_retired\000\000instruction\000Micro-operation architecturally executed\000event=0x3a\000\00000\000Micro-operation architecturally executed.  The counter counts each operation counted by OP_SPEC that would be executed in a simple sequential execution of the program\000 */
{ 173697 }, /* op_spec\000\000instruction\000Micro-operation speculatively executed\000event=0x3b\000\00000\000Micro-operation speculatively executed.  The counter counts the number of operations executed by the processing element, including those that are executed speculatively and would not be executed in a simple sequential execution of the program\000 */
{ 175115 }, /* pc_write_spec\000\000instruction\000Operation speculatively executed, software change of the PC\000event=0x76\000\00000\000Operation speculatively executed, software change of the PC\000 */
{ 177878 }, /* rc_ld_spec\000\000instruction\000Release consistency operation speculatively executed, Load-Acquire\000event=0x90\000\00000\000Release consistency operation speculatively executed, Load-Acquire\000 */
{ 178251 }, /* rc_st_spec\000\000instruction\000Release consistency operation speculatively executed, Store-Release\000event=0x91\000\00000\000Release consistency operation speculatively executed, Store-Release\000 */
{ 185522 }, /* st_spec\000\000instruction\000Operation speculatively executed, store\000event=0x71\000\00000\000Operation speculatively executed, store\000 */
{ 200454 }, /* strex_fail_spec\000\000instruction\000Exclusive operation speculatively executed, STREX or STX fail\000event=0x6e\000\00000\000Exclusive operation speculatively executed, STREX or STX fail\000 */
{ 201025 }, /* strex_pass_spec\000\000instruction\000Exclusive operation speculatively executed, STREX or STX pass\000event=0x6d\000\00000\000Exclusive operation speculatively executed, STREX or STX pass\000 */
{ 201592 }, /* strex_spec\000\000instruction\000Exclusive operation speculatively executed, STREX or STX\000event=0x6f\000\00000\000Exclusive operation speculatively executed, STREX or STX\000 */
{ 202321 }, /* sve_inst_spec\000\000instruction\000SVE operations speculatively executed\000event=0x8006\000\00000\000SVE operations speculatively executed\000 */
{ 202582 }, /* sve_ldff_fault_spec\000\000instruction\000SVE first-fault load operations speculatively executed which set FFR bit to 0\000event=0x80bd\000\00000\000SVE first-fault load operations speculatively executed which set FFR bit to 0\000 */
{ 202788 }, /* sve_ldff_spec\000\000instruction\000SVE First-fault load Operations speculatively executed\000event=0x80bc\000\00000\000SVE First-fault load Operations speculatively executed\000 */
{ 203847 }, /* sve_pred_empty_spec\000\000instruction\000SVE predicated operations with no active predicates speculatively executed\000event=0x8075\000\00000\000SVE predicated operations with no active predicates speculatively executed\000 */
{ 204047 }, /* sve_pred_full_spec\000\000instruction\000SVE predicated operations speculatively executed with all active predicates\000event=0x8076\000\00000\000SVE predicated operations speculatively executed with all active predicates\000 */
{ 204248 }, /* sve_pred_not_full_spec\000\000instruction\000SVE predicated operations with empty or partially active predicates\000event=0x8079\000\00000\000SVE predicated operations with empty or partially active predicates\000 */
{ 204437 }, /* sve_pred_partial_spec\000\000instruction\000SVE predicated operations speculatively executed with partially active predicates\000event=0x8077\000\00000\000SVE predicated operations speculatively executed with partially active predicates\000 */
{ 204653 }, /* sve_pred_spec\000\000instruction\000SVE predicated Operations speculatively executed\000event=0x8074\000\00000\000SVE predicated Operations speculatively executed\000 */
{ 205736 }, /* sw_incr\000\000instruction\000Instruction architecturally executed, Condition code check pass, software increment\000event=0\000\00000\000Instruction architecturally executed, Condition code check pass, software increment\000 */
{ 210294 }, /* ttbr_write_retired\000\000instruction\000Instruction architecturally executed, Condition code check pass, write to TTBR\000event=0x1c\000\00000\000Instruction architecturally executed, Condition code check pass, write to TTBR\000 */
{ 214354 }, /* vfp_spec\000\000instruction\000Operation speculatively executed, floating-point instruction\000event=0x75\000\00000\000Operation speculatively executed, floating-point instruction\000 */
{ 155492 }, /* ld_align_lat\000\000memory\000Load with additional latency from alignment\000event=0x4021\000\00000\000Load with additional latency from alignment\000 */
{ 159841 }, /* ldst_align_lat\000\000memory\000Access with additional latency from alignment\000event=0x4020\000\00000\000Access with additional latency from alignment\000 */
{ 163148 }, /* mem_access\000\000memory\000Data memory access\000event=0x13\000\00000\000Data memory access\000 */
{ 163403 }, /* mem_access_checked\000\000memory\000Checked data memory access\000event=0x4024\000\00000\000Checked data memory access\000 */
{ 163501 }, /* mem_access_checked_rd\000\000memory\000Checked data memory access, read\000event=0x4025\000\00000\000Checked data memory access, read\000 */
{ 163614 }, /* mem_access_checked_wr\000\000memory\000Checked data memory access, write\000event=0x4026\000\00000\000Checked data memory access, write\000 */
{ 164298 }, /* mem_access_rd\000\000memory\000Data memory access, read\000event=0x66\000\00000\000Data memory access, read\000 */
{ 164956 }, /* mem_access_wr\000\000memory\000Data memory access, write\000event=0x67\000\00000\000Data memory access, write\000 */
{ 179852 }, /* remote_access\000\000memory\000Access to another socket in a multi-socket system\000event=0x31\000\00000\000Access to another socket in a multi-socket system\000 */
{ 185116 }, /* st_align_lat\000\000memory\000Store with additional latency from alignment\000event=0x4022\000\00000\000Store with additional latency from alignment\000 */
{ 211505 }, /* unaligned_ld_spec\000\000memory\000Unaligned access, read\000event=0x68\000\00000\000Unaligned access, read\000 */
{ 212200 }, /* unaligned_ldst_spec\000\000memory\000Unaligned access\000event=0x6a\000\00000\000Unaligned access\000 */
{ 212637 }, /* unaligned_st_spec\000\000memory\000Unaligned access, write\000event=0x69\000\00000\000Unaligned access, write\000 */
{ 185831 }, /* stall\000\000pipeline\000No operation sent for execution\000event=0x3c\000\00000\000No operation sent for execution.  The counter counts every attributable cycle on which no attributable instruction or operation was sent for execution on this processing element\000 */
{ 186072 }, /* stall_backend\000\000pipeline\000No operation issued due to the backend\000event=0x24\000\00000\000No operation issued due to the backend\000 */
{ 192744 }, /* stall_backend_mem\000\000pipeline\000Memory stall cycles\000event=0x4005\000\00000\000Memory stall cycles.  The counter counts each cycle counted by STALL_BACKEND where there is a cache miss in the last level of cache within the processing element clock domain\000 */
{ 195875 }, /* stall_frontend\000\000pipeline\000No operation issued because of the frontend\000event=0x23\000\00000\000No operation issued because of the frontend\000 */
{ 199011 }, /* stall_slot\000\000pipeline\000No operation sent for execution on a slot\000event=0x3f\000\00000\000No operation sent for execution on a slot.  The counter counts on each attributable cycle the number of instruction or operation slots that were not occupied by an instruction or operation attributable to the processing element\000 */
{ 199317 }, /* stall_slot_backend\000\000pipeline\000No operation sent for execution on a slot due to the backend\000event=0x3d\000\00000\000No operation sent for execution on a slot due to the backend.  Counts each slot counted by STALL_SLOT where no attributable instruction or operation was sent for execution because the backend is unable to accept it\000 */
{ 199953 }, /* stall_slot_frontend\000\000pipeline\000No operation sent for execution on a slot due to the frontend\000event=0x3e\000\00000\000No operation sent for execution on a slot due to the frontend.  Counts each slot counted by STALL_SLOT where no attributable instruction or operation was sent for execution because there was no attributable instruction or operation available to issue from the processing element from the frontend for the slot\000 */
{ 48775 }, /* cti_trigout4\000\000trace\000Cross-trigger Interface output trigger 4\000event=0x4018\000\00000\000Cross-trigger Interface output trigger 4\000 */
{ 48894 }, /* cti_trigout5\000\000trace\000Cross-trigger Interface output trigger 5 \000event=0x4019\000\00000\000Cross-trigger Interface output trigger 5 \000 */
{ 49015 }, /* cti_trigout6\000\000trace\000Cross-trigger Interface output trigger 6\000event=0x401a\000\00000\000Cross-trigger Interface output trigger 6\000 */
{ 49134 }, /* cti_trigout7\000\000trace\000Cross-trigger Interface output trigger 7\000event=0x401b\000\00000\000Cross-trigger Interface output trigger 7\000 */
{ 209779 }, /* trb_wrap\000\000trace\000Trace buffer current write pointer wrapped\000event=0x400c\000\00000\000Trace buffer current write pointer wrapped\000 */
{ 209898 }, /* trcextout0\000\000trace\000PE Trace Unit external output 0\000event=0x4010\000\00000\000PE Trace Unit external output 0\000 */
{ 209997 }, /* trcextout1\000\000trace\000PE Trace Unit external output 1\000event=0x4011\000\00000\000PE Trace Unit external output 1\000 */
{ 210096 }, /* trcextout2\000\000trace\000PE Trace Unit external output 2\000event=0x4012\000\00000\000PE Trace Unit external output 2\000 */
{ 210195 }, /* trcextout3\000\000trace\000PE Trace Unit external output 3\000event=0x4013\000\00000\000PE Trace Unit external output 3\000 */
};

static const struct compact_pmu_event pmu_events__arm_cortex_a73[] = {
{ 35038 }, /* br_indirect_spec\000\000branch\000Branch speculatively executed, indirect branch\000event=0x7a\000\00000\000Branch speculatively executed, indirect branch\000 */
{ 35743 }, /* br_mis_pred\000\000branch\000Mispredicted or not predicted branch speculatively executed\000event=0x10\000\00000\000Mispredicted or not predicted branch speculatively executed\000 */
{ 36925 }, /* br_pred\000\000branch\000Predictable branch speculatively executed\000event=0x12\000\00000\000Predictable branch speculatively executed\000 */
{ 41358 }, /* bus_access\000\000bus\000Attributable Bus access\000event=0x19\000\00000\000Attributable Bus access\000 */
{ 41979 }, /* bus_access_normal\000\000bus\000Bus access, Normal\000event=0x64\000\00000\000Bus access, Normal\000 */
{ 42055 }, /* bus_access_not_shared\000\000bus\000Bus access, not Normal, Cacheable, Shareable\000event=0x63\000\00000\000Bus access, not Normal, Cacheable, Shareable\000 */
{ 42187 }, /* bus_access_periph\000\000bus\000Bus access, peripheral\000event=0x65\000\00000\000Bus access, peripheral\000 */
{ 42562 }, /* bus_access_shared\000\000bus\000Bus access, Normal, Cacheable, Shareable\000event=0x62\000\00000\000Bus access, Normal, Cacheable, Shareable\000 */
{ 42979 }, /* bus_cycles\000\000bus\000Bus cycle\000event=0x1d\000\00000\000Bus cycle\000 */
{ 47992 }, /* cpu_cycles\000\000bus\000Cycle\000event=0x11\000\00000\000Cycle\000 */
{ 46623 }, /* cp15_utlb_refill\000\000cache\000Level 1 CP15 TLB refill\000event=0xe8\000\00000\000Level 1 CP15 TLB refill\000 */
{ 49591 }, /* dcache_self_hit_vipt\000\000cache\000Data cache hit in itself due to VIPT aliasing\000event=0xec\000\00000\000Data cache hit in itself due to VIPT aliasing\000 */
{ 71725 }, /* i_btac_ram_rd\000\000cache\000Number of ways read in the instruction BTAC RAM\000event=0xc4\000\00000\000Number of ways read in the instruction BTAC RAM\000 */
{ 71857 }, /* i_data_ram_rd\000\000cache\000Number of ways read in the instruction cache - Data RAM\000event=0xc3\000\00000\000Number of ways read in the instruction cache - Data RAM\000 */
{ 72005 }, /* i_tag_ram_rd\000\000cache\000Number of ways read in the instruction cache - Tag RAM\000event=0xc2\000\00000\000Number of ways read in the instruction cache - Tag RAM\000 */
{ 100289 }, /* l1d_cache\000\000cache\000Level 1 data cache access\000event=0x4\000\00000\000Level 1 data cache access\000 */
{ 102693 }, /* l1d_cache_rd\000\000cache\000L1D cache access, read\000event=0x40\000\00000\000Attributable Level 1 data cache access, read\000 */
{ 103129 }, /* l1d_cache_refill\000\000cache\000Level 1 data cache refill\000event=0x3\000\00000\000Level 1 data cache refill\000 */
{ 106891 }, /* l1d_cache_wb\000\000cache\000Attributable Level 1 data cache write-back\000event=0x15\000\00000\000Attributable Level 1 data cache write-back\000 */
{ 108997 }, /* l1d_cache_wr\000\000cache\000L1D cache access, write\000event=0x41\000\00000\000Attributable Level 1 data cache access, write\000 */
{ 111207 }, /* l1d_tlb_refill\000\000cache\000Attributable Level 1 data TLB refill\000event=0x5\000\00000\000Attributable Level 1 data TLB refill\000 */
{ 116626 }, /* l1i_cache\000\000cache\000Attributable Level 1 instruction cache access\000event=0x14\000\00000\000Attributable Level 1 instruction cache access\000 */
{ 118420 }, /* l1i_cache_refill\000\000cache\000Level 1 instruction cache refill\000event=0x1\000\00000\000Level 1 instruction cache refill\000 */
{ 120198 }, /* l1i_tlb_refill\000\000cache\000Attributable Level 1 instruction TLB refill\000event=0x2\000\00000\000Attributable Level 1 instruction TLB refill\000 */
{ 126244 }, /* l2d_cache\000\000cache\000Level 2 data cache access\000event=0x16\000\00000\000Level 2 data cache access\000 */
{ 127466 }, /* l2d_cache_inval\000\000cache\000L2D cache invalidate\000event=0x58\000\00000\000Attributable Level 2 data cache invalidate\000 */
{ 129126 }, /* l2d_cache_rd\000\000cache\000L2D cache access, read\000event=0x50\000\00000\000Attributable Level 2 data cache access, read\000 */
{ 129862 }, /* l2d_cache_refill\000\000cache\000Level 2 data refill\000event=0x17\000\00000\000Level 2 data refill\000 */
{ 136414 }, /* l2d_cache_wb\000\000cache\000Attributable Level 2 data cache write-back\000event=0x18\000\00000\000Attributable Level 2 data cache write-back\000 */
{ 137387 }, /* l2d_cache_wb_clean\000\000cache\000L2D cache Write-Back, cleaning and coherency\000event=0x57\000\00000\000Level 2 data cache Write-Back, cleaning and coherency\000 */
{ 137968 }, /* l2d_cache_wb_victim\000\000cache\000L2D cache Write-Back, victim\000event=0x56\000\00000\000Attributable Level 2 data cache Write-Back, victim\000 */
{ 138385 }, /* l2d_cache_wr\000\000cache\000L2D cache access, write\000event=0x51\000\00000\000Attributable Level 2 data cache access, write\000 */
{ 175516 }, /* pld_utlb_refill\000\000cache\000Level 1 PLD TLB refill\000event=0xe7\000\00000\000Level 1 PLD TLB refill\000 */
{ 207110 }, /* tlb_access\000\000cache\000Level 2 TLB access\000event=0xea\000\00000\000Level 2 TLB access\000 */
{ 209627 }, /* tlb_miss\000\000cache\000Level 2 TLB miss\000event=0xeb\000\00000\000Level 2 TLB miss\000 */
{ 214285 }, /* utlb_flush\000\000cache\000Level 1 TLB flush\000event=0xe9\000\00000\000Level 1 TLB flush\000 */
{ 55450 }, /* etm_ext_out0\000\000etm\000ETM trace unit output 0\000event=0xde\000\00000\000ETM trace unit output 0\000 */
{ 55531 }, /* etm_ext_out1\000\000etm\000ETM trace unit output 1\000event=0xdf\000\00000\000ETM trace unit output 1\000 */
{ 57633 }, /* exc_hvc\000\000exception\000Exception taken, Hypervisor Call\000event=0x8a\000\00000\000Exception taken, Hypervisor Call\000 */
{ 59896 }, /* exc_taken\000\000exception\000Exception taken\000event=0x9\000\00000\000Exception taken\000 */
{ 61197 }, /* exc_trap_hyp\000\000exception\000Number of Traps to hypervisor\000event=0xdc\000\00000\000Number of Traps to hypervisor\000 */
{ 20878 }, /* ase_spec\000\000instruction\000Operation speculatively executed, Advanced SIMD instruction\000event=0x74\000\00000\000Operation speculatively executed, Advanced SIMD instruction\000 */
{ 30599 }, /* br_immed_retired\000\000instruction\000Instruction architecturally executed, immediate branch\000event=0xd\000\00000\000Instruction architecturally executed, immediate branch\000 */
{ 40483 }, /* br_return_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, procedure return\000event=0xe\000\00000\000Instruction architecturally executed, condition code check pass, procedure return\000 */
{ 45546 }, /* cid_write_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000event=0xb\000\00000\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000 */
{ 48419 }, /* crypto_spec\000\000instruction\000Operation speculatively executed, Cryptographic instruction\000event=0x77\000\00000\000Operation speculatively executed, Cryptographic instruction\000 */
{ 51117 }, /* dmb_spec\000\000instruction\000Barrier speculatively executed, DMB\000event=0x7e\000\00000\000Barrier speculatively executed, DMB\000 */
{ 52153 }, /* dp_spec\000\000instruction\000Operation speculatively executed, integer data processing\000event=0x73\000\00000\000Operation speculatively executed, integer data processing\000 */
{ 53209 }, /* dsb_spec\000\000instruction\000Barrier speculatively executed, DSB\000event=0x7d\000\00000\000Barrier speculatively executed, DSB\000 */
{ 58850 }, /* exc_return\000\000instruction\000Instruction architecturally executed, condition check pass, exception return\000event=0xa\000\00000\000Instruction architecturally executed, condition check pass, exception return\000 */
{ 90819 }, /* inst_retired\000\000instruction\000Instruction architecturally executed\000event=0x8\000\00000\000Instruction architecturally executed\000 */
{ 91235 }, /* inst_spec\000\000instruction\000Operation speculatively executed\000event=0x1b\000\00000\000Operation speculatively executed\000 */
{ 91769 }, /* isb_spec\000\000instruction\000Barrier speculatively executed, ISB\000event=0x7c\000\00000\000Barrier speculatively executed, ISB\000 */
{ 159047 }, /* ld_spec\000\000instruction\000Operation speculatively executed, load\000event=0x70\000\00000\000Operation speculatively executed, load\000 */
{ 159351 }, /* ldrex_spec\000\000instruction\000Exclusive operation speculatively executed, LDREX or LDX\000event=0x6c\000\00000\000Exclusive operation speculatively executed, LDREX or LDX\000 */
{ 159973 }, /* ldst_spec\000\000instruction\000Operation speculatively executed, load or store\000event=0x72\000\00000\000Operation speculatively executed, load or store\000 */
{ 174889 }, /* pc_write_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, software change of the PC\000event=0xc\000\00000\000Instruction architecturally executed, condition code check pass, software change of the PC\000 */
{ 185522 }, /* st_spec\000\000instruction\000Operation speculatively executed, store\000event=0x71\000\00000\000Operation speculatively executed, store\000 */
{ 200454 }, /* strex_fail_spec\000\000instruction\000Exclusive operation speculatively executed, STREX or STX fail\000event=0x6e\000\00000\000Exclusive operation speculatively executed, STREX or STX fail\000 */
{ 205736 }, /* sw_incr\000\000instruction\000Instruction architecturally executed, Condition code check pass, software increment\000event=0\000\00000\000Instruction architecturally executed, Condition code check pass, software increment\000 */
{ 210294 }, /* ttbr_write_retired\000\000instruction\000Instruction architecturally executed, Condition code check pass, write to TTBR\000event=0x1c\000\00000\000Instruction architecturally executed, Condition code check pass, write to TTBR\000 */
{ 214354 }, /* vfp_spec\000\000instruction\000Operation speculatively executed, floating-point instruction\000event=0x75\000\00000\000Operation speculatively executed, floating-point instruction\000 */
{ 163148 }, /* mem_access\000\000memory\000Data memory access\000event=0x13\000\00000\000Data memory access\000 */
{ 164298 }, /* mem_access_rd\000\000memory\000Data memory access, read\000event=0x66\000\00000\000Data memory access, read\000 */
{ 164956 }, /* mem_access_wr\000\000memory\000Data memory access, write\000event=0x67\000\00000\000Data memory access, write\000 */
{ 212200 }, /* unaligned_ldst_spec\000\000memory\000Unaligned access\000event=0x6a\000\00000\000Unaligned access\000 */
{ 170698 }, /* mmu_ptw\000\000mmu\000Duration of a translation table walk handled by the MMU\000event=0xe0\000\00000\000Duration of a translation table walk handled by the MMU\000 */
{ 170838 }, /* mmu_ptw_cp15\000\000mmu\000Duration of a translation table walk requested by a CP15 operation (maintenance by MVA and VA to PA operations)\000event=0xe6\000\00000\000Duration of a translation table walk requested by a CP15 operation (maintenance by MVA and VA to PA operations)\000 */
{ 171294 }, /* mmu_ptw_iside\000\000mmu\000Duration of a translation table walk requested by the Instruction Side\000event=0xe4\000\00000\000Duration of a translation table walk requested by the Instruction Side\000 */
{ 171646 }, /* mmu_ptw_lsu\000\000mmu\000Duration of a translation table walk requested by the LSU\000event=0xe3\000\00000\000Duration of a translation table walk requested by the LSU\000 */
{ 171794 }, /* mmu_ptw_pld\000\000mmu\000Duration of a translation table walk requested by a Preload instruction or Prefetch request\000event=0xe5\000\00000\000Duration of a translation table walk requested by a Preload instruction or Prefetch request\000 */
{ 172322 }, /* mmu_ptw_st1\000\000mmu\000Duration of a Stage 1 translation table walk handled by the MMU\000event=0xe1\000\00000\000Duration of a Stage 1 translation table walk handled by the MMU\000 */
{ 172794 }, /* mmu_ptw_st2\000\000mmu\000Duration of a Stage 2 translation table walk handled by the MMU\000event=0xe2\000\00000\000Duration of a Stage 2 translation table walk handled by the MMU\000 */
{ 49428 }, /* d_lsu_slot_full\000\000pipeline\000Duration for which all slots in the Load-Store Unit are busy\000event=0xd3\000\00000\000Duration for which all slots in the Load-Store Unit are busy\000 */
{ 49965 }, /* de_iq_full\000\000pipeline\000Duration for which all slots in the Data Engine issue queue are busy\000event=0xda\000\00000\000Duration for which all slots in the Data Engine issue queue are busy\000 */
{ 51971 }, /* dp_iq_full\000\000pipeline\000Duration for which all slots in the data processing issue queue are busy\000event=0xd9\000\00000\000Duration for which all slots in the data processing issue queue are busy\000 */
{ 160107 }, /* lf_stall\000\000pipeline\000A linefill caused an instruction side stall\000event=0xc0\000\00000\000A linefill caused an instruction side stall\000 */
{ 162253 }, /* ls_iq_full\000\000pipeline\000Duration for which all slots in the load-store issue queue are busy\000event=0xd8\000\00000\000Duration for which all slots in the load-store issue queue are busy\000 */
{ 177727 }, /* ptw_stall\000\000pipeline\000A translation table walk caused an instruction side stall\000event=0xc1\000\00000\000A translation table walk caused an instruction side stall\000 */
};

static const struct compact_pmu_event pmu_events__arm_cortex_a75[] = {
{ 35038 }, /* br_indirect_spec\000\000branch\000Branch speculatively executed, indirect branch\000event=0x7a\000\00000\000Branch speculatively executed, indirect branch\000 */
{ 35743 }, /* br_mis_pred\000\000branch\000Mispredicted or not predicted branch speculatively executed\000event=0x10\000\00000\000Mispredicted or not predicted branch speculatively executed\000 */
{ 36925 }, /* br_pred\000\000branch\000Predictable branch speculatively executed\000event=0x12\000\00000\000Predictable branch speculatively executed\000 */
{ 41358 }, /* bus_access\000\000bus\000Attributable Bus access\000event=0x19\000\00000\000Attributable Bus access\000 */
{ 42271 }, /* bus_access_rd\000\000bus\000Bus access read\000event=0x60\000\00000\000Bus access read\000 */
{ 42682 }, /* bus_access_wr\000\000bus\000Bus access write\000event=0x61\000\00000\000Bus access write\000 */
{ 42979 }, /* bus_cycles\000\000bus\000Bus cycle\000event=0x1d\000\00000\000Bus cycle\000 */
{ 47992 }, /* cpu_cycles\000\000bus\000Cycle\000event=0x11\000\00000\000Cycle\000 */
{ 53526 }, /* dtlb_walk\000\000cache\000Access to data TLB causes a translation table walk\000event=0x34\000\00000\000Access to data TLB causes a translation table walk\000 */
{ 71725 }, /* i_btac_ram_rd\000\000cache\000Number of ways read in the instruction BTAC RAM\000event=0xc4\000\00000\000Number of ways read in the instruction BTAC RAM\000 */
{ 71857 }, /* i_data_ram_rd\000\000cache\000Number of ways read in the instruction cache - Data RAM\000event=0xc3\000\00000\000Number of ways read in the instruction cache - Data RAM\000 */
{ 72005 }, /* i_tag_ram_rd\000\000cache\000Number of ways read in the instruction cache - Tag RAM\000event=0xc2\000\00000\000Number of ways read in the instruction cache - Tag RAM\000 */
{ 92044 }, /* itlb_walk\000\000cache\000Access to instruction TLB that causes a translation table walk\000event=0x35\000\00000\000Access to instruction TLB that causes a translation table walk\000 */
{ 100289 }, /* l1d_cache\000\000cache\000Level 1 data cache access\000event=0x4\000\00000\000Level 1 data cache access\000 */
{ 101155 }, /* l1d_cache_allocate\000\000cache\000Level 1 data cache allocation without refill\000event=0x1f\000\00000\000Level 1 data cache allocation without refill\000 */
{ 101403 }, /* l1d_cache_inval\000\000cache\000L1D cache invalidate\000event=0x48\000\00000\000Attributable Level 1 data cache invalidate\000 */
{ 102693 }, /* l1d_cache_rd\000\000cache\000L1D cache access, read\000event=0x40\000\00000\000Attributable Level 1 data cache access, read\000 */
{ 103129 }, /* l1d_cache_refill\000\000cache\000Level 1 data cache refill\000event=0x3\000\00000\000Level 1 data cache refill\000 */
{ 106891 }, /* l1d_cache_wb\000\000cache\000Attributable Level 1 data cache write-back\000event=0x15\000\00000\000Attributable Level 1 data cache write-back\000 */
{ 107957 }, /* l1d_cache_wb_clean\000\000cache\000L1D cache Write-Back, cleaning and coherency\000event=0x47\000\00000\000Level 1 data cache Write-Back, cleaning and coherency\000 */
{ 108495 }, /* l1d_cache_wb_victim\000\000cache\000L1D cache Write-Back, victim\000event=0x46\000\00000\000Attributable Level 1 data cache Write-Back, victim\000 */
{ 108997 }, /* l1d_cache_wr\000\000cache\000L1D cache access, write\000event=0x41\000\00000\000Attributable Level 1 data cache access, write\000 */
{ 109867 }, /* l1d_tlb\000\000cache\000Attributable Level 1 data or unified TLB access\000event=0x25\000\00000\000Attributable Level 1 data or unified TLB access\000 */
{ 111207 }, /* l1d_tlb_refill\000\000cache\000Attributable Level 1 data TLB refill\000event=0x5\000\00000\000Attributable Level 1 data TLB refill\000 */
{ 116626 }, /* l1i_cache\000\000cache\000Attributable Level 1 instruction cache access\000event=0x14\000\00000\000Attributable Level 1 instruction cache access\000 */
{ 118420 }, /* l1i_cache_refill\000\000cache\000Level 1 instruction cache refill\000event=0x1\000\00000\000Level 1 instruction cache refill\000 */
{ 119419 }, /* l1i_tlb\000\000cache\000Attributable Level 1 instruction TLB access\000event=0x26\000\00000\000Attributable Level 1 instruction TLB access\000 */
{ 120198 }, /* l1i_tlb_refill\000\000cache\000Attributable Level 1 instruction TLB refill\000event=0x2\000\00000\000Attributable Level 1 instruction TLB refill\000 */
{ 120953 }, /* l1pld_tlb\000\000cache\000Level 1 preload TLB access. This event only counts software and hardware prefetches at Level 1. This event counts all accesses to the preload data micro TLB, that is L1 prefetcher and preload instructions. This event does not take into account whether the MMU is enabled or not\000event=0xeb\000\00000\000Level 1 preload TLB access. This event only counts software and hardware prefetches at Level 1. This event counts all accesses to the preload data micro TLB, that is L1 prefetcher and preload instructions. This event does not take into account whether the MMU is enabled or not\000 */
{ 121541 }, /* l1pld_tlb_refill\000\000cache\000Level 1 PLD TLB refill\000event=0xe7\000\00000\000Level 1 PLD TLB refill\000 */
{ 126244 }, /* l2d_cache\000\000cache\000Level 2 data cache access\000event=0x16\000\00000\000Level 2 data cache access\000 */
{ 126922 }, /* l2d_cache_allocate\000\000cache\000Attributable Level 2 data cache allocation without refill\000event=0x20\000\00000\000Attributable Level 2 data cache allocation without refill\000 */
{ 127466 }, /* l2d_cache_inval\000\000cache\000L2D cache invalidate\000event=0x58\000\00000\000Attributable Level 2 data cache invalidate\000 */
{ 129126 }, /* l2d_cache_rd\000\000cache\000L2D cache access, read\000event=0x50\000\00000\000Attributable Level 2 data cache access, read\000 */
{ 129862 }, /* l2d_cache_refill\000\000cache\000Level 2 data refill\000event=0x17\000\00000\000Level 2 data refill\000 */
{ 136414 }, /* l2d_cache_wb\000\000cache\000Attributable Level 2 data cache write-back\000event=0x18\000\00000\000Attributable Level 2 data cache write-back\000 */
{ 137387 }, /* l2d_cache_wb_clean\000\000cache\000L2D cache Write-Back, cleaning and coherency\000event=0x57\000\00000\000Level 2 data cache Write-Back, cleaning and coherency\000 */
{ 137968 }, /* l2d_cache_wb_victim\000\000cache\000L2D cache Write-Back, victim\000event=0x56\000\00000\000Attributable Level 2 data cache Write-Back, victim\000 */
{ 138385 }, /* l2d_cache_wr\000\000cache\000L2D cache access, write\000event=0x51\000\00000\000Attributable Level 2 data cache access, write\000 */
{ 143389 }, /* l2d_tlb\000\000cache\000Attributable Level 2 data or unified TLB access\000event=0x2f\000\00000\000Attributable Level 2 data or unified TLB access\000 */
{ 144725 }, /* l2d_tlb_refill\000\000cache\000Attributable Level 2 data TLB refill\000event=0x2d\000\00000\000Attributable Level 2 data TLB refill\000 */
{ 148794 }, /* l2i_tlb\000\000cache\000Attributable Level 2 instruction TLB access\000event=0x30\000\00000\000Attributable Level 2 instruction TLB access\000 */
{ 149235 }, /* l2pld_tlb\000\000cache\000Level 2 preload and MMU prefetcher TLB access. This event only counts software and hardware prefetches at Level 2\000event=0xe8\000\00000\000Level 2 preload and MMU prefetcher TLB access. This event only counts software and hardware prefetches at Level 2\000 */
{ 150641 }, /* l3d_cache\000\000cache\000Attributable Level 3 data cache access\000event=0x2b\000\00000\000Attributable Level 3 data cache access\000 */
{ 151459 }, /* l3d_cache_allocate\000\000cache\000Attributable Level 3 data cache allocation without refill\000event=0x29\000\00000\000Attributable Level 3 data cache allocation without refill\000 */
{ 152420 }, /* l3d_cache_rd\000\000cache\000Attributable Level 3 data or unified cache access, read\000event=0xa0\000\00000\000Attributable Level 3 data or unified cache access, read\000 */
{ 152665 }, /* l3d_cache_refill\000\000cache\000Attributable Level 3 data cache refill\000event=0x2a\000\00000\000Attributable Level 3 data cache refill\000 */
{ 154312 }, /* l3d_cache_refill_rd\000\000cache\000Attributable Level 3 data or unified cache refill, read\000event=0xa2\000\00000\000Attributable Level 3 data or unified cache refill, read\000 */
{ 160229 }, /* ll_cache_miss_rd\000\000cache\000Last level cache miss, read\000event=0x37\000\00000\000Last level cache miss, read\000 */
{ 160879 }, /* ll_cache_rd\000\000cache\000Attributable Last level cache memory read\000event=0x36\000\00000\000Attributable Last level cache memory read\000 */
{ 175600 }, /* pldtlb_walk\000\000cache\000Prefetch access to unified TLB that caused a page table walk. This event counts software and hardware prefetches\000event=0xec\000\00000\000Prefetch access to unified TLB that caused a page table walk. This event counts software and hardware prefetches\000 */
{ 207110 }, /* tlb_access\000\000cache\000Level 2 TLB access\000event=0xea\000\00000\000Level 2 TLB access\000 */
{ 214285 }, /* utlb_flush\000\000cache\000Level 1 TLB flush\000event=0xe9\000\00000\000Level 1 TLB flush\000 */
{ 55450 }, /* etm_ext_out0\000\000etm\000ETM trace unit output 0\000event=0xde\000\00000\000ETM trace unit output 0\000 */
{ 55531 }, /* etm_ext_out1\000\000etm\000ETM trace unit output 1\000event=0xdf\000\00000\000ETM trace unit output 1\000 */
{ 57633 }, /* exc_hvc\000\000exception\000Exception taken, Hypervisor Call\000event=0x8a\000\00000\000Exception taken, Hypervisor Call\000 */
{ 59896 }, /* exc_taken\000\000exception\000Exception taken\000event=0x9\000\00000\000Exception taken\000 */
{ 61296 }, /* exc_trap_hyp\000\000exception\000Number of traps to hypervisor. This event counts the number of exception traps taken to EL2, excluding HVC instructions. This event is set every time that an exception is executed because of a decoded trap to the hypervisor. CCFAIL exceptions and traps caused by HVC instructions are excluded. This event is not counted when it is accessible from Non-secure EL0 or EL1\000event=0xdc\000\00000\000Number of traps to hypervisor. This event counts the number of exception traps taken to EL2, excluding HVC instructions. This event is set every time that an exception is executed because of a decoded trap to the hypervisor. CCFAIL exceptions and traps caused by HVC instructions are excluded. This event is not counted when it is accessible from Non-secure EL0 or EL1\000 */
{ 64129 }, /* exc_undef\000\000exception\000Exception taken, Other synchronous\000event=0x81\000\00000\000Exception taken, Other synchronous\000 */
{ 20878 }, /* ase_spec\000\000instruction\000Operation speculatively executed, Advanced SIMD instruction\000event=0x74\000\00000\000Operation speculatively executed, Advanced SIMD instruction\000 */
{ 30599 }, /* br_immed_retired\000\000instruction\000Instruction architecturally executed, immediate branch\000event=0xd\000\00000\000Instruction architecturally executed, immediate branch\000 */
{ 37393 }, /* br_retired\000\000instruction\000Instruction architecturally executed, branch\000event=0x21\000\00000\000Instruction architecturally executed, branch\000 */
{ 40483 }, /* br_return_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, procedure return\000event=0xe\000\00000\000Instruction architecturally executed, condition code check pass, procedure return\000 */
{ 45546 }, /* cid_write_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000event=0xb\000\00000\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000 */
{ 48419 }, /* crypto_spec\000\000instruction\000Operation speculatively executed, Cryptographic instruction\000event=0x77\000\00000\000Operation speculatively executed, Cryptographic instruction\000 */
{ 51117 }, /* dmb_spec\000\000instruction\000Barrier speculatively executed, DMB\000event=0x7e\000\00000\000Barrier speculatively executed, DMB\000 */
{ 52153 }, /* dp_spec\000\000instruction\000Operation speculatively executed, integer data processing\000event=0x73\000\00000\000Operation speculatively executed, integer data processing\000 */
{ 53209 }, /* dsb_spec\000\000instruction\000Barrier speculatively executed, DSB\000event=0x7d\000\00000\000Barrier speculatively executed, DSB\000 */
{ 58850 }, /* exc_return\000\000instruction\000Instruction architecturally executed, condition check pass, exception return\000event=0xa\000\00000\000Instruction architecturally executed, condition check pass, exception return\000 */
{ 90819 }, /* inst_retired\000\000instruction\000Instruction architecturally executed\000event=0x8\000\00000\000Instruction architecturally executed\000 */
{ 91235 }, /* inst_spec\000\000instruction\000Operation speculatively executed\000event=0x1b\000\00000\000Operation speculatively executed\000 */
{ 91769 }, /* isb_spec\000\000instruction\000Barrier speculatively executed, ISB\000event=0x7c\000\00000\000Barrier speculatively executed, ISB\000 */
{ 159047 }, /* ld_spec\000\000instruction\000Operation speculatively executed, load\000event=0x70\000\00000\000Operation speculatively executed, load\000 */
{ 159351 }, /* ldrex_spec\000\000instruction\000Exclusive operation speculatively executed, LDREX or LDX\000event=0x6c\000\00000\000Exclusive operation speculatively executed, LDREX or LDX\000 */
{ 159973 }, /* ldst_spec\000\000instruction\000Operation speculatively executed, load or store\000event=0x72\000\00000\000Operation speculatively executed, load or store\000 */
{ 174889 }, /* pc_write_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, software change of the PC\000event=0xc\000\00000\000Instruction architecturally executed, condition code check pass, software change of the PC\000 */
{ 185522 }, /* st_spec\000\000instruction\000Operation speculatively executed, store\000event=0x71\000\00000\000Operation speculatively executed, store\000 */
{ 200454 }, /* strex_fail_spec\000\000instruction\000Exclusive operation speculatively executed, STREX or STX fail\000event=0x6e\000\00000\000Exclusive operation speculatively executed, STREX or STX fail\000 */
{ 201025 }, /* strex_pass_spec\000\000instruction\000Exclusive operation speculatively executed, STREX or STX pass\000event=0x6d\000\00000\000Exclusive operation speculatively executed, STREX or STX pass\000 */
{ 201592 }, /* strex_spec\000\000instruction\000Exclusive operation speculatively executed, STREX or STX\000event=0x6f\000\00000\000Exclusive operation speculatively executed, STREX or STX\000 */
{ 205736 }, /* sw_incr\000\000instruction\000Instruction architecturally executed, Condition code check pass, software increment\000event=0\000\00000\000Instruction architecturally executed, Condition code check pass, software increment\000 */
{ 210294 }, /* ttbr_write_retired\000\000instruction\000Instruction architecturally executed, Condition code check pass, write to TTBR\000event=0x1c\000\00000\000Instruction architecturally executed, Condition code check pass, write to TTBR\000 */
{ 214354 }, /* vfp_spec\000\000instruction\000Operation speculatively executed, floating-point instruction\000event=0x75\000\00000\000Operation speculatively executed, floating-point instruction\000 */
{ 163148 }, /* mem_access\000\000memory\000Data memory access\000event=0x13\000\00000\000Data memory access\000 */
{ 164298 }, /* mem_access_rd\000\000memory\000Data memory access, read\000event=0x66\000\00000\000Data memory access, read\000 */
{ 164956 }, /* mem_access_wr\000\000memory\000Data memory access, write\000event=0x67\000\00000\000Data memory access, write\000 */
{ 180400 }, /* remote_access_rd\000\000memory\000Attributable memory read access to another socket in a multi-socket system\000event=0x38\000\00000\000Attributable memory read access to another socket in a multi-socket system\000 */
{ 212200 }, /* unaligned_ldst_spec\000\000memory\000Unaligned access\000event=0x6a\000\00000\000Unaligned access\000 */
{ 170698 }, /* mmu_ptw\000\000mmu\000Duration of a translation table walk handled by the MMU\000event=0xe0\000\00000\000Duration of a translation table walk handled by the MMU\000 */
{ 171095 }, /* mmu_ptw_cp15\000\000mmu\000Duration of a translation table walk requested by an address translation operation\000event=0xe6\000\00000\000Duration of a translation table walk requested by an address translation operation\000 */
{ 171470 }, /* mmu_ptw_iside\000\000mmu\000Duration of a translation table walk requested by the instruction side\000event=0xe4\000\00000\000Duration of a translation table walk requested by the instruction side\000 */
{ 171646 }, /* mmu_ptw_lsu\000\000mmu\000Duration of a translation table walk requested by the LSU\000event=0xe3\000\00000\000Duration of a translation table walk requested by the LSU\000 */
{ 171794 }, /* mmu_ptw_pld\000\000mmu\000Duration of a translation table walk requested by a Preload instruction or Prefetch request\000event=0xe5\000\00000\000Duration of a translation table walk requested by a Preload instruction or Prefetch request\000 */
{ 172010 }, /* mmu_ptw_st1\000\000mmu\000Duration of a Stage 1 translation table walk handled by the MMU. This event is not counted when it is accessible from Non-secure EL0 or EL1\000event=0xe1\000\00000\000Duration of a Stage 1 translation table walk handled by the MMU. This event is not counted when it is accessible from Non-secure EL0 or EL1\000 */
{ 172482 }, /* mmu_ptw_st2\000\000mmu\000Duration of a Stage 2 translation table walk handled by the MMU. This event is not counted when it is accessible from Non-secure EL0 or EL1\000event=0xe2\000\00000\000Duration of a Stage 2 translation table walk handled by the MMU. This event is not counted when it is accessible from Non-secure EL0 or EL1\000 */
{ 49253 }, /* d_lsu_slot_full\000\000pipeline\000Duration for which all slots in the Load-Store Unit (LSU) are busy\000event=0xd3\000\00000\000Duration for which all slots in the Load-Store Unit (LSU) are busy\000 */
{ 50139 }, /* de_iq_full\000\000pipeline\000Duration for which all slots in the data engine issue queue are busy. This event is set every time that the data engine rename has at least one valid instruction, excluding No Operations (NOPs), that cannot move to the issue stage because accpt_instr is LOW\000event=0xda\000\00000\000Duration for which all slots in the data engine issue queue are busy. This event is set every time that the data engine rename has at least one valid instruction, excluding No Operations (NOPs), that cannot move to the issue stage because accpt_instr is LOW\000 */
{ 51475 }, /* dp_iq_full\000\000pipeline\000Duration for which all slots in the data processing issue queue are busy. This event counts the cycles where all slots in the DP0 and DP1 IQs are full with micro-operations waiting for issuing, and the despatch stage is not empty\000event=0xd9\000\00000\000Duration for which all slots in the data processing issue queue are busy. This event counts the cycles where all slots in the DP0 and DP1 IQs are full with micro-operations waiting for issuing, and the despatch stage is not empty\000 */
{ 160107 }, /* lf_stall\000\000pipeline\000A linefill caused an instruction side stall\000event=0xc0\000\00000\000A linefill caused an instruction side stall\000 */
{ 161785 }, /* ls_iq_full\000\000pipeline\000Duration for which all slots in the load-store issue queue are busy. This event counts the cycles where all slots in the LS IQs are full with micro-operations waiting for issuing, and the dispatch stage is not empty\000event=0xd8\000\00000\000Duration for which all slots in the load-store issue queue are busy. This event counts the cycles where all slots in the LS IQs are full with micro-operations waiting for issuing, and the dispatch stage is not empty\000 */
{ 177727 }, /* ptw_stall\000\000pipeline\000A translation table walk caused an instruction side stall\000event=0xc1\000\00000\000A translation table walk caused an instruction side stall\000 */
{ 186072 }, /* stall_backend\000\000pipeline\000No operation issued due to the backend\000event=0x24\000\00000\000No operation issued due to the backend\000 */
{ 195875 }, /* stall_frontend\000\000pipeline\000No operation issued because of the frontend\000event=0x23\000\00000\000No operation issued because of the frontend\000 */
};

static const struct compact_pmu_event pmu_events__arm_cortex_a76[] = {
{ 35898 }, /* br_mis_pred\000\000branch\000Mispredicted or not predicted branch speculatively executed\000event=0x10\000\00000\000This event counts any predictable branch instruction which is mispredicted either due to dynamic misprediction or because the MMU is off and the branches are statically predicted not taken\000 */
{ 37040 }, /* br_pred\000\000branch\000Predictable branch speculatively executed\000event=0x12\000\00000\000This event counts all predictable branches\000 */
{ 41646 }, /* bus_access\000\000bus\000Attributable Bus access\000event=0x19\000\00000\000This event counts for every beat of data transferred over the data channels between the core and the SCU. If both read and write data beats are transferred on a given cycle, this event is counted twice on that cycle. This event counts the sum of BUS_ACCESS_RD and BUS_ACCESS_WR\000 */
{ 42271 }, /* bus_access_rd\000\000bus\000Bus access read\000event=0x60\000\00000\000Bus access read\000 */
{ 42682 }, /* bus_access_wr\000\000bus\000Bus access write\000event=0x61\000\00000\000Bus access write\000 */
{ 43343 }, /* bus_cycles\000\000bus\000Bus cycle\000event=0x1d\000\00000\000This event duplicates CPU_CYCLES\000 */
{ 48035 }, /* cpu_cycles\000\000bus\000The number of core clock cycles\000event=0x11\000\00000\000The number of core clock cycles\000 */
{ 53660 }, /* dtlb_walk\000\000cache\000Access to data TLB causes a translation table walk\000event=0x34\000\00000\000This event counts on any data access which causes L2D_TLB_REFILL to count\000 */
{ 92202 }, /* itlb_walk\000\000cache\000Access to instruction TLB that causes a translation table walk\000event=0x35\000\00000\000This event counts on any instruction access which causes L2D_TLB_REFILL to count\000 */
{ 100372 }, /* l1d_cache\000\000cache\000Level 1 data cache access\000event=0x4\000\00000\000This event counts any load or store operation or page table walk access which looks up in the L1 data cache. In particular, any access which could count the L1D_CACHE_REFILL event causes this event to count\000 */
{ 101403 }, /* l1d_cache_inval\000\000cache\000L1D cache invalidate\000event=0x48\000\00000\000Attributable Level 1 data cache invalidate\000 */
{ 102693 }, /* l1d_cache_rd\000\000cache\000L1D cache access, read\000event=0x40\000\00000\000Attributable Level 1 data cache access, read\000 */
{ 103219 }, /* l1d_cache_refill\000\000cache\000Level 1 data cache refill\000event=0x3\000\00000\000This event counts any load or store operation or page table walk access which causes data to be read from outside the L1, including accesses which do not allocate into L1\000 */
{ 104145 }, /* l1d_cache_refill_inner\000\000cache\000L1D cache refill, inner\000event=0x44\000\00000\000Attributable Level 1 data cache refill, inner\000 */
{ 104573 }, /* l1d_cache_refill_outer\000\000cache\000L1D cache refill, outer\000event=0x45\000\00000\000Attributable Level 1 data cache refill, outer\000 */
{ 105894 }, /* l1d_cache_refill_rd\000\000cache\000L1D cache refill, read\000event=0x42\000\00000\000Attributable Level 1 data cache refill, read\000 */
{ 106389 }, /* l1d_cache_refill_wr\000\000cache\000L1D cache refill, write\000event=0x43\000\00000\000Attributable Level 1 data cache refill, write\000 */
{ 107012 }, /* l1d_cache_wb\000\000cache\000Attributable Level 1 data cache write-back\000event=0x15\000\00000\000This event counts any write-back of data from the L1 data cache to L2 or L3. This counts both victim line evictions and snoops, including cache maintenance operations\000 */
{ 107957 }, /* l1d_cache_wb_clean\000\000cache\000L1D cache Write-Back, cleaning and coherency\000event=0x47\000\00000\000Level 1 data cache Write-Back, cleaning and coherency\000 */
{ 108495 }, /* l1d_cache_wb_victim\000\000cache\000L1D cache Write-Back, victim\000event=0x46\000\00000\000Attributable Level 1 data cache Write-Back, victim\000 */
{ 108997 }, /* l1d_cache_wr\000\000cache\000L1D cache access, write\000event=0x41\000\00000\000Attributable Level 1 data cache access, write\000 */
{ 110115 }, /* l1d_tlb\000\000cache\000Level 1 data TLB access\000event=0x25\000\00000\000This event counts any load or store operation which accesses the data L1 TLB. If both a load and a store are executed on a cycle, this event counts twice. This event counts regardless of whether the MMU is enabled\000 */
{ 110748 }, /* l1d_tlb_rd\000\000cache\000L1D tlb access, read\000event=0x4e\000\00000\000Attributable Level 1 data or unified TLB access, read\000 */
{ 111317 }, /* l1d_tlb_refill\000\000cache\000Attributable Level 1 data TLB refill\000event=0x5\000\00000\000This event counts any refill of the data L1 TLB from the L2 TLB. This includes refills that result in a translation fault\000 */
{ 112199 }, /* l1d_tlb_refill_rd\000\000cache\000L1D tlb refill, read\000event=0x4c\000\00000\000Attributable Level 1 data TLB refill, read\000 */
{ 113093 }, /* l1d_tlb_refill_wr\000\000cache\000L1D tlb refill, write\000event=0x4d\000\00000\000Attributable Level 1 data TLB refill, write\000 */
{ 113993 }, /* l1d_tlb_wr\000\000cache\000L1D tlb access, write\000event=0x4f\000\00000\000Attributable Level 1 data or unified TLB access, write\000 */
{ 116750 }, /* l1i_cache\000\000cache\000Attributable Level 1 instruction cache access\000event=0x14\000\00000\000Level 1 instruction cache access or Level 0 Macro-op cache access. This event counts any instruction fetch which accesses the L1 instruction cache or L0 Macro-op cache\000 */
{ 118524 }, /* l1i_cache_refill\000\000cache\000Level 1 instruction cache refill\000event=0x1\000\00000\000This event counts any instruction fetch which misses in the cache\000 */
{ 119689 }, /* l1i_tlb\000\000cache\000Level 1 instruction TLB access\000event=0x26\000\00000\000This event counts any instruction fetch which accesses the instruction L1 TLB.This event counts regardless of whether the MMU is enabled\000 */
{ 120322 }, /* l1i_tlb_refill\000\000cache\000Attributable Level 1 instruction TLB refill\000event=0x2\000\00000\000This event counts any refill of the instruction L1 TLB from the L2 TLB. This includes refills that result in a translation fault\000 */
{ 126328 }, /* l2d_cache\000\000cache\000Level 2 data cache access\000event=0x16\000\00000\000This event counts any transaction from L1 which looks up in the L2 cache, and any write-back from the L1 to the L2. Snoops from outside the core and cache maintenance operations are not counted\000 */
{ 127079 }, /* l2d_cache_allocate\000\000cache\000Attributable Level 2 data cache allocation without refill\000event=0x20\000\00000\000This event counts any full cache line write into the L2 cache which does not cause a linefill, including write-backs from L1 to L2 and full-line writes which do not allocate into L1\000 */
{ 127466 }, /* l2d_cache_inval\000\000cache\000L2D cache invalidate\000event=0x58\000\00000\000Attributable Level 2 data cache invalidate\000 */
{ 129126 }, /* l2d_cache_rd\000\000cache\000L2D cache access, read\000event=0x50\000\00000\000Attributable Level 2 data cache access, read\000 */
{ 129618 }, /* l2d_cache_refill\000\000cache\000Level 2 data refill\000event=0x17\000\00000\000L2 data cache refill. This event counts any cacheable transaction from L1 which causes data to be read from outside the core. L2 refills caused by stashes into L2 should not be counted\000 */
{ 132288 }, /* l2d_cache_refill_rd\000\000cache\000L2D cache refill, read\000event=0x52\000\00000\000Attributable Level 2 data cache refill, read\000 */
{ 132829 }, /* l2d_cache_refill_wr\000\000cache\000L2D cache refill, write\000event=0x53\000\00000\000Attributable Level 2 data cache refill, write\000 */
{ 136535 }, /* l2d_cache_wb\000\000cache\000Attributable Level 2 data cache write-back\000event=0x18\000\00000\000This event counts any write-back of data from the L2 cache to outside the core. This includes snoops to the L2 which return data, regardless of whether they cause an invalidation. Invalidations from the L2 which do not write data outside of the core and snoops which return data from the L1 are not counted\000 */
{ 137387 }, /* l2d_cache_wb_clean\000\000cache\000L2D cache Write-Back, cleaning and coherency\000event=0x57\000\00000\000Level 2 data cache Write-Back, cleaning and coherency\000 */
{ 137968 }, /* l2d_cache_wb_victim\000\000cache\000L2D cache Write-Back, victim\000event=0x56\000\00000\000Attributable Level 2 data cache Write-Back, victim\000 */
{ 138385 }, /* l2d_cache_wr\000\000cache\000L2D cache access, write\000event=0x51\000\00000\000Attributable Level 2 data cache access, write\000 */
{ 143515 }, /* l2d_tlb\000\000cache\000Attributable Level 2 data or unified TLB access\000event=0x2f\000\00000\000This event counts on any access to the L2 TLB (caused by a refill of any of the L1 TLBs). This event does not count if the MMU is disabled\000 */
{ 144070 }, /* l2d_tlb_rd\000\000cache\000L2D cache access, read\000event=0x5e\000\00000\000Attributable Level 2 data or unified TLB access, read\000 */
{ 144503 }, /* l2d_tlb_refill\000\000cache\000Attributable L2 data or unified TLB refill\000event=0x2d\000\00000\000This event counts on anyrefill of the L2 TLB, caused by either an instruction or data access.This event does not count if the MMU is disabled\000 */
{ 145079 }, /* l2d_tlb_refill_rd\000\000cache\000L2D cache refill, read\000event=0x5c\000\00000\000Attributable Level 2 data or unified TLB refill, read\000 */
{ 145555 }, /* l2d_tlb_refill_wr\000\000cache\000L2D cache refill, write\000event=0x5d\000\00000\000Attributable Level 2 data or unified TLB refill, write\000 */
{ 146014 }, /* l2d_tlb_wr\000\000cache\000L2D cache access, write\000event=0x5f\000\00000\000Attributable Level 2 data or unified TLB access, write\000 */
{ 150751 }, /* l3d_cache\000\000cache\000Attributable Level 3 unified cache access\000event=0x2b\000\00000\000This event counts for any cacheable read transaction returning datafrom the SCU, or for any cacheable write to the SCU\000 */
{ 151210 }, /* l3d_cache_allocate\000\000cache\000Allocation without refill\000event=0x29\000\00000\000This event counts any full cache line write into the L3 cache which does not cause a linefill, including write-backs from L2 to L3 and full-line writes which do not allocate into L2\000 */
{ 152420 }, /* l3d_cache_rd\000\000cache\000Attributable Level 3 data or unified cache access, read\000event=0xa0\000\00000\000Attributable Level 3 data or unified cache access, read\000 */
{ 152782 }, /* l3d_cache_refill\000\000cache\000Attributable Level 3 unified cache refill\000event=0x2a\000\00000\000This event counts for any cacheable read transaction returning datafrom the SCU for which the data source was outside the cluster. Transactions such as ReadUnique are counted here as 'read' transactions, even though they can be generated by store instructions\000 */
{ 160229 }, /* ll_cache_miss_rd\000\000cache\000Last level cache miss, read\000event=0x37\000\00000\000Last level cache miss, read\000 */
{ 160879 }, /* ll_cache_rd\000\000cache\000Attributable Last level cache memory read\000event=0x36\000\00000\000Attributable Last level cache memory read\000 */
{ 56959 }, /* exc_dabort\000\000exception\000Exception taken, Data Abort and SError\000event=0x84\000\00000\000Exception taken, Data Abort and SError\000 */
{ 57280 }, /* exc_fiq\000\000exception\000Exception taken, FIQ\000event=0x87\000\00000\000Exception taken, FIQ\000 */
{ 57633 }, /* exc_hvc\000\000exception\000Exception taken, Hypervisor Call\000event=0x8a\000\00000\000Exception taken, Hypervisor Call\000 */
{ 57939 }, /* exc_irq\000\000exception\000Exception taken, IRQ\000event=0x86\000\00000\000Exception taken, IRQ\000 */
{ 58355 }, /* exc_pabort\000\000exception\000Exception taken, Instruction Abort\000event=0x83\000\00000\000Exception taken, Instruction Abort\000 */
{ 59258 }, /* exc_smc\000\000exception\000Exception taken, Secure Monitor Call\000event=0x88\000\00000\000Exception taken, Secure Monitor Call\000 */
{ 59572 }, /* exc_svc\000\000exception\000Exception taken, Supervisor Call\000event=0x82\000\00000\000Exception taken, Supervisor Call\000 */
{ 59896 }, /* exc_taken\000\000exception\000Exception taken\000event=0x9\000\00000\000Exception taken\000 */
{ 60575 }, /* exc_trap_dabort\000\000exception\000Exception taken, Data Abort or SError not taken locally\000event=0x8c\000\00000\000Exception taken, Data Abort or SError not taken locally\000 */
{ 61080 }, /* exc_trap_fiq\000\000exception\000Exception taken, FIQ not taken locally\000event=0x8f\000\00000\000Exception taken, FIQ not taken locally\000 */
{ 62347 }, /* exc_trap_irq\000\000exception\000Exception taken, IRQ not taken locally\000event=0x8e\000\00000\000Exception taken, IRQ not taken locally\000 */
{ 62841 }, /* exc_trap_other\000\000exception\000Exception taken, Other traps not taken locally\000event=0x8d\000\00000\000Exception taken, Other traps not taken locally\000 */
{ 63383 }, /* exc_trap_pabort\000\000exception\000Exception taken, Instruction Abort not taken locally\000event=0x8b\000\00000\000Exception taken, Instruction Abort not taken locally\000 */
{ 64129 }, /* exc_undef\000\000exception\000Exception taken, Other synchronous\000event=0x81\000\00000\000Exception taken, Other synchronous\000 */
{ 165947 }, /* memory_error\000\000exception\000Local memory error\000event=0x1a\000\00000\000This event counts any correctable or uncorrectable memory error (ECC or parity) in the protected core RAMs\000 */
{ 20878 }, /* ase_spec\000\000instruction\000Operation speculatively executed, Advanced SIMD instruction\000event=0x74\000\00000\000Operation speculatively executed, Advanced SIMD instruction\000 */
{ 30886 }, /* br_immed_spec\000\000instruction\000Branch speculatively executed, immediate branch\000event=0x78\000\00000\000Branch speculatively executed, immediate branch\000 */
{ 35172 }, /* br_indirect_spec\000\000instruction\000Branch speculatively executed, indirect branch\000event=0x7a\000\00000\000Branch speculatively executed, indirect branch\000 */
{ 36515 }, /* br_mis_pred_retired\000\000instruction\000Instruction architecturally executed, mispredicted branch\000event=0x22\000\00000\000This event counts any branch counted by BR_RETIRED which is not correctly predicted and causes a pipeline flush\000 */
{ 37522 }, /* br_retired\000\000instruction\000Instruction architecturally executed, branch\000event=0x21\000\00000\000This event counts all branches, taken or not. This excludes exception entries, debug entries and CCFAIL branches\000 */
{ 40826 }, /* br_return_spec\000\000instruction\000Branch speculatively executed, procedure return\000event=0x79\000\00000\000Branch speculatively executed, procedure return\000 */
{ 45761 }, /* cid_write_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000event=0xb\000\00000\000This event only counts writes to CONTEXTIDR in AArch32 state, and via the CONTEXTIDR_EL1 mnemonic in AArch64 state\000 */
{ 48419 }, /* crypto_spec\000\000instruction\000Operation speculatively executed, Cryptographic instruction\000event=0x77\000\00000\000Operation speculatively executed, Cryptographic instruction\000 */
{ 51117 }, /* dmb_spec\000\000instruction\000Barrier speculatively executed, DMB\000event=0x7e\000\00000\000Barrier speculatively executed, DMB\000 */
{ 52153 }, /* dp_spec\000\000instruction\000Operation speculatively executed, integer data processing\000event=0x73\000\00000\000Operation speculatively executed, integer data processing\000 */
{ 53209 }, /* dsb_spec\000\000instruction\000Barrier speculatively executed, DSB\000event=0x7d\000\00000\000Barrier speculatively executed, DSB\000 */
{ 58850 }, /* exc_return\000\000instruction\000Instruction architecturally executed, condition check pass, exception return\000event=0xa\000\00000\000Instruction architecturally executed, condition check pass, exception return\000 */
{ 90933 }, /* inst_retired\000\000instruction\000Instruction architecturally executed\000event=0x8\000\00000\000This event counts all retired instructions, including those that fail their condition check\000 */
{ 91235 }, /* inst_spec\000\000instruction\000Operation speculatively executed\000event=0x1b\000\00000\000Operation speculatively executed\000 */
{ 91769 }, /* isb_spec\000\000instruction\000Barrier speculatively executed, ISB\000event=0x7c\000\00000\000Barrier speculatively executed, ISB\000 */
{ 159047 }, /* ld_spec\000\000instruction\000Operation speculatively executed, load\000event=0x70\000\00000\000Operation speculatively executed, load\000 */
{ 159351 }, /* ldrex_spec\000\000instruction\000Exclusive operation speculatively executed, LDREX or LDX\000event=0x6c\000\00000\000Exclusive operation speculatively executed, LDREX or LDX\000 */
{ 159973 }, /* ldst_spec\000\000instruction\000Operation speculatively executed, load or store\000event=0x72\000\00000\000Operation speculatively executed, load or store\000 */
{ 175115 }, /* pc_write_spec\000\000instruction\000Operation speculatively executed, software change of the PC\000event=0x76\000\00000\000Operation speculatively executed, software change of the PC\000 */
{ 177878 }, /* rc_ld_spec\000\000instruction\000Release consistency operation speculatively executed, Load-Acquire\000event=0x90\000\00000\000Release consistency operation speculatively executed, Load-Acquire\000 */
{ 178251 }, /* rc_st_spec\000\000instruction\000Release consistency operation speculatively executed, Store-Release\000event=0x91\000\00000\000Release consistency operation speculatively executed, Store-Release\000 */
{ 185522 }, /* st_spec\000\000instruction\000Operation speculatively executed, store\000event=0x71\000\00000\000Operation speculatively executed, store\000 */
{ 200454 }, /* strex_fail_spec\000\000instruction\000Exclusive operation speculatively executed, STREX or STX fail\000event=0x6e\000\00000\000Exclusive operation speculatively executed, STREX or STX fail\000 */
{ 201025 }, /* strex_pass_spec\000\000instruction\000Exclusive operation speculatively executed, STREX or STX pass\000event=0x6d\000\00000\000Exclusive operation speculatively executed, STREX or STX pass\000 */
{ 201592 }, /* strex_spec\000\000instruction\000Exclusive operation speculatively executed, STREX or STX\000event=0x6f\000\00000\000Exclusive operation speculatively executed, STREX or STX\000 */
{ 205736 }, /* sw_incr\000\000instruction\000Instruction architecturally executed, Condition code check pass, software increment\000event=0\000\00000\000Instruction architecturally executed, Condition code check pass, software increment\000 */
{ 210499 }, /* ttbr_write_retired\000\000instruction\000Instruction architecturally executed, Condition code check pass, write to TTBR\000event=0x1c\000\00000\000This event only counts writes to TTBR0/TTBR1 in AArch32 state and TTBR0_EL1/TTBR1_EL1 in AArch64 state\000 */
{ 214354 }, /* vfp_spec\000\000instruction\000Operation speculatively executed, floating-point instruction\000event=0x75\000\00000\000Operation speculatively executed, floating-point instruction\000 */
{ 163220 }, /* mem_access\000\000memory\000Data memory access\000event=0x13\000\00000\000This event counts memory accesses due to load or store instructions. This event counts the sum of MEM_ACCESS_RD and MEM_ACCESS_WR\000 */
{ 164298 }, /* mem_access_rd\000\000memory\000Data memory access, read\000event=0x66\000\00000\000Data memory access, read\000 */
{ 164956 }, /* mem_access_wr\000\000memory\000Data memory access, write\000event=0x67\000\00000\000Data memory access, write\000 */
{ 179852 }, /* remote_access\000\000memory\000Access to another socket in a multi-socket system\000event=0x31\000\00000\000Access to another socket in a multi-socket system\000 */
{ 211505 }, /* unaligned_ld_spec\000\000memory\000Unaligned access, read\000event=0x68\000\00000\000Unaligned access, read\000 */
{ 212200 }, /* unaligned_ldst_spec\000\000memory\000Unaligned access\000event=0x6a\000\00000\000Unaligned access\000 */
{ 212637 }, /* unaligned_st_spec\000\000memory\000Unaligned access, write\000event=0x69\000\00000\000Unaligned access, write\000 */
{ 186189 }, /* stall_backend\000\000pipeline\000No operation issued due to the backend\000event=0x24\000\00000\000The counter counts on any cycle fetched instructions are not dispatched due to resource constraints\000 */
{ 196003 }, /* stall_frontend\000\000pipeline\000No operation issued because of the frontend\000event=0x23\000\00000\000The counter counts on any cycle when there are no fetched instructions available to dispatch\000 */
};

static const struct compact_pmu_event pmu_events__arm_cortex_a77[] = {
{ 30753 }, /* br_immed_spec\000\000branch\000Branch speculatively executed, immediate branch\000event=0x78\000\00000\000Branch speculatively executed, immediate branch\000 */
{ 35038 }, /* br_indirect_spec\000\000branch\000Branch speculatively executed, indirect branch\000event=0x7a\000\00000\000Branch speculatively executed, indirect branch\000 */
{ 35743 }, /* br_mis_pred\000\000branch\000Mispredicted or not predicted branch speculatively executed\000event=0x10\000\00000\000Mispredicted or not predicted branch speculatively executed\000 */
{ 36925 }, /* br_pred\000\000branch\000Predictable branch speculatively executed\000event=0x12\000\00000\000Predictable branch speculatively executed\000 */
{ 40692 }, /* br_return_spec\000\000branch\000Branch speculatively executed, procedure return\000event=0x79\000\00000\000Branch speculatively executed, procedure return\000 */
{ 41358 }, /* bus_access\000\000bus\000Attributable Bus access\000event=0x19\000\00000\000Attributable Bus access\000 */
{ 42271 }, /* bus_access_rd\000\000bus\000Bus access read\000event=0x60\000\00000\000Bus access read\000 */
{ 42682 }, /* bus_access_wr\000\000bus\000Bus access write\000event=0x61\000\00000\000Bus access write\000 */
{ 42979 }, /* bus_cycles\000\000bus\000Bus cycle\000event=0x1d\000\00000\000Bus cycle\000 */
{ 47992 }, /* cpu_cycles\000\000bus\000Cycle\000event=0x11\000\00000\000Cycle\000 */
{ 53526 }, /* dtlb_walk\000\000cache\000Access to data TLB causes a translation table walk\000event=0x34\000\00000\000Access to data TLB causes a translation table walk\000 */
{ 92044 }, /* itlb_walk\000\000cache\000Access to instruction TLB that causes a translation table walk\000event=0x35\000\00000\000Access to instruction TLB that causes a translation table walk\000 */
{ 100289 }, /* l1d_cache\000\000cache\000Level 1 data cache access\000event=0x4\000\00000\000Level 1 data cache access\000 */
{ 101403 }, /* l1d_cache_inval\000\000cache\000L1D cache invalidate\000event=0x48\000\00000\000Attributable Level 1 data cache invalidate\000 */
{ 102693 }, /* l1d_cache_rd\000\000cache\000L1D cache access, read\000event=0x40\000\00000\000Attributable Level 1 data cache access, read\000 */
{ 103129 }, /* l1d_cache_refill\000\000cache\000Level 1 data cache refill\000event=0x3\000\00000\000Level 1 data cache refill\000 */
{ 104145 }, /* l1d_cache_refill_inner\000\000cache\000L1D cache refill, inner\000event=0x44\000\00000\000Attributable Level 1 data cache refill, inner\000 */
{ 104573 }, /* l1d_cache_refill_outer\000\000cache\000L1D cache refill, outer\000event=0x45\000\00000\000Attributable Level 1 data cache refill, outer\000 */
{ 105894 }, /* l1d_cache_refill_rd\000\000cache\000L1D cache refill, read\000event=0x42\000\00000\000Attributable Level 1 data cache refill, read\000 */
{ 106389 }, /* l1d_cache_refill_wr\000\000cache\000L1D cache refill, write\000event=0x43\000\00000\000Attributable Level 1 data cache refill, write\000 */
{ 106891 }, /* l1d_cache_wb\000\000cache\000Attributable Level 1 data cache write-back\000event=0x15\000\00000\000Attributable Level 1 data cache write-back\000 */
{ 107957 }, /* l1d_cache_wb_clean\000\000cache\000L1D cache Write-Back, cleaning and coherency\000event=0x47\000\00000\000Level 1 data cache Write-Back, cleaning and coherency\000 */
{ 108495 }, /* l1d_cache_wb_victim\000\000cache\000L1D cache Write-Back, victim\000event=0x46\000\00000\000Attributable Level 1 data cache Write-Back, victim\000 */
{ 108997 }, /* l1d_cache_wr\000\000cache\000L1D cache access, write\000event=0x41\000\00000\000Attributable Level 1 data cache access, write\000 */
{ 109867 }, /* l1d_tlb\000\000cache\000Attributable Level 1 data or unified TLB access\000event=0x25\000\00000\000Attributable Level 1 data or unified TLB access\000 */
{ 110748 }, /* l1d_tlb_rd\000\000cache\000L1D tlb access, read\000event=0x4e\000\00000\000Attributable Level 1 data or unified TLB access, read\000 */
{ 111207 }, /* l1d_tlb_refill\000\000cache\000Attributable Level 1 data TLB refill\000event=0x5\000\00000\000Attributable Level 1 data TLB refill\000 */
{ 112199 }, /* l1d_tlb_refill_rd\000\000cache\000L1D tlb refill, read\000event=0x4c\000\00000\000Attributable Level 1 data TLB refill, read\000 */
{ 113093 }, /* l1d_tlb_refill_wr\000\000cache\000L1D tlb refill, write\000event=0x4d\000\00000\000Attributable Level 1 data TLB refill, write\000 */
{ 113993 }, /* l1d_tlb_wr\000\000cache\000L1D tlb access, write\000event=0x4f\000\00000\000Attributable Level 1 data or unified TLB access, write\000 */
{ 116626 }, /* l1i_cache\000\000cache\000Attributable Level 1 instruction cache access\000event=0x14\000\00000\000Attributable Level 1 instruction cache access\000 */
{ 118420 }, /* l1i_cache_refill\000\000cache\000Level 1 instruction cache refill\000event=0x1\000\00000\000Level 1 instruction cache refill\000 */
{ 119419 }, /* l1i_tlb\000\000cache\000Attributable Level 1 instruction TLB access\000event=0x26\000\00000\000Attributable Level 1 instruction TLB access\000 */
{ 120198 }, /* l1i_tlb_refill\000\000cache\000Attributable Level 1 instruction TLB refill\000event=0x2\000\00000\000Attributable Level 1 instruction TLB refill\000 */
{ 126244 }, /* l2d_cache\000\000cache\000Level 2 data cache access\000event=0x16\000\00000\000Level 2 data cache access\000 */
{ 126922 }, /* l2d_cache_allocate\000\000cache\000Attributable Level 2 data cache allocation without refill\000event=0x20\000\00000\000Attributable Level 2 data cache allocation without refill\000 */
{ 127466 }, /* l2d_cache_inval\000\000cache\000L2D cache invalidate\000event=0x58\000\00000\000Attributable Level 2 data cache invalidate\000 */
{ 129126 }, /* l2d_cache_rd\000\000cache\000L2D cache access, read\000event=0x50\000\00000\000Attributable Level 2 data cache access, read\000 */
{ 129862 }, /* l2d_cache_refill\000\000cache\000Level 2 data refill\000event=0x17\000\00000\000Level 2 data refill\000 */
{ 132288 }, /* l2d_cache_refill_rd\000\000cache\000L2D cache refill, read\000event=0x52\000\00000\000Attributable Level 2 data cache refill, read\000 */
{ 132829 }, /* l2d_cache_refill_wr\000\000cache\000L2D cache refill, write\000event=0x53\000\00000\000Attributable Level 2 data cache refill, write\000 */
{ 136414 }, /* l2d_cache_wb\000\000cache\000Attributable Level 2 data cache write-back\000event=0x18\000\00000\000Attributable Level 2 data cache write-back\000 */
{ 137387 }, /* l2d_cache_wb_clean\000\000cache\000L2D cache Write-Back, cleaning and coherency\000event=0x57\000\00000\000Level 2 data cache Write-Back, cleaning and coherency\000 */
{ 137968 }, /* l2d_cache_wb_victim\000\000cache\000L2D cache Write-Back, victim\000event=0x56\000\00000\000Attributable Level 2 data cache Write-Back, victim\000 */
{ 138385 }, /* l2d_cache_wr\000\000cache\000L2D cache access, write\000event=0x51\000\00000\000Attributable Level 2 data cache access, write\000 */
{ 143389 }, /* l2d_tlb\000\000cache\000Attributable Level 2 data or unified TLB access\000event=0x2f\000\00000\000Attributable Level 2 data or unified TLB access\000 */
{ 144070 }, /* l2d_tlb_rd\000\000cache\000L2D cache access, read\000event=0x5e\000\00000\000Attributable Level 2 data or unified TLB access, read\000 */
{ 144725 }, /* l2d_tlb_refill\000\000cache\000Attributable Level 2 data TLB refill\000event=0x2d\000\00000\000Attributable Level 2 data TLB refill\000 */
{ 145079 }, /* l2d_tlb_refill_rd\000\000cache\000L2D cache refill, read\000event=0x5c\000\00000\000Attributable Level 2 data or unified TLB refill, read\000 */
{ 145555 }, /* l2d_tlb_refill_wr\000\000cache\000L2D cache refill, write\000event=0x5d\000\00000\000Attributable Level 2 data or unified TLB refill, write\000 */
{ 146014 }, /* l2d_tlb_wr\000\000cache\000L2D cache access, write\000event=0x5f\000\00000\000Attributable Level 2 data or unified TLB access, write\000 */
{ 150641 }, /* l3d_cache\000\000cache\000Attributable Level 3 data cache access\000event=0x2b\000\00000\000Attributable Level 3 data cache access\000 */
{ 151459 }, /* l3d_cache_allocate\000\000cache\000Attributable Level 3 data cache allocation without refill\000event=0x29\000\00000\000Attributable Level 3 data cache allocation without refill\000 */
{ 152420 }, /* l3d_cache_rd\000\000cache\000Attributable Level 3 data or unified cache access, read\000event=0xa0\000\00000\000Attributable Level 3 data or unified cache access, read\000 */
{ 152665 }, /* l3d_cache_refill\000\000cache\000Attributable Level 3 data cache refill\000event=0x2a\000\00000\000Attributable Level 3 data cache refill\000 */
{ 160229 }, /* ll_cache_miss_rd\000\000cache\000Last level cache miss, read\000event=0x37\000\00000\000Last level cache miss, read\000 */
{ 160879 }, /* ll_cache_rd\000\000cache\000Attributable Last level cache memory read\000event=0x36\000\00000\000Attributable Last level cache memory read\000 */
{ 56959 }, /* exc_dabort\000\000exception\000Exception taken, Data Abort and SError\000event=0x84\000\00000\000Exception taken, Data Abort and SError\000 */
{ 57280 }, /* exc_fiq\000\000exception\000Exception taken, FIQ\000event=0x87\000\00000\000Exception taken, FIQ\000 */
{ 57633 }, /* exc_hvc\000\000exception\000Exception taken, Hypervisor Call\000event=0x8a\000\00000\000Exception taken, Hypervisor Call\000 */
{ 57939 }, /* exc_irq\000\000exception\000Exception taken, IRQ\000event=0x86\000\00000\000Exception taken, IRQ\000 */
{ 58355 }, /* exc_pabort\000\000exception\000Exception taken, Instruction Abort\000event=0x83\000\00000\000Exception taken, Instruction Abort\000 */
{ 59258 }, /* exc_smc\000\000exception\000Exception taken, Secure Monitor Call\000event=0x88\000\00000\000Exception taken, Secure Monitor Call\000 */
{ 59572 }, /* exc_svc\000\000exception\000Exception taken, Supervisor Call\000event=0x82\000\00000\000Exception taken, Supervisor Call\000 */
{ 59896 }, /* exc_taken\000\000exception\000Exception taken\000event=0x9\000\00000\000Exception taken\000 */
{ 60575 }, /* exc_trap_dabort\000\000exception\000Exception taken, Data Abort or SError not taken locally\000event=0x8c\000\00000\000Exception taken, Data Abort or SError not taken locally\000 */
{ 61080 }, /* exc_trap_fiq\000\000exception\000Exception taken, FIQ not taken locally\000event=0x8f\000\00000\000Exception taken, FIQ not taken locally\000 */
{ 62347 }, /* exc_trap_irq\000\000exception\000Exception taken, IRQ not taken locally\000event=0x8e\000\00000\000Exception taken, IRQ not taken locally\000 */
{ 62841 }, /* exc_trap_other\000\000exception\000Exception taken, Other traps not taken locally\000event=0x8d\000\00000\000Exception taken, Other traps not taken locally\000 */
{ 63383 }, /* exc_trap_pabort\000\000exception\000Exception taken, Instruction Abort not taken locally\000event=0x8b\000\00000\000Exception taken, Instruction Abort not taken locally\000 */
{ 64129 }, /* exc_undef\000\000exception\000Exception taken, Other synchronous\000event=0x81\000\00000\000Exception taken, Other synchronous\000 */
{ 165870 }, /* memory_error\000\000exception\000Local memory error\000event=0x1a\000\00000\000Local memory error\000 */
{ 20878 }, /* ase_spec\000\000instruction\000Operation speculatively executed, Advanced SIMD instruction\000event=0x74\000\00000\000Operation speculatively executed, Advanced SIMD instruction\000 */
{ 36351 }, /* br_mis_pred_retired\000\000instruction\000Instruction architecturally executed, mispredicted branch\000event=0x22\000\00000\000Instruction architecturally executed, mispredicted branch\000 */
{ 37393 }, /* br_retired\000\000instruction\000Instruction architecturally executed, branch\000event=0x21\000\00000\000Instruction architecturally executed, branch\000 */
{ 45546 }, /* cid_write_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000event=0xb\000\00000\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000 */
{ 48419 }, /* crypto_spec\000\000instruction\000Operation speculatively executed, Cryptographic instruction\000event=0x77\000\00000\000Operation speculatively executed, Cryptographic instruction\000 */
{ 51117 }, /* dmb_spec\000\000instruction\000Barrier speculatively executed, DMB\000event=0x7e\000\00000\000Barrier speculatively executed, DMB\000 */
{ 52153 }, /* dp_spec\000\000instruction\000Operation speculatively executed, integer data processing\000event=0x73\000\00000\000Operation speculatively executed, integer data processing\000 */
{ 53209 }, /* dsb_spec\000\000instruction\000Barrier speculatively executed, DSB\000event=0x7d\000\00000\000Barrier speculatively executed, DSB\000 */
{ 58850 }, /* exc_return\000\000instruction\000Instruction architecturally executed, condition check pass, exception return\000event=0xa\000\00000\000Instruction architecturally executed, condition check pass, exception return\000 */
{ 90819 }, /* inst_retired\000\000instruction\000Instruction architecturally executed\000event=0x8\000\00000\000Instruction architecturally executed\000 */
{ 91235 }, /* inst_spec\000\000instruction\000Operation speculatively executed\000event=0x1b\000\00000\000Operation speculatively executed\000 */
{ 91769 }, /* isb_spec\000\000instruction\000Barrier speculatively executed, ISB\000event=0x7c\000\00000\000Barrier speculatively executed, ISB\000 */
{ 159047 }, /* ld_spec\000\000instruction\000Operation speculatively executed, load\000event=0x70\000\00000\000Operation speculatively executed, load\000 */
{ 159351 }, /* ldrex_spec\000\000instruction\000Exclusive operation speculatively executed, LDREX or LDX\000event=0x6c\000\00000\000Exclusive operation speculatively executed, LDREX or LDX\000 */
{ 159973 }, /* ldst_spec\000\000instruction\000Operation speculatively executed, load or store\000event=0x72\000\00000\000Operation speculatively executed, load or store\000 */
{ 175115 }, /* pc_write_spec\000\000instruction\000Operation speculatively executed, software change of the PC\000event=0x76\000\00000\000Operation speculatively executed, software change of the PC\000 */
{ 177878 }, /* rc_ld_spec\000\000instruction\000Release consistency operation speculatively executed, Load-Acquire\000event=0x90\000\00000\000Release consistency operation speculatively executed, Load-Acquire\000 */
{ 178251 }, /* rc_st_spec\000\000instruction\000Release consistency operation speculatively executed, Store-Release\000event=0x91\000\00000\000Release consistency operation speculatively executed, Store-Release\000 */
{ 185522 }, /* st_spec\000\000instruction\000Operation speculatively executed, store\000event=0x71\000\00000\000Operation speculatively executed, store\000 */
{ 200454 }, /* strex_fail_spec\000\000instruction\000Exclusive operation speculatively executed, STREX or STX fail\000event=0x6e\000\00000\000Exclusive operation speculatively executed, STREX or STX fail\000 */
{ 201025 }, /* strex_pass_spec\000\000instruction\000Exclusive operation speculatively executed, STREX or STX pass\000event=0x6d\000\00000\000Exclusive operation speculatively executed, STREX or STX pass\000 */
{ 201592 }, /* strex_spec\000\000instruction\000Exclusive operation speculatively executed, STREX or STX\000event=0x6f\000\00000\000Exclusive operation speculatively executed, STREX or STX\000 */
{ 205736 }, /* sw_incr\000\000instruction\000Instruction architecturally executed, Condition code check pass, software increment\000event=0\000\00000\000Instruction architecturally executed, Condition code check pass, software increment\000 */
{ 210294 }, /* ttbr_write_retired\000\000instruction\000Instruction architecturally executed, Condition code check pass, write to TTBR\000event=0x1c\000\00000\000Instruction architecturally executed, Condition code check pass, write to TTBR\000 */
{ 214354 }, /* vfp_spec\000\000instruction\000Operation speculatively executed, floating-point instruction\000event=0x75\000\00000\000Operation speculatively executed, floating-point instruction\000 */
{ 163148 }, /* mem_access\000\000memory\000Data memory access\000event=0x13\000\00000\000Data memory access\000 */
{ 164298 }, /* mem_access_rd\000\000memory\000Data memory access, read\000event=0x66\000\00000\000Data memory access, read\000 */
{ 164956 }, /* mem_access_wr\000\000memory\000Data memory access, write\000event=0x67\000\00000\000Data memory access, write\000 */
{ 179852 }, /* remote_access\000\000memory\000Access to another socket in a multi-socket system\000event=0x31\000\00000\000Access to another socket in a multi-socket system\000 */
{ 211505 }, /* unaligned_ld_spec\000\000memory\000Unaligned access, read\000event=0x68\000\00000\000Unaligned access, read\000 */
{ 212200 }, /* unaligned_ldst_spec\000\000memory\000Unaligned access\000event=0x6a\000\00000\000Unaligned access\000 */
{ 212637 }, /* unaligned_st_spec\000\000memory\000Unaligned access, write\000event=0x69\000\00000\000Unaligned access, write\000 */
{ 186072 }, /* stall_backend\000\000pipeline\000No operation issued due to the backend\000event=0x24\000\00000\000No operation issued due to the backend\000 */
{ 195875 }, /* stall_frontend\000\000pipeline\000No operation issued because of the frontend\000event=0x23\000\00000\000No operation issued because of the frontend\000 */
};

static const struct compact_pmu_event pmu_events__arm_cortex_a78[] = {
{ 30753 }, /* br_immed_spec\000\000branch\000Branch speculatively executed, immediate branch\000event=0x78\000\00000\000Branch speculatively executed, immediate branch\000 */
{ 35038 }, /* br_indirect_spec\000\000branch\000Branch speculatively executed, indirect branch\000event=0x7a\000\00000\000Branch speculatively executed, indirect branch\000 */
{ 35743 }, /* br_mis_pred\000\000branch\000Mispredicted or not predicted branch speculatively executed\000event=0x10\000\00000\000Mispredicted or not predicted branch speculatively executed\000 */
{ 36925 }, /* br_pred\000\000branch\000Predictable branch speculatively executed\000event=0x12\000\00000\000Predictable branch speculatively executed\000 */
{ 40692 }, /* br_return_spec\000\000branch\000Branch speculatively executed, procedure return\000event=0x79\000\00000\000Branch speculatively executed, procedure return\000 */
{ 41358 }, /* bus_access\000\000bus\000Attributable Bus access\000event=0x19\000\00000\000Attributable Bus access\000 */
{ 42271 }, /* bus_access_rd\000\000bus\000Bus access read\000event=0x60\000\00000\000Bus access read\000 */
{ 42682 }, /* bus_access_wr\000\000bus\000Bus access write\000event=0x61\000\00000\000Bus access write\000 */
{ 42979 }, /* bus_cycles\000\000bus\000Bus cycle\000event=0x1d\000\00000\000Bus cycle\000 */
{ 46424 }, /* cnt_cycles\000\000bus\000Constant frequency cycles\000event=0x4004\000\00000\000Constant frequency cycles.  The counter increments at a constant frequency equal to the rate of increment of the system counter, CNTPCT_EL0\000 */
{ 47992 }, /* cpu_cycles\000\000bus\000Cycle\000event=0x11\000\00000\000Cycle\000 */
{ 53526 }, /* dtlb_walk\000\000cache\000Access to data TLB causes a translation table walk\000event=0x34\000\00000\000Access to data TLB causes a translation table walk\000 */
{ 92044 }, /* itlb_walk\000\000cache\000Access to instruction TLB that causes a translation table walk\000event=0x35\000\00000\000Access to instruction TLB that causes a translation table walk\000 */
{ 100289 }, /* l1d_cache\000\000cache\000Level 1 data cache access\000event=0x4\000\00000\000Level 1 data cache access\000 */
{ 101403 }, /* l1d_cache_inval\000\000cache\000L1D cache invalidate\000event=0x48\000\00000\000Attributable Level 1 data cache invalidate\000 */
{ 102276 }, /* l1d_cache_lmiss_rd\000\000cache\000Level 1 data cache long-latency read miss\000event=0x39\000\00000\000Level 1 data cache long-latency read miss.  The counter counts each memory read access counted by L1D_CACHE that incurs additional latency because it returns data from outside the Level 1 data or unified cache of this processing element\000 */
{ 102693 }, /* l1d_cache_rd\000\000cache\000L1D cache access, read\000event=0x40\000\00000\000Attributable Level 1 data cache access, read\000 */
{ 103129 }, /* l1d_cache_refill\000\000cache\000Level 1 data cache refill\000event=0x3\000\00000\000Level 1 data cache refill\000 */
{ 104145 }, /* l1d_cache_refill_inner\000\000cache\000L1D cache refill, inner\000event=0x44\000\00000\000Attributable Level 1 data cache refill, inner\000 */
{ 104573 }, /* l1d_cache_refill_outer\000\000cache\000L1D cache refill, outer\000event=0x45\000\00000\000Attributable Level 1 data cache refill, outer\000 */
{ 105894 }, /* l1d_cache_refill_rd\000\000cache\000L1D cache refill, read\000event=0x42\000\00000\000Attributable Level 1 data cache refill, read\000 */
{ 106389 }, /* l1d_cache_refill_wr\000\000cache\000L1D cache refill, write\000event=0x43\000\00000\000Attributable Level 1 data cache refill, write\000 */
{ 106891 }, /* l1d_cache_wb\000\000cache\000Attributable Level 1 data cache write-back\000event=0x15\000\00000\000Attributable Level 1 data cache write-back\000 */
{ 107957 }, /* l1d_cache_wb_clean\000\000cache\000L1D cache Write-Back, cleaning and coherency\000event=0x47\000\00000\000Level 1 data cache Write-Back, cleaning and coherency\000 */
{ 108495 }, /* l1d_cache_wb_victim\000\000cache\000L1D cache Write-Back, victim\000event=0x46\000\00000\000Attributable Level 1 data cache Write-Back, victim\000 */
{ 108997 }, /* l1d_cache_wr\000\000cache\000L1D cache access, write\000event=0x41\000\00000\000Attributable Level 1 data cache access, write\000 */
{ 109867 }, /* l1d_tlb\000\000cache\000Attributable Level 1 data or unified TLB access\000event=0x25\000\00000\000Attributable Level 1 data or unified TLB access\000 */
{ 110748 }, /* l1d_tlb_rd\000\000cache\000L1D tlb access, read\000event=0x4e\000\00000\000Attributable Level 1 data or unified TLB access, read\000 */
{ 111207 }, /* l1d_tlb_refill\000\000cache\000Attributable Level 1 data TLB refill\000event=0x5\000\00000\000Attributable Level 1 data TLB refill\000 */
{ 112199 }, /* l1d_tlb_refill_rd\000\000cache\000L1D tlb refill, read\000event=0x4c\000\00000\000Attributable Level 1 data TLB refill, read\000 */
{ 113093 }, /* l1d_tlb_refill_wr\000\000cache\000L1D tlb refill, write\000event=0x4d\000\00000\000Attributable Level 1 data TLB refill, write\000 */
{ 113993 }, /* l1d_tlb_wr\000\000cache\000L1D tlb access, write\000event=0x4f\000\00000\000Attributable Level 1 data or unified TLB access, write\000 */
{ 116626 }, /* l1i_cache\000\000cache\000Attributable Level 1 instruction cache access\000event=0x14\000\00000\000Attributable Level 1 instruction cache access\000 */
{ 117316 }, /* l1i_cache_lmiss\000\000cache\000Level 1 instruction cache long-latency read miss\000event=0x4006\000\00000\000Level 1 instruction cache long-latency read miss.  If the L1I_CACHE_RD event is implemented, the counter counts each access counted by L1I_CACHE_RD that incurs additional latency because it returns instructions from outside of the Level 1 instruction cache of this PE.  If the L1I_CACHE_RD event is not implemented, the counter counts each access counted by L1I_CACHE that incurs additional latency because it returns instructions from outside the Level 1 instruction cache of this PE.  The event indicates to software that the access missed in the Level 1 instruction cache and might have a significant performance impact due to the additional latency, compared to the latency of an access that hits in the Level 1 instruction cache\000 */
{ 118420 }, /* l1i_cache_refill\000\000cache\000Level 1 instruction cache refill\000event=0x1\000\00000\000Level 1 instruction cache refill\000 */
{ 119419 }, /* l1i_tlb\000\000cache\000Attributable Level 1 instruction TLB access\000event=0x26\000\00000\000Attributable Level 1 instruction TLB access\000 */
{ 120198 }, /* l1i_tlb_refill\000\000cache\000Attributable Level 1 instruction TLB refill\000event=0x2\000\00000\000Attributable Level 1 instruction TLB refill\000 */
{ 126244 }, /* l2d_cache\000\000cache\000Level 2 data cache access\000event=0x16\000\00000\000Level 2 data cache access\000 */
{ 126922 }, /* l2d_cache_allocate\000\000cache\000Attributable Level 2 data cache allocation without refill\000event=0x20\000\00000\000Attributable Level 2 data cache allocation without refill\000 */
{ 127466 }, /* l2d_cache_inval\000\000cache\000L2D cache invalidate\000event=0x58\000\00000\000Attributable Level 2 data cache invalidate\000 */
{ 128191 }, /* l2d_cache_lmiss_rd\000\000cache\000Level 2 data cache long-latency read miss\000event=0x4009\000\00000\000Level 2 data cache long-latency read miss.  The counter counts each memory read access counted by L2D_CACHE that incurs additional latency because it returns data from outside the Level 2 data or unified cache of this processing element.  The event indicates to software that the access missed in the Level 2 data or unified cache and might have a significant performance impact compared to the latency of an access that hits in the Level 2 data or unified cache\000 */
{ 129126 }, /* l2d_cache_rd\000\000cache\000L2D cache access, read\000event=0x50\000\00000\000Attributable Level 2 data cache access, read\000 */
{ 129862 }, /* l2d_cache_refill\000\000cache\000Level 2 data refill\000event=0x17\000\00000\000Level 2 data refill\000 */
{ 132288 }, /* l2d_cache_refill_rd\000\000cache\000L2D cache refill, read\000event=0x52\000\00000\000Attributable Level 2 data cache refill, read\000 */
{ 132829 }, /* l2d_cache_refill_wr\000\000cache\000L2D cache refill, write\000event=0x53\000\00000\000Attributable Level 2 data cache refill, write\000 */
{ 136414 }, /* l2d_cache_wb\000\000cache\000Attributable Level 2 data cache write-back\000event=0x18\000\00000\000Attributable Level 2 data cache write-back\000 */
{ 137387 }, /* l2d_cache_wb_clean\000\000cache\000L2D cache Write-Back, cleaning and coherency\000event=0x57\000\00000\000Level 2 data cache Write-Back, cleaning and coherency\000 */
{ 137968 }, /* l2d_cache_wb_victim\000\000cache\000L2D cache Write-Back, victim\000event=0x56\000\00000\000Attributable Level 2 data cache Write-Back, victim\000 */
{ 138385 }, /* l2d_cache_wr\000\000cache\000L2D cache access, write\000event=0x51\000\00000\000Attributable Level 2 data cache access, write\000 */
{ 143389 }, /* l2d_tlb\000\000cache\000Attributable Level 2 data or unified TLB access\000event=0x2f\000\00000\000Attributable Level 2 data or unified TLB access\000 */
{ 144070 }, /* l2d_tlb_rd\000\000cache\000L2D cache access, read\000event=0x5e\000\00000\000Attributable Level 2 data or unified TLB access, read\000 */
{ 144725 }, /* l2d_tlb_refill\000\000cache\000Attributable Level 2 data TLB refill\000event=0x2d\000\00000\000Attributable Level 2 data TLB refill\000 */
{ 145079 }, /* l2d_tlb_refill_rd\000\000cache\000L2D cache refill, read\000event=0x5c\000\00000\000Attributable Level 2 data or unified TLB refill, read\000 */
{ 145555 }, /* l2d_tlb_refill_wr\000\000cache\000L2D cache refill, write\000event=0x5d\000\00000\000Attributable Level 2 data or unified TLB refill, write\000 */
{ 146014 }, /* l2d_tlb_wr\000\000cache\000L2D cache access, write\000event=0x5f\000\00000\000Attributable Level 2 data or unified TLB access, write\000 */
{ 150641 }, /* l3d_cache\000\000cache\000Attributable Level 3 data cache access\000event=0x2b\000\00000\000Attributable Level 3 data cache access\000 */
{ 151459 }, /* l3d_cache_allocate\000\000cache\000Attributable Level 3 data cache allocation without refill\000event=0x29\000\00000\000Attributable Level 3 data cache allocation without refill\000 */
{ 151872 }, /* l3d_cache_lmiss_rd\000\000cache\000Level 3 data cache long-latency read miss\000event=0x400b\000\00000\000Level 3 data cache long-latency read miss.  The counter counts each memory read access counted by L3D_CACHE that incurs additional latency because it returns data from outside the Level 3 data or unified cache of this processing element.  The event indicates to software that the access missed in the Level 3 data or unified cache and might have a significant performance impact compared to the latency of an access that hits in the Level 3 data or unified cache\000 */
{ 152420 }, /* l3d_cache_rd\000\000cache\000Attributable Level 3 data or unified cache access, read\000event=0xa0\000\00000\000Attributable Level 3 data or unified cache access, read\000 */
{ 152665 }, /* l3d_cache_refill\000\000cache\000Attributable Level 3 data cache refill\000event=0x2a\000\00000\000Attributable Level 3 data cache refill\000 */
{ 160229 }, /* ll_cache_miss_rd\000\000cache\000Last level cache miss, read\000event=0x37\000\00000\000Last level cache miss, read\000 */
{ 160879 }, /* ll_cache_rd\000\000cache\000Attributable Last level cache memory read\000event=0x36\000\00000\000Attributable Last level cache memory read\000 */
{ 56959 }, /* exc_dabort\000\000exception\000Exception taken, Data Abort and SError\000event=0x84\000\00000\000Exception taken, Data Abort and SError\000 */
{ 57280 }, /* exc_fiq\000\000exception\000Exception taken, FIQ\000event=0x87\000\00000\000Exception taken, FIQ\000 */
{ 57633 }, /* exc_hvc\000\000exception\000Exception taken, Hypervisor Call\000event=0x8a\000\00000\000Exception taken, Hypervisor Call\000 */
{ 57939 }, /* exc_irq\000\000exception\000Exception taken, IRQ\000event=0x86\000\00000\000Exception taken, IRQ\000 */
{ 58355 }, /* exc_pabort\000\000exception\000Exception taken, Instruction Abort\000event=0x83\000\00000\000Exception taken, Instruction Abort\000 */
{ 59258 }, /* exc_smc\000\000exception\000Exception taken, Secure Monitor Call\000event=0x88\000\00000\000Exception taken, Secure Monitor Call\000 */
{ 59572 }, /* exc_svc\000\000exception\000Exception taken, Supervisor Call\000event=0x82\000\00000\000Exception taken, Supervisor Call\000 */
{ 59896 }, /* exc_taken\000\000exception\000Exception taken\000event=0x9\000\00000\000Exception taken\000 */
{ 60575 }, /* exc_trap_dabort\000\000exception\000Exception taken, Data Abort or SError not taken locally\000event=0x8c\000\00000\000Exception taken, Data Abort or SError not taken locally\000 */
{ 61080 }, /* exc_trap_fiq\000\000exception\000Exception taken, FIQ not taken locally\000event=0x8f\000\00000\000Exception taken, FIQ not taken locally\000 */
{ 62347 }, /* exc_trap_irq\000\000exception\000Exception taken, IRQ not taken locally\000event=0x8e\000\00000\000Exception taken, IRQ not taken locally\000 */
{ 62841 }, /* exc_trap_other\000\000exception\000Exception taken, Other traps not taken locally\000event=0x8d\000\00000\000Exception taken, Other traps not taken locally\000 */
{ 63383 }, /* exc_trap_pabort\000\000exception\000Exception taken, Instruction Abort not taken locally\000event=0x8b\000\00000\000Exception taken, Instruction Abort not taken locally\000 */
{ 64129 }, /* exc_undef\000\000exception\000Exception taken, Other synchronous\000event=0x81\000\00000\000Exception taken, Other synchronous\000 */
{ 165870 }, /* memory_error\000\000exception\000Local memory error\000event=0x1a\000\00000\000Local memory error\000 */
{ 20878 }, /* ase_spec\000\000instruction\000Operation speculatively executed, Advanced SIMD instruction\000event=0x74\000\00000\000Operation speculatively executed, Advanced SIMD instruction\000 */
{ 36351 }, /* br_mis_pred_retired\000\000instruction\000Instruction architecturally executed, mispredicted branch\000event=0x22\000\00000\000Instruction architecturally executed, mispredicted branch\000 */
{ 37393 }, /* br_retired\000\000instruction\000Instruction architecturally executed, branch\000event=0x21\000\00000\000Instruction architecturally executed, branch\000 */
{ 45546 }, /* cid_write_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000event=0xb\000\00000\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000 */
{ 48419 }, /* crypto_spec\000\000instruction\000Operation speculatively executed, Cryptographic instruction\000event=0x77\000\00000\000Operation speculatively executed, Cryptographic instruction\000 */
{ 51117 }, /* dmb_spec\000\000instruction\000Barrier speculatively executed, DMB\000event=0x7e\000\00000\000Barrier speculatively executed, DMB\000 */
{ 52153 }, /* dp_spec\000\000instruction\000Operation speculatively executed, integer data processing\000event=0x73\000\00000\000Operation speculatively executed, integer data processing\000 */
{ 53209 }, /* dsb_spec\000\000instruction\000Barrier speculatively executed, DSB\000event=0x7d\000\00000\000Barrier speculatively executed, DSB\000 */
{ 58850 }, /* exc_return\000\000instruction\000Instruction architecturally executed, condition check pass, exception return\000event=0xa\000\00000\000Instruction architecturally executed, condition check pass, exception return\000 */
{ 90819 }, /* inst_retired\000\000instruction\000Instruction architecturally executed\000event=0x8\000\00000\000Instruction architecturally executed\000 */
{ 91235 }, /* inst_spec\000\000instruction\000Operation speculatively executed\000event=0x1b\000\00000\000Operation speculatively executed\000 */
{ 91769 }, /* isb_spec\000\000instruction\000Barrier speculatively executed, ISB\000event=0x7c\000\00000\000Barrier speculatively executed, ISB\000 */
{ 159047 }, /* ld_spec\000\000instruction\000Operation speculatively executed, load\000event=0x70\000\00000\000Operation speculatively executed, load\000 */
{ 159351 }, /* ldrex_spec\000\000instruction\000Exclusive operation speculatively executed, LDREX or LDX\000event=0x6c\000\00000\000Exclusive operation speculatively executed, LDREX or LDX\000 */
{ 173450 }, /* op_retired\000\000instruction\000Micro-operation architecturally executed\000event=0x3a\000\00000\000Micro-operation architecturally executed.  The counter counts each operation counted by OP_SPEC that would be executed in a simple sequential execution of the program\000 */
{ 173697 }, /* op_spec\000\000instruction\000Micro-operation speculatively executed\000event=0x3b\000\00000\000Micro-operation speculatively executed.  The counter counts the number of operations executed by the processing element, including those that are executed speculatively and would not be executed in a simple sequential execution of the program\000 */
{ 175115 }, /* pc_write_spec\000\000instruction\000Operation speculatively executed, software change of the PC\000event=0x76\000\00000\000Operation speculatively executed, software change of the PC\000 */
{ 177878 }, /* rc_ld_spec\000\000instruction\000Release consistency operation speculatively executed, Load-Acquire\000event=0x90\000\00000\000Release consistency operation speculatively executed, Load-Acquire\000 */
{ 178251 }, /* rc_st_spec\000\000instruction\000Release consistency operation speculatively executed, Store-Release\000event=0x91\000\00000\000Release consistency operation speculatively executed, Store-Release\000 */
{ 185522 }, /* st_spec\000\000instruction\000Operation speculatively executed, store\000event=0x71\000\00000\000Operation speculatively executed, store\000 */
{ 200454 }, /* strex_fail_spec\000\000instruction\000Exclusive operation speculatively executed, STREX or STX fail\000event=0x6e\000\00000\000Exclusive operation speculatively executed, STREX or STX fail\000 */
{ 201025 }, /* strex_pass_spec\000\000instruction\000Exclusive operation speculatively executed, STREX or STX pass\000event=0x6d\000\00000\000Exclusive operation speculatively executed, STREX or STX pass\000 */
{ 201592 }, /* strex_spec\000\000instruction\000Exclusive operation speculatively executed, STREX or STX\000event=0x6f\000\00000\000Exclusive operation speculatively executed, STREX or STX\000 */
{ 205736 }, /* sw_incr\000\000instruction\000Instruction architecturally executed, Condition code check pass, software increment\000event=0\000\00000\000Instruction architecturally executed, Condition code check pass, software increment\000 */
{ 210294 }, /* ttbr_write_retired\000\000instruction\000Instruction architecturally executed, Condition code check pass, write to TTBR\000event=0x1c\000\00000\000Instruction architecturally executed, Condition code check pass, write to TTBR\000 */
{ 214354 }, /* vfp_spec\000\000instruction\000Operation speculatively executed, floating-point instruction\000event=0x75\000\00000\000Operation speculatively executed, floating-point instruction\000 */
{ 163148 }, /* mem_access\000\000memory\000Data memory access\000event=0x13\000\00000\000Data memory access\000 */
{ 164298 }, /* mem_access_rd\000\000memory\000Data memory access, read\000event=0x66\000\00000\000Data memory access, read\000 */
{ 164956 }, /* mem_access_wr\000\000memory\000Data memory access, write\000event=0x67\000\00000\000Data memory access, write\000 */
{ 179852 }, /* remote_access\000\000memory\000Access to another socket in a multi-socket system\000event=0x31\000\00000\000Access to another socket in a multi-socket system\000 */
{ 211505 }, /* unaligned_ld_spec\000\000memory\000Unaligned access, read\000event=0x68\000\00000\000Unaligned access, read\000 */
{ 212200 }, /* unaligned_ldst_spec\000\000memory\000Unaligned access\000event=0x6a\000\00000\000Unaligned access\000 */
{ 212637 }, /* unaligned_st_spec\000\000memory\000Unaligned access, write\000event=0x69\000\00000\000Unaligned access, write\000 */
{ 185831 }, /* stall\000\000pipeline\000No operation sent for execution\000event=0x3c\000\00000\000No operation sent for execution.  The counter counts every attributable cycle on which no attributable instruction or operation was sent for execution on this processing element\000 */
{ 186072 }, /* stall_backend\000\000pipeline\000No operation issued due to the backend\000event=0x24\000\00000\000No operation issued due to the backend\000 */
{ 192744 }, /* stall_backend_mem\000\000pipeline\000Memory stall cycles\000event=0x4005\000\00000\000Memory stall cycles.  The counter counts each cycle counted by STALL_BACKEND where there is a cache miss in the last level of cache within the processing element clock domain\000 */
{ 195875 }, /* stall_frontend\000\000pipeline\000No operation issued because of the frontend\000event=0x23\000\00000\000No operation issued because of the frontend\000 */
{ 199011 }, /* stall_slot\000\000pipeline\000No operation sent for execution on a slot\000event=0x3f\000\00000\000No operation sent for execution on a slot.  The counter counts on each attributable cycle the number of instruction or operation slots that were not occupied by an instruction or operation attributable to the processing element\000 */
{ 199317 }, /* stall_slot_backend\000\000pipeline\000No operation sent for execution on a slot due to the backend\000event=0x3d\000\00000\000No operation sent for execution on a slot due to the backend.  Counts each slot counted by STALL_SLOT where no attributable instruction or operation was sent for execution because the backend is unable to accept it\000 */
{ 199953 }, /* stall_slot_frontend\000\000pipeline\000No operation sent for execution on a slot due to the frontend\000event=0x3e\000\00000\000No operation sent for execution on a slot due to the frontend.  Counts each slot counted by STALL_SLOT where no attributable instruction or operation was sent for execution because there was no attributable instruction or operation available to issue from the processing element from the frontend for the slot\000 */
};

static const struct compact_pmu_event pmu_events__arm_cortex_x1[] = {
{ 30753 }, /* br_immed_spec\000\000branch\000Branch speculatively executed, immediate branch\000event=0x78\000\00000\000Branch speculatively executed, immediate branch\000 */
{ 35038 }, /* br_indirect_spec\000\000branch\000Branch speculatively executed, indirect branch\000event=0x7a\000\00000\000Branch speculatively executed, indirect branch\000 */
{ 35743 }, /* br_mis_pred\000\000branch\000Mispredicted or not predicted branch speculatively executed\000event=0x10\000\00000\000Mispredicted or not predicted branch speculatively executed\000 */
{ 36925 }, /* br_pred\000\000branch\000Predictable branch speculatively executed\000event=0x12\000\00000\000Predictable branch speculatively executed\000 */
{ 40692 }, /* br_return_spec\000\000branch\000Branch speculatively executed, procedure return\000event=0x79\000\00000\000Branch speculatively executed, procedure return\000 */
{ 41358 }, /* bus_access\000\000bus\000Attributable Bus access\000event=0x19\000\00000\000Attributable Bus access\000 */
{ 42271 }, /* bus_access_rd\000\000bus\000Bus access read\000event=0x60\000\00000\000Bus access read\000 */
{ 42682 }, /* bus_access_wr\000\000bus\000Bus access write\000event=0x61\000\00000\000Bus access write\000 */
{ 42979 }, /* bus_cycles\000\000bus\000Bus cycle\000event=0x1d\000\00000\000Bus cycle\000 */
{ 46424 }, /* cnt_cycles\000\000bus\000Constant frequency cycles\000event=0x4004\000\00000\000Constant frequency cycles.  The counter increments at a constant frequency equal to the rate of increment of the system counter, CNTPCT_EL0\000 */
{ 47992 }, /* cpu_cycles\000\000bus\000Cycle\000event=0x11\000\00000\000Cycle\000 */
{ 53526 }, /* dtlb_walk\000\000cache\000Access to data TLB causes a translation table walk\000event=0x34\000\00000\000Access to data TLB causes a translation table walk\000 */
{ 92044 }, /* itlb_walk\000\000cache\000Access to instruction TLB that causes a translation table walk\000event=0x35\000\00000\000Access to instruction TLB that causes a translation table walk\000 */
{ 100289 }, /* l1d_cache\000\000cache\000Level 1 data cache access\000event=0x4\000\00000\000Level 1 data cache access\000 */
{ 101403 }, /* l1d_cache_inval\000\000cache\000L1D cache invalidate\000event=0x48\000\00000\000Attributable Level 1 data cache invalidate\000 */
{ 102276 }, /* l1d_cache_lmiss_rd\000\000cache\000Level 1 data cache long-latency read miss\000event=0x39\000\00000\000Level 1 data cache long-latency read miss.  The counter counts each memory read access counted by L1D_CACHE that incurs additional latency because it returns data from outside the Level 1 data or unified cache of this processing element\000 */
{ 102693 }, /* l1d_cache_rd\000\000cache\000L1D cache access, read\000event=0x40\000\00000\000Attributable Level 1 data cache access, read\000 */
{ 103129 }, /* l1d_cache_refill\000\000cache\000Level 1 data cache refill\000event=0x3\000\00000\000Level 1 data cache refill\000 */
{ 104145 }, /* l1d_cache_refill_inner\000\000cache\000L1D cache refill, inner\000event=0x44\000\00000\000Attributable Level 1 data cache refill, inner\000 */
{ 104573 }, /* l1d_cache_refill_outer\000\000cache\000L1D cache refill, outer\000event=0x45\000\00000\000Attributable Level 1 data cache refill, outer\000 */
{ 105894 }, /* l1d_cache_refill_rd\000\000cache\000L1D cache refill, read\000event=0x42\000\00000\000Attributable Level 1 data cache refill, read\000 */
{ 106389 }, /* l1d_cache_refill_wr\000\000cache\000L1D cache refill, write\000event=0x43\000\00000\000Attributable Level 1 data cache refill, write\000 */
{ 106891 }, /* l1d_cache_wb\000\000cache\000Attributable Level 1 data cache write-back\000event=0x15\000\00000\000Attributable Level 1 data cache write-back\000 */
{ 107957 }, /* l1d_cache_wb_clean\000\000cache\000L1D cache Write-Back, cleaning and coherency\000event=0x47\000\00000\000Level 1 data cache Write-Back, cleaning and coherency\000 */
{ 108495 }, /* l1d_cache_wb_victim\000\000cache\000L1D cache Write-Back, victim\000event=0x46\000\00000\000Attributable Level 1 data cache Write-Back, victim\000 */
{ 108997 }, /* l1d_cache_wr\000\000cache\000L1D cache access, write\000event=0x41\000\00000\000Attributable Level 1 data cache access, write\000 */
{ 109867 }, /* l1d_tlb\000\000cache\000Attributable Level 1 data or unified TLB access\000event=0x25\000\00000\000Attributable Level 1 data or unified TLB access\000 */
{ 110748 }, /* l1d_tlb_rd\000\000cache\000L1D tlb access, read\000event=0x4e\000\00000\000Attributable Level 1 data or unified TLB access, read\000 */
{ 111207 }, /* l1d_tlb_refill\000\000cache\000Attributable Level 1 data TLB refill\000event=0x5\000\00000\000Attributable Level 1 data TLB refill\000 */
{ 112199 }, /* l1d_tlb_refill_rd\000\000cache\000L1D tlb refill, read\000event=0x4c\000\00000\000Attributable Level 1 data TLB refill, read\000 */
{ 113093 }, /* l1d_tlb_refill_wr\000\000cache\000L1D tlb refill, write\000event=0x4d\000\00000\000Attributable Level 1 data TLB refill, write\000 */
{ 113993 }, /* l1d_tlb_wr\000\000cache\000L1D tlb access, write\000event=0x4f\000\00000\000Attributable Level 1 data or unified TLB access, write\000 */
{ 116626 }, /* l1i_cache\000\000cache\000Attributable Level 1 instruction cache access\000event=0x14\000\00000\000Attributable Level 1 instruction cache access\000 */
{ 117316 }, /* l1i_cache_lmiss\000\000cache\000Level 1 instruction cache long-latency read miss\000event=0x4006\000\00000\000Level 1 instruction cache long-latency read miss.  If the L1I_CACHE_RD event is implemented, the counter counts each access counted by L1I_CACHE_RD that incurs additional latency because it returns instructions from outside of the Level 1 instruction cache of this PE.  If the L1I_CACHE_RD event is not implemented, the counter counts each access counted by L1I_CACHE that incurs additional latency because it returns instructions from outside the Level 1 instruction cache of this PE.  The event indicates to software that the access missed in the Level 1 instruction cache and might have a significant performance impact due to the additional latency, compared to the latency of an access that hits in the Level 1 instruction cache\000 */
{ 118420 }, /* l1i_cache_refill\000\000cache\000Level 1 instruction cache refill\000event=0x1\000\00000\000Level 1 instruction cache refill\000 */
{ 119419 }, /* l1i_tlb\000\000cache\000Attributable Level 1 instruction TLB access\000event=0x26\000\00000\000Attributable Level 1 instruction TLB access\000 */
{ 120198 }, /* l1i_tlb_refill\000\000cache\000Attributable Level 1 instruction TLB refill\000event=0x2\000\00000\000Attributable Level 1 instruction TLB refill\000 */
{ 126244 }, /* l2d_cache\000\000cache\000Level 2 data cache access\000event=0x16\000\00000\000Level 2 data cache access\000 */
{ 126922 }, /* l2d_cache_allocate\000\000cache\000Attributable Level 2 data cache allocation without refill\000event=0x20\000\00000\000Attributable Level 2 data cache allocation without refill\000 */
{ 127466 }, /* l2d_cache_inval\000\000cache\000L2D cache invalidate\000event=0x58\000\00000\000Attributable Level 2 data cache invalidate\000 */
{ 128191 }, /* l2d_cache_lmiss_rd\000\000cache\000Level 2 data cache long-latency read miss\000event=0x4009\000\00000\000Level 2 data cache long-latency read miss.  The counter counts each memory read access counted by L2D_CACHE that incurs additional latency because it returns data from outside the Level 2 data or unified cache of this processing element.  The event indicates to software that the access missed in the Level 2 data or unified cache and might have a significant performance impact compared to the latency of an access that hits in the Level 2 data or unified cache\000 */
{ 129126 }, /* l2d_cache_rd\000\000cache\000L2D cache access, read\000event=0x50\000\00000\000Attributable Level 2 data cache access, read\000 */
{ 129862 }, /* l2d_cache_refill\000\000cache\000Level 2 data refill\000event=0x17\000\00000\000Level 2 data refill\000 */
{ 132288 }, /* l2d_cache_refill_rd\000\000cache\000L2D cache refill, read\000event=0x52\000\00000\000Attributable Level 2 data cache refill, read\000 */
{ 132829 }, /* l2d_cache_refill_wr\000\000cache\000L2D cache refill, write\000event=0x53\000\00000\000Attributable Level 2 data cache refill, write\000 */
{ 136414 }, /* l2d_cache_wb\000\000cache\000Attributable Level 2 data cache write-back\000event=0x18\000\00000\000Attributable Level 2 data cache write-back\000 */
{ 137387 }, /* l2d_cache_wb_clean\000\000cache\000L2D cache Write-Back, cleaning and coherency\000event=0x57\000\00000\000Level 2 data cache Write-Back, cleaning and coherency\000 */
{ 137968 }, /* l2d_cache_wb_victim\000\000cache\000L2D cache Write-Back, victim\000event=0x56\000\00000\000Attributable Level 2 data cache Write-Back, victim\000 */
{ 138385 }, /* l2d_cache_wr\000\000cache\000L2D cache access, write\000event=0x51\000\00000\000Attributable Level 2 data cache access, write\000 */
{ 143389 }, /* l2d_tlb\000\000cache\000Attributable Level 2 data or unified TLB access\000event=0x2f\000\00000\000Attributable Level 2 data or unified TLB access\000 */
{ 144070 }, /* l2d_tlb_rd\000\000cache\000L2D cache access, read\000event=0x5e\000\00000\000Attributable Level 2 data or unified TLB access, read\000 */
{ 144725 }, /* l2d_tlb_refill\000\000cache\000Attributable Level 2 data TLB refill\000event=0x2d\000\00000\000Attributable Level 2 data TLB refill\000 */
{ 145079 }, /* l2d_tlb_refill_rd\000\000cache\000L2D cache refill, read\000event=0x5c\000\00000\000Attributable Level 2 data or unified TLB refill, read\000 */
{ 145555 }, /* l2d_tlb_refill_wr\000\000cache\000L2D cache refill, write\000event=0x5d\000\00000\000Attributable Level 2 data or unified TLB refill, write\000 */
{ 146014 }, /* l2d_tlb_wr\000\000cache\000L2D cache access, write\000event=0x5f\000\00000\000Attributable Level 2 data or unified TLB access, write\000 */
{ 150641 }, /* l3d_cache\000\000cache\000Attributable Level 3 data cache access\000event=0x2b\000\00000\000Attributable Level 3 data cache access\000 */
{ 151459 }, /* l3d_cache_allocate\000\000cache\000Attributable Level 3 data cache allocation without refill\000event=0x29\000\00000\000Attributable Level 3 data cache allocation without refill\000 */
{ 151872 }, /* l3d_cache_lmiss_rd\000\000cache\000Level 3 data cache long-latency read miss\000event=0x400b\000\00000\000Level 3 data cache long-latency read miss.  The counter counts each memory read access counted by L3D_CACHE that incurs additional latency because it returns data from outside the Level 3 data or unified cache of this processing element.  The event indicates to software that the access missed in the Level 3 data or unified cache and might have a significant performance impact compared to the latency of an access that hits in the Level 3 data or unified cache\000 */
{ 152420 }, /* l3d_cache_rd\000\000cache\000Attributable Level 3 data or unified cache access, read\000event=0xa0\000\00000\000Attributable Level 3 data or unified cache access, read\000 */
{ 152665 }, /* l3d_cache_refill\000\000cache\000Attributable Level 3 data cache refill\000event=0x2a\000\00000\000Attributable Level 3 data cache refill\000 */
{ 160229 }, /* ll_cache_miss_rd\000\000cache\000Last level cache miss, read\000event=0x37\000\00000\000Last level cache miss, read\000 */
{ 160879 }, /* ll_cache_rd\000\000cache\000Attributable Last level cache memory read\000event=0x36\000\00000\000Attributable Last level cache memory read\000 */
{ 56959 }, /* exc_dabort\000\000exception\000Exception taken, Data Abort and SError\000event=0x84\000\00000\000Exception taken, Data Abort and SError\000 */
{ 57280 }, /* exc_fiq\000\000exception\000Exception taken, FIQ\000event=0x87\000\00000\000Exception taken, FIQ\000 */
{ 57633 }, /* exc_hvc\000\000exception\000Exception taken, Hypervisor Call\000event=0x8a\000\00000\000Exception taken, Hypervisor Call\000 */
{ 57939 }, /* exc_irq\000\000exception\000Exception taken, IRQ\000event=0x86\000\00000\000Exception taken, IRQ\000 */
{ 58355 }, /* exc_pabort\000\000exception\000Exception taken, Instruction Abort\000event=0x83\000\00000\000Exception taken, Instruction Abort\000 */
{ 59258 }, /* exc_smc\000\000exception\000Exception taken, Secure Monitor Call\000event=0x88\000\00000\000Exception taken, Secure Monitor Call\000 */
{ 59572 }, /* exc_svc\000\000exception\000Exception taken, Supervisor Call\000event=0x82\000\00000\000Exception taken, Supervisor Call\000 */
{ 59896 }, /* exc_taken\000\000exception\000Exception taken\000event=0x9\000\00000\000Exception taken\000 */
{ 60575 }, /* exc_trap_dabort\000\000exception\000Exception taken, Data Abort or SError not taken locally\000event=0x8c\000\00000\000Exception taken, Data Abort or SError not taken locally\000 */
{ 61080 }, /* exc_trap_fiq\000\000exception\000Exception taken, FIQ not taken locally\000event=0x8f\000\00000\000Exception taken, FIQ not taken locally\000 */
{ 62347 }, /* exc_trap_irq\000\000exception\000Exception taken, IRQ not taken locally\000event=0x8e\000\00000\000Exception taken, IRQ not taken locally\000 */
{ 62841 }, /* exc_trap_other\000\000exception\000Exception taken, Other traps not taken locally\000event=0x8d\000\00000\000Exception taken, Other traps not taken locally\000 */
{ 63383 }, /* exc_trap_pabort\000\000exception\000Exception taken, Instruction Abort not taken locally\000event=0x8b\000\00000\000Exception taken, Instruction Abort not taken locally\000 */
{ 64129 }, /* exc_undef\000\000exception\000Exception taken, Other synchronous\000event=0x81\000\00000\000Exception taken, Other synchronous\000 */
{ 165870 }, /* memory_error\000\000exception\000Local memory error\000event=0x1a\000\00000\000Local memory error\000 */
{ 20878 }, /* ase_spec\000\000instruction\000Operation speculatively executed, Advanced SIMD instruction\000event=0x74\000\00000\000Operation speculatively executed, Advanced SIMD instruction\000 */
{ 36351 }, /* br_mis_pred_retired\000\000instruction\000Instruction architecturally executed, mispredicted branch\000event=0x22\000\00000\000Instruction architecturally executed, mispredicted branch\000 */
{ 37393 }, /* br_retired\000\000instruction\000Instruction architecturally executed, branch\000event=0x21\000\00000\000Instruction architecturally executed, branch\000 */
{ 45546 }, /* cid_write_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000event=0xb\000\00000\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000 */
{ 48419 }, /* crypto_spec\000\000instruction\000Operation speculatively executed, Cryptographic instruction\000event=0x77\000\00000\000Operation speculatively executed, Cryptographic instruction\000 */
{ 51117 }, /* dmb_spec\000\000instruction\000Barrier speculatively executed, DMB\000event=0x7e\000\00000\000Barrier speculatively executed, DMB\000 */
{ 52153 }, /* dp_spec\000\000instruction\000Operation speculatively executed, integer data processing\000event=0x73\000\00000\000Operation speculatively executed, integer data processing\000 */
{ 53209 }, /* dsb_spec\000\000instruction\000Barrier speculatively executed, DSB\000event=0x7d\000\00000\000Barrier speculatively executed, DSB\000 */
{ 58850 }, /* exc_return\000\000instruction\000Instruction architecturally executed, condition check pass, exception return\000event=0xa\000\00000\000Instruction architecturally executed, condition check pass, exception return\000 */
{ 90819 }, /* inst_retired\000\000instruction\000Instruction architecturally executed\000event=0x8\000\00000\000Instruction architecturally executed\000 */
{ 91235 }, /* inst_spec\000\000instruction\000Operation speculatively executed\000event=0x1b\000\00000\000Operation speculatively executed\000 */
{ 91769 }, /* isb_spec\000\000instruction\000Barrier speculatively executed, ISB\000event=0x7c\000\00000\000Barrier speculatively executed, ISB\000 */
{ 159047 }, /* ld_spec\000\000instruction\000Operation speculatively executed, load\000event=0x70\000\00000\000Operation speculatively executed, load\000 */
{ 159351 }, /* ldrex_spec\000\000instruction\000Exclusive operation speculatively executed, LDREX or LDX\000event=0x6c\000\00000\000Exclusive operation speculatively executed, LDREX or LDX\000 */
{ 173450 }, /* op_retired\000\000instruction\000Micro-operation architecturally executed\000event=0x3a\000\00000\000Micro-operation architecturally executed.  The counter counts each operation counted by OP_SPEC that would be executed in a simple sequential execution of the program\000 */
{ 173697 }, /* op_spec\000\000instruction\000Micro-operation speculatively executed\000event=0x3b\000\00000\000Micro-operation speculatively executed.  The counter counts the number of operations executed by the processing element, including those that are executed speculatively and would not be executed in a simple sequential execution of the program\000 */
{ 175115 }, /* pc_write_spec\000\000instruction\000Operation speculatively executed, software change of the PC\000event=0x76\000\00000\000Operation speculatively executed, software change of the PC\000 */
{ 177878 }, /* rc_ld_spec\000\000instruction\000Release consistency operation speculatively executed, Load-Acquire\000event=0x90\000\00000\000Release consistency operation speculatively executed, Load-Acquire\000 */
{ 178251 }, /* rc_st_spec\000\000instruction\000Release consistency operation speculatively executed, Store-Release\000event=0x91\000\00000\000Release consistency operation speculatively executed, Store-Release\000 */
{ 185522 }, /* st_spec\000\000instruction\000Operation speculatively executed, store\000event=0x71\000\00000\000Operation speculatively executed, store\000 */
{ 200454 }, /* strex_fail_spec\000\000instruction\000Exclusive operation speculatively executed, STREX or STX fail\000event=0x6e\000\00000\000Exclusive operation speculatively executed, STREX or STX fail\000 */
{ 201025 }, /* strex_pass_spec\000\000instruction\000Exclusive operation speculatively executed, STREX or STX pass\000event=0x6d\000\00000\000Exclusive operation speculatively executed, STREX or STX pass\000 */
{ 201592 }, /* strex_spec\000\000instruction\000Exclusive operation speculatively executed, STREX or STX\000event=0x6f\000\00000\000Exclusive operation speculatively executed, STREX or STX\000 */
{ 205736 }, /* sw_incr\000\000instruction\000Instruction architecturally executed, Condition code check pass, software increment\000event=0\000\00000\000Instruction architecturally executed, Condition code check pass, software increment\000 */
{ 210294 }, /* ttbr_write_retired\000\000instruction\000Instruction architecturally executed, Condition code check pass, write to TTBR\000event=0x1c\000\00000\000Instruction architecturally executed, Condition code check pass, write to TTBR\000 */
{ 214354 }, /* vfp_spec\000\000instruction\000Operation speculatively executed, floating-point instruction\000event=0x75\000\00000\000Operation speculatively executed, floating-point instruction\000 */
{ 163148 }, /* mem_access\000\000memory\000Data memory access\000event=0x13\000\00000\000Data memory access\000 */
{ 164298 }, /* mem_access_rd\000\000memory\000Data memory access, read\000event=0x66\000\00000\000Data memory access, read\000 */
{ 164956 }, /* mem_access_wr\000\000memory\000Data memory access, write\000event=0x67\000\00000\000Data memory access, write\000 */
{ 179852 }, /* remote_access\000\000memory\000Access to another socket in a multi-socket system\000event=0x31\000\00000\000Access to another socket in a multi-socket system\000 */
{ 211505 }, /* unaligned_ld_spec\000\000memory\000Unaligned access, read\000event=0x68\000\00000\000Unaligned access, read\000 */
{ 212200 }, /* unaligned_ldst_spec\000\000memory\000Unaligned access\000event=0x6a\000\00000\000Unaligned access\000 */
{ 212637 }, /* unaligned_st_spec\000\000memory\000Unaligned access, write\000event=0x69\000\00000\000Unaligned access, write\000 */
{ 185831 }, /* stall\000\000pipeline\000No operation sent for execution\000event=0x3c\000\00000\000No operation sent for execution.  The counter counts every attributable cycle on which no attributable instruction or operation was sent for execution on this processing element\000 */
{ 186072 }, /* stall_backend\000\000pipeline\000No operation issued due to the backend\000event=0x24\000\00000\000No operation issued due to the backend\000 */
{ 192744 }, /* stall_backend_mem\000\000pipeline\000Memory stall cycles\000event=0x4005\000\00000\000Memory stall cycles.  The counter counts each cycle counted by STALL_BACKEND where there is a cache miss in the last level of cache within the processing element clock domain\000 */
{ 195875 }, /* stall_frontend\000\000pipeline\000No operation issued because of the frontend\000event=0x23\000\00000\000No operation issued because of the frontend\000 */
{ 199011 }, /* stall_slot\000\000pipeline\000No operation sent for execution on a slot\000event=0x3f\000\00000\000No operation sent for execution on a slot.  The counter counts on each attributable cycle the number of instruction or operation slots that were not occupied by an instruction or operation attributable to the processing element\000 */
{ 199317 }, /* stall_slot_backend\000\000pipeline\000No operation sent for execution on a slot due to the backend\000event=0x3d\000\00000\000No operation sent for execution on a slot due to the backend.  Counts each slot counted by STALL_SLOT where no attributable instruction or operation was sent for execution because the backend is unable to accept it\000 */
{ 199953 }, /* stall_slot_frontend\000\000pipeline\000No operation sent for execution on a slot due to the frontend\000event=0x3e\000\00000\000No operation sent for execution on a slot due to the frontend.  Counts each slot counted by STALL_SLOT where no attributable instruction or operation was sent for execution because there was no attributable instruction or operation available to issue from the processing element from the frontend for the slot\000 */
};

static const struct compact_pmu_event pmu_events__arm_cortex_x2[] = {
{ 30753 }, /* br_immed_spec\000\000branch\000Branch speculatively executed, immediate branch\000event=0x78\000\00000\000Branch speculatively executed, immediate branch\000 */
{ 35038 }, /* br_indirect_spec\000\000branch\000Branch speculatively executed, indirect branch\000event=0x7a\000\00000\000Branch speculatively executed, indirect branch\000 */
{ 35743 }, /* br_mis_pred\000\000branch\000Mispredicted or not predicted branch speculatively executed\000event=0x10\000\00000\000Mispredicted or not predicted branch speculatively executed\000 */
{ 36925 }, /* br_pred\000\000branch\000Predictable branch speculatively executed\000event=0x12\000\00000\000Predictable branch speculatively executed\000 */
{ 40692 }, /* br_return_spec\000\000branch\000Branch speculatively executed, procedure return\000event=0x79\000\00000\000Branch speculatively executed, procedure return\000 */
{ 41358 }, /* bus_access\000\000bus\000Attributable Bus access\000event=0x19\000\00000\000Attributable Bus access\000 */
{ 42271 }, /* bus_access_rd\000\000bus\000Bus access read\000event=0x60\000\00000\000Bus access read\000 */
{ 42682 }, /* bus_access_wr\000\000bus\000Bus access write\000event=0x61\000\00000\000Bus access write\000 */
{ 42979 }, /* bus_cycles\000\000bus\000Bus cycle\000event=0x1d\000\00000\000Bus cycle\000 */
{ 46424 }, /* cnt_cycles\000\000bus\000Constant frequency cycles\000event=0x4004\000\00000\000Constant frequency cycles.  The counter increments at a constant frequency equal to the rate of increment of the system counter, CNTPCT_EL0\000 */
{ 47992 }, /* cpu_cycles\000\000bus\000Cycle\000event=0x11\000\00000\000Cycle\000 */
{ 53526 }, /* dtlb_walk\000\000cache\000Access to data TLB causes a translation table walk\000event=0x34\000\00000\000Access to data TLB causes a translation table walk\000 */
{ 92044 }, /* itlb_walk\000\000cache\000Access to instruction TLB that causes a translation table walk\000event=0x35\000\00000\000Access to instruction TLB that causes a translation table walk\000 */
{ 100289 }, /* l1d_cache\000\000cache\000Level 1 data cache access\000event=0x4\000\00000\000Level 1 data cache access\000 */
{ 101403 }, /* l1d_cache_inval\000\000cache\000L1D cache invalidate\000event=0x48\000\00000\000Attributable Level 1 data cache invalidate\000 */
{ 102276 }, /* l1d_cache_lmiss_rd\000\000cache\000Level 1 data cache long-latency read miss\000event=0x39\000\00000\000Level 1 data cache long-latency read miss.  The counter counts each memory read access counted by L1D_CACHE that incurs additional latency because it returns data from outside the Level 1 data or unified cache of this processing element\000 */
{ 102693 }, /* l1d_cache_rd\000\000cache\000L1D cache access, read\000event=0x40\000\00000\000Attributable Level 1 data cache access, read\000 */
{ 103129 }, /* l1d_cache_refill\000\000cache\000Level 1 data cache refill\000event=0x3\000\00000\000Level 1 data cache refill\000 */
{ 104145 }, /* l1d_cache_refill_inner\000\000cache\000L1D cache refill, inner\000event=0x44\000\00000\000Attributable Level 1 data cache refill, inner\000 */
{ 104573 }, /* l1d_cache_refill_outer\000\000cache\000L1D cache refill, outer\000event=0x45\000\00000\000Attributable Level 1 data cache refill, outer\000 */
{ 105894 }, /* l1d_cache_refill_rd\000\000cache\000L1D cache refill, read\000event=0x42\000\00000\000Attributable Level 1 data cache refill, read\000 */
{ 106389 }, /* l1d_cache_refill_wr\000\000cache\000L1D cache refill, write\000event=0x43\000\00000\000Attributable Level 1 data cache refill, write\000 */
{ 106891 }, /* l1d_cache_wb\000\000cache\000Attributable Level 1 data cache write-back\000event=0x15\000\00000\000Attributable Level 1 data cache write-back\000 */
{ 107957 }, /* l1d_cache_wb_clean\000\000cache\000L1D cache Write-Back, cleaning and coherency\000event=0x47\000\00000\000Level 1 data cache Write-Back, cleaning and coherency\000 */
{ 108495 }, /* l1d_cache_wb_victim\000\000cache\000L1D cache Write-Back, victim\000event=0x46\000\00000\000Attributable Level 1 data cache Write-Back, victim\000 */
{ 108997 }, /* l1d_cache_wr\000\000cache\000L1D cache access, write\000event=0x41\000\00000\000Attributable Level 1 data cache access, write\000 */
{ 109867 }, /* l1d_tlb\000\000cache\000Attributable Level 1 data or unified TLB access\000event=0x25\000\00000\000Attributable Level 1 data or unified TLB access\000 */
{ 110748 }, /* l1d_tlb_rd\000\000cache\000L1D tlb access, read\000event=0x4e\000\00000\000Attributable Level 1 data or unified TLB access, read\000 */
{ 111207 }, /* l1d_tlb_refill\000\000cache\000Attributable Level 1 data TLB refill\000event=0x5\000\00000\000Attributable Level 1 data TLB refill\000 */
{ 112199 }, /* l1d_tlb_refill_rd\000\000cache\000L1D tlb refill, read\000event=0x4c\000\00000\000Attributable Level 1 data TLB refill, read\000 */
{ 113093 }, /* l1d_tlb_refill_wr\000\000cache\000L1D tlb refill, write\000event=0x4d\000\00000\000Attributable Level 1 data TLB refill, write\000 */
{ 113993 }, /* l1d_tlb_wr\000\000cache\000L1D tlb access, write\000event=0x4f\000\00000\000Attributable Level 1 data or unified TLB access, write\000 */
{ 116626 }, /* l1i_cache\000\000cache\000Attributable Level 1 instruction cache access\000event=0x14\000\00000\000Attributable Level 1 instruction cache access\000 */
{ 117316 }, /* l1i_cache_lmiss\000\000cache\000Level 1 instruction cache long-latency read miss\000event=0x4006\000\00000\000Level 1 instruction cache long-latency read miss.  If the L1I_CACHE_RD event is implemented, the counter counts each access counted by L1I_CACHE_RD that incurs additional latency because it returns instructions from outside of the Level 1 instruction cache of this PE.  If the L1I_CACHE_RD event is not implemented, the counter counts each access counted by L1I_CACHE that incurs additional latency because it returns instructions from outside the Level 1 instruction cache of this PE.  The event indicates to software that the access missed in the Level 1 instruction cache and might have a significant performance impact due to the additional latency, compared to the latency of an access that hits in the Level 1 instruction cache\000 */
{ 118420 }, /* l1i_cache_refill\000\000cache\000Level 1 instruction cache refill\000event=0x1\000\00000\000Level 1 instruction cache refill\000 */
{ 119419 }, /* l1i_tlb\000\000cache\000Attributable Level 1 instruction TLB access\000event=0x26\000\00000\000Attributable Level 1 instruction TLB access\000 */
{ 120198 }, /* l1i_tlb_refill\000\000cache\000Attributable Level 1 instruction TLB refill\000event=0x2\000\00000\000Attributable Level 1 instruction TLB refill\000 */
{ 126244 }, /* l2d_cache\000\000cache\000Level 2 data cache access\000event=0x16\000\00000\000Level 2 data cache access\000 */
{ 126922 }, /* l2d_cache_allocate\000\000cache\000Attributable Level 2 data cache allocation without refill\000event=0x20\000\00000\000Attributable Level 2 data cache allocation without refill\000 */
{ 127466 }, /* l2d_cache_inval\000\000cache\000L2D cache invalidate\000event=0x58\000\00000\000Attributable Level 2 data cache invalidate\000 */
{ 128191 }, /* l2d_cache_lmiss_rd\000\000cache\000Level 2 data cache long-latency read miss\000event=0x4009\000\00000\000Level 2 data cache long-latency read miss.  The counter counts each memory read access counted by L2D_CACHE that incurs additional latency because it returns data from outside the Level 2 data or unified cache of this processing element.  The event indicates to software that the access missed in the Level 2 data or unified cache and might have a significant performance impact compared to the latency of an access that hits in the Level 2 data or unified cache\000 */
{ 129126 }, /* l2d_cache_rd\000\000cache\000L2D cache access, read\000event=0x50\000\00000\000Attributable Level 2 data cache access, read\000 */
{ 129862 }, /* l2d_cache_refill\000\000cache\000Level 2 data refill\000event=0x17\000\00000\000Level 2 data refill\000 */
{ 132288 }, /* l2d_cache_refill_rd\000\000cache\000L2D cache refill, read\000event=0x52\000\00000\000Attributable Level 2 data cache refill, read\000 */
{ 132829 }, /* l2d_cache_refill_wr\000\000cache\000L2D cache refill, write\000event=0x53\000\00000\000Attributable Level 2 data cache refill, write\000 */
{ 136414 }, /* l2d_cache_wb\000\000cache\000Attributable Level 2 data cache write-back\000event=0x18\000\00000\000Attributable Level 2 data cache write-back\000 */
{ 137387 }, /* l2d_cache_wb_clean\000\000cache\000L2D cache Write-Back, cleaning and coherency\000event=0x57\000\00000\000Level 2 data cache Write-Back, cleaning and coherency\000 */
{ 137968 }, /* l2d_cache_wb_victim\000\000cache\000L2D cache Write-Back, victim\000event=0x56\000\00000\000Attributable Level 2 data cache Write-Back, victim\000 */
{ 138385 }, /* l2d_cache_wr\000\000cache\000L2D cache access, write\000event=0x51\000\00000\000Attributable Level 2 data cache access, write\000 */
{ 143389 }, /* l2d_tlb\000\000cache\000Attributable Level 2 data or unified TLB access\000event=0x2f\000\00000\000Attributable Level 2 data or unified TLB access\000 */
{ 144070 }, /* l2d_tlb_rd\000\000cache\000L2D cache access, read\000event=0x5e\000\00000\000Attributable Level 2 data or unified TLB access, read\000 */
{ 144725 }, /* l2d_tlb_refill\000\000cache\000Attributable Level 2 data TLB refill\000event=0x2d\000\00000\000Attributable Level 2 data TLB refill\000 */
{ 145079 }, /* l2d_tlb_refill_rd\000\000cache\000L2D cache refill, read\000event=0x5c\000\00000\000Attributable Level 2 data or unified TLB refill, read\000 */
{ 145555 }, /* l2d_tlb_refill_wr\000\000cache\000L2D cache refill, write\000event=0x5d\000\00000\000Attributable Level 2 data or unified TLB refill, write\000 */
{ 146014 }, /* l2d_tlb_wr\000\000cache\000L2D cache access, write\000event=0x5f\000\00000\000Attributable Level 2 data or unified TLB access, write\000 */
{ 150641 }, /* l3d_cache\000\000cache\000Attributable Level 3 data cache access\000event=0x2b\000\00000\000Attributable Level 3 data cache access\000 */
{ 151459 }, /* l3d_cache_allocate\000\000cache\000Attributable Level 3 data cache allocation without refill\000event=0x29\000\00000\000Attributable Level 3 data cache allocation without refill\000 */
{ 151872 }, /* l3d_cache_lmiss_rd\000\000cache\000Level 3 data cache long-latency read miss\000event=0x400b\000\00000\000Level 3 data cache long-latency read miss.  The counter counts each memory read access counted by L3D_CACHE that incurs additional latency because it returns data from outside the Level 3 data or unified cache of this processing element.  The event indicates to software that the access missed in the Level 3 data or unified cache and might have a significant performance impact compared to the latency of an access that hits in the Level 3 data or unified cache\000 */
{ 152420 }, /* l3d_cache_rd\000\000cache\000Attributable Level 3 data or unified cache access, read\000event=0xa0\000\00000\000Attributable Level 3 data or unified cache access, read\000 */
{ 152665 }, /* l3d_cache_refill\000\000cache\000Attributable Level 3 data cache refill\000event=0x2a\000\00000\000Attributable Level 3 data cache refill\000 */
{ 160229 }, /* ll_cache_miss_rd\000\000cache\000Last level cache miss, read\000event=0x37\000\00000\000Last level cache miss, read\000 */
{ 160879 }, /* ll_cache_rd\000\000cache\000Attributable Last level cache memory read\000event=0x36\000\00000\000Attributable Last level cache memory read\000 */
{ 56959 }, /* exc_dabort\000\000exception\000Exception taken, Data Abort and SError\000event=0x84\000\00000\000Exception taken, Data Abort and SError\000 */
{ 57280 }, /* exc_fiq\000\000exception\000Exception taken, FIQ\000event=0x87\000\00000\000Exception taken, FIQ\000 */
{ 57633 }, /* exc_hvc\000\000exception\000Exception taken, Hypervisor Call\000event=0x8a\000\00000\000Exception taken, Hypervisor Call\000 */
{ 57939 }, /* exc_irq\000\000exception\000Exception taken, IRQ\000event=0x86\000\00000\000Exception taken, IRQ\000 */
{ 58355 }, /* exc_pabort\000\000exception\000Exception taken, Instruction Abort\000event=0x83\000\00000\000Exception taken, Instruction Abort\000 */
{ 59258 }, /* exc_smc\000\000exception\000Exception taken, Secure Monitor Call\000event=0x88\000\00000\000Exception taken, Secure Monitor Call\000 */
{ 59572 }, /* exc_svc\000\000exception\000Exception taken, Supervisor Call\000event=0x82\000\00000\000Exception taken, Supervisor Call\000 */
{ 59896 }, /* exc_taken\000\000exception\000Exception taken\000event=0x9\000\00000\000Exception taken\000 */
{ 60575 }, /* exc_trap_dabort\000\000exception\000Exception taken, Data Abort or SError not taken locally\000event=0x8c\000\00000\000Exception taken, Data Abort or SError not taken locally\000 */
{ 61080 }, /* exc_trap_fiq\000\000exception\000Exception taken, FIQ not taken locally\000event=0x8f\000\00000\000Exception taken, FIQ not taken locally\000 */
{ 62347 }, /* exc_trap_irq\000\000exception\000Exception taken, IRQ not taken locally\000event=0x8e\000\00000\000Exception taken, IRQ not taken locally\000 */
{ 62841 }, /* exc_trap_other\000\000exception\000Exception taken, Other traps not taken locally\000event=0x8d\000\00000\000Exception taken, Other traps not taken locally\000 */
{ 63383 }, /* exc_trap_pabort\000\000exception\000Exception taken, Instruction Abort not taken locally\000event=0x8b\000\00000\000Exception taken, Instruction Abort not taken locally\000 */
{ 64129 }, /* exc_undef\000\000exception\000Exception taken, Other synchronous\000event=0x81\000\00000\000Exception taken, Other synchronous\000 */
{ 165870 }, /* memory_error\000\000exception\000Local memory error\000event=0x1a\000\00000\000Local memory error\000 */
{ 20758 }, /* ase_inst_spec\000\000instruction\000ASE operations speculatively executed\000event=0x8005\000\00000\000ASE operations speculatively executed\000 */
{ 20878 }, /* ase_spec\000\000instruction\000Operation speculatively executed, Advanced SIMD instruction\000event=0x74\000\00000\000Operation speculatively executed, Advanced SIMD instruction\000 */
{ 21288 }, /* ase_sve_int16_spec\000\000instruction\000Advanced SIMD and SVE 16-bit integer operations speculatively executed\000event=0x80e7\000\00000\000Advanced SIMD and SVE 16-bit integer operations speculatively executed\000 */
{ 21479 }, /* ase_sve_int32_spec\000\000instruction\000Advanced SIMD and SVE 32-bit integer operations speculatively executed\000event=0x80eb\000\00000\000Advanced SIMD and SVE 32-bit integer operations speculatively executed\000 */
{ 21670 }, /* ase_sve_int64_spec\000\000instruction\000Advanced SIMD and SVE 64-bit integer operations speculatively executed\000event=0x80ef\000\00000\000Advanced SIMD and SVE 64-bit integer operations speculatively executed\000 */
{ 21861 }, /* ase_sve_int8_spec\000\000instruction\000Advanced SIMD and SVE 8-bit integer operations speculatively executed\000event=0x80e3\000\00000\000Advanced SIMD and SVE 8-bit integer operations speculatively executed\000 */
{ 36351 }, /* br_mis_pred_retired\000\000instruction\000Instruction architecturally executed, mispredicted branch\000event=0x22\000\00000\000Instruction architecturally executed, mispredicted branch\000 */
{ 37393 }, /* br_retired\000\000instruction\000Instruction architecturally executed, branch\000event=0x21\000\00000\000Instruction architecturally executed, branch\000 */
{ 45546 }, /* cid_write_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000event=0xb\000\00000\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000 */
{ 48419 }, /* crypto_spec\000\000instruction\000Operation speculatively executed, Cryptographic instruction\000event=0x77\000\00000\000Operation speculatively executed, Cryptographic instruction\000 */
{ 51117 }, /* dmb_spec\000\000instruction\000Barrier speculatively executed, DMB\000event=0x7e\000\00000\000Barrier speculatively executed, DMB\000 */
{ 52153 }, /* dp_spec\000\000instruction\000Operation speculatively executed, integer data processing\000event=0x73\000\00000\000Operation speculatively executed, integer data processing\000 */
{ 53209 }, /* dsb_spec\000\000instruction\000Barrier speculatively executed, DSB\000event=0x7d\000\00000\000Barrier speculatively executed, DSB\000 */
{ 58850 }, /* exc_return\000\000instruction\000Instruction architecturally executed, condition check pass, exception return\000event=0xa\000\00000\000Instruction architecturally executed, condition check pass, exception return\000 */
{ 67066 }, /* fp_dp_spec\000\000instruction\000Floating-point double-precision operations speculatively executed\000event=0x801c\000\00000\000Floating-point double-precision operations speculatively executed\000 */
{ 67239 }, /* fp_fixed_ops_spec\000\000instruction\000Non-scalable floating-point element Operations speculatively executed\000event=0x80c1\000\00000\000Non-scalable floating-point element Operations speculatively executed\000 */
{ 68165 }, /* fp_hp_spec\000\000instruction\000Floating-point half-precision operations speculatively executed\000event=0x8014\000\00000\000Floating-point half-precision operations speculatively executed\000 */
{ 68926 }, /* fp_scale_ops_spec\000\000instruction\000Scalable floating-point element Operations speculatively executed\000event=0x80c0\000\00000\000Scalable floating-point element Operations speculatively executed\000 */
{ 69704 }, /* fp_sp_spec\000\000instruction\000Floating-point single-precision operations speculatively executed\000event=0x8018\000\00000\000Floating-point single-precision operations speculatively executed\000 */
{ 90819 }, /* inst_retired\000\000instruction\000Instruction architecturally executed\000event=0x8\000\00000\000Instruction architecturally executed\000 */
{ 91235 }, /* inst_spec\000\000instruction\000Operation speculatively executed\000event=0x1b\000\00000\000Operation speculatively executed\000 */
{ 91769 }, /* isb_spec\000\000instruction\000Barrier speculatively executed, ISB\000event=0x7c\000\00000\000Barrier speculatively executed, ISB\000 */
{ 159047 }, /* ld_spec\000\000instruction\000Operation speculatively executed, load\000event=0x70\000\00000\000Operation speculatively executed, load\000 */
{ 159351 }, /* ldrex_spec\000\000instruction\000Exclusive operation speculatively executed, LDREX or LDX\000event=0x6c\000\00000\000Exclusive operation speculatively executed, LDREX or LDX\000 */
{ 173450 }, /* op_retired\000\000instruction\000Micro-operation architecturally executed\000event=0x3a\000\00000\000Micro-operation architecturally executed.  The counter counts each operation counted by OP_SPEC that would be executed in a simple sequential execution of the program\000 */
{ 173697 }, /* op_spec\000\000instruction\000Micro-operation speculatively executed\000event=0x3b\000\00000\000Micro-operation speculatively executed.  The counter counts the number of operations executed by the processing element, including those that are executed speculatively and would not be executed in a simple sequential execution of the program\000 */
{ 175115 }, /* pc_write_spec\000\000instruction\000Operation speculatively executed, software change of the PC\000event=0x76\000\00000\000Operation speculatively executed, software change of the PC\000 */
{ 177878 }, /* rc_ld_spec\000\000instruction\000Release consistency operation speculatively executed, Load-Acquire\000event=0x90\000\00000\000Release consistency operation speculatively executed, Load-Acquire\000 */
{ 178251 }, /* rc_st_spec\000\000instruction\000Release consistency operation speculatively executed, Store-Release\000event=0x91\000\00000\000Release consistency operation speculatively executed, Store-Release\000 */
{ 185522 }, /* st_spec\000\000instruction\000Operation speculatively executed, store\000event=0x71\000\00000\000Operation speculatively executed, store\000 */
{ 200454 }, /* strex_fail_spec\000\000instruction\000Exclusive operation speculatively executed, STREX or STX fail\000event=0x6e\000\00000\000Exclusive operation speculatively executed, STREX or STX fail\000 */
{ 201025 }, /* strex_pass_spec\000\000instruction\000Exclusive operation speculatively executed, STREX or STX pass\000event=0x6d\000\00000\000Exclusive operation speculatively executed, STREX or STX pass\000 */
{ 201592 }, /* strex_spec\000\000instruction\000Exclusive operation speculatively executed, STREX or STX\000event=0x6f\000\00000\000Exclusive operation speculatively executed, STREX or STX\000 */
{ 202321 }, /* sve_inst_spec\000\000instruction\000SVE operations speculatively executed\000event=0x8006\000\00000\000SVE operations speculatively executed\000 */
{ 202582 }, /* sve_ldff_fault_spec\000\000instruction\000SVE first-fault load operations speculatively executed which set FFR bit to 0\000event=0x80bd\000\00000\000SVE first-fault load operations speculatively executed which set FFR bit to 0\000 */
{ 202788 }, /* sve_ldff_spec\000\000instruction\000SVE First-fault load Operations speculatively executed\000event=0x80bc\000\00000\000SVE First-fault load Operations speculatively executed\000 */
{ 203847 }, /* sve_pred_empty_spec\000\000instruction\000SVE predicated operations with no active predicates speculatively executed\000event=0x8075\000\00000\000SVE predicated operations with no active predicates speculatively executed\000 */
{ 204047 }, /* sve_pred_full_spec\000\000instruction\000SVE predicated operations speculatively executed with all active predicates\000event=0x8076\000\00000\000SVE predicated operations speculatively executed with all active predicates\000 */
{ 204248 }, /* sve_pred_not_full_spec\000\000instruction\000SVE predicated operations with empty or partially active predicates\000event=0x8079\000\00000\000SVE predicated operations with empty or partially active predicates\000 */
{ 204437 }, /* sve_pred_partial_spec\000\000instruction\000SVE predicated operations speculatively executed with partially active predicates\000event=0x8077\000\00000\000SVE predicated operations speculatively executed with partially active predicates\000 */
{ 204653 }, /* sve_pred_spec\000\000instruction\000SVE predicated Operations speculatively executed\000event=0x8074\000\00000\000SVE predicated Operations speculatively executed\000 */
{ 205736 }, /* sw_incr\000\000instruction\000Instruction architecturally executed, Condition code check pass, software increment\000event=0\000\00000\000Instruction architecturally executed, Condition code check pass, software increment\000 */
{ 210294 }, /* ttbr_write_retired\000\000instruction\000Instruction architecturally executed, Condition code check pass, write to TTBR\000event=0x1c\000\00000\000Instruction architecturally executed, Condition code check pass, write to TTBR\000 */
{ 214354 }, /* vfp_spec\000\000instruction\000Operation speculatively executed, floating-point instruction\000event=0x75\000\00000\000Operation speculatively executed, floating-point instruction\000 */
{ 155492 }, /* ld_align_lat\000\000memory\000Load with additional latency from alignment\000event=0x4021\000\00000\000Load with additional latency from alignment\000 */
{ 159841 }, /* ldst_align_lat\000\000memory\000Access with additional latency from alignment\000event=0x4020\000\00000\000Access with additional latency from alignment\000 */
{ 163148 }, /* mem_access\000\000memory\000Data memory access\000event=0x13\000\00000\000Data memory access\000 */
{ 163403 }, /* mem_access_checked\000\000memory\000Checked data memory access\000event=0x4024\000\00000\000Checked data memory access\000 */
{ 163501 }, /* mem_access_checked_rd\000\000memory\000Checked data memory access, read\000event=0x4025\000\00000\000Checked data memory access, read\000 */
{ 163614 }, /* mem_access_checked_wr\000\000memory\000Checked data memory access, write\000event=0x4026\000\00000\000Checked data memory access, write\000 */
{ 164298 }, /* mem_access_rd\000\000memory\000Data memory access, read\000event=0x66\000\00000\000Data memory access, read\000 */
{ 164956 }, /* mem_access_wr\000\000memory\000Data memory access, write\000event=0x67\000\00000\000Data memory access, write\000 */
{ 179852 }, /* remote_access\000\000memory\000Access to another socket in a multi-socket system\000event=0x31\000\00000\000Access to another socket in a multi-socket system\000 */
{ 185116 }, /* st_align_lat\000\000memory\000Store with additional latency from alignment\000event=0x4022\000\00000\000Store with additional latency from alignment\000 */
{ 211505 }, /* unaligned_ld_spec\000\000memory\000Unaligned access, read\000event=0x68\000\00000\000Unaligned access, read\000 */
{ 212200 }, /* unaligned_ldst_spec\000\000memory\000Unaligned access\000event=0x6a\000\00000\000Unaligned access\000 */
{ 212637 }, /* unaligned_st_spec\000\000memory\000Unaligned access, write\000event=0x69\000\00000\000Unaligned access, write\000 */
{ 185831 }, /* stall\000\000pipeline\000No operation sent for execution\000event=0x3c\000\00000\000No operation sent for execution.  The counter counts every attributable cycle on which no attributable instruction or operation was sent for execution on this processing element\000 */
{ 186072 }, /* stall_backend\000\000pipeline\000No operation issued due to the backend\000event=0x24\000\00000\000No operation issued due to the backend\000 */
{ 192744 }, /* stall_backend_mem\000\000pipeline\000Memory stall cycles\000event=0x4005\000\00000\000Memory stall cycles.  The counter counts each cycle counted by STALL_BACKEND where there is a cache miss in the last level of cache within the processing element clock domain\000 */
{ 195875 }, /* stall_frontend\000\000pipeline\000No operation issued because of the frontend\000event=0x23\000\00000\000No operation issued because of the frontend\000 */
{ 199011 }, /* stall_slot\000\000pipeline\000No operation sent for execution on a slot\000event=0x3f\000\00000\000No operation sent for execution on a slot.  The counter counts on each attributable cycle the number of instruction or operation slots that were not occupied by an instruction or operation attributable to the processing element\000 */
{ 199317 }, /* stall_slot_backend\000\000pipeline\000No operation sent for execution on a slot due to the backend\000event=0x3d\000\00000\000No operation sent for execution on a slot due to the backend.  Counts each slot counted by STALL_SLOT where no attributable instruction or operation was sent for execution because the backend is unable to accept it\000 */
{ 199953 }, /* stall_slot_frontend\000\000pipeline\000No operation sent for execution on a slot due to the frontend\000event=0x3e\000\00000\000No operation sent for execution on a slot due to the frontend.  Counts each slot counted by STALL_SLOT where no attributable instruction or operation was sent for execution because there was no attributable instruction or operation available to issue from the processing element from the frontend for the slot\000 */
{ 48775 }, /* cti_trigout4\000\000trace\000Cross-trigger Interface output trigger 4\000event=0x4018\000\00000\000Cross-trigger Interface output trigger 4\000 */
{ 48894 }, /* cti_trigout5\000\000trace\000Cross-trigger Interface output trigger 5 \000event=0x4019\000\00000\000Cross-trigger Interface output trigger 5 \000 */
{ 49015 }, /* cti_trigout6\000\000trace\000Cross-trigger Interface output trigger 6\000event=0x401a\000\00000\000Cross-trigger Interface output trigger 6\000 */
{ 49134 }, /* cti_trigout7\000\000trace\000Cross-trigger Interface output trigger 7\000event=0x401b\000\00000\000Cross-trigger Interface output trigger 7\000 */
{ 209779 }, /* trb_wrap\000\000trace\000Trace buffer current write pointer wrapped\000event=0x400c\000\00000\000Trace buffer current write pointer wrapped\000 */
{ 209898 }, /* trcextout0\000\000trace\000PE Trace Unit external output 0\000event=0x4010\000\00000\000PE Trace Unit external output 0\000 */
{ 209997 }, /* trcextout1\000\000trace\000PE Trace Unit external output 1\000event=0x4011\000\00000\000PE Trace Unit external output 1\000 */
{ 210096 }, /* trcextout2\000\000trace\000PE Trace Unit external output 2\000event=0x4012\000\00000\000PE Trace Unit external output 2\000 */
{ 210195 }, /* trcextout3\000\000trace\000PE Trace Unit external output 3\000event=0x4013\000\00000\000PE Trace Unit external output 3\000 */
};

static const struct compact_pmu_event pmu_events__arm_neoverse_n1[] = {
{ 41437 }, /* bus_access\000\000bus\000Attributable Bus access\000event=0x19\000\00000\000Counts memory transactions issued by the CPU to the external bus, including snoop requests and snoop responses. Each beat of data is counted individually\000 */
{ 42337 }, /* bus_access_rd\000\000bus\000Bus access read\000event=0x60\000\00000\000Counts memory read transactions seen on the external bus. Each beat of data is counted individually\000 */
{ 42750 }, /* bus_access_wr\000\000bus\000Bus access write\000event=0x61\000\00000\000Counts memory write transactions seen on the external bus. Each beat of data is counted individually\000 */
{ 43030 }, /* bus_cycles\000\000bus\000Bus cycle\000event=0x1d\000\00000\000Counts bus cycles in the CPU. Bus cycles represent a clock cycle in which a transaction could be sent or received on the interface from the CPU to the external bus. Since that interface is driven at the same clock speed as the CPU, this event is a duplicate of CPU_CYCLES\000 */
{ 56556 }, /* exc_dabort\000\000exception\000Exception taken, Data Abort and SError\000event=0x84\000\00000\000Counts exceptions that are taken locally and are caused by data aborts or SErrors. Conditions that could cause those exceptions are attempting to read or write memory where the MMU generates a fault, attempting to read or write memory with a misaligned address, interrupts from the nSEI inputs and internally generated SErrors\000 */
{ 57153 }, /* exc_fiq\000\000exception\000Exception taken, FIQ\000event=0x87\000\00000\000Counts FIQ exceptions including the virtual FIQs that are taken locally\000 */
{ 57531 }, /* exc_hvc\000\000exception\000Exception taken, Hypervisor Call\000event=0x8a\000\00000\000Counts HVC exceptions taken to EL2\000 */
{ 57812 }, /* exc_irq\000\000exception\000Exception taken, IRQ\000event=0x86\000\00000\000Counts IRQ exceptions including the virtual IRQs that are taken locally\000 */
{ 58197 }, /* exc_pabort\000\000exception\000Exception taken, Instruction Abort\000event=0x83\000\00000\000Counts synchronous exceptions that are taken locally and caused by Instruction Aborts\000 */
{ 58462 }, /* exc_return\000\000exception\000Instruction architecturally executed, condition check pass, exception return\000event=0xa\000\00000\000Counts any architecturally executed exception return instructions. Eg: AArch64: ERET\000 */
{ 59153 }, /* exc_smc\000\000exception\000Exception taken, Secure Monitor Call\000event=0x88\000\00000\000Counts SMC exceptions take to EL3\000 */
{ 59469 }, /* exc_svc\000\000exception\000Exception taken, Supervisor Call\000event=0x82\000\00000\000Counts SVC exceptions taken locally\000 */
{ 59672 }, /* exc_taken\000\000exception\000Exception taken\000event=0x9\000\00000\000Counts any taken architecturally visible exceptions such as IRQ, FIQ, SError, and other synchronous exceptions. Exceptions are counted whether or not they are taken locally\000 */
{ 60120 }, /* exc_trap_dabort\000\000exception\000Exception taken, Data Abort or SError not taken locally\000event=0x8c\000\00000\000Counts exceptions which are traps not taken locally and are caused by Data Aborts or SError interrupts. Conditions that could cause those exceptions are:\n\n1. Attempting to read or write memory where the MMU generates a fault,\n2. Attempting to read or write memory with a misaligned address,\n3. Interrupts from the SEI input.\n4. internally generated SErrors\000 */
{ 60849 }, /* exc_trap_fiq\000\000exception\000Exception taken, FIQ not taken locally\000event=0x8f\000\00000\000Counts FIQs which are not taken locally but taken from EL0, EL1,\n or EL2 to EL3 (which would be the normal behavior for FIQs when not executing\n in EL3)\000 */
{ 62193 }, /* exc_trap_irq\000\000exception\000Exception taken, IRQ not taken locally\000event=0x8e\000\00000\000Counts IRQ exceptions including the virtual IRQs that are not taken locally\000 */
{ 62602 }, /* exc_trap_other\000\000exception\000Exception taken, Other traps not taken locally\000event=0x8d\000\00000\000Counts the number of synchronous trap exceptions which are not taken locally and are not SVC, SMC, HVC, data aborts, Instruction Aborts, or interrupts\000 */
{ 63127 }, /* exc_trap_pabort\000\000exception\000Exception taken, Instruction Abort not taken locally\000event=0x8b\000\00000\000Counts exceptions which are traps not taken locally and are caused by Instruction Aborts. For example, attempting to execute an instruction with a misaligned PC\000 */
{ 63640 }, /* exc_undef\000\000exception\000Exception taken, Other synchronous\000event=0x81\000\00000\000Counts the number of synchronous exceptions which are taken locally that are due to attempting to execute an instruction that is UNDEFINED. Attempting to execute instruction bit patterns that have not been allocated. Attempting to execute instructions when they are disabled. Attempting to execute instructions at an inappropriate Exception level. Attempting to execute an instruction when the value of PSTATE.IL is 1\000 */
{ 48246 }, /* cpu_cycles\000\000general\000Cycle\000event=0x11\000\00000\000Counts CPU clock cycles (not timer cycles). The clock measured by this event is defined as the physical clock driving the CPU logic\000 */
{ 100636 }, /* l1d_cache\000\000l1d_cache\000Level 1 data cache access\000event=0x4\000\00000\000Counts level 1 data cache accesses from any load/store operations. Atomic operations that resolve in the CPUs caches (near atomic operations) counts as both a write access and read access. Each access to a cache line is counted including the multiple accesses caused by single instructions such as LDM or STM. Each access to other level 1 data or unified memory structures, for example refill buffers, write buffers, and write-back buffers, are also counted\000 */
{ 101614 }, /* l1d_cache_inval\000\000l1d_cache\000L1D cache invalidate\000event=0x48\000\00000\000Counts each explicit invalidation of a cache line in the level 1 data cache caused by:\n\n- Cache Maintenance Operations (CMO) that operate by a virtual address.\n- Broadcast cache coherency operations from another CPU in the system.\n\nThis event does not count for the following conditions:\n\n1. A cache refill invalidates a cache line.\n2. A CMO which is executed on that CPU and invalidates a cache line specified by set/way.\n\nNote that CMOs that operate by set/way cannot be broadcast from one CPU to another\000 */
{ 102906 }, /* l1d_cache_rd\000\000l1d_cache\000L1D cache access, read\000event=0x40\000\00000\000Counts level 1 data cache accesses from any load operation. Atomic load operations that resolve in the CPUs caches counts as both a write access and read access\000 */
{ 103454 }, /* l1d_cache_refill\000\000l1d_cache\000Level 1 data cache refill\000event=0x3\000\00000\000Counts level 1 data cache refills caused by speculatively executed load or store operations that missed in the level 1 data cache. This event only counts one event per cache line. This event does not count cache line allocations from preload instructions or from hardware cache prefetching\000 */
{ 104382 }, /* l1d_cache_refill_inner\000\000l1d_cache\000L1D cache refill, inner\000event=0x44\000\00000\000Counts level 1 data cache refills where the cache line data came from caches inside the immediate cluster of the core\000 */
{ 104810 }, /* l1d_cache_refill_outer\000\000l1d_cache\000L1D cache refill, outer\000event=0x45\000\00000\000Counts level 1 data cache refills for which the cache line data came from outside the immediate cluster of the core, like an SLC in the system interconnect or DRAM\000 */
{ 106121 }, /* l1d_cache_refill_rd\000\000l1d_cache\000L1D cache refill, read\000event=0x42\000\00000\000Counts level 1 data cache refills caused by speculatively executed load instructions where the memory read operation misses in the level 1 data cache. This event only counts one event per cache line\000 */
{ 106620 }, /* l1d_cache_refill_wr\000\000l1d_cache\000L1D cache refill, write\000event=0x43\000\00000\000Counts level 1 data cache refills caused by speculatively executed store instructions where the memory write operation misses in the level 1 data cache. This event only counts one event per cache line\000 */
{ 107257 }, /* l1d_cache_wb\000\000l1d_cache\000Attributable Level 1 data cache write-back\000event=0x15\000\00000\000Counts write-backs of dirty data from the L1 data cache to the L2 cache. This occurs when either a dirty cache line is evicted from L1 data cache and allocated in the L2 cache or dirty data is written to the L2 and possibly to the next level of cache. This event counts both victim cache line evictions and cache write-backs from snoops or cache maintenance operations. The following cache operations are not counted:\n\n1. Invalidations which do not result in data being transferred out of the L1 (such as evictions of clean data),\n2. Full line writes which write to L2 without writing L1, such as write streaming mode\000 */
{ 108244 }, /* l1d_cache_wb_clean\000\000l1d_cache\000L1D cache Write-Back, cleaning and coherency\000event=0x47\000\00000\000Counts write-backs from the level 1 data cache that are a result of a coherency operation made by another CPU. Event count includes cache maintenance operations\000 */
{ 108746 }, /* l1d_cache_wb_victim\000\000l1d_cache\000L1D cache Write-Back, victim\000event=0x46\000\00000\000Counts dirty cache line evictions from the level 1 data cache caused by a new cache line allocation. This event does not count evictions caused by cache maintenance operations\000 */
{ 109214 }, /* l1d_cache_wr\000\000l1d_cache\000L1D cache access, write\000event=0x41\000\00000\000Counts level 1 data cache accesses generated by store operations. This event also counts accesses caused by a DC ZVA (data cache zero, specified by virtual address) instruction. Near atomic operations that resolve in the CPUs caches count as a write access and read access\000 */
{ 116996 }, /* l1i_cache\000\000l1i_cache\000Attributable Level 1 instruction cache access\000event=0x14\000\00000\000Counts instruction fetches which access the level 1 instruction cache. Instruction cache accesses caused by cache maintenance operations are not counted\000 */
{ 118661 }, /* l1i_cache_refill\000\000l1i_cache\000Level 1 instruction cache refill\000event=0x1\000\00000\000Counts cache line refills in the level 1 instruction cache caused by a missed instruction fetch. Instruction fetches may include accessing multiple instructions, but the single cache line allocation is counted once\000 */
{ 126580 }, /* l2d_cache\000\000l2_cache\000Level 2 data cache access\000event=0x16\000\00000\000Counts level 2 cache accesses. level 2 cache is a unified cache for data and instruction accesses. Accesses are for misses in the first level caches or translation resolutions due to accesses. This event also counts write back of dirty data from level 1 data cache to the L2 cache\000 */
{ 127360 }, /* l2d_cache_allocate\000\000l2_cache\000Attributable Level 2 data cache allocation without refill\000event=0x20\000\00000\000TBD\000 */
{ 127677 }, /* l2d_cache_inval\000\000l2_cache\000L2D cache invalidate\000event=0x58\000\00000\000Counts each explicit invalidation of a cache line in the level 2 cache by cache maintenance operations that operate by a virtual address, or by external coherency operations. This event does not count if either:\n\n1. A cache refill invalidates a cache line or,\n2. A Cache Maintenance Operation (CMO), which invalidates a cache line specified by set/way, is executed on that CPU.\n\nCMOs that operate by set/way cannot be broadcast from one CPU to another\000 */
{ 129339 }, /* l2d_cache_rd\000\000l2_cache\000L2D cache access, read\000event=0x50\000\00000\000Counts level 2 cache accesses due to memory read operations. level 2 cache is a unified cache for data and instruction accesses, accesses are for misses in the level 1 caches or translation resolutions due to accesses\000 */
{ 129941 }, /* l2d_cache_refill\000\000l2_cache\000Level 2 data refill\000event=0x17\000\00000\000Counts cache line refills into the level 2 cache. level 2 cache is a unified cache for data and instruction accesses. Accesses are for misses in the level 1 caches or translation resolutions due to accesses\000 */
{ 132515 }, /* l2d_cache_refill_rd\000\000l2_cache\000L2D cache refill, read\000event=0x52\000\00000\000Counts refills for memory accesses due to memory read operation counted by L2D_CACHE_RD. level 2 cache is a unified cache for data and instruction accesses, accesses are for misses in the level 1 caches or translation resolutions due to accesses\000 */
{ 133060 }, /* l2d_cache_refill_wr\000\000l2_cache\000L2D cache refill, write\000event=0x53\000\00000\000Counts refills for memory accesses due to memory write operation counted by L2D_CACHE_WR. level 2 cache is a unified cache for data and instruction accesses, accesses are for misses in the level 1 caches or translation resolutions due to accesses\000 */
{ 136920 }, /* l2d_cache_wb\000\000l2_cache\000Attributable Level 2 data cache write-back\000event=0x18\000\00000\000Counts write-backs of data from the L2 cache to outside the CPU. This includes snoops to the L2 (from other CPUs) which return data even if the snoops cause an invalidation. L2 cache line invalidations which do not write data outside the CPU and snoops which return data from an L1 cache are not counted. Data would not be written outside the cache when invalidating a clean cache line\000 */
{ 137674 }, /* l2d_cache_wb_clean\000\000l2_cache\000L2D cache Write-Back, cleaning and coherency\000event=0x57\000\00000\000Counts write-backs from the level 2 cache that are a result of either:\n\n1. Cache maintenance operations,\n\n2. Snoop responses or,\n\n3. Direct cache transfers to another CPU due to a forwarding snoop request\000 */
{ 138219 }, /* l2d_cache_wb_victim\000\000l2_cache\000L2D cache Write-Back, victim\000event=0x56\000\00000\000Counts evictions from the level 2 cache because of a line being allocated into the L2 cache\000 */
{ 138602 }, /* l2d_cache_wr\000\000l2_cache\000L2D cache access, write\000event=0x51\000\00000\000Counts level 2 cache accesses due to memory write operations. level 2 cache is a unified cache for data and instruction accesses, accesses are for misses in the level 1 caches or translation resolutions due to accesses\000 */
{ 150944 }, /* l3d_cache\000\000l3_cache\000Attributable Level 3 data cache access\000event=0x2b\000\00000\000Counts level 3 cache accesses. level 3 cache is a unified cache for data and instruction accesses. Accesses are for misses in the lower level caches or translation resolutions due to accesses\000 */
{ 151616 }, /* l3d_cache_allocate\000\000l3_cache\000Attributable Level 3 data cache allocation without refill\000event=0x29\000\00000\000Counts level 3 cache line allocates that do not fetch data from outside the level 3 data or unified cache. For example, allocates due to streaming stores\000 */
{ 152567 }, /* l3d_cache_rd\000\000l3_cache\000Attributable Level 3 data or unified cache access, read\000event=0xa0\000\00000\000TBD\000 */
{ 153123 }, /* l3d_cache_refill\000\000l3_cache\000Attributable Level 3 data cache refill\000event=0x2a\000\00000\000Counts level 3 accesses that receive data from outside the L3 cache\000 */
{ 160324 }, /* ll_cache_miss_rd\000\000ll_cache\000Last level cache miss, read\000event=0x37\000\00000\000Counts read transactions that were returned from outside the core cluster but missed in the system level cache. This event counts when the system register CPUECTLR.EXTLLC bit is set. This event counts read transactions returned from outside the core if those transactions are missed in the System level Cache. The data source of the transaction is indicated by a field in the CHI transaction returning to the CPU. This event does not count reads caused by cache maintenance operations\000 */
{ 160997 }, /* ll_cache_rd\000\000ll_cache\000Attributable Last level cache memory read\000event=0x36\000\00000\000Counts read transactions that were returned from outside the core cluster. This event counts when the system register CPUECTLR.EXTLLC bit is set. This event counts read transactions returned from outside the core if those transactions are either hit in the system level cache or missed in the SLC and are returned from any other external sources\000 */
{ 162670 }, /* mem_access\000\000memory\000Data memory access\000event=0x13\000\00000\000Counts memory accesses issued by the CPU load store unit, where those accesses are issued due to load or store operations. This event counts memory accesses no matter whether the data is received from any level of cache hierarchy or external memory. If memory accesses are broken up into smaller transactions than what were specified in the load or store instructions, then the event counts those smaller memory transactions\000 */
{ 163822 }, /* mem_access_rd\000\000memory\000Data memory access, read\000event=0x66\000\00000\000Counts memory accesses issued by the CPU due to load operations. The event counts any memory load access, no matter whether the data is received from any level of cache hierarchy or external memory. The event also counts atomic load operations. If memory accesses are broken up by the load/store unit into smaller transactions that are issued by the bus interface, then the event counts those smaller transactions\000 */
{ 164480 }, /* mem_access_wr\000\000memory\000Data memory access, write\000event=0x67\000\00000\000Counts memory accesses issued by the CPU due to store operations. The event counts any memory store access, no matter whether the data is located in any level of cache or external memory. The event also counts atomic load and store operations. If memory accesses are broken up by the load/store unit into smaller transactions that are issued by the bus interface, then the event counts those smaller transactions\000 */
{ 166112 }, /* memory_error\000\000memory\000Local memory error\000event=0x1a\000\00000\000Counts any detected correctable or uncorrectable physical memory errors (ECC or parity) in protected CPUs RAMs. On the core, this event counts errors in the caches (including data and tag rams). Any detected memory error (from either a speculative and abandoned access, or an architecturally executed access) is counted. Note that errors are only detected when the actual protected memory is accessed by an operation\000 */
{ 179989 }, /* remote_access\000\000memory\000Access to another socket in a multi-socket system\000event=0x31\000\00000\000Counts accesses to another chip, which is implemented as a different CMN mesh in the system. If the CHI bus response back to the core indicates that the data source is from another chip (mesh), then the counter is updated. If no data is returned, even if the system snoops another chip/mesh, then the counter is not updated\000 */
{ 36733 }, /* br_mis_pred_retired\000\000retired\000Instruction architecturally executed, mispredicted branch\000event=0x22\000\00000\000Counts branches counted by BR_RETIRED which were mispredicted and caused a pipeline flush\000 */
{ 37719 }, /* br_retired\000\000retired\000Instruction architecturally executed, branch\000event=0x21\000\00000\000Counts architecturally executed branches, whether the branch is taken or not. Instructions that explicitly write to the PC are also counted\000 */
{ 46156 }, /* cid_write_retired\000\000retired\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000event=0xb\000\00000\000Counts architecturally executed writes to the CONTEXTIDR register, which usually contain the kernel PID and can be output with hardware trace\000 */
{ 91102 }, /* inst_retired\000\000retired\000Instruction architecturally executed\000event=0x8\000\00000\000Counts instructions that have been architecturally executed\000 */
{ 206046 }, /* sw_incr\000\000retired\000Instruction architecturally executed, Condition code check pass, software increment\000event=0\000\00000\000Counts software writes to the PMSWINC_EL0 (software PMU increment) register. The PMSWINC_EL0 register is a manually updated counter for use by application software.\n\nThis event could be used to measure any user program event, such as accesses to a particular data structure (by writing to the PMSWINC_EL0 register each time the data structure is accessed).\n\nTo use the PMSWINC_EL0 register and event, developers must insert instructions that write to the PMSWINC_EL0 register into the source code.\n\nSince the SW_INCR event records writes to the PMSWINC_EL0 register, there is no need to do a read/increment/write sequence to the PMSWINC_EL0 register\000 */
{ 210728 }, /* ttbr_write_retired\000\000retired\000Instruction architecturally executed, Condition code check pass, write to TTBR\000event=0x1c\000\00000\000Counts architectural writes to TTBR0/1_EL1. If virtualization host extensions are enabled (by setting the HCR_EL2.E2H bit to 1), then accesses to TTBR0/1_EL1 that are redirected to TTBR0/1_EL2, or accesses to TTBR0/1_EL12, are counted. TTBRn registers are typically updated when the kernel is swapping user-space threads or applications\000 */
{ 182929 }, /* sample_collision\000\000spe\000Sample collided with previous sample\000event=0x4003\000\00000\000Counts statistical profiling samples that have collided with a previous sample and so therefore not taken\000 */
{ 183224 }, /* sample_feed\000\000spe\000Sample Taken\000event=0x4001\000\00000\000Counts statistical profiling samples taken for sampling\000 */
{ 183387 }, /* sample_filtrate\000\000spe\000Sample Taken and not removed by filtering\000event=0x4002\000\00000\000Counts statistical profiling samples taken which are not removed by filtering\000 */
{ 183667 }, /* sample_pop\000\000spe\000Sample Population\000event=0x4000\000\00000\000Counts statistical profiling sample population, the count of all operations that could be sampled but may or may not be chosen for sampling\000 */
{ 21035 }, /* ase_spec\000\000spec_operation\000Operation speculatively executed, Advanced SIMD instruction\000event=0x74\000\00000\000Counts speculatively executed Advanced SIMD operations excluding load, store and move micro-operations that move data to or from SIMD (vector) registers\000 */
{ 31024 }, /* br_immed_spec\000\000spec_operation\000Branch speculatively executed, immediate branch\000event=0x78\000\00000\000Counts immediate branch operations which are speculatively executed\000 */
{ 35311 }, /* br_indirect_spec\000\000spec_operation\000Branch speculatively executed, indirect branch\000event=0x7a\000\00000\000Counts indirect branch operations including procedure returns, which are speculatively executed. This includes operations that force a software change of the PC, other than exception-generating operations.  Eg: BR Xn, RET\000 */
{ 36182 }, /* br_mis_pred\000\000spec_operation\000Mispredicted or not predicted branch speculatively executed\000event=0x10\000\00000\000Counts branches which are speculatively executed and mispredicted\000 */
{ 37248 }, /* br_pred\000\000spec_operation\000Predictable branch speculatively executed\000event=0x12\000\00000\000Counts branches speculatively executed and were predicted right\000 */
{ 40965 }, /* br_return_spec\000\000spec_operation\000Branch speculatively executed, procedure return\000event=0x79\000\00000\000Counts procedure return operations (RET) which are speculatively executed\000 */
{ 48579 }, /* crypto_spec\000\000spec_operation\000Operation speculatively executed, Cryptographic instruction\000event=0x77\000\00000\000Counts speculatively executed cryptographic operations except for PMULL and VMULL operations\000 */
{ 51226 }, /* dmb_spec\000\000spec_operation\000Barrier speculatively executed, DMB\000event=0x7e\000\00000\000Counts DMB operations that are speculatively issued to the Load/Store unit in the CPU. This event does not count implied barriers from load acquire/store release operations\000 */
{ 52305 }, /* dp_spec\000\000spec_operation\000Operation speculatively executed, integer data processing\000event=0x73\000\00000\000Counts speculatively executed logical or arithmetic instructions such as MOV/MVN operations\000 */
{ 53318 }, /* dsb_spec\000\000spec_operation\000Barrier speculatively executed, DSB\000event=0x7d\000\00000\000Counts DSB operations that are speculatively issued to Load/Store unit in the CPU\000 */
{ 91339 }, /* inst_spec\000\000spec_operation\000Operation speculatively executed\000event=0x1b\000\00000\000Counts operations that have been speculatively executed\000 */
{ 91878 }, /* isb_spec\000\000spec_operation\000Barrier speculatively executed, ISB\000event=0x7c\000\00000\000Counts ISB operations that are executed\000 */
{ 159161 }, /* ld_spec\000\000spec_operation\000Operation speculatively executed, load\000event=0x70\000\00000\000Counts speculatively executed load operations including Single Instruction Multiple Data (SIMD) load operations\000 */
{ 159655 }, /* ldrex_spec\000\000spec_operation\000Exclusive operation speculatively executed, LDREX or LDX\000event=0x6c\000\00000\000Counts Load-Exclusive operations that have been speculatively executed. Eg: LDREX, LDX\000 */
{ 175277 }, /* pc_write_spec\000\000spec_operation\000Operation speculatively executed, software change of the PC\000event=0x76\000\00000\000Counts speculatively executed operations which cause software changes of the PC. Those operations include all taken branch operations\000 */
{ 178051 }, /* rc_ld_spec\000\000spec_operation\000Release consistency operation speculatively executed, Load-Acquire\000event=0x90\000\00000\000Counts any load acquire operations that are speculatively executed. Eg: LDAR, LDARH, LDARB\000 */
{ 178426 }, /* rc_st_spec\000\000spec_operation\000Release consistency operation speculatively executed, Store-Release\000event=0x91\000\00000\000Counts any store release operations that are speculatively executed. Eg: STLR, STLRH, STLRB'\000 */
{ 185638 }, /* st_spec\000\000spec_operation\000Operation speculatively executed, store\000event=0x71\000\00000\000Counts speculatively executed store operations including Single Instruction Multiple Data (SIMD) store operations\000 */
{ 200788 }, /* strex_fail_spec\000\000spec_operation\000Exclusive operation speculatively executed, STREX or STX fail\000event=0x6e\000\00000\000Counts store-exclusive operations that have been speculatively executed and have not successfully completed the store operation\000 */
{ 201359 }, /* strex_pass_spec\000\000spec_operation\000Exclusive operation speculatively executed, STREX or STX pass\000event=0x6d\000\00000\000Counts store-exclusive operations that have been speculatively executed and have successfully completed the store operation\000 */
{ 201896 }, /* strex_spec\000\000spec_operation\000Exclusive operation speculatively executed, STREX or STX\000event=0x6f\000\00000\000Counts store-exclusive operations that have been speculatively executed\000 */
{ 211592 }, /* unaligned_ld_spec\000\000spec_operation\000Unaligned access, read\000event=0x68\000\00000\000Counts unaligned memory read operations issued by the CPU. This event counts unaligned accesses (as defined by the actual instruction), even if they are subsequently issued as multiple aligned accesses. The event does not count preload operations (PLD, PLI)\000 */
{ 212277 }, /* unaligned_ldst_spec\000\000spec_operation\000Unaligned access\000event=0x6a\000\00000\000Counts unaligned memory operations issued by the CPU. This event counts unaligned accesses (as defined by the actual instruction), even if they are subsequently issued as multiple aligned accesses\000 */
{ 212726 }, /* unaligned_st_spec\000\000spec_operation\000Unaligned access, write\000event=0x69\000\00000\000Counts unaligned memory write operations issued by the CPU. This event counts unaligned accesses (as defined by the actual instruction), even if they are subsequently issued as multiple aligned accesses\000 */
{ 214513 }, /* vfp_spec\000\000spec_operation\000Operation speculatively executed, floating-point instruction\000event=0x75\000\00000\000Counts speculatively executed floating point operations. This event does not count operations that move data to or from floating point (vector) registers\000 */
{ 186367 }, /* stall_backend\000\000stall\000No operation issued due to the backend\000event=0x24\000\00000\000Counts cycles whenever the rename unit is unable to send any micro-operations to the backend of the pipeline because of backend resource constraints. Backend resource constraints can include issue stage fullness, execution stage fullness, or other internal pipeline resource fullness. All the backend slots were empty during the cycle when this event counts\000 */
{ 196180 }, /* stall_frontend\000\000stall\000No operation issued because of the frontend\000event=0x23\000\00000\000Counts cycles when frontend could not send any micro-operations to the rename stage because of frontend resource stalls caused by fetch memory latency or branch prediction flow stalls. All the frontend slots were empty during the cycle when this event counts\000 */
{ 53817 }, /* dtlb_walk\000\000tlb\000Access to data TLB causes a translation table walk\000event=0x34\000\00000\000Counts data memory translation table walks caused by a miss in the L2 TLB driven by a memory access. Note that partial translations that also cause a table walk are counted. This event does not count table walks caused by TLB maintenance operations\000 */
{ 92378 }, /* itlb_walk\000\000tlb\000Access to instruction TLB that causes a translation table walk\000event=0x35\000\00000\000Counts instruction memory translation table walks caused by a miss in the L2 TLB driven by a memory access. Partial translations that also cause a table walk are counted. This event does not count table walks caused by TLB maintenance operations\000 */
{ 110383 }, /* l1d_tlb\000\000tlb\000Attributable Level 1 data or unified TLB access\000event=0x25\000\00000\000Counts level 1 data TLB accesses caused by any memory load or store operation. Note that load or store instructions can be broken up into multiple memory operations. This event does not count TLB maintenance operations\000 */
{ 110971 }, /* l1d_tlb_rd\000\000tlb\000L1D tlb access, read\000event=0x4e\000\00000\000Counts level 1 data TLB accesses caused by memory read operations. This event counts whether the access hits or misses in the TLB. This event does not count TLB maintenance operations\000 */
{ 111512 }, /* l1d_tlb_refill\000\000tlb\000Attributable Level 1 data TLB refill\000event=0x5\000\00000\000Counts level 1 data TLB accesses that resulted in TLB refills. If there are multiple misses in the TLB that are resolved by the refill, then this event only counts once. This event counts for refills caused by preload instructions or hardware prefetch accesses. This event counts regardless of whether the miss hits in L2 or results in a translation table walk. This event will not count if the translation table walk results in a fault (such as a translation or access fault), since there is no new translation created for the TLB. This event will not count on an access from an AT(address translation) instruction\000 */
{ 112414 }, /* l1d_tlb_refill_rd\000\000tlb\000L1D tlb refill, read\000event=0x4c\000\00000\000Counts level 1 data TLB refills caused by memory read operations. If there are multiple misses in the TLB that are resolved by the refill, then this event only counts once. This event counts for refills caused by preload instructions or hardware prefetch accesses. This event counts regardless of whether the miss hits in L2 or results in a translation table walk. This event will not count if the translation table walk results in a fault (such as a translation or access fault), since there is no new translation created for the TLB. This event will not count on an access from an Address Translation (AT) instruction\000 */
{ 113312 }, /* l1d_tlb_refill_wr\000\000tlb\000L1D tlb refill, write\000event=0x4d\000\00000\000Counts level 1 data TLB refills caused by data side memory write operations. If there are multiple misses in the TLB that are resolved by the refill, then this event only counts once. This event counts for refills caused by preload instructions or hardware prefetch accesses. This event counts regardless of whether the miss hits in L2 or results in a translation table walk. This event will not count if the table walk results in a fault (such as a translation or access fault), since there is no new translation created for the TLB. This event will not count with an access from an Address Translation (AT) instruction\000 */
{ 114220 }, /* l1d_tlb_wr\000\000tlb\000L1D tlb access, write\000event=0x4f\000\00000\000Counts any L1 data side TLB accesses caused by memory write operations. This event counts whether the access hits or misses in the TLB. This event does not count TLB maintenance operations\000 */
{ 119887 }, /* l1i_tlb\000\000tlb\000Attributable Level 1 instruction TLB access\000event=0x26\000\00000\000Counts level 1 instruction TLB accesses, whether the access hits or misses in the TLB. This event counts both demand accesses and prefetch or preload generated accesses\000 */
{ 120531 }, /* l1i_tlb_refill\000\000tlb\000Attributable Level 1 instruction TLB refill\000event=0x2\000\00000\000Counts level 1 instruction TLB refills from any Instruction fetch. If there are multiple misses in the TLB that are resolved by the refill, then this event only counts once. This event will not count if the translation table walk results in a fault (such as a translation or access fault), since there is no new translation created for the TLB\000 */
{ 143732 }, /* l2d_tlb\000\000tlb\000Attributable Level 2 data or unified TLB access\000event=0x2f\000\00000\000Counts level 2 TLB accesses except those caused by TLB maintenance operations\000 */
{ 144297 }, /* l2d_tlb_rd\000\000tlb\000L2D cache access, read\000event=0x5e\000\00000\000Counts level 2 TLB accesses caused by memory read operations from both data and instruction fetch except for those caused by TLB maintenance operations\000 */
{ 144836 }, /* l2d_tlb_refill\000\000tlb\000Attributable Level 2 data TLB refill\000event=0x2d\000\00000\000Counts level 2 TLB refills caused by memory operations from both data and instruction fetch, except for those caused by TLB maintenance operations and hardware prefetches\000 */
{ 145320 }, /* l2d_tlb_refill_rd\000\000tlb\000L2D cache refill, read\000event=0x5c\000\00000\000Counts level 2 TLB refills caused by memory read operations from both data and instruction fetch except for those caused by TLB maintenance operations or hardware prefetches\000 */
{ 145800 }, /* l2d_tlb_refill_wr\000\000tlb\000L2D cache refill, write\000event=0x5d\000\00000\000Counts level 2 TLB refills caused by memory write operations from both data and instruction fetch except for those caused by TLB maintenance operations\000 */
{ 146245 }, /* l2d_tlb_wr\000\000tlb\000L2D cache access, write\000event=0x5f\000\00000\000Counts level 2 TLB accesses caused by memory write operations from both data and instruction fetch except for those caused by TLB maintenance operations\000 */
};

static const struct compact_pmu_event pmu_metrics__arm_neoverse_n1[] = {
{ 1301 }, /* \000backend_stalled_cycles\000Cycle_Accounting\000STALL_BACKEND / CPU_CYCLES * 100\000\000This metric is the percentage of cycles that were stalled due to resource constraints in the backend unit of the processor\000\0001percent of cycles\000\000\000\00000 */
{ 2400 }, /* \000branch_misprediction_ratio\000Miss_Ratio;Branch_Effectiveness\000BR_MIS_PRED_RETIRED / BR_RETIRED\000\000This metric measures the ratio of branches mispredicted to the total number of branches architecturally executed. This gives an indication of the effectiveness of the branch prediction unit\000\0001per branch\000\000\000\00000 */
{ 3154 }, /* \000branch_mpki\000MPKI;Branch_Effectiveness\000BR_MIS_PRED_RETIRED / INST_RETIRED * 1e3\000\000This metric measures the number of branch mispredictions per thousand instructions executed\000\0001MPKI\000\000\000\00000 */
{ 3339 }, /* \000branch_percentage\000Operation_Mix\000(BR_IMMED_SPEC + BR_INDIRECT_SPEC) / INST_SPEC * 100\000\000This metric measures branch operations as a percentage of operations speculatively executed\000\0001percent of operations\000\000\000\00000 */
{ 4371 }, /* \000crypto_percentage\000Operation_Mix\000CRYPTO_SPEC / INST_SPEC * 100\000\000This metric measures crypto operations as a percentage of operations speculatively executed\000\0001percent of operations\000\000\000\00000 */
{ 5100 }, /* \000dtlb_mpki\000MPKI;DTLB_Effectiveness\000DTLB_WALK / INST_RETIRED * 1e3\000\000This metric measures the number of data TLB Walks per thousand instructions executed\000\0001MPKI\000\000\000\00000 */
{ 5491 }, /* \000dtlb_walk_ratio\000Miss_Ratio;DTLB_Effectiveness\000DTLB_WALK / L1D_TLB\000\000This metric measures the ratio of instruction TLB Walks to the total number of data TLB accesses. This gives an indication of the effectiveness of the data TLB accesses\000\0001per TLB access\000\000\000\00000 */
{ 6709 }, /* \000frontend_stalled_cycles\000Cycle_Accounting\000STALL_FRONTEND / CPU_CYCLES * 100\000\000This metric is the percentage of cycles that were stalled due to resource constraints in the frontend unit of the processor\000\0001percent of cycles\000\000\000\00000 */
{ 7642 }, /* \000integer_dp_percentage\000Operation_Mix\000DP_SPEC / INST_SPEC * 100\000\000This metric measures scalar integer operations as a percentage of operations speculatively executed\000\0001percent of operations\000\000\000\00000 */
{ 7835 }, /* \000ipc\000General\000INST_RETIRED / CPU_CYCLES\000\000This metric measures the number of instructions retired per cycle\000\0001per cycle\000\000\000\00000 */
{ 8159 }, /* \000itlb_mpki\000MPKI;ITLB_Effectiveness\000ITLB_WALK / INST_RETIRED * 1e3\000\000This metric measures the number of instruction TLB Walks per thousand instructions executed\000\0001MPKI\000\000\000\00000 */
{ 8564 }, /* \000itlb_walk_ratio\000Miss_Ratio;ITLB_Effectiveness\000ITLB_WALK / L1I_TLB\000\000This metric measures the ratio of instruction TLB Walks to the total number of instruction TLB accesses. This gives an indication of the effectiveness of the instruction TLB accesses\000\0001per TLB access\000\000\000\00000 */
{ 9094 }, /* \000l1d_cache_miss_ratio\000Miss_Ratio;L1D_Cache_Effectiveness\000L1D_CACHE_REFILL / L1D_CACHE\000\000This metric measures the ratio of level 1 data cache accesses missed to the total number of level 1 data cache accesses. This gives an indication of the effectiveness of the level 1 data cache\000\0001per cache access\000\000\000\00000 */
{ 9523 }, /* \000l1d_cache_mpki\000MPKI;L1D_Cache_Effectiveness\000L1D_CACHE_REFILL / INST_RETIRED * 1e3\000\000This metric measures the number of level 1 data cache accesses missed per thousand instructions executed\000\0001MPKI\000\000\000\00000 */
{ 9842 }, /* \000l1d_tlb_miss_ratio\000Miss_Ratio;DTLB_Effectiveness\000L1D_TLB_REFILL / L1D_TLB\000\000This metric measures the ratio of level 1 data TLB accesses missed to the total number of level 1 data TLB accesses. This gives an indication of the effectiveness of the level 1 data TLB\000\0001per TLB access\000\000\000\00000 */
{ 10127 }, /* \000l1d_tlb_mpki\000MPKI;DTLB_Effectiveness\000L1D_TLB_REFILL / INST_RETIRED * 1e3\000\000This metric measures the number of level 1 instruction TLB accesses missed per thousand instructions executed\000\0001MPKI\000\000\000\00000 */
{ 10448 }, /* \000l1i_cache_miss_ratio\000Miss_Ratio;L1I_Cache_Effectiveness\000L1I_CACHE_REFILL / L1I_CACHE\000\000This metric measures the ratio of level 1 instruction cache accesses missed to the total number of level 1 instruction cache accesses. This gives an indication of the effectiveness of the level 1 instruction cache\000\0001per cache access\000\000\000\00000 */
{ 10898 }, /* \000l1i_cache_mpki\000MPKI;L1I_Cache_Effectiveness\000L1I_CACHE_REFILL / INST_RETIRED * 1e3\000\000This metric measures the number of level 1 instruction cache accesses missed per thousand instructions executed\000\0001MPKI\000\000\000\00000 */
{ 11224 }, /* \000l1i_tlb_miss_ratio\000Miss_Ratio;ITLB_Effectiveness\000L1I_TLB_REFILL / L1I_TLB\000\000This metric measures the ratio of level 1 instruction TLB accesses missed to the total number of level 1 instruction TLB accesses. This gives an indication of the effectiveness of the level 1 instruction TLB\000\0001per TLB access\000\000\000\00000 */
{ 11530 }, /* \000l1i_tlb_mpki\000MPKI;ITLB_Effectiveness\000L1I_TLB_REFILL / INST_RETIRED * 1e3\000\000This metric measures the number of level 1 instruction TLB accesses missed per thousand instructions executed\000\0001MPKI\000\000\000\00000 */
{ 11859 }, /* \000l2_cache_miss_ratio\000Miss_Ratio;L2_Cache_Effectiveness\000L2D_CACHE_REFILL / L2D_CACHE\000\000This metric measures the ratio of level 2 cache accesses missed to the total number of level 2 cache accesses. This gives an indication of the effectiveness of the level 2 cache, which is a unified cache that stores both data and instruction. Note that cache accesses in this cache are either data memory access or instruction fetch as this is a unified cache\000\0001per cache access\000\000\000\00000 */
{ 12328 }, /* \000l2_cache_mpki\000MPKI;L2_Cache_Effectiveness\000L2D_CACHE_REFILL / INST_RETIRED * 1e3\000\000This metric measures the number of level 2 unified cache accesses missed per thousand instructions executed. Note that cache accesses in this cache are either data memory access or instruction fetch as this is a unified cache\000\0001MPKI\000\000\000\00000 */
{ 12765 }, /* \000l2_tlb_miss_ratio\000Miss_Ratio;ITLB_Effectiveness;DTLB_Effectiveness\000L2D_TLB_REFILL / L2D_TLB\000\000This metric measures the ratio of level 2 unified TLB accesses missed to the total number of level 2 unified TLB accesses. This gives an indication of the effectiveness of the level 2 TLB\000\0001per TLB access\000\000\000\00000 */
{ 13069 }, /* \000l2_tlb_mpki\000MPKI;ITLB_Effectiveness;DTLB_Effectiveness\000L2D_TLB_REFILL / INST_RETIRED * 1e3\000\000This metric measures the number of level 2 unified TLB accesses missed per thousand instructions executed\000\0001MPKI\000\000\000\00000 */
{ 13926 }, /* \000ll_cache_read_hit_ratio\000LL_Cache_Effectiveness\000(LL_CACHE_RD - LL_CACHE_MISS_RD) / LL_CACHE_RD\000\000This metric measures the ratio of last level cache read accesses hit in the cache to the total number of last level cache accesses. This gives an indication of the effectiveness of the last level cache for read traffic. Note that cache accesses in this cache are either data memory access or instruction fetch as this is a system level cache\000\0001per cache access\000\000\000\00000 */
{ 14524 }, /* \000ll_cache_read_miss_ratio\000Miss_Ratio;LL_Cache_Effectiveness\000LL_CACHE_MISS_RD / LL_CACHE_RD\000\000This metric measures the ratio of last level cache read accesses missed to the total number of last level cache accesses. This gives an indication of the effectiveness of the last level cache for read traffic. Note that cache accesses in this cache are either data memory access or instruction fetch as this is a system level cache\000\0001per cache access\000\000\000\00000 */
{ 15104 }, /* \000ll_cache_read_mpki\000MPKI;LL_Cache_Effectiveness\000LL_CACHE_MISS_RD / INST_RETIRED * 1e3\000\000This metric measures the number of last level cache read accesses missed per thousand instructions executed\000\0001MPKI\000\000\000\00000 */
{ 15311 }, /* \000load_percentage\000Operation_Mix\000LD_SPEC / INST_SPEC * 100\000\000This metric measures load operations as a percentage of operations speculatively executed\000\0001percent of operations\000\000\000\00000 */
{ 18147 }, /* \000scalar_fp_percentage\000Operation_Mix\000VFP_SPEC / INST_SPEC * 100\000\000This metric measures scalar floating point operations as a percentage of operations speculatively executed\000\0001percent of operations\000\000\000\00000 */
{ 18347 }, /* \000simd_percentage\000Operation_Mix\000ASE_SPEC / INST_SPEC * 100\000\000This metric measures advanced SIMD operations as a percentage of total operations speculatively executed\000\0001percent of operations\000\000\000\00000 */
{ 18934 }, /* \000store_percentage\000Operation_Mix\000ST_SPEC / INST_SPEC * 100\000\000This metric measures store operations as a percentage of operations speculatively executed\000\0001percent of operations\000\000\000\00000 */
};

static const struct compact_pmu_event pmu_events__arm_neoverse_n2_v2[] = {
{ 35743 }, /* br_mis_pred\000\000branch\000Mispredicted or not predicted branch speculatively executed\000event=0x10\000\00000\000Mispredicted or not predicted branch speculatively executed\000 */
{ 36925 }, /* br_pred\000\000branch\000Predictable branch speculatively executed\000event=0x12\000\00000\000Predictable branch speculatively executed\000 */
{ 41358 }, /* bus_access\000\000bus\000Attributable Bus access\000event=0x19\000\00000\000Attributable Bus access\000 */
{ 42271 }, /* bus_access_rd\000\000bus\000Bus access read\000event=0x60\000\00000\000Bus access read\000 */
{ 42682 }, /* bus_access_wr\000\000bus\000Bus access write\000event=0x61\000\00000\000Bus access write\000 */
{ 42979 }, /* bus_cycles\000\000bus\000Bus cycle\000event=0x1d\000\00000\000Bus cycle\000 */
{ 46424 }, /* cnt_cycles\000\000bus\000Constant frequency cycles\000event=0x4004\000\00000\000Constant frequency cycles.  The counter increments at a constant frequency equal to the rate of increment of the system counter, CNTPCT_EL0\000 */
{ 47992 }, /* cpu_cycles\000\000bus\000Cycle\000event=0x11\000\00000\000Cycle\000 */
{ 53526 }, /* dtlb_walk\000\000cache\000Access to data TLB causes a translation table walk\000event=0x34\000\00000\000Access to data TLB causes a translation table walk\000 */
{ 92044 }, /* itlb_walk\000\000cache\000Access to instruction TLB that causes a translation table walk\000event=0x35\000\00000\000Access to instruction TLB that causes a translation table walk\000 */
{ 100289 }, /* l1d_cache\000\000cache\000Level 1 data cache access\000event=0x4\000\00000\000Level 1 data cache access\000 */
{ 101403 }, /* l1d_cache_inval\000\000cache\000L1D cache invalidate\000event=0x48\000\00000\000Attributable Level 1 data cache invalidate\000 */
{ 102276 }, /* l1d_cache_lmiss_rd\000\000cache\000Level 1 data cache long-latency read miss\000event=0x39\000\00000\000Level 1 data cache long-latency read miss.  The counter counts each memory read access counted by L1D_CACHE that incurs additional latency because it returns data from outside the Level 1 data or unified cache of this processing element\000 */
{ 102693 }, /* l1d_cache_rd\000\000cache\000L1D cache access, read\000event=0x40\000\00000\000Attributable Level 1 data cache access, read\000 */
{ 103129 }, /* l1d_cache_refill\000\000cache\000Level 1 data cache refill\000event=0x3\000\00000\000Level 1 data cache refill\000 */
{ 104145 }, /* l1d_cache_refill_inner\000\000cache\000L1D cache refill, inner\000event=0x44\000\00000\000Attributable Level 1 data cache refill, inner\000 */
{ 104573 }, /* l1d_cache_refill_outer\000\000cache\000L1D cache refill, outer\000event=0x45\000\00000\000Attributable Level 1 data cache refill, outer\000 */
{ 105894 }, /* l1d_cache_refill_rd\000\000cache\000L1D cache refill, read\000event=0x42\000\00000\000Attributable Level 1 data cache refill, read\000 */
{ 106389 }, /* l1d_cache_refill_wr\000\000cache\000L1D cache refill, write\000event=0x43\000\00000\000Attributable Level 1 data cache refill, write\000 */
{ 106891 }, /* l1d_cache_wb\000\000cache\000Attributable Level 1 data cache write-back\000event=0x15\000\00000\000Attributable Level 1 data cache write-back\000 */
{ 107957 }, /* l1d_cache_wb_clean\000\000cache\000L1D cache Write-Back, cleaning and coherency\000event=0x47\000\00000\000Level 1 data cache Write-Back, cleaning and coherency\000 */
{ 108495 }, /* l1d_cache_wb_victim\000\000cache\000L1D cache Write-Back, victim\000event=0x46\000\00000\000Attributable Level 1 data cache Write-Back, victim\000 */
{ 108997 }, /* l1d_cache_wr\000\000cache\000L1D cache access, write\000event=0x41\000\00000\000Attributable Level 1 data cache access, write\000 */
{ 109867 }, /* l1d_tlb\000\000cache\000Attributable Level 1 data or unified TLB access\000event=0x25\000\00000\000Attributable Level 1 data or unified TLB access\000 */
{ 110748 }, /* l1d_tlb_rd\000\000cache\000L1D tlb access, read\000event=0x4e\000\00000\000Attributable Level 1 data or unified TLB access, read\000 */
{ 111207 }, /* l1d_tlb_refill\000\000cache\000Attributable Level 1 data TLB refill\000event=0x5\000\00000\000Attributable Level 1 data TLB refill\000 */
{ 112199 }, /* l1d_tlb_refill_rd\000\000cache\000L1D tlb refill, read\000event=0x4c\000\00000\000Attributable Level 1 data TLB refill, read\000 */
{ 113093 }, /* l1d_tlb_refill_wr\000\000cache\000L1D tlb refill, write\000event=0x4d\000\00000\000Attributable Level 1 data TLB refill, write\000 */
{ 113993 }, /* l1d_tlb_wr\000\000cache\000L1D tlb access, write\000event=0x4f\000\00000\000Attributable Level 1 data or unified TLB access, write\000 */
{ 116626 }, /* l1i_cache\000\000cache\000Attributable Level 1 instruction cache access\000event=0x14\000\00000\000Attributable Level 1 instruction cache access\000 */
{ 117316 }, /* l1i_cache_lmiss\000\000cache\000Level 1 instruction cache long-latency read miss\000event=0x4006\000\00000\000Level 1 instruction cache long-latency read miss.  If the L1I_CACHE_RD event is implemented, the counter counts each access counted by L1I_CACHE_RD that incurs additional latency because it returns instructions from outside of the Level 1 instruction cache of this PE.  If the L1I_CACHE_RD event is not implemented, the counter counts each access counted by L1I_CACHE that incurs additional latency because it returns instructions from outside the Level 1 instruction cache of this PE.  The event indicates to software that the access missed in the Level 1 instruction cache and might have a significant performance impact due to the additional latency, compared to the latency of an access that hits in the Level 1 instruction cache\000 */
{ 118420 }, /* l1i_cache_refill\000\000cache\000Level 1 instruction cache refill\000event=0x1\000\00000\000Level 1 instruction cache refill\000 */
{ 119419 }, /* l1i_tlb\000\000cache\000Attributable Level 1 instruction TLB access\000event=0x26\000\00000\000Attributable Level 1 instruction TLB access\000 */
{ 120198 }, /* l1i_tlb_refill\000\000cache\000Attributable Level 1 instruction TLB refill\000event=0x2\000\00000\000Attributable Level 1 instruction TLB refill\000 */
{ 126244 }, /* l2d_cache\000\000cache\000Level 2 data cache access\000event=0x16\000\00000\000Level 2 data cache access\000 */
{ 126922 }, /* l2d_cache_allocate\000\000cache\000Attributable Level 2 data cache allocation without refill\000event=0x20\000\00000\000Attributable Level 2 data cache allocation without refill\000 */
{ 127466 }, /* l2d_cache_inval\000\000cache\000L2D cache invalidate\000event=0x58\000\00000\000Attributable Level 2 data cache invalidate\000 */
{ 128191 }, /* l2d_cache_lmiss_rd\000\000cache\000Level 2 data cache long-latency read miss\000event=0x4009\000\00000\000Level 2 data cache long-latency read miss.  The counter counts each memory read access counted by L2D_CACHE that incurs additional latency because it returns data from outside the Level 2 data or unified cache of this processing element.  The event indicates to software that the access missed in the Level 2 data or unified cache and might have a significant performance impact compared to the latency of an access that hits in the Level 2 data or unified cache\000 */
{ 129126 }, /* l2d_cache_rd\000\000cache\000L2D cache access, read\000event=0x50\000\00000\000Attributable Level 2 data cache access, read\000 */
{ 129862 }, /* l2d_cache_refill\000\000cache\000Level 2 data refill\000event=0x17\000\00000\000Level 2 data refill\000 */
{ 132288 }, /* l2d_cache_refill_rd\000\000cache\000L2D cache refill, read\000event=0x52\000\00000\000Attributable Level 2 data cache refill, read\000 */
{ 132829 }, /* l2d_cache_refill_wr\000\000cache\000L2D cache refill, write\000event=0x53\000\00000\000Attributable Level 2 data cache refill, write\000 */
{ 136414 }, /* l2d_cache_wb\000\000cache\000Attributable Level 2 data cache write-back\000event=0x18\000\00000\000Attributable Level 2 data cache write-back\000 */
{ 137387 }, /* l2d_cache_wb_clean\000\000cache\000L2D cache Write-Back, cleaning and coherency\000event=0x57\000\00000\000Level 2 data cache Write-Back, cleaning and coherency\000 */
{ 137968 }, /* l2d_cache_wb_victim\000\000cache\000L2D cache Write-Back, victim\000event=0x56\000\00000\000Attributable Level 2 data cache Write-Back, victim\000 */
{ 138385 }, /* l2d_cache_wr\000\000cache\000L2D cache access, write\000event=0x51\000\00000\000Attributable Level 2 data cache access, write\000 */
{ 143389 }, /* l2d_tlb\000\000cache\000Attributable Level 2 data or unified TLB access\000event=0x2f\000\00000\000Attributable Level 2 data or unified TLB access\000 */
{ 144070 }, /* l2d_tlb_rd\000\000cache\000L2D cache access, read\000event=0x5e\000\00000\000Attributable Level 2 data or unified TLB access, read\000 */
{ 144725 }, /* l2d_tlb_refill\000\000cache\000Attributable Level 2 data TLB refill\000event=0x2d\000\00000\000Attributable Level 2 data TLB refill\000 */
{ 145079 }, /* l2d_tlb_refill_rd\000\000cache\000L2D cache refill, read\000event=0x5c\000\00000\000Attributable Level 2 data or unified TLB refill, read\000 */
{ 145555 }, /* l2d_tlb_refill_wr\000\000cache\000L2D cache refill, write\000event=0x5d\000\00000\000Attributable Level 2 data or unified TLB refill, write\000 */
{ 146014 }, /* l2d_tlb_wr\000\000cache\000L2D cache access, write\000event=0x5f\000\00000\000Attributable Level 2 data or unified TLB access, write\000 */
{ 150641 }, /* l3d_cache\000\000cache\000Attributable Level 3 data cache access\000event=0x2b\000\00000\000Attributable Level 3 data cache access\000 */
{ 151459 }, /* l3d_cache_allocate\000\000cache\000Attributable Level 3 data cache allocation without refill\000event=0x29\000\00000\000Attributable Level 3 data cache allocation without refill\000 */
{ 151872 }, /* l3d_cache_lmiss_rd\000\000cache\000Level 3 data cache long-latency read miss\000event=0x400b\000\00000\000Level 3 data cache long-latency read miss.  The counter counts each memory read access counted by L3D_CACHE that incurs additional latency because it returns data from outside the Level 3 data or unified cache of this processing element.  The event indicates to software that the access missed in the Level 3 data or unified cache and might have a significant performance impact compared to the latency of an access that hits in the Level 3 data or unified cache\000 */
{ 152420 }, /* l3d_cache_rd\000\000cache\000Attributable Level 3 data or unified cache access, read\000event=0xa0\000\00000\000Attributable Level 3 data or unified cache access, read\000 */
{ 152665 }, /* l3d_cache_refill\000\000cache\000Attributable Level 3 data cache refill\000event=0x2a\000\00000\000Attributable Level 3 data cache refill\000 */
{ 160229 }, /* ll_cache_miss_rd\000\000cache\000Last level cache miss, read\000event=0x37\000\00000\000Last level cache miss, read\000 */
{ 160879 }, /* ll_cache_rd\000\000cache\000Attributable Last level cache memory read\000event=0x36\000\00000\000Attributable Last level cache memory read\000 */
{ 56959 }, /* exc_dabort\000\000exception\000Exception taken, Data Abort and SError\000event=0x84\000\00000\000Exception taken, Data Abort and SError\000 */
{ 57280 }, /* exc_fiq\000\000exception\000Exception taken, FIQ\000event=0x87\000\00000\000Exception taken, FIQ\000 */
{ 57633 }, /* exc_hvc\000\000exception\000Exception taken, Hypervisor Call\000event=0x8a\000\00000\000Exception taken, Hypervisor Call\000 */
{ 57939 }, /* exc_irq\000\000exception\000Exception taken, IRQ\000event=0x86\000\00000\000Exception taken, IRQ\000 */
{ 58355 }, /* exc_pabort\000\000exception\000Exception taken, Instruction Abort\000event=0x83\000\00000\000Exception taken, Instruction Abort\000 */
{ 59258 }, /* exc_smc\000\000exception\000Exception taken, Secure Monitor Call\000event=0x88\000\00000\000Exception taken, Secure Monitor Call\000 */
{ 59572 }, /* exc_svc\000\000exception\000Exception taken, Supervisor Call\000event=0x82\000\00000\000Exception taken, Supervisor Call\000 */
{ 59896 }, /* exc_taken\000\000exception\000Exception taken\000event=0x9\000\00000\000Exception taken\000 */
{ 60575 }, /* exc_trap_dabort\000\000exception\000Exception taken, Data Abort or SError not taken locally\000event=0x8c\000\00000\000Exception taken, Data Abort or SError not taken locally\000 */
{ 61080 }, /* exc_trap_fiq\000\000exception\000Exception taken, FIQ not taken locally\000event=0x8f\000\00000\000Exception taken, FIQ not taken locally\000 */
{ 62347 }, /* exc_trap_irq\000\000exception\000Exception taken, IRQ not taken locally\000event=0x8e\000\00000\000Exception taken, IRQ not taken locally\000 */
{ 62841 }, /* exc_trap_other\000\000exception\000Exception taken, Other traps not taken locally\000event=0x8d\000\00000\000Exception taken, Other traps not taken locally\000 */
{ 63383 }, /* exc_trap_pabort\000\000exception\000Exception taken, Instruction Abort not taken locally\000event=0x8b\000\00000\000Exception taken, Instruction Abort not taken locally\000 */
{ 64129 }, /* exc_undef\000\000exception\000Exception taken, Other synchronous\000event=0x81\000\00000\000Exception taken, Other synchronous\000 */
{ 165870 }, /* memory_error\000\000exception\000Local memory error\000event=0x1a\000\00000\000Local memory error\000 */
{ 20758 }, /* ase_inst_spec\000\000instruction\000ASE operations speculatively executed\000event=0x8005\000\00000\000ASE operations speculatively executed\000 */
{ 20878 }, /* ase_spec\000\000instruction\000Operation speculatively executed, Advanced SIMD instruction\000event=0x74\000\00000\000Operation speculatively executed, Advanced SIMD instruction\000 */
{ 21288 }, /* ase_sve_int16_spec\000\000instruction\000Advanced SIMD and SVE 16-bit integer operations speculatively executed\000event=0x80e7\000\00000\000Advanced SIMD and SVE 16-bit integer operations speculatively executed\000 */
{ 21479 }, /* ase_sve_int32_spec\000\000instruction\000Advanced SIMD and SVE 32-bit integer operations speculatively executed\000event=0x80eb\000\00000\000Advanced SIMD and SVE 32-bit integer operations speculatively executed\000 */
{ 21670 }, /* ase_sve_int64_spec\000\000instruction\000Advanced SIMD and SVE 64-bit integer operations speculatively executed\000event=0x80ef\000\00000\000Advanced SIMD and SVE 64-bit integer operations speculatively executed\000 */
{ 21861 }, /* ase_sve_int8_spec\000\000instruction\000Advanced SIMD and SVE 8-bit integer operations speculatively executed\000event=0x80e3\000\00000\000Advanced SIMD and SVE 8-bit integer operations speculatively executed\000 */
{ 30886 }, /* br_immed_spec\000\000instruction\000Branch speculatively executed, immediate branch\000event=0x78\000\00000\000Branch speculatively executed, immediate branch\000 */
{ 35172 }, /* br_indirect_spec\000\000instruction\000Branch speculatively executed, indirect branch\000event=0x7a\000\00000\000Branch speculatively executed, indirect branch\000 */
{ 36351 }, /* br_mis_pred_retired\000\000instruction\000Instruction architecturally executed, mispredicted branch\000event=0x22\000\00000\000Instruction architecturally executed, mispredicted branch\000 */
{ 37393 }, /* br_retired\000\000instruction\000Instruction architecturally executed, branch\000event=0x21\000\00000\000Instruction architecturally executed, branch\000 */
{ 40826 }, /* br_return_spec\000\000instruction\000Branch speculatively executed, procedure return\000event=0x79\000\00000\000Branch speculatively executed, procedure return\000 */
{ 45546 }, /* cid_write_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000event=0xb\000\00000\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000 */
{ 48419 }, /* crypto_spec\000\000instruction\000Operation speculatively executed, Cryptographic instruction\000event=0x77\000\00000\000Operation speculatively executed, Cryptographic instruction\000 */
{ 51117 }, /* dmb_spec\000\000instruction\000Barrier speculatively executed, DMB\000event=0x7e\000\00000\000Barrier speculatively executed, DMB\000 */
{ 52153 }, /* dp_spec\000\000instruction\000Operation speculatively executed, integer data processing\000event=0x73\000\00000\000Operation speculatively executed, integer data processing\000 */
{ 53209 }, /* dsb_spec\000\000instruction\000Barrier speculatively executed, DSB\000event=0x7d\000\00000\000Barrier speculatively executed, DSB\000 */
{ 58850 }, /* exc_return\000\000instruction\000Instruction architecturally executed, condition check pass, exception return\000event=0xa\000\00000\000Instruction architecturally executed, condition check pass, exception return\000 */
{ 67066 }, /* fp_dp_spec\000\000instruction\000Floating-point double-precision operations speculatively executed\000event=0x801c\000\00000\000Floating-point double-precision operations speculatively executed\000 */
{ 67239 }, /* fp_fixed_ops_spec\000\000instruction\000Non-scalable floating-point element Operations speculatively executed\000event=0x80c1\000\00000\000Non-scalable floating-point element Operations speculatively executed\000 */
{ 68165 }, /* fp_hp_spec\000\000instruction\000Floating-point half-precision operations speculatively executed\000event=0x8014\000\00000\000Floating-point half-precision operations speculatively executed\000 */
{ 68926 }, /* fp_scale_ops_spec\000\000instruction\000Scalable floating-point element Operations speculatively executed\000event=0x80c0\000\00000\000Scalable floating-point element Operations speculatively executed\000 */
{ 69704 }, /* fp_sp_spec\000\000instruction\000Floating-point single-precision operations speculatively executed\000event=0x8018\000\00000\000Floating-point single-precision operations speculatively executed\000 */
{ 90819 }, /* inst_retired\000\000instruction\000Instruction architecturally executed\000event=0x8\000\00000\000Instruction architecturally executed\000 */
{ 91235 }, /* inst_spec\000\000instruction\000Operation speculatively executed\000event=0x1b\000\00000\000Operation speculatively executed\000 */
{ 91769 }, /* isb_spec\000\000instruction\000Barrier speculatively executed, ISB\000event=0x7c\000\00000\000Barrier speculatively executed, ISB\000 */
{ 159047 }, /* ld_spec\000\000instruction\000Operation speculatively executed, load\000event=0x70\000\00000\000Operation speculatively executed, load\000 */
{ 159351 }, /* ldrex_spec\000\000instruction\000Exclusive operation speculatively executed, LDREX or LDX\000event=0x6c\000\00000\000Exclusive operation speculatively executed, LDREX or LDX\000 */
{ 173450 }, /* op_retired\000\000instruction\000Micro-operation architecturally executed\000event=0x3a\000\00000\000Micro-operation architecturally executed.  The counter counts each operation counted by OP_SPEC that would be executed in a simple sequential execution of the program\000 */
{ 173697 }, /* op_spec\000\000instruction\000Micro-operation speculatively executed\000event=0x3b\000\00000\000Micro-operation speculatively executed.  The counter counts the number of operations executed by the processing element, including those that are executed speculatively and would not be executed in a simple sequential execution of the program\000 */
{ 175115 }, /* pc_write_spec\000\000instruction\000Operation speculatively executed, software change of the PC\000event=0x76\000\00000\000Operation speculatively executed, software change of the PC\000 */
{ 177878 }, /* rc_ld_spec\000\000instruction\000Release consistency operation speculatively executed, Load-Acquire\000event=0x90\000\00000\000Release consistency operation speculatively executed, Load-Acquire\000 */
{ 178251 }, /* rc_st_spec\000\000instruction\000Release consistency operation speculatively executed, Store-Release\000event=0x91\000\00000\000Release consistency operation speculatively executed, Store-Release\000 */
{ 185522 }, /* st_spec\000\000instruction\000Operation speculatively executed, store\000event=0x71\000\00000\000Operation speculatively executed, store\000 */
{ 200454 }, /* strex_fail_spec\000\000instruction\000Exclusive operation speculatively executed, STREX or STX fail\000event=0x6e\000\00000\000Exclusive operation speculatively executed, STREX or STX fail\000 */
{ 201025 }, /* strex_pass_spec\000\000instruction\000Exclusive operation speculatively executed, STREX or STX pass\000event=0x6d\000\00000\000Exclusive operation speculatively executed, STREX or STX pass\000 */
{ 201592 }, /* strex_spec\000\000instruction\000Exclusive operation speculatively executed, STREX or STX\000event=0x6f\000\00000\000Exclusive operation speculatively executed, STREX or STX\000 */
{ 202321 }, /* sve_inst_spec\000\000instruction\000SVE operations speculatively executed\000event=0x8006\000\00000\000SVE operations speculatively executed\000 */
{ 202582 }, /* sve_ldff_fault_spec\000\000instruction\000SVE first-fault load operations speculatively executed which set FFR bit to 0\000event=0x80bd\000\00000\000SVE first-fault load operations speculatively executed which set FFR bit to 0\000 */
{ 202788 }, /* sve_ldff_spec\000\000instruction\000SVE First-fault load Operations speculatively executed\000event=0x80bc\000\00000\000SVE First-fault load Operations speculatively executed\000 */
{ 203847 }, /* sve_pred_empty_spec\000\000instruction\000SVE predicated operations with no active predicates speculatively executed\000event=0x8075\000\00000\000SVE predicated operations with no active predicates speculatively executed\000 */
{ 204047 }, /* sve_pred_full_spec\000\000instruction\000SVE predicated operations speculatively executed with all active predicates\000event=0x8076\000\00000\000SVE predicated operations speculatively executed with all active predicates\000 */
{ 204248 }, /* sve_pred_not_full_spec\000\000instruction\000SVE predicated operations with empty or partially active predicates\000event=0x8079\000\00000\000SVE predicated operations with empty or partially active predicates\000 */
{ 204437 }, /* sve_pred_partial_spec\000\000instruction\000SVE predicated operations speculatively executed with partially active predicates\000event=0x8077\000\00000\000SVE predicated operations speculatively executed with partially active predicates\000 */
{ 204653 }, /* sve_pred_spec\000\000instruction\000SVE predicated Operations speculatively executed\000event=0x8074\000\00000\000SVE predicated Operations speculatively executed\000 */
{ 205736 }, /* sw_incr\000\000instruction\000Instruction architecturally executed, Condition code check pass, software increment\000event=0\000\00000\000Instruction architecturally executed, Condition code check pass, software increment\000 */
{ 210294 }, /* ttbr_write_retired\000\000instruction\000Instruction architecturally executed, Condition code check pass, write to TTBR\000event=0x1c\000\00000\000Instruction architecturally executed, Condition code check pass, write to TTBR\000 */
{ 214354 }, /* vfp_spec\000\000instruction\000Operation speculatively executed, floating-point instruction\000event=0x75\000\00000\000Operation speculatively executed, floating-point instruction\000 */
{ 155492 }, /* ld_align_lat\000\000memory\000Load with additional latency from alignment\000event=0x4021\000\00000\000Load with additional latency from alignment\000 */
{ 159841 }, /* ldst_align_lat\000\000memory\000Access with additional latency from alignment\000event=0x4020\000\00000\000Access with additional latency from alignment\000 */
{ 163148 }, /* mem_access\000\000memory\000Data memory access\000event=0x13\000\00000\000Data memory access\000 */
{ 163403 }, /* mem_access_checked\000\000memory\000Checked data memory access\000event=0x4024\000\00000\000Checked data memory access\000 */
{ 163501 }, /* mem_access_checked_rd\000\000memory\000Checked data memory access, read\000event=0x4025\000\00000\000Checked data memory access, read\000 */
{ 163614 }, /* mem_access_checked_wr\000\000memory\000Checked data memory access, write\000event=0x4026\000\00000\000Checked data memory access, write\000 */
{ 164298 }, /* mem_access_rd\000\000memory\000Data memory access, read\000event=0x66\000\00000\000Data memory access, read\000 */
{ 164956 }, /* mem_access_wr\000\000memory\000Data memory access, write\000event=0x67\000\00000\000Data memory access, write\000 */
{ 179852 }, /* remote_access\000\000memory\000Access to another socket in a multi-socket system\000event=0x31\000\00000\000Access to another socket in a multi-socket system\000 */
{ 185116 }, /* st_align_lat\000\000memory\000Store with additional latency from alignment\000event=0x4022\000\00000\000Store with additional latency from alignment\000 */
{ 211505 }, /* unaligned_ld_spec\000\000memory\000Unaligned access, read\000event=0x68\000\00000\000Unaligned access, read\000 */
{ 212200 }, /* unaligned_ldst_spec\000\000memory\000Unaligned access\000event=0x6a\000\00000\000Unaligned access\000 */
{ 212637 }, /* unaligned_st_spec\000\000memory\000Unaligned access, write\000event=0x69\000\00000\000Unaligned access, write\000 */
{ 185831 }, /* stall\000\000pipeline\000No operation sent for execution\000event=0x3c\000\00000\000No operation sent for execution.  The counter counts every attributable cycle on which no attributable instruction or operation was sent for execution on this processing element\000 */
{ 186072 }, /* stall_backend\000\000pipeline\000No operation issued due to the backend\000event=0x24\000\00000\000No operation issued due to the backend\000 */
{ 192744 }, /* stall_backend_mem\000\000pipeline\000Memory stall cycles\000event=0x4005\000\00000\000Memory stall cycles.  The counter counts each cycle counted by STALL_BACKEND where there is a cache miss in the last level of cache within the processing element clock domain\000 */
{ 195875 }, /* stall_frontend\000\000pipeline\000No operation issued because of the frontend\000event=0x23\000\00000\000No operation issued because of the frontend\000 */
{ 199011 }, /* stall_slot\000\000pipeline\000No operation sent for execution on a slot\000event=0x3f\000\00000\000No operation sent for execution on a slot.  The counter counts on each attributable cycle the number of instruction or operation slots that were not occupied by an instruction or operation attributable to the processing element\000 */
{ 199317 }, /* stall_slot_backend\000\000pipeline\000No operation sent for execution on a slot due to the backend\000event=0x3d\000\00000\000No operation sent for execution on a slot due to the backend.  Counts each slot counted by STALL_SLOT where no attributable instruction or operation was sent for execution because the backend is unable to accept it\000 */
{ 199953 }, /* stall_slot_frontend\000\000pipeline\000No operation sent for execution on a slot due to the frontend\000event=0x3e\000\00000\000No operation sent for execution on a slot due to the frontend.  Counts each slot counted by STALL_SLOT where no attributable instruction or operation was sent for execution because there was no attributable instruction or operation available to issue from the processing element from the frontend for the slot\000 */
{ 183111 }, /* sample_collision\000\000spe\000Sample collided with previous sample\000event=0x4003\000\00000\000Sample collided with previous sample\000 */
{ 183327 }, /* sample_feed\000\000spe\000Sample Taken\000event=0x4001\000\00000\000Sample Taken\000 */
{ 183545 }, /* sample_filtrate\000\000spe\000Sample Taken and not removed by filtering\000event=0x4002\000\00000\000Sample Taken and not removed by filtering\000 */
{ 183858 }, /* sample_pop\000\000spe\000Sample Population\000event=0x4000\000\00000\000Sample Population\000 */
{ 48775 }, /* cti_trigout4\000\000trace\000Cross-trigger Interface output trigger 4\000event=0x4018\000\00000\000Cross-trigger Interface output trigger 4\000 */
{ 48894 }, /* cti_trigout5\000\000trace\000Cross-trigger Interface output trigger 5 \000event=0x4019\000\00000\000Cross-trigger Interface output trigger 5 \000 */
{ 49015 }, /* cti_trigout6\000\000trace\000Cross-trigger Interface output trigger 6\000event=0x401a\000\00000\000Cross-trigger Interface output trigger 6\000 */
{ 49134 }, /* cti_trigout7\000\000trace\000Cross-trigger Interface output trigger 7\000event=0x401b\000\00000\000Cross-trigger Interface output trigger 7\000 */
{ 209779 }, /* trb_wrap\000\000trace\000Trace buffer current write pointer wrapped\000event=0x400c\000\00000\000Trace buffer current write pointer wrapped\000 */
{ 209898 }, /* trcextout0\000\000trace\000PE Trace Unit external output 0\000event=0x4010\000\00000\000PE Trace Unit external output 0\000 */
{ 209997 }, /* trcextout1\000\000trace\000PE Trace Unit external output 1\000event=0x4011\000\00000\000PE Trace Unit external output 1\000 */
{ 210096 }, /* trcextout2\000\000trace\000PE Trace Unit external output 2\000event=0x4012\000\00000\000PE Trace Unit external output 2\000 */
{ 210195 }, /* trcextout3\000\000trace\000PE Trace Unit external output 3\000event=0x4013\000\00000\000PE Trace Unit external output 3\000 */
};

static const struct compact_pmu_event pmu_metrics__arm_neoverse_n2_v2[] = {
{ 828 }, /* \000advanced_simd_spec_rate\000InstructionMix\000ASE_SPEC / INST_SPEC\000\000The rate of advanced SIMD instructions speculatively executed to overall instructions speclatively executed\000\000100%\000\000\000\00000 */
{ 1172 }, /* \000backend_bound\000Default;TopdownL1\000stall_slot_backend / (#slots * cpu_cycles)\000\000Backend Bound L1 topdown metric\000\000100%\000\000\000TopdownL1\00000 */
{ 1689 }, /* \000bad_speculation\000Default;TopdownL1\000(1 - op_retired / op_spec) * (1 - (stall_slot if #slots - 5 else stall_slot - cpu_cycles) / (#slots * cpu_cycles))\000\000Bad speculation L1 topdown metric\000\000100%\000\000\000TopdownL1\00000 */
{ 2019 }, /* \000branch_immed_spec_rate\000InstructionMix\000BR_IMMED_SPEC / INST_SPEC\000\000The rate of branch immediate instructions speculatively executed to overall instructions speclatively executed\000\000100%\000\000\000\00000 */
{ 2207 }, /* \000branch_indirect_spec_rate\000InstructionMix\000BR_INDIRECT_SPEC / INST_SPEC\000\000The rate of indirect branch instructions speculatively executed to overall instructions speclatively executed\000\000100%\000\000\000\00000 */
{ 2890 }, /* \000branch_miss_pred_rate\000Branch\000BR_MIS_PRED_RETIRED / BR_RETIRED\000\000The rate of branches mis-predited to the overall branches\000\000100%\000\000\000\00000 */
{ 3023 }, /* \000branch_mpki\000Branch\000BR_MIS_PRED_RETIRED / INST_RETIRED * 1e3\000\000The rate of branches mis-predicted per kilo instructions\000\0001MPKI\000\000\000\00000 */
{ 3547 }, /* \000branch_pki\000Branch\000BR_RETIRED / INST_RETIRED * 1e3\000\000The rate of branches retired per kilo instructions\000\0001PKI\000\000\000\00000 */
{ 3661 }, /* \000branch_return_spec_rate\000InstructionMix\000BR_RETURN_SPEC / INST_SPEC\000\000The rate of procedure return instructions speculatively executed to overall instructions speclatively executed\000\000100%\000\000\000\00000 */
{ 4094 }, /* \000cpu_utilization\000PEutilization\000OP_RETIRED / OP_SPEC * (1 - (STALL_SLOT if #slots - 5 else STALL_SLOT - CPU_CYCLES) / (#slots * CPU_CYCLES))\000\000The truly effective ratio of micro-operations executed by the CPU, which means that misprediction and stall are not included\000\000100%\000\000\000\00000 */
{ 4556 }, /* \000crypto_spec_rate\000InstructionMix\000CRYPTO_SPEC / INST_SPEC\000\000The rate of crypto instructions speculatively executed to overall instructions speclatively executed\000\000100%\000\000\000\00000 */
{ 4726 }, /* \000data_process_spec_rate\000InstructionMix\000DP_SPEC / INST_SPEC\000\000The rate of integer data-processing instructions speculatively executed to overall instructions speclatively executed\000\000100%\000\000\000\00000 */
{ 5264 }, /* \000dtlb_mpki\000TLB\000DTLB_WALK / INST_RETIRED * 1e3\000\000The rate of TLB Walks per kilo instructions for data accesses\000\0001MPKI\000\000\000\00000 */
{ 5385 }, /* \000dtlb_walk_rate\000TLB\000DTLB_WALK / L1D_TLB\000\000The rate of DTLB Walks to the overall L1D TLB lookups\000\000100%\000\000\000\00000 */
{ 6195 }, /* \000float_point_spec_rate\000InstructionMix\000VFP_SPEC / INST_SPEC\000\000The rate of floating point instructions speculatively executed to overall instructions speclatively executed\000\000100%\000\000\000\00000 */
{ 6523 }, /* \000frontend_bound\000Default;TopdownL1\000(stall_slot_frontend if #slots - 5 else stall_slot_frontend - cpu_cycles) / (#slots * cpu_cycles)\000\000Frontend bound L1 topdown metric\000\000100%\000\000\000TopdownL1\00000 */
{ 7958 }, /* \000ipc\000PEutilization\000instructions / CPU_CYCLES\000\000The average number of instructions executed for each cycle\000\000\000\000\000\00000 */
{ 8070 }, /* \000ipc_rate\000PEutilization\000ipc / 5\000\000IPC percentage of peak. The peak of IPC is 5\000\000100%\000\000\000\00000 */
{ 8330 }, /* \000itlb_mpki\000TLB\000ITLB_WALK / INST_RETIRED * 1e3\000\000The rate of TLB Walks per kilo instructions for instruction accesses\000\0001MPKI\000\000\000\00000 */
{ 8458 }, /* \000itlb_walk_rate\000TLB\000ITLB_WALK / L1I_TLB\000\000The rate of ITLB Walks to the overall L1I TLB lookups\000\000100%\000\000\000\00000 */
{ 8970 }, /* \000l1d_cache_miss_rate\000Cache\000L1D_CACHE_REFILL / L1D_CACHE\000\000The rate of L1 D-Cache misses to the overall L1 D-Cache\000\000100%\000\000\000\00000 */
{ 9398 }, /* \000l1d_cache_mpki\000Cache\000L1D_CACHE_REFILL / INST_RETIRED * 1e3\000\000The rate of L1 D-Cache misses per kilo instructions\000\0001MPKI\000\000\000\00000 */
{ 9724 }, /* \000l1d_tlb_miss_rate\000TLB\000L1D_TLB_REFILL / L1D_TLB\000\000The rate of L1D TLB refill to the overall L1D TLB lookups\000\000100%\000\000\000\00000 */
{ 10324 }, /* \000l1i_cache_miss_rate\000Cache\000L1I_CACHE_REFILL / L1I_CACHE\000\000The rate of L1 I-Cache misses to the overall L1 I-Cache\000\000100%\000\000\000\00000 */
{ 10773 }, /* \000l1i_cache_mpki\000Cache\000L1I_CACHE_REFILL / INST_RETIRED * 1e3\000\000The rate of L1 I-Cache misses per kilo instructions\000\0001MPKI\000\000\000\00000 */
{ 11106 }, /* \000l1i_tlb_miss_rate\000TLB\000L1I_TLB_REFILL / L1I_TLB\000\000The rate of L1I TLB refill to the overall L1I TLB lookups\000\000100%\000\000\000\00000 */
{ 12648 }, /* \000l2_tlb_miss_rate\000TLB\000L2D_TLB_REFILL / L2D_TLB\000\000The rate of L2D TLB refill to the overall L2D TLB lookups\000\000100%\000\000\000\00000 */
{ 13280 }, /* \000l2d_cache_miss_rate\000Cache\000L2D_CACHE_REFILL / L2D_CACHE\000\000The rate of L2 D-Cache misses to the overall L2 D-Cache\000\000100%\000\000\000\00000 */
{ 13404 }, /* \000l2d_cache_mpki\000Cache\000L2D_CACHE_REFILL / INST_RETIRED * 1e3\000\000The rate of L2 D-Cache misses per kilo instructions\000\0001MPKI\000\000\000\00000 */
{ 13529 }, /* \000l3d_cache_miss_rate\000Cache\000L3D_CACHE_REFILL / L3D_CACHE\000\000The rate of L3 D-Cache misses to the overall L3 D-Cache\000\000100%\000\000\000\00000 */
{ 13653 }, /* \000l3d_cache_mpki\000Cache\000L3D_CACHE_REFILL / INST_RETIRED * 1e3\000\000The rate of L3 D-Cache misses per kilo instructions\000\0001MPKI\000\000\000\00000 */
{ 13778 }, /* \000ll_cache_read_hit_rate\000Cache\000(LL_CACHE_RD - LL_CACHE_MISS_RD) / LL_CACHE_RD\000\000The rate of LL Cache read hit to the overall LL Cache read\000\000100%\000\000\000\00000 */
{ 14388 }, /* \000ll_cache_read_miss_rate\000Cache\000LL_CACHE_MISS_RD / LL_CACHE_RD\000\000The rate of LL Cache read misses to the overall LL Cache read\000\000100%\000\000\000\00000 */
{ 14972 }, /* \000ll_cache_read_mpki\000Cache\000LL_CACHE_MISS_RD / INST_RETIRED * 1e3\000\000The rate of LL Cache read misses per kilo instructions\000\0001MPKI\000\000\000\00000 */
{ 15488 }, /* \000load_spec_rate\000InstructionMix\000LD_SPEC / INST_SPEC\000\000The rate of load instructions speculatively executed to overall instructions speclatively executed\000\000100%\000\000\000\00000 */
{ 17317 }, /* \000retired_ipc\000PEutilization\000INST_RETIRED / CPU_CYCLES\000\000Architecturally executed Instructions Per Cycle (IPC)\000\000\000\000\000\00000 */
{ 17432 }, /* \000retired_rate\000PEutilization\000OP_RETIRED / OP_SPEC\000\000Of all the micro-operations issued, what percentage are retired(committed)\000\000100%\000\000\000\00000 */
{ 17698 }, /* \000retiring\000Default;TopdownL1\000op_retired / op_spec * (1 - (stall_slot if #slots - 5 else stall_slot - cpu_cycles) / (#slots * cpu_cycles))\000\000Retiring L1 topdown metric\000\000100%\000\000\000TopdownL1\00000 */
{ 18695 }, /* \000spec_ipc\000PEutilization\000INST_SPEC / CPU_CYCLES\000\000Speculatively executed Instructions Per Cycle (IPC)\000\000\000\000\000\00000 */
{ 19113 }, /* \000store_spec_rate\000InstructionMix\000ST_SPEC / INST_SPEC\000\000The rate of store instructions speculatively executed to overall instructions speclatively executed\000\000100%\000\000\000\00000 */
{ 19406 }, /* \000wasted_rate\000PEutilization\0001 - OP_RETIRED / OP_SPEC\000\000Of all the micro-operations issued, what percentage are not retired(committed)\000\000100%\000\000\000\00000 */
};

static const struct compact_pmu_event pmu_events__arm_neoverse_v1[] = {
{ 35743 }, /* br_mis_pred\000\000branch\000Mispredicted or not predicted branch speculatively executed\000event=0x10\000\00000\000Mispredicted or not predicted branch speculatively executed\000 */
{ 36925 }, /* br_pred\000\000branch\000Predictable branch speculatively executed\000event=0x12\000\00000\000Predictable branch speculatively executed\000 */
{ 41358 }, /* bus_access\000\000bus\000Attributable Bus access\000event=0x19\000\00000\000Attributable Bus access\000 */
{ 42271 }, /* bus_access_rd\000\000bus\000Bus access read\000event=0x60\000\00000\000Bus access read\000 */
{ 42682 }, /* bus_access_wr\000\000bus\000Bus access write\000event=0x61\000\00000\000Bus access write\000 */
{ 42979 }, /* bus_cycles\000\000bus\000Bus cycle\000event=0x1d\000\00000\000Bus cycle\000 */
{ 46424 }, /* cnt_cycles\000\000bus\000Constant frequency cycles\000event=0x4004\000\00000\000Constant frequency cycles.  The counter increments at a constant frequency equal to the rate of increment of the system counter, CNTPCT_EL0\000 */
{ 47992 }, /* cpu_cycles\000\000bus\000Cycle\000event=0x11\000\00000\000Cycle\000 */
{ 53526 }, /* dtlb_walk\000\000cache\000Access to data TLB causes a translation table walk\000event=0x34\000\00000\000Access to data TLB causes a translation table walk\000 */
{ 92044 }, /* itlb_walk\000\000cache\000Access to instruction TLB that causes a translation table walk\000event=0x35\000\00000\000Access to instruction TLB that causes a translation table walk\000 */
{ 100289 }, /* l1d_cache\000\000cache\000Level 1 data cache access\000event=0x4\000\00000\000Level 1 data cache access\000 */
{ 101403 }, /* l1d_cache_inval\000\000cache\000L1D cache invalidate\000event=0x48\000\00000\000Attributable Level 1 data cache invalidate\000 */
{ 102276 }, /* l1d_cache_lmiss_rd\000\000cache\000Level 1 data cache long-latency read miss\000event=0x39\000\00000\000Level 1 data cache long-latency read miss.  The counter counts each memory read access counted by L1D_CACHE that incurs additional latency because it returns data from outside the Level 1 data or unified cache of this processing element\000 */
{ 102693 }, /* l1d_cache_rd\000\000cache\000L1D cache access, read\000event=0x40\000\00000\000Attributable Level 1 data cache access, read\000 */
{ 103129 }, /* l1d_cache_refill\000\000cache\000Level 1 data cache refill\000event=0x3\000\00000\000Level 1 data cache refill\000 */
{ 104145 }, /* l1d_cache_refill_inner\000\000cache\000L1D cache refill, inner\000event=0x44\000\00000\000Attributable Level 1 data cache refill, inner\000 */
{ 104573 }, /* l1d_cache_refill_outer\000\000cache\000L1D cache refill, outer\000event=0x45\000\00000\000Attributable Level 1 data cache refill, outer\000 */
{ 105894 }, /* l1d_cache_refill_rd\000\000cache\000L1D cache refill, read\000event=0x42\000\00000\000Attributable Level 1 data cache refill, read\000 */
{ 106389 }, /* l1d_cache_refill_wr\000\000cache\000L1D cache refill, write\000event=0x43\000\00000\000Attributable Level 1 data cache refill, write\000 */
{ 106891 }, /* l1d_cache_wb\000\000cache\000Attributable Level 1 data cache write-back\000event=0x15\000\00000\000Attributable Level 1 data cache write-back\000 */
{ 107957 }, /* l1d_cache_wb_clean\000\000cache\000L1D cache Write-Back, cleaning and coherency\000event=0x47\000\00000\000Level 1 data cache Write-Back, cleaning and coherency\000 */
{ 108495 }, /* l1d_cache_wb_victim\000\000cache\000L1D cache Write-Back, victim\000event=0x46\000\00000\000Attributable Level 1 data cache Write-Back, victim\000 */
{ 108997 }, /* l1d_cache_wr\000\000cache\000L1D cache access, write\000event=0x41\000\00000\000Attributable Level 1 data cache access, write\000 */
{ 109867 }, /* l1d_tlb\000\000cache\000Attributable Level 1 data or unified TLB access\000event=0x25\000\00000\000Attributable Level 1 data or unified TLB access\000 */
{ 110748 }, /* l1d_tlb_rd\000\000cache\000L1D tlb access, read\000event=0x4e\000\00000\000Attributable Level 1 data or unified TLB access, read\000 */
{ 111207 }, /* l1d_tlb_refill\000\000cache\000Attributable Level 1 data TLB refill\000event=0x5\000\00000\000Attributable Level 1 data TLB refill\000 */
{ 112199 }, /* l1d_tlb_refill_rd\000\000cache\000L1D tlb refill, read\000event=0x4c\000\00000\000Attributable Level 1 data TLB refill, read\000 */
{ 113093 }, /* l1d_tlb_refill_wr\000\000cache\000L1D tlb refill, write\000event=0x4d\000\00000\000Attributable Level 1 data TLB refill, write\000 */
{ 113993 }, /* l1d_tlb_wr\000\000cache\000L1D tlb access, write\000event=0x4f\000\00000\000Attributable Level 1 data or unified TLB access, write\000 */
{ 116626 }, /* l1i_cache\000\000cache\000Attributable Level 1 instruction cache access\000event=0x14\000\00000\000Attributable Level 1 instruction cache access\000 */
{ 117316 }, /* l1i_cache_lmiss\000\000cache\000Level 1 instruction cache long-latency read miss\000event=0x4006\000\00000\000Level 1 instruction cache long-latency read miss.  If the L1I_CACHE_RD event is implemented, the counter counts each access counted by L1I_CACHE_RD that incurs additional latency because it returns instructions from outside of the Level 1 instruction cache of this PE.  If the L1I_CACHE_RD event is not implemented, the counter counts each access counted by L1I_CACHE that incurs additional latency because it returns instructions from outside the Level 1 instruction cache of this PE.  The event indicates to software that the access missed in the Level 1 instruction cache and might have a significant performance impact due to the additional latency, compared to the latency of an access that hits in the Level 1 instruction cache\000 */
{ 118420 }, /* l1i_cache_refill\000\000cache\000Level 1 instruction cache refill\000event=0x1\000\00000\000Level 1 instruction cache refill\000 */
{ 119419 }, /* l1i_tlb\000\000cache\000Attributable Level 1 instruction TLB access\000event=0x26\000\00000\000Attributable Level 1 instruction TLB access\000 */
{ 120198 }, /* l1i_tlb_refill\000\000cache\000Attributable Level 1 instruction TLB refill\000event=0x2\000\00000\000Attributable Level 1 instruction TLB refill\000 */
{ 126244 }, /* l2d_cache\000\000cache\000Level 2 data cache access\000event=0x16\000\00000\000Level 2 data cache access\000 */
{ 126922 }, /* l2d_cache_allocate\000\000cache\000Attributable Level 2 data cache allocation without refill\000event=0x20\000\00000\000Attributable Level 2 data cache allocation without refill\000 */
{ 127466 }, /* l2d_cache_inval\000\000cache\000L2D cache invalidate\000event=0x58\000\00000\000Attributable Level 2 data cache invalidate\000 */
{ 128191 }, /* l2d_cache_lmiss_rd\000\000cache\000Level 2 data cache long-latency read miss\000event=0x4009\000\00000\000Level 2 data cache long-latency read miss.  The counter counts each memory read access counted by L2D_CACHE that incurs additional latency because it returns data from outside the Level 2 data or unified cache of this processing element.  The event indicates to software that the access missed in the Level 2 data or unified cache and might have a significant performance impact compared to the latency of an access that hits in the Level 2 data or unified cache\000 */
{ 129126 }, /* l2d_cache_rd\000\000cache\000L2D cache access, read\000event=0x50\000\00000\000Attributable Level 2 data cache access, read\000 */
{ 129862 }, /* l2d_cache_refill\000\000cache\000Level 2 data refill\000event=0x17\000\00000\000Level 2 data refill\000 */
{ 132288 }, /* l2d_cache_refill_rd\000\000cache\000L2D cache refill, read\000event=0x52\000\00000\000Attributable Level 2 data cache refill, read\000 */
{ 132829 }, /* l2d_cache_refill_wr\000\000cache\000L2D cache refill, write\000event=0x53\000\00000\000Attributable Level 2 data cache refill, write\000 */
{ 136414 }, /* l2d_cache_wb\000\000cache\000Attributable Level 2 data cache write-back\000event=0x18\000\00000\000Attributable Level 2 data cache write-back\000 */
{ 137387 }, /* l2d_cache_wb_clean\000\000cache\000L2D cache Write-Back, cleaning and coherency\000event=0x57\000\00000\000Level 2 data cache Write-Back, cleaning and coherency\000 */
{ 137968 }, /* l2d_cache_wb_victim\000\000cache\000L2D cache Write-Back, victim\000event=0x56\000\00000\000Attributable Level 2 data cache Write-Back, victim\000 */
{ 138385 }, /* l2d_cache_wr\000\000cache\000L2D cache access, write\000event=0x51\000\00000\000Attributable Level 2 data cache access, write\000 */
{ 143389 }, /* l2d_tlb\000\000cache\000Attributable Level 2 data or unified TLB access\000event=0x2f\000\00000\000Attributable Level 2 data or unified TLB access\000 */
{ 144070 }, /* l2d_tlb_rd\000\000cache\000L2D cache access, read\000event=0x5e\000\00000\000Attributable Level 2 data or unified TLB access, read\000 */
{ 144725 }, /* l2d_tlb_refill\000\000cache\000Attributable Level 2 data TLB refill\000event=0x2d\000\00000\000Attributable Level 2 data TLB refill\000 */
{ 145079 }, /* l2d_tlb_refill_rd\000\000cache\000L2D cache refill, read\000event=0x5c\000\00000\000Attributable Level 2 data or unified TLB refill, read\000 */
{ 145555 }, /* l2d_tlb_refill_wr\000\000cache\000L2D cache refill, write\000event=0x5d\000\00000\000Attributable Level 2 data or unified TLB refill, write\000 */
{ 146014 }, /* l2d_tlb_wr\000\000cache\000L2D cache access, write\000event=0x5f\000\00000\000Attributable Level 2 data or unified TLB access, write\000 */
{ 150641 }, /* l3d_cache\000\000cache\000Attributable Level 3 data cache access\000event=0x2b\000\00000\000Attributable Level 3 data cache access\000 */
{ 151459 }, /* l3d_cache_allocate\000\000cache\000Attributable Level 3 data cache allocation without refill\000event=0x29\000\00000\000Attributable Level 3 data cache allocation without refill\000 */
{ 151872 }, /* l3d_cache_lmiss_rd\000\000cache\000Level 3 data cache long-latency read miss\000event=0x400b\000\00000\000Level 3 data cache long-latency read miss.  The counter counts each memory read access counted by L3D_CACHE that incurs additional latency because it returns data from outside the Level 3 data or unified cache of this processing element.  The event indicates to software that the access missed in the Level 3 data or unified cache and might have a significant performance impact compared to the latency of an access that hits in the Level 3 data or unified cache\000 */
{ 152420 }, /* l3d_cache_rd\000\000cache\000Attributable Level 3 data or unified cache access, read\000event=0xa0\000\00000\000Attributable Level 3 data or unified cache access, read\000 */
{ 152665 }, /* l3d_cache_refill\000\000cache\000Attributable Level 3 data cache refill\000event=0x2a\000\00000\000Attributable Level 3 data cache refill\000 */
{ 160229 }, /* ll_cache_miss_rd\000\000cache\000Last level cache miss, read\000event=0x37\000\00000\000Last level cache miss, read\000 */
{ 160879 }, /* ll_cache_rd\000\000cache\000Attributable Last level cache memory read\000event=0x36\000\00000\000Attributable Last level cache memory read\000 */
{ 56959 }, /* exc_dabort\000\000exception\000Exception taken, Data Abort and SError\000event=0x84\000\00000\000Exception taken, Data Abort and SError\000 */
{ 57280 }, /* exc_fiq\000\000exception\000Exception taken, FIQ\000event=0x87\000\00000\000Exception taken, FIQ\000 */
{ 57633 }, /* exc_hvc\000\000exception\000Exception taken, Hypervisor Call\000event=0x8a\000\00000\000Exception taken, Hypervisor Call\000 */
{ 57939 }, /* exc_irq\000\000exception\000Exception taken, IRQ\000event=0x86\000\00000\000Exception taken, IRQ\000 */
{ 58355 }, /* exc_pabort\000\000exception\000Exception taken, Instruction Abort\000event=0x83\000\00000\000Exception taken, Instruction Abort\000 */
{ 59258 }, /* exc_smc\000\000exception\000Exception taken, Secure Monitor Call\000event=0x88\000\00000\000Exception taken, Secure Monitor Call\000 */
{ 59572 }, /* exc_svc\000\000exception\000Exception taken, Supervisor Call\000event=0x82\000\00000\000Exception taken, Supervisor Call\000 */
{ 59896 }, /* exc_taken\000\000exception\000Exception taken\000event=0x9\000\00000\000Exception taken\000 */
{ 60575 }, /* exc_trap_dabort\000\000exception\000Exception taken, Data Abort or SError not taken locally\000event=0x8c\000\00000\000Exception taken, Data Abort or SError not taken locally\000 */
{ 61080 }, /* exc_trap_fiq\000\000exception\000Exception taken, FIQ not taken locally\000event=0x8f\000\00000\000Exception taken, FIQ not taken locally\000 */
{ 62347 }, /* exc_trap_irq\000\000exception\000Exception taken, IRQ not taken locally\000event=0x8e\000\00000\000Exception taken, IRQ not taken locally\000 */
{ 62841 }, /* exc_trap_other\000\000exception\000Exception taken, Other traps not taken locally\000event=0x8d\000\00000\000Exception taken, Other traps not taken locally\000 */
{ 63383 }, /* exc_trap_pabort\000\000exception\000Exception taken, Instruction Abort not taken locally\000event=0x8b\000\00000\000Exception taken, Instruction Abort not taken locally\000 */
{ 64129 }, /* exc_undef\000\000exception\000Exception taken, Other synchronous\000event=0x81\000\00000\000Exception taken, Other synchronous\000 */
{ 165870 }, /* memory_error\000\000exception\000Local memory error\000event=0x1a\000\00000\000Local memory error\000 */
{ 20758 }, /* ase_inst_spec\000\000instruction\000ASE operations speculatively executed\000event=0x8005\000\00000\000ASE operations speculatively executed\000 */
{ 20878 }, /* ase_spec\000\000instruction\000Operation speculatively executed, Advanced SIMD instruction\000event=0x74\000\00000\000Operation speculatively executed, Advanced SIMD instruction\000 */
{ 30886 }, /* br_immed_spec\000\000instruction\000Branch speculatively executed, immediate branch\000event=0x78\000\00000\000Branch speculatively executed, immediate branch\000 */
{ 35172 }, /* br_indirect_spec\000\000instruction\000Branch speculatively executed, indirect branch\000event=0x7a\000\00000\000Branch speculatively executed, indirect branch\000 */
{ 36351 }, /* br_mis_pred_retired\000\000instruction\000Instruction architecturally executed, mispredicted branch\000event=0x22\000\00000\000Instruction architecturally executed, mispredicted branch\000 */
{ 37393 }, /* br_retired\000\000instruction\000Instruction architecturally executed, branch\000event=0x21\000\00000\000Instruction architecturally executed, branch\000 */
{ 40826 }, /* br_return_spec\000\000instruction\000Branch speculatively executed, procedure return\000event=0x79\000\00000\000Branch speculatively executed, procedure return\000 */
{ 45546 }, /* cid_write_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000event=0xb\000\00000\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000 */
{ 48419 }, /* crypto_spec\000\000instruction\000Operation speculatively executed, Cryptographic instruction\000event=0x77\000\00000\000Operation speculatively executed, Cryptographic instruction\000 */
{ 51117 }, /* dmb_spec\000\000instruction\000Barrier speculatively executed, DMB\000event=0x7e\000\00000\000Barrier speculatively executed, DMB\000 */
{ 52153 }, /* dp_spec\000\000instruction\000Operation speculatively executed, integer data processing\000event=0x73\000\00000\000Operation speculatively executed, integer data processing\000 */
{ 53209 }, /* dsb_spec\000\000instruction\000Barrier speculatively executed, DSB\000event=0x7d\000\00000\000Barrier speculatively executed, DSB\000 */
{ 58850 }, /* exc_return\000\000instruction\000Instruction architecturally executed, condition check pass, exception return\000event=0xa\000\00000\000Instruction architecturally executed, condition check pass, exception return\000 */
{ 67239 }, /* fp_fixed_ops_spec\000\000instruction\000Non-scalable floating-point element Operations speculatively executed\000event=0x80c1\000\00000\000Non-scalable floating-point element Operations speculatively executed\000 */
{ 68926 }, /* fp_scale_ops_spec\000\000instruction\000Scalable floating-point element Operations speculatively executed\000event=0x80c0\000\00000\000Scalable floating-point element Operations speculatively executed\000 */
{ 90819 }, /* inst_retired\000\000instruction\000Instruction architecturally executed\000event=0x8\000\00000\000Instruction architecturally executed\000 */
{ 91235 }, /* inst_spec\000\000instruction\000Operation speculatively executed\000event=0x1b\000\00000\000Operation speculatively executed\000 */
{ 91769 }, /* isb_spec\000\000instruction\000Barrier speculatively executed, ISB\000event=0x7c\000\00000\000Barrier speculatively executed, ISB\000 */
{ 159047 }, /* ld_spec\000\000instruction\000Operation speculatively executed, load\000event=0x70\000\00000\000Operation speculatively executed, load\000 */
{ 159351 }, /* ldrex_spec\000\000instruction\000Exclusive operation speculatively executed, LDREX or LDX\000event=0x6c\000\00000\000Exclusive operation speculatively executed, LDREX or LDX\000 */
{ 173450 }, /* op_retired\000\000instruction\000Micro-operation architecturally executed\000event=0x3a\000\00000\000Micro-operation architecturally executed.  The counter counts each operation counted by OP_SPEC that would be executed in a simple sequential execution of the program\000 */
{ 173697 }, /* op_spec\000\000instruction\000Micro-operation speculatively executed\000event=0x3b\000\00000\000Micro-operation speculatively executed.  The counter counts the number of operations executed by the processing element, including those that are executed speculatively and would not be executed in a simple sequential execution of the program\000 */
{ 175115 }, /* pc_write_spec\000\000instruction\000Operation speculatively executed, software change of the PC\000event=0x76\000\00000\000Operation speculatively executed, software change of the PC\000 */
{ 177878 }, /* rc_ld_spec\000\000instruction\000Release consistency operation speculatively executed, Load-Acquire\000event=0x90\000\00000\000Release consistency operation speculatively executed, Load-Acquire\000 */
{ 178251 }, /* rc_st_spec\000\000instruction\000Release consistency operation speculatively executed, Store-Release\000event=0x91\000\00000\000Release consistency operation speculatively executed, Store-Release\000 */
{ 185522 }, /* st_spec\000\000instruction\000Operation speculatively executed, store\000event=0x71\000\00000\000Operation speculatively executed, store\000 */
{ 200454 }, /* strex_fail_spec\000\000instruction\000Exclusive operation speculatively executed, STREX or STX fail\000event=0x6e\000\00000\000Exclusive operation speculatively executed, STREX or STX fail\000 */
{ 201025 }, /* strex_pass_spec\000\000instruction\000Exclusive operation speculatively executed, STREX or STX pass\000event=0x6d\000\00000\000Exclusive operation speculatively executed, STREX or STX pass\000 */
{ 201592 }, /* strex_spec\000\000instruction\000Exclusive operation speculatively executed, STREX or STX\000event=0x6f\000\00000\000Exclusive operation speculatively executed, STREX or STX\000 */
{ 202321 }, /* sve_inst_spec\000\000instruction\000SVE operations speculatively executed\000event=0x8006\000\00000\000SVE operations speculatively executed\000 */
{ 202582 }, /* sve_ldff_fault_spec\000\000instruction\000SVE first-fault load operations speculatively executed which set FFR bit to 0\000event=0x80bd\000\00000\000SVE first-fault load operations speculatively executed which set FFR bit to 0\000 */
{ 202788 }, /* sve_ldff_spec\000\000instruction\000SVE First-fault load Operations speculatively executed\000event=0x80bc\000\00000\000SVE First-fault load Operations speculatively executed\000 */
{ 203847 }, /* sve_pred_empty_spec\000\000instruction\000SVE predicated operations with no active predicates speculatively executed\000event=0x8075\000\00000\000SVE predicated operations with no active predicates speculatively executed\000 */
{ 204047 }, /* sve_pred_full_spec\000\000instruction\000SVE predicated operations speculatively executed with all active predicates\000event=0x8076\000\00000\000SVE predicated operations speculatively executed with all active predicates\000 */
{ 204437 }, /* sve_pred_partial_spec\000\000instruction\000SVE predicated operations speculatively executed with partially active predicates\000event=0x8077\000\00000\000SVE predicated operations speculatively executed with partially active predicates\000 */
{ 204653 }, /* sve_pred_spec\000\000instruction\000SVE predicated Operations speculatively executed\000event=0x8074\000\00000\000SVE predicated Operations speculatively executed\000 */
{ 205736 }, /* sw_incr\000\000instruction\000Instruction architecturally executed, Condition code check pass, software increment\000event=0\000\00000\000Instruction architecturally executed, Condition code check pass, software increment\000 */
{ 210294 }, /* ttbr_write_retired\000\000instruction\000Instruction architecturally executed, Condition code check pass, write to TTBR\000event=0x1c\000\00000\000Instruction architecturally executed, Condition code check pass, write to TTBR\000 */
{ 214354 }, /* vfp_spec\000\000instruction\000Operation speculatively executed, floating-point instruction\000event=0x75\000\00000\000Operation speculatively executed, floating-point instruction\000 */
{ 163148 }, /* mem_access\000\000memory\000Data memory access\000event=0x13\000\00000\000Data memory access\000 */
{ 164298 }, /* mem_access_rd\000\000memory\000Data memory access, read\000event=0x66\000\00000\000Data memory access, read\000 */
{ 164956 }, /* mem_access_wr\000\000memory\000Data memory access, write\000event=0x67\000\00000\000Data memory access, write\000 */
{ 179852 }, /* remote_access\000\000memory\000Access to another socket in a multi-socket system\000event=0x31\000\00000\000Access to another socket in a multi-socket system\000 */
{ 211505 }, /* unaligned_ld_spec\000\000memory\000Unaligned access, read\000event=0x68\000\00000\000Unaligned access, read\000 */
{ 212200 }, /* unaligned_ldst_spec\000\000memory\000Unaligned access\000event=0x6a\000\00000\000Unaligned access\000 */
{ 212637 }, /* unaligned_st_spec\000\000memory\000Unaligned access, write\000event=0x69\000\00000\000Unaligned access, write\000 */
{ 185831 }, /* stall\000\000pipeline\000No operation sent for execution\000event=0x3c\000\00000\000No operation sent for execution.  The counter counts every attributable cycle on which no attributable instruction or operation was sent for execution on this processing element\000 */
{ 186072 }, /* stall_backend\000\000pipeline\000No operation issued due to the backend\000event=0x24\000\00000\000No operation issued due to the backend\000 */
{ 192744 }, /* stall_backend_mem\000\000pipeline\000Memory stall cycles\000event=0x4005\000\00000\000Memory stall cycles.  The counter counts each cycle counted by STALL_BACKEND where there is a cache miss in the last level of cache within the processing element clock domain\000 */
{ 195875 }, /* stall_frontend\000\000pipeline\000No operation issued because of the frontend\000event=0x23\000\00000\000No operation issued because of the frontend\000 */
{ 199011 }, /* stall_slot\000\000pipeline\000No operation sent for execution on a slot\000event=0x3f\000\00000\000No operation sent for execution on a slot.  The counter counts on each attributable cycle the number of instruction or operation slots that were not occupied by an instruction or operation attributable to the processing element\000 */
{ 199317 }, /* stall_slot_backend\000\000pipeline\000No operation sent for execution on a slot due to the backend\000event=0x3d\000\00000\000No operation sent for execution on a slot due to the backend.  Counts each slot counted by STALL_SLOT where no attributable instruction or operation was sent for execution because the backend is unable to accept it\000 */
{ 199953 }, /* stall_slot_frontend\000\000pipeline\000No operation sent for execution on a slot due to the frontend\000event=0x3e\000\00000\000No operation sent for execution on a slot due to the frontend.  Counts each slot counted by STALL_SLOT where no attributable instruction or operation was sent for execution because there was no attributable instruction or operation available to issue from the processing element from the frontend for the slot\000 */
};

static const struct compact_pmu_event pmu_events__cavium_thunderx2[] = {
{ 42487 }, /* bus_access_rd\000\000core imp def\000Bus access read\000event=0x60\000\00000\000Bus access read\000 */
{ 42902 }, /* bus_access_wr\000\000core imp def\000Bus access write\000event=0x61\000\00000\000Bus access write\000 */
{ 56438 }, /* exc_dabort\000\000core imp def\000Exception taken, Data Abort and SError\000event=0x84\000\00000\000Exception taken, Data Abort and SError\000 */
{ 57074 }, /* exc_fiq\000\000core imp def\000Exception taken, FIQ\000event=0x87\000\00000\000Exception taken, FIQ\000 */
{ 57428 }, /* exc_hvc\000\000core imp def\000Exception taken, Hypervisor Call\000event=0x8a\000\00000\000Exception taken, Hypervisor Call\000 */
{ 57733 }, /* exc_irq\000\000core imp def\000Exception taken, IRQ\000event=0x86\000\00000\000Exception taken, IRQ\000 */
{ 58087 }, /* exc_pabort\000\000core imp def\000Exception taken, Instruction Abort\000event=0x83\000\00000\000Exception taken, Instruction Abort\000 */
{ 59042 }, /* exc_smc\000\000core imp def\000Exception taken, Secure Monitor Call\000event=0x88\000\00000\000Exception taken, Secure Monitor Call\000 */
{ 59366 }, /* exc_svc\000\000core imp def\000Exception taken, Supervisor Call\000event=0x82\000\00000\000Exception taken, Supervisor Call\000 */
{ 59963 }, /* exc_trap_dabort\000\000core imp def\000Exception taken, Data Abort or SError not taken locally\000event=0x8c\000\00000\000Exception taken, Data Abort or SError not taken locally\000 */
{ 60729 }, /* exc_trap_fiq\000\000core imp def\000Exception taken, FIQ not taken locally\000event=0x8f\000\00000\000Exception taken, FIQ not taken locally\000 */
{ 62073 }, /* exc_trap_irq\000\000core imp def\000Exception taken, IRQ not taken locally\000event=0x8e\000\00000\000Exception taken, IRQ not taken locally\000 */
{ 62464 }, /* exc_trap_other\000\000core imp def\000Exception taken, Other traps not taken locally\000event=0x8d\000\00000\000Exception taken, Other traps not taken locally\000 */
{ 62976 }, /* exc_trap_pabort\000\000core imp def\000Exception taken, Instruction Abort not taken locally\000event=0x8b\000\00000\000Exception taken, Instruction Abort not taken locally\000 */
{ 63531 }, /* exc_undef\000\000core imp def\000Exception taken, Other synchronous\000event=0x81\000\00000\000Exception taken, Other synchronous\000 */
{ 101505 }, /* l1d_cache_inval\000\000core imp def\000L1D cache invalidate\000event=0x48\000\00000\000Attributable Level 1 data cache invalidate\000 */
{ 102796 }, /* l1d_cache_rd\000\000core imp def\000L1D cache access, read\000event=0x40\000\00000\000Attributable Level 1 data cache access, read\000 */
{ 104260 }, /* l1d_cache_refill_inner\000\000core imp def\000L1D cache refill, inner\000event=0x44\000\00000\000Attributable Level 1 data cache refill, inner\000 */
{ 104688 }, /* l1d_cache_refill_outer\000\000core imp def\000L1D cache refill, outer\000event=0x45\000\00000\000Attributable Level 1 data cache refill, outer\000 */
{ 106004 }, /* l1d_cache_refill_rd\000\000core imp def\000L1D cache refill, read\000event=0x42\000\00000\000Attributable Level 1 data cache refill, read\000 */
{ 106501 }, /* l1d_cache_refill_wr\000\000core imp def\000L1D cache refill, write\000event=0x43\000\00000\000Attributable Level 1 data cache refill, write\000 */
{ 108097 }, /* l1d_cache_wb_clean\000\000core imp def\000L1D cache Write-Back, cleaning and coherency\000event=0x47\000\00000\000Level 1 data cache Write-Back, cleaning and coherency\000 */
{ 108617 }, /* l1d_cache_wb_victim\000\000core imp def\000L1D cache Write-Back, victim\000event=0x46\000\00000\000Attributable Level 1 data cache Write-Back, victim\000 */
{ 109102 }, /* l1d_cache_wr\000\000core imp def\000L1D cache access, write\000event=0x41\000\00000\000Attributable Level 1 data cache access, write\000 */
{ 110856 }, /* l1d_tlb_rd\000\000core imp def\000L1D tlb access, read\000event=0x4e\000\00000\000Attributable Level 1 data or unified TLB access, read\000 */
{ 112303 }, /* l1d_tlb_refill_rd\000\000core imp def\000L1D tlb refill, read\000event=0x4c\000\00000\000Attributable Level 1 data TLB refill, read\000 */
{ 113199 }, /* l1d_tlb_refill_wr\000\000core imp def\000L1D tlb refill, write\000event=0x4d\000\00000\000Attributable Level 1 data TLB refill, write\000 */
{ 114103 }, /* l1d_tlb_wr\000\000core imp def\000L1D tlb access, write\000event=0x4f\000\00000\000Attributable Level 1 data or unified TLB access, write\000 */
{ 144180 }, /* l2d_tlb_rd\000\000core imp def\000L2D cache access, read\000event=0x5e\000\00000\000Attributable Level 2 data or unified TLB access, read\000 */
{ 145196 }, /* l2d_tlb_refill_rd\000\000core imp def\000L2D cache refill, read\000event=0x5c\000\00000\000Attributable Level 2 data or unified TLB refill, read\000 */
{ 145674 }, /* l2d_tlb_refill_wr\000\000core imp def\000L2D cache refill, write\000event=0x5d\000\00000\000Attributable Level 2 data or unified TLB refill, write\000 */
{ 146126 }, /* l2d_tlb_wr\000\000core imp def\000L2D cache access, write\000event=0x5f\000\00000\000Attributable Level 2 data or unified TLB access, write\000 */
{ 163729 }, /* mem_access_rd\000\000core imp def\000Data memory access, read\000event=0x66\000\00000\000Data memory access, read\000 */
{ 164385 }, /* mem_access_wr\000\000core imp def\000Data memory access, write\000event=0x67\000\00000\000Data memory access, write\000 */
{ 211412 }, /* unaligned_ld_spec\000\000core imp def\000Unaligned access, read\000event=0x68\000\00000\000Unaligned access, read\000 */
{ 212117 }, /* unaligned_ldst_spec\000\000core imp def\000Unaligned access\000event=0x6a\000\00000\000Unaligned access\000 */
{ 212542 }, /* unaligned_st_spec\000\000core imp def\000Unaligned access, write\000event=0x69\000\00000\000Unaligned access, write\000 */
};

static const struct compact_pmu_event pmu_events__freescale_imx8mm_sys[] = {
{ 88987 }, /* imx8mm_ddr.cycles\000imx8_ddr\000ddrc\000ddr cycles event. Unit: imx8_ddr \000event=0\000i.MX8MM\00000\000\000 */
{ 89073 }, /* imx8mm_ddr.read\000imx8_ddr\000ddrc\000ddr read event. Unit: imx8_ddr \000event=0x35\000i.MX8MM\00000\000\000 */
{ 89158 }, /* imx8mm_ddr.read_cycles\000imx8_ddr\000ddrc\000ddr read-cycles event. Unit: imx8_ddr \000event=0x2a\000i.MX8MM\00000\000\000 */
{ 89257 }, /* imx8mm_ddr.write\000imx8_ddr\000ddrc\000ddr write event. Unit: imx8_ddr \000event=0x38\000i.MX8MM\00000\000\000 */
{ 89344 }, /* imx8mm_ddr.write_cycles\000imx8_ddr\000ddrc\000ddr write-cycles event. Unit: imx8_ddr \000event=0x2b\000i.MX8MM\00000\000\000 */
};

static const struct compact_pmu_event pmu_metrics__freescale_imx8mm_sys[] = {
{ 78109 }, /* imx8_ddr\000imx8mm_ddr_read.all\000\000imx8mm_ddr.read_cycles * 4 * 4\000\000bytes all masters read from ddr based on read-cycles event. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MM\000\000\00000 */
{ 78274 }, /* imx8_ddr\000imx8mm_ddr_write.all\000\000imx8mm_ddr.write_cycles * 4 * 4\000\000bytes all masters write to ddr based on write-cycles event. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MM\000\000\00000 */
};

static const struct compact_pmu_event pmu_events__freescale_imx8mn_sys[] = {
{ 89445 }, /* imx8mn_ddr.cycles\000imx8_ddr\000ddrc\000ddr cycles event. Unit: imx8_ddr \000event=0\000i.MX8MN\00000\000\000 */
{ 89531 }, /* imx8mn_ddr.read\000imx8_ddr\000ddrc\000ddr read event. Unit: imx8_ddr \000event=0x35\000i.MX8MN\00000\000\000 */
{ 89616 }, /* imx8mn_ddr.read_cycles\000imx8_ddr\000ddrc\000ddr read-cycles event. Unit: imx8_ddr \000event=0x2a\000i.MX8MN\00000\000\000 */
{ 89715 }, /* imx8mn_ddr.write\000imx8_ddr\000ddrc\000ddr write event. Unit: imx8_ddr \000event=0x38\000i.MX8MN\00000\000\000 */
{ 89802 }, /* imx8mn_ddr.write_cycles\000imx8_ddr\000ddrc\000ddr write-cycles event. Unit: imx8_ddr \000event=0x2b\000i.MX8MN\00000\000\000 */
};

static const struct compact_pmu_event pmu_metrics__freescale_imx8mn_sys[] = {
{ 78441 }, /* imx8_ddr\000imx8mn_ddr_read.all\000\000imx8mn_ddr.read_cycles * 4 * 2\000\000bytes all masters read from ddr based on read-cycles event. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MN\000\000\00000 */
{ 78606 }, /* imx8_ddr\000imx8mn_ddr_write.all\000\000imx8mn_ddr.write_cycles * 4 * 2\000\000bytes all masters write to ddr based on write-cycles event. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MN\000\000\00000 */
};

static const struct compact_pmu_event pmu_events__freescale_imx8mp_sys[] = {
{ 89903 }, /* imx8mp_ddr.cycles\000imx8_ddr\000ddrc\000ddr cycles event. Unit: imx8_ddr \000event=0\000i.MX8MP\00000\000\000 */
{ 89989 }, /* imx8mp_ddr.read\000imx8_ddr\000ddrc\000ddr read event. Unit: imx8_ddr \000event=0x35\000i.MX8MP\00000\000\000 */
{ 90074 }, /* imx8mp_ddr.read_cycles\000imx8_ddr\000ddrc\000ddr read-cycles event. Unit: imx8_ddr \000event=0x2a\000i.MX8MP\00000\000\000 */
{ 90173 }, /* imx8mp_ddr.write\000imx8_ddr\000ddrc\000ddr write event. Unit: imx8_ddr \000event=0x38\000i.MX8MP\00000\000\000 */
{ 90260 }, /* imx8mp_ddr.write_cycles\000imx8_ddr\000ddrc\000ddr write-cycles event. Unit: imx8_ddr \000event=0x2b\000i.MX8MP\00000\000\000 */
};

static const struct compact_pmu_event pmu_metrics__freescale_imx8mp_sys[] = {
{ 78773 }, /* imx8_ddr\000imx8mp_ddr_read.2d\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x0071@\000\000bytes of gpu 2d read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 78933 }, /* imx8_ddr\000imx8mp_ddr_read.3d\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x0070@\000\000bytes of gpu 3d read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 79093 }, /* imx8_ddr\000imx8mp_ddr_read.a53\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x0000@\000\000bytes of a53 core read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 79256 }, /* imx8_ddr\000imx8mp_ddr_read.all\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0xffff\\,axi_id\\=0x0000@\000\000bytes of all masters read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 79422 }, /* imx8_ddr\000imx8mp_ddr_read.audio_dsp\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x0041@\000\000bytes of audio dsp read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 79592 }, /* imx8_ddr\000imx8mp_ddr_read.audio_sdma2_burst\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x0063@\000\000bytes of audio sdma2_burst read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 79778 }, /* imx8_ddr\000imx8mp_ddr_read.audio_sdma2_per\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x0062@\000\000bytes of audio sdma2_per read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 79960 }, /* imx8_ddr\000imx8mp_ddr_read.audio_sdma3_burst\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x0065@\000\000bytes of audio sdma3_burst read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 80146 }, /* imx8_ddr\000imx8mp_ddr_read.audio_sdma3_per\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x0064@\000\000bytes of audio sdma3_per read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 80328 }, /* imx8_ddr\000imx8mp_ddr_read.audio_sdma_pif\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x0066@\000\000bytes of audio sdma_pif read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 80508 }, /* imx8_ddr\000imx8mp_ddr_read.dewarp\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x006f@\000\000bytes of display dewarp read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 80680 }, /* imx8_ddr\000imx8mp_ddr_read.hdmi_hdcp\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x0076@\000\000bytes of hdmi_tx tx_hdcp read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 80856 }, /* imx8_ddr\000imx8mp_ddr_read.hdmi_hrv_mwr\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x0074@\000\000bytes of hdmi_tx hrv_mwr read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 81035 }, /* imx8_ddr\000imx8mp_ddr_read.hdmi_lcdif\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x0075@\000\000bytes of hdmi_tx lcdif read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 81210 }, /* imx8_ddr\000imx8mp_ddr_read.isi1\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x006a@\000\000bytes of display isi1 read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 81378 }, /* imx8_ddr\000imx8mp_ddr_read.isi2\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x006b@\000\000bytes of display isi2 read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 81546 }, /* imx8_ddr\000imx8mp_ddr_read.isi3\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x006c@\000\000bytes of display isi3 read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 81714 }, /* imx8_ddr\000imx8mp_ddr_read.isp1\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x006d@\000\000bytes of display isp1 read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 81882 }, /* imx8_ddr\000imx8mp_ddr_read.isp2\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x006e@\000\000bytes of display isp2 read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 82050 }, /* imx8_ddr\000imx8mp_ddr_read.lcdif1\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x0068@\000\000bytes of display lcdif1 read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 82222 }, /* imx8_ddr\000imx8mp_ddr_read.lcdif2\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x0069@\000\000bytes of display lcdif2 read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 82394 }, /* imx8_ddr\000imx8mp_ddr_read.npu\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x0073@\000\000bytes of npu read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 82552 }, /* imx8_ddr\000imx8mp_ddr_read.pci\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x007a@\000\000bytes of hsio pci read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 82715 }, /* imx8_ddr\000imx8mp_ddr_read.supermix\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x000f\\,axi_id\\=0x0020@\000\000bytes of supermix(m7) core read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 82892 }, /* imx8_ddr\000imx8mp_ddr_read.usb1\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x0078@\000\000bytes of hsio usb1 read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 83057 }, /* imx8_ddr\000imx8mp_ddr_read.usb2\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x0079@\000\000bytes of hsio usb2 read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 83222 }, /* imx8_ddr\000imx8mp_ddr_read.vpu1\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x007c@\000\000bytes of vpu1 read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 83382 }, /* imx8_ddr\000imx8mp_ddr_read.vpu2\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x007d@\000\000bytes of vpu2 read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 83542 }, /* imx8_ddr\000imx8mp_ddr_read.vpu3\000\000imx8_ddr0@axid\\-read\\,axi_mask\\=0x0000\\,axi_id\\=0x007e@\000\000bytes of vpu3 read from ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 83702 }, /* imx8_ddr\000imx8mp_ddr_write.2d\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x0071@\000\000bytes of gpu 2d write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 83863 }, /* imx8_ddr\000imx8mp_ddr_write.3d\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x0070@\000\000bytes of gpu 3d write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 84024 }, /* imx8_ddr\000imx8mp_ddr_write.a53\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x0000@\000\000bytes of a53 core write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 84188 }, /* imx8_ddr\000imx8mp_ddr_write.all\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0xffff\\,axi_id\\=0x0000@\000\000bytes of all masters write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 84355 }, /* imx8_ddr\000imx8mp_ddr_write.audio_dsp\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x0041@\000\000bytes of audio dsp write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 84526 }, /* imx8_ddr\000imx8mp_ddr_write.audio_sdma2_burst\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x0063@\000\000bytes of audio sdma2_burst write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 84713 }, /* imx8_ddr\000imx8mp_ddr_write.audio_sdma2_per\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x0062@\000\000bytes of audio sdma2_per write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 84896 }, /* imx8_ddr\000imx8mp_ddr_write.audio_sdma3_burst\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x0065@\000\000bytes of audio sdma3_burst write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 85083 }, /* imx8_ddr\000imx8mp_ddr_write.audio_sdma3_per\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x0064@\000\000bytes of audio sdma3_per write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 85266 }, /* imx8_ddr\000imx8mp_ddr_write.audio_sdma_pif\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x0066@\000\000bytes of audio sdma_pif write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 85447 }, /* imx8_ddr\000imx8mp_ddr_write.dewarp\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x006f@\000\000bytes of display dewarp write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 85620 }, /* imx8_ddr\000imx8mp_ddr_write.hdmi_hdcp\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x0076@\000\000bytes of hdmi_tx tx_hdcp write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 85797 }, /* imx8_ddr\000imx8mp_ddr_write.hdmi_hrv_mwr\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x0074@\000\000bytes of hdmi_tx hrv_mwr write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 85977 }, /* imx8_ddr\000imx8mp_ddr_write.hdmi_lcdif\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x0075@\000\000bytes of hdmi_tx lcdif write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 86153 }, /* imx8_ddr\000imx8mp_ddr_write.isi1\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x006a@\000\000bytes of display isi1 write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 86322 }, /* imx8_ddr\000imx8mp_ddr_write.isi2\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x006b@\000\000bytes of display isi2 write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 86491 }, /* imx8_ddr\000imx8mp_ddr_write.isi3\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x006c@\000\000bytes of display isi3 write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 86660 }, /* imx8_ddr\000imx8mp_ddr_write.isp1\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x006d@\000\000bytes of display isp1 write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 86829 }, /* imx8_ddr\000imx8mp_ddr_write.isp2\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x006e@\000\000bytes of display isp2 write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 86998 }, /* imx8_ddr\000imx8mp_ddr_write.lcdif1\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x0068@\000\000bytes of display lcdif1 write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 87171 }, /* imx8_ddr\000imx8mp_ddr_write.lcdif2\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x0069@\000\000bytes of display lcdif2 write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 87344 }, /* imx8_ddr\000imx8mp_ddr_write.npu\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x0073@\000\000bytes of npu write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 87503 }, /* imx8_ddr\000imx8mp_ddr_write.pci\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x007a@\000\000bytes of hsio pci write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 87667 }, /* imx8_ddr\000imx8mp_ddr_write.supermix\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x000f\\,axi_id\\=0x0020@\000\000bytes of supermix(m7) write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 87840 }, /* imx8_ddr\000imx8mp_ddr_write.usb1\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x0078@\000\000bytes of hsio usb1 write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 88006 }, /* imx8_ddr\000imx8mp_ddr_write.usb2\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x0079@\000\000bytes of hsio usb2 write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 88172 }, /* imx8_ddr\000imx8mp_ddr_write.vpu1\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x007c@\000\000bytes of vpu1 write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 88333 }, /* imx8_ddr\000imx8mp_ddr_write.vpu2\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x007d@\000\000bytes of vpu2 write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
{ 88494 }, /* imx8_ddr\000imx8mp_ddr_write.vpu3\000\000imx8_ddr0@axid\\-write\\,axi_mask\\=0x0000\\,axi_id\\=0x007e@\000\000bytes of vpu3 write to ddr. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MP\000\000\00000 */
};

static const struct compact_pmu_event pmu_events__freescale_imx8mq_sys[] = {
{ 90361 }, /* imx8mq_ddr.cycles\000imx8_ddr\000ddrc\000ddr cycles event. Unit: imx8_ddr \000event=0\000i.MX8MQ\00000\000\000 */
{ 90447 }, /* imx8mq_ddr.read\000imx8_ddr\000ddrc\000ddr read event. Unit: imx8_ddr \000event=0x35\000i.MX8MQ\00000\000\000 */
{ 90532 }, /* imx8mq_ddr.read_cycles\000imx8_ddr\000ddrc\000ddr read-cycles event. Unit: imx8_ddr \000event=0x2a\000i.MX8MQ\00000\000\000 */
{ 90631 }, /* imx8mq_ddr.write\000imx8_ddr\000ddrc\000ddr write event. Unit: imx8_ddr \000event=0x38\000i.MX8MQ\00000\000\000 */
{ 90718 }, /* imx8mq_ddr.write_cycles\000imx8_ddr\000ddrc\000ddr write-cycles event. Unit: imx8_ddr \000event=0x2b\000i.MX8MQ\00000\000\000 */
};

static const struct compact_pmu_event pmu_metrics__freescale_imx8mq_sys[] = {
{ 88655 }, /* imx8_ddr\000imx8mq_ddr_read.all\000\000imx8mq_ddr.read_cycles * 4 * 4\000\000bytes all masters read from ddr based on read-cycles event. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MQ\000\000\00000 */
{ 88820 }, /* imx8_ddr\000imx8mq_ddr_write.all\000\000imx8mq_ddr.write_cycles * 4 * 4\000\000bytes all masters write to ddr based on write-cycles event. Unit: imx8_ddr \000\0009.765625e-4KB\000i.MX8MQ\000\000\00000 */
};

static const struct compact_pmu_event pmu_events__fujitsu_a64fx[] = {
{ 35743 }, /* br_mis_pred\000\000branch\000Mispredicted or not predicted branch speculatively executed\000event=0x10\000\00000\000Mispredicted or not predicted branch speculatively executed\000 */
{ 36925 }, /* br_pred\000\000branch\000Predictable branch speculatively executed\000event=0x12\000\00000\000Predictable branch speculatively executed\000 */
{ 43417 }, /* bus_read_total_mem\000\000bus\000This event counts read transactions from measured CMG local memory to measured CMG\000event=0x316\000\00000\000This event counts read transactions from measured CMG local memory to measured CMG\000 */
{ 43623 }, /* bus_read_total_pci\000\000bus\000This event counts read transactions from PCI controller to measured CMG\000event=0x315\000\00000\000This event counts read transactions from PCI controller to measured CMG\000 */
{ 43807 }, /* bus_read_total_tofu\000\000bus\000This event counts read transactions from tofu controller to measured CMG\000event=0x314\000\00000\000This event counts read transactions from tofu controller to measured CMG\000 */
{ 43994 }, /* bus_write_total_cmg0\000\000bus\000This event counts write transactions from measured CMG to CMG0, if measured CMG is not CMG0\000event=0x318\000\00000\000This event counts write transactions from measured CMG to CMG0, if measured CMG is not CMG0\000 */
{ 44220 }, /* bus_write_total_cmg1\000\000bus\000This event counts write transactions from measured CMG to CMG1, if measured CMG is not CMG1\000event=0x319\000\00000\000This event counts write transactions from measured CMG to CMG1, if measured CMG is not CMG1\000 */
{ 44446 }, /* bus_write_total_cmg2\000\000bus\000This event counts write transactions from measured CMG to CMG2, if measured CMG is not CMG2\000event=0x31a\000\00000\000This event counts write transactions from measured CMG to CMG2, if measured CMG is not CMG2\000 */
{ 44672 }, /* bus_write_total_cmg3\000\000bus\000This event counts write transactions from measured CMG to CMG3, if measured CMG is not CMG3\000event=0x31b\000\00000\000This event counts write transactions from measured CMG to CMG3, if measured CMG is not CMG3\000 */
{ 44898 }, /* bus_write_total_mem\000\000bus\000This event counts write transactions from measured CMG to measured CMG local memory\000event=0x31e\000\00000\000This event counts write transactions from measured CMG to measured CMG local memory\000 */
{ 45107 }, /* bus_write_total_pci\000\000bus\000This event counts write transactions from measured CMG to PCI controller\000event=0x31d\000\00000\000This event counts write transactions from measured CMG to PCI controller\000 */
{ 45294 }, /* bus_write_total_tofu\000\000bus\000This event counts write transactions from measured CMG to tofu controller\000event=0x31c\000\00000\000This event counts write transactions from measured CMG to tofu controller\000 */
{ 54288 }, /* ea_l2\000\000cache\000This event counts energy consumption per cycle of L2 cache\000event=0x3e0\000\00000\000This event counts energy consumption per cycle of L2 cache\000 */
{ 95112 }, /* l1_miss_wait\000\000cache\000This event counts outstanding L1D cache miss requests per cycle\000event=0x208\000\00000\000This event counts outstanding L1D cache miss requests per cycle\000 */
{ 100289 }, /* l1d_cache\000\000cache\000Level 1 data cache access\000event=0x4\000\00000\000Level 1 data cache access\000 */
{ 103129 }, /* l1d_cache_refill\000\000cache\000Level 1 data cache refill\000event=0x3\000\00000\000Level 1 data cache refill\000 */
{ 103812 }, /* l1d_cache_refill_dm\000\000cache\000This event counts L1D_CACHE_REFILL caused by demand access\000event=0x200\000\00000\000This event counts L1D_CACHE_REFILL caused by demand access\000 */
{ 103973 }, /* l1d_cache_refill_hwprf\000\000cache\000This event counts L1D_CACHE_REFILL caused by hardware prefetch\000event=0x202\000\00000\000This event counts L1D_CACHE_REFILL caused by hardware prefetch\000 */
{ 105701 }, /* l1d_cache_refill_prf\000\000cache\000This event counts L1D_CACHE_REFILL caused by software or hardware prefetch\000event=0x49\000\00000\000This event counts L1D_CACHE_REFILL caused by software or hardware prefetch\000 */
{ 106891 }, /* l1d_cache_wb\000\000cache\000Attributable Level 1 data cache write-back\000event=0x15\000\00000\000Attributable Level 1 data cache write-back\000 */
{ 111207 }, /* l1d_tlb_refill\000\000cache\000Attributable Level 1 data TLB refill\000event=0x5\000\00000\000Attributable Level 1 data TLB refill\000 */
{ 116626 }, /* l1i_cache\000\000cache\000Attributable Level 1 instruction cache access\000event=0x14\000\00000\000Attributable Level 1 instruction cache access\000 */
{ 118420 }, /* l1i_cache_refill\000\000cache\000Level 1 instruction cache refill\000event=0x1\000\00000\000Level 1 instruction cache refill\000 */
{ 118951 }, /* l1i_miss_wait\000\000cache\000This event counts outstanding L1I cache miss requests per cycle\000event=0x209\000\00000\000This event counts outstanding L1I cache miss requests per cycle\000 */
{ 120198 }, /* l1i_tlb_refill\000\000cache\000Attributable Level 1 instruction TLB refill\000event=0x2\000\00000\000Attributable Level 1 instruction TLB refill\000 */
{ 124301 }, /* l2_miss_count\000\000cache\000This event counts the number of times of L2 cache miss\000event=0x309\000\00000\000This event counts the number of times of L2 cache miss\000 */
{ 124448 }, /* l2_miss_wait\000\000cache\000This event counts outstanding L2 cache miss requests per cycle\000event=0x308\000\00000\000This event counts outstanding L2 cache miss requests per cycle\000 */
{ 126244 }, /* l2d_cache\000\000cache\000Level 2 data cache access\000event=0x16\000\00000\000Level 2 data cache access\000 */
{ 128739 }, /* l2d_cache_mibmch_prf\000\000cache\000This event counts operations where software or hardware prefetch hits an L2 cache refill buffer allocated by demand access\000event=0x326\000\00000\000This event counts operations where software or hardware prefetch hits an L2 cache refill buffer allocated by demand access\000 */
{ 129862 }, /* l2d_cache_refill\000\000cache\000Level 2 data refill\000event=0x17\000\00000\000Level 2 data refill\000 */
{ 130210 }, /* l2d_cache_refill_dm\000\000cache\000This event counts L2D_CACHE_REFILL caused by demand access\000event=0x300\000\00000\000This event counts L2D_CACHE_REFILL caused by demand access\000 */
{ 130371 }, /* l2d_cache_refill_hwprf\000\000cache\000This event counts L2D_CACHE_REFILL caused by hardware prefetch\000event=0x302\000\00000\000This event counts L2D_CACHE_REFILL caused by hardware prefetch\000 */
{ 132095 }, /* l2d_cache_refill_prf\000\000cache\000This event counts L2D_CACHE_REFILL caused by software or hardware prefetch\000event=0x59\000\00000\000This event counts L2D_CACHE_REFILL caused by software or hardware prefetch\000 */
{ 136124 }, /* l2d_cache_swap_local\000\000cache\000This event counts operations where demand access hits an L2 cache refill buffer allocated by software or hardware prefetch\000event=0x396\000\00000\000This event counts operations where demand access hits an L2 cache refill buffer allocated by software or hardware prefetch\000 */
{ 136414 }, /* l2d_cache_wb\000\000cache\000Attributable Level 2 data cache write-back\000event=0x18\000\00000\000Attributable Level 2 data cache write-back\000 */
{ 143108 }, /* l2d_swap_dm\000\000cache\000This event counts operations where demand access hits an L2 cache refill buffer allocated by software or hardware prefetch\000event=0x325\000\00000\000This event counts operations where demand access hits an L2 cache refill buffer allocated by software or hardware prefetch\000 */
{ 143389 }, /* l2d_tlb\000\000cache\000Attributable Level 2 data or unified TLB access\000event=0x2f\000\00000\000Attributable Level 2 data or unified TLB access\000 */
{ 144725 }, /* l2d_tlb_refill\000\000cache\000Attributable Level 2 data TLB refill\000event=0x2d\000\00000\000Attributable Level 2 data TLB refill\000 */
{ 148794 }, /* l2i_tlb\000\000cache\000Attributable Level 2 instruction TLB access\000event=0x30\000\00000\000Attributable Level 2 instruction TLB access\000 */
{ 149110 }, /* l2i_tlb_refill\000\000cache\000Attributable Level 2 instruction TLB refill\000event=0x2e\000\00000\000Attributable Level 2 instruction TLB refill\000 */
{ 48201 }, /* cpu_cycles\000\000cycle\000Cycle\000event=0x11\000\00000\000Cycle\000 */
{ 56959 }, /* exc_dabort\000\000exception\000Exception taken, Data Abort and SError\000event=0x84\000\00000\000Exception taken, Data Abort and SError\000 */
{ 57280 }, /* exc_fiq\000\000exception\000Exception taken, FIQ\000event=0x87\000\00000\000Exception taken, FIQ\000 */
{ 57633 }, /* exc_hvc\000\000exception\000Exception taken, Hypervisor Call\000event=0x8a\000\00000\000Exception taken, Hypervisor Call\000 */
{ 57939 }, /* exc_irq\000\000exception\000Exception taken, IRQ\000event=0x86\000\00000\000Exception taken, IRQ\000 */
{ 58355 }, /* exc_pabort\000\000exception\000Exception taken, Instruction Abort\000event=0x83\000\00000\000Exception taken, Instruction Abort\000 */
{ 59258 }, /* exc_smc\000\000exception\000Exception taken, Secure Monitor Call\000event=0x88\000\00000\000Exception taken, Secure Monitor Call\000 */
{ 59572 }, /* exc_svc\000\000exception\000Exception taken, Supervisor Call\000event=0x82\000\00000\000Exception taken, Supervisor Call\000 */
{ 59896 }, /* exc_taken\000\000exception\000Exception taken\000event=0x9\000\00000\000Exception taken\000 */
{ 64129 }, /* exc_undef\000\000exception\000Exception taken, Other synchronous\000event=0x81\000\00000\000Exception taken, Other synchronous\000 */
{ 20878 }, /* ase_spec\000\000instruction\000Operation speculatively executed, Advanced SIMD instruction\000event=0x74\000\00000\000Operation speculatively executed, Advanced SIMD instruction\000 */
{ 23494 }, /* bc_ld_spec\000\000instruction\000This event counts architecturally executed SIMD broadcast floating-point load operations\000event=0x11a\000\00000\000This event counts architecturally executed SIMD broadcast floating-point load operations\000 */
{ 30886 }, /* br_immed_spec\000\000instruction\000Branch speculatively executed, immediate branch\000event=0x78\000\00000\000Branch speculatively executed, immediate branch\000 */
{ 35172 }, /* br_indirect_spec\000\000instruction\000Branch speculatively executed, indirect branch\000event=0x7a\000\00000\000Branch speculatively executed, indirect branch\000 */
{ 40826 }, /* br_return_spec\000\000instruction\000Branch speculatively executed, procedure return\000event=0x79\000\00000\000Branch speculatively executed, procedure return\000 */
{ 45546 }, /* cid_write_retired\000\000instruction\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000event=0xb\000\00000\000Instruction architecturally executed, condition code check pass, write to CONTEXTIDR\000 */
{ 48419 }, /* crypto_spec\000\000instruction\000Operation speculatively executed, Cryptographic instruction\000event=0x77\000\00000\000Operation speculatively executed, Cryptographic instruction\000 */
{ 49726 }, /* dczva_spec\000\000instruction\000This event counts architecturally executed zero blocking operations due to the 'DC ZVA' instruction\000event=0x9f\000\00000\000This event counts architecturally executed zero blocking operations due to the 'DC ZVA' instruction\000 */
{ 51117 }, /* dmb_spec\000\000instruction\000Barrier speculatively executed, DMB\000event=0x7e\000\00000\000Barrier speculatively executed, DMB\000 */
{ 52153 }, /* dp_spec\000\000instruction\000Operation speculatively executed, integer data processing\000event=0x73\000\00000\000Operation speculatively executed, integer data processing\000 */
{ 53209 }, /* dsb_spec\000\000instruction\000Barrier speculatively executed, DSB\000event=0x7d\000\00000\000Barrier speculatively executed, DSB\000 */
{ 55093 }, /* effective_inst_spec\000\000instruction\000This event counts architecturally executed instructions, excluding the MOVPRFX instruction\000event=0x121\000\00000\000This event counts architecturally executed instructions, excluding the MOVPRFX instruction\000 */
{ 58850 }, /* exc_return\000\000instruction\000Instruction architecturally executed, condition check pass, exception return\000event=0xa\000\00000\000Instruction architecturally executed, condition check pass, exception return\000 */
{ 68334 }, /* fp_ld_spec\000\000instruction\000This event counts architecturally executed NOSIMD load operations that using SIMD&FP registers\000event=0x112\000\00000\000This event counts architecturally executed NOSIMD load operations that using SIMD&FP registers\000 */
{ 68564 }, /* fp_mv_spec\000\000instruction\000This event counts architecturally executed floating-point move operations\000event=0x105\000\00000\000This event counts architecturally executed floating-point move operations\000 */
{ 70005 }, /* fp_st_spec\000\000instruction\000This event counts architecturally executed NOSIMD store operations that using SIMD&FP registers\000event=0x113\000\00000\000This event counts architecturally executed NOSIMD store operations that using SIMD&FP registers\000 */
{ 74484 }, /* iel_spec\000\000instruction\000This event counts architecturally executed inter-element manipulation operations\000event=0x109\000\00000\000This event counts architecturally executed inter-element manipulation operations\000 */
{ 90819 }, /* inst_retired\000\000instruction\000Instruction architecturally executed\000event=0x8\000\00000\000Instruction architecturally executed\000 */
{ 91235 }, /* inst_spec\000\000instruction\000Operation speculatively executed\000event=0x1b\000\00000\000Operation speculatively executed\000 */
{ 91566 }, /* ireg_spec\000\000instruction\000This event counts architecturally executed inter-register manipulation operations\000event=0x10a\000\00000\000This event counts architecturally executed inter-register manipulation operations\000 */
{ 91769 }, /* isb_spec\000\000instruction\000Barrier speculatively executed, ISB\000event=0x7c\000\00000\000Barrier speculatively executed, ISB\000 */
{ 159047 }, /* ld_spec\000\000instruction\000Operation speculatively executed, load\000event=0x70\000\00000\000Operation speculatively executed, load\000 */
{ 159351 }, /* ldrex_spec\000\000instruction\000Exclusive operation speculatively executed, LDREX or LDX\000event=0x6c\000\00000\000Exclusive operation speculatively executed, LDREX or LDX\000 */
{ 159973 }, /* ldst_spec\000\000instruction\000Operation speculatively executed, load or store\000event=0x72\000\00000\000Operation speculatively executed, load or store\000 */
{ 175115 }, /* pc_write_spec\000\000instruction\000Operation speculatively executed, software change of the PC\000event=0x76\000\00000\000Operation speculatively executed, software change of the PC\000 */
{ 176115 }, /* post_index_spec\000\000instruction\000This event counts architecturally executed operations that uses 'post-index' as its addressing mode\000event=0x124\000\00000\000This event counts architecturally executed operations that uses 'post-index' as its addressing mode\000 */
{ 176360 }, /* prd_spec\000\000instruction\000This event counts architecturally executed operations that using predicate register\000event=0x108\000\00000\000This event counts architecturally executed operations that using predicate register\000 */
{ 176742 }, /* pre_index_spec\000\000instruction\000This event counts architecturally executed operations that uses 'pre-index' as its addressing mode\000event=0x123\000\00000\000This event counts architecturally executed operations that uses 'pre-index' as its addressing mode\000 */
{ 185522 }, /* st_spec\000\000instruction\000Operation speculatively executed, store\000event=0x71\000\00000\000Operation speculatively executed, store\000 */
{ 201592 }, /* strex_spec\000\000instruction\000Exclusive operation speculatively executed, STREX or STX\000event=0x6f\000\00000\000Exclusive operation speculatively executed, STREX or STX\000 */
{ 205736 }, /* sw_incr\000\000instruction\000Instruction architecturally executed, Condition code check pass, software increment\000event=0\000\00000\000Instruction architecturally executed, Condition code check pass, software increment\000 */
{ 214354 }, /* vfp_spec\000\000instruction\000Operation speculatively executed, floating-point instruction\000event=0x75\000\00000\000Operation speculatively executed, floating-point instruction\000 */
{ 54435 }, /* ea_memory\000\000memory\000This event counts energy consumption per cycle of CMG local memory\000event=0x3e8\000\00000\000This event counts energy consumption per cycle of CMG local memory\000 */
{ 19549 }, /* _0inst_commit\000\000other\000This event counts every cycle that no instruction was committed, but counts at the time when commits MOVPRFX only\000event=0x190\000\00000\000This event counts every cycle that no instruction was committed, but counts at the time when commits MOVPRFX only\000 */
{ 19814 }, /* _1inst_commit\000\000other\000This event counts every cycle that one instruction is committed\000event=0x191\000\00000\000This event counts every cycle that one instruction is committed\000 */
{ 19979 }, /* _2inst_commit\000\000other\000This event counts every cycle that two instructions are committed\000event=0x192\000\00000\000This event counts every cycle that two instructions are committed\000 */
{ 20148 }, /* _3inst_commit\000\000other\000This event counts every cycle that three instructions are committed\000event=0x193\000\00000\000This event counts every cycle that three instructions are committed\000 */
{ 20321 }, /* _4inst_commit\000\000other\000This event counts every cycle that four instructions are committed\000event=0x194\000\00000\000This event counts every cycle that four instructions are committed\000 */
{ 27645 }, /* br_comp_wait\000\000other\000This event counts every cycle that no instruction was committed and the oldest and uncommitted instruction is a branch instruction\000event=0x18b\000\00000\000This event counts every cycle that no instruction was committed and the oldest and uncommitted instruction is a branch instruction\000 */
{ 54147 }, /* ea_core\000\000other\000This event counts energy consumption per cycle of core\000event=0x1e0\000\00000\000This event counts energy consumption per cycle of core\000 */
{ 55612 }, /* eu_comp_wait\000\000other\000This event counts every cycle that no instruction was committed and the oldest and uncommitted instruction is an integer or floating-point/SIMD instruction\000event=0x189\000\00000\000This event counts every cycle that no instruction was committed and the oldest and uncommitted instruction is an integer or floating-point/SIMD instruction\000 */
{ 64746 }, /* fl_comp_wait\000\000other\000This event counts every cycle that no instruction was committed and the oldest and uncommitted instruction is a floating-point/SIMD instruction\000event=0x18a\000\00000\000This event counts every cycle that no instruction was committed and the oldest and uncommitted instruction is a floating-point/SIMD instruction\000 */
{ 115874 }, /* l1hwpf_inj_alloc_pf\000\000other\000This event counts allocation type prefetch injection requests to L1D cache generated by hardware prefetcher\000event=0x231\000\00000\000This event counts allocation type prefetch injection requests to L1D cache generated by hardware prefetcher\000 */
{ 116133 }, /* l1hwpf_inj_noalloc_pf\000\000other\000This event counts non-allocation type prefetch injection requests to L1D cache generated by hardware prefetcher\000event=0x232\000\00000\000This event counts non-allocation type prefetch injection requests to L1D cache generated by hardware prefetcher\000 */
{ 116402 }, /* l1hwpf_stream_pf\000\000other\000This event counts streaming prefetch requests to L1D cache generated by hardware prefetcher\000event=0x230\000\00000\000This event counts streaming prefetch requests to L1D cache generated by hardware prefetcher\000 */
{ 147858 }, /* l2hwpf_inj_alloc_pf\000\000other\000This event counts allocation type prefetch injection requests to L2 cache generated by hardware prefetcher\000event=0x234\000\00000\000This event counts allocation type prefetch injection requests to L2 cache generated by hardware prefetcher\000 */
{ 148115 }, /* l2hwpf_inj_noalloc_pf\000\000other\000This event counts non-allocation type prefetch injection requests to L2 cache generated by hardware prefetcher\000event=0x235\000\00000\000This event counts non-allocation type prefetch injection requests to L2 cache generated by hardware prefetcher\000 */
{ 148382 }, /* l2hwpf_other\000\000other\000This event counts prefetch requests to L2 cache generated by the other causes\000event=0x236\000\00000\000This event counts prefetch requests to L2 cache generated by the other causes\000 */
{ 148574 }, /* l2hwpf_stream_pf\000\000other\000This event counts streaming prefetch requests to L2 cache generated by hardware prefecher\000event=0x233\000\00000\000This event counts streaming prefetch requests to L2 cache generated by hardware prefecher\000 */
{ 155618 }, /* ld_comp_wait\000\000other\000This event counts every cycle that no instruction was committed because the oldest and uncommitted load/store/prefetch operation waits for L1D cache, L2 cache and memory access\000event=0x184\000\00000\000This event counts every cycle that no instruction was committed because the oldest and uncommitted load/store/prefetch operation waits for L1D cache, L2 cache and memory access\000 */
{ 156008 }, /* ld_comp_wait_ex\000\000other\000This event counts every cycle that no instruction was committed because the oldest and uncommitted integer load operation waits for L1D cache, L2 cache and memory access\000event=0x185\000\00000\000This event counts every cycle that no instruction was committed because the oldest and uncommitted integer load operation waits for L1D cache, L2 cache and memory access\000 */
{ 156387 }, /* ld_comp_wait_l1_miss\000\000other\000This event counts every cycle that no instruction was committed because the oldest and uncommitted load/store/prefetch operation waits for L2 cache access\000event=0x182\000\00000\000This event counts every cycle that no instruction was committed because the oldest and uncommitted load/store/prefetch operation waits for L2 cache access\000 */
{ 156741 }, /* ld_comp_wait_l1_miss_ex\000\000other\000This event counts every cycle that no instruction was committed because the oldest and uncommitted integer load operation waits for L2 cache access\000event=0x183\000\00000\000This event counts every cycle that no instruction was committed because the oldest and uncommitted integer load operation waits for L2 cache access\000 */
{ 157084 }, /* ld_comp_wait_l2_miss\000\000other\000This event counts every cycle that no operation was committed because the oldest and uncommitted load/store/prefetch operation waits for memory access\000event=0x180\000\00000\000This event counts every cycle that no operation was committed because the oldest and uncommitted load/store/prefetch operation waits for memory access\000 */
{ 157430 }, /* ld_comp_wait_l2_miss_ex\000\000other\000This event counts every cycle that no instruction was committed because the oldest and uncommitted integer load operation waits for memory access\000event=0x181\000\00000\000This event counts every cycle that no instruction was committed because the oldest and uncommitted integer load operation waits for memory access\000 */
{ 157769 }, /* ld_comp_wait_pfp_busy\000\000other\000This event counts every cycle that no instruction was committed due to the lack of an available prefetch port\000event=0x186\000\00000\000This event counts every cycle that no instruction was committed due to the lack of an available prefetch port\000 */
{ 158034 }, /* ld_comp_wait_pfp_busy_ex\000\000other\000This event counts the LD_COMP_WAIT_PFP_BUSY caused by an integer load operation\000event=0x187\000\00000\000This event counts the LD_COMP_WAIT_PFP_BUSY caused by an integer load operation\000 */
{ 158242 }, /* ld_comp_wait_pfp_busy_swpf\000\000other\000This event counts the LD_COMP_WAIT_PFP_BUSY caused by a software prefetch instruction\000event=0x188\000\00000\000This event counts the LD_COMP_WAIT_PFP_BUSY caused by a software prefetch instruction\000 */
{ 181122 }, /* rob_empty\000\000other\000This event counts every cycle that no instruction was committed because the CSE is empty\000event=0x18c\000\00000\000This event counts every cycle that no instruction was committed because the CSE is empty\000 */
{ 181333 }, /* rob_empty_stq_busy\000\000other\000This event counts every cycle that no instruction was committed because the CSE is empty and the store port (SP) is full\000event=0x18d\000\00000\000This event counts every cycle that no instruction was committed because the CSE is empty and the store port (SP) is full\000 */
{ 184497 }, /* single_movprfx_commit\000\000other\000This event counts every cycle that only the MOVPRFX instruction is committed\000event=0x199\000\00000\000This event counts every cycle that only the MOVPRFX instruction is committed\000 */
{ 213768 }, /* uop_only_commit\000\000other\000This event counts every cycle that only any micro-operations are committed\000event=0x198\000\00000\000This event counts every cycle that only any micro-operations are committed\000 */
{ 214116 }, /* uop_split\000\000other\000This event counts the occurrence count of the micro-operation split\000event=0x139\000\00000\000This event counts the occurrence count of the micro-operation split\000 */
{ 215078 }, /* wfe_wfi_cycle\000\000other\000This event counts every cycle that the instruction unit is halted by the WFE/WFI instruction\000event=0x18e\000\00000\000This event counts every cycle that the instruction unit is halted by the WFE/WFI instruction\000 */
{ 54603 }, /* eaga_val\000\000pipeline\000This event counts valid cycles of EAGA pipeline\000event=0x1a0\000\00000\000This event counts valid cycles of EAGA pipeline\000 */
{ 54734 }, /* eagb_val\000\000pipeline\000This event counts valid cycles of EAGB pipeline\000event=0x1a1\000\00000\000This event counts valid cycles of EAGB pipeline\000 */
{ 56182 }, /* exa_val\000\000pipeline\000This event counts valid cycles of EXA pipeline\000event=0x1a2\000\00000\000This event counts valid cycles of EXA pipeline\000 */
{ 56310 }, /* exb_val\000\000pipeline\000This event counts valid cycles of EXB pipeline\000event=0x1a3\000\00000\000This event counts valid cycles of EXB pipeline\000 */
{ 65070 }, /* fla_val\000\000pipeline\000This event counts valid cycles of FLA pipeline\000event=0x1a4\000\00000\000This event counts valid cycles of FLA pipeline\000 */
{ 65198 }, /* fla_val_prd_cnt\000\000pipeline\000This event counts the number of 1's in the predicate bits of request in FLA pipeline, where it is corrected so that it becomes 16 when all bits are 1\000event=0x1b4\000\00000\000This event counts the number of 1's in the predicate bits of request in FLA pipeline, where it is corrected so that it becomes 16 when all bits are 1\000 */
{ 65540 }, /* flb_val\000\000pipeline\000This event counts valid cycles of FLB pipeline\000event=0x1a5\000\00000\000This event counts valid cycles of FLB pipeline\000 */
{ 65668 }, /* flb_val_prd_cnt\000\000pipeline\000This event counts the number of 1's in the predicate bits of request in FLB pipeline, where it is corrected so that it becomes 16 when all bits are 1\000event=0x1b5\000\00000\000This event counts the number of 1's in the predicate bits of request in FLB pipeline, where it is corrected so that it becomes 16 when all bits are 1\000 */
{ 96088 }, /* l1_pipe0_comp\000\000pipeline\000This event counts completed requests in L1D cache pipeline#0\000event=0x260\000\00000\000This event counts completed requests in L1D cache pipeline#0\000 */
{ 96250 }, /* l1_pipe0_comp_prd_cnt\000\000pipeline\000This event counts the number of 1's in the predicate bits of request in L1D cache pipeline#0, where it is corrected so that it becomes 16 when all bits are 1\000event=0x2b8\000\00000\000This event counts the number of 1's in the predicate bits of request in L1D cache pipeline#0, where it is corrected so that it becomes 16 when all bits are 1\000 */
{ 96614 }, /* l1_pipe0_val\000\000pipeline\000This event counts valid cycles of L1D cache pipeline#0\000event=0x240\000\00000\000This event counts valid cycles of L1D cache pipeline#0\000 */
{ 96763 }, /* l1_pipe0_val_iu_not_sec0\000\000pipeline\000This event counts requests in L1D cache pipeline#0 that its sector cache ID is not 0\000event=0x2a0\000\00000\000This event counts requests in L1D cache pipeline#0 that its sector cache ID is not 0\000 */
{ 96984 }, /* l1_pipe0_val_iu_tag_adrs_pfe\000\000pipeline\000This event counts requests in L1D cache pipeline#0 that its pfe bit of tagged address is 1\000event=0x251\000\00000\000This event counts requests in L1D cache pipeline#0 that its pfe bit of tagged address is 1\000 */
{ 97221 }, /* l1_pipe0_val_iu_tag_adrs_sce\000\000pipeline\000This event counts requests in L1D cache pipeline#0 that its sce bit of tagged address is 1\000event=0x250\000\00000\000This event counts requests in L1D cache pipeline#0 that its sce bit of tagged address is 1\000 */
{ 97458 }, /* l1_pipe1_comp\000\000pipeline\000This event counts completed requests in L1D cache pipeline#1\000event=0x261\000\00000\000This event counts completed requests in L1D cache pipeline#1\000 */
{ 97620 }, /* l1_pipe1_comp_prd_cnt\000\000pipeline\000This event counts the number of 1's in the predicate bits of request in L1D cache pipeline#1, where it is corrected so that it becomes 16 when all bits are 1\000event=0x2b9\000\00000\000This event counts the number of 1's in the predicate bits of request in L1D cache pipeline#1, where it is corrected so that it becomes 16 when all bits are 1\000 */
{ 97984 }, /* l1_pipe1_val\000\000pipeline\000This event counts valid cycles of L1D cache pipeline#1\000event=0x241\000\00000\000This event counts valid cycles of L1D cache pipeline#1\000 */
{ 98133 }, /* l1_pipe1_val_iu_not_sec0\000\000pipeline\000This event counts requests in L1D cache pipeline#1 that its sector cache ID is not 0\000event=0x2a1\000\00000\000This event counts requests in L1D cache pipeline#1 that its sector cache ID is not 0\000 */
{ 98354 }, /* l1_pipe1_val_iu_tag_adrs_pfe\000\000pipeline\000This event counts requests in L1D cache pipeline#1 that its pfe bit of tagged address is 1\000event=0x253\000\00000\000This event counts requests in L1D cache pipeline#1 that its pfe bit of tagged address is 1\000 */
{ 98591 }, /* l1_pipe1_val_iu_tag_adrs_sce\000\000pipeline\000This event counts requests in L1D cache pipeline#1 that its sce bit of tagged address is 1\000event=0x252\000\00000\000This event counts requests in L1D cache pipeline#1 that its sce bit of tagged address is 1\000 */
{ 98828 }, /* l1_pipe_abort_stld_intlk\000\000pipeline\000This event counts aborted requests in L1D pipelines that due to store-load interlock\000event=0x274\000\00000\000This event counts aborted requests in L1D pipelines that due to store-load interlock\000 */
{ 99049 }, /* l1_pipe_comp_gather_0flow\000\000pipeline\000This event counts the number of times where 2 elements of the gather instructions became 0 flow because both predicate values are 0\000event=0x2b2\000\00000\000This event counts the number of times where 2 elements of the gather instructions became 0 flow because both predicate values are 0\000 */
{ 99365 }, /* l1_pipe_comp_gather_1flow\000\000pipeline\000This event counts the number of times where 2 elements of the gather instructions became 1 flow because 2 elements could be combined\000event=0x2b1\000\00000\000This event counts the number of times where 2 elements of the gather instructions became 1 flow because 2 elements could be combined\000 */
{ 99683 }, /* l1_pipe_comp_gather_2flow\000\000pipeline\000This event counts the number of times where 2 elements of the gather instructions became 2 flows because 2 elements could not be combined\000event=0x2b0\000\00000\000This event counts the number of times where 2 elements of the gather instructions became 2 flows because 2 elements could not be combined\000 */
{ 100011 }, /* l1_pipe_comp_scatter_1flow\000\000pipeline\000This event counts the number of flows of the scatter instructions\000event=0x2b3\000\00000\000This event counts the number of flows of the scatter instructions\000 */
{ 119116 }, /* l1i_pipe_comp\000\000pipeline\000This event counts completed requests in L1I cache pipeline\000event=0x268\000\00000\000This event counts completed requests in L1I cache pipeline\000 */
{ 119274 }, /* l1i_pipe_val\000\000pipeline\000This event counts valid cycles of L1I cache pipeline\000event=0x269\000\00000\000This event counts valid cycles of L1I cache pipeline\000 */
{ 124610 }, /* l2_pipe_comp_all\000\000pipeline\000This event counts completed requests in L2 cache pipeline\000event=0x350\000\00000\000This event counts completed requests in L2 cache pipeline\000 */
{ 124769 }, /* l2_pipe_comp_pf_l2mib_mch\000\000pipeline\000This event counts operations where software or hardware prefetch hits an L2 cache refill buffer allocated by demand access\000event=0x370\000\00000\000This event counts operations where software or hardware prefetch hits an L2 cache refill buffer allocated by demand access\000 */
{ 125067 }, /* l2_pipe_val\000\000pipeline\000This event counts valid cycles of L2 cache pipeline\000event=0x330\000\00000\000This event counts valid cycles of L2 cache pipeline\000 */
{ 177599 }, /* prx_val\000\000pipeline\000This event counts valid cycles of PRX pipeline\000event=0x1a6\000\00000\000This event counts valid cycles of PRX pipeline\000 */
{ 186072 }, /* stall_backend\000\000pipeline\000No operation issued due to the backend\000event=0x24\000\00000\000No operation issued due to the backend\000 */
{ 195875 }, /* stall_frontend\000\000pipeline\000No operation issued because of the frontend\000event=0x23\000\00000\000No operation issued because of the frontend\000 */
{ 22049 }, /* ase_sve_int_spec\000\000sve\000Advanced SIMD and SVE integer Operations speculatively executed\000event=0x8043\000\00000\000Advanced SIMD and SVE integer Operations speculatively executed\000 */
{ 22216 }, /* ase_sve_ld_multi_spec\000\000sve\000Advanced SIMD and SVE contiguous load multiple vector Operations speculatively executed\000event=0x80a5\000\00000\000Advanced SIMD and SVE contiguous load multiple vector Operations speculatively executed\000 */
{ 22436 }, /* ase_sve_ld_spec\000\000sve\000Advanced SIMD and SVE load Operations speculatively executed\000event=0x8085\000\00000\000Advanced SIMD and SVE load Operations speculatively executed\000 */
{ 22596 }, /* ase_sve_st_multi_spec\000\000sve\000Advanced SIMD and SVE contiguous store multiple vector Operations speculatively executed\000event=0x80a6\000\00000\000Advanced SIMD and SVE contiguous store multiple vector Operations speculatively executed\000 */
{ 22818 }, /* ase_sve_st_spec\000\000sve\000Advanced SIMD and SVE store Operations speculatively executed\000event=0x8086\000\00000\000Advanced SIMD and SVE store Operations speculatively executed\000 */
{ 23158 }, /* base_ld_reg_spec\000\000sve\000General-purpose register load Operations speculatively executed\000event=0x8089\000\00000\000General-purpose register load Operations speculatively executed\000 */
{ 23325 }, /* base_st_reg_spec\000\000sve\000General-purpose register store Operations speculatively executed\000event=0x808a\000\00000\000General-purpose register store Operations speculatively executed\000 */
{ 66492 }, /* fp_cvt_spec\000\000sve\000floating-point convert Operations speculatively executed\000event=0x8038\000\00000\000floating-point convert Operations speculatively executed\000 */
{ 66640 }, /* fp_dp_fixed_ops_spec\000\000sve\000Non-scalable double-precision floating-point element Operations speculatively executed\000event=0x80c7\000\00000\000Non-scalable double-precision floating-point element Operations speculatively executed\000 */
{ 66857 }, /* fp_dp_scale_ops_spec\000\000sve\000Scalable double-precision floating-point element Operations speculatively executed\000event=0x80c6\000\00000\000Scalable double-precision floating-point element Operations speculatively executed\000 */
{ 67427 }, /* fp_fixed_ops_spec\000\000sve\000Non-scalable floating-point element Operations speculatively executed\000event=0x80c1\000\00000\000Non-scalable floating-point element Operations speculatively executed\000 */
{ 67607 }, /* fp_fma_spec\000\000sve\000Floating-point FMA Operations speculatively executed\000event=0x8028\000\00000\000Floating-point FMA Operations speculatively executed\000 */
{ 67747 }, /* fp_hp_fixed_ops_spec\000\000sve\000Non-scalable half-precision floating-point element Operations speculatively executed\000event=0x80c3\000\00000\000Non-scalable half-precision floating-point element Operations speculatively executed\000 */
{ 67960 }, /* fp_hp_scale_ops_spec\000\000sve\000Scalable half-precision floating-point element Operations speculatively executed\000event=0x80c2\000\00000\000Scalable half-precision floating-point element Operations speculatively executed\000 */
{ 68752 }, /* fp_recpe_spec\000\000sve\000Floating-point reciprocal estimate Operations speculatively executed\000event=0x8034\000\00000\000Floating-point reciprocal estimate Operations speculatively executed\000 */
{ 69106 }, /* fp_scale_ops_spec\000\000sve\000Scalable floating-point element Operations speculatively executed\000event=0x80c0\000\00000\000Scalable floating-point element Operations speculatively executed\000 */
{ 69278 }, /* fp_sp_fixed_ops_spec\000\000sve\000Non-scalable single-precision floating-point element Operations speculatively executed\000event=0x80c5\000\00000\000Non-scalable single-precision floating-point element Operations speculatively executed\000 */
{ 69495 }, /* fp_sp_scale_ops_spec\000\000sve\000Scalable single-precision floating-point element Operations speculatively executed\000event=0x80c4\000\00000\000Scalable single-precision floating-point element Operations speculatively executed\000 */
{ 69877 }, /* fp_spec\000\000sve\000Floating-point Operations speculatively executed\000event=0x8010\000\00000\000Floating-point Operations speculatively executed\000 */
{ 177482 }, /* prf_spec\000\000sve\000Prefetch Operations speculatively executed\000event=0x8087\000\00000\000Prefetch Operations speculatively executed\000 */
{ 184373 }, /* simd_inst_retired\000\000sve\000SIMD Instruction architecturally executed\000event=0x8000\000\00000\000SIMD Instruction architecturally executed\000 */
{ 202198 }, /* sve_inst_retired\000\000sve\000Instruction architecturally executed, SVE\000event=0x8002\000\00000\000Instruction architecturally executed, SVE\000 */
{ 202441 }, /* sve_ld_gather_spec\000\000sve\000SVE gather-load Operations speculatively executed\000event=0x80ad\000\00000\000SVE gather-load Operations speculatively executed\000 */
{ 202942 }, /* sve_ldff_spec\000\000sve\000SVE First-fault load Operations speculatively executed\000event=0x80bc\000\00000\000SVE First-fault load Operations speculatively executed\000 */
{ 203088 }, /* sve_ldr_preg_spec\000\000sve\000SVE load predicate register Operations speculatively executed\000event=0x8095\000\00000\000SVE load predicate register Operations speculatively executed\000 */
{ 203252 }, /* sve_ldr_reg_spec\000\000sve\000SVE unpredicated load register Operations speculatively executed\000event=0x8091\000\00000\000SVE unpredicated load register Operations speculatively executed\000 */
{ 203421 }, /* sve_math_spec\000\000sve\000SVE Math accelerator Operations speculatively executed\000event=0x800e\000\00000\000SVE Math accelerator Operations speculatively executed\000 */
{ 203567 }, /* sve_movprfx_spec\000\000sve\000SVE MOVPRFX Operations speculatively executed\000event=0x807c\000\00000\000SVE MOVPRFX Operations speculatively executed\000 */
{ 203698 }, /* sve_movprfx_u_spec\000\000sve\000SVE MOVPRFX unfused Operations speculatively executed\000event=0x807f\000\00000\000SVE MOVPRFX unfused Operations speculatively executed\000 */
{ 204795 }, /* sve_pred_spec\000\000sve\000SVE predicated Operations speculatively executed\000event=0x8074\000\00000\000SVE predicated Operations speculatively executed\000 */
{ 204929 }, /* sve_prf_contig_spec\000\000sve\000SVE contiguous prefetch element Operations speculatively executed\000event=0x809f\000\00000\000SVE contiguous prefetch element Operations speculatively executed\000 */
{ 205103 }, /* sve_prf_gather_spec\000\000sve\000SVE gather-prefetch Operations speculatively executed\000event=0x80af\000\00000\000SVE gather-prefetch Operations speculatively executed\000 */
{ 205253 }, /* sve_st_scatter_spec\000\000sve\000SVE scatter-store Operations speculatively executed\000event=0x80ae\000\00000\000SVE scatter-store Operations speculatively executed\000 */
{ 205399 }, /* sve_str_preg_spec\000\000sve\000SVE store predicate register Operations speculatively executed\000event=0x8096\000\00000\000SVE store predicate register Operations speculatively executed\000 */
{ 205565 }, /* sve_str_reg_spec\000\000sve\000SVE unpredicated store register Operations speculatively executed\000event=0x8092\000\00000\000SVE unpredicated store register Operations speculatively executed\000 */
{ 213957 }, /* uop_spec\000\000sve\000Microarchitectural operation, Operations speculatively executed\000event=0x8008\000\00000\000Microarchitectural operation, Operations speculatively executed\000 */
};

static const struct compact_pmu_event pmu_events__hisilicon_hip08[] = {
{ 64235 }, /* exe_stall_cycle\000\000core imp def\000Cycles of that the number of issue ups are less than 4\000event=0x7001\000\00000\000Cycles of that the number of issuing micro operations are less than 4\000 */
{ 64624 }, /* fetch_bubble\000\000core imp def\000Instructions can receive, but not send\000event=0x2014\000\00000\000Instructions can receive, but not send\000 */
{ 71637 }, /* hit_on_prf\000\000core imp def\000Hit on prefetched data\000event=0x6014\000\00000\000Hit on prefetched data\000 */
{ 74684 }, /* if_is_stall\000\000core imp def\000Instruction fetch stall cycles\000event=0x1044\000\00000\000Instruction fetch stall cycles\000 */
{ 91469 }, /* iq_is_empty\000\000core imp def\000Instruction queue is empty\000event=0x1043\000\00000\000Instruction queue is empty\000 */
{ 101505 }, /* l1d_cache_inval\000\000core imp def\000L1D cache invalidate\000event=0x48\000\00000\000Attributable Level 1 data cache invalidate\000 */
{ 102796 }, /* l1d_cache_rd\000\000core imp def\000L1D cache access, read\000event=0x40\000\00000\000Attributable Level 1 data cache access, read\000 */
{ 106004 }, /* l1d_cache_refill_rd\000\000core imp def\000L1D cache refill, read\000event=0x42\000\00000\000Attributable Level 1 data cache refill, read\000 */
{ 106501 }, /* l1d_cache_refill_wr\000\000core imp def\000L1D cache refill, write\000event=0x43\000\00000\000Attributable Level 1 data cache refill, write\000 */
{ 108097 }, /* l1d_cache_wb_clean\000\000core imp def\000L1D cache Write-Back, cleaning and coherency\000event=0x47\000\00000\000Level 1 data cache Write-Back, cleaning and coherency\000 */
{ 108617 }, /* l1d_cache_wb_victim\000\000core imp def\000L1D cache Write-Back, victim\000event=0x46\000\00000\000Attributable Level 1 data cache Write-Back, victim\000 */
{ 109102 }, /* l1d_cache_wr\000\000core imp def\000L1D cache access, write\000event=0x41\000\00000\000Attributable Level 1 data cache access, write\000 */
{ 110856 }, /* l1d_tlb_rd\000\000core imp def\000L1D tlb access, read\000event=0x4e\000\00000\000Attributable Level 1 data or unified TLB access, read\000 */
{ 112303 }, /* l1d_tlb_refill_rd\000\000core imp def\000L1D tlb refill, read\000event=0x4c\000\00000\000Attributable Level 1 data TLB refill, read\000 */
{ 113199 }, /* l1d_tlb_refill_wr\000\000core imp def\000L1D tlb refill, write\000event=0x4d\000\00000\000Attributable Level 1 data TLB refill, write\000 */
{ 114103 }, /* l1d_tlb_wr\000\000core imp def\000L1D tlb access, write\000event=0x4f\000\00000\000Attributable Level 1 data or unified TLB access, write\000 */
{ 118139 }, /* l1i_cache_prf\000\000core imp def\000L1I cache prefetch access count\000event=0x102e\000\00000\000Level 1 instruction cache prefetch access count\000 */
{ 118264 }, /* l1i_cache_prf_refill\000\000core imp def\000L1I cache miss due to prefetch access count\000event=0x102f\000\00000\000Level 1 instruction cache miss due to prefetch access count\000 */
{ 127568 }, /* l2d_cache_inval\000\000core imp def\000L2D cache invalidate\000event=0x58\000\00000\000Attributable Level 2 data cache invalidate\000 */
{ 129229 }, /* l2d_cache_rd\000\000core imp def\000L2D cache access, read\000event=0x50\000\00000\000Attributable Level 2 data cache access, read\000 */
{ 132398 }, /* l2d_cache_refill_rd\000\000core imp def\000L2D cache refill, read\000event=0x52\000\00000\000Attributable Level 2 data cache refill, read\000 */
{ 132941 }, /* l2d_cache_refill_wr\000\000core imp def\000L2D cache refill, write\000event=0x53\000\00000\000Attributable Level 2 data cache refill, write\000 */
{ 137527 }, /* l2d_cache_wb_clean\000\000core imp def\000L2D cache Write-Back, cleaning and coherency\000event=0x57\000\00000\000Level 2 data cache Write-Back, cleaning and coherency\000 */
{ 138090 }, /* l2d_cache_wb_victim\000\000core imp def\000L2D cache Write-Back, victim\000event=0x56\000\00000\000Attributable Level 2 data cache Write-Back, victim\000 */
{ 138490 }, /* l2d_cache_wr\000\000core imp def\000L2D cache access, write\000event=0x51\000\00000\000Attributable Level 2 data cache access, write\000 */
{ 165045 }, /* mem_stall_anyload\000\000core imp def\000No any micro operation is issued and meanwhile any load operation is not resolved\000event=0x7004\000\00000\000No any micro operation is issued and meanwhile any load operation is not resolved\000 */
{ 165258 }, /* mem_stall_l1miss\000\000core imp def\000No any micro operation is issued and meanwhile there is any load operation missing L1 cache and pending data refill\000event=0x7006\000\00000\000No any micro operation is issued and meanwhile there is any load operation missing L1 cache and pending data refill\000 */
{ 165538 }, /* mem_stall_l2miss\000\000core imp def\000No any micro operation is issued and meanwhile there is any load operation missing both L1 and L2 cache and pending data refill from L3 cache\000event=0x7007\000\00000\000No any micro operation is issued and meanwhile there is any load operation missing both L1 and L2 cache and pending data refill from L3 cache\000 */
{ 177391 }, /* prf_req\000\000core imp def\000Prefetch request from LSU\000event=0x6013\000\00000\000Prefetch request from LSU\000 */
{ 20492 }, /* act_cmd\000hisi_sccl,ddrc\000uncore ddrc\000DDRC active commands. Unit: hisi_sccl,ddrc \000config=0x5\000\00000\000DDRC active commands\000 */
{ 66010 }, /* flux_rcmd\000hisi_sccl,ddrc\000uncore ddrc\000DDRC read commands. Unit: hisi_sccl,ddrc \000config=0x3\000\00000\000DDRC read commands\000 */
{ 66123 }, /* flux_rd\000hisi_sccl,ddrc\000uncore ddrc\000DDRC total read operations. Unit: hisi_sccl,ddrc \000config=0x1\000\00000\000DDRC total read operations\000 */
{ 66250 }, /* flux_wcmd\000hisi_sccl,ddrc\000uncore ddrc\000DDRC write commands. Unit: hisi_sccl,ddrc \000config=0x2\000\00000\000DDRC write commands\000 */
{ 66365 }, /* flux_wr\000hisi_sccl,ddrc\000uncore ddrc\000DDRC total write operations. Unit: hisi_sccl,ddrc \000config=0\000\00000\000DDRC total write operations\000 */
{ 176566 }, /* pre_cmd\000hisi_sccl,ddrc\000uncore ddrc\000DDRC precharge commands. Unit: hisi_sccl,ddrc \000config=0x4\000\00000\000DDRC precharge commands\000 */
{ 181011 }, /* rnk_chg\000hisi_sccl,ddrc\000uncore ddrc\000DDRC rank commands. Unit: hisi_sccl,ddrc \000config=0x6\000\00000\000DDRC rank commands\000 */
{ 181617 }, /* rw_chg\000hisi_sccl,ddrc\000uncore ddrc\000DDRC read and write changes. Unit: hisi_sccl,ddrc \000config=0x7\000\00000\000DDRC read and write changes\000 */
{ 23712 }, /* bi_num\000hisi_sccl,hha\000uncore hha\000Unit: hisi_sccl,hha \000config=0x23\000\00000\000\000 */
{ 54865 }, /* edir-hit\000hisi_sccl,hha\000uncore hha\000Unit: hisi_sccl,hha \000config=0x43\000\00000\000\000 */
{ 54937 }, /* edir-home-migrate\000hisi_sccl,hha\000uncore hha\000Unit: hisi_sccl,hha \000config=0x4d\000\00000\000\000 */
{ 55018 }, /* edir-lookup\000hisi_sccl,hha\000uncore hha\000Unit: hisi_sccl,hha \000config=0x41\000\00000\000\000 */
{ 162594 }, /* mediated_num\000hisi_sccl,hha\000uncore hha\000Unit: hisi_sccl,hha \000config=0x32\000\00000\000\000 */
{ 178738 }, /* rd_ddr_128b\000hisi_sccl,hha\000uncore hha\000The number of read operations sent by HHA to DDRC which size is 128 bytes. Unit: hisi_sccl,hha \000config=0x1e\000\00000\000The number of read operations sent by HHA to DDRC which size is 128 bytes\000 */
{ 178961 }, /* rd_ddr_64b\000hisi_sccl,hha\000uncore hha\000The number of read operations sent by HHA to DDRC which size is 64 bytes. Unit: hisi_sccl,hha \000config=0x1c\000\00000\000The number of read operations sent by HHA to DDRC which size is 64bytes\000 */
{ 181745 }, /* rx_ccix\000hisi_sccl,hha\000uncore hha\000Count of the number of operations that HHA has received from CCIX. Unit: hisi_sccl,hha \000config=0x3\000\00000\000Count of the number of operations that HHA has received from CCIX\000 */
{ 181947 }, /* rx_ops_num\000hisi_sccl,hha\000uncore hha\000The number of all operations received by the HHA. Unit: hisi_sccl,hha \000config=0\000\00000\000The number of all operations received by the HHA\000 */
{ 182116 }, /* rx_outer\000hisi_sccl,hha\000uncore hha\000The number of all operations received by the HHA from another socket. Unit: hisi_sccl,hha \000config=0x1\000\00000\000The number of all operations received by the HHA from another socket\000 */
{ 182325 }, /* rx_sccl\000hisi_sccl,hha\000uncore hha\000The number of all operations received by the HHA from another SCCL in this socket. Unit: hisi_sccl,hha \000config=0x2\000\00000\000The number of all operations received by the HHA from another SCCL in this socket\000 */
{ 182559 }, /* rx_snprsp_outer\000hisi_sccl,hha\000uncore hha\000Unit: hisi_sccl,hha \000config=0x3c\000\00000\000\000 */
{ 182638 }, /* rx_snprspdata\000hisi_sccl,hha\000uncore hha\000Unit: hisi_sccl,hha \000config=0x38\000\00000\000\000 */
{ 182715 }, /* rx_wbi\000hisi_sccl,hha\000uncore hha\000Unit: hisi_sccl,hha \000config=0x4\000\00000\000\000 */
{ 182784 }, /* rx_wbip\000hisi_sccl,hha\000uncore hha\000Unit: hisi_sccl,hha \000config=0x5\000\00000\000\000 */
{ 182854 }, /* rx_wtistash\000hisi_sccl,hha\000uncore hha\000Unit: hisi_sccl,hha \000config=0x11\000\00000\000\000 */
{ 183927 }, /* sdir-hit\000hisi_sccl,hha\000uncore hha\000Unit: hisi_sccl,hha \000config=0x42\000\00000\000\000 */
{ 183999 }, /* sdir-home-migrate\000hisi_sccl,hha\000uncore hha\000Unit: hisi_sccl,hha \000config=0x4c\000\00000\000\000 */
{ 184080 }, /* sdir-lookup\000hisi_sccl,hha\000uncore hha\000Unit: hisi_sccl,hha \000config=0x40\000\00000\000\000 */
{ 184696 }, /* spill_num\000hisi_sccl,hha\000uncore hha\000Count of the number of spill operations that the HHA has sent. Unit: hisi_sccl,hha \000config=0x20\000\00000\000Count of the number of spill operations that the HHA has sent\000 */
{ 184893 }, /* spill_success\000hisi_sccl,hha\000uncore hha\000Count of the number of successful spill operations that the HHA has sent. Unit: hisi_sccl,hha \000config=0x21\000\00000\000Count of the number of successful spill operations that the HHA has sent\000 */
{ 211187 }, /* tx_snp_ccix\000hisi_sccl,hha\000uncore hha\000Unit: hisi_sccl,hha \000config=0x35\000\00000\000\000 */
{ 211262 }, /* tx_snp_num\000hisi_sccl,hha\000uncore hha\000Unit: hisi_sccl,hha \000config=0x33\000\00000\000\000 */
{ 211336 }, /* tx_snp_outer\000hisi_sccl,hha\000uncore hha\000Unit: hisi_sccl,hha \000config=0x34\000\00000\000\000 */
{ 215414 }, /* wr_ddr_128b\000hisi_sccl,hha\000uncore hha\000The number of write operations sent by HHA to DDRC which size is 128 bytes. Unit: hisi_sccl,hha \000config=0x1f\000\00000\000The number of write operations sent by HHA to DDRC which size is 128 bytes\000 */
{ 215639 }, /* wr_ddr_64b\000hisi_sccl,hha\000uncore hha\000The number of write operations sent by HHA to DDRC which size is 64 bytes. Unit: hisi_sccl,hha \000config=0x1d\000\00000\000The number of write operations sent by HHA to DDRC which size is 64 bytes\000 */
{ 22980 }, /* back_invalid\000hisi_sccl,l3c\000uncore l3c\000Count of the number of L3C back invalid operations. Unit: hisi_sccl,l3c \000config=0x29\000\00000\000Count of the number of L3C back invalid operations\000 */
{ 177058 }, /* prefetch_drop\000hisi_sccl,l3c\000uncore l3c\000Count of the number of prefetch drops from this L3C. Unit: hisi_sccl,l3c \000config=0x42\000\00000\000Count of the number of prefetch drops from this L3C\000 */
{ 178629 }, /* rd_cpipe\000hisi_sccl,l3c\000uncore l3c\000Total read accesses. Unit: hisi_sccl,l3c \000config=0\000\00000\000Total read accesses\000 */
{ 179180 }, /* rd_hit_cpipe\000hisi_sccl,l3c\000uncore l3c\000Total read hits. Unit: hisi_sccl,l3c \000config=0x2\000\00000\000Total read hits\000 */
{ 179287 }, /* rd_hit_spipe\000hisi_sccl,l3c\000uncore l3c\000Count of the number of read lines that hits in spipe of this L3C. Unit: hisi_sccl,l3c \000config=0x22\000\00000\000Count of the number of read lines that hits in spipe of this L3C\000 */
{ 179493 }, /* rd_spipe\000hisi_sccl,l3c\000uncore l3c\000Count of the number of read lines that come from this cluster of CPU core in spipe. Unit: hisi_sccl,l3c \000config=0x20\000\00000\000Count of the number of read lines that come from this cluster of CPU core in spipe\000 */
{ 180590 }, /* retry_cpu\000hisi_sccl,l3c\000uncore l3c\000Count of the number of retry that L3C suppresses the CPU operations. Unit: hisi_sccl,l3c \000config=0x40\000\00000\000Count of the number of retry that L3C suppresses the CPU operations\000 */
{ 180799 }, /* retry_ring\000hisi_sccl,l3c\000uncore l3c\000Count of the number of retry that L3C suppresses the ring operations. Unit: hisi_sccl,l3c \000config=0x41\000\00000\000Count of the number of retry that L3C suppresses the ring operations\000 */
{ 214768 }, /* victim_num\000hisi_sccl,l3c\000uncore l3c\000l3c precharge commands. Unit: hisi_sccl,l3c \000config=0x4\000\00000\000l3c precharge commands\000 */
{ 215301 }, /* wr_cpipe\000hisi_sccl,l3c\000uncore l3c\000Total write accesses. Unit: hisi_sccl,l3c \000config=0x1\000\00000\000Total write accesses\000 */
{ 215861 }, /* wr_hit_cpipe\000hisi_sccl,l3c\000uncore l3c\000Total write hits. Unit: hisi_sccl,l3c \000config=0x3\000\00000\000Total write hits\000 */
{ 215970 }, /* wr_hit_spipe\000hisi_sccl,l3c\000uncore l3c\000Count of the number of write lines that hits in spipe of this L3C. Unit: hisi_sccl,l3c \000config=0x23\000\00000\000Count of the number of write lines that hits in spipe of this L3C\000 */
{ 216178 }, /* wr_spipe\000hisi_sccl,l3c\000uncore l3c\000Count of the number of write lines that come from this cluster of CPU core in spipe. Unit: hisi_sccl,l3c \000config=0x21\000\00000\000Count of the number of write lines that come from this cluster of CPU core in spipe\000 */
};

static const struct compact_pmu_event pmu_metrics__hisilicon_hip08[] = {
{ 1009 }, /* \000backend_bound\000Default;TopDownL1\0001 - (frontend_bound + bad_speculation + retiring)\000\000Backend Bound L1 topdown metric\000Backend Bound L1 topdown metric\000\000\000\000TopDownL1\00000 */
{ 1524 }, /* \000bad_speculation\000Default;TopDownL1\000(INST_SPEC - INST_RETIRED) / (4 * CPU_CYCLES)\000\000Bad Speculation L1 topdown metric\000Bad Speculation L1 topdown metric\000\000\000\000TopDownL1\00000 */
{ 1894 }, /* \000bp_misp_flush\000TopDownL3\000BR_MIS_PRED * 5 / CPU_CYCLES\000\000BP misp flush L3 topdown metric\000BP misp flush L3 topdown metric\000\000\000\000\00000 */
{ 2702 }, /* \000branch_mispredicts\000TopDownL2\000bad_speculation * BR_MIS_PRED / (BR_MIS_PRED + armv8_pmuv3_0@event\\=0x2013@)\000\000Branch mispredicts L2 topdown metric\000Branch mispredicts L2 topdown metric\000\000\000\000\00000 */
{ 3923 }, /* \000core_bound\000TopDownL2\000(EXE_STALL_CYCLE - (MEM_STALL_ANYLOAD + armv8_pmuv3_0@event\\=0x7005@)) / CPU_CYCLES\000\000Core bound L2 topdown metric\000Core bound L2 topdown metric\000\000\000\000\00000 */
{ 4980 }, /* \000divider\000TopDownL3\000armv8_pmuv3_0@event\\=0x7002@ / CPU_CYCLES\000\000Divider L3 topdown metric\000Divider L3 topdown metric\000\000\000\000\00000 */
{ 5750 }, /* \000exe_ports_util\000TopDownL3\000core_bound - divider - fsu_stall\000\000EXE ports util L3 topdown metric\000EXE ports util L3 topdown metric\000\000\000\000\00000 */
{ 5882 }, /* \000fetch_bandwidth_bound\000TopDownL2\000frontend_bound - fetch_latency_bound\000\000Fetch bandwidth bound L2 topdown metric\000Fetch bandwidth bound L2 topdown metric\000\000\000\000\00000 */
{ 6039 }, /* \000fetch_latency_bound\000TopDownL2\000armv8_pmuv3_0@event\\=0x201d@ / CPU_CYCLES\000\000Fetch latency bound L2 topdown metric\000Fetch latency bound L2 topdown metric\000\000\000\000\00000 */
{ 6375 }, /* \000frontend_bound\000Default;TopDownL1\000FETCH_BUBBLE / (4 * CPU_CYCLES)\000\000Frontend bound L1 topdown metric\000Frontend bound L1 topdown metric\000\000\000\000TopDownL1\00000 */
{ 6935 }, /* \000fsu_stall\000TopDownL3\000armv8_pmuv3_0@event\\=0x7003@ / CPU_CYCLES\000\000FSU stall L3 topdown metric\000FSU stall L3 topdown metric\000\000\000\000\00000 */
{ 7129 }, /* \000idle_by_icache_miss\000TopDownL3\000((L2I_CACHE - L2I_CACHE_REFILL) * 15 + L2I_CACHE_REFILL * 100) / CPU_CYCLES\000\000Idle by icache miss L3 topdown metric\000Idle by icache miss L3 topdown metric\000\000\000\000\00000 */
{ 7319 }, /* \000idle_by_itlb_miss\000TopDownL3\000((L2I_TLB - L2I_TLB_REFILL) * 15 + L2I_TLB_REFILL * 100) / CPU_CYCLES\000\000Idle by itlb miss L3 topdown metric\000Idle by itlb miss L3 topdown metric\000\000\000\000\00000 */
{ 7497 }, /* \000indirect_branch\000TopDownL3\000armv8_pmuv3_0@event\\=0x1010@ / BR_MIS_PRED\000\000Indirect branch L3 topdown metric\000Indirect branch L3 topdown metric\000\000\000\000\00000 */
{ 8837 }, /* \000l1_bound\000TopDownL3\000(MEM_STALL_ANYLOAD - MEM_STALL_L1MISS) / CPU_CYCLES\000\000L1 bound L3 topdown metric\000L1 bound L3 topdown metric\000\000\000\000\00000 */
{ 11727 }, /* \000l2_bound\000TopDownL3\000(MEM_STALL_L1MISS - MEM_STALL_L2MISS) / CPU_CYCLES\000\000L2 bound L3 topdown metric\000L2 bound L3 topdown metric\000\000\000\000\00000 */
{ 15650 }, /* \000machine_clears\000TopDownL2\000bad_speculation - branch_mispredicts\000\000Machine clears L2 topdown metric\000Machine clears L2 topdown metric\000\000\000\000\00000 */
{ 15786 }, /* \000mem_bound\000TopDownL3\000MEM_STALL_L2MISS / CPU_CYCLES\000\000Mem bound L3 topdown metric\000Mem bound L3 topdown metric\000\000\000\000\00000 */
{ 15900 }, /* \000memory_bound\000TopDownL2\000(MEM_STALL_ANYLOAD + armv8_pmuv3_0@event\\=0x7005@) / CPU_CYCLES\000\000Memory bound L2 topdown metric\000Memory bound L2 topdown metric\000\000\000\000\00000 */
{ 16057 }, /* \000nuke_flush\000TopDownL3\000armv8_pmuv3_0@event\\=0x2012@ / armv8_pmuv3_0@event\\=0x2013@\000\000Nuke flush L3 topdown metric\000Nuke flush L3 topdown metric\000\000\000\000\00000 */
{ 16204 }, /* \000ooo_flush\000TopDownL3\000armv8_pmuv3_0@event\\=0x2013@ * 5 / CPU_CYCLES\000\000OOO flush L3 topdown metric\000OOO flush L3 topdown metric\000\000\000\000\00000 */
{ 16334 }, /* \000other_branch\000TopDownL3\000(BR_MIS_PRED - armv8_pmuv3_0@event\\=0x1010@ - armv8_pmuv3_0@event\\=0x1013@ - armv8_pmuv3_0@event\\=0x1016@ - armv8_pmuv3_0@event\\=0x100d@) / BR_MIS_PRED\000\000Other branch L3 topdown metric\000Other branch L3 topdown metric\000\000\000\000\00000 */
{ 16579 }, /* \000other_flush\000TopDownL3\0001 - nuke_flush\000\000Other flush L3 topdown metric\000Other flush L3 topdown metric\000\000\000\000\00000 */
{ 16684 }, /* \000pc_buffer_stall\000TopDownL3\000armv8_pmuv3_0@event\\=0x2005@ / CPU_CYCLES\000\000PC buffer stall L3 topdown metric\000PC buffer stall L3 topdown metric\000\000\000\000\00000 */
{ 16828 }, /* \000pop_branch\000TopDownL3\000armv8_pmuv3_0@event\\=0x100d@ / BR_MIS_PRED\000\000Pop branch L3 topdown metric\000Pop branch L3 topdown metric\000\000\000\000\00000 */
{ 16958 }, /* \000ptag_stall\000TopDownL3\000(armv8_pmuv3_0@event\\=0x2006@ + armv8_pmuv3_0@event\\=0x2007@ + armv8_pmuv3_0@event\\=0x2008@) / CPU_CYCLES\000\000Ptag stall L3 topdown metric\000Ptag stall L3 topdown metric\000\000\000\000\00000 */
{ 17151 }, /* \000push_branch\000TopDownL3\000(armv8_pmuv3_0@event\\=0x1013@ + armv8_pmuv3_0@event\\=0x1016@) / BR_MIS_PRED\000\000Push branch L3 topdown metric\000Push branch L3 topdown metric\000\000\000\000\00000 */
{ 17568 }, /* \000retiring\000Default;TopDownL1\000INST_RETIRED / (CPU_CYCLES * 4)\000\000Retiring L1 topdown metric\000Retiring L1 topdown metric\000\000\000\000TopDownL1\00000 */
{ 17883 }, /* \000rob_stall\000TopDownL3\000armv8_pmuv3_0@event\\=0x2004@ / CPU_CYCLES\000\000Rob stall L3 topdown metric\000Rob stall L3 topdown metric\000\000\000\000\00000 */
{ 18009 }, /* \000saveopq_stall\000TopDownL3\000armv8_pmuv3_0@event\\=0x201e@ / CPU_CYCLES\000\000SaveOpQ stall L3 topdown metric\000SaveOpQ stall L3 topdown metric\000\000\000\000\00000 */
{ 18540 }, /* \000sp_flush\000TopDownL3\000armv8_pmuv3_0@event\\=0x1001@ * 5 / CPU_CYCLES\000\000Static predictor flush L3 topdown metric\000Static predictor flush L3 topdown metric\000\000\000\000\00000 */
{ 18802 }, /* \000store_bound\000TopDownL3\000armv8_pmuv3_0@event\\=0x7005@ / CPU_CYCLES\000\000Store bound L3 topdown metric\000Store bound L3 topdown metric\000\000\000\000\00000 */
{ 19277 }, /* \000sync_stall\000TopDownL3\000armv8_pmuv3_0@event\\=0x2010@ / CPU_CYCLES\000\000Sync stall L3 topdown metric\000Sync stall L3 topdown metric\000\000\000\000\00000 */
};

static const struct compact_pmu_event pmu_events__hisilicon_hip09_sys[] = {
{ 46710 }, /* cpa_cycles\000hisi_sicl,cpa\000uncore cpa\000count of CPA cycles. Unit: hisi_sicl,cpa \000config=0\0000x00000030\00000\000\000 */
{ 46812 }, /* cpa_p0_rd_dat\000hisi_sicl,cpa\000uncore cpa\000Number of read ops transmitted by the P0 port. Unit: hisi_sicl,cpa \000config=0xe2\0000x00000030\00000\000\000 */
{ 46946 }, /* cpa_p0_rd_dat_32b\000hisi_sicl,cpa\000uncore cpa\000Number of read ops transmitted by the P0 port which size is 32 bytes. Unit: hisi_sicl,cpa \000config=0x84\0000x00000030\00000\000\000 */
{ 47107 }, /* cpa_p0_rd_dat_64b\000hisi_sicl,cpa\000uncore cpa\000Number of read ops transmitted by the P0 port which size is 64 bytes. Unit: hisi_sicl,cpa \000config=0x83\0000x00000030\00000\000\000 */
{ 47268 }, /* cpa_p0_wr_dat\000hisi_sicl,cpa\000uncore cpa\000Number of write ops transmitted by the P0 port. Unit: hisi_sicl,cpa \000config=0xe1\0000x00000030\00000\000\000 */
{ 47403 }, /* cpa_p1_rd_dat\000hisi_sicl,cpa\000uncore cpa\000Number of read ops transmitted by the P1 port. Unit: hisi_sicl,cpa \000config=0x62\0000x00000030\00000\000\000 */
{ 47537 }, /* cpa_p1_rd_dat_32b\000hisi_sicl,cpa\000uncore cpa\000Number of read ops transmitted by the P1 port which size is 32 bytes. Unit: hisi_sicl,cpa \000config=0x4\0000x00000030\00000\000\000 */
{ 47697 }, /* cpa_p1_rd_dat_64b\000hisi_sicl,cpa\000uncore cpa\000Number of read ops transmitted by the P1 port which size is 64 bytes. Unit: hisi_sicl,cpa \000config=0x3\0000x00000030\00000\000\000 */
{ 47857 }, /* cpa_p1_wr_dat\000hisi_sicl,cpa\000uncore cpa\000Number of write ops transmitted by the P1 port. Unit: hisi_sicl,cpa \000config=0x61\0000x00000030\00000\000\000 */
};

static const struct compact_pmu_event pmu_metrics__hisilicon_hip09_sys[] = {
{ 71261 }, /* hisi_sicl,cpa\000cpa_p0_avg_bw\000CPA\000(cpa_p0_wr_dat * 64 + cpa_p0_rd_dat_64b * 64 + cpa_p0_rd_dat_32b * 32) / cpa_cycles\000\000Average bandwidth of CPA Port 0. Unit: hisi_sicl,cpa \000\000\0000x00000030\000\000\00000 */
{ 71449 }, /* hisi_sicl,cpa\000cpa_p1_avg_bw\000CPA\000(cpa_p1_wr_dat * 64 + cpa_p1_rd_dat_64b * 64 + cpa_p1_rd_dat_32b * 32) / cpa_cycles\000\000Average bandwidth of CPA Port 1. Unit: hisi_sicl,cpa \000\000\0000x00000030\000\000\00000 */
};

static const struct compact_pmu_event pmu_events__test_soc_cpu[] = {
{ 24007 }, /* bp_l1_btb_correct\000\000branch\000L1 BTB Correction\000event=0x8a\000\00000\000\000 */
{ 24067 }, /* bp_l2_btb_correct\000\000branch\000L2 BTB Correction\000event=0x8b\000\00000\000\000 */
{ 149495 }, /* l3_cache_rd\000\000cache\000L3 cache access, read\000event=0x40\000\00000\000Attributable Level 3 cache access, read\000 */
{ 50900 }, /* dispatch_blocked.any\000\000other\000Memory cluster signals to block micro-op dispatch for any reason\000event=0x9,period=200000,umask=0x20\000\00000\000\000 */
{ 55324 }, /* eist_trans\000\000other\000Number of Enhanced Intel SpeedStep(R) Technology (EIST) transitions\000event=0x3a,period=200000,umask=0x0\000\00000\000\000 */
{ 184155 }, /* segment_reg_loads.any\000\000other\000Number of segment register loads\000event=0x6,period=200000,umask=0x80\000\00000\000\000 */
{ 55960 }, /* event-hyphen\000uncore_cbox\000uncore\000UNC_CBO_HYPHEN. Unit: uncore_cbox \000event=0xe0,umask=0x00\000\00000\000UNC_CBO_HYPHEN\000 */
{ 56068 }, /* event-two-hyph\000uncore_cbox\000uncore\000UNC_CBO_TWO_HYPH. Unit: uncore_cbox \000event=0xc0,umask=0x00\000\00000\000UNC_CBO_TWO_HYPH\000 */
{ 213002 }, /* unc_cbo_xsnp_response.miss_eviction\000uncore_cbox\000uncore\000A cross-core snoop resulted from L3 Eviction which misses in some processor core. Unit: uncore_cbox \000event=0x22,umask=0x81\000\00000\000A cross-core snoop resulted from L3 Eviction which misses in some processor core\000 */
{ 213265 }, /* uncore_hisi_ddrc.flux_wcmd\000hisi_sccl,ddrc\000uncore\000DDRC write commands. Unit: hisi_sccl,ddrc \000event=0x2\000\00000\000DDRC write commands\000 */
{ 213391 }, /* uncore_hisi_l3c.rd_hit_cpipe\000hisi_sccl,l3c\000uncore\000Total read hits. Unit: hisi_sccl,l3c \000event=0x7\000\00000\000Total read hits\000 */
{ 213509 }, /* uncore_imc.cache_hits\000uncore_imc\000uncore\000Total cache hits. Unit: uncore_imc \000event=0x34\000\00000\000Total cache hits\000 */
{ 213617 }, /* uncore_imc_free_running.cache_miss\000uncore_imc_free_running\000uncore\000Total cache misses. Unit: uncore_imc_free_running \000event=0x12\000\00000\000Total cache misses\000 */
};

static const struct compact_pmu_event pmu_metrics__test_soc_cpu[] = {
{ 0 }, /* \000CPI\000\0001 / IPC\000\000\000\000\000\000\000\00000 */
{ 23 }, /* \000DCache_L2_All\000\000DCache_L2_All_Hits + DCache_L2_All_Miss\000\000\000\000\000\000\000\00000 */
{ 88 }, /* \000DCache_L2_All_Hits\000\000l2_rqsts.demand_data_rd_hit + l2_rqsts.pf_hit + l2_rqsts.rfo_hit\000\000\000\000\000\000\000\00000 */
{ 183 }, /* \000DCache_L2_All_Miss\000\000max(l2_rqsts.all_demand_data_rd - l2_rqsts.demand_data_rd_hit, 0) + l2_rqsts.pf_miss + l2_rqsts.rfo_miss\000\000\000\000\000\000\000\00000 */
{ 318 }, /* \000DCache_L2_Hits\000\000d_ratio(DCache_L2_All_Hits, DCache_L2_All)\000\000\000\000\000\000\000\00000 */
{ 387 }, /* \000DCache_L2_Misses\000\000d_ratio(DCache_L2_All_Miss, DCache_L2_All)\000\000\000\000\000\000\000\00000 */
{ 458 }, /* \000Frontend_Bound_SMT\000\000idq_uops_not_delivered.core / (4 * (cpu_clk_unhalted.thread / 2 * (1 + cpu_clk_unhalted.one_thread_active / cpu_clk_unhalted.ref_xclk)))\000\000\000\000\000\000\000\00000 */
{ 625 }, /* \000IPC\000group1\000inst_retired.any / cpu_clk_unhalted.thread\000\000\000\000\000\000\000\00000 */
{ 689 }, /* \000L1D_Cache_Fill_BW\000\00064 * l1d.replacement / 1e9 / duration_time\000\000\000\000\000\000\000\00000 */
{ 761 }, /* \000M1\000\000ipc + M2\000\000\000\000\000\000\000\00000 */
{ 784 }, /* \000M2\000\000ipc + M1\000\000\000\000\000\000\000\00000 */
{ 807 }, /* \000M3\000\0001 / M3\000\000\000\000\000\000\000\00000 */
{ 3851 }, /* \000cache_miss_cycles\000group1\000dcache_miss_cpi + icache_miss_cycles\000\000\000\000\000\000\000\00000 */
{ 4915 }, /* \000dcache_miss_cpi\000\000l1d\\-loads\\-misses / inst_retired.any\000\000\000\000\000\000\000\00000 */
{ 7061 }, /* \000icache_miss_cycles\000\000l1i\\-loads\\-misses / inst_retired.any\000\000\000\000\000\000\000\00000 */
};

static const struct compact_pmu_event pmu_events__test_soc_sys[] = {
{ 206871 }, /* sys_ccn_pmu.read_cycles\000uncore_sys_ccn_pmu\000uncore\000ccn read-cycles event. Unit: uncore_sys_ccn_pmu \000config=0x2c\0000x01\00000\000\000 */
{ 206991 }, /* sys_ddr_pmu.write_cycles\000uncore_sys_ddr_pmu\000uncore\000ddr write-cycles event. Unit: uncore_sys_ddr_pmu \000event=0x2b\000v8\00000\000\000 */
};


/* Struct used to make the PMU event table implementation opaque to callers. */
struct pmu_events_table {
        const struct compact_pmu_event *entries;
        size_t length;
};

/* Struct used to make the PMU metric table implementation opaque to callers. */
struct pmu_metrics_table {
        const struct compact_pmu_event *entries;
        size_t length;
};

/*
 * Map a CPU to its table of PMU events. The CPU is identified by the
 * cpuid field, which is an arch-specific identifier for the CPU.
 * The identifier specified in tools/perf/pmu-events/arch/xxx/mapfile
 * must match the get_cpuid_str() in tools/perf/arch/xxx/util/header.c)
 *
 * The  cpuid can contain any character other than the comma.
 */
struct pmu_events_map {
        const char *arch;
        const char *cpuid;
        struct pmu_events_table event_table;
        struct pmu_metrics_table metric_table;
};

/*
 * Global table mapping each known CPU for the architecture to its
 * table of PMU events.
 */
const struct pmu_events_map pmu_events_map[] = {
{
	.arch = "arm64",
	.cpuid = "0x00000000410fd020",
	.event_table = {
		.entries = pmu_events__arm_cortex_a34,
		.length = ARRAY_SIZE(pmu_events__arm_cortex_a34)
	},
	.metric_table = {
		.entries = NULL,
		.length = 0
	}
},
{
	.arch = "arm64",
	.cpuid = "0x00000000410fd030",
	.event_table = {
		.entries = pmu_events__arm_cortex_a53,
		.length = ARRAY_SIZE(pmu_events__arm_cortex_a53)
	},
	.metric_table = {
		.entries = NULL,
		.length = 0
	}
},
{
	.arch = "arm64",
	.cpuid = "0x00000000420f1000",
	.event_table = {
		.entries = pmu_events__arm_cortex_a53,
		.length = ARRAY_SIZE(pmu_events__arm_cortex_a53)
	},
	.metric_table = {
		.entries = NULL,
		.length = 0
	}
},
{
	.arch = "arm64",
	.cpuid = "0x00000000410fd040",
	.event_table = {
		.entries = pmu_events__arm_cortex_a35,
		.length = ARRAY_SIZE(pmu_events__arm_cortex_a35)
	},
	.metric_table = {
		.entries = NULL,
		.length = 0
	}
},
{
	.arch = "arm64",
	.cpuid = "0x00000000410fd050",
	.event_table = {
		.entries = pmu_events__arm_cortex_a55,
		.length = ARRAY_SIZE(pmu_events__arm_cortex_a55)
	},
	.metric_table = {
		.entries = NULL,
		.length = 0
	}
},
{
	.arch = "arm64",
	.cpuid = "0x00000000410fd060",
	.event_table = {
		.entries = pmu_events__arm_cortex_a65_e1,
		.length = ARRAY_SIZE(pmu_events__arm_cortex_a65_e1)
	},
	.metric_table = {
		.entries = NULL,
		.length = 0
	}
},
{
	.arch = "arm64",
	.cpuid = "0x00000000410fd4a0",
	.event_table = {
		.entries = pmu_events__arm_cortex_a65_e1,
		.length = ARRAY_SIZE(pmu_events__arm_cortex_a65_e1)
	},
	.metric_table = {
		.entries = NULL,
		.length = 0
	}
},
{
	.arch = "arm64",
	.cpuid = "0x00000000410fd070",
	.event_table = {
		.entries = pmu_events__arm_cortex_a57_a72,
		.length = ARRAY_SIZE(pmu_events__arm_cortex_a57_a72)
	},
	.metric_table = {
		.entries = NULL,
		.length = 0
	}
},
{
	.arch = "arm64",
	.cpuid = "0x00000000410fd080",
	.event_table = {
		.entries = pmu_events__arm_cortex_a57_a72,
		.length = ARRAY_SIZE(pmu_events__arm_cortex_a57_a72)
	},
	.metric_table = {
		.entries = NULL,
		.length = 0
	}
},
{
	.arch = "arm64",
	.cpuid = "0x00000000410fd090",
	.event_table = {
		.entries = pmu_events__arm_cortex_a73,
		.length = ARRAY_SIZE(pmu_events__arm_cortex_a73)
	},
	.metric_table = {
		.entries = NULL,
		.length = 0
	}
},
{
	.arch = "arm64",
	.cpuid = "0x00000000410fd0a0",
	.event_table = {
		.entries = pmu_events__arm_cortex_a75,
		.length = ARRAY_SIZE(pmu_events__arm_cortex_a75)
	},
	.metric_table = {
		.entries = NULL,
		.length = 0
	}
},
{
	.arch = "arm64",
	.cpuid = "0x00000000410fd0b0",
	.event_table = {
		.entries = pmu_events__arm_cortex_a76,
		.length = ARRAY_SIZE(pmu_events__arm_cortex_a76)
	},
	.metric_table = {
		.entries = NULL,
		.length = 0
	}
},
{
	.arch = "arm64",
	.cpuid = "0x00000000410fd0c0",
	.event_table = {
		.entries = pmu_events__arm_neoverse_n1,
		.length = ARRAY_SIZE(pmu_events__arm_neoverse_n1)
	},
	.metric_table = {
		.entries = pmu_metrics__arm_neoverse_n1,
		.length = ARRAY_SIZE(pmu_metrics__arm_neoverse_n1)
	}
},
{
	.arch = "arm64",
	.cpuid = "0x00000000410fd0d0",
	.event_table = {
		.entries = pmu_events__arm_cortex_a77,
		.length = ARRAY_SIZE(pmu_events__arm_cortex_a77)
	},
	.metric_table = {
		.entries = NULL,
		.length = 0
	}
},
{
	.arch = "arm64",
	.cpuid = "0x00000000410fd400",
	.event_table = {
		.entries = pmu_events__arm_neoverse_v1,
		.length = ARRAY_SIZE(pmu_events__arm_neoverse_v1)
	},
	.metric_table = {
		.entries = NULL,
		.length = 0
	}
},
{
	.arch = "arm64",
	.cpuid = "0x00000000410fd410",
	.event_table = {
		.entries = pmu_events__arm_cortex_a78,
		.length = ARRAY_SIZE(pmu_events__arm_cortex_a78)
	},
	.metric_table = {
		.entries = NULL,
		.length = 0
	}
},
{
	.arch = "arm64",
	.cpuid = "0x00000000410fd4b0",
	.event_table = {
		.entries = pmu_events__arm_cortex_a78,
		.length = ARRAY_SIZE(pmu_events__arm_cortex_a78)
	},
	.metric_table = {
		.entries = NULL,
		.length = 0
	}
},
{
	.arch = "arm64",
	.cpuid = "0x00000000410fd440",
	.event_table = {
		.entries = pmu_events__arm_cortex_x1,
		.length = ARRAY_SIZE(pmu_events__arm_cortex_x1)
	},
	.metric_table = {
		.entries = NULL,
		.length = 0
	}
},
{
	.arch = "arm64",
	.cpuid = "0x00000000410fd4c0",
	.event_table = {
		.entries = pmu_events__arm_cortex_x1,
		.length = ARRAY_SIZE(pmu_events__arm_cortex_x1)
	},
	.metric_table = {
		.entries = NULL,
		.length = 0
	}
},
{
	.arch = "arm64",
	.cpuid = "0x00000000410fd460",
	.event_table = {
		.entries = pmu_events__arm_cortex_a510,
		.length = ARRAY_SIZE(pmu_events__arm_cortex_a510)
	},
	.metric_table = {
		.entries = NULL,
		.length = 0
	}
},
{
	.arch = "arm64",
	.cpuid = "0x00000000410fd470",
	.event_table = {
		.entries = pmu_events__arm_cortex_a710,
		.length = ARRAY_SIZE(pmu_events__arm_cortex_a710)
	},
	.metric_table = {
		.entries = NULL,
		.length = 0
	}
},
{
	.arch = "arm64",
	.cpuid = "0x00000000410fd480",
	.event_table = {
		.entries = pmu_events__arm_cortex_x2,
		.length = ARRAY_SIZE(pmu_events__arm_cortex_x2)
	},
	.metric_table = {
		.entries = NULL,
		.length = 0
	}
},
{
	.arch = "arm64",
	.cpuid = "0x00000000410fd490",
	.event_table = {
		.entries = pmu_events__arm_neoverse_n2_v2,
		.length = ARRAY_SIZE(pmu_events__arm_neoverse_n2_v2)
	},
	.metric_table = {
		.entries = pmu_metrics__arm_neoverse_n2_v2,
		.length = ARRAY_SIZE(pmu_metrics__arm_neoverse_n2_v2)
	}
},
{
	.arch = "arm64",
	.cpuid = "0x00000000410fd4f0",
	.event_table = {
		.entries = pmu_events__arm_neoverse_n2_v2,
		.length = ARRAY_SIZE(pmu_events__arm_neoverse_n2_v2)
	},
	.metric_table = {
		.entries = pmu_metrics__arm_neoverse_n2_v2,
		.length = ARRAY_SIZE(pmu_metrics__arm_neoverse_n2_v2)
	}
},
{
	.arch = "arm64",
	.cpuid = "0x00000000420f5160",
	.event_table = {
		.entries = pmu_events__cavium_thunderx2,
		.length = ARRAY_SIZE(pmu_events__cavium_thunderx2)
	},
	.metric_table = {
		.entries = NULL,
		.length = 0
	}
},
{
	.arch = "arm64",
	.cpuid = "0x00000000430f0af0",
	.event_table = {
		.entries = pmu_events__cavium_thunderx2,
		.length = ARRAY_SIZE(pmu_events__cavium_thunderx2)
	},
	.metric_table = {
		.entries = NULL,
		.length = 0
	}
},
{
	.arch = "arm64",
	.cpuid = "0x00000000460f0010",
	.event_table = {
		.entries = pmu_events__fujitsu_a64fx,
		.length = ARRAY_SIZE(pmu_events__fujitsu_a64fx)
	},
	.metric_table = {
		.entries = NULL,
		.length = 0
	}
},
{
	.arch = "arm64",
	.cpuid = "0x00000000480fd010",
	.event_table = {
		.entries = pmu_events__hisilicon_hip08,
		.length = ARRAY_SIZE(pmu_events__hisilicon_hip08)
	},
	.metric_table = {
		.entries = pmu_metrics__hisilicon_hip08,
		.length = ARRAY_SIZE(pmu_metrics__hisilicon_hip08)
	}
},
{
	.arch = "arm64",
	.cpuid = "0x00000000500f0000",
	.event_table = {
		.entries = pmu_events__ampere_emag,
		.length = ARRAY_SIZE(pmu_events__ampere_emag)
	},
	.metric_table = {
		.entries = NULL,
		.length = 0
	}
},
{
	.arch = "arm64",
	.cpuid = "0x00000000c00fac30",
	.event_table = {
		.entries = pmu_events__ampere_ampereone,
		.length = ARRAY_SIZE(pmu_events__ampere_ampereone)
	},
	.metric_table = {
		.entries = NULL,
		.length = 0
	}
},
{
	.arch = "testarch",
	.cpuid = "testcpu",
	.event_table = {
		.entries = pmu_events__test_soc_cpu,
		.length = ARRAY_SIZE(pmu_events__test_soc_cpu),
	},
	.metric_table = {
		.entries = pmu_metrics__test_soc_cpu,
		.length = ARRAY_SIZE(pmu_metrics__test_soc_cpu),
	}
},
{
	.arch = 0,
	.cpuid = 0,
	.event_table = { 0, 0 },
	.metric_table = { 0, 0 },
}
};

struct pmu_sys_events {
	const char *name;
	struct pmu_events_table event_table;
	struct pmu_metrics_table metric_table;
};

static const struct pmu_sys_events pmu_sys_event_tables[] = {
	{
		.event_table = {
			.entries = pmu_events__freescale_imx8mm_sys,
			.length = ARRAY_SIZE(pmu_events__freescale_imx8mm_sys)
		},
		.metric_table = {
			.entries = pmu_metrics__freescale_imx8mm_sys,
			.length = ARRAY_SIZE(pmu_metrics__freescale_imx8mm_sys)
		},
		.name = "pmu_events__freescale_imx8mm_sys",
	},
	{
		.event_table = {
			.entries = pmu_events__freescale_imx8mn_sys,
			.length = ARRAY_SIZE(pmu_events__freescale_imx8mn_sys)
		},
		.metric_table = {
			.entries = pmu_metrics__freescale_imx8mn_sys,
			.length = ARRAY_SIZE(pmu_metrics__freescale_imx8mn_sys)
		},
		.name = "pmu_events__freescale_imx8mn_sys",
	},
	{
		.event_table = {
			.entries = pmu_events__freescale_imx8mp_sys,
			.length = ARRAY_SIZE(pmu_events__freescale_imx8mp_sys)
		},
		.metric_table = {
			.entries = pmu_metrics__freescale_imx8mp_sys,
			.length = ARRAY_SIZE(pmu_metrics__freescale_imx8mp_sys)
		},
		.name = "pmu_events__freescale_imx8mp_sys",
	},
	{
		.event_table = {
			.entries = pmu_events__freescale_imx8mq_sys,
			.length = ARRAY_SIZE(pmu_events__freescale_imx8mq_sys)
		},
		.metric_table = {
			.entries = pmu_metrics__freescale_imx8mq_sys,
			.length = ARRAY_SIZE(pmu_metrics__freescale_imx8mq_sys)
		},
		.name = "pmu_events__freescale_imx8mq_sys",
	},
	{
		.event_table = {
			.entries = pmu_events__hisilicon_hip09_sys,
			.length = ARRAY_SIZE(pmu_events__hisilicon_hip09_sys)
		},
		.metric_table = {
			.entries = pmu_metrics__hisilicon_hip09_sys,
			.length = ARRAY_SIZE(pmu_metrics__hisilicon_hip09_sys)
		},
		.name = "pmu_events__hisilicon_hip09_sys",
	},
	{
		.event_table = {
			.entries = pmu_events__test_soc_sys,
			.length = ARRAY_SIZE(pmu_events__test_soc_sys)
		},
		.name = "pmu_events__test_soc_sys",
	},
	{
		.event_table = { 0, 0 },
		.metric_table = { 0, 0 },
	},
};

static void decompress_event(int offset, struct pmu_event *pe)
{
	const char *p = &big_c_string[offset];

	pe->name = (*p == '\0' ? NULL : p);
	while (*p++);
	pe->pmu = (*p == '\0' ? NULL : p);
	while (*p++);
	pe->topic = (*p == '\0' ? NULL : p);
	while (*p++);
	pe->desc = (*p == '\0' ? NULL : p);
	while (*p++);
	pe->event = (*p == '\0' ? NULL : p);
	while (*p++);
	pe->compat = (*p == '\0' ? NULL : p);
	while (*p++);
	pe->deprecated = *p - '0';
	p++;
	pe->perpkg = *p - '0';
	p++;
	pe->unit = (*p == '\0' ? NULL : p);
	while (*p++);
	pe->long_desc = (*p == '\0' ? NULL : p);
}

static void decompress_metric(int offset, struct pmu_metric *pm)
{
	const char *p = &big_c_string[offset];

	pm->pmu = (*p == '\0' ? NULL : p);
	while (*p++);
	pm->metric_name = (*p == '\0' ? NULL : p);
	while (*p++);
	pm->metric_group = (*p == '\0' ? NULL : p);
	while (*p++);
	pm->metric_expr = (*p == '\0' ? NULL : p);
	while (*p++);
	pm->metric_threshold = (*p == '\0' ? NULL : p);
	while (*p++);
	pm->desc = (*p == '\0' ? NULL : p);
	while (*p++);
	pm->long_desc = (*p == '\0' ? NULL : p);
	while (*p++);
	pm->unit = (*p == '\0' ? NULL : p);
	while (*p++);
	pm->compat = (*p == '\0' ? NULL : p);
	while (*p++);
	pm->metricgroup_no_group = (*p == '\0' ? NULL : p);
	while (*p++);
	pm->default_metricgroup_name = (*p == '\0' ? NULL : p);
	while (*p++);
	pm->aggr_mode = *p - '0';
	p++;
	pm->event_grouping = *p - '0';
}

int pmu_events_table_for_each_event(const struct pmu_events_table *table,
                                    pmu_event_iter_fn fn,
                                    void *data)
{
        for (size_t i = 0; i < table->length; i++) {
                struct pmu_event pe;
                int ret;

                decompress_event(table->entries[i].offset, &pe);
                if (!pe.name)
                        continue;
                ret = fn(&pe, table, data);
                if (ret)
                        return ret;
        }
        return 0;
}

int pmu_metrics_table_for_each_metric(const struct pmu_metrics_table *table,
                                     pmu_metric_iter_fn fn,
                                     void *data)
{
        for (size_t i = 0; i < table->length; i++) {
                struct pmu_metric pm;
                int ret;

                decompress_metric(table->entries[i].offset, &pm);
                if (!pm.metric_expr)
                        continue;
                ret = fn(&pm, table, data);
                if (ret)
                        return ret;
        }
        return 0;
}

const struct pmu_events_table *perf_pmu__find_events_table(struct perf_pmu *pmu)
{
        const struct pmu_events_table *table = NULL;
        char *cpuid = perf_pmu__getcpuid(pmu);
        int i;

        /* on some platforms which uses cpus map, cpuid can be NULL for
         * PMUs other than CORE PMUs.
         */
        if (!cpuid)
                return NULL;

        i = 0;
        for (;;) {
                const struct pmu_events_map *map = &pmu_events_map[i++];
                if (!map->arch)
                        break;

                if (!strcmp_cpuid_str(map->cpuid, cpuid)) {
                        table = &map->event_table;
                        break;
                }
        }
        free(cpuid);
        return table;
}

const struct pmu_metrics_table *perf_pmu__find_metrics_table(struct perf_pmu *pmu)
{
        const struct pmu_metrics_table *table = NULL;
        char *cpuid = perf_pmu__getcpuid(pmu);
        int i;

        /* on some platforms which uses cpus map, cpuid can be NULL for
         * PMUs other than CORE PMUs.
         */
        if (!cpuid)
                return NULL;

        i = 0;
        for (;;) {
                const struct pmu_events_map *map = &pmu_events_map[i++];
                if (!map->arch)
                        break;

                if (!strcmp_cpuid_str(map->cpuid, cpuid)) {
                        table = &map->metric_table;
                        break;
                }
        }
        free(cpuid);
        return table;
}

const struct pmu_events_table *find_core_events_table(const char *arch, const char *cpuid)
{
        for (const struct pmu_events_map *tables = &pmu_events_map[0];
             tables->arch;
             tables++) {
                if (!strcmp(tables->arch, arch) && !strcmp_cpuid_str(tables->cpuid, cpuid))
                        return &tables->event_table;
        }
        return NULL;
}

const struct pmu_metrics_table *find_core_metrics_table(const char *arch, const char *cpuid)
{
        for (const struct pmu_events_map *tables = &pmu_events_map[0];
             tables->arch;
             tables++) {
                if (!strcmp(tables->arch, arch) && !strcmp_cpuid_str(tables->cpuid, cpuid))
                        return &tables->metric_table;
        }
        return NULL;
}

int pmu_for_each_core_event(pmu_event_iter_fn fn, void *data)
{
        for (const struct pmu_events_map *tables = &pmu_events_map[0];
             tables->arch;
             tables++) {
                int ret = pmu_events_table_for_each_event(&tables->event_table, fn, data);

                if (ret)
                        return ret;
        }
        return 0;
}

int pmu_for_each_core_metric(pmu_metric_iter_fn fn, void *data)
{
        for (const struct pmu_events_map *tables = &pmu_events_map[0];
             tables->arch;
             tables++) {
                int ret = pmu_metrics_table_for_each_metric(&tables->metric_table, fn, data);

                if (ret)
                        return ret;
        }
        return 0;
}

const struct pmu_events_table *find_sys_events_table(const char *name)
{
        for (const struct pmu_sys_events *tables = &pmu_sys_event_tables[0];
             tables->name;
             tables++) {
                if (!strcmp(tables->name, name))
                        return &tables->event_table;
        }
        return NULL;
}

int pmu_for_each_sys_event(pmu_event_iter_fn fn, void *data)
{
        for (const struct pmu_sys_events *tables = &pmu_sys_event_tables[0];
             tables->name;
             tables++) {
                int ret = pmu_events_table_for_each_event(&tables->event_table, fn, data);

                if (ret)
                        return ret;
        }
        return 0;
}

int pmu_for_each_sys_metric(pmu_metric_iter_fn fn, void *data)
{
        for (const struct pmu_sys_events *tables = &pmu_sys_event_tables[0];
             tables->name;
             tables++) {
                int ret = pmu_metrics_table_for_each_metric(&tables->metric_table, fn, data);

                if (ret)
                        return ret;
        }
        return 0;
}

static const int metricgroups[][2] = {

};

const char *describe_metricgroup(const char *group)
{
        int low = 0, high = (int)ARRAY_SIZE(metricgroups) - 1;

        while (low <= high) {
                int mid = (low + high) / 2;
                const char *mgroup = &big_c_string[metricgroups[mid][0]];
                int cmp = strcmp(mgroup, group);

                if (cmp == 0) {
                        return &big_c_string[metricgroups[mid][1]];
                } else if (cmp < 0) {
                        low = mid + 1;
                } else {
                        high = mid - 1;
                }
        }
        return NULL;
}
